  (set-file-docstring "Function for modeling 3d embeds")

  (defun-q draw-3d_embed (dat ins / blk em-1 em-2 em-3 em-4 em-5 oc-q oc-x ad-s 45-s)
    "Creates a 3d block of an embed via passed xdata and insert"
    (declare (vars (dat list (data-list-p dat)) (ins list (point-p ins))))
    (setq blk (p-blk (uppercase (xd-value "style" dat))))
    (psc-include
     '("3D/3d-model.lsp" "shapes/shape.lsp" "shapes/decking.lsp")
    ) ;_ psc-include
    (defun 45-s () ;;; trig for 45 degree stud yz pts
      (mapcar '(lambda (l) (* l (cos (dtr 45)))) '(0.5 4.125 4.5))
    ) ;_ defun
    (defun oc-q (lng spc / n) ;;; on center quantity of studs
      (while (> lng spc) (setq lng (- lng spc) n (1+ (?n n))))
      (if n (+ n 2) 2)
    ) ;_ defun oc-q
    (defun oc-x (lng spc) ;;; on center stud spacing
      (if (> lng spc) (/ lng (1- (oc-q lng spc))) lng)
    ) ;_ defun oc-x
    (defun ad-s (p1 p2 p3) ;;; add stud objects from list of 3D pts
      (mapcar
       '(lambda (a r)
          (vla-put-layer
            (car (draw-pipe-3d (list (vla-Add3DPoly blk a)) r))
            "Embed"
          ) ;_ vla-put-layer
        ) ;_ lambda
        (mapcar 'vlist->safearray (list (list p1 p2) (list p2 p3)))
        (list 0.25 0.50)
      ) ;_ mapcar
    ) ;_ defun ad-s
    (defun em-1 (lng / sx3 ang cen) ;;; add EM-1 model
      (defun sx3 (x) (list (- x 4) x (+ x 4)))
      (setq
        ang (apply 'angle-pts (shape-def "L3x2x1/4" nil))
        lng (mapcar 'float lng)
      ) ;_ setq
      (vla-put-layer
        (plineObj->solidObj
          (vla-put-normal-r (add-polyline blk '(0 0) ang T) (3dpt 1 0))
          (apply '+ lng)
          T
        ) ;_ plineObj->solidObj
        "Embed"
      ) ;_ vla-put-later
      (setq cen
        (apply
         '(lambda (lg lw db rw rg / ln)
            (setq ln (apply '+ lng))
            (append
              (if (zerop lw)
                (list 1)
                (append (sx3 (+ lg 4)) (sx3 (+ lg lw -4)))
              ) ;_ if
              (if (zerop rw)
                (list (- ln 1))
                (append (sx3 (- ln rg rw -4)) (sx3 (- ln rg 4)))
              ) ;_ if
            ) ;_ append
          ) ;_ lambda
          lng
        ) ;_ apply
      ) ;_ setq
      (apply-by-twos
       '(lambda (x1 x2 / sp)
          (setq sp (oc-x (- x2 x1) 16.0))
          (while (> (- x2 x1) 16.0) (add-to-list 'cen (setq x1 (+ x1 sp))))
        ) ;_ lambda
        cen
      ) ;_ apply-by-twos
      (setq cen (vl-sort cen '<))
      (map-apply 'ad-s
        (mapcar
         '(lambda (x) (mapcar '(lambda (yz) (list x yz (- yz))) (45-s)))
          cen
        ) ;_ mapcar
      ) ;_ map-apply
    ) ;_ defun em-1
    (defun em-2 (lng / qty spc ang cen)  ;;; add EM-2 model
      (setq
        qty (oc-q (- lng 6.0) 16.0)
        spc (oc-x (- lng 6.0) 16.0)
        ang (apply 'angle-pts (shape-def "L4x4x1/4" nil))
      ) ;_ setq
      (vla-put-layer
        (plineObj->solidObj
          (vla-put-normal-r (add-polyline blk '(0 0) ang T) (3dpt 1 0))
          lng
          T
        ) ;_ plineObj->solidObj
        "Embed"
      ) ;_ vla-put-layer
      (add-to-list 'cen 3.0)
      (while (< (length cen) qty) (add-to-list 'cen (+ (last cen) spc)))
      (map-apply 'ad-s
        (mapcar
         '(lambda (x) (mapcar '(lambda (yz) (list x yz (- yz))) (45-s)))
          cen
        ) ;_ mapcar
      ) ;_ map-apply
    ) ;_ defun em-2
    (defun em-3 (lng / qty spc ang cen)  ;;; add EM-3 model
      (setq
        qty (oc-q (- lng 6.0) 6.0)
        spc (oc-x (- lng 6.0) 6.0)
        ang (apply 'angle-pts (shape-def "L4x4x3/8" nil))
      ) ;_ setq
      (vla-put-layer
        (plineObj->solidObj
          (vla-put-normal-r
            (mirror&delete (add-polyline blk '(0 0) ang T) '(0 0) 'X)
            (3dpt 1 0)
          ) ;_ vla-put-normal-r
          lng
          T
        ) ;_ plineObj->solidObj
        "Embed"
      ) ;_ vla-put-layer
      (add-to-list 'cen '(3.0 Z))
      (while (< (length cen) qty)
        (add-to-list 'cen
          (list
            (+ spc (car (last cen)))
            (if (= 'Z (cadr (last cen))) 'Y 'Z)
          ) ;_ cons
        ) ;_ add-to-list
      ) ;_ while
      (map-apply 'ad-s
        (mapcar
         '(lambda (x / d)
            (setq d (cadr x) x (car x))
            (mapcar
             '(lambda (yz) (if (= 'Z d) (list x 3.0 yz) (list x yz 3.0)))
             '(0.375 4.0 4.375)
            ) ;_ mapcar
          ) ;_ lambda
          cen
        ) ;_ mapcar
      ) ;_ map-apply
    ) ;_ defun em-3
    (defun em-4 ()  ;;; add EM-4 model
      (vla-put-layer
        (plineobj->solidobj
          ( (lambda (lst / obj)
              (setq obj (vla-add3DPoly blk (vlist->safearray lst)))
              (vla-put-closed obj :vlax-true)
              obj
            ) ;_ lambda
           '((0 0 0) (10 0 0) (10 0 10) (0 0 10))
          ) ;_ lambda-func
          -0.25 T
        ) ;_ plineobj->solidobj
        "Embed"
      ) ;_ vla-put-layer
      (map-apply 'ad-s
        (mapcar
         '(lambda (p)
            (mapcar
             '(lambda (z) (list (car p) z (cadr p)))
             '(0.25 3.875 4.25)
            ) ;_ mapcar
          ) ;_ lambda
         '((1.5 1.5) (8.5 1.5) (5.0 5.0) (1.5 8.5) (8.5 8.5))
        ) ;_ mapcar
      ) ;_ map-apply
    ) ;_ defun em-4
    (defun em-5 () ;;; add EM-5 model
      (vla-put-layer
        (plineObj->solidObj (add-rectangle blk 6 6 '(0 0 0)) -0.25 T)
        "Embed"
      ) ;_ va-put-layer
      (map-apply 'ad-s
        (list
         '((1.0 3.0 -0.25) (1.0 3.0 -3.875) (1.0 3.0 -4.25))
         '((5.0 3.0 -0.25) (5.0 3.0 -3.875) (5.0 3.0 -4.25))
        ) ;_ list
      ) ;_ map-apply
    ) ;_ defun em-5
    (cond
      ( (= style "em-1") (em-1 (list l_gap l_str d_btw r_str r_gap)))
      ( (= style "em-2") (em-2 t_lng))
      ( (= style "em-3") (em-3 t_lng))
      ( (= style "em-4") (em-4))
      ( (= style "em-5") (em-5))
    ) ;_ cond
    (setq blk (insert-block modelSpace ins blk))
    (assign-xdata blk "3D_Embed" (format-xdata dat))
    (vla-put-layer blk "3D Embed")
    blk
  ) ;_ defun-q draw-3d_embed
  (defun-r 'draw-3d_embed)
