  (set-file-docstring "Function for modeling 3d landings")

  (psc-include
    (list
      "land/util.lsp"
      "land/land_supp.lsp"
      "land/derived.lsp"
      "land/migrations.lsp"
      "land/details.lsp"
      "shapes/calc-tmatrix.lsp"
      "shapes/decking.lsp"
      "shapes/shape.lsp"
    ) ;_ list
  ) ;_ psc-include

  (defun-q Draw-3D_Land (data ins / blockObj d-chn d-plt)
    "Creates a 3d block of a landing via passed xdata and insert"
    (declare (vars (data list (data-list-p data)) (ins list (point-p ins))))
    (setq blockobj (p-blk (xd-value "assembly" data)))
    (setup-env T)

    ;;; draw landing channels
    (defun d-chn ( / slot-vol)
      (foreach corner '("fl" "fr" "bl" "br")
        (if
          (and
            (null slot-vol)
            (member
              (substr (cornerkey "chan") 1 2)
             '("66" "67" "68")
            ) ;_ member
          ) ;_ and
          (defun slot-vol (ins)
            (plineobj->solidobj
              (vlax-put-property-r
                (add-slot blockObj ins 3.0 0.6875)
                'normal
                (3dpt '(1 0 0))
              ) ;_ vlax-put-property-r
              (* dir 6)
              T
            ) ;_ plineobj->solidobj
          ) ;_ defun slot-vol
        ) ;_ if
      ) ;_ foreach
      ;;; CHANNEL - FRONT
      (vla-put-layer-r
        ( (lambda (obj)
            (if (member (substr fl_chan 1 2) '("66" "67" "68"))
              (vla-boolean-r
                obj
                acsubtraction
                (slot-vol
                  (list
                    (* dir (+ fl_ext (/ plt_wid 2.0) -2))
                    (- (+ flr_thick 4))
                    (- dir)
                  ) ;_ list
                ) ;_ slot-vol
              ) ;_ vla-boolean-r
            ) ;_ if
            (if (member (substr fr_chan 1 2) '("66" "67" "68"))
              (vla-boolean-r
                obj
                acsubtraction
                (slot-vol
                  (list
                    (* (- dir) (+ fr_ext (/ plt_wid 2.0) -2))
                    (- (+ flr_thick 4))
                    (- dir)
                  ) ;_ list
                ) ;_ slot-vol
              ) ;_ vla-boolean-r
            ) ;_ if
            obj
          ) ;_ lambda
          (vla-transformby-r
            (plineobj->solidobj
              (add-shape blockobj '(0 0) front_chan)
              (* dir front_chan_len)
              T
            ) ;_ plineobj->solidobj
            (calc-tmatrix
              90 0 0
              0
              (* dir
                (+
                  (-
                    fl_ext
                    (if (and (> fl_ext 0) (/= fl_ext bl_ext))
                      0
                      (num-val left_chan_wid)
                    ) ;_ if
                  ) ;_ -
                  (/ plt_wid 2.0)
                ) ;_ +
              ) ;_ *
              (- flr_thick)
              (> 0 dir) nil nil
            ) ;_ calc-tmatrix
          ) ;_ vla-transformby-r
        )
        "Land Channels"
      ) ;_ vla-put-layer-r
      ;;; CHANNEL - BACK
      (vla-put-layer-r
        ( (lambda (obj)
            (if (member (substr bl_chan 1 2) '("66" "67" "68"))
              (vla-boolean-r
                obj
                acsubtraction
                (slot-vol
                  (list
                    (* dir (+ bl_ext (/ plt_wid 2.0) -2))
                    (- (+ flr_thick 4))
                    (* dir (- back_chan_loc 1))
                  ) ;_ list
                ) ;_ slot-vol
              ) ;_ vla-boolean-r
            ) ;_ if
            (if (member (substr br_chan 1 2) '("66" "67" "68"))
              (vla-boolean-r
                obj
                acsubtraction
                (slot-vol
                  (list
                    (* (- dir) (+ br_ext (/ plt_wid 2.0) -2))
                    (- (+ flr_thick 4))
                    (* dir (- back_chan_loc 1))
                  ) ;_ list
                ) ;_ slot-vol
              ) ;_ vla-boolean-r
            ) ;_ if
            obj
          ) ;_ lambda
          (vla-transformby-r
            (plineobj->solidobj
              (add-shape blockobj '(0 0) back_chan)
              (* dir back_chan_len)
              T
            ) ;_ plineobj->solidobj
            (calc-tmatrix
              90 0 0
              (* dir back_chan_loc)
              (* dir
                (+
                  (-
                    bl_ext
                    (if (and (> bl_ext 0) (/= fl_ext bl_ext))
                      0
                      (num-val left_chan_wid)
                    ) ;_ if
                  ) ;_ -
                  (/ plt_wid 2.0)
                ) ;_ +
              ) ;_ *
              (- flr_thick)
              (or
                (and (> 0 dir) (= flip_back "0"))
                (and (> dir 0) (= flip_back "1"))
              ) ;_ or
              nil
              nil
            ) ;_ calc-tmatrix
          ) ;_ vla-transformby-r
        )
        "Land Channels"
      ) ;_ vla-put-layer-r
      ;;; CHANNEL - LEFT
      (if left_chan
        ( (lambda (obj / f-cope b-cope)
            (if (and (> fl_ext 0) (> fl_ext bl_ext))
              (progn
                (setq f-cope
                  (plineobj->solidobj
                    (add-shape blockobj '(0 0) left_chan)
                    (* (- dir) front_chan_wid)
                    T
                  ) ;_ plineobj->solidobj
                ) ;_ setq
                ( (lambda (obj)
                    (vla-put-normal obj (3dpt (- dir) 0 0))
                    (vla-rotate3d obj (3dpt 0 0 0) (3dpt 1 0 0) (dtr -90))
                    (vla-move
                      obj
                      (3dpt 0 0 0)
                      (3dpt -1 0 (* (- dir) front_chan_wid))
                    ) ;_ vla-move
                    (setq obj (plineobj->solidobj obj (* dir -6.0) T))
                    (vla-boolean f-cope acsubtraction obj)
                  ) ;_ lambda
                  (add-shape blockobj '(0 0) left_chan)
                ) ;_ lambda func
                (vla-boolean obj acunion f-cope)
              ) ;_ progn
            ) ;_ if
            (if (and (> bl_ext 0) (> bl_ext fl_ext) (= flip_back "1"))
              (progn
                (setq b-cope
                  (plineobj->solidobj
                    (add-shape
                      blockobj
                      (list 0 0 (* dir (+ back_chan_wid left_chan_len)))
                      left_chan
                    ) ;_ add-shape
                    (* (- dir) front_chan_wid)
                    T
                  ) ;_ plineobj->solidobj
                ) ;_ setq
                ( (lambda (obj)
                    (vla-put-normal obj (3dpt dir 0 0))
                    (vla-rotate3d obj (3dpt 0 0 0) (3dpt 1 0 0) (dtr -90))
                    (vla-move
                      obj
                      (3dpt 0 0 0)
                      (3dpt -1 0 (* dir (+ back_chan_wid left_chan_len)))
                    ) ;_ vla-move
                    (setq obj (plineobj->solidobj obj (* dir 6.0) T))
                    (vla-boolean b-cope acsubtraction obj)
                  ) ;_ lambda
                  (add-shape blockobj '(0 0) left_chan)
                ) ;_ lambda func
                (vla-boolean obj acunion b-cope)
              ) ;_ progn
            ) ;_ if
            (vla-transformby-r
              obj
              (calc-tmatrix
                (* dir 90)
                0
                (* dir 90)
                (if (and (> fl_ext 0) (/= fl_ext bl_ext)) (* dir front_chan_wid) 0)
                (* dir (+ (/ plt_wid 2.0) (min fl_ext bl_ext) (- left_chan_wid)))
                (- flr_thick)
                nil nil (< dir 0)
              ) ;_ calc-tmatrix
            ) ;_ vla-transformby-r
          ) ;_ lambda
          (vla-put-layer-r
            (plineobj->solidobj
              (add-shape blockobj '(0 0) left_chan)
              (* dir left_chan_len)
              T
            ) ;_ plineobj->solidobj
            "Land Channels"
          ) ;_ vla-put-layer-r
        ) ;_ lambda func
      ) ;_ if
      ;;; CHANNEL - RIGHT
      (if right_chan
        ( (lambda (obj / f-cope b-cope)
            (if (and (> fr_ext 0) (/= fr_ext br_ext))
              (progn
                (setq f-cope
                  (plineobj->solidobj
                    (add-shape blockobj '(0 0) right_chan)
                    (* dir front_chan_wid)
                    T
                  ) ;_ plineobj->solidobj
                ) ;_ setq
                ( (lambda (obj)
                    (vla-put-normal obj (3dpt dir 0 0))
                    (vla-rotate3d obj (3dpt 0 0 0) (3dpt 1 0 0) (dtr -90))
                    (vla-move obj (3dpt 0 0 0) (3dpt -1 0 (* dir front_chan_wid)))
                    (setq obj (plineobj->solidobj obj (* dir 6.0) T))
                    (vla-boolean f-cope acsubtraction obj)
                  ) ;_ lambda
                  (add-shape blockobj '(0 0) right_chan)
                ) ;_ lambda func
                (vla-boolean obj acunion f-cope)
              ) ;_ progn
            ) ;_ if
            (if (and (> br_ext 0) (> br_ext fr_ext) (= flip_back "1"))
              (progn
                (setq b-cope
                  (plineobj->solidobj
                    (add-shape
                      blockobj
                      (list 0 0 (* (- dir) (+ back_chan_wid right_chan_len)))
                      right_chan
                    ) ;_ add-shape
                    (* dir front_chan_wid)
                    T
                  ) ;_ plineobj->solidobj
                ) ;_ setq
                ( (lambda (obj)
                    (vla-put-normal obj (3dpt (- dir) 0 0))
                    (vla-rotate3d obj (3dpt 0 0 0) (3dpt 1 0 0) (dtr -90))
                    (vla-move
                      obj
                      (3dpt 0 0 0)
                      (3dpt -1 0 (* (- dir) (+ back_chan_wid right_chan_len)))
                    ) ;_ vla-move
                    (setq obj (plineobj->solidobj obj (* dir -6.0) T))
                    (vla-boolean b-cope acsubtraction obj)
                  ) ;_ lambda
                  (add-shape blockobj '(0 0) right_chan)
                ) ;_ lambda func
                (vla-boolean obj acunion b-cope)
              ) ;_ progn
            ) ;_ if
            (vla-transformby-r
              obj
              (calc-tmatrix
                (* dir 90)
                0
                (* (- dir) 90)
                (if (and (> fr_ext 0) (/= fr_ext br_ext)) (* dir front_chan_wid) 0)
                (* (- dir) (+ (/ plt_wid 2.0) (min fr_ext br_ext) (- right_chan_wid)))
                (- flr_thick)
                nil nil (< dir 0)
              ) ;_ calc-tmatrix
            ) ;_ vla-transformby-r
          ) ;_ lambda
          (vla-put-layer-r
            (plineobj->solidobj
              (add-shape blockobj '(0 0) right_chan)
              (* (- dir) right_chan_len)
              T
            ) ;_ plineobj->solidobj
            "Land Channels"
          ) ;_ vla-put-layer-r
        ) ;_ lambda func
      ) ;_ if
    ) ;_ defun d-chn

    ;;; draw landing platform
    (defun d-plt ( / add-break add-conc add-hss add-chk add-can)
      (cond
        ( (begins-with tread_style "100")
          (defun add-break (def)
            (remove! 'def '(null (caar x)))
            (setq def (join-vertices (mapcar 'cdr def)))
            (mapcar
             '(lambda (lst / pth off brk nor)
                (setq
                  pth (add-polyline blockObj '(0 0) lst nil)
                  off (vlax-curve-getstartpoint pth)
                  brk (quote ((3.5 -3.5) (0.0 -3.5) (0.0 0.0) (0.75 0.0)))
                  brk (add-polyline blockObj '(0 0) brk nil)
                  brk (offset&close brk 0.1345 nil 0)
                  nor (mapcar 'unit-dir (mapcar '- (cadr lst) (car lst)))
                ) ;_ setq
                (vla-put-normal brk (3dpt nor))
                (vla-move brk (3dpt 0 0) (3dpt off))
                (plineObj->solidObj brk pth T)
              ) ;_ lambda
              def
            ) ;_ mapcar
          ) ;_ defun add-break
          (defun add-conc (def / nil-off ret-lst)
            (setq nil-off (mapcar 'cdr (remove def '(caar x))))
            (setq ret-lst
              (mapcar
               '(lambda (lst / pth off ret)
                  (setq pth (add-polyline blockObj '(0 0) lst nil))
                  (setq off (offset-polyline pth -0.75))
                  (setq ret (if (member lst nil-off) pth off))
                  (setq ret (line-obj->vert-lst ret))
                  (setq ret (mapcar 'remove-last ret))
                  (mapcar 'vla-delete (list pth off))
                  (if
                    (and
                      (not (equal (car lst) (last lst)))
                      (apply '=f (rcons (mapcar 'car (take 2 lst)) 0.00001))
                      (not
                        (vl-member-if
                         '(lambda (lst) (equal (car ret) lst 0.00001))
                          (mapcar 'last nil-off)
                        ) ;_ vl-member-if
                      ) ;_ not
                    ) ;_ and
                    (setq ret (cons (car lst) ret))
                  ) ;_ if
                  (if
                    (and
                      (not (equal (car lst) (last lst)))
                      (apply '=f (rcons (mapcar 'car (take 2 (reverse lst))) 0.00001))
                      (not
                        (vl-member-if
                         '(lambda (lst) (equal (last ret) lst 0.00001))
                          (mapcar 'car nil-off)
                        ) ;_ vl-member-if
                      ) ;_ not
                    ) ;_ and
                    (setq ret (rcons ret (last lst)))
                  ) ;_ if
                  ret
                ) ;_ lambda
                (join-vertices (mapcar 'cdr def))
              ) ;_ mapcar
            ) ;_ setq
            (plineobj->regionobj
              (add-polyline blockObj '(0 0) (apply 'append ret-lst) T)
              T
            ) ;_ plineobj->regionobj
          ) ;_ defun add-conc
          (set-active-layer "Land Platform")
          (add-break (break-path))
          (plineObj->solidObj
            (vla-transformby-r
              (add-decking
                blockObj
                (list (* (- dir) (- (/ plt_wid 2.0) 1.5)) 0)
                (list (* dir (- (/ plt_wid 2.0) 1.5)) 0)
              ) ;_ add-decking
              (calc-tmatrix
                90 0 (* dir -90)
                (* dir 0.25) 0 (- 0 flr_thick -0.1345)
                nil nil nil
              ) ;_ calc-tmatrix
            ) ;_ vla-transformby-r
            (- (- plt_dpt 1.0))
            T
          ) ;_ plineObj->solidObj
          (set-active-layer "Land Conc Fill")
          (add-conc (break-path))
        ) ;_ condif
        ( (begins-with tread_style "200")
          (defun add-hss (loc / off out)  ;;; hss @ land front/back
            (setq out (member "85" (list fl_chan_num bl_chan_num)))
            (setq out (if out (* dir (min fl_ext bl_ext)) 0))
            (setq off
              (list (* dir loc) (+ (* dir (/ plt_wid 2.0)) out) (- chk_thk))
            ) ;_ setq
            (apply
             '(lambda (s1 s2) (vla-boolean s1 acSubtraction s2))
              (mapcar
               '(lambda (lst / obj)
                  (setq obj (add-polyline blockObj '(0.0 0.0) lst T))
                  (vla-put-normal obj (3dpt 0 -1 0))
                  (vla-move obj (3dpt 0 0) (3dpt off))
                  (plineobj->solidobj obj (* dir plt_wid_eff) T)
                ) ;_ lambda
                (apply 'hss-pts (list 2.5 1.5 0.065))
              ) ;_ mapcar
            ) ;_ apply
          ) ;_ defun add-hss
          (defun add-can (cen / off can obj lng out)
            (setq cen (* dir (- (/ plt_wid 2.0) cen)))
            (setq off (if (= front_bolt_to "None") 0.25 1.5))
            (foreach con (list front_bolt_to back_bolt_to)
              (setq lng (- (?? lng plt_dpt) (if (= con "None") 0.25 1.5)))
            ) ;_ foreach
            (setq can '((-3.0 2.5) (-3.0 0.0) (3.0 0.0) (3.0 2.5)))
            (setq obj (add-polyline blockObj '(0 0) can nil))
            (offset&close obj -0.0747 nil 0)
            (vla-put-normal obj (3dpt dir 0 0))
            (vla-move obj (3dpt 0 0) (3dpt (* dir off) cen -2.625))
            (plineobj->solidobj obj lng T)
          ) ;_ defun add-can
          (defun add-chk ( / out rec obj)
            (setq out (member "85" (list fl_chan_num bl_chan_num)))
            (setq out (if out (min fl_ext bl_ext) 0))
            (setq rec
              (rect-pts
                (* dir chk_dpt)
                (* (- dir) plt_wid_eff)
                (list
                  (* dir chk_offset)
                  (+ (* dir (/ plt_wid 2.0)) (* dir out))
                ) ;_ list
              ) ;_ rect-pts
            ) ;_ setq
            (setq obj (add-polyline blockObj '(0 0) rec T))
            (plineobj->solidobj obj (- chk_thk) T)
          ) ;_ defun add-chk
          (set-active-layer "Land Platform")
          (if (/= front_bolt_to "None") (add-hss 0.75))
          (if (/= back_bolt_to "None") (add-hss (- back_chan_loc 0.75)))
          (mapcar 'add-can (land-calc-can-spacing))
          (add-chk)
          ;;; QUIET TREAD
          (if (= "1" quiet_tread)
            (progn
              (set-active-layer "Land Quiet Tread")
              (mapcar
               '(lambda (data)
                  (with-data data
                   '( (plineObj->solidObj
                        (add-rectangle
                          blockobj (* dir lng) wid
                          (list
                            (* dir off)
                            (-
                              (* dir (/ plt_wid 2.0))
                              (* dir cen)
                              (/ wid 2.0)
                            ) ;_ -
                            hgt
                          ) ;_ list
                        ) ;_ add-rectangle
                        -0.125 T
                    ) ) ;_ plineObj->solidObj
                  ) ;_ with-data
                ) ;_ lambda
               (map-append 'cdr (calc-qtread-spacing))
              ) ;_ mapcar
            ) ;_ progn
          ) ;_ if
        ) ;_ condif
        ( (= T)
          (vla-put-layer-r
            (vla-transformby-r
              (plineobj->solidobj
                (add-rectangle
                  blockObj
                  (* dir plt_dpt)
                  (- flr_thick)
                 '(0 0)
                ) ;_ add-rectangle
                (* dir plt_wid)
                T
              ) ;_ plineobj->solidobj
              (calc-tmatrix
                90 0 0
                0 (* dir (/ plt_wid 2.0)) 0
                nil nil nil
              ) ;_ calc-tmatrix
            ) ;_ vla-transformby-r
            "Land Platform"
          ) ;_ vla-put-layer-r
        ) ;_ condif
      ) ;_ cond
    ) ;_ defun d-plt

    (defun l-off ( / off)
      (setq off (* (- (?n front_l-stair_wid) (?n front_r-stair_wid)) dir 0.5))
      (mapcar
       '(lambda (obj) (vla-move obj (3dpt 0 0) (3dpt 0 off)))
        (collection->list blockobj)
      ) ;_ mapcar
    ) ;_ defun l-off

    (with-data data '((d-chn) (d-plt) (draw-land-support '3d) (l-off)))

    (setq blockObj
      (vla-put-layer-r
        (insert-block modelSpace ins blockObj)
        "3D Land"
      ) ;_ vla-put-layer-r
    ) ;_ setq
    (vla-put-rotation blockobj (?f rot))
    (assign-xdata blockObj "3D_Land" (format-xdata data))
    (vlobj>ename blockObj)

  ) ;_ defun-q Draw-3D_Land
  (defun-r 'Draw-3D_Land)
