  (set-file-docstring "Functions for modeling multiple stairs and landings")

  (psc-include
   '( "3D/3d-stair"       "3D/3d-land"        "3D/3d-guard"
      "3D/3d-embed"       "rail/util.lsp"     "shapes/line.lsp"
      "stair/tread.lsp"   "stair/util.lsp"
    ) ;_ list
  ) ;_ psc-include

  (defun-q Draw-3D-Model ( / stair-p ins data asc-int ret-lst off-lst app-id)
    "Creates stair and landing blocks in sequence from entity list"
    (declare (vars (ent-lst list)))
    (defun stair-p (ent)
      (= "Stair" (xd-value "app_id" (read-xdata ent "")))
    ) ;_ defun stair-p
    (setq off-lst
      (cons
        0.0
        (apply-by-twos
         '(lambda (l1 l2) (- (car l2) (apply '+ l1)))
          (mapcar
           '(lambda (ent)
              (setq data (read-xdata-migrate ent "Stair"))
              (list
                (cadr (assoc 10 (entget ent)))
                (with-data data '((* dir full_run)))
              ) ;_ list
            ) ;_ lambda
            (vl-remove-if-not 'stair-p ent-lst)
          ) ;_ mapcar
        ) ;_ apply-by-twos
      ) ;_ cons
    ) ;_ setq
    (foreach ent ent-lst
      (setq app-id (xd-value "app_id" (read-xdata ent "")))
      (if (null (setq data (read-xdata-migrate ent app-id)))
        (error "\nPassed entity has no APP_ID")
      ) ;_ if
      (with-data data
       '( (cond
            ( (ends-with (strcase app_id) "STAIR")
              (setq asc-int (if (= "L" (substr ascend 1 1)) 1 -1))
              (setq ins
                (if (null ins) '(0.0 0.0 0.0)
                  (mapcar '+ ins
                    (list
                      (car off-lst)
                      (* asc-int (- dir) (* width 0.5))
                      0
                    ) ;_ list
                  ) ;_ mapcar
                ) ;_ if
              ) ;_ setq
              (add-to-list 'ret-lst (cons (append '("blk-ins") ins) data))
              (Draw-3D_Stair data ins)
              (setq off-lst (cdr off-lst))
              (setq ins
                (mapcar '+ ins
                  (list
                    (* dir full_run)
                    (* asc-int dir (+ (* width 0.5) dist_btwn))
                    flight_hgt
                  ) ;_ list
                ) ;_ mapcar
              ) ;_ setq
              (if (= 0 (car off-lst))
                (draw-closure
                  (mapcar '- ins (list 0 (* asc-int dir dist_btwn) 0))
                ) ;_ draw-closure
              ) ;_ if
            ) ;_ condif
            ( (ends-with (strcase app_id) "LAND")
              (if (null (xd-value "dist_btwn" data))
                (setq
                  dist_btwn (list plt_wid front_l-stair_wid front_r-stair_wid)
                  dist_btwn (- (apply '- (mapcar '?n dist_btwn)))
                ) ;_ setq
              ) ;_ if
              (setq ins
                (if (null ins) '(0 0 0)
                  (mapcar '+ ins
                    (list
                      0
                      (* (?? asc-int 1) (- dir) (* dist_btwn 0.5))
                      0
                    ) ;_ list
                  ) ;_ mapcar
                ) ;_ if
              ) ;_ setq
              (Draw-3D_Land data ins)
              (setq ins
                (mapcar '+ ins
                  (list 0 (* (?? asc-int 1) dir (* dist_btwn 0.5)) 0)
                ) ;_ mapcar
              ) ;_ setq
            ) ;_ condif
            ( (= T) (error "\nPassed entity has unsupported APP_ID"))
        ) ) ;_ cond
      ) ;_ with-data
    ) ;_ foreach
    (apply-by-twos '3dr-ret ret-lst)
    (princ)
  ) ;_ defun-q Draw-3D-Model
  (defun-r 'Draw-3D-Model)

  (defun-q draw-closure (ins / n blk)
    "Draws closure plate between stairs"
    (declare (vars (ins list (point-p ins))))
    (setq n (1+ (vl-position ent ent-lst)))
    (while
      (and
        (nth n ent-lst)
        (with-data (read-xdata (nth n ent-lst) "")
         '((/= "Stair" app_id))
        ) ;_ with-data
      ) ;_ and
      (setq n (1+ n))
    ) ;_ while
    (if (nth n ent-lst)
      (apply
       '(lambda (y1 bt y2 / obj dir)
          (setq blk (p-blk "closure"))
          (if (< y2 y1) (setq y2 y1))
          (setq dir (cadr (assoc "dir" data)))
          (setq obj
            (list
              (list 0.0 y1)
              (list (* dir asc-int bt) y2)
              (list (* dir asc-int bt) (- y2 (if (<= y2 2) 6 12)))
              (list 0.0 (- y2 (if (<= y2 2) 6 12)))
            ) ;_ list
          ) ;_ setq
          (setq obj (add-polyline blk '(0 0 0) obj T))
          (vla-put-normal obj (3dpt 1 0 0))
          (vla-put-layer-r
            (plineobj->solidobj obj (* dir -0.25) T)
            "Land Platform"
          ) ;_ vla-put-layer-r
          (insert-block modelspace ins blk)
        ) ;_ lambda
        (append
          (with-data (read-xdata ent "")
           '( (list
                (if (= ascend "Left") l-nosing_top r-nosing_top)
                dist_btwn
            ) ) ;_ list
          ) ;_ with-data
          ( (lambda (lst)
              (list
                (apply 'max
                  (mapcar 'cadr
                    (vl-remove-if '(lambda (p) (> (car p) 0.25)) lst)
                  ) ;_ mapcar
                ) ;_ apply
              ) ;_ list
            ) ;_ lambda
            (with-data (read-xdata (nth n ent-lst) "")
             '((cdar (stair-stringer-calc ascend)))
            ) ;_ with-data
          ) ;_ lambda func
        ) ;_ append
      ) ;_ apply
    ) ;_ if
  ) ;_ defun-q draw-closure
  (defun-r 'draw-closure)

  (defun-q 3d-ret-calc (l-dat u-dat / b-off l-lst u-lst r-lst)
    "Calculates and returns an insert and vertex list for a handrail return"
    (declare
      (vars (l-dat list (data-list-p l-dat)) (u-dat list (data-list-p u-dat)))
    ) ;_ declare
    (setq b-off
      (-
        (with-data l-dat '((* dir full_run)))
        (car
          (rotate-pt
            (displace-pt
              (cdr (assoc "blk-ins" u-dat))
              (mapcar '- (cdr (assoc "blk-ins" l-dat)))
            ) ;_ displace-pt
            (- (?f rot))
          ) ;_ rotate-pt
        ) ;_ car
      ) ;_ -
    ) ;_ setq
    (if (equal b-off 0 0.01) (setq b-off 0.0))
    (mapcar
     '(lambda (dat lst / pts brk nos off dis t/b)
        (with-data dat
         '(
            (setq t/b (if (= dat l-dat) 'bot 'top))
            (setq dis
              (cond
                ( (= dat l-dat)
                  (* dir (+ (max 0 (* (- dir) b-off)) top_ext))
                ) ;_ condif
                ( (>= (* dir b-off) 0)
                  (if (<= bot_ext tread_depth)
                    0
                    (* (- dir) (- bot_ext tread_depth))
                  ) ;_ if
                ) ;_ condif
                ( (= T)
                  (if (<= (+ bot_ext (abs b-off)) tread_depth)
                    0
                    (- b-off (* dir (- bot_ext tread_depth)))
                  ) ;_ if
                ) ;_ condif
              ) ;_ cond
            ) ;_ setq
            (setq dis (list dis 0 0))
            (setq off (list (z-offset-stair-rail-handgrab "i")))
            (setq nos (calc-nose-pts))
            (setq pts (cdar (reverse (calc-stair-rail "i" nil nil))))
            (setq pts (first-two ((if (= dat l-dat) last car) pts)))
            (setq brk
              (line-point-@x
                (line-offset
                  (stair-slope)
                  (if (< (read rail_style) 300) 28.545 29.545)
                ) ;_ line-offset
                (cond
                  ( (= dat l-dat)
                    (caar (reverse nos))
                  ) ;_ condif
                  ( (>= (* dir b-off) 0)
                    (- (caar nos) (* dir (min bot_ext tread_depth)))
                  ) ;_ condif
                  ( (= T)
                    (-
                      (caar nos)
                      (* dir (min (+ (abs b-off) bot_ext) tread_depth))
                    ) ;_ -
                  ) ;_ condif
                ) ;_ cond
              ) ;_ line-point-@x
            ) ;_ set
          ) ;_ with-data_func
        ) ;_ with-data
        (set lst
          (mapcar
           '(lambda (l) (append l off))
            (vl-sort
              (list pts brk)
             '(lambda (l1 l2) (< (cadr l1) (cadr l2)))
            ) ;_ vl-sort
          ) ;_ mapcar
        ) ;_ set
        (set lst
          (displace-pts
            (rotate-pts
              (mapcar
               '(lambda (p) (trans p '(0 -1 0) 0))
                (append
                  (if (= t/b 'top)
                    (list (mapcar '+ (car (eval lst)) dis))
                  ) ;_ if
                  (eval lst)
                  (if (= t/b 'bot)
                    (list (mapcar '+ (last (eval lst)) dis))
                  ) ;_ if
                ) ;_ append
              ) ;_ mapcar
              (?f rot)
            ) ;_ rotate-pts
            (cdr (assoc "blk-ins" dat))
          ) ;_ displace-pts
        ) ;_ set
      ) ;_ lambda
      (list l-dat u-dat)
     '(l-lst u-lst)
    ) ;_ mapcar
    (setq r-lst (uniquify (append l-lst u-lst)))
    (setq r-lst (displace-pts r-lst (mapcar '- (car l-lst))))
    (list (car l-lst) r-lst)
  ) ;_ defun-q 3d-ret-calc
  (defun-r '3d-ret-calc)

  (defun-q 3dr-ret (l-dat u-dat / b-obj r-def)
    "Creates a 3d polyline and extruded solid via passed xdata lists"
    (declare
      (vars (l-dat list (data-list-p l-dat)) (u-dat list (data-list-p u-dat)))
    ) ;_ declare
    (if
      (vl-every
       '(lambda (x) (/= "None" (xd-value "i-rail_type" x)))
        (list l-dat u-dat)
      ) ;_ vl-every
      (progn
        (setq b-obj (p-blk "ret"))
        (setq r-def (3d-ret-calc l-dat u-dat))
        (poly&pipe b-obj (cadr r-def))
        (setq b-obj (insert-block modelspace (car r-def) b-obj))
        b-obj
      ) ;_ progn
    ) ;_ if
  ) ;_ defun-q 3dr-ret
  (defun-r '3dr-ret)

  (defun-q poly&pipe (con lst / ret)
    "Creates a 3d polyline and extruded solid via passed vertex list"
    (declare (vars (con vla-object) (lst list (pts-list-p lst))))
    (vla-put-layer
      (vla-add3dpoly
        con
        (vlax-safearray-fill
          (vlax-make-safearray
            vlax-vbDouble
            (cons 0 (1- (length (apply 'append lst))))
          ) ;_ vlax-make-safearray
          (apply 'append lst)
        ) ;_ vlax-safearray-fill
      ) ;_ vla-add3dpoly
      "No Plot"
    ) ;_ vla-put-layer
    (foreach obj
      (draw-pipe-3d
        (draw-fillet-3d con (safe-calc-fillet-3d lst 2.25))
        0.83
      ) ;_ draw-pipe-3d
      (vla-put-layer obj "Rail Handgrab")
      (add-to-list 'ret obj)
    ) ;_ foreach
    ret
  ) ;_ defun-q poly&pipe
  (defun-r 'poly&pipe)
