  (set-file-docstring "Functions for modeling 3d stairs & rails")

  (psc-include
    (list
      "rail/guard-rail.lsp"
      "rail/migrations.lsp"
      "rail/stair-rail.lsp"
      "rail/wall-rail.lsp"
      "rail/return.lsp"
      "shapes/shape.lsp"
      "stair/details.lsp"
      "stair/details/3d-conns.lsp"
      "stair/derived.lsp"
      "stair/migrations.lsp"
      "stair/stringer.lsp"
      "stair/tread.lsp"
      "shapes/calc-tmatrix.lsp"
      "stair/util.lsp"
    ) ;_ list
  ) ;_ psc-include

  (defun-q Draw-3D_Stair (data ins / blockObj d-str d-trd d-conc d-stn d-qtr d-hclr)
    "Creates a 3d block of a stair via passed xdata and insert"
    (declare (vars (data list (data-list-p data)) (ins list (point-p ins))))
    (setq blockobj (p-blk (xd-value "assembly" data)))
    (setup-env T)

    ;;; Ins-pt Ref
    (if nil
      (vla-put-layer-r
        (vla-AddSphere blockObj (3dpt 0 0 0) 1.0)
        "No Plot"
      ) ;_ vla-put-layer-r
    ) ;_ if

    ;;; draw stringers
    (defun d-str ( / n-pts)
      (setq n-pts (calc-nose-pts))
      (mapcar ;;; left segs
       '(lambda (stringObj)
          (if stringObj
            (vla-put-layer-r
              (vla-transformby-r
                stringObj
                (calc-tmatrix
                  90 0 0
                  0 (* dir (- (/ width 2.0) l-string_wid)) 0
                  nil nil nil
                ) ;_ calc-tmatrix
              ) ;_ vla-transformby-r
              "Stair Stringers"
            ) ;_ vla-put-layer-r
          ) ;_ if
        ) ;_ lambda
        (mapcar
         '(lambda (seg / f-bot pts vol shp)
            (if
              (setq vol
                (car (add-stair-stringer (list seg) blockObj "L" "0" "0"))
              ) ;_ setq
              (progn
                (setq vol
                  (plineobj->solidobj
                    vol
                    (* (- dir) l-string_wid)
                    T
                  ) ;_ plineobj->solidobj
                ) ;_ setq
                (setq f-bot
                  (and
                    (<= (+ flight_hgt l-nosing_top) l-string_dpt)
                    (= starter_p "1")
                  ) ;_ and
                ) ;_ setq
                (setq pts
                  ( (lambda (lst)
                      (cdr
                        (nth
                          (vl-position
                            (read
                              (strcat "STRING-" (vl-symbol-name seg))
                            ) ;_ read
                            (mapcar 'car lst)
                          ) ;_ vl-position
                          lst
                        ) ;_ nth
                      ) ;_ cdr
                    ) ;_ lambda
                    (stair-stringer-calc "L")
                  )
                ) ;_ setq
                (if (member (substr l-string_mat 1 1) '("C" "M"))
                  (progn
                    (setq shp
                      (plineobj->solidobj
                        (vla-transformby-r
                          (add-shape blockObj '(0 0 0) l-string_mat)
                          (calc-tmatrix
                            0
                            (* 90 dir)
                            (if (and (not f-bot) (= seg 'mid-seg))
                              (* dir pitch) 0
                            ) ;_ if
                            (cond
                              ( (= f-bot T)
                                (* dir (caar pts))
                              ) ;_ condif
                              ( (= seg 'bot-seg)
                                (* dir (caadr (reverse pts)))
                              ) ;_ condif
                              ( (= seg 'mid-seg)
                                (- (caar n-pts) (* dir 3 tread_depth))
                              ) ;_ condif
                              ( (= seg 'top-seg)
                                (* dir (caar pts))
                              ) ;_ condif
                            ) ;_ cond
                            (cond
                              ( (= f-bot T)
                                (+ l-string_dpt)
                              ) ;_ condif
                              ( (= seg 'bot-seg)
                                (cadadr (reverse pts))
                              ) ;_ condif
                              ( (= seg 'mid-seg)
                                (-
                                  (+
                                    (/ l-nosing_mid (cos (dtr pitch)))
                                    (cadar n-pts)
                                  ) ;_ +
                                  (* 3 rise_hgt)
                                ) ;_ -
                              ) ;_ condif
                              ( (= seg 'top-seg)
                                (cadar pts)
                              ) ;_ condif
                            ) ;_ cond
                            0
                            nil
                            nil
                            nil
                          ) ;_ calc-tmatrix
                        ) ;_ vla-transformby-r
                        360
                        T
                      ) ;_ plineobj->solidobj
                    ) ;_ setq
                    (vla-boolean-r shp acIntersection vol)
                  ) ;_ progn
                  vol
                ) ;_ if
              ) ;_ progn
            ) ;_ if
          ) ;_ lambda
         '(bot-seg mid-seg top-seg)
        ) ;_ mapcar
      ) ;_ mapcar
      (mapcar ;;; left caps
       '(lambda (plineObj)
          (vla-transformby
            (vla-put-layer-r
              (plineObj->solidObj
                plineObj
                (* dir l-string_wid)
                T
              ) ;_ plineObj->solidObj
              "Stair Stringers"
            ) ;_ vla-put-layer-r
            (calc-tmatrix
              90 0 0
              0 (* dir (/ width 2.0)) 0
              nil nil nil
            ) ;_ calc-tmatrix
          ) ;_ vla-transformby
        ) ;_ lambda
        (add-stair-stringer '(cap) blockObj "L" "0" "0")
      ) ;_ mapcar
      (mapcar ;;; right segs
       '(lambda (stringObj)
          (if stringObj
            (vla-put-layer-r
              (vla-transformby-r
                stringObj
                (calc-tmatrix
                  90 0 0
                  0 (* (- dir) (- (/ width 2.0) r-string_wid)) 0
                  nil nil nil
                ) ;_ calc-tmatrix
              ) ;_ vla-transformby-r
              "Stair Stringers"
            ) ;_ vla-put-layer-r
          ) ;_ if
        ) ;_ lambda
        (mapcar
         '(lambda (seg / f-bot pts vol shp)
            (if
              (setq vol
                (car (add-stair-stringer (list seg) blockObj "R" "0" "0"))
              ) ;_ setq
              (progn
                (setq vol
                  (plineobj->solidobj
                    vol
                    (* dir r-string_wid)
                    T
                  ) ;_ plineobj->solidobj
                ) ;_ setq
                (setq f-bot
                  (and
                    (<= (+ flight_hgt r-nosing_top) r-string_dpt)
                    (= starter_p "1")
                  ) ;_ and
                ) ;_ setq
                (setq pts
                  ( (lambda (lst)
                      (cdr
                        (nth
                          (vl-position
                            (read (strcat "STRING-" (vl-symbol-name seg)))
                            (mapcar 'car lst)
                          ) ;_ vl-position
                          lst
                        ) ;_ nth
                      ) ;_ cdr
                    ) ;_ lambda
                    (with-data data '((stair-stringer-calc "R")))
                  )
                ) ;_ setq
                (if (member (substr r-string_mat 1 1) '("C" "M"))
                  (progn
                    (setq shp
                      (plineobj->solidobj
                        (vla-transformby-r
                          (add-shape blockObj '(0 0 0) r-string_mat)
                          (calc-tmatrix
                            0
                            (* 90 (- dir))
                            (if (and (not f-bot) (= seg 'mid-seg))
                              (* dir pitch) 0
                            ) ;_ if
                            (cond
                              ( (= f-bot T)
                                (* dir (caar pts))
                              ) ;_ condif
                              ( (= seg 'bot-seg)
                                (* dir (caadr (reverse pts)))
                              ) ;_ condif
                              ( (= seg 'mid-seg)
                                (- (caar n-pts) (* dir 3 tread_depth))
                              ) ;_ condif
                              ( (= seg 'top-seg)
                                (* dir (caar pts))
                              ) ;_ condif
                            ) ;_ cond
                            (cond
                              ( (= f-bot T)
                                (+ r-string_dpt)
                              ) ;_ condif
                              ( (= seg 'bot-seg)
                                (cadadr (reverse pts))
                              ) ;_ condif
                              ( (= seg 'mid-seg)
                                (-
                                  (+
                                    (/ r-nosing_mid (cos (dtr pitch)))
                                    (cadar n-pts)
                                  ) ;_ +
                                  (* 3 rise_hgt)
                                ) ;_ -
                              ) ;_ condif
                              ( (= seg 'top-seg)
                                (cadar pts)
                              ) ;_ condif
                            ) ;_ cond
                            0
                            nil
                            nil
                            nil
                          ) ;_ calc-tmatrix
                        ) ;_ vla-transformby-r
                        -360
                        T
                      ) ;_ plineobj->solidobj
                    ) ;_ setq
                    (vla-boolean-r shp acIntersection vol)
                  ) ;_ progn
                  vol
                ) ;_ if
              ) ;_ progn
            ) ;_ if
          ) ;_ lambda
         '(bot-seg mid-seg top-seg)
        ) ;_ mapcar
      ) ;_ mapcar
      (mapcar ;;; right caps
       '(lambda (plineObj)
          (vla-transformby
            (vla-put-layer-r
              (plineObj->solidObj
                plineObj
                (* (- dir) r-string_wid)
                T
              ) ;_ plineObj->solidObj
              "Stair Stringers"
            ) ;_ vla-put-layer-r
            (calc-tmatrix
              90 0 0
              0 (* (- dir) (/ width 2.0)) 0
              nil nil nil
            ) ;_ calc-tmatrix
          ) ;_ vla-transformby
        ) ;_ lambda
        (add-stair-stringer '(cap) blockObj "R" "0" "0")
      ) ;_ mapcar
    ) ;_ defun d-str

    ;;; draw treads
    (defun d-trd ()
      (mapcar
       '(lambda (plineObj)
          (vla-transformby
            (vla-put-layer-r
              (plineObj->solidObj
                plineObj
                (* dir (- width r-string_wid l-string_wid))
                T
              ) ;_ plineObj->solidObj
              "Stair Pans"
            ) ;_ vla-put-layer-r
            (calc-tmatrix
              90 0 0
              0 (* dir (- (/ width 2.0) l-string_wid)) 0
              nil nil nil
            ) ;_ calc-tmatrix
          ) ;_ vla-transformby
        ) ;_ lambda
        (add-stair-treads blockObj "0")
      ) ;_ mapcar
    ) ;_ defun d-trd

    ;;; draw fill/precast
    (defun d-conc ()
      (if (member (substr tread_style 1 3) '("100" "400"))
        (mapcar
         '(lambda (plineObj)
            (vla-transformby
              (vla-put-layer-r
                (apply
                  (if (= (vla-get-closed plineObj) :vlax-true)
                   'plineObj->solidObj
                   'plineObj->faceObj
                  ) ;_ if
                  (list
                    plineObj
                    (* dir (- width r-string_wid l-string_wid))
                    T
                  ) ;_ list
                ) ;_ apply
                "Stair Conc Fill"
              ) ;_ vla-put-layer-r
              (calc-tmatrix
                90 0 0
                0 (* dir (- (/ width 2.0) l-string_wid)) 0
                nil nil nil
              ) ;_ calc-tmatrix
            ) ;_ vla-transformby
          ) ;_ lambda
          (add-stair-conc blockObj)
        ) ;_ mapcar
      ) ;_ if
    ) ;_ defun d-conc

    ;;; draw safe-t-nose
    (defun d-stn ()
      (if (stair-stn-p)
        (mapcar
         '(lambda (plineObj)
            (vla-transformby
              (vla-put-layer-r
                (plineObj->solidObj
                  plineObj
                  (* dir (- width r-string_wid l-string_wid))
                  T
                ) ;_ plineObj->solidObj
                "Stair Safe-T-Nose"
              ) ;_ vla-put-layer-r
              (calc-tmatrix
                90 0 0
                0 (* dir (- (/ width 2.0) l-string_wid)) 0
                nil nil nil
              ) ;_ calc-tmatrix
            ) ;_ vla-transformby
          ) ;_ lambda
          (add-stair-stn blockObj "0")
        ) ;_ mapcar
      ) ;_ if
    ) ;_ defun d-stn

    ;;; draw quiet tread
    (defun d-qtr ()
      (if (= "1" quiet_tread)
        (progn
          (mapcar
           '(lambda (plineObj)
              (vla-transformby
                (vla-put-layer-r
                  (plineObj->solidObj
                    plineObj
                    (* dir (- width r-string_wid l-string_wid 2))
                    T
                  ) ;_ plineObj->solidObj
                  "Stair Quiet Tread"
                ) ;_ vla-put-layer-r
                (calc-tmatrix
                  90 0 0
                  0 (* dir (- (/ width 2.0) l-string_wid 1.0)) 0
                  nil nil nil
                ) ;_ calc-tmatrix
              ) ;_ vla-transformby
            ) ;_ lambda
            (add-stair-qtread blockObj "0")
          ) ;_ mapcar
          (foreach t/b '("bot" "top")
            (cond
              ( (= starter_p "1") (princ) )
              ( (member (symcat '(#t/b "_conn_det")) '("1" "2" "3"))
                ( (lambda (obj)
                    (if obj
                      (progn
                        (vla-put-normal obj (3dpt (list 0 (- dir) 0)))
                        (vla-move obj (3dpt 0 0 0)
                          (3dpt
                            (mapcar '*
                              (list dir dir 1)
                              (if (= t/b "bot")
                                (list 0 (- (/ width 2.0) l-string_wid 1) 0)
                                (list
                                  (+ bot_ext bot_cdist run top_cdist top_ext)
                                  (- (/ width 2.0) l-string_wid 1)
                                  (* rise_qty rise_hgt)
                                ) ;_ list
                              ) ;_ if
                            ) ;_ mapcar
                          ) ;_ 3dpt
                        ) ;_ vla-move
                        (setq obj
                          (plineobj->solidobj obj (- btwn_stringers 2.0) T)
                        ) ;_ setq
                        (vla-put-layer-r obj "Stair Quiet Tread")
                      ) ;_ progn
                    ) ;_ if
                  ) ;_ lambda
                  (if (= t/b "bot") (add-bot-ext-qt) (add-top-ext-qt))
                ) ;_ lambda func
              ) ;_ condif
              ( (member (symcat '(#t/b "_conn_det")) '("4" "5"))
                (mapcar '(lambda (obj)
                  (vla-put-normal obj (3dpt (list 0 (- dir) 0)))
                    (vla-move obj (3dpt 0 0 0)
                      (3dpt
                        (mapcar '*
                          (list dir dir 1)
                          (if (= t/b "bot")
                            (list 0 (- (/ width 2.0) l-string_wid 1) 0)
                            (list
                              (+ bot_ext bot_cdist run top_cdist top_ext)
                              (- (/ width 2.0) l-string_wid 1)
                              (* rise_qty rise_hgt)
                            ) ;_ list
                          ) ;_ if
                        ) ;_ mapcar
                      ) ;_ 3dpt
                    ) ;_ vla-move
                   (setq obj (plineobj->solidobj obj (- btwn_stringers 2.0) T))
                   (vla-put-layer-r obj "Stair Quiet Tread")
                  ) ;_ lambda
                  (mapcar
                   '(lambda (qt-cen)
                      (add-qt (car qt-cen) (- pan_thick) (cadr qt-cen))
                    ) ;_ lambda
                    (qt-cen-lst t/b)
                  ) ;_ mapcar
                ) ;_ mapcar
              ) ;_ condif
            ) ;_ cond
          ) ;_ foreach
        ) ;_ progn
      ) ;_ if
    ) ;_ defun d-qtr

    (defun d-hclr ( / clr-box-vol clr-arc-vol z-dir v-sol)
      (defun clr-box-vol (lst dst / sol)
        (setq
          lst (append lst (displace-pts (reverse lst) (list 0 dst)))
          sol (add-polyline blockobj '(0 0) lst T)
        ) ;_ setq
        (vla-put-normal sol (3dpt 0 -1 0))
        (vla-move sol (3dpt 0 0) (3dpt 0 (* width 0.5 z-dir)))
        (setq sol (plineobj->solidobj sol (* z-dir width) T))
      ) ;_ defun clr-box-vol
      (defun clr-arc-vol (t/b / a-dir ext)
        (setq
          a-dir (if (= ascend "Left") 1 -1)
          ext (eval (read (strcat (lowercase (to-string t/b)) "_ext")))
          t/b (if (= t/b 'top) 1 -1)
          ext (* (?f ext) dir t/b)
        ) ;_ setq
        (plineobj->solidobj
          (add-polyline
            blockobj
            (apply
             '(lambda (x z)
                (list (+ x ext) (* width -0.5 z-dir) (if (> 0 t/b) 0 z))
              ) ;_ lambda
              ((if (> 0 t/b) car last) (calc-nose-pts))
            ) ;_ apply
            (append
              (if (and (> 0 t/b) (> (?f bot_ext) 0))
                (list (list (- ext) 0))
              ) ;_ if
              (list
                (list 0 0 (calc-bulge (dtr (* t/b a-dir 90))))
                (list (* dir width t/b) (* z-dir width))
              ) ;_ list
              (if (> t/b 0)
                (mapcar
                 '(lambda (x) (list x (* z-dir (+ width dist_btwn))))
                  (list (* dir width t/b) 0)
                ) ;_ mapcar
                (list
                  (list (if (> (?f bot_ext) 0) (- ext) 0) (* z-dir width))
                ) ;_ list
              ) ;_ if
            ) ;_ append
            T
          ) ;_ add-polyline
          (+ head_clr (if (> t/b 0) 0 rise_hgt))
          T
        ) ;_ plineobj->solidobj
      ) ;_ clr-arc-vol
      (setq z-dir (if (= z_pos "Near") 1 -1))
      ;;; Bot Volume
      (setq v-sol
        (vla-boolean-r
          (clr-box-vol
            (mapcar
             '(lambda (x y)
                (mapcar '- (car (calc-nose-pts)) (list (* dir x) y))
              ) ;_ lambda
              (list 0 tread_depth (+ bot_ext width))
              (cons 0 (rpt-lst 2 rise_hgt))
            ) ;_ mapcar
            head_clr
          ) ;_ clr-box-vol
          acintersection
          (clr-arc-vol 'bot)
        ) ;_ vla-boolean-r
      ) ;_ setq
      ;;; Mid Volume
      (setq v-sol
        (vla-boolean-r
          v-sol
          acunion
          (clr-box-vol
            (append
              (list (car (calc-nose-pts)) (last (calc-nose-pts)))
              (if (> (?f top_ext) 0)
                (list
                  (mapcar '+
                    (last (calc-nose-pts))
                    (list (* dir top_ext) 0 0)
                  ) ;_ mapcar
                ) ;_ list
              ) ;_ if
            ) ;_ append
            head_clr
          ) ;_ clr-box-vol
        ) ;_ vla-boolean-r
      ) ;_ setq
      ;;; Top Volume
      (setq v-sol
        (vla-boolean-r
          v-sol
          acunion
          (clr-arc-vol 'top)
        ) ;_ vla-boolean-r
      ) ;_ setq
      (vla-put-entitytransparency v-sol 90)
      (vla-put-layer v-sol "Head Clear")
    ) ;_ defun d-hclr

    (with-data data
     '( (d-str) (d-trd) (d-stn) (d-conc) (d-qtr) (add-3d-conns)
        (if (> head_clr 0.0) (d-hclr)) (d-3dr)
      ) ;_ quoted-list
    ) ;_ with-data data

    (setq blockObj
      (vla-put-layer-r
        (insert-block modelSpace ins blockObj)
        "3D Stair"
      ) ;_ vla-put-layer-r
    ) ;_ setq
    (vla-put-rotation blockobj (?f rot))
    (assign-xdata blockObj "3D_Stair" (format-xdata data))
    (vlobj>ename blockObj)
  ) ;_ defun-q Draw-3D_Stair
  (defun-r 'Draw-3D_Stair)

  ;;; draw 3d rail
  (defun-q d-3dr ( / ret)
    "wrapper function for 3d rail block generation with 3d stair block"
    (declare with-data)
    (foreach side '("i" "o")
      (cond
        ( (wcmatch (sidekey-rail "type") "*None*") (princ))
        ( (wcmatch (sidekey-rail "type") "*Wall*")
          (3dr-add (stair-rail-assembly side)
                   (list (calc-wall-rail side "rail handgrab")))
        ) ;_ condif
        ( (wcmatch (sidekey-rail "type") "*Stair*")
          (3dr-add (stair-rail-assembly side)
                   (calc-stair-rail side "rail frame" "rail handgrab"))
        ) ;_ condif
        ( (= T) (princ))
      ) ;_ cond
    ) ;_ foreach
    (mapcar
     '(lambda (def / loc side 3d-p)
        (setq loc (xd-value "location" def) side (substr loc 1 1) 3d-p T)
        (3dr-add
          (xd-value "assembly" def)
          (list
            (calc-stair-guard-ins '3d def)
            (calc-guard-rail def)
          ) ;_ list
        ) ;_ 3dr-add
      ) ;_ lambda
      (stair-xdata->guard-xdata)
    ) ;_ mapcar
    ret
  ) ;_ defun-q d-3dr
  (defun-r 'd-3dr)

  ;;; add 3d rail
  (defun-q 3dr-add (name def / ins off obj)
    "Creates a 3d block of a stair rail or wall rail based on passed rail definition"
    (declare with-data (vars (name str) (def list)))
    (if (point-p (car def))
      (setq
        ins '(0 0 0)
        off (car def)
        def (cadr def)
      ) ;_ setq
      (setq ins '(0 0 0))
    ) ;_ if
    (setq obj (p-blk name))
    (add-calcd-objs obj T def)
    (setq obj (insert-block blockobj (?? off ins) obj))
    (if off
      (progn
        (if (< dir 0) (setq obj (mirror&delete obj ins 'y)))
        (cond
          ( (= side "b")
            (if (= z_pos "Far") (setq obj (mirror&delete obj off 'x)))
            (vla-put-rotation obj (* dir (/ pi 2)))
          ) ;_ condif
          ( (member (list side z_pos) '(("i" "Far") ("o" "Near")))
            (setq obj (mirror&delete obj off 'x))
          ) ;_ condif
          ( (= T) (list))
        ) ;_ cond
      ) ;_ progn
    ) ;_ if
    (vla-put-layer obj "3D Rail")
    (add-to-list 'ret obj)
  ) ;_ defun-q 3dr-add
  (defun-r '3dr-add)
