  (psc-include '("3D/3d-model.lsp"))

  (defun-q C:3D-Model ( / ent-lst acaddoc modelspace)
    "Opens a new drawing and generates a 3D model of selected assemblies"
    (vl-load-com)
    (if (setq ent-lst (get-stairland 'both T))
      (progn
        (vla-activate
          (setq acaddoc
            (vla-add
              (vla-get-documents acadobj)
              (strcat psc-src-dir "3D/3d_template.dwg")
            ) ;_ vla-add
          ) ;_ setq
        ) ;_ vla-activate
        (setq modelspace (vla-get-ModelSpace acaddoc))
        (Draw-3D-Model)
      ) ;_ progn
    ) ;_if
  ) ;_ defun-q C:3D-Model
  (defun-r 'C:3D-Model)

  (defun-q C:3D-Ret ()
    "Generates rail return blocks from stair block selection"
    (apply-by-twos
     '(lambda (e1 e2 / rot)
        (setq rot (vla-get-rotation (ename>vlobj e1)))
        (3dr-ret
          (cons
            (cons "blk-ins" (cdr (assoc 10 (entget e1))))
            (read-xdata e1 "")
          ) ;_ cons
          (cons
            (cons "blk-ins" (cdr (assoc 10 (entget e2))))
            (read-xdata e2 "")
          ) ;_ cons
        ) ;_ 3dr-ret
      ) ;_ lambda_func
      (get-stairland 'stair T)
    ) ;_ apply-by-twos
  ) ;_ defun-q C:3D-Ret
  (defun-r 'C:3D-Ret)

  (defun-q c:line->pipe ( / l-obj v-lst b-ins b-obj)
    "Converts a 3Dpolyline into an extruded pipe"
    (setq
      l-obj (ename>vlobj (car (entsel "\nSelect 3D Polyline: ")))
      v-lst (line-obj->vert-lst l-obj)
      b-ins (car v-lst)
      v-lst (displace-pts v-lst (mapcar '- b-ins))
      b-obj (p-blk "pipe")
    ) ;_ setq
    (vla-delete l-obj)
    (poly&pipe b-obj v-lst)
    (insert-block modelspace b-ins b-obj)
  ) ;_ defun-q c:line->pipe
  (defun-r 'c:line->pipe)

  (defun-q c:3d-in-place ( / ent dat app ins del obj)
    "Generates 3D blocks in current drawing from related 2D block selection"
    (setup-env T)
    (setq
      ent (car (entsel))
      app (if ent (xd-value "app_id" (read-xdata ent "")))
      dat (if app (read-xdata-migrate ent app))
      ins (getpoint "\n3D Block Ins: ")
    ) ;_ setq
    (initget "Y N")
    (setq del (= (getkword "\nDelete Ent?: [Yes/No] <N>: ") "Y"))
    (with-data dat
      (list
        (quote
          (cond
            ( (null dat) (error "No XDATA Found"))
            ( (null ins) (error "Insert is REQUIRED"))
            ( (= "Stair" (app-id-equiv (xd-value "app_id" dat)))
              (setq obj (draw-3d_stair dat ins))
            ) ;_ condif
            ( (= "Land" (app-id-equiv (xd-value "app_id" dat)))
              (setq obj (draw-3d_land dat ins))
            ) ;_ condif
            ( (= "Guard" (app-id-equiv (xd-value "app_id" dat)))
              (setq obj (draw-3d_guard dat ins))
            ) ;_ condif
            ( (= "Embed" (app-id-equiv (xd-value "app_id" dat)))
              (setq obj (draw-3d_embed dat ins))
            ) ;_ condif
            ( (= T) (error "Unsupported App-ID"))
          ) ;_ cond
        ) ;_ quote
      ) ;_ list
    ) ;_ with-data
    (if del (vla-delete (ename>vlobj ent)))
    obj
  ) ;_ defun-q c:3d-in-place
  (defun-r 'c:3d-in-place)

  (defun-q c:3d-batch-plan ( / del lst dat app ins rot fnc)
    "Generates 3D models of multiple selected embed and/or guard assemblies"
    (setup-env T)
    (initget "Y N")
    (setq
      del (= (getkword "\nDelete Ents?: [Yes/No] <N>: ") "Y")
      lst (get-sset '("Embed,Guard"))
    ) ;_ setq
    (foreach ent lst
      (setq
        dat (read-xdata ent "")
        app (xd-value "app_id" dat)
        ins (cdr (assoc 10 (entget ent)))
        rot (vla-get-rotation (ename>vlobj ent))
        fnc (eval (read (strcat "draw-3d_" app)))
      ) ;_ setq
      (if del (entdel ent))
      (with-data dat '((vla-put-rotation (fnc dat ins) rot)))
    ) ;_ foreach
  ) ;_ defun-q c:3d-batch-plan
  (defun-r 'c:3d-batch-plan)
