(psc-include '("dialog/dialog.lsp"
               "shapes/shape.lsp"))

(setq *drawshape-name* "W10x12"
      *drawshape-list-type* "W"
      *drawshape-insert-on* "Top"
      *drawshape-perspective* "Section"
      *drawshape-L-orient* "H")

(defun-q c:drawshape ( / name profile blockname data def ctr plines ent ins ang match-last-p)
  "Draws an AISC shape in section, plan, or elevation

When drawing in section view, you may be prompted for an attachment point, mirroring,
and/or rotation. The attachment point can be Top (for beams in main section views) or
Center (for columns in plan view). Mirroring is along the spine for channels, and will
flip the leg orientation for unequal angles. Angles and columns (i.e. things attached at
Center) may be rotated after inserting. This table summarizes the prompts for each shape
type in Section perspective:

| Type | Attachment point | Mirroring         | Rotation             |
|------+------------------+-------------------+----------------------|
| W    | Prompt top/ctr   | N                 | When attached at ctr |
| HSS  | Prompt top/ctr   | N                 | When attached at ctr |
| C/MC | Top of spine     | Y                 | N                    |
| L    | Corner point     | When legs unequal | Y                    |

When drawing in plan or elevation view, you pick two points and the shape is drawn along
the line between them. In plan view, the line is the centerline, while in elevation view,
the line is the top line."
  (setup-env nil)
  
  (setq name (getdkword '(128 "List MatchLast") '*drawshape-name* "Shape name or"))
  (if (= name "List")
      (setq name (shapelist-init (getdkword '(0 "W C L") '*drawshape-list-type* "Profile?"))))
  (if (null name)
      (error "DRAWSHAPE: No shape selected")
      (setq *drawshape-name* name))
  (setq perspective (getdkword '(0 "Section Plan Elevation")
                               '*drawshape-perspective*
                               "Perspective?"))
  (with-data (shape-def name T)
    (list (list (read (strcat "drawshape--draw-" (lowercase perspective)))))))
(defun-r 'c:drawshape)
