(setq dualoffset-dist 0.83)

(defun-q c:dualoffset ( / dist pline)
  "Offsets a polyline in both directions and closes both ends

This command is equivalent to offsetting a polyline twice (once in the positive direction,
and once in the negative), drawing connecting lines between the ends of both resulting
lines, and joining the result into a closed polyline.

The joining lines may optionally be bulged out to create a 180 degree fillet at the
end (useful for drawing bent pipe handgrabs in elevation). When DUALOFFSET is run, it
first displays the current settings for rounding either end of the line, like this:

Current settings: Round head=No    Round tail=No

on the line before its main prompt. At the prompt, you can input the offset distance to
continue or optionally input H or T to toggle the rounding setting for that end. H, for
head, means to round the end at the first point of the line, and T, for Tail, means to
round the other end. These settings are preserved between calls to DUALOFFSET within the
same drawing."
  (while (not (= (type dist) 'REAL))
    (princ (strcat "\nCurrent settings: Round head="
                   (if dualoffset-round-head-p "Yes" "No")
                   "\tRound tail="
                   (if dualoffset-round-tail-p "Yes" "No")))
    (initget (+ 2 4 128) "Head Tail")
    (setq dist (getkword (strcat "\nSpecify offset distance or [Head/Tail] <"
                                 (to-string dualoffset-dist)
                                 ">: ")))
    (cond ((begins-with (lowercase (to-string dist)) "h")
           (setq dualoffset-round-head-p (not dualoffset-round-head-p)))
          ((begins-with (lowercase (to-string dist)) "t")
           (setq dualoffset-round-tail-p (not dualoffset-round-tail-p)))
          ((not dist)
           (setq dist dualoffset-dist))
          ((setq dist (distof dist))
           (setq dualoffset-dist dist))))

  (setq pline (car (entsel)))
  (if (not (= (vla-get-ObjectName (setq pline (ename>vlobj pline))) "AcDbPolyline"))
      (error "DUALOFFSET: object must be a polyline"))

  (offset&close pline dist T (mapcar '(lambda(x) (if x 1 0))
                                     (list dualoffset-round-head-p dualoffset-round-tail-p)))
  (print))
(defun-r 'c:dualoffset)
