(psc-include '("common-data.lsp"))

(defun-q c:dumplands (/ enames keys ename data datas col row table proc-key)
  "Dumps raw landing data to an Excel spreasheet, opens it, and formats it

Columns will be formatted so as to hilight patterns of values. When a column has all the
same value (or blank) in all rows, it will be left alone. But when a column has at least
one row that doesn't match the rest, all rows will be colored by value so that the pattern
can be visually distinguished."
  (setq enames (get-stairland 'land T)
        keys '(("number" "Stair")
               ("level" "Level")
               ("sequence" "Sequence")
               ("side" "Side")
               ("finish" "Finish")
               ("comment" "Comment")
               ("tread_style" "Tread style")
               ("chk_hold" "Checker plate holdback")
               ("quiet_tread" "Quiet Tread" T)
               ("tot_dpt" "Nosing to wall")
               ("stair_ndist" "Front stair nosing")
               ("plt_dpt" "Platform depth")
               ("back_gap" "Back gap")
               ("tot_wid" "Shaft width")
               ("plt_wid" "Platform width")
               ("left_gap" "Left gap")
               ("right_gap" "Right gap")
               ("front_chan_size" "Front chan size")
               ("back_chan_size" "Back chan size")
               ("back_chan_loc" "Back chan location")
               ("flip_back" "Back channel flipped" T)
               ("l_brace" "Left x-bracing" T)
               ("l_brace_hold" "Left x-bracing hold" T)
               ("r_brace" "Right x-bracing" T)
               ("r_brace_hold" "Right x-bracing hold" T)
               ("f_brace" "Front x-bracing" T)
               ("f_brace_hold" "Front x-bracing hold" T)
               ("b_brace" "Back x-bracing" T)
               ("b_brace_hold" "Back x-bracing hold" T)

               ("fl_chan" "Front-left chan detail")
               ("fl_ext" "Front-left chan extension")
               ("fl_supp" "Front-left support detail")
               ("fl_len" "Front-left support length")
               ("fl_xtra_holes" "Front-left extra holes")
               ("fl_plt_hgt" "Front-left plate height")
               ("fl_plt_wid" "Front-left plate width")
               ("fl_plt_thk" "Front-left plate thickness")
               ("fl_kick_style" "Front-left kick brace style")
               ("fl_kick_len" "Front-left kick brace length")
               ("fl_kick_deck" "Front-left deck direction")
               ("fl_angle" "Front-left ledger angle")
               ("fl_gap" "Front-left gap to structure")

               ("fr_chan" "Front-right chan detail")
               ("fr_ext" "Front-right chan extension")
               ("fr_supp" "Front-right support detail")
               ("fr_len" "Front-right support length")
               ("fr_xtra_holes" "Front-right extra holes")
               ("fr_plt_hgt" "Front-right plate height")
               ("fr_plt_wid" "Front-right plate width")
               ("fr_plt_thk" "Front-right plate thickness")
               ("fr_kick_style" "Front-right kick brace style")
               ("fr_kick_len" "Front-right kick brace length")
               ("fr_kick_deck" "Front-right deck direction")
               ("fr_angle" "Front-right ledger angle")
               ("fr_gap" "Front-right gap to structure")

               ("bl_chan" "Back-left chan detail")
               ("bl_ext" "Back-left chan extension")
               ("bl_supp" "Back-left support detail")
               ("bl_len" "Back-left support length")
               ("bl_xtra_holes" "Back-left extra holes")
               ("bl_plt_hgt" "Back-left plate height")
               ("bl_plt_wid" "Back-left plate width")
               ("bl_plt_thk" "Back-left plate thickness")
               ("bl_kick_style" "Back-left kick brace style")
               ("bl_kick_len" "Back-left kick brace length")
               ("bl_kick_deck" "Back-left deck direction")
               ("bl_angle" "Back-left ledger angle")
               ("bl_gap" "Back-left gap to structure")

               ("br_chan" "Back-right chan detail")
               ("br_ext" "Back-right chan extension")
               ("br_supp" "Back-right support detail")
               ("br_len" "Back-right support length")
               ("br_xtra_holes" "Back-right extra holes")
               ("br_plt_hgt" "Back-right plate height")
               ("br_plt_wid" "Back-right plate width")
               ("br_plt_thk" "Back-right plate thickness")
               ("br_kick_style" "Back-right kick brace style")
               ("br_kick_len" "Back-right kick brace length")
               ("br_kick_deck" "Back-right deck direction")
               ("br_angle" "Back-right ledger angle")
               ("br_gap" "Back-right gap to structure")

               ("front_bolt_to" "Front chan bolt to...")
               ("front_l-stair_offset" "Front-left stair offset")
               ("front_l-stair_wid" "Front-left stair width")
               ("front_l-stair_vbolt" "Front-left stair bolt depth")
               ("front_l-stair_hbolt" "Front-left stair bolt spc 1")
               ("front_l-stair_hbolt2" "Front-left stair bolt spc 2")
               ("front_l-stair_dbl" "Front-left stair double bolt")
               ("front_r-stair_offset" "Front-right stair offset")
               ("front_r-stair_wid" "Front-right stair width")
               ("front_r-stair_vbolt" "Front-right stair bolt depth")
               ("front_r-stair_hbolt" "Front-right stair bolt spc 1")
               ("front_r-stair_hbolt2" "Front-right stair bolt spc 2")
               ("front_r-stair_dbl" "Front-right stair double bolt" T)

               ("back_bolt_to" "Back chan bolt to...")
               ("back_l-stair_offset" "Back-left stair offset")
               ("back_l-stair_wid" "Back-left stair width")
               ("back_l-stair_vbolt" "Back-left stair bolt depth")
               ("back_l-stair_hbolt" "Back-left stair bolt spc 1")
               ("back_l-stair_hbolt2" "Back-left stair bolt spc 2")
               ("back_l-stair_dbl" "Back-left stair double bolt")
               ("back_r-stair_offset" "Back-right stair offset")
               ("back_r-stair_wid" "Back-right stair width")
               ("back_r-stair_vbolt" "Back-right stair bolt depth")
               ("back_r-stair_hbolt" "Back-right stair bolt spc 1")
               ("back_r-stair_hbolt2" "Back-right stair bolt spc 2")
               ("back_r-stair_dbl" "Back-right stair double bolt" T)))

  (defun proc-key (key / name toggle-p val)
    (setq val (to-string (xd-value (car key) data))
          name (cadr key)
          toggle-p (= (length key) 3))
    (if toggle-p
        (setq val (if (= val "1") "X" "")))
    val)

  (foreach ename enames
           (setq data (draw-xdata-migrate 'ename "Land")
                 data (mapcar 'proc-key keys))
           (add-to-list 'datas data))

  (setq filename (strcat (vla-get-Path acadDoc)
                         "\\"
                         (xd-value "number" (get-job-info))
                         " land "
                         (caar datas)
                         " check.csv"))

  (setq try-csv (vl-catch-all-apply 'write-csv
                                    (list (cons (mapcar 'cadr keys) datas) filename T)))
  (if (vl-catch-all-error-p try-csv)
      (error (strcat "Cannot write " filename ". Ensure the file is not open and try again."))
      (startapp (findfile "checkopen/dist/checkopen.exe")
                (strcat "\"" filename "\""))))
(defun-r 'c:dumplands)

