(psc-include '("common-data.lsp"))

(defun-q c:dumpstairs (/ enames keys ename data datas col row table proc-key)
  "Dumps raw stair data to an Excel spreadsheet, opens it, and formats it

Columns will be formatted so as to hilight patterns of values. When a column has all the
same value (or blank) in all rows, it will be left alone. But when a column has at least
one row that doesn't match the rest, all rows will be colored by value so that the pattern
can be visually distinguished."
  (setq enames (get-stairland 'stair T)
        keys '(("number" "Stair")
               ("level" "Level")
               ("sequence" "Sequence")
               ("finish" "Finish")
               ("comment" "Comment")
               ("ascend" "Ascend")
               ("width" "Width")
               ("dist_btwn" "Dist between")
               ("rise_hgt" "Rise")
               ("rise_qty" "Rise #")
               ("l-nosing_top" "L top nosing")
               ("l-nosing_mid" "L mid nosing")
               ("l-nosing_bot" "L bot nosing")
               ("r-nosing_top" "R top nosing")
               ("r-nosing_mid" "R mid nosing")
               ("r-nosing_bot" "R bot nosing")
               ("tread_depth" "Tread depth")
               ("tread_style" "Tread style")
               ("tread_thick" "Tread thickness")
               ("pan_mat" "Pan material")
               ("tread_mat" "Tread material")
               ("quiet_tread" "Quiet Tread" T)
               ("kickback" "Kickback" T)
               ("l-string_mat" "L stringer material")
               ("r-string_mat" "R stringer material")
               ("l-string_xtra_tex" "L stringer top ext")
               ("l-string_xtra_bex" "L stringer bot ext")
               ("r-string_xtra_tex" "R stringer top ext")
               ("r-string_xtra_bex" "R stringer bot ext")
               ("bot_conn" "Bot detail")
               ("bot_ext" "Bot tread ext")
               ("bot_cdist" "Bot conn dist")
               ("bot_hdw" "Bot hardware")
               ("bot_member" "Bot conn member")
               ("bot_vbolt" "Bot vert bolt dist")
               ("bot_hbolt" "Bot horz bolt dist 1")
               ("bot_hbolt2" "Bot horz bolt dist 2")
               ("bot_slab" "Bot slab thickness")
               ("bot_blkout" "Bot blockout")
               ("top_conn" "Top detail")
               ("top_ext" "Top tread ext")
               ("top_cdist" "Top conn dist")
               ("top_hdw" "Top hardware")
               ("top_member" "Top conn member")
               ("top_vbolt" "Top vert bolt dist")
               ("top_hbolt" "Top horz bolt dist 1")
               ("top_hbolt2" "Top horz bolt dist 2")
               ("top_slab" "Top slab thickness")
               ("top_blkout" "Top blockout")
               ("stn_style" "STN style")
               ("stn_tb_nclr" "STN TB color")
               ("stn_tb_iclr" "STN TB iso color")
               ("stn_tb_ins1" "STN TB insert 1")
               ("stn_tb_ins2" "STN TB insert 2")
               ("stn_tb_sku" "STN TB SKU")
               ("stn_mid_nclr" "STN mid color")
               ("stn_mid_iclr" "STN mid iso color")
               ("stn_mid_ins1" "STN mid insert 1")
               ("stn_mid_ins2" "STN mid insert 2")
               ("stn_mid_sku" "STN mid SKU")
               ("stn_tb_only" "STN TB only" T)
               ("rail_style" "Rail style")
               ("rail_lvl-off" "Rail level-off" T)
               ("rail_topmnt" "Rail top mount" T)
               ("i-rail_type" "I rail type")
               ("i-rail_top_ext" "I rail top ext")
               ("i-rail_bot_ext" "I rail bot ext")
               ("i-rail_cutoff" "I rail cutoff")
               ("i-rail_top_dloop" "I rail top D-loop" T)
               ("i-rail_bot_dloop" "I rail bot D-loop" T)
               ("o-rail_type" "O rail type")
               ("o-rail_top_ext" "O rail top ext")
               ("o-rail_bot_ext" "O rail bot ext")
               ("o-rail_cutoff" "O rail cutoff")
               ("o-rail_top_dloop" "O rail top D-loop" T)
               ("o-rail_bot_dloop" "O rail bot D-loop" T)))

  (defun proc-key (key / name toggle-p val)
    (setq val (to-string (xd-value (car key) data))
          name (cadr key)
          toggle-p (= (length key) 3))
    (if toggle-p
        (setq val (if (= val "1") "X" "")))
    val)

  (foreach ename enames
           (setq data (draw-xdata-migrate 'ename "Stair")
                 data (mapcar 'proc-key keys))
           (add-to-list 'datas data))

  (setq filename (strcat (vla-get-Path acadDoc)
                         "\\"
                         (xd-value "number" (get-job-info))
                         " stair "
                         (caar datas)
                         " check.csv"))

  (setq try-csv (vl-catch-all-apply 'write-csv
                                    (list (cons (mapcar 'cadr keys) datas) filename T)))
  (if (vl-catch-all-error-p try-csv)
      (error (strcat "Cannot write " filename ". Ensure the file is not open and try again."))
      (startapp (findfile "checkopen/dist/checkopen.exe")
                (strcat "\"" filename "\""))))
(defun-r 'c:dumpstairs)
