(setq *landing-embeds-delete-p* "N")

(defun-q c:landing-embeds (/ enames delete-p lands floor-ys grid-xs get-floor-y get-grid-x)
  "Draws EM-4s for landing ledgers in main section

The selection filter for this command allows main landing blocks, grid lines (lines only,
not polylines, on CENTER layer), and floor-to-floor dimensions (those with 90 degree
angles and no text overwrite). If only landings are selected, embeds are located relative
to the channels per the standard detail. Selecting grid lines or dimensions causes the
embed inserts to be rounded to the nearest inch relative to the nearest grid line or
bottom of the enclosing dimension."
  (setq enames (ss->lst
                (ssget (dxf-filter '((or (appid "Land")
                                         (and (type "DIMENSION")
                                              (layer "DIM")
                                              (text "")
                                              (angle #(dtr 90)))
                                         (and (type "LINE")
                                              (layer "CENTER"))))))))

  (foreach ename enames
           ((lambda (def / objtype)
              (setq objtype (cdr (assoc 0 def)))
              (cond ((= objtype "DIMENSION")
	             (add-to-list 'floor-ys
                                  ;; account for dimension draw direction
                                  (min (caddr (assoc 13 def))
				       (caddr (assoc 14 def)))))
                    ((and (= objtype "LINE")
                          (=f (cadr (assoc 10 def))
                              (cadr (assoc 11 def))
                              0.01))
                     (add-to-list 'grid-xs (cadr (assoc 10 def))))
                    ((= objtype "INSERT")
	             (add-to-list 'lands ename))))
            (entget ename)))

  (if lands
      (progn
        (setq delete-p (= (getdkword '(0 "Y N") '*landing-embeds-delete-p* "Delete landings?") "Y"))
        (sort! 'floor-ys '>)))

  (defun get-floor-y (em-y / nearest)
    (setq nearest (vl-member-if '(lambda (floor-y) (> em-y floor-y)) floor-ys))
    (if nearest (car nearest)))

  (defun get-grid-x (em-x / nearest diff)
    (setq nearest (car grid-xs))
    (foreach grid-x (cdr grid-xs)
             (if (< (abs (- em-x grid-x))
                    (abs (- em-x nearest)))
                 (setq nearest grid-x)))
    nearest)

  (defun process-land (ename / inner def ins)
    (defun inner (/ f-ins b-ins em-ins draw-em)
      ;; calculate insert point at this corner
      (defun em-ins (fb lr / fb-short lr-short num dx dy back-chan relfix)
        ;; adjust back chan
        (defun back-chan ()
	  (if (= flip_back "0") back_chan_loc (- back_chan_loc back_chan_wid)))
        ;; fix n relative to m
        (defun relfix (n m)
          (if (/= n m)
              ;; IEEE 754 can go to hell
              (+ m (fix (- n m -0.0001)))
            n))
        ;; adjust x
        (defun dx (em-x / grid-x land-x)
          (setq land-x (car ins)
                em-x (+ (* dir 0.5) (* dir (fix em-x)))
                grid-x (get-grid-x em-x))
          (if grid-x
              (setq em-x (- (relfix (+ em-x land-x) grid-x) land-x)))
          em-x)
        ;; adjust y
        (defun dy (chan-dpt / land-y em-y floor-y)
          (setq land-y (cadr ins)
                em-y (- land-y flr_thick chan-dpt 2)
                floor-y (get-floor-y em-y))
          (- (relfix em-y (if floor-y floor-y em-y)) land-y))
        
	(setq fb-short (substr fb 1 1)
	      lr-short (substr lr 1 1)
	      num (symcat '(#fb-short #lr-short "_chan_num")))
	(displace-pt ins
          (cond ((member num '("60" "70"))
	         (list (dx (if (= fb "front") 5 (- (back-chan) 3)))
                       (dy (lrkey "chan_dpt"))))
	        ((member num '("66" "77"))
	         (list (dx (+ (if (= fb "front") 0 (back-chan)) (/ (fbkey "chan_wid") 2.0)))
                       (dy (fbkey "chan_dpt")))))))

      (defun draw-em (ins / layer blockObj)
        (defun layer (l) (if delete-p l "NO PLOT"))
	(setq blockObj (p-blk "EM-4"))
        (vla-put-Layer (add-rectangle blockObj 10 10 '(-5 -5))
                       (layer "SOLID"))
        (mapcar '(lambda(pts)
                   (vla-put-LinetypeScale (vla-put-Layer-r (add-polyline blockObj '(0 0) pts nil)
							   (layer "CENTER"))
					  0.125))
		'(((0 6 0) (0 -6 0))
		  ((6 0 0) (-6 0 0))))
	(vla-put-Layer (insert-block modelSpace ins blockObj)
                       (layer (getvar 'clayer))))

      (mapcar 'draw-em
        (uniquify
         (strip-nil
          (map-apply 'em-ins
	    '(("front" "left")
	      ("front" "right")
	      ("back" "left")
	      ("back" "right")))))))

    (setq def (read-xdata (ename>vlobj ename) "Land")
          ins (if delete-p (get-ins-and-delete ename) (cdr (assoc 10 (entget ename)))))
    (with-data def '((inner))))

  (mapcar 'process-land lands)
  (princ))
(defun-r 'c:landing-embeds)
