(psc-include
 '(
   "land/land_supp.lsp"
   "land/topview.lsp"
   "land/util.lsp"
   "shapes/shape.lsp"
   "stair/topview.lsp"
   ))

(defun-q c:topview ( / enames ename ins tmp data break stairs db pdb stairp)
  "Draws plan views of stairs and landings

To draw a plan view, run this command, select up to 3 stairs and 2 landings, press ENTER,
and click to insert. You can have more than 2 landings in a plan view, but they probably
won't be laid out how you want.

Selecting a single stair will prompt to show it as full or broken, and if broken, which
end to show. By default, this command will prefer to show the lower or middle flight as
full. It will show the top flight as full if there are only two flights and the top one
has a zero distance between."
  ;; load linetypes, layers and textstyles
  (setup-env nil)
  
  (defun stairp (ename) (assoc -3 (entget ename '("Stair"))))

  (setq enames (get-stairland 'both T))
  (if enames (progn
    (setq ins (getpoint "\nInsert: ")
          data (read-xdata (car enames) (if (stairp (car enames)) "Stair" "Land")))
    (cond
     ;; 1 block selected
     ((= 1 (length enames))
      (with-data data
        '((if (not (stairp (car enames)))
              (draw-land-top-view data ins)
            (progn
              (setq tmp (/ (tk 'dist_btwn) 2.0))
              (if (> (cadr (assoc "rise_qty" data)) 4)
                  (progn
                    (initget 0 "No Bot Top")
                    (setq break (getkword "\nBreak? [No/Bot/Top] "))
                    (if (not break) (setq break "No"))
                    (draw-stair-top-view data ins break tmp))
                (draw-stair-top-view data ins "No" tmp)))))))
     ;; multiple blocks selected
     ((> (length enames) 1)
      (tk '(setq stairs 0
                 pdb dist_btwn))
      (mapcar '(lambda (x) (if (stairp x) (inc! 'stairs))) enames)
      (cond ((= stairs 1)
             (setq stairs '("No")))
            ((= stairs 2)
             (setq tmp (read-xdata (last enames) "Stair")
                   stairs (if (and (setq tmp (assoc "dist_btwn" tmp))
                                   (= (cadr tmp) 0))
                              '("Top" "No")
                            '("No" "Bot"))))
            ((= stairs 3)
             (setq stairs '("Top" "No" "Bot")))
            ((> stairs 3)
             (error "TOPVIEW: Too many stairs")))
      (foreach ename enames
        (setq data (read-xdata ename (if (stairp ename) "Stair" "Land")))
        (with-data data
          '((if (not (stairp ename))
                (draw-land-top-view data ins)
              (progn
                (setq tmp (car stairs)
                      stairs (cdr stairs))
                (draw-stair-top-view data ins tmp (/ pdb 2.0))
                (if (= tmp "No")
                    (setq ins (list ((if (or (and (= ascend "Left")
                                                  (= z_pos "Far"))
                                             (and (= ascend "Right")
                                                  (= z_pos "Near")))
                                         - +)
                                     (car ins)
                                     bot_ext
                                     bot_cdist
                                     run
                                     top_cdist
                                     top_ext)
                                    ((if (= z_pos "Near") + -)
                                     (cadr ins)
                                     (/ (- dist_btwn pdb) 2.0))
                                    0)))
                (setq pdb dist_btwn))))))))))
  (princ))
(defun-r 'c:topview)
