(psc-include '(
               "common-data.lsp"
               "rail/return.lsp"
               "rail/topview.lsp"
               "stair/util.lsp"
               ))

(defun-q c:topviewrails (/
                         enames
                         stairs
                         clean-ent
                         strings
                         )
  "Converts an E drawing plan view to R drawing style

To format a plan view for the rail layout, run TOPVIEWRAILS, select all the blocks in the
plan view that were created with TOPVIEW, and press ENTER. Material tags and hole
centerlines will be removed from the block. Landing channels will be put on the NO PLOT
layer. Stair rails, guard rails, wall rails, and handgrab returns will be drawn along with
their tags."
  ;; remove unnecessary lines from topview
  (defun clean-ent (ename / blockdef ents origin bd)
    (defun blockdef (name)
      (vla-Item (vla-get-Blocks acadDoc) name))
    ((lambda(/ blockObj old-name new-name)
       (setq blockObj (ename>vlobj ename)
             old-name (vla-get-EffectiveName blockObj)
             new-name (p-blk-name (substr old-name 1 (vl-string-position (ascii ".") old-name))))
       (vla-ConvertToStaticBlock blockObj new-name)
       (setq bd (blockdef new-name)
             origin (vlax-get bd 'origin))))
    (vla-put-Origin bd (vlax-3d-point origin))
    (vlax-for ent bd
              ((lambda(/ layer)
                 (setq layer (vla-get-Layer ent))
                 (if (or (member layer '("HIDDEN" "CENTER"))
                         (and (= (vla-get-ObjectName ent) "AcDbText")
                              (member (vla-get-TextString ent) strings)))
                     (vla-Delete ent)
                     (if (= layer "OBJ")
                         (vla-put-Layer ent "NO PLOT")))))))

  (setq enames (ss->lst (ssget (list (list -3 (list "StairTop,LandTop")))))
        stairs (filter enames '(assoc -3 (entget x '("StairTop"))))
        stairs (mapcar '(lambda(stair / ins)
                         (setq ins (cdr (assoc 10 (entget stair))))
                         (append (list (list "ins" ins)
                                       (list "insx" (car ins)))
                                 (read-xdata stair "StairTop")))
                       stairs)
        stairs (vl-sort stairs '(lambda(a b / alvl blvl aseq bseq)
                                  (setq alvl (cadr (assoc "level" a))
                                        blvl (cadr (assoc "level" b)))
                                  (if (= alvl blvl)
                                      (progn
                                        (setq aseq (cadr (assoc "sequence" a))
                                              bseq (cadr (assoc "sequence" b)))
                                        (< aseq bseq))
                                    (< alvl blvl))))
        strings (append stringers channels)
        clayer (vla-getvar 'clayer))
  (mapcar 'clean-ent enames)
  (vla-Regen acadDoc acAllViewPorts)
  (set-active-layer "RAILS")
  (mapcar 'draw-top-rails stairs)
  (apply-by-twos 'draw-hg-rtrn stairs)
  (set-active-layer clayer)
  (princ))
(defun-r 'c:topviewrails)
