import json
from jinja2 import Environment, FileSystemLoader
import os
import sys


def read_json() -> dict:
    "Returns the JSON file files.json as a dictionary"
    with open('commands.json') as f:
        return json.load(f)


def error_pause(e: Exception):
    "Displays an Exception's message, waits for input, and exits"
    print(f"[{type(e).__name__}] {e}")
    input("Press ENTER to continue...")
    sys.exit()


if __name__ == '__main__':
    try:
        os.chdir(sys.argv[1])
        commands = read_json()
    except Exception as e:
        error_pause(e)

    env = Environment(
        loader=FileSystemLoader('.'),
    )

    full_commands = []
    for cmd in commands:
        cmd['idxname'] = cmd['name']
        if cmd['doc_short'] != cmd['doc_full']:
            cmd['idxname'] = f'<a href="#{cmd["name"]}">{cmd["name"]}</a>'
            full_commands.append(cmd)

    try:
        with open('commands.html', 'w') as f:
            f.write(env.get_template('commands.jinja').render(
                {
                    'commands': commands,
                    'full_commands': full_commands,
                }
            ))
            print("Wrote commands.html")
    except Exception as e:
        error_pause(e)
