---
title: "PSC AutoCAD Automation User Guide"
geometry: margin=1in
---

# List of Commands #

[Click here](commands.html) to see command list

# Procedures #

## Double-click action ##

To edit stair and landing blocks by double-clicking:

1. Run CUI command
2. In the top-left panel, find `Double Click Actions` -> `Block` -> `Block - Double Click`
3. In the right panel, edit the **Macro** field to say `editblock`
4. Click OK

# Stair Dialog Overview #

![](snip/stair-dialog.png "Stair dialog")

## Basic Information ##

This is mostly stuff that we don't expect to change after being initially set
for a given stair.

| Property           | Description                                                      |
|--------------------|------------------------------------------------------------------|
| Stair              | The stair "number" but this can be any string                    |
| Tread Depth        | Horizontal distance from nosing to nosing                        |
| Width              | The width measured from outside of stringers                     |
| Distance Between   | Clear space to the next stair's stringer                         |
| Conn. Distances    | Distance from nosing to start of platform extension              |
| Head Clearance     | Distance above finished floor of head clearance line (0 to omit) |
| Ascend             | Direction to turn in order to reach the next stair               |
| Tread              | The tread style                                                  |
| Rail               | The rail style                                                   |
| Finish             | What kind/color of coating                                       |
| Show Connection    | Uncheck to place connection members on non-plotting layer        |
| Show Topview Arc   | Uncheck to omit clearance arcs from top view                     |
| Separate Stringers | Uncheck to make the right stringer options match the left        |

### Tread Options ###

The supported tread styles each have a different set of customization options,
available by clicking the Options button next to the popup list.

![](snip/stair-tread-opts-100.png "Tread Options for 100-series w/ STNP")

| Property           | Series | Description                                            |
|--------------------|--------|--------------------------------------------------------|
| Pan Material       | 1,2,4  | Thickness of bent plates                               |
| Tread Material     | 4,6    | Material makeup of the tread                           |
| STN Style          | 1,2,4  | None or an applicable STN product line                 |
| Concrete Thickness | 1,4    | Thickness of pan-filled/precast treads                 |
| STN T/B Only       | 1,2,4  | Check to omit STN at middle nosings                    |
| Kickback           | 1,2    | Check to include series-appropriate kickback           |
| Quiet-Tread        | 2      | Check to include Quiet Tread(tm)                       |
| STN SKU            | 1,2,4  | Calculated automatically based on selected STN options |

#### STN ####

STN product lines apply to particular tread series and support particular
options. Note that STNR is now called STNRS. This is how Safe-T-Nose is
referring to shop-applied 2" aluminum nosings. (STNR refers only to the
field-applied nosings).

| Style | Series | Nosing Color | Iso Color | 1" Strip | 1 1/2" Strip |
|-------|--------|:------------:|:---------:|:--------:|:------------:|
| STNP  | 1      | X            | X         | X        |              |
| STNS  | 2      | X            |           | X        |              |
| STNRS | 2      | X            |           | X        |              |
| STNPT | 4      |              |           | X        |              |
| STNC3 | 1,4    | X            |           | X        | X            |

Options descriptions:

| Property       | Description                                         |
|----------------|-----------------------------------------------------|
| Nosing Color   | The color of the metal part of the nosing           |
| Isolator Color | The color of the plastic part of the nosing         |
| Insert Strip 1 | The content of the 1" insert strip                  |
| Insert Strip 2 | The content of the 1 1/2" insert strip (STNC3 only) |

### Rail Options ###

![](snip/stair-rail-opts.png "Rail Options")

Stairs contain at least two stair/wall rails: inside and outside. The style of
rails is set in the main dialog and the options for left/right stair/wall rails
can be set in the Options dialog. By default, the **inside** will be a standard
**Stair Rail** with 0" extensions at the top and bottom and no D-loops, and the
**outside** will be a standard **Wall Rail** with no D-loops. A third option
exists: Chop-Top stair rails cut off horizontally at a given distance below the
top nosing.

Either side can be any kind of rail.

| Property           | Description                                                                                                                                          |
|--------------------|------------------------------------------------------------------------------------------------------------------------------------------------------|
| Handrail Level-off | Check to include a 15" horizontal segment at handrail extensions                                                                                     |
| Top Mount          | Cut the bottoms of the posts off at pitch for mounting to the top of a channel stringer                                                              |
| Type               | Whether a particular rail is a Stair Rail, Chop-top Stair Rail or Wall Rail                                                                          |
| Frame Extensions   | Horizontal extension of a post from its default position (3 5/8" from edge of platform). Positive values grow the rail and negative values shrink it |
| Cutoff             | How far down the top of a Chop-top Stair Rail is from the top nosing                                                                                 |
| D-loops            | Check to have handrail wrap around and back to a post/wall                                                                                           |

## Location ##

Properties related to the stair's position in the main section view.

| Property          | Description                                                                             |
|-------------------|-----------------------------------------------------------------------------------------|
| Level Designation | The floor "number", but this can be any string to acount for floors like P1, PH, M, etc |
| Flight Sequence   | A letter denoting flight position between levels                                        |
| Z-position        | Whether the stair is in the near or far plane                                           |

## Risers ##

Properties related to riser geometry. 

| Property              | Description                                                                                   |
|-----------------------|-----------------------------------------------------------------------------------------------|
| Floor-to-Floor Height | Distance between floors. Used for rise height calculation                                     |
| Total Risers          | Number of risers between floors. Calculated automatically. User-settable                      |
| Risers This Flight    | Number of risers in this flight                                                               |
| Rise Height           | Vertical distance from nosing to nosing. Calculated automatically. Not directly user-settable |

To set a specific rise height, set the Floor-to-Floor Height to the desired
flight height and make Risers This Flight match Total Risers.

## Stringers ##

Properties related to stringer geometry. By default, the Right Stringer values
will match whatever is set for the Left Stringer. To set them individually,
check the Separate Stringers box in Basic Information.

| Property      | Description                                                                                |
|---------------|--------------------------------------------------------------------------------------------|
| Material      | Plate or channel used                                                                      |
| Top Offset    | How far the stringer extends vertically above the top nosing                               |
| Mid Offset    | Top of stringer offset from nosing line at pitch                                           |
| Bottom Offset | How far the stringer extends vertically above bottom finished floor (bottom haunches only) |
| Extends       | How far the stringer extends past the end of the platform at top/bottom                    |

## Connections ##

How the stair connects at the top and bottom.

| Property                         | Description                                                       |
|----------------------------------|-------------------------------------------------------------------|
| Main popup list tile [unlabeled] | Detail number from standard details and description of connection |
| Extension                        | How far the top/bottm platform extends                            |

**Note**: the platform extension described by the **Extension** property does not
include the **Top/Bottom Connection Distance** in Basic Information. For example
a Connection Distance of 1 1/4" (default) with an Extension of 11" results in a
distance of 1'-1/4" from nosing to edge of platform.

# Landing Dialog Overview #

![](snip/land-dialog.png "Land dialog")

## Basic Information ##

| Property       | Description                                                      |
|----------------|------------------------------------------------------------------|
| Stair          | The stair "number" but this can be any string                    |
| Head Clearance | Distance above finished floor of head clearance line (0 to omit) |
| Tread Style    | The tread style                                                  |
| Finish         | What kind/color of coating                                       |

## Location ##

| Property | Description                                                |
|----------|------------------------------------------------------------|
| Level    | The floor "number", but this can be any string             |
| Sequence | A letter denoting the landing position between levels      |
| Side     | Which side the landing appears on in the main section view |

## X-Bracing ##

Allows toggling of cross bracing in each of four bays. User may
specify a distance from bottom of shorter post to centerline of bottom
brace.

## Miscellaneous ##

"Get X Stair Info" buttons will fill in the "Bolts" sub dialog (see
below) for either the front or back channel.

## Depth ##

| Property                              | Description                                                                              |
|---------------------------------------|------------------------------------------------------------------------------------------|
| Nosing to Wall / Nosing to Nosing     | Distance from **nearest front stair nosing** to **back wall** or **back stair nosing**   |
| Front Stair Nosing Distance           | Distance from **nearest front stair nosing** to **front edge of landing**                |
| Platform Depth                        | Distance from **front edge of landing** to **back edge of landing**                      |
| Back Gap / Back Stair Nosing Distance | Distance from **back edge of landing** to **back wall** or **nearest back stair nosing** |

## Width ##

| Property       | Description                                             |
|----------------|---------------------------------------------------------|
| Shaft Width    | Total clear distance between faces of finish            |
| Platform Width | Total width of platform                                 |
| Gaps           | Distance from **edge of landing** to **face of finish** |

## Channels ##

Each of the main channels has a box in the dialog for independent
settings. The default mode of the dialog is to have the back channel
match the front unless otherwise specified. To set channel sizes
independently, check the box next to the back channel size, which will
enable its individual dropdown box. To set back channel connections to
be different from the front, check the "Separate" box in the
connection you want to change.

| Property | Description                                                 |
|----------|-------------------------------------------------------------|
| Size     | Channel size                                                |
| Bolts    | Opens a sub-dialog for setting stair bolt information       |
| Location | Distance from front edge of landing to face of back channel |
| Flip     | Check to flip the back channel                              |

### Bolts ###

![](snip/land-bolts.png "Bolts Dialog")

| Property          | Description                                               |
|-------------------|-----------------------------------------------------------|
| Bolt to...        | What this channel is bolting to, if anything              |
| Offset From Edge  | Distance from **edge of landing** to **outside stringer** |
| Width             | Stair overall width                                       |
| Bolt spacing keys | Correspond to stair connection options                    |
| Double bolt       | Stair is a 5X connection rather than a 2X                 |

### Connections ###

| Property          | Description                                                                             |
|-------------------|-----------------------------------------------------------------------------------------|
| Channel Detail    | Connection detail that applies at the end of the channel                                |
| Support Detail    | Connection detail that applies at the end of the post or hanger, if applicable          |
| Channel Extension | Distance from **edge of landing** to **edge of channel**                                |
| Support Length    | Distance from **landing finished floor** to **top or bottom of structure**              |
| Options           | Opens a sub-dialog for connection options, if any                                       |
| Separate          | Allow a back channel connection to have different properties from its pair on the front |

### Connection Options ###

![](snip/land-conn-opts.png "Connection Options")

| Property           | Description                                                                                                         |
|--------------------|---------------------------------------------------------------------------------------------------------------------|
| Extra Holes Offset | For post connections with a shared-post landing below, offset of lower landing (i.e. additional bolt holes on post) |
| Plate Height/Width | Hanger rod connection plate geometry                                                                                |
| Kick Brace Options | Kick brace style and geometry                                                                                       |

