  (set-file-docstring "Embed dialog functions")

  (psc-include (list "common-data.lsp" "dialog/dialog.lsp"))

  (defun-q embed-dialog (dat / int-keys dist-keys toggle-keys default-actions dialog-after-load dialog-after-save dialog-before-done tile-actions extra-keys dialog-ties sub-dialogs)
    "Embed dialog functions"
    (defun dialog-after-load ()
      (with-data dat '((set_tile style "1")))
      ((lambda (k) (em-x k (get_tile k))) (get-em-style))
    ) ;_ defun dialog-after-load
    (defun dialog-before-done ()
      (append! 'data (list (list "style" (get-em-style))))
      (remove! 'data '(begins-with (car x) "em-"))
    ) ;_ defun dialog-before-done
    (setq
      int-keys '("qty")
      dist-keys '("l_gap" "l_str" "d_btw" "r_str" "r_gap" "t_lng")
      toggle-keys '("em-1" "em-2" "em-3" "em-4" "em-5" "hide")
      default-actions
       (append
         (mapcar 'cons
          '("em-1" "em-2" "em-3" "em-4" "em-5")
           (lst* 'em-x 5)
         ) ;_ mapcar
         (mapcar 'cons
          '("l_gap" "l_str" "d_btw" "r_str" "r_gap")
           (lst* 'calc-em1 5)
         ) ;_ mapcar
       ) ;_ append
    ) ;_ setq
    (dialog-init "embed/embed.dcl" "EMBED_DCL" dat)
  ) ;_ defun-q embed-dialog
  (defun-r 'embed-dialog)

  (defun-q em-x (key val)
    "dialog tile handling"
    (force-single-toggle)
    (toggle-dist-keys)
  ) ;_ defun-q em-x
  (defun-r 'em-x)

  (defun-q force-single-toggle ( / lst)
    "restrincts toggle to one selection at most"
    (setq lst '("em-1" "em-2" "em-3" "em-4" "em-5"))
    (mapcar
     '(lambda (key) (set_tile key "0"))
      (remove lst '(= x key))
    ) ;_ mapcar
    (if (null (get_tile "em-tog")) (set_tile key "1"))
  ) ;_ defun-q force-single-toggle
  (defun-r 'force-single-toggle)

  (defun-q toggle-dist-keys ()
    "Enable or diable tiles based on embed type selected"
    (cond
      ( (null (get-em-style))
        (mapcar
         '(lambda (k) (mode_tile (car k) 1) (apply 'set_tile k))
          (append
            (mapcar 'list
             '("lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog" "tl-tog")
              (lst* "0" 6)
            ) ;_ mapcar
            (mapcar 'list
             '("l_gap" "l_str" "d_btw" "r_str" "r_gap" "t_lng")
              (lst* "" 6)
            ) ;_ mapcar
          ) ;_ append
        ) ;_ mapcar
      ) ;_ condif
      ( (= key "em-1")
        (mapcar
         '(lambda (key def)
            (mode_tile key 0)
            (set_tile key (to-string (?? (cadr (assoc key dat)) def)))
          ) ;_ lambda
          (if (get_tile "em-tog")
           '("lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog")
           '("l_gap" "l_str" "d_btw" "r_str" "r_gap")
          ) ;_ if
          (if (get_tile "em-tog")
            (lst* "" 5)
            (mapcar 'to-string '(1.0 44.0 4.0 44.0 1.0))
          ) ;_ if
        ) ;_ mapcar
        (mode_tile "tl-tog" 1)
        (mode_tile "t_lng" 1)
        (set_tile "t_lng" (if (get_tile "em-tog") "" (calc-em1 nil nil)))
      ) ;_ condif
      ( (member key '("em-2" "em-3"))
        (mapcar
         '(lambda (key) (mode_tile key 1) (set_tile key ""))
         '("l_gap" "l_str" "d_btw" "r_str" "r_gap"
           "lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog")
        ) ;_ mapcar
        (if (get_tile "tl-tog")
          (progn
            (mode_tile "tl-tog" 0)
            (mode_tile "t_lng" 1)
            (set_tile "t_lng" "")
          ) ;_ progn
          (progn
            (mode_tile "t_lng" 0)
            (set_tile "t_lng"
              (to-string (?? (cadr (assoc "t_lng" dat)) 48.0))
            ) ;_ set_tile
          ) ;_ progn
        ) ;_ if
      ) ;_ condif
      ( (member key '("em-4" "em-5"))
        (mapcar
         '(lambda (key) (mode_tile key 1) (set_tile key ""))
         '("l_gap" "l_str" "d_btw" "r_str" "r_gap"
           "lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog"
           "t_lng" "tl-tog")
        ) ;_ mapcar
      ) ;_ condif
    ) ;_ cond
  ) ;_ defun-q toggle-dist-keys
  (defun-r 'toggle-dist-keys)

  (defun-q calc-em1 (key val)
    "calculates total length of em-1 from segment lengths"
    (set_tile "t_lng"
      (to-string
        (apply '+
          (mapcar
           '(lambda (key) (?n (read (get_tile key))))
           '("l_gap" "l_str" "d_btw" "r_str" "r_gap")
          ) ;_ mapcar
        ) ;_ apply
      ) ;_ to-string
    ) ;_ set_tile
  ) ;_ defun-q calc-em1
  (defun-r 'calc-em1)

  (defun-q get-em-style ()
    "conversion function between dcl and xdata"
    (car
      (filter
       '("em-1" "em-2" "em-3" "em-4" "em-5")
       '(= "1" (get_tile x))
      ) ;_ filter
    ) ;_ car
  ) ;_ defun-q get-em-style
  (defun-r 'get-em-style)

  (defun-q embed-dcl-defaults ()
    "reads or generates default dialog values"
    (vl-propagate 'embed-dat-val)
    (if (null embed-dat-val)
      (setq embed-dat-val
        (list
         '("qty" "1")
         '("l_gap" "1.0")
         '("l_str" "44.0")
         '("d_btw" "4.0")
         '("r_str" "44.0")
         '("r_gap" "1.0")
         '("t_lng" "94.0")
         '("style" "em-1")
        ) ;_ list
      ) ;_ setq
    ) ;_ if
    embed-dat-val
  ) ;_ defun-q embed-dcl-defaults
  (defun-r 'embed-dcl-defaults)

  (defun-q multi-embed-dialog ( / int-keys dist-keys toggle-keys default-actions dialog-after-load dialog-after-save dialog-before-done tile-actions extra-keys dialog-ties sub-dialogs con-lst)
    "Embed dialog functions for batch editing"
    (defun con-lst (lst)
      (mapcar 'cons (cdr lst) (lst* (car lst) (length (cdr lst))))
    ) ;_ defun con-lst
    (defun dialog-after-load ( / tog-lst int-lst txt-lst dst-lst)
      (setq
        tog-lst
          (append
           '("em-1" "em-2" "em-3" "em-4" "em-5")
           '("lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog")
          ) ;_ append
        int-lst '("qty")
        txt-lst '("tx-1" "tx-2" "tx-3" "tx-4" "tx-5")
        dst-lst '("l_gap" "l_str" "d_btw" "r_str" "r_gap" "t_lng")
      ) ;_ setq
      (mapcar
       '(lambda (k) (mode_tile k 1))
        (append tog-lst int-lst txt-lst dst-lst)
      ) ;_ mapcar
    ) ;_ defun dialog-after-load
    (defun dialog-before-done ()
      (append! 'data (list (list "style" (get-em-style))))
      (remove! 'data '(begins-with (car x) "em-"))
      (remove! 'data '(ends-with (car x) "-tog"))
    ) ;_ defun dialog-before-done
    (setq
      int-keys '("qty")
      dist-keys '("l_gap" "l_str" "d_btw" "r_str" "r_gap" "t_lng")
      toggle-keys
       '( "em-1" "em-2" "em-3" "em-4" "em-5"
          "lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog"
          "hide" "em-tog" "tl-tog" "qt-tog" )
      default-actions
        (append
          (con-lst '(em1-tog "lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog"))
          (con-lst '(em-x "em-1" "em-2" "em-3" "em-4" "em-5"))
          (con-lst '(calc-em1 "l_gap" "l_str" "d_btw" "r_str" "r_gap"))
          (con-lst '(multi-tog "em-tog" "qt-tog" "tl-tog"))
        ) ;_ append
    ) ;_ setq
    (dialog-init "embed/embed.dcl" "MULTI_EMBED_DCL" nil)
  ) ;_ defun-q multi-embed-dialog
  (defun-r 'multi-embed-dialog)

  (defun-q multi-tog (key val / em-box-lst em-txt-lst)
    "Toggle handling for multi edit dialog"
    (setq
      em-box-lst '("em-1" "em-2" "em-3" "em-4" "em-5")
      em-txt-lst '("tx-1" "tx-2" "tx-3" "tx-4" "tx-5")
    ) ;_ setq
    (mapcar
     '(lambda (k) (mode_tile k (if (zerop (read val)) 1 0)))
      (cond
        ( (= key "qt-tog") '("qty"))
        ( (= key "tl-tog") '("t_lng"))
        ( (= T) (append em-box-lst em-txt-lst))
      ) ;_ condif
    ) ;_ mapcar
    (if (= key "em-tog")
      ( (lambda (k)
          (if (and k (= (get_tile key) "1"))
            (em-x k (get_tile k))
            (progn
              (mapcar '(lambda (k) (set_tile k "0")) em-box-lst)
              (toggle-dist-keys)
            ) ;_ progn
          ) ;_ if
        ) ;_ lambda
        (get-em-style)
      ) ;_ lambda-func
    ) ;_ if
  ) ;_ defun-q multi-tog
  (defun-r 'multi-tog)

  (defun-q em1-tog (key val / lst)
    "Toggle handling for em-1 lengths in multi edit dialog"
    (setq lst
      (mapcar 'list
       '("lg-tog" "ls-tog" "db-tog" "rs-tog" "rg-tog")
       '("l_gap" "l_str" "d_btw" "r_str" "r_gap")
        (mapcar 'to-string '(1.0 44.0 4.0 44.0 1.0))
      ) ;_ mapcar
    ) ;_ setq
    (apply
     '(lambda (tog-key tog-val)
        (if (zerop (read val))
          (progn (mode_tile tog-key 1) (set_tile tog-key ""))
          (progn (mode_tile tog-key 0) (set_tile tog-key tog-val))
        ) ;_ if
      ) ;_ lambda
      (cdr (assoc key lst))
    ) ;_ apply
  ) ;_ defun-q em1-tog
  (defun-r 'em1-tog)
