(set-file-docstring "Functions for closure plate fab drawings")

(psc-include '("fab/land-ship-loose.lsp"))

(defun-q make-closure-plate-fabs (stairs / plates plate-map)
  "Creates all the closure plate fab drawings for this group of STAIRS"
  (declare (vars (stairs list (vl-every 'data-list-p stairs))))
  (foreach plate (strip-nil (apply-by-twos 'calc-closure-plate stairs))
           (add-to-alist 'plate-map (car plate) (list (cdr plate)) T))
  (foreach plates plate-map
           ((lambda(/ finish)
              (setq finish (pop! 'plates))
              (while plates
                     (make-closure-plate-fab finish (take! 85 'plates)))))))
(defun-r 'make-closure-plate-fabs)

(defun-q calc-closure-plate (s1 s2 / calc)
  "Returns the closure plate definition for the plate between stairs S1 and S2

The return value will be a list whose car is the finish string and cdr is a datalist for
the table."
  (defun calc (s1 s2 / s1-hgt s2-hgt diff ret)
    (setq s1-hgt (xd-value "i-nosing_top" s1)
          s2-hgt (if (wcmatch (xd-value "bot_conn" s2) "*haunch*")
                     (xd-value "i-nosing_bot" s2)
                   (with-data s2
                     '((line-val (line-offset (pt-slope (/ rise_hgt tread_depth)
                                                        (+ bot_cdist bot_ext)
                                                        rise_hgt)
                                              i-nosing_mid)
                                 0))))
          diff (- s2-hgt s1-hgt)
          ret (with-data s1
                '((#! '(("piece_type" "CP")
                        ("stair_number" #number)
                        ("level" #level)
                        ("sequence" #sequence)
                        ("qty" 1)
                        ("desc" "PLATE 1/4")
                        ("length" #(frac (xd-value "dist_btwn" s1)))
                        ("note" "SL"))))))
    (append (list (xd-value "finish" s1))
            ret
            (if (< diff 0.5)
                '(("width" "6"))
                (#! '(("width" "12")
                      ("detail" "P1")
                      ("cut_height" #(frac (- 12 diff))))))))

  (if (> (xd-value "dist_btwn" s1) 0) (calc s1 s2)))
(defun-r 'calc-closure-plate)

(defun-q make-closure-plate-fab (finish plates / dwgno path login prefix)
  "Creates a single closure plate fab drawing"
  (declare (vars (finish str)
                 (plates list (vl-every 'data-list-p plates))))
  (setq path (vla-get-Path acadDoc)
        login (getvar 'loginname)
        dwgno (land-sl--calc-dwg-no jobno stairno)
        prefix (next-mark-prefix)
        acadDoc (vla-Add (vla-get-Documents acadObj)
                         (strcat psc-src-dir "fab/closure-plate.dwt"))
        modelspace (vla-get-ModelSpace acadDoc))
  (write-title-block job-info stairno login "CLOSURE PLATES")
  ;; write finish table
  (set-finish finish)
  ;; write main table
  (table-add-data (get-table-by-title "Misc Material")
                  ((lambda(/ i)
                     (setq i 0)
                     (mapcar '(lambda(plate)
                               (add-to-list 'plate
                                (list "mark" (strcat prefix (to-string (inc! 'i))))))
                             plates))))
  (vla-Regen acadDoc acAllViewports)
  (vla-SaveAs acadDoc (strcat path "/" dwgno ".dwg"))
  (if (not *dev-mode*) (vla-Close acadDoc))
  (reset-doc))
(defun-r 'make-closure-plate-fab)
