  (set-file-docstring "Functions for guard fab drawings")

  (defun-q make-guard-fabs (guards)
    "Create all fab drawings for GUARDS

Essentially a composition of %make-generic-fab and %fab-combine-guards"
    (mapcar
     '(lambda(x) (make-generic-fab x 'guard))
      (fab-combine-guards guards)
    ) ;_ mapcar
  ) ;_ defun-q make-guard-fabs
  (defun-r 'make-guard-fabs)

  (defun-q fab-combine-guards (guards)
    "Uses %fab-combine-like-data to combine GUARDS"
    (fab-combine-like-data
      guards
      (mapcar
        (compose '(lowercase to-string))
       '( guard_lng   guard_hgt
          guard_sty   hrl_con     hrr_con
          guard_hand  hrl_off     hrr_off
          guard_kick  hrl_ang     hrr_ang
          finish      hrl_gap     hrr_gap
          lpost_con   mpost_con   rpost_con
          lpost_ext   mpost_ext   rpost_ext
          lpost_tab   mpost_tab   rpost_tab
        ) ;_ quoted-list
      ) ;_ mapcar
    ) ;_ fab-combine-like-data
  ) ;_ defun-q fab-combine-guards
  (defun-r 'fab-combine-guards)

  ;;; this function assumes it is called by make-generic-fab and
  ;;; thus has access to all local vars in make-generic-fab
  (defun make-guard-fab ( / hor-dim ver-dim inv-xpt adj-btw bnd-lst cane-p post-p side-p ins mat-lst hdw-lst tag-lst)
    (psc-include
     '( "fab/part-details.lsp"
        "fab/util.lsp"
        "rail/derived.lsp"
        "rail/guard-rail.lsp"
        "rail/migrations.lsp"
        "rail/rail-section.lsp"
        "rail/util.lsp"
        "shapes/shape.lsp"
        "util/object/dimension.lsp"
      ) ;_ quoted-list
    ) ;_ psc-include
    (setvar 'luprec 4)
    (defun hor-dim (p1 p2)
      (vla-put-layer-r
        (vla-AddDimRotated blk
          (3dpt (inv-xpt p1)) (3dpt (inv-xpt p2))
          (3dpt 0 dim-y) 0
        ) ;_ vla-AddDimRotated blk
        "DIM"
      ) ;_ vla-put-layer-r
    ) ;_ defun hor-dim
    (defun ver-dim (p1 p2)
      (vla-put-layer-r
        (vla-AddDimRotated blk
          (3dpt (inv-xpt p1)) (3dpt (inv-xpt p2))
          (3dpt (- dim-x) 0) (/ pi 2)
        ) ;_ vla-AddDimRotated blk
        "DIM"
      ) ;_ vla-put-layer-r
    ) ;_ defun ver-dim
    (defun inv-xpt (inv-lst)
      (cond
        ( (and (data-list-p inv-lst) (xd-value "ins" inv-lst))
          (subst-key "ins" (inv-xpt (xd-value "ins" inv-lst)) inv-lst)
        ) ;_ condif
        ( (point-p inv-lst) (mapcar '* inv-lst '(-1 1 1)))
        ( (vl-every 'listp inv-lst) (mapcar 'inv-xpt inv-lst))
        ( (= T) inv-lst)
      ) ;_ cond
    ) ;_ defun inv-xpt
    (defun adj-btw ()
      ;;; deduct 1/8 from width @ btwn. per C.G. 2024/04/24
      (setq guard_lng (- guard_lng 0.125))
      (subst-key! "guard_lng" guard_lng 'data)
    ) ;_ defun adj-btw
    (setq
      ins    (?? dd-test-ins '(36.0 90.0))
      cane-p (zerop (read guard_sty))
      post-p (= 'X11 (read lpost_con))
      side-p (and (not cane-p) (not post-p))
    ) ;_ setq
    (if (or post-p (all-X12)) (adj-btw))
    (draw-guard-front ins)
    (draw-guard-top ins)
    (if side-p (draw-guard-side ins))
    (setq mat-lst
      (mapcar
       '(lambda (mat n / mark)
          (setq mark (strcat piece-mark (to-string n)))
          (setq mat (cons (car mat) (cons (list "mark" mark) (cdr mat))))
        ) ;_ lambda
        (combine-like-data mat-lst '("dtl" "desc" "length") "ref")
        (range 1 (1+ (length mat-lst)) 1)
      ) ;_ mapcar
    ) ;_ setq
    (setq hdw-lst (combine-like-data hdw-lst '("desc" "note") nil))
    (map-apply 'add-call-outs tag-lst)
    (add-details (filter mat-lst '(xd-value "dtl" x)))
    (if mat-table (table-add-data mat-table mat-lst))
    (if hdw-table (table-add-data hdw-table hdw-lst))
    ;; update assemblies table title
    (if ass-table
      (progn
        (vla-SetCellState ass-table 0 0 acCellStateNone)
        (vla-SetText ass-table 0 0
          (strcat "Assembly: "
            (cond
              ( (begins-with guard_sty "0") "CANE ")
              ( (begins-with guard_sty "1") "2-LINE ")
              ( (begins-with guard_sty "2") "6-LINE ")
              ( (begins-with guard_sty "3") "TUBE ")
              ( (begins-with guard_sty "4") "PICKET ")
            ) ;_ cond
            "GUARD " (to-string (fix guard_lng))
          ) ;_ strcat
        ) ;_ vla-SetText
      ) ;_ progn
    ) ;_ if
  ) ;_ defun make-guard-fab

  (defun draw-guard-front (ins / dim-x dim-y blk def cen)
    (setq
      blk (p-blk "front-view")
      def (calc-guard-rail data)
      ins (displace-pt ins (list guard_lng 0))
      cen (if (zerop (read guard_sty)) "cl" "handrail")
      cen (mapcar 'cdr (filter def '(assoc cen (car x))))
      cen (mapcar '(lambda (lst) (remove lst '(< (length x) 3))) cen)
      cen (mapcar '(lambda (lst) (mapcar 'remove-last lst)) cen)
    ) ;_ setq
    (foreach lst (mapcar 'inv-xpt cen)
      (vla-put-layer (add-polyline blk '(0 0) lst nil) "CENTER")
    ) ;_ foreach
    (add-calcd-objs blk nil (mapcar! 'inv-xpt 'def))
    (cond
      ( post-p (dim-guard-front-post))
      ( cane-p (dim-guard-front-cane))
      ( T (dim-guard-front-std))
    ) ;_ cond
    (insert-block modelSpace ins blk)
  ) ;_ defun draw-guard-front

  (defun dim-guard-front-post ( / t-lng)
    (setq t-lng (/ (- guard_lng 1.5) 2.0))
    (setq dim-y (+ guard_hgt (* 2 dimspace)))
    (hor-dim
      (list t-lng guard_hgt)
      (list 0 (- guard_hgt 3))
    ) ;_ hor-dim
    (hor-dim
      (list (- guard_lng t-lng) guard_hgt)
      (list guard_lng (- guard_hgt 3))
    ) ;_ hor-dim
    (setq dim-y (+ dim-y dimspace))
    (hor-dim
      (list 0 (- guard_hgt 3))
      (list guard_lng (- guard_hgt 3))
    ) ;_ hor-dim
    (setq dim-x (* -3 dimspace))
    (ver-dim
      (list 0 (- guard_hgt 3))
      (list t-lng guard_hgt)
    ) ;_ hor-dim
    (ver-dim
      (list 0 (- guard_hgt 3))
      (list 0 (- guard_hgt 33))
    ) ;_ hor-dim
    (setq dim-x (+ guard_lng (* 3 dimspace)))
    (ver-dim
      (list guard_lng (- guard_hgt 4))
      (list (- guard_lng t-lng) guard_hgt)
    ) ;_ hor-dim
    (ver-dim
      (list guard_lng (- guard_hgt 4))
      (list guard_lng (- guard_hgt 34))
    ) ;_ hor-dim
    (ver-dim
      (list guard_lng (- guard_hgt 34))
      (list (- guard_lng t-lng) 0)
    ) ;_ hor-dim
    (setq dim-x (+ dim-x dimspace))
    (ver-dim
      (list (- guard_lng t-lng) 0)
      (list (- guard_lng t-lng) guard_hgt)
    ) ;_ hor-dim
    (add-mat-lst "s-post"
      (list (- t-lng) (- guard_hgt 9.0))
     '((0 0) (1 0) (1 -3) (3 -3))
    ) ;_ add-mat-lst
    (add-mat-lst "t-post"
      (list 0 (- guard_hgt 34))
     '((0 0) (1 0) (1 -3) (2 -3))
    ) ;_ add-mat-lst
  ) ;_ defun dim-guard-front-post

  (defun dim-guard-front-cane ( / bpl)
    (setq bpl (= 30 (read lpost_con)))
    (setq dim-y -12.0)
    (hor-dim '(-0.83 0.0) (list (+ guard_lng 0.83) 0.0))
    (add-mtext blk
      (list (/ guard_lng -2.0) -14)
      "REF. OUT-TO-OUT" 24.0 'middlecenter
    ) ;_ add-mtext
    (setq dim-x '(0) dim-y (+ guard_hgt 9.83))
    (repeat (1+ num-mid) (add-to-list 'dim-x (+ cen-dst (last dim-x))))
    (mapcar! '(lambda (x) (list x guard_hgt)) 'dim-x)
    (apply-by-twos '(lambda (p1 p2) (hor-dim p1 p2)) dim-x)
    (setq dim-y (+ guard_hgt 15.83))
    (if (> num-mid 0)
      (progn
        (hor-dim (car dim-x) (last dim-x))
        (mapcar
         '(lambda (x / dim-x p1 p2)
            (setq dim-x x p1 (- guard_hgt 0.38) p2 (if bpl 0.1875 0.0))
            (foreach p '(p1 p2) (set p (list x (eval p))))
            (vla-put-layer (tog-bg-fill (ver-dim p1 p2)) "No Plot")
          ) ;_ lambda
          (mapcar 'car (cdr (remove-last dim-x)))
        ) ;_ mapcar
      ) ;_ progn
    ) ;_ if
    (if (>= guard_hgt 24.0)
      (progn
        (setq dim-y 4.0)
        (mapcar! '(lambda (pt) (list (car pt) 4.0)) 'dim-x)
        (apply-by-twos
         '(lambda (p1 p2)
            (setq p1 (mapcar '+ p1 '(0.38 0.0)))
            (setq p2 (mapcar '- p2 '(0.38 0.0)))
            (vla-put-layer (tog-bg-fill (hor-dim p1 p2)) "No Plot")
          ) ;_ lambda
          dim-x
        ) ;_ apply-by-twos
        (setq dim-x -9.83)
        (ver-dim (if bpl '(-2.0625 0.1875) '(-0.83 0.0)) '(0.0 4.0))
      ) ;_ progn
    ) ;_ if
    (setq dim-x -15.83)
    (ver-dim (if bpl '(-2.0625 0.1875) '(-0.83 0.0)) (list 0.0 guard_hgt))
    (if bpl (ver-dim '(-2.0625 0.1875) '(-2.0625 0.0)))
    (setq dim-x -21.83)
    (if bpl (ver-dim '(-2.0625 0.0) (list 0.0 guard_hgt)))
    (setq dim-x (+ guard_lng 9.83))
    (vla-put-layer
      (ver-dim
        (list (+ guard_lng (if bpl 2.0625 0.83)) 0.0)
        (list guard_lng (+ guard_hgt 0.83))
      ) ;_ ver-dim
      "No Plot"
    ) ;_ vla-put-layer
    (add-marks-front-cane)
  ) ;_ defun dim-guard-front-cane

  (defun dim-guard-front-std ( / hrail-dim)
    (defun hrail-dim ( / bend-p flat-p off-pt hr-pts)
      (defun bend-p (con) (member (read con) '(X13 X14)))
      (defun flat-p (con) (member (read con) '(X16 X17)))
      (defun off-pt (p) (subst! (mapcar '+ p '(0.0 0.83)) p 'hr-pts))
      (setq hr-pts cen)
      (if (= (read hrl_con) 'X15) (setq hr-pts (cdr hr-pts)))
      (if (= (read hrr_con) 'X15) (setq hr-pts (remove-last hr-pts)))
      (set-active-dimstyle "1xFRACTIONAL_HR")
      (apply-by-twos
       '(lambda (pt1 pt2 / ang)
          (setq ang (angle pt1 pt2) pt1 (3dpt pt1) pt2 (3dpt pt2))
          (vla-put-layer (vla-AddDimRotated blk pt1 pt2 pt1 ang) "DIM")
        ) ;_ lambda
        (inv-xpt hr-pts)
      ) ;_ apply-by-twos
      (set-active-dimstyle "1xFRACTIONAL")
      (if (bend-p hrl_con) (setq hr-pts (cdr hr-pts)))
      (if (bend-p hrr_con) (setq hr-pts (remove-last hr-pts)))
      (if (flat-p hrl_con) (off-pt (car hr-pts)))
      (if (flat-p hrr_con) (off-pt (last hr-pts)))
      (hor-dim (list 0 guard_hgt) (car hr-pts))
      (hor-dim (list guard_lng guard_hgt) (last hr-pts))
    ) ;_ defun hrail-dim
    (setq cen (car cen))
    (repeat num-mid
      (add-to-list 'dim-x (+ 1.5 clr-dst (?f (last dim-x))))
    ) ;_ repeat
    (mapcar! '(lambda (x) (list x (- guard_hgt 1.5))) 'dim-x)
    (setq dim-y (+ guard_hgt (* 2 dimspace)))
    (while dim-x
      (hor-dim (car dim-x) (mapcar '+ (car dim-x) '(1.5 0.0)))
      (setq dim-x (cdr dim-x))
    ) ;_ while
    (if (= "1" guard_hand) (hrail-dim))
    (repeat (1+ num-mid)
      (add-to-list 'dim-x (+ 1.5 (?f (last dim-x))))
      (add-to-list 'dim-x (+ clr-dst (last dim-x)))
    ) ;_ repeat
    (mapcar! '(lambda (x) (list x (- guard_hgt 1.5))) 'dim-x)
    (setq dim-y (+ dim-y dimspace))
    (hor-dim
      (list 1.5 (- guard_hgt 1.5))
      (list 0 guard_hgt)
    ) ;_ hor-dim
    (while dim-x
      (hor-dim (car dim-x) (cadr dim-x))
      (setq dim-x (cddr dim-x))
    ) ;_ while
    (hor-dim
      (list (- guard_lng 1.5) (- guard_hgt 1.5))
      (list guard_lng guard_hgt)
    ) ;_ hor-dim
    (setq dim-y (+ dim-y dimspace))
    (hor-dim
      (list 0 guard_hgt)
      (list guard_lng guard_hgt)
    ) ;_ hor-dim
    (if (= 400 (read guard_sty))
      ( (lambda ( / dim-x dim-y)
          (setq dim-x (- guard_lng 1.5) dim-y (- guard_hgt 29.75))
          (set-active-dimstyle "1xFRACTIONAL_TYP")
          (hor-dim (list (- dim-x pick-spc) dim-y) (list dim-x dim-y))
          (set-active-dimstyle "1xFRACTIONAL")
        ) ;_ lambda
      ) ;_ lambda-func
    ) ;_ if
    (add-marks-front-std)
  ) ;_ defun dim-guard-front-std

  (defun add-marks-front-cane ( / con num i x)
    (setq
      con (list lpost_con mpost_con rpost_con)
      con (mapcar 'read con)
      num 0
    ) ;_ setq
    ;;; main
    (add-mat-lst "cane-f"
      (list (- (- guard_lng 4.5)) (+ guard_hgt 0.83))
     '((0 0) (0 1.5) (-7.5 3.0))
    ) ;_ add-mat-lst
    ;;; mid
    (if (> num-mid 0)
      (progn
        (setq i 1 num (1+ num))
        (repeat num-mid
          (setq x (* i cen-dst))
          (add-mat-lst "cane-m"
            (mapcar '- (list 0.83 (- guard_hgt 3.0)) (list x 0))
           '((0 0) (2.0 0) (2.0 -3) (3 -3))
          ) ;_ add-mat-lst
          (inc! 'i)
        ) ;_ repeat
      ) ;_ progn
    ) ;_ if
    ;;; hor
    (if (>= guard_hgt 24.0)
      (progn
        (setq i 0 num (1+ num))
        (repeat (1+ num-mid)
          (setq x (* i cen-dst))
          (add-mat-lst "cane-h"
            (mapcar '- (list -4.0 3.17) (list x 0))
           '((0 0) (0 -3) (-1 -3))
          ) ;_ add-mat-lst
          (inc! 'i)
        ) ;_ repeat
      ) ;_ progn
    ) ;_ if
    (if (vl-some '(lambda (n) (= n 30)) con)
      (progn
        (setq i 1 num (strcat piece-mark (itoa (1+ num))))
        (if (= 30 (car con))
          (progn
            (add-hdw-lst (car con))
            (add-mat-lst "wgnr-f" nil nil)
            (add-mat-lst "wgnr-p" '(-1.5 0.0) '((0 0) (0 -3) (4.5 -3)))
          ) ;_ progn
        ) ;_ if
        (if (= 30 (cadr con))
          (progn
            (repeat num-mid
              (add-hdw-lst (car con))
              (setq x (+ (* i (- cen-dst)) 1.5))
              (add-mat-lst "wgnr-f" nil nil)
              (add-mat-lst "wgnr-p" (list x 0) '((0 0) (0 -3) (-1 -3)))
              (inc! 'i)
            ) ;_ repeat
          ) ;_ progn
        ) ;_ if
        (if (= 30 (last con))
          (progn
            (add-hdw-lst (car con))
            (setq x (- (- guard_lng 1.5)))
            (add-mat-lst "wgnr-f" nil nil)
            (add-mat-lst "wgnr-p" (list x 0.0) '((0 0) (0 -3) (-4.5 -3)))
          ) ;_ progn
        ) ;_ if
      ) ;_ progn
    ) ;_ if
  ) ;_ defun add-marks-front-cane

  (defun add-marks-front-std ( / num con sty i x)
    (setq
      num 0
      con (list lpost_con mpost_con rpost_con)
      con (mapcar 'guard-con-read con)
      sty (read guard_sty)
    ) ;_ setq
    (if (and (= "1" guard_hand) (= 'X13 (read hrl_con)))
      (progn
        (add-mat-lst "rail-c"
          (list (- 3.0) guard_hgt)
         '((0 0) (0 4.5) (6 4.5))
        ) ;_ add-mat-lst
      ) ;_ progn
      (progn
        (add-mat-lst "rail-c"
          (list (- (- guard_lng 3.0)) guard_hgt)
         '((0 0) (0 4.5) (-6 4.5))
        ) ;_ add-mat-lst
      ) ;_ progn
    ) ;_ if
    (add-mat-lst "l-post"
      (list 0 (- guard_hgt 18.0))
     '((0 0) (2 0) (2 -4) (4 -4))
    ) ;_ add-mat-lst
    (add-mat-lst "r-post"
      (list (- guard_lng) (- guard_hgt 18.0))
     '((0 0) (-2 0) (-2 -4) (-4 -4))
    ) ;_ add-mat-lst
    (if (member (cadr con) '(12 52))
      (progn
        (add-mat-lst "rail-b"
          (list (- 0 (* clr-dst 0.5) 1.5) (- (?n mpost_ext)))
         '((0 0) (0 -3) (-6 -3))
        ) ;_ add-mat-lst
      ) ;_ progn
    ) ;_ if
    (if (> num-mid 0)
      (progn
        (add-mat-lst "m-post"
          (list (- (- guard_lng 1.5 clr-dst)) (- guard_hgt 3.0))
         '((0 0) (-2 0) (-2 6) (-3 6))
        ) ;_ add-mat-lst
      ) ;_ progn
    ) ;_ if
    (cond
      ( (member sty '(100 200))
        (add-marks-front-200)
      ) ;_ condif
      ( (= sty 300)
        (add-mat-lst "t-fill"
          (list (- (- guard_lng 3.0)) (- guard_hgt 31.5))
         '((0 0) (0 1.5) (-6 1.5))
        ) ;_ add-mat-lst
      ) ;_ condif
      ( (= sty 400)
        (if
          (not
            (vl-every
             '(lambda (x) (member (guard-con-read x) '(12)))
              (list lpost_con mpost_con rpost_con)
            ) ;_ vl-every
          ) ;_ not
          (add-mat-lst "rail-h"
            (list -3.0 (- guard_hgt 35.75))
           '((0 0) (0 3) (6 3))
          ) ;_ add-mat-lst
        ) ;_ if
        (add-mat-lst "k-fill"
          (list (- (- guard_lng 1.5 pick-spc)) (- guard_hgt 34.25))
         '((0 0) (-1.5 0) (-1.5 -4.5) (-7.5 -4.5))
        ) ;_ add-mat-lst
      ) ;_ condif
      ( (= T) (list))
    ) ;_ cond
    (if (= "1" guard_hand)
      ( (lambda (x-ins k-off x-off y-off)
          (add-mat-lst "h-rail"
            (list (- (+ x-ins k-off)) (+ (cadadr cen) 0.83))
            (#! '((0 0) (0 #y-off) (#x-off #y-off)))
          ) ;_ add-mat-lst
          (add-mat-lst "z-brkt" nil nil)
        ) ;_ lambda
        (if (= 'X13 (read hrl_con)) (- guard_lng 4.5) 4.5)
        (cond ( (/= sty 400) 0) ( (= 'X13 (read hrl_con)) 1.5) (-1.5))
        (if (= 'X13 (read hrl_con)) -7.5 7.5)
        (if (= sty 200) 12.75 10.5)
      ) ;_ lambda-func
    ) ;_ if
    (if (= guard_kick "1")
      (progn
        (add-mat-lst "kick-p"
          (cond
            ( (member 46 con) (list -3.0 2.5))
            ( (or (member 41 con) (member 42 con)) (list -3.0 2.25))
            ( (= T) (list -3.0 2.0))
          ) ;_ cond
         '((0 0) (0 1) (6 1))
        ) ;_ add-mat-lst
      ) ;_ progn
    ) ;_ if
    (add-marks-front-con)
  ) ;_ defun add-marks-front-std

  (defun add-marks-front-200 ( / h-fill-p p-off y-lst)
    (defun h-fill-p (con off)
      (and (= sty 200) (= 'X13 (read (?s con))) (<= (+ 1.25 off) -3.5))
    ) ;_ defun h-fill-p
    (defun p-off (pt) (mapcar '+ pt (?? ins '(0 0 0))))
    (setq y-lst (filter def '(member '("type" "pipe") (car x))))
    (setq y-lst (remove y-lst '(member '("handrail" "1") (car x))))
    (mapcar!
     '(lambda (lst / ins)
        (cadadr (with-data (car lst) '((mapcar 'p-off (cdr lst)))))
      ) ;_ lambda
     'y-lst
    ) ;_ mapcar
    (progn (sort! 'y-lst '<) (if (= sty 200) (setq y-lst (cdr y-lst))))
    ;;; hss infill
    (if (= sty 200)
      ( (lambda (x-ins x-off)
          (add-mat-lst "rail-h"
            (list x-ins (- guard_hgt 4.5))
            (#! '((0 0) (0 6.0) (#x-off 6.0)))
          ) ;_ add-mat-lst
        ) ;_ lambda
        (if (= 'X13 (read (?s hrl_con))) (- (- guard_lng 3)) -3)
        (if (= 'X13 (read (?s hrl_con))) -6 6)
      ) ;_ lambda-func
    ) ;_ if
    ;;; non-hrail pipe
    (add-mat-lst "p-fill"
      (list (- (- guard_lng 3.0)) (- (car y-lst) 0.83))
      (if (= sty 100) '((0 0) (0 -6) (-6 -6)) '((0 0) (0 -1.5) (-6 -1.5)))
    ) ;_ add-mat-lst
    ;;; triangle fill left
    (if (h-fill-p hrl_con hrl_off)
      (add-mat-lst "h-fill"
        (list -4.25 (+ (last y-lst) 4.625))
       '((0 0) (-0.25 0.4375) (5 3.375) (8 3.375))
      ) ;_ add-mat-lst
    ) ;_ if
    ;;; triangle fill right
    (if (h-fill-p hrr_con hrr_off)
      (add-mat-lst "h-fill"
        (list (- (- guard_lng 4.25)) (+ (last y-lst) 4.625))
       '((0 0) (0.25 0.4375) (-5 3.375) (-8 3.375))
      ) ;_ add-mat-lst
    ) ;_ if
  ) ;_ defun add-marks-front-200

  (defun add-marks-front-con ( / i con tab-lng)
    (setq con (guard-con-read lpost_con))
    (cond
      ( (= con 48)
        (add-mat-lst "base-d"
          (list -0.75 (- lpost_ext))
         '((0 0) (0 -3) (3 -3))
        ) ;_ add-mat-lst
      ) ;_ condif
      ( (member con '(41 42))
        (add-mat-lst "base-a"
          (list -0.75 0.0)
         '((0 0) (0 -3) (3 -3))
        ) ;_ add-mat-lst
        (add-hdw-lst con)
      ) ;_ condif
      ( (member con '(7 47 51))
        (setq tab-lng lpost_tab)
        (add-mat-lst (if (= con 47) "l-guss-c" "l-guss-s")
          (list -0.625 (- 1.5 lpost_ext))
         '((0 0) (2 0) (2 -6) (3 -6))
        ) ;_ add-mat-lst
        (setq tab-lng nil)
      ) ;_ condif
    ) ;_ cond
    (setq con (guard-con-read rpost_con))
    (cond
      ( (= con 48)
        (add-mat-lst "base-d"
          (list (- (- guard_lng 0.75)) (- rpost_ext))
         '((0 0) (0 -3) (-3 -3))
        ) ;_ add-mat-lst
      ) ;_ condif
      ( (member con '(41 42))
        (add-mat-lst "base-a"
          (list (- (- guard_lng 0.75)) 0.0)
         '((0 0) (0 -3) (-3 -3))
        ) ;_ add-mat-lst
        (add-hdw-lst con)
      ) ;_ condif
      ( (member con '(7 47 51))
        (setq tab-lng rpost_tab)
        (add-mat-lst (if (= con 47) "r-guss-c" "r-guss-s")
          (list (- (- guard_lng 0.875)) (- 1.5 rpost_ext))
         '((0 0) (-2 0) (-2 -6) (-3 -6))
        ) ;_ add-mat-lst
        (setq tab-lng nil)
      ) ;_ condif
    ) ;_ cond
    (setq con (guard-con-read mpost_con))
    (repeat num-mid
      (setq i (1+ (?n i)))
      (cond
        ( (= con 48)
          (add-mat-lst "base-d"
            (list (- (* i (- cen-dst)) 0.75) (- mpost_ext))
           '((0 0) (0 -3) (3 -3))
          ) ;_ add-mat-lst
        ) ;_ condif
        ( (member con '(41 42))
          (add-mat-lst "base-a-mid"
            (list (- (* i (- cen-dst)) 0.75) 0.0)
           '((0 0) (0 -3) (3 -3))
          ) ;_ add-mat-lst
          (add-hdw-lst con)
        ) ;_ condif
        ( (member con '(7 47 51))
          (setq tab-lng mpost_tab)
          (add-mat-lst (if (= con 47) "m-guss-c" "m-guss-s")
            (list (- (* i (- cen-dst)) 0.625) (- 1.5 mpost_ext))
           '((0 0) (2 0) (2 -6) (3 -6))
          ) ;_ add-mat-lst
          (setq tab-lng nil)
        ) ;_ condif
      ) ;_ cond
    ) ;_ repeat
  ) ;_ defun add-marks-front-con

  (defun draw-guard-top (ins / X15-dim hr-end off blk sty def cen)
    (defun X15-dim ( / pt1 pt2 dim-x)
      (setq
        pt1   (list (if (= con 'hrl_con) -0.83 0.83) -0.375)
        pt1   (mapcar '+ ins pt1) pt2 (mapcar '+ ins '(0.0 2.785))
        dim-x (* (if (= con 'hrl_con) -2.0 2.0) dimspace)
        dim-x (+ (car pt1) dim-x)
      ) ;_ setq
      (ver-dim pt1 pt2)
    ) ;_ defun X15-dim
    (defun hr-end (con / ins pts str obj)
      (setq
        ins  (calc-guard-rail data)
        ins  (filter ins '(assoc "handrail" (car x)))
        ins  (map-apply '(lambda (x y z) (list x (+ z 0.375))) (cdar ins))
        ins  ((if (= con 'hrl_con) car last) ins)
        pts '((0 0) (2 0) (2 -4) (3 -4))
        pts  (if (= con 'hrl_con) pts (inv-xpt pts))
      ) ;_ setq
      (setq str
        (cond
          ( (member (read (eval con)) '(X13 X14 X17)) "FIELD SPLICE")
          ( (member (read (eval con)) '(X16)) "CAP END")
          ( (= T) nil)
        ) ;_ cond
      ) ;_ setq
      (if str (setq obj (add-mleader blk (inv-xpt ins) pts str)))
      (if obj (vla-put-layer-r obj "DIM"))
      (if (= (read (eval con)) 'X15) (X15-dim))
    ) ;_ defun hr-end
    (setq
      off (list guard_lng (+ guard_hgt 36.0))
      blk (draw-guard dat (mapcar '+ ins off))
      blk (blockref2def blk)
      sty (read (xd-value "guard_sty" data))
    ) ;_ setq
    (cond
      ( (= cane-p T)
        (add-tag-lst "cane-f"
          (list (+ (- guard_lng) (* 0.5 cen-dst) 1.5) -1.66)
         '((0 0) (0 -6) (1.5 -6))
        ) ;_ add-tag-lst
      ) ;_ condif
      ( (= post-p T)
        (add-tag-lst "s-post"
          (list (* guard_lng -0.5) -1.5)
         '((0 0) (0 -3) (1.5 -3))
        ) ;_ add-tag-lst
        (add-tag-lst "t-post"
          (list (* (- guard_lng 1.5) -0.25) -0.625)
         '((0 0) (0 3) (-1.5 3))
        ) ;_ add-tag-lst
      ) ;_ condif
      ( (= T)
        (add-tag-lst "rail-c"
          (list (+ (- guard_lng) (* 0.5 clr-dst) 1.5) -1.5)
         '((0 0) (0 -6) (1.5 -6))
        ) ;_ add-tag-lst
        (if (or (= "1" guard_hand) (member sty '(100 200)))
          (add-tag-lst (if (= "1" guard_hand) "h-rail" "p-fill")
            (list
              (- (* -0.5 clr-dst) 1.5)
              (if (= "1" guard_hand) 3.16 1.66)
            ) ;_ list
           '((0 0) (0 3) (-1.5 3))
          ) ;_ add-tag-lst
        ) ;_ if
        (if (= "1" guard_hand) (mapcar 'hr-end '(hrl_con hrr_con)))
      ) ;_ condif
    ) ;_ cond
  ) ;_ defun draw-guard-top

  (defun draw-guard-side (ins / off blk obj dim-i dim-y dim-x)
    (setq
      off (list (+ guard_lng 36.0) 0.0)
      blk (guard-section (mapcar '+ ins off))
      blk (blockref2def blk)
    ) ;_ setq
    (if (member (read lpost_con) '(41 42))
      (foreach ins '((1.25 0.125) (-2.75 0.125))
        (setq obj (add-polyline blk ins '((0 -1.5) (0 1.5)) nil))
        (vla-put-layer obj "CENTER")
      ) ;_ foreach
    ) ;_ if
    (dim-guard-side)
    (set-active-layer "HIDDEN")
    (move-to-bottom
      (vla-put-color-r
        (add-polyline modelspace ins
          (#! '((#(- dimspace) 0) (#(+ guard_lng (abs dim-x) 36) 0)))
          nil
        ) ;_ add-polyline modelspace
        253
      ) ;_ vla-put-color-r
      modelspace
    ) ;_ move-to-bottom
    (set-active-layer "DIM")
  ) ;_ defun draw-guard-side

  (defun dim-guard-side ( / sec-ver-dim cen-y g-sty l-con l-ext l-tab)
    (defun sec-ver-dim (p1 p2 / obj)
      (setq obj (ver-dim p1 p2))
      (setq dim-x (* (dec! 'dim-i) dimspace))
      obj
    ) ;_ defun sec-ver-dim
    (setq
      l-tab lpost_tab
      l-ext lpost_ext
      l-con (read lpost_con)
      g-sty (read guard_sty)
      dim-i (cond
              ( (or (= g-sty 200) (/= l-con 48)) -2)
              ( (= "1" guard_hand) -3)
              ( (= T) -4)
            ) ;_ cond
      dim-x (* dim-i dimspace)
      dim-y (+ guard_hgt (* 2 dimspace))
      cen-y (if (member (read guard_sty) '(100 200)) 9.08 7.955)
    ) ;_ setq
    (if (= "1" guard_hand)
      (progn
        (hor-dim (list -1.5 (- guard_hgt cen-y)) (list 0 guard_hgt))
        (add-tag-lst "z-brkt"
          (list 1.16277 (- guard_hgt cen-y 2.43764))
         '((0.0 0.0) (0.0 1.5) (-4.5 1.5))
        ) ;_ add-tag-lst
      ) ;_ progn
    ) ;_ if
    (if (= "1" guard_kick)
      (progn
        (add-tag-lst "kick-p"
         '(0 1.5)
         '((0 0) (-2.25 0) (-2.25 3) (-3.25 3))
        ) ;_ add-tag-lst
        (cond
          ( (member l-con '(41 42)) (ver-dim '(-2.25 0.25) '(-0.25 2.25)))
          ( (= l-con 46)
            (ver-dim '(-0.25 -1.5) '(-0.25 2.5))
            (ver-dim '(-0.25 -1.5) (list 0.0 (- l-ext)))
          ) ;_ condif
          ( (= T) (ver-dim '(-0.25 2.0) 	(list dim-x 0)))
        ) ;_ cond
      ) ;_ progn
    ) ;_ if
    (cond
      ( (member l-con '(41 42))
        (add-tag-lst "base-a" (list -0.75 0) '((0 0) (0 -3) (-6 -3)))
        (ver-dim '(-2.25 0.25) (list dim-x 0))
      ) ;_ condif
      ( (= l-con 48)
        (add-tag-lst "base-d" (list 1.5 (- l-ext)) '((0 0) (0 -3) (-6 -3)))
        ( (lambda ( / dim-x)
            (setq dim-x (* -4 dimspace))
            (ver-dim (list -9.25 (- 0.25 l-ext)) (list -9.25 (- l-ext)))
          ) ;_ lambda
        ) ;_ lambda-func
      ) ;_ condif
      ( (member l-con '(47 51 07X))
        (add-tag-lst (if (= l-con 47) "l-guss-c" "l-guss-s")
          (list 0 (- 4.5 l-ext))
         '((0 0) (-3 0) (-3 -6) (-4.5 -6))
        ) ;_ add-tag-lst
        (setq dim-y (+ (- l-ext) (* -2 dimspace)))
        (hor-dim (list 0 (- l-ext)) (list (- l-tab) (- l-ext)))
        (ver-dim
          (list (- l-tab) (- l-ext))
          (list (- (if (= l-con 47) 1 0) l-tab) (- 6 l-ext))
        ) ;_ ver-dim
      ) ;_ condif
      ( (= T) (list))
    ) ;_ cond
    (cond
      ( (= 100 (read guard_sty))
        (if (= "1" guard_hand)
          (sec-ver-dim
            (list -2.33 (- guard_hgt (- cen-y 0.83)))
            (list 0 guard_hgt)
          ) ;_ sec-ver-dim
        ) ;_ if
        (sec-ver-dim
          (list -0.83 (- guard_hgt 20.0))
          (list 0 guard_hgt)
        ) ;_ sec-ver-dim
      ) ;_ condif
      ( (= 200 (read guard_sty))
        (sec-ver-dim (list 0 (- guard_hgt 4.5)) (list 0 guard_hgt))
        (sec-ver-dim
          (cond
            ( (all-X12) (list 0 (- guard_hgt 9.0)))
            ( (= "1" guard_hand) (list -2.33 (- guard_hgt cen-y -0.83)))
            ( (= T) (list -0.83 (- guard_hgt cen-y -0.83)))
          ) ;_ cond
          (list 0 guard_hgt)
        ) ;_ sec-ver-dim
        (set-active-dimstyle "1xFRACTIONAL_TYP")
        (ver-dim
          (list -0.83 (- guard_hgt 15.16))
          (list -0.83 (- guard_hgt 19.035))
        ) ;_ ver-dim
        (set-active-dimstyle "1xFRACTIONAL")
        (sec-ver-dim
          (list -0.83 (- guard_hgt 13.5))
          (list -0.83 guard_hgt)
        ) ;_ sec-ver-dim
      ) ;_ condif
      ( (= 300 (read guard_sty))
        (set-active-dimstyle "1xFRACTIONAL_TYP")
        (ver-dim
          (list 0 (- guard_hgt 17.25))
          (list 0 (- guard_hgt 21.0))
        ) ;_ ver-dim
        (set-active-dimstyle "1xFRACTIONAL")
        (sec-ver-dim
          (list 0 (- guard_hgt 5.25))
          (list 0 guard_hgt)
        ) ;_ sec-ver-dim
        (if (= "1" guard_hand)
          (sec-ver-dim
            (list -2.33 (- guard_hgt (- cen-y 0.83)))
            (list 0 guard_hgt)
          ) ;_ sec-ver-dim
        ) ;_ if
      ) ;_ condif
      ( (= 400 (read guard_sty))
        (if (= "1" guard_hand)
          (sec-ver-dim
            (list -2.33 (- guard_hgt (- cen-y 0.83)))
            (list 0 guard_hgt)
          ) ;_ sec-ver-dim
        ) ;_ if
        (if (not (all-X12))
          (sec-ver-dim (list 0 (- guard_hgt 35.75)) (list 0 guard_hgt))
        ) ;_ if
      ) ;_ condif
      ( (= T) (list))
    ) ;_ cond
    (if (> l-ext 0.0)
      (ver-dim
        (list dim-x 0)
        (list (if (= l-con 48) -9.25 (- l-tab)) (- l-ext))
      ) ;_ ver-dim
    ) ;_ if
    (if (= l-con '08X)
      (progn
        (ver-dim (list 0 (- l-ext)) (list dim-x 0))
        (sec-ver-dim (list 0 (- l-ext)) (list 0 guard_hgt))
      ) ;_ progn
    ) ;_ if
    (sec-ver-dim (list dim-x 0) (list 0 guard_hgt))
    (if (> l-ext 0.0)
      (sec-ver-dim
        (list (if (= l-con 48) -9.25 (- l-tab)) (- l-ext))
        (list 0 guard_hgt)
      ) ;_ sec-ver-dim
    ) ;_ if
  ) ;_ defun dim-guard-side

  (defun add-mat-lst (ref ldr-ins ldr-pts / rail-b-calc cane-fdst cane-vdst post-calc hr-lng)
    (defun cane-fdst ()
      (setq bnd-lst (mapcar 'cane-vdst (list lpost_con rpost_con)))
      (setq bnd-lst (zip (zip bnd-lst (list guard_lng)) '(90 90)))
      (pipe-bend-lines bnd-lst)
    ) ;_ defun cane-fdst
    (defun cane-vdst (con)
      (- guard_hgt (if (= 30 (guard-con-read con)) 0.5625 0.375))
    ) ;_ defun cane-vdst
    (defun post-calc (loc con ext / str ext con)
      (setq con (guard-con-read con))
      (if (member con '(41 42 48)) (setq ext (- ext 0.25)))
      (if (= loc 'm)
        (setq ext (- ext 1.5 (if (member con '(12 52)) 1.5 0.0)))
      ) ;_ if
      (+ guard_hgt ext)
    ) ;_ defun post-calc
    (defun rail-b-calc ( / bot-p c-lst hss-lng)
      (defun bot-p (c) (if (member c '(12 52)) 0.0 1.5))
      (setq c-lst (mapcar 'guard-con-read (list lpost_con rpost_con)))
      (list
        (if (vl-every 'bot-p c-lst) '2-end-miter-45 '1-end-miter-45)
        (apply '- (cons guard_lng (mapcar 'bot-p c-lst)))
      ) ;_ list
    ) ;_ defun rail-b-calc
    (defun hr-lng ( / ang cen lng)
      (setq
        ang (vl-remove-if 'null (list hrl_ang hrr_ang))
        cen (filter def '(assoc "handrail" (car x)))
        cen (uniquify (car (mapcar 'cdr cen)))
        lng (apply-by-twos '(lambda (p1 p2) (distance p1 p2)) cen)
      ) ;_ setq
      (if (> (car lng) (last lng))
        (progn (setq lng (reverse lng)) (setq ang (reverse ang)))
      ) ;_ if
      (setq bnd-lst
        (cond
          ( (cddr lng)
            (list (max 6 (car lng)) (cadr lng) (max 18 (caddr lng)))
          ) ;_ condif
          ( (cdr lng)
            (list (max 6 (car lng)) (max 18 (cadr lng)))
          ) ;_ condif
          ( (= T) lng)
        ) ;_ cond
      ) ;_ setq
      (pipe-bend-lines (zip bnd-lst ang))
    ) ;_ defun hr-lng
    (if (and ldr-ins ldr-pts) (add-tag-lst ref ldr-ins ldr-pts))
    (add-to-list 'mat-lst
      (#!
        (cond
          ;;; hss @ post (X11)
          ( (= ref "s-post")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "HSS 1 1/2 x 1 1/2 x .120")
              ("length" #(rtos guard_hgt 5 5))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; tab @ post (X11)
          ( (= ref "t-post")
           '( ("ref" #ref) ("qty" 4)
              ("desc" "FLAT BAR 1/4 x 2")
              ("length" #(rtos t-lng 5 5)) ("typ" T)
            ) ;_ quoted-list
          ) ;_ condif
          ;;; cane frame
          ( (= ref "cane-f")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "PIPE 1 1/4 DIA (1.66 OD) x 0.140")
              ("length" #(last (cane-fdst)))
              ("col" col-1)
              ("dtl" #(list 'bent-pipe (cane-fdst)))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; cane mid
          ( (= ref "cane-m")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "PIPE 1 1/4 DIA (1.66 OD) x 0.140")
              ("length" #(rtos (cane-vdst mpost_con) 5 5))
              ("col" col-4)
              ("dtl" (cope-1))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; cane horz
          ( (= ref "cane-h")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "PIPE 1 1/4 DIA (1.66 OD) x 0.140")
              ("length" #(rtos (- cen-dst 0.75) 5 5))
              ("col" col-4)
              ("dtl" (cope-2))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; wagner flange
          ( (= ref "wgnr-f")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "WAGNER BASE FLANGE (1613)")
            ) ;_ quoted-list
          ) ;_ condif
          ;;; wagner baseplate
          ( (= ref "wgnr-p")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "WAGNER ANCHOR PLATE (B1608)")
              ("note" "SL")
            ) ;_ quoted-list
          ) ;_ condif
          ;;; rail cap
          ( (= ref "rail-c")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "HSS 1 1/2 x 1 1/2 x .120")
              ("length" #(rtos guard_lng 5 5))
              ("col" col-1)
              ("dtl" (2-end-miter-45 1 #guard_lng 1.5))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; left post
          ( (= ref "l-post")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "HSS 1 1/2 x 1 1/2 x .120")
              ("length" #(rtos (post-calc 'l lpost_con lpost_ext) 5 5))
              ("col" col-2)
              ("dtl"
                (#(if (member (guard-con-read lpost_con) '(12 52))
                   '2-end-miter-45 '1-end-miter-45
                  ) ;_ if
                  1 #(post-calc 'l lpost_con lpost_ext) 1.5
                ) ;_ list
              ) ;_ list
            ) ;_ quoted-list
          ) ;_ condif
          ;;; mid post
          ( (= ref "m-post")
           '( ("ref" #ref) ("qty" #num-mid)
              ("desc" "HSS 1 1/2 x 1 1/2 x .120")
              ("length" #(rtos (post-calc 'm mpost_con mpost_ext) 5 5))
              ("typ" T)
            ) ;_ quoted-list
          ) ;_ condif
          ;;; right post
          ( (= ref "r-post")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "HSS 1 1/2 x 1 1/2 x .120")
              ("length" #(rtos (post-calc 'r rpost_con rpost_ext) 5 5))
              ("col" col-2)
              ("dtl"
                (#(if (member (guard-con-read rpost_con) '(12 52))
                   '2-end-miter-45 '1-end-miter-45
                  ) ;_ if
                  1 #(post-calc 'r rpost_con rpost_ext) 1.5
                ) ;_ list
              ) ;_ list
            ) ;_ quoted-list
          ) ;_ condif
          ;;; rail bot
          ( (= ref "rail-b")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "HSS 1 1/2 x 1 1/2 x .120")
              ("length" #(rtos (cadr (rail-b-calc)) 5 5))
              ("col" col-1)
              ("dtl" #(zip (rail-b-calc) '(1 1.5)))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; rail horz (HSS btw posts @ 200 & 400)
          ( (= ref "rail-h")
           '( ("ref" #ref) ("qty" #(* (1+ num-mid) (if (all-X12) 2 1)))
              ("desc" "HSS 1 1/2 x 1 1/2 x .120")
              ("length" #(rtos clr-dst 5 5)) ("typ" T)
            ) ;_ quoted-list
          ) ;_ condif
          ;;; hand rail
          ( (= ref "h-rail")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "PIPE 1 1/4 DIA (1.66 OD) x 0.140")
              ("length" #(last (hr-lng)))
              ("col" col-1)
              ("dtl" #(if (> (length (hr-lng)) 1) (list 'bent-pipe (hr-lng))))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; hr fill (triangle pl @ 200 w/ hr bend up)
          ( (= ref "h-fill")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "FLAT BAR 1/4 x 6")
              ("length" "9 1/2")
              ("col" col-4)
              ("dtl" (clip-fbar 2 9.5 9.0 6.0 5.5))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; pipe fill
          ( (= ref "p-fill")
           '( ("ref" #ref) ("qty" #num-pipe)
              ("desc" "PIPE 1 1/4 DIA (1.66 OD) x 0.140")
              ("length" #(rtos (- guard_lng 0.625) 5 5)) ("typ" T)
            ) ;_ quoted-list
          ) ;_ condif
          ;;; tube fill
          ( (= ref "t-fill")
           '( ("ref" #ref) ("qty" #num-tube)
              ("desc" "HSS 1 1/2 x 3/4 x .083")
              ("length" #(rtos clr-dst 5 5)) ("typ" T)
            ) ;_ quoted-list
          ) ;_ condif
          ;;; picket fill
          ( (= ref "k-fill")
           '( ("ref" #ref) ("qty" #num-pick)
              ("desc" "SQUARE BAR 1/2")
              ("length" #(rtos pick-lng 5 5)) ("typ" T)
            ) ;_ quoted-list
          ) ;_ condif
          ;;; kick plate
          ( (= ref "kick-p")
           '( ("ref" #ref) ("qty" 1)
              ("desc" #(if (member 46 con) "FLAT BAR 1/4 x 4" "FLAT BAR 1/4 x 2"))
              ("length" #(rtos guard_lng 5 5))
              ("note" #(if (= finish "Galvanized") "SL" ""))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; anchor baseplate
          ( (member ref '("base-a" "base-a-mid"))
           '( ("ref" #ref) ("qty" 1)
              ("desc" "FLAT BAR 1/4 x 3")
              ("length" "6")
              ("col" col-3)
              ("dtl" (2h-flatbar 2 6 3 1 1 1.5 #std-hole-dia))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; decking baseplate
          ( (= ref "base-d")
           '( ("ref" #ref) ("qty" 1)
              ("desc" "FLAT BAR 1/4 x 3")
              ("length" "20") ("note" "SL")
            ) ;_ quoted-list
          ) ;_ condif
          ;;; gusset @ post (square)
          ( (member ref '("l-guss-s" "m-guss-s" "r-guss-s"))
           '( ("ref" #ref) ("qty" 1)
              ("desc" "FLAT BAR 1/4 x 6")
              ("length" #(rtos tab-lng 5 5))
            ) ;_ quoted-list
          ) ;_ condif
          ;;; gusset @ post (clipped)
          ( (member ref '("l-guss-c" "m-guss-c" "r-guss-c"))
           '( ("ref" #ref) ("qty" 1)
              ("desc" "FLAT BAR 1/4 x 6")
              ("length" #(rtos tab-lng 5 5))
              ("col" col-3)
              ("dtl" (clip-fbar 2 #tab-lng 1.0 6.0 1.0))
            ) ;_ quoted-list
          ) ;_ condif
          ( (= ref "z-brkt")
           '( ("ref" #ref) ("qty" #(+ num-mid 2))
              ("desc" "PSC Z-BRACKET")
            ) ;_ quoted-list
          ) ;_ condif
          ( (= T) (#! '( ("NULL" #ref))) )
        ) ;_ cond
      ) ;_ macro-expand
    ) ;_ add-to-list
  ) ;_ defun add-mat-lst

  (defun add-hdw-lst (con / hdw-str)
    (defun hdw-str (str) (if galv-p (hardware-galv-desc str) str))
    (add-to-list 'hdw-lst
      (#!
       '( ("qty" #(if (= con 30) 1 2))
          ("desc"
           #(if (member con '(30 41))
             #(hdw-str "HILTI KB-TZ WEDGE ANCHOR 5/8 DIA. x 4 3/4")
             #(hdw-str "HILTI KH-EZ SCREW ANCHOR 5/8 DIA. x 4")
            ) ;_ if
          ) ;_ list
          ("note" "SL")
        ) ;_ quoted-list
      ) ;_ macro-expand
    ) ;_ add-to-list
  ) ;_ defun add-hdw-lst

  (defun add-tag-lst (ref ldr-ins ldr-pts)
    (add-to-list 'tag-lst
      (#! '(#ref #(cons blk (cons ldr-ins ldr-pts))))
    ) ;_ add-to-list
  ) ;_ defun add-tag-lst

  (defun add-call-outs (ref def / find-ref add-flg dat mrk obj)
    (defun find-ref (ref)
      (find-first '(lambda (x) (member ref (xd-value "ref" x))) mat-lst)
    ) ;_ defun find-ref
    (defun add-flg (bpl-mrk / flg-mrk)
      (strcat (xd-value "mark" (find-ref "wgnr-f")) bpl-mrk)
    ) ;_ defun add-flg
    (setq dat (find-ref ref) mrk (xd-value "mark" dat))
    (if (= ref "wgnr-p") (setq mrk (add-flg (strcat "\n" mrk))))
    (if (and (xd-value "typ" dat) (> (xd-value "qty" dat) 1))
      (setq mrk (strcat mrk " (TYP.)"))
    ) ;_ if
    (if (= ref "base-a-mid") (setq mrk (strcat mrk " (CTR'D.)")))
    (if (= (xd-value "note" dat) "SL") (setq mrk (strcat mrk " (S.L.)")))
    (setq obj (add-mleader (car def) (cadr def) (cddr def) mrk))
    (vla-put-layer-r obj "DIM")
  ) ;_ defun add-call-outs

  (defun add-details (dtl-lst / ext-max dtl-ins col-1 col-2 col-3 col-4)
    (setq ext-max (apply 'max (list lpost_ext mpost_ext rpost_ext)))
    (setq col-1 (displace-pt ins (list 0 (- ext-max))))
    (setq col-1 (displace-pts '((0 -36) (0 -60)) col-1))
    (setq col-2 (displace-pt (car col-1) (list (+ guard_lng 36) 0)))
    (setq col-2 (displace-pts '((0 0) (0 -24)) col-2))
    (setq col-3 '((226.0 33.0 0.0) (226.0 63.0 0.0) (226.0 93.0 0.0)))
    (setq col-4 '((265.0 33.0 0.0)))
    (foreach dat dtl-lst
      (if (eval (car (xd-value "dtl" dat)))
        (progn
          (insert-block modelspace (pop! (xd-value "col" dat))
            (apply 'part-detail
              (mapcar '(lambda (val) (xd-value val dat)) '("mark" "dtl"))
            ) ;_ apply
          ) ;_ insert-block
        ) ;_ progn
      ) ;_ if
    ) ;_ foreach
  ) ;_ defun add-details
