;; add channel material and part details
(defun land-fab-channels (/
                          row
                          )
  (land-fab-channels-add-material)
  (land-fab-channels-front)
  (land-fab-channels-back)
  (if left_chan
      (land-fab-channels-left))
  (if right_chan
      (land-fab-channels-right)))

;; convert channel material key to format suitable for fab material system
(defun land-fab-chan-desc (chan)
  (strcat "CHANNEL " (vl-string-subst " x " "x" chan)))

;; return the detail number at a corner
(defun land-fab-detail-num (fb lr)
  (atoi (symcat '(#(substr fb 1 1) #(substr lr 1 1) "_chan_num"))))

;; return a definition of the hanger sleeve at a corner
(defun land-fab-calc-hanger-sleeve (fb lr / detail def)
  (setq detail (land-fab-detail-num fb lr)
        def (cond ((= detail 40)
	           '(("desc" "HSS 1 1/4 x 1 1/4 x 3/16")
                     ("shape" "HSS1-1/4x1-1/4x3/16")
	             ("wid" 1.25)
	             ("thk" 0.1875)))
	          ((= detail 50)
	           '(("desc" "HSS 1 1/2 x 1 1/2 x .120")
                     ("shape" "HSS1-1/2x1-1/2x0.12")
	             ("wid" 1.5)
	             ("thk" 0.12)))))
  (if def (append def
                  (#! '(("id" #(strcat fb "-" lr "-sleeve"))
                        ("qty" 1)
                        ("length" #(symcat '(#fb "_chan_dpt"))))))))

;; return the length of any channel
(defun land-fab-chan-len (fblr)
  (if (member fblr '("front" "back"))
      (symcat '(#fblr "_chan_len"))
      (land-fab-side-chan-len fblr)))

;; return the length of a side channel, accounting for copes
(defun land-fab-side-chan-len (lr / len cope)
  (setq len (symcat '(#lr "_chan_len"))
        cope (symcat '(#lr "_cope")))
  (cond ((= cope "front")
         (+ len (- front_chan_wid front_chan_web)))
        ((= cope "back")
         (+ len (- back_chan_wid back_chan_web)))
        (t len)))

;; return the base material definition for a channel
(defun land-fab-chan-base-mat (fblr)
  (#! '(("id" #(strcat fblr "-chan"))
	("qty" 1)
        ("desc" #(land-fab-chan-desc (symcat '(#fblr "_chan"))))
        ("length" #(frac (land-fab-chan-len fblr))))))

;; return a string representing a channel end style for the part detail
;; based on overall corner of landing
(defun land-fab-fb-chan-detail-name (fb lr / detail)
  (setq detail (land-fab-detail-num fb lr))
  (cond ((member detail '(20 27)) "front-bolt-2")
        ((member detail '(30 37)) "front-bolt-3")
	((member detail '(25 35)) "bot-bolt-main")
	((member detail '(40 50)) "hanger-sleeve")
	((member detail '(66 67 68)) "front-slot")
	(T "none")))
(defun land-fab-lr-chan-detail-name (fb lr / detail cope)
  (setq detail (land-fab-detail-num fb lr)
	cope (symcat '(#lr "_cope")))
  (cond ((= cope fb) "cope")
	((member detail '(25 35)) "bot-bolt-side")
	(T "none")))

;; return detail order for main channel
;; the left side of a channel when looking at the face of the channel will be the right
;; side of the platform if the channel is flipped
(defun land-fab-fb-chan-order (fb)
  (if (and (= fb "back") flip-back)
      '("right" "left")
      '("left" "right")))

;; return the definition of a main channel
(defun land-fab-calc-fb-chan (fb / order bolt-slots)
  (setq order (land-fab-fb-chan-order fb)
        bolt-slots (land-fab-calc-chan-bolt-slots fb))
  (append (land-fab-chan-base-mat fb)
          (if bolt-slots (list (list "bolt-slots" bolt-slots)))
          (#! '(
                ;; l/rdetail based on looking at channel face
                ("ldetail" #(land-fab-fb-chan-detail-name fb (car order)))
		("rdetail" #(land-fab-fb-chan-detail-name fb (cadr order)))))))

;; return detail order for a side channel
(defun land-fab-side-chan-order (lr)
  (if (= lr "left")
      '("front" "back")
      '("back" "front")))

;; return the definition of a side channel
(defun land-fab-calc-lr-chan (lr / order)
  (setq order (land-fab-side-chan-order lr))
  (if (symcat '(#lr "_chan"))
      (append (land-fab-chan-base-mat lr)
              (#! '(
                    ;; l/rdetail based on looking at channel face
                    ("ldetail" #(land-fab-lr-chan-detail-name (car order) lr))
                    ("rdetail" #(land-fab-lr-chan-detail-name (cadr order) lr)))))))

;; return a list of stair bolt slot locations relative to top left
;; of channel, looking at the channel face
(defun land-fab-calc-chan-bolt-slots (fb
                                      / adjust fb-short order order-short lext bolt-to)
  (defun adjust (slots / lr lr-short offset)
    (setq lr (if (= fb "front") "left" "right")
          lr-short (substr lr 1 1)
          offset (- (symcat '(#fb-short #lr-short "_ext"))
                    (?f (if (and (symcat '(#lr "_chan"))
                                 (/= (symcat '(#lr "_cope")) fb))
                            (symcat '(#lr "_chan_wid"))))))
    (mapcar '(lambda(pt) (cons (+ (car pt) offset) (cdr pt)))
            slots))

  (setq fb-short (substr fb 1 1)
        order (land-fab-fb-chan-order fb)
        order-short (mapcar '(lambda(s) (substr s 1 1))
                            order)
        lext (symcat '(#fb-short #(car order-short) "_ext"))
        bolt-to (fbkey "bolt_to"))
  (adjust
   (vl-sort
    (cond ((= bolt-to "None")
           nil)
          ((= bolt-to "Landing Channel")
           (#! '((4 -4)
                 (#(/ plt_wid 2.0) -4)
                 (#(- plt_wid 4) -4))))
          (T (append
              (if (member bolt-to '("2 Stairs" "Left Stair"))
                  (land-fab-calc-single-stair-bolt-slots fb "l"))
              (if (member bolt-to '("2 Stairs" "Right Stair"))
                  (land-fab-calc-single-stair-bolt-slots fb "r")))))
    (sort-by 'car '<))))

;; returns a list of slot locations relative to top left corner of channel
(defun land-fab-calc-single-stair-bolt-slots (fb lr
                                              / fb-short lr-short
                                              offset hbolt hbolt2
                                              double wid vbolt ret)
  (setq fb-short (substr fb 1 1)
        lr-short (substr lr 1 1))
  (foreach key '("offset" "hbolt" "hbolt2" "double" "wid" "vbolt")
           (set (read key) (symcat '(#fb "_" #lr-short "-stair_" #key))))
  (setq double (= double "1")
        vbolt (- (- vbolt flr_thick))
        ret (list (list (+ offset hbolt) vbolt)
                  (list (+ offset (- wid hbolt)) vbolt)))
  (if double (append! 'ret (list (list (+ offset hbolt hbolt2) vbolt)
                                 (list (+ offset (- wid (+ hbolt hbolt2))) vbolt))))
  (if (or (and (= fb "front") (= lr "r"))
          (and (= fb "back") (= lr "l")))
      (mirror-pts-v ret (/ plt_wid 2.0))
      ret))

;; calculate and add material for all four channels, combine and calculate marks
(defun land-fab-channels-add-material (/ chan-mat)
  (setq chan-mat (append
                  (mapcar 'land-fab-calc-fb-chan '("front" "back"))
                  (strip-nil (mapcar 'land-fab-calc-lr-chan '("left" "right")))
                  (strip-nil (map-apply 'land-fab-calc-hanger-sleeve '(("front" "left")
                                                                       ("front" "right")
                                                                       ("back" "left")
                                                                       ("back" "right")))))
        chan-mat (combine-like-data chan-mat
                                    '("desc"
			              "length"
			              "ldetail"
			              "rdetail"
                                      "bolt-slots"
			              )
                                    "id"))
  (table-add-data mat-table (mapcar 'process-material chan-mat)))

;; set local symbols for channel props
(defun land-fab-set-chan-props (fb props)
  (foreach p props
           (set p (symcat '(#fb "_chan_" #(vl-symbol-name p))))))

;; add a front view of front/back channel to container
;; used for front, back and detail views for channels
(defun land-fab-add-fb-chan-elev (container fb ins mirrorp layer
                                  / add-detail front-bolt-2 front-bolt-3 bot-bolt-main
                                  hanger-sleeve front-slot len dpt flg)
  ;; detail draw functions
  (defun add-detail (container name farp / blockObj lr)
    (setq blockObj (p-blk "chan-detail")
          lr (if farp (cadr order) (car order)))
    (apply (read name) 'nil)
    ((lambda(blockref / objs)
       (setq objs (vlax-safearray->list (vlax-variant-value (vla-Explode blockref))))
       (vla-Delete blockref)
       (if farp
           (mapcar '(lambda(obj)
                     (setq obj (mirror&delete obj (list (/ len 2.0) 0 0) 'y))
                     (vla-Update obj))
                   objs)))
     (insert-block container '(0 0 0) blockObj)))
  (defun front-bolt-2 (/ bolt)
    (defun hole (ins)
      (vla-put-Layer (vla-AddCircle blockObj (3dpt ins) (/ 11.0 32.0)) "SOLID")
      (add-bolt-centerline ins 'both))
    (mapcar 'hole '((1.25 -1.5 0)
                    (1.25 -4.5 0))))
  (defun front-bolt-3 (/ bolt)
    (defun hole (ins)
      (vla-put-Layer (vla-AddCircle blockObj (3dpt ins) (/ 11.0 32.0)) "SOLID")
      (add-bolt-centerline ins 'both))
    (mapcar 'hole '((1.5 -1.5 0)
                    (1.5 -4.5 0))))
  (defun bot-bolt-main ()
    (add-bolt-centerline (list (- 5 (lrkey "chan_wid")) (- dpt) 0) 'v))
  (defun hanger-sleeve (/ sleeve wid len thick ins)
    (setq sleeve (land-fab-calc-hanger-sleeve fb lr)
          wid (xd-value "wid" sleeve)
          len (xd-value "length" sleeve)
          thick (xd-value "thk" sleeve)
          ins (list (- 1.25 (/ wid 2.0)) 0))
    (vla-put-Layer (add-rectangle blockObj wid (- len) ins) layer)
    ((lambda(layer pts)
       (set-active-layer layer)
       (add-polyline blockObj (displace-pt ins (list thick 0)) pts nil)
       (add-polyline blockObj (displace-pt ins (list (- wid thick) 0)) pts nil)
       (set-active-layer "DIM"))
     "HIDDEN"
     (list '(0 0 0) (list 0 (- len) 0))))
  (defun front-slot (/ ins)
    (setq ins (list 2 (+ (- dpt) 4)))
    (vla-put-Layer (add-slot blockObj ins 3 std-hole-dia) "SOLID")
    (add-bolt-centerline ins 'both))

  (setq blockObj (p-blk nil))
  (land-fab-set-chan-props fb '(len dpt flg))
  ;; main rect
  (vla-put-Layer (add-rectangle blockObj len (- dpt) '(0 0 0)) layer)
  ;; flange lines
  ((lambda(flg-lay pts)
     (vla-put-Layer (add-polyline blockObj (list 0 (- flg) 0) pts nil) flg-lay)
     (vla-put-Layer (add-polyline blockObj (list 0 (+ (- dpt) flg) 0) pts nil) flg-lay))
   (if mirrorp layer "HIDDEN")
   (list '(0 0 0) (list len 0 0)))
  ;; bolt slots
  (foreach ins (land-fab-calc-chan-bolt-slots fb)
           (vla-put-Layer (add-slot blockObj ins 1 std-hole-dia) "SOLID")
           (add-bolt-centerline ins 'both))
  ;; connection
  ((lambda(/ order ldet rdet)
     (setq order (land-fab-fb-chan-order fb)
           ldet (land-fab-fb-chan-detail-name fb (car order))
           rdet (land-fab-fb-chan-detail-name fb (cadr order)))
     (if (/= ldet "none")
         (add-detail blockObj ldet nil))
     (if (/= rdet "none")
         (add-detail blockObj rdet T))
     ;; hanger sleeve callouts
     ;; need to be separate from block-mirroring technique for connections because text
     (if (= ldet "hanger-sleeve")
         (vla-put-Layer
          (add-mleader blockObj
                       (list (- 1.25 (/ (xd-value "wid" (land-fab-calc-hanger-sleeve fb (car order)))
                                        2.0))
                             (- (/ dpt 2.0)))
                       '((0 0 0)
                         (-2 0 0)
                         (-2 -3 0)
                         (-3 -3 0))
                       (xd-value (strcat fb "-" (car order) "-sleeve") marks))
          "DIM"))
     (if (= rdet "hanger-sleeve")
         (vla-put-Layer
          (add-mleader blockObj
                       (list (+ (- len 1.25)
                                (/ (xd-value "wid" (land-fab-calc-hanger-sleeve fb (cadr order))) 2.0))
                             (- (/ dpt 2.0)))
                       '((0 0 0)
                         (2 0 0)
                         (2 -3 0)
                         (3 -3 0))
                       (xd-value (strcat fb "-" (cadr order) "-sleeve") marks))
          "DIM"))))

  ;; insert
  ((lambda(blockref)
     (if mirrorp
         (progn
           (setq blockref (mirror&delete blockref ins 'y))
           (vla-Move blockref (3dpt 0 0) (3dpt len 0))))
     (vla-Explode blockref)
     (vla-Delete blockref))
   (insert-block container ins blockObj)))

;; set the front/back-chan-detail-ht symbol
(defun land-fab-set-fb-chan-detail-ht (fb / ldetail rdetail)
  (setq ldetail (land-fab-fb-chan-detail-name fb "left")
        rdetail (land-fab-fb-chan-detail-name fb "right"))
  (set (read (strcat fb "-chan-detail-ht"))
       (+ chan-top (* (if (member "bot-bolt-main" (list ldetail rdetail))
                          6
                          3)
                      dimspace))))

;; populate the detail view block for a main channel
(defun land-fab-draw-fb-chan-det (fb / view def chan-bot chan-top dpt len wid y-adj holes order)
  (defun y-adj (y pts)
    (mapcar '(lambda(s) (displace-pt s (list 0 y))) pts))
  (defun add-side-holes (tb holes / ldet rdet fby)
    (setq ldet (xd-value "ldetail" def)
          rdet (xd-value "rdetail" def)
          fby (if (= tb 't) -1.5 -4.5))
    (append (cond ((= ldet "front-bolt-2") (list (list 1.25 fby)))
                  ((= ldet "front-bolt-3") (list (list 1.5 fby)))
                  ((= ldet "front-slot") (list (list 2 (+ (- dpt) 4)))))
            holes
            (cond ((= rdet "front-bolt-2") (list (list (- len 1.25) fby)))
                  ((= rdet "front-bolt-3") (list (list (- len 1.5) fby)))
                  ((= rdet "front-slot") (list (list (- len 2) (+ (- dpt) 4)))))))

  (land-fab-set-chan-props fb '(dpt len wid))
  (setq def (land-fab-calc-fb-chan fb)
        holes (land-fab-calc-chan-bolt-slots fb)
        top-holes (y-adj 1 (add-side-holes 't holes))
        bot-holes (reverse (y-adj -1 (add-side-holes 'b holes)))
        view (symcat '(#fb "-chan-detail"))
        chan-bot (* (max 1 (length bot-holes)) dimspace)
        chan-top (+ chan-bot dpt)
        order (land-fab-fb-chan-order fb))
  (land-fab-set-fb-chan-detail-ht fb)
  (foreach sym '(holes top-holes bot-holes)
           (set sym (y-adj chan-top (vl-symbol-value sym))))
  (add-part-detail-callout view (xd-value (strcat fb "-chan") marks) '(0 0 0))
  (land-fab-add-fb-chan-elev view fb (list 0 chan-top 0) nil "SOLID")
  (vla-AddDimRotated view
                     (3dpt 0 chan-bot)
                     (3dpt len chan-bot)
                     (3dpt 0 0)
                     0)
  ;; top hole dims
  (if top-holes
      ((lambda(holes dim-line)
        ;; left
        (vla-AddDimRotated view (3dpt (car holes)) (3dpt 0 chan-top) (3dpt 0 dim-line) 0)
        ;; right
        (vla-AddDimRotated view (3dpt (last holes)) (3dpt len chan-top) (3dpt 0 dim-line) 0)
        ;; mid
        (while (> (length holes) 1)
          (vla-AddDimRotated view (3dpt (car holes)) (3dpt (cadr holes)) (3dpt 0 dim-line) 0)
          (pop! 'holes)))
       top-holes
       (+ chan-top (* 3 dimspace))))
  ;; bottom hole dims
  ((lambda(holes dim-line)
     (while holes
       (vla-AddDimRotated view (3dpt len chan-bot) (3dpt (car holes)) (3dpt 0 dim-line) 0)
       (pop! 'holes)
       (setq dim-line (- dim-line dimspace))))
   (cdr bot-holes)
   (- chan-bot dimspace))
  ;; side hole dims
  (land-fab-fb-chan-det-side-dims)
  ((lambda(bot-line / hole-ins)
     (defun hole-ins (side-chan-wid)
       (list (- 5 side-chan-wid)
             (if (and (= fb "back") (not flip-back))
                 (- wid 1.125)
                 1.125)
             0))
     (if (= (xd-value "ldetail" def) "bot-bolt-main")
         (land-fab-chan-det-bot-bolt fb
                                     "left"
                                     bot-line
                                     (hole-ins (symcat '(#(car order) "_chan_wid")))))
     (if (= (xd-value "rdetail" def) "bot-bolt-main")
         (land-fab-chan-det-bot-bolt fb
                                     "right"
                                     bot-line
                                     (hole-ins (symcat '(#(cadr order) "_chan_wid"))))))
   (+ chan-top (* (if top-holes 5 2) dimspace)))
  ;; end view
  ((lambda(ins / blockObj bolt-offset)
     (setq blockObj view)
     (mirror&delete (vla-put-layer-r (add-shape view ins (symcat '(#fb "_chan"))) "SOLID") ins 'y)
     (foreach bolt (uniquify (append (mapcar (compose '(1- cadr)) top-holes)
                                     (mapcar (compose '(1+ cadr)) bot-holes)))
              (add-bolt-centerline (list (car ins) bolt) 'h))
     (if (member "bot-bolt-main" (list (xd-value "ldetail" def) (xd-value "rdetail" def)))
         (progn
           (setq bolt-offset (list (- (if (and (= fb "back") flip-back)
                                          (- wid 1.125)
                                          1.125))
                                   (- dpt)))
           (add-bolt-centerline (displace-pt ins bolt-offset) 'v))))
   (list (- (* 3 dimspace)) chan-top)))

;; draw side dimensions in main channel detail
(defun land-fab-fb-chan-det-side-dims ()
  (if (begins-with (xd-value "ldetail" def) "front-bolt")
      ((lambda(x y ang)
         (vla-AddDimRotated view (3dpt x (- y 1.5)) (3dpt 0 y) (3dpt (- dimspace) 0) ang)
         (vla-AddDimRotated view (3dpt x (- y 4.5)) (3dpt 0 y) (3dpt (- (* 2 dimspace)) 0) ang))
       (if (= (xd-value "ldetail" def) "front-bolt-2") 0.25 0.5)
       chan-top
       (dtr 90)))
  (if (= (xd-value "ldetail" def) "front-slot")
      (progn
        (vla-setvar 'cmleaderstyle "Standard")
        (add-mleader view
                     '(0 0 0)
                     (list (list 3 (- chan-top 3.65625))
                           (list (if (< (caar holes) 23) (1+ (caar holes)) 5)
                                 (+ chan-top (* 2 dimspace))))
                     (strcat (frac std-hole-dia) " x 3 SLOT"))
        (vla-setvar 'cmleaderstyle "Callout")
        (vla-AddDimRotated view
                           (3dpt 1 (- chan-top 4))
                           (3dpt 0 chan-top)
                           (3dpt (- dimspace) 0)
                           (dtr 90))))
  (if (= (xd-value "ldetail" def) "hanger-sleeve")
      (vla-AddDimRotated view
                         (3dpt 1.25 chan-top)
                         (3dpt 0 chan-top)
                         (3dpt 0 (+ chan-top (* 2 dimspace)))
                         0))
  (if (begins-with (xd-value "rdetail" def) "front-bolt")
      ((lambda(x y ang)
         (vla-AddDimRotated view (3dpt x (- y 1.5)) (3dpt len y) (3dpt (+ len dimspace) 0) ang)
         (vla-AddDimRotated view
                            (3dpt x (- y 4.5))
                            (3dpt len y)
                            (3dpt (+ len (* 2 dimspace)) 0)
                            ang))
       (- len (if (= (xd-value "rdetail" def) "front-bolt-2") 0.25 0.5))
       chan-top
       (dtr 90)))
  (if (= (xd-value "rdetail" def) "front-slot")
      (progn
        (vla-setvar 'cmleaderstyle "Standard")
        (add-mleader view
                     (list len 0 0)
                     (list (list -3 (- chan-top 3.65625))
                           (list (if (> (car (last holes)) (- len 23)) (1- (car (last holes))) -5)
                                 (+ chan-top (* 2 dimspace))))
                     (strcat (frac std-hole-dia) " x 3 SLOT"))
        (vla-setvar 'cmleaderstyle "Callout")
        (vla-AddDimRotated view
                           (3dpt (- len 1) (- chan-top 4))
                           (3dpt len chan-top)
                           (3dpt (+ len dimspace) 0)
                           (dtr 90))))
  (if (= (xd-value "rdetail" def) "hanger-sleeve")
      (vla-AddDimRotated view
                         (3dpt (- len 1.25) chan-top)
                         (3dpt len chan-top)
                         (3dpt 0 (+ chan-top (* 2 dimspace)))
                         0))
  (if holes
      (progn
        (vla-setvar 'cmleaderstyle "Standard")
        (add-mleader view
                     '(0 0 0)
                     (list (displace-pt (car holes) '(0 0.34375))
                           (list (+ (caar holes) dimspace) (+ chan-top dimspace)))
                     (strcat (frac std-hole-dia) " x 1 SLOT (" (to-string (length holes)) ")"))
        (vla-setvar 'cmleaderstyle "Callout")
        (cond ((not (member (xd-value "ldetail" def) '("front-bolt-2" "front-bolt-3" "front-slot")))
               (vla-AddDimRotated view
                                  (3dpt (displace-pt (car holes) '(-1 0)))
                                  (3dpt 0 chan-top)
                                  (3dpt (- dimspace) 0)
                                  (dtr 90)))
              ((not (member (xd-value "rdetail" def) '("front-bolt-2" "front-bolt-3" "front-slot")))
               (vla-AddDimRotated view
                                  (3dpt (displace-pt (last holes) '(1 0)))
                                  (3dpt len chan-top)
                                  (3dpt (+ len dimspace) 0)
                                  (dtr 90)))
              (T
               (vla-AddDimRotated view
                                  (3dpt (displace-pt (cadr holes) '(-1 0)))
                                  (3dpt (- (caadr holes) dimspace) chan-top)
                                  (3dpt (- (caadr holes) dimspace) chan-top)
                                  (dtr 90)))))))

;; draw view for post-under connection in channel detail
(defun land-fab-chan-det-bot-bolt (chan lr bot-line hole-ins / obj bot-line wid web hole-ins)
  ;; setq obj, mirror
  (defun som (o)
    (setq obj o)
    (if (= lr "right")
        (setq obj (mirror&delete obj (list (/ len 2.0) 0 0) 'y)))
    obj)

  (land-fab-set-chan-props chan '(wid web))
  (setq hole-ins (displace-pt hole-ins (list 0 bot-line)))
  ;; channel
  (vla-put-Layer (som (add-polyline view
                                    (list 0 bot-line)
                                    (#! '((12 0 0)
                                          (0 0 0)
                                          (0 #wid 0)
                                          (12 #wid 0)))
                                    nil))
                 "SOLID")
  (vla-put-Layer
   (som (add-polyline view (list 0 (+ bot-line web)) '((0 0 0) (12 0 0)) nil))
   "SOLID")
  ;; hole
  (vla-put-Layer (som (vla-AddCircle view (3dpt hole-ins) (/ 11 32.0))) "SOLID")
  ((lambda(blockObj)
     (mapcar 'som (add-bolt-centerline hole-ins 'both)))
   view)
  ;; arrow
  (som (add-polyline view
                     (list (- dimspace) (+ chan-bot 2))
                     (list '(-1 0 0)
                           (list (+ dimspace 12) 0 0))
                     nil))
  (som (add-polyline view
                     (list (- dimspace) (+ chan-bot 2))
                     '((0 -3 0)
                       (0 -2 0)
                       (0 1 0))
                     nil))
  (vla-SetWidth obj 0 0 1)
  ;; hole dims
  (if (= lr "left")
      (progn
        (vla-AddDimRotated view
                           (3dpt 0 bot-line)
                           (3dpt (displace-pt hole-ins '(-1 0)))
                           (3dpt (- dimspace) 0)
                           (dtr 90))
        (vla-AddDimRotated view
                           (3dpt 0 (+ bot-line wid))
                           (3dpt (displace-pt hole-ins '(0 1)))
                           (3dpt 0 (+ bot-line wid dimspace))
                           0))
      ;; can't mirror text inside a block or it will display backwards
      ((lambda(hole-ins)
         (vla-AddDimRotated view
                            (3dpt len bot-line)
                            (3dpt (displace-pt hole-ins '(1 0)))
                            (3dpt (+ len dimspace) 0)
                            (dtr 90))
         (vla-AddDimRotated view
                            (3dpt len (+ bot-line wid))
                            (3dpt (displace-pt hole-ins '(0 1)))
                            (3dpt 0 (+ bot-line wid dimspace))
                            0))
       (car (mirror-pts-v (list hole-ins) (/ len 2.0))))))

;; populate the detail view block for a side channel
(defun land-fab-draw-lr-chan-det (lr

                                  / view dpt len flg chan-bot chan-top order cope
                                  cope-chan cope-trim def ldetail rdetail)
  (land-fab-set-chan-props lr '(dpt flg))
  (setq len (land-fab-side-chan-len lr)
        view (symcat '(#lr "-chan-detail"))
        cope (?? (symcat '(#lr "_cope")) "none")
        chan-bot (if (= cope "none") dimspace (* 3 dimspace))
        chan-top (+ chan-bot dpt)
        order (land-fab-side-chan-order lr)
        cope-chan (if (/= cope "none") (symcat '(#cope "_chan")))
        cope-trim (if (/= cope "none") (land-fab-coped-chan-flg-trim cope-chan))
        def (land-fab-calc-lr-chan lr)
        ldetail (xd-value "ldetail" def)
        rdetail (xd-value "rdetail" def))
  (add-part-detail-callout view (xd-value (strcat lr "-chan") marks) '(0 0 0))
  ;; outline
  (if (= cope "none")
      (vla-put-Layer (add-rectangle view len dpt (list 0 chan-bot)) "SOLID")
      (progn
        (vla-put-Layer
         (add-polyline view
                       (list 0 chan-top)
                       (land-fab-coped-chan-pts len
                                                dpt
                                                cope-chan
                                                (= (symcat '(#lr "_cope")) (cadr order)))
                       T)
         "SOLID")
        ;; cope note
        (vla-setvar 'cmleaderstyle "Standard")
        (vla-put-Layer
         (add-mleader view
                      '(0 0 0)
                      (if (= cope (car order))
                          (list (list 0 (+ chan-bot (/ dpt 2.0)))
                                (list dimspace (- chan-bot dimspace)))
                          (list (list len (+ chan-bot (/ dpt 2.0)))
                                (list (- len dimspace) (- chan-bot dimspace))))
                      (strcat "COPE TO FIT " cope-chan))
         "DIM")
        (vla-setvar 'cmleaderstyle "Callout")))
  ;; flange lines
  ((lambda(pts xins)
     (vla-put-Layer (add-polyline view (list xins (+ chan-bot flg)) pts nil) "HIDDEN")
     (vla-put-Layer (add-polyline view (list xins (- chan-top flg)) pts nil) "HIDDEN"))
   (if (= cope "none")
       (list '(0 0 0) (list len 0 0))
       (list '(0 0 0) (list (- len cope-trim) 0 0)))
   (if (= cope (car order)) cope-trim 0))
  ;; bottom bolt detail
  ((lambda(bot-line hole-ins / blockObj)
     (setq blockObj view)
     (if (= ldetail "bot-bolt-side")
         (progn
           (add-bolt-centerline (list (car hole-ins) chan-bot) 'v)
           (land-fab-chan-det-bot-bolt lr "left" bot-line hole-ins)))
     (if (= rdetail "bot-bolt-side")
         (progn
           (add-bolt-centerline (list (- len (car hole-ins)) chan-bot) 'v)
           (land-fab-chan-det-bot-bolt lr "right" bot-line hole-ins))))
   (+ chan-top dimspace)
   '(5 1.125))
  ;; section
  ((lambda(ins / blockObj)
     (mirror&delete (vla-put-layer-r (add-shape view ins (symcat '(#lr "_chan"))) "SOLID") ins 'y)
     (if (member "bot-bolt-side" (list ldetail rdetail))
         (progn
           (setq blockObj view)
           (add-bolt-centerline (displace-pt ins (list -1.125 (- dpt))) 'v))))
   (list (- (* 2 dimspace)) chan-top))
  ;; main dim
  (vla-AddDimRotated view (3dpt 0 chan-bot) (3dpt len chan-bot) (3dpt 0 0) 0))

;; return a coped channel outline
(defun land-fab-coped-chan-pts (len dpt cope-chan mirror-p / cope-def cope-web >cope pts)
  (setq cope-def (shape-def cope-chan nil)
        cope-web (nth 2 cope-def)
        >cope (> (car cope-def) dpt)
        pts (displace-pts (slice (apply 'c-channel-pts cope-def) 1 (if >cope 5 10))
                          (list (- cope-web) 0 0)))
  (if >cope
      (progn
        (setq pts (reverse pts)
              pts (cons (list (caar pts) (- dpt) 0) pts)
              pts (reverse pts))))
  (setq pts (append pts
                    (#! '((#len #(- dpt) 0)
                          (#len 0 0)))))
  (if mirror-p
      (mapcar '(lambda(pt) (list (nth 0 pt)
                                 (nth 1 pt)
                                 (- (nth 2 pt))))
              (mirror-pts-v pts (/ len 2.0)))
    pts))

;; return the trim amount for flange lines on a coped channel
(defun land-fab-coped-chan-flg-trim (chan)
  (cond ((begins-with chan "C6") 0.86)
        ((begins-with chan "C8") 1.02)
        ((begins-with chan "C10") 1.18)
        ((begins-with chan "C12") 1.33)))

;; return x offset of front channel relative to platform
(defun land-fab-front-chan-x ()
  (- 0
     fl_ext
     (if (and left_chan
              (<= fl_ext bl_ext))
         (- left_chan_wid)
         0)))

;; add front channel to applicable views and material list
(defun land-fab-channels-front (/ x-ins def)
  (setq x-ins (land-fab-front-chan-x)
        def (land-fab-calc-fb-chan "front"))

  ;; top view
  (vla-put-Layer (add-rectangle top-view front_chan_len front_chan_wid (list x-ins 0 0)) "HIDDEN")
  (vla-put-Layer (add-polyline top-view
                               (list x-ins front_chan_web 0)
                               (list '(0 0 0)
                                     (list front_chan_len 0 0))
                               nil)
                 "HIDDEN")
  (if (= (xd-value "ldetail" def) "hanger-sleeve")
      ((lambda (def)
         (mapcar '(lambda(obj) (vla-put-Layer obj "OBJ"))
                 (add-shape top-view
                            (list (+ 1.25 x-ins) 0 0)
                            (xd-value "shape" def)))
         (vla-put-Layer
          (add-mleader top-view
                       (list (+ x-ins 1.25) (- (xd-value "wid" def)))
                       '((0 0 0)
                         (0 -2 0)
                         (-1 -2 0))
                       (xd-value "front-left-sleeve" marks))
          "DIM"))
       (land-fab-calc-hanger-sleeve "front" "left")))
  (if (= (xd-value "rdetail" def) "hanger-sleeve")
      ((lambda (def)
         (mapcar '(lambda(obj) (vla-put-Layer obj "OBJ"))
                 (add-shape top-view
                            (list (+ plt_wid fr_ext -1.25) 0 0)
                            (xd-value "shape" def)))
         (vla-put-Layer
          (add-mleader top-view
                       (list (+ x-ins front_chan_len -1.25) (- (xd-value "wid" def)))
                       '((0 0 0)
                         (0 -2 0)
                         (1 -2 0))
                       (xd-value "front-right-sleeve" marks))
          "DIM"))
       (land-fab-calc-hanger-sleeve "front" "right")))

  ;; front-view
  (land-fab-add-fb-chan-elev front-view "front" (list x-ins 0 0) nil "OBJ")

  ;; left view
  (vla-put-Layer (add-shape left-view '(0 0 0) front_chan) "OBJ")

  ;; right view
  (if right-view
      (vla-put-Layer (mirror&delete (add-shape right-view '(0 0 0) front_chan) '(0 0) 'y) "OBJ"))

  ;; detail
  (land-fab-draw-fb-chan-det "front"))

;; return the x offset of the back channel relative to the platform
(defun land-fab-back-chan-x ()
  (- 0
     bl_ext
     (if (and left_chan
              (>= fl_ext bl_ext))
         (- left_chan_wid)
         0)))

;; add back channel to applicable views and material list
(defun land-fab-channels-back (/ x-ins def fby)
  (defun fby (y)
    (if flip-back (- y) y))

  (setq x-ins (land-fab-back-chan-x)
        def (land-fab-calc-fb-chan "back")
        order (land-fab-fb-chan-order "back"))

  ;;;;;;;;;;;;
  ;; top view
  ;;;;;;;;;;;;

  ;; channel
  (vla-put-Layer (add-rectangle top-view
                                back_chan_len
                                ((if flip-back - +) back_chan_wid)
                                (list x-ins back_chan_loc 0))
                 "HIDDEN")
  (vla-put-Layer (add-polyline top-view
                               (list x-ins
                                     ((if flip-back - +) back_chan_loc back_chan_web)
                                     0)
                               (list '(0 0 0)
                                     (list back_chan_len 0 0))
                               nil)
                 "HIDDEN")
  ;; hanger sleeves
  (if (= (xd-value (strcat (substr (car order) 1 1) "detail") def) "hanger-sleeve")
      ((lambda(def)
         ((lambda(lines)
            (mapcar '(lambda(obj) (vla-put-Layer obj "OBJ")) lines)
            (if flip-back
                (mapcar '(lambda(line) (mirror&delete line (list 0 back_chan_loc) 'x)) lines)))
          (add-shape top-view
                     (list (+ 1.25 x-ins) back_chan_loc 0)
                     (xd-value "shape" def)))
         (vla-put-Layer
          (add-mleader top-view
                       (list (+ x-ins 1.25) (+ back_chan_loc (fby (- (xd-value "wid" def)))))
                       (#! '((0 0 0)
                             (0 #(fby -2) 0)
                             (-1 #(fby -2) 0)))
                       (xd-value "back-left-sleeve" marks))
          "DIM"))
       (land-fab-calc-hanger-sleeve "back" "left")))
  (if (= (xd-value (strcat (substr (cadr order) 1 1) "detail") def) "hanger-sleeve")
      ((lambda(def)
         ((lambda(lines)
            (mapcar '(lambda(obj) (vla-put-Layer obj "OBJ")) lines)
            (if flip-back
                (mapcar '(lambda(line) (mirror&delete line (list 0 back_chan_loc) 'x)) lines)))
          (add-shape top-view
                     (list (+ plt_wid br_ext -1.25) back_chan_loc 0)
                     (xd-value "shape" def)))
         (vla-put-Layer
          (add-mleader top-view
                       (list (+ x-ins back_chan_len -1.25)
                             (+ back_chan_loc (fby (- (xd-value "wid" def)))))
                       (#! '((0 0 0)
                             (0 #(fby -2) 0)
                             (1 #(fby -2) 0)))
                       (xd-value "back-right-sleeve" marks))
          "DIM"))
         (land-fab-calc-hanger-sleeve "back" "right")))

  ;; back view
  (if back-view
      (land-fab-add-fb-chan-elev back-view "back" (list x-ins 0 0) flip-back "OBJ"))

  ;; left view
  ((lambda (ins / obj)
     (setq obj (add-shape left-view ins back_chan))
     (vla-put-Layer obj "OBJ")
     (if flip-back (mirror&delete obj ins 'y)))
   (list back_chan_loc 0 0))
  ;; right view
  (if right-view
      ((lambda(ins)
	 (setq obj (add-shape right-view ins back_chan))
	 (vla-put-Layer obj "OBJ")
	 (if (not flip-back) (mirror&delete obj ins 'y)))
       (list (- back_chan_loc) 0 0)))

  (land-fab-draw-fb-chan-det "back"))
  
;; add left channel to applicable views and material list
(defun land-fab-channels-left (/ fchanx bchanx lchanx lchany len)
  ;; top view
  (setq fchanx (land-fab-front-chan-x)
        bchanx (land-fab-back-chan-x)
        lchanx (max fchanx bchanx)
        lchany (if (< fchanx bchanx) front_chan_web 0)
        len (land-fab-side-chan-len "left"))
  (vla-put-Layer (add-rectangle top-view
                                (- left_chan_wid)
                                len
                                (list lchanx lchany 0))
                 "HIDDEN")
  (vla-put-Layer (add-polyline top-view
                               (list (- lchanx left_chan_web) lchany 0)
                               (list '(0 0 0)
                                     (list 0 len 0))
                               nil)
                 "HIDDEN")
  ((lambda(/ ins obj)
     ;; front view
     (setq ins (list lchanx 0 0)
           obj (add-shape front-view ins left_chan))
     (vla-put-Layer obj (if (= left_cope "front") "HIDDEN" "OBJ"))
     (mirror&delete obj ins 'y)
     (if (member fl_chan_num '("25" "35"))
         ((lambda(blockObj)
            (add-bolt-centerline (list (- lchanx 1.125) (- left_chan_dpt)) 'v))
          front-view))
     ;; back view
     (if back-view
         (progn
           (setq obj (add-shape back-view ins left_chan))
           (vla-put-Layer obj "OBJ")
           (mirror&delete obj ins 'y)
           (if (member bl_chan_num '("25" "35"))
               ((lambda(blockObj)
                  (add-bolt-centerline (list (- lchanx 1.125) (- left_chan_dpt)) 'v))
                back-view))))))

  ;; left view
  ((lambda (/ x pts)
     (setq x (if (= left_cope "front") front_chan_web 0)
	   pts (list '(0 0 0) (list len 0 0)))
     (vla-put-Layer (add-rectangle left-view len (- left_chan_dpt) (list x 0 0)) "OBJ")
     (vla-put-Layer (add-polyline left-view (list x (- left_chan_flg) 0) pts nil) "HIDDEN")
     (vla-put-Layer (add-polyline left-view
                                  (list x (+ (- left_chan_dpt) left_chan_flg) 0)
                                  pts
                                  nil)
                    "HIDDEN")))

  ;; detail
  (if left-chan-detail
      (land-fab-draw-lr-chan-det "left")))

(defun land-fab-channels-right (/ fchanx bchanx rchanx rchany len)
  ;; top view
  (setq fchanx (+ (land-fab-front-chan-x) front_chan_len)
        bchanx (+ (land-fab-back-chan-x) back_chan_len)
        rchanx (min fchanx bchanx)
        rchany (if (>f fchanx bchanx 0.01)
                   (if (= right_cope "front")
                       front_chan_web
                       front_chan_wid)
                   0)
        len (land-fab-side-chan-len "right"))
  (vla-put-Layer (add-rectangle top-view
                                right_chan_wid
                                len
                                (list rchanx rchany 0))
                 "HIDDEN")
  (vla-put-Layer (add-polyline top-view
                               (list (+ rchanx right_chan_web) rchany 0)
                               (list '(0 0 0)
                                     (list 0 len 0))
                               nil)
                 "HIDDEN")
  ;; front view
  (vla-put-Layer
   (add-shape front-view (list rchanx 0 0) right_chan)
   (if (= right_cope "front") "HIDDEN" "OBJ"))
  (if (member fr_chan_num '("25" "35"))
         ((lambda(blockObj)
            (add-bolt-centerline (list (+ rchanx 1.125) (- right_chan_dpt)) 'v))
          front-view))

  ;; back view
  (set-active-layer "OBJ")
  (if back-view
      (progn
        (add-shape back-view (list rchanx 0 0) right_chan)
        (if (member br_chan_num '("25" "35"))
            ((lambda(blockObj)
               (add-bolt-centerline (list (+ rchanx 1.125) (- right_chan_dpt)) 'v))
             back-view))))

  ;; right view
  (if right-view
      ((lambda (/ x pts)
         (setq x (if (= right_cope "front") (- front_chan_web) 0)
	       pts (list '(0 0 0) (list (- len) 0 0)))
         (add-rectangle right-view (- len) (- right_chan_dpt) (list x 0 0))
         (vla-put-Layer (add-polyline right-view (list x (- right_chan_flg) 0) pts nil) "HIDDEN")
         (vla-put-Layer
          (add-polyline right-view (list x (+ (- right_chan_dpt) right_chan_flg) 0) pts nil)
          "HIDDEN"))))
  (set-active-layer "DIM")

  ;; detail
  (if right-chan-detail
      (land-fab-draw-lr-chan-det "right")))

