(set-file-docstring "Functions for calculating landing hanger fabs")

(defun-q land-hanger-p (corner)
  "Returns T if there is a hanger at this corner"
  (declare (vars (corner str (member corner '("fl" "fr" "bl" "br"))))
           with-data)
  (member (cornerkey "chan_num") '("40" "50")))
(defun-r 'land-hanger-p)

(defun-q land-hanger-calc (corner / chan supp len qty dia kick-p)
  "Calculates the hanger material and hardware for this CORNER of the current landing

Assumed to be called by %land-sl--calc"
  (declare (vars (corner str (member corner '("fl" "fr" "bl" "br"))))
           with-data)
  (setq chan (cornerkey "chan_num")
        supp (cornerkey "supp_num")
        len (ceil (+ (cornerkey "len")
                     flr_thick
                     (if (begins-with corner "f") front_chan_dpt back_chan_dpt)
                     6))
        qty (ceil (/ len 144.0))
        len (ceil (/ len qty))
        dia (strcat (if (= chan "40") "3/4" "1") " DIA.")
        kick-p (member (substr supp 2 1) '("7" "8" "9")))
  ;; hanger rod and hardware
  (add-material
   '(("qty" #qty)
     ("desc" #(strcat "ALL THREAD " dia))
     ("length" #len)
     ("note" "SL")))
  (mapcar 'add-hardware
    '((("qty" 3)
       ("desc" #(strcat "HEX NUT " dia))
       ("note" "SL"))
      (("qty" 2)
       ("desc" #(strcat "FLAT WASHER " dia))
       ("note" "SL"))))
  (if (> qty 1)
      (add-hardware
       '(("qty" #(1- qty))
         ("desc" #(strcat "COUPLER NUT " dia))
         ("note" "SL"))))
  ;; flag plate
  (if (member (substr supp 2 1) '("3" "8"))
      (add-material
       '(("detail" "P1")
         ("qty" 1)
         ("desc" #(strcat "PLATE " (frac (cornerkey "plt_thick"))))
         ("width" #(frac (cornerkey "plt_wid")))
         ("length" #(frac (- (cornerkey "plt_hgt") 0.125)))
         ("note" "SL"))))
  ;; 8x8 plate
  (if (member (substr supp 2 1) '("1" "2" "4" "5" "7"))
      (add-material
       '(("qty" 1)
         ("desc" "PLATE 3/8")
         ("width" "8")
         ("length" "8")
         ("note" "SL"))))
  (if kick-p
      (progn
        ;; corner-clipped gusset
        (mapcar 'add-material
          '((("detail" "P2")
             ("qty" #(+ 1 (if (or (begins-with (cornerkey "kick_style") "06")
                                  (= (cornerkey "kick_deck") "Parallel"))
                              1 0)))
             ("desc" "PLATE 1/4")
             ("width" "4")
             ("length" "4")
             ("note" "SL"))
            (("qty" 1)
             ("desc" "ANGLE 3 x 3 x 1/4")
             ("length" #(frac (cornerkey "kick_len"))))))
        ;; built-up flag plate
        (if (ends-with supp "9")
            (mapcar 'add-material
              '((("detail" "P3")
                 ("qty" 1)
                 ("desc" #(strcat "PLATE " (frac (cornerkey "plt_thick"))))
                 ("width" #(frac (cornerkey "plt_wid")))
                 ("length" #(frac (- (cornerkey "plt_hgt") 0.125)))
                 ("note" "SL"))
                (("detail" "P3a")
                 ("qty" 4)
                 ("desc" "ANGLE 3 x 3 x 1/4")
                 ("length" #(frac (- (cornerkey "plt_wid") 3.75)))))))
        ;; slab angle
        (cond ((begins-with (cornerkey "kick_style") "07")
               (add-material
                '(("detail" "A1")
                  ("qty" 1)
                  ("desc" "ANGLE 4 x 4 x 3/8")
                  ("length" "18")
                  ("note" "SL")))
               (add-hardware
                '(("qty" 2)
                  ("desc" "HILTI KB-TZ2 WEDGE ANCHOR 5/8 DIA. x 4 3/4")
                  ("note" "SL"))))
              ((begins-with (cornerkey "kick_style") "08")
               (add-material
                '(("qty" 1)
                  ("desc" "ANGLE 4 x 4 x 3/8")
                  ("length" "18")
                  ("note" "SL")))))))
  (list material hardware))
(defun-r 'land-hanger-calc)
