(set-file-docstring "Functions for making landing ledger fabs")

(defun-q land-ledger-p (corner)
  "Returns T if there is a ledger at this corner"
  (declare (vars (corner str (member corner '("fl" "fr" "bl" "br"))))
           with-data)
  (member (substr (cornerkey "chan_num") 1 1) '("6" "7")))
(defun-r 'land-ledger-p)

(defun-q land-ledger-calc (corner / gusset chan)
  "Calculates the ledger material and hardware for this CORNER of the current landing

Assumed to be called by %land-sl--calc"
  (declare (vars (corner str (member corner '("fl" "fr" "bl" "br"))))
           with-data)
  (defun gusset (ang)
    (setq ang (strsplit (substr ang 2) "x"))
    (list (frac (- (read (car ang)) 0.5)) (frac (- (read (cadr ang)) 0.5))))
  (setq chan (cornerkey "chan_num"))
  ;; ledgers
  (if (= chan "76")
      ;; inverted fixed
      (add-material
       '(("qty" 1)
         ("desc" "ANGLE 6 x 4 x 3/8")
         ("length" "8")
         ("note" "SL")))
      (if (member chan '("60" "61" "62" "70" "71" "72"))
          ;; clip angle ledger
          (mapcar 'add-material
            '((("detail" "L1")
               ("qty" 1)
               ("desc" "ANGLE 4 x 4 x 3/8")
               ("length" "8")
               ("note" "SL"))
              (("detail" "P1")
               ("qty" 2)
               ("desc" "PLATE 1/4")
               ("length" #(car (gusset (cornerkey "angle"))))
               ("width" #(cadr (gusset (cornerkey "angle")))))
              (("detail" "L1a")
               ("qty" 1)
               ("desc" "ANGLE 4 x 3 x 1/4")
               ("length" "2"))))
          ;; plain ledger
          (progn
            (mapcar 'add-material
              '((("detail" "L2")
                 ("qty" 1)
                 ("desc" "ANGLE 4 x 4 x 3/8")
                 ("length" "8")
                 ("note" "SL"))
                (("detail" "P1")
                 ("qty" 2)
                 ("desc" "PLATE 1/4")
                 ("length" #(car (gusset (cornerkey "angle"))))
                 ("width" #(cadr (gusset (cornerkey "angle")))))))
            (if (begins-with chan "6")
                ;; bent rod clip
                (add-material
                 '(("detail" "R1")
                   ("qty" 1)
                   ("desc" "BENT C-SHAPE ROUND BAR 5/8 DIA.")
                   ("note" "SL")))))))
  ;; anchors and hardware
  (if (member chan '("61" "62" "67" "68" "71" "72" "78" "79"))
      (progn
        ;; anchor plate
        (add-material
         '(("detail" "P2")
           ("qty" 1)
           ("desc" "PLATE 3/8")
           ("width" "12")
           ("length" "12")
           ("note" "SL")))
        (if (member chan '("61" "67" "71" "78"))
            ;; expansion anchors
            (add-hardware
             '(("qty" 4)
               ("desc" "HILTI KB-TZ2 WEDGE ANCHOR 5/8 DIA. x 6")
               ("note" "SL")))
            ;; screw anchors
            (add-hardware
             '(("qty" 4)
               ("desc" "HILTI KH-EZ SCREW ANCHOR 5/8 DIA. x 5 1/2")
               ("note" "SL")))))))
(defun-r 'land-ledger-calc)
