(psc-include '("land/util.lsp"))

;; platform function dispatch
(defun land-fab-platform ()
  (attempt-apply (read (strcat "land-fab-platform-" (substr tread_style 1 3))) nil))

;; 100-series pan-filled platform draw function
(defun land-fab-platform-100 (/ material decking-def)
  (setq edge-forms-def (land-fab-platform-100-calc-edge-forms)
        decking-def (land-fab-platform-100-calc-decking))
  (append! 'material (combine-like-data edge-forms-def '("desc" "length" "width" "detail") "id"))
  (add-to-list 'material decking-def)
  (table-add-data mat-table (mapcar 'process-material (strip-nil material)))
  (land-fab-platform-100-draw-decking decking-def)
  (land-fab-platform-100-draw-edge-forms edge-forms-def)
  ;; top view dim
  (vla-AddDimRotated top-view
                     (3dpt plt-offset plt_dpt)
                     (3dpt plt-end plt_dpt)
                     (3dpt 0 (+ plt_dpt (* 2 dimspace)))
                     0)
  ;; side view dims
  (vla-AddDimRotated left-view
                     (3dpt 0 flr_thick)
                     (3dpt plt_dpt flr_thick)
                     (3dpt 0 (+ flr_thick (* 2 dimspace)))
                     0)
  (if right-view
      (vla-AddDimRotated right-view
                         (3dpt 0 flr_thick)
                         (3dpt (- plt_dpt) flr_thick)
                         (3dpt 0 (+ flr_thick (* 2 dimspace)))
                         0)))

(defun land-fab-platform-100-calc-decking ()
  (#! '(("id" ("decking"))
        ("desc" "BR DECKING 1 1/2 x 36")
        ("qty" #(ceil (/ (- plt_wid_eff 4) 36.0)))
        ("length" #(fix (- plt_dpt 0.5))))))

(defun land-fab-platform-100-draw-decking (def)
  ;; top view
  (add-polyline top-view
                (list (/ plt_wid 2.0) (/ plt_dpt 2.0))
                '((-1 -4 0)
                  (0 -6 0)
                  (0 6 0)
                  (1 4 0))
                nil)
  (add-mtext top-view
             (list (1+ (/ plt_wid 2.0)) (/ plt_dpt 2.0))
             (xd-value "decking" marks)
             1
             'middleleft)
  ;; front and back views
  ((lambda(/ obj)
     (setq obj (add-decking front-view (list (+ plt-offset 2) 0) (list (- plt-end 2) 0)))
     (if (= front_bolt_to "None")
         (progn
           (vla-put-Layer obj "HIDDEN")
           (vla-put-LinetypeScale obj 0.5))
         (vla-put-Layer obj "SOLID"))))
  (if back-view
      (vla-put-Layer (add-decking back-view (list (+ plt-offset 2) 0) (list (- plt-end 2) 0))
                     "SOLID"))
  ;; side views
  ((lambda(/ back-end pts)
     (setq back-end (+ 0.25 (xd-value "length" def))
           pts (list '(0.25 0 0) '(0.25 1.5 0) (list back-end 1.5) (list back-end 0))
           pts (rect-pts (xd-value "length" def) 1.5 '(0.25 0)))
     (vla-put-Layer (add-polyline left-view '(0 0 0) pts T) "SOLID")
     (if right-view
         (vla-put-Layer (add-polyline right-view '(0 0 0) (mirror-pts-v pts 0) T) "SOLID")))))

;; return a list of edge form definitions
(defun land-fab-platform-100-calc-edge-forms (/ calc-len calc-lr calc-fb locs forms)
  (setq locs (land-edge-form-locs))

  ;; calculate length based on nominal length and intersections
  (defun calc-len (full-len end1-locs end2-locs / e1p e2p)
    (setq e1p (member* end1-locs locs)
          e2p (member* end2-locs locs))
    (frac (- full-len (cond ((and e1p e2p) 0.3125)
                            ((or e1p e2p) 0.125)
                            (T 0)))))

  ;; return left/right definition
  (defun calc-lr (lr / lr-sym form-p)
    (setq lr-sym (read lr)
          form-p (member lr-sym locs))
    (add-to-list 'forms
                 (strip-nil
                  (#! '(("id" #(strcat lr "-edge-form"))
                        ("loc" #lr)
                        ("qty" 1)
                        ("desc" #(if form-p "PLATE 10GA" "ANGLE 2 1/2 x 2 1/2 x 1/4"))
                        ("full-len" #plt_dpt)
                        ("length" #(calc-len plt_dpt
                                             (list 'front (read (strcat "front-" lr)))
                                             (list 'back (read (strcat "back-" lr)))))
                        #(if form-p '("width" "7.22"))
                        #(if form-p '("detail" (edge-form))))))))
  (mapcar 'calc-lr '("left" "right"))

  ;; return front/back definition
  (defun calc-fb (fb / calc-full-len base fb-sym full-len calc-len-args)
    ;; return the full len of a partial fb edge form
    (defun calc-full-len (fb lr / offset wid)
      (setq lr (substr lr 1 1)
            offset (symcat '(#fb "_" #lr "-stair_offset")))
      (if offset
          (+ offset 0.25)
        (progn
          (setq lr (if (= lr "l") "r" "l")
                offset (symcat '(#fb "_" #lr "-stair_offset"))
                wid (symcat '(#fb "_" #lr "-stair_wid")))
          (- plt_wid offset wid -0.25))))

    (setq base '(("qty" 1)
                 ("desc" "PLATE 10GA")
                 ("width" "7.22")
                 ("detail" (edge-form)))
          fb-sym (read fb)
          full-len (if (member fb '("front" "back"))
                       plt_wid_eff
                     (apply 'calc-full-len (strsplit fb "-")))
          calc-len-args (cons full-len (cond ((wcmatch fb "*left*") '(left nil))
                                             ((wcmatch fb "*right*") '(right nil))
                                             (T '(left right)))))
    (if (member fb-sym locs)
        (add-to-list 'forms
          (append base
                  (#! '(("id" #(strcat fb "-edge-form"))
                        ("loc" fb)
                        ("full-len" #full-len)
                        ("length" #(apply 'calc-len calc-len-args))))))))
  (foreach fb '("front" "front-left" "front-right" "back" "back-left" "back-right")
           (if (member (read fb) locs) (calc-fb fb)))
  forms)

(setq edge-form-profile-pts-l '((0.75 3.5 0)
                                (0 3.5 0)
                                (0 0 0)
                                (3.5  0))
      edge-form-profile-pts-r (mirror-pts-v edge-form-profile-pts-l 0))

;; draw a front or back edge form in all views
(defun land-fab-platform-100-draw-fb-edge-form (def /
                                                id front-p back-p left-p right-p full-p
                                                len wid)
  (setq id (xd-value "id" def)
        back-p (wcmatch id "*back*")
        front-p (not back-p)
        right-p (wcmatch id "*right*")
        left-p (wcmatch id "*left*")
        full-p (and (not right-p) (not left-p))
        len ((if right-p - +) (xd-value "full-len" def))
        wid ((if back-p - +) 3.5))
  (mapcar '(lambda(obj) (vla-put-Layer obj "SOLID"))
    (append
     ;; top view
     ((lambda(/ ins)
        (setq ins (list (if right-p plt-end plt-offset)
                        (if back-p plt_dpt 0)))
        (add-mleader top-view
                     (displace-pt ins (list (/ len 2.0) wid))
                     (if back-p
                         '((0 0) (0 -2) (1 -2))
                         '((0 0) (0 2) (1 2)))
                     (xd-value id marks))
        (list
         (add-rectangle top-view len wid ins)
         (add-polyline top-view
                       (displace-pt ins (list 0 ((if back-p - +) 0.75) 0))
                       (list '(0 0 0) (list len 0 0))
                       nil))))
     ;; left view
     (if (not right-p)
         ((lambda(/ ins pts offset)
            (setq ins (list (if front-p 0 plt_dpt) 0)
                  pts (if front-p edge-form-profile-pts-l edge-form-profile-pts-r)
                  offset ((if front-p - +) 0.1345))
            (add-mleader left-view
                         (displace-pt ins '(0 1))
                         (if front-p
                             '((0 0) (-2 0) (-1 2) (-2 2))
                             '((0 0) (2 0) (1 2) (2 2)))
                         (xd-value id marks))
            (list (offset&close (add-polyline left-view ins pts nil) offset nil nil)))))
     ;; right view
     (if (and right-view
              (or right-p full-p))
         ((lambda(/ ins pts offset)
            (setq ins (list (if front-p 0 (- plt_dpt)) 0)
                  pts (if front-p edge-form-profile-pts-r edge-form-profile-pts-l)
                  offset ((if front-p + -) 0.1345))
            (add-mleader right-view
                         (displace-pt ins '(0 1))
                         (if front-p
                             '((0 0) (2 0) (1 2) (2 2))
                             '((0 0) (-2 0) (-1 2) (-2 2)))
                         (xd-value id marks))
            (list (offset&close (add-polyline right-view ins pts nil) offset nil nil)))))
     ((lambda(/ ins)
        (setq ins (list (if right-p plt-end plt-offset) 0))
        (strip-nil
         (list
          ;; front view
          (if (or (not back-view) front-p) (add-rectangle front-view len 3.5 ins))
          ;; back view
          (if (and back-view back-p) (add-rectangle back-view len 3.5 ins)))))))))

;; use above for all front/back locations
(foreach loc '("front" "front-left" "front-right" "back" "back-left" "back-right")
         (set (read (strcat "land-fab-platform-100-draw-" loc "-edge-form"))
              land-fab-platform-100-draw-fb-edge-form))

;; draw a left or right edge form in all views
(defun land-fab-platform-100-draw-lr-edge-form (def / id right-p form-p wid len)
  (setq id (xd-value "id" def)
        right-p (= id "right-edge-form")
        form-p (= (xd-value "desc" def) "PLATE 10GA")
        wid (if form-p 3.5 2.5)
        len (xd-value "full-len" def))
  ;; geometry
  (mapcar '(lambda(obj) (vla-put-Layer obj "SOLID"))
    (append
     ;; top view
     ((lambda(/ ins thk)
        (setq ins (list (if right-p plt-end plt-offset) 0)
              thk ((if right-p - +) (if form-p 0.75 0.25)))
        (add-mleader top-view
                     (displace-pt ins (list ((if right-p - +) wid) (/ len 2.0)))
                     (if right-p
                         '((0 0) (-2 0) (-1 2) (-2 2))
                         '((0 0) (2 0) (1 2) (2 2)))
                     (xd-value id marks))
        (list
         (add-rectangle top-view ((if right-p - +) wid) len ins)
         (add-polyline top-view
                       (displace-pt ins (list thk 0))
                       (list '(0 0 0) (list 0 len 0))
                       nil))))
     ;; left view
     (if (not right-p)
         (progn
           (add-mleader left-view
                        (list (/ plt_dpt 2.0) flr_thick)
                        '((0 0) (0 2) (1 2))
                        (xd-value id marks))
           (list (add-rectangle left-view len wid '(0 0)))))
     ;; right view
     (if (and right-view right-p)
         (progn
           (add-mleader right-view
                        (list (- (/ plt_dpt 2.0)) flr_thick)
                        '((0 0) (0 2) (1 2))
                        (xd-value id marks))
           (list (add-rectangle right-view (- len) wid '(0 0)))))
     ;; front and back views
     ((lambda(/ ins pts add)
        (defun add (view / pline)
          (setq pline (add-polyline view ins pts (not form-p)))
          (if form-p
              (offset&close pline ((if right-p + -) 0.1345) nil nil)
              pline))
        (setq ins (list (if right-p plt-end plt-offset) 0)
              pts (if form-p
                      edge-form-profile-pts-l
                    (rotate-pts (apply 'angle-pts (shape-def "L2-1/2x2-1/2x1/4" nil)) (dtr 90))))
        (if right-p
            (setq pts (mapcar '(lambda(pt) (rcons (first-two pt) (- (last pt))))
                              (mirror-pts-v pts 0))))
        (strip-nil (list (add front-view) (if back-view (add back-view)))))))))

;; use above for left and right
(foreach loc '("left" "right")
         (set (read (strcat "land-fab-platform-100-draw-" loc "-edge-form"))
              land-fab-platform-100-draw-lr-edge-form))

;; dispatch to draw function for all edge forms
(defun land-fab-platform-100-draw-edge-forms (defs)
  (foreach def defs
           (apply (read (strcat "land-fab-platform-100-draw-" (xd-value "id" def))) (list def)))
  ;; top view dims
  ((lambda(defs)
     (mapcar
      '(lambda(fb / dim-line snap-line left right)
        (if (= fb "front")
            (setq dim-line (+ 3.5 (* 2 dimspace))
                  snap-line 3.5)
            (setq dim-line (- plt_dpt (+ 3.5 (* 2 dimspace)))
                  snap-line (- plt_dpt 3.5)))
        (setq left (xd-value (strcat fb "-left-edge-form") defs)
         right (xd-value (strcat fb "-right-edge-form") defs))
        (if (or left right)
            (progn
              (vla-AddDimRotated top-view
                                 (3dpt (if left
                                           (xd-value "full-len" left)
                                           (- plt_wid_eff (xd-value "full-len" right)))
                                       snap-line)
                                 (3dpt 0 dim-line)
                                 (3dpt 0 dim-line)
                                 0)
              (if (and left right)
                  (vla-AddDimRotated top-view
                                     (3dpt (xd-value "full-len" left) snap-line)
                                     (3dpt (- plt_wid_eff (xd-value "full-len" right)) snap-line)
                                     (3dpt 0 dim-line)
                                     0))
              (vla-AddDimRotated top-view
                                 (3dpt (if right
                                           (- plt_wid_eff (xd-value "full-len" right))
                                           (xd-value "full-len" left))
                                       snap-line)
                                 (3dpt plt_wid dim-line)
                                 (3dpt 0 dim-line)
                                 0))))
      '("front" "back")))
   (mapcar '(lambda(def) (list (xd-value "id" def) def)) defs)))

;; 200 series checker plate platform draw function
(defun land-fab-platform-200 (/ material chk-def can-def tube-def)
  (setq tube-def (land-fab-platform-200-calc-tubes)
        can-def (land-fab-platform-200-calc-cans)
        chk-def (land-fab-platform-200-calc-floor-plate))
  (if (> (xd-value "qty" tube-def) 0)
      (add-to-list 'material tube-def))
  (add-to-list 'material can-def)
  (add-to-list 'material chk-def)
  (table-add-data mat-table (mapcar 'process-material (strip-nil material)))
  (if (= quiet_tread "1")
      (table-add-data mat-table
                      (mapcar '(lambda(x / id mark desc)
                                (#! '(("mark" #(strcat "QT" (caar x)))
                                      ("qty" #(cadr x))
                                      ("desc" "QUIET TREAD 1/8")
                                      ("width" #(cadar x))
                                      ("length" #(caddar x)))))
                              (land-qt-takeoff))))

  (if galv-p
      (table-add-data (get-table-by-title "Hardware")
                      (#! '((("qty" #(* 3 (xd-value "qty" can-def)))
                             ("desc" "DRIVE PIN 3/16 x 5/8"))))))

  (land-fab-platform-200-draw-cans can-def)
  (if (= quiet_tread "1") (land-fab-platform-200-draw-qt))
  (land-fab-platform-200-draw-tubes)
  (land-fab-platform-200-draw-floor-plate chk-def))

;; return material list + extras for floor plate
(defun land-fab-platform-200-calc-floor-plate (/ y-offset)
  (#! '(("id" ("floor-plate"))
        ("desc" #(strcat "CHECKER PLATE " (frac chk_thk)))
        ("qty" 1)
        ("width" #(frac chk_dpt))
        ("length" #(frac plt_wid_eff))
        ("x-offset" #plt-offset)
        ("y-offset" #chk_offset))))

;; draw floor plate in all views
(defun land-fab-platform-200-draw-floor-plate (def / len wid x y)
  (setq len (distof (xd-value "length" def))
        wid (distof (xd-value "width" def))
        x (xd-value "x-offset" def)
        y (xd-value "y-offset" def))
  ;; top view length dim
  ((lambda(/ back)
     (setq back (max plt_dpt (+ y wid)))
     (vla-AddDimRotated top-view
                        (3dpt x back)
                        (3dpt (+ x len) back)
                        (3dpt 0 (+ back (* 2 dimspace)))
                        0)))
  ;; left and right view dims
  (mapcar '(lambda(lr / view high-dims low-dims front back y-diff add-dims)
             (defun add-dims (dim-line dims)
               (mapcar '(lambda(d)
                          (apply 'vla-AddDimRotated
                                 (cons view (rcons (mapcar '3dpt (rcons d dim-line)) 0))))
                       (if (= lr "right") (mapcar '(lambda(d) (mirror-pts-v d 0)) dims) dims)))
             (setq view (if (= lr "left") left-view right-view)
                   dim-line (list 0 (+ flr_thick (* 2 dimspace)))
                   front (list (xd-value "y-offset" def) flr_thick)
                   back (list (+ (xd-value "y-offset" def) wid) flr_thick)
                   y-diff (if galv-p chk_thk 0.32))
             ;; overall width
             (add-to-list 'high-dims (list front back))
             ;; front holdback
             (if (= front_bolt_to "None")
                 (add-to-list 'low-dims (list (list 0.25 (- flr_thick chk_thk)) (list 0 flr_thick)))
                 (if (/= chk_hold 0)
                     (add-to-list 'high-dims (list front (list 0 (- flr_thick y-diff))))))
             ;; back holdback
             (if (wcmatch back_bolt_to "*Stair*")
                 (if (/= chk_hold 0)
                     (add-to-list 'high-dims
                                  (list back (displace-pt back (list chk_hold (- y-diff))))))
                 (add-to-list 'low-dims (list (displace-pt back (list -0.25 (- chk_thk))) back)))
             (add-dims dim-line high-dims)
             (add-dims (displace-pt dim-line (list 0 (- dimspace))) low-dims)
             ;; callout
             ((lambda(dir)
                (add-mleader view
                             (list (* dir (/ plt_dpt 2.0)) flr_thick)
                             (#! '((0 0) (0 #dimspace) (#(* dir 2) #dimspace)))
                             (xd-value "floor-plate" marks)))
              (if (= lr "left") 1 -1)))
          (if right-view '("left" "right") '("left")))
  (mapcar '(lambda(obj) (vla-put-Layer obj "SOLID"))
          (strip-nil
           (list
            ;; top view
            (add-rectangle top-view len wid (list x y))
            ;; front view
            (add-rectangle front-view len (- chk_thk) (list x flr_thick))
            ;; back view
            (if back-view
                (add-rectangle back-view len (- chk_thk) (list x flr_thick)))
            ;; left view
            (add-rectangle left-view wid (- chk_thk) (list y flr_thick))
            ;; right view
            (if right-view
                (add-rectangle right-view (- wid) (- chk_thk) (list (- y) flr_thick)))))))

;; return material list + extras for cans
(defun land-fab-platform-200-calc-cans (/ def y-offset len base)
  (setq def (land-calc-can-spacing)
        y-offset (cond (galv-p 0)
                       ((= front_bolt_to "None") 0.25)
                       (T 1.5))
        len (- plt_dpt
               y-offset
               (cond ((= back_bolt_to "None") 0.25)
                     ((or galv-p (= back_bolt_to "Landing Channel")) 0)
                     (T 1.5)))
        base (#! '(("id" ("cans"))
                   ("length" #(frac len))
                   ("qty" #(length def))
                   ("locs" #def)
                   ("y-offset" #y-offset)
                   ("detail" (can)))))
  (append base
          (if galv-p
              (#! '(("desc" "CHANNEL C6 x 8.2")
                    ("height" 1.92)
                    ("pts" #(displace-pts
                             (rotate-pts (apply 'c-channel-pts (shape-def "C6x8.2" nil)) (dtr -90))
                             '(3 1.92 0)))))
              (#! '(("desc" "PLATE 14GA")
                    ("width" "10.64")
                    ("height" 2.5)
                    ("pts" #(fillet-all-pts '((-3 2.5 0)
                                              (-3 0 0)
                                              (3 0 0)
                                              (3 2.5 0))
                                            0.25)))))))

;; draw cans in all views
(defun land-fab-platform-200-draw-cans (def / locs)
  (setq locs (xd-value "locs" def))
  ;; top view dims
  ((lambda(/ can-back plt-back dim-line)
     (setq can-back (- plt_dpt (cond (galv-p 0)
                                     ((= back_bolt_to "2 Stairs") 1.5)
                                     (T 0.25)))
           plt-back (max plt_dpt (+ (xd-value "y-offset" chk-def) (distof (xd-value "width" chk-def))))
           dim-line (3dpt 0 (+ plt-back dimspace)))
     (vla-AddDimRotated top-view (3dpt (car locs) can-back) (3dpt plt-offset plt-back) dim-line 0)
     (vla-AddDimRotated top-view (3dpt (last locs) can-back) (3dpt plt-end plt-back) dim-line 0)
     (set-active-dimstyle "1xFRACTIONAL_TYP")
     (vla-AddDimRotated top-view (3dpt (car locs) can-back) (3dpt (cadr locs) can-back) dim-line 0)
     (set-active-dimstyle "1xFRACTIONAL")
     (add-mleader top-view
                  (list (+ plt-offset 2) can-back)
                  '((0 0) (0 2) (-5 2))
                  (strcat (xd-value "cans" marks) " TYP"))))
  ;; top view
  ((lambda(/ y add)
     (defun add (x / len)
       (setq len (distof (xd-value "length" def)))
       (vla-put-Layer (add-rectangle top-view 6 len (list (- x 3) y)) "HIDDEN")
       (vla-put-Layer (add-polyline top-view (list x y) (list '(0 0 0) (list 0 len 0)) nil)
                      "CENTER"))
     (setq y (xd-value "y-offset" def))
     (mapcar 'add locs)))
  ;; front and back views
  ((lambda(/ add add-to pts view)
     (defun add (x / obj)
       (setq obj (add-polyline view (list x 0) (xd-value "pts" def) galv-p))
       (vla-put-Layer-r obj "SOLID")
       (if (not galv-p)
           (offset&close obj -0.0747 nil nil)))
     (defun add-to (view) (mapcar 'add locs))
     (if (or galv-p (= front_bolt_to "None"))
         (add-to front-view))
     (if back-view
         (add-to back-view))))
  ;; left and right views
  ((lambda(/ add)
     (defun add (view dir / x len)
       (setq x (* dir (xd-value "y-offset" def))
             len (* dir (distof (xd-value "length" def))))
       (vla-put-Layer (add-rectangle view len (xd-value "height" def) (list x 0)) "SOLID")
       (if galv-p
           (vla-put-Layer (add-polyline view (list x 1.72) (list '(0 0 0) (list len 0 0)) nil)
                          "HIDDEN")))
     (add left-view 1)
     (if right-view (add right-view -1)))))
       
;; return material list for header tubes
(defun land-fab-platform-200-calc-tubes ()
  (#! '(("id" ("tube"))
        ("desc" "HSS 2 1/2 x 1 1/2 x .065")
        ("qty" #(if galv-p
                    0
                    (+ (if (/= front_bolt_to "None") 1 0)
                       (if (wcmatch back_bolt_to "*Stair*") 1 0))))
        ("length" #(frac plt_wid_eff)))))

;; return a 2-item list representing whether there is a header tube at
;; the front and the back
(defun land-fab-platform-200-tube-p ()
  (if (begins-with tread_style "2")
      (list (and (not galv-p) (/= front_bolt_to "None"))
            (and (not galv-p) (wcmatch back_bolt_to "*Stair*")))
    '(nil nil)))

;; draw header tubes in all views
(defun land-fab-platform-200-draw-tubes (/ draw-tube front-p back-p)
  (setq front-p (land-fab-platform-200-tube-p)
        back-p (last front-p)
        front-p (car front-p))
  ;; top view
  ((lambda(/ add mirror l1 l2)
     (defun add (y l1 l2)
       (vla-put-Layer (add-polyline top-view (list plt-offset y) l1 nil) "SOLID")
       (vla-put-Layer (add-polyline top-view (list plt-offset y) l2 nil) "HIDDEN"))
     (setq l1 (#! '((0 #chk_hold 0)
                    (0 0 0)
                    (#plt_wid_eff 0 0)
                    (#plt_wid_eff #chk_hold 0)))
           l2 (#! '((0 1.5 0) (#plt_wid_eff 1.5 0))))
     (if front-p
         (apply 'add (list 0 l1 l2)))
     (if back-p
         (progn
           (apply 'add (cons plt_dpt
                             (mapcar '(lambda (pts) (mirror-pts-h pts 0))
                                     (list l1 l2))))))))
  ;; front view
  (if front-p
      (vla-put-Layer (add-rectangle front-view plt_wid_eff 2.5 (list plt-offset 0)) "SOLID"))

  ;; side views
  ((lambda(/ add)
     (defun add (view x dir)
       (mapcar '(lambda(obj) (vla-put-Layer obj "OBJ"))
               (add-shape view (list (+ x (* dir 0.75)) 2.5) "HSS2.5x1.5x0.065"))
       (setq dir (* -1 dir))
       (add-mleader view
                    (list x 2)
                    (#! '((0 0)
                          (#(* dir 1) 0)
                          (#(* dir 1) 1)
                          (#(* dir 2) 1)))
                    (xd-value "tube" marks)))
     (if front-p
         (progn
           (add left-view 0 1)
           (if right-view (add right-view 0 -1))))
     (if back-p
         (progn
           (add left-view plt_dpt -1)
           (if right-view (add right-view (- plt_dpt) 1)))))))

;; draw quiet tread in all views
(defun land-fab-platform-200-draw-qt (/ spacing draw-top draw-back)
  (defun draw-top (len wid x y z / obj)
    (setq x (- x (/ wid 2.0)))
    (setq obj (add-rectangle top-view wid len (list x y)))
    (vla-put-Layer obj "HIDDEN")
    (vla-put-Color obj 251))

  (defun draw-back (len wid x y z)
    (setq x (- x (/ wid 2.0)))
    (vla-put-Layer (add-rectangle back-view wid (- chk_thk) (list x (+ flr_thick z)))
                   "Stair Quiet Tread"))

  (setq spacing (mapcar '(lambda(qt) (mapcar '(lambda(k) (xd-value k qt))
                                             '("lng" "wid" "cen" "off" "hgt")))
                        (map-append 'cdr (calc-qtread-spacing))))
  (map-apply 'draw-top spacing)
  (if back-view (map-apply 'draw-back spacing)))
