(set-file-docstring "Functions for calculating landing post fabs")

(defun-q land-post-p (corner)
  "Returns T if there is a post at this corner"
  (declare (vars (corner str (member corner '("fl" "fr" "bl" "br"))))
           with-data)
  (member (cornerkey "chan_num") '("20" "25" "30" "35")))
(defun-r 'land-post-p)

(defun-q land-post-calc (corner / chan supp xtra xtra-p bay)
  "Calculates the post material and hardware for this CORNER of the current landing

Assumed to be called by %land-sl--calc"
  (declare (vars (corner str (member corner '("fl" "fr" "bl" "br"))))
           with-data)
  (setq chan (cornerkey "chan_num")
        supp (cornerkey "supp_num")
        xtra (cornerkey "xtra_holes")
        xtra-p (and (> xtra 0)
                    (not (ends-with chan "5")))
        bay (xd-value corner '(("fl" "f")
                               ("fr" "r")
                               ("br" "b")
                               ("bl" "l"))))
  ;; post
  (add-material
   '(("detail" #(strcat "T"
                        (if (ends-with chan "5") "3" (if xtra-p "2" "1"))
                        "/B"
                        (cond ((ends-with supp "6") "3")
                              ((ends-with supp "1") "2")
                              (T "1"))))
     ("qty" 1)
     ("desc" #(if (begins-with chan "2")
               "HSS 2 1/2 x 2 1/2 x 1/4"
               "HSS 3 x 3 x 1/4"))
     ("length" #(frac (- (if (ends-with chan "5")
                             (- (cornerkey "len")
                                flr_thick
                                (if (begins-with corner "f") front_chan_dpt back_chan_dpt)
                                0.25)
                             (- (cornerkey "len") flr_thick))
                       0.25)))
     ("note" "SL")
     ("extra_holes_offset" #(if xtra-p xtra ""))))
  ;; chan
  (if (ends-with chan "0")
      (mapcar 'add-hardware
        ;; hex bolt
        '((("qty" #(if xtra-p 4 2))
           ("desc" #(strcat "HEX BOLT (A307) 5/8 DIA. x "
                     (if (begins-with chan "2") "4" "4 1/2")))
           ("note" "SL"))
          (("qty" #(if xtra-p 4 2))
           ("desc" "HEX NUT 5/8 DIA.")
           ("note" "SL"))
          (("qty" #(if xtra-p 4 2))
           ("desc" "LOCK WASHER 5/8 DIA.")
           ("note" "SL"))
          (("qty" #(if xtra-p 8 4))
           ("desc" "FLAT WASHER 5/8 DIA.")
           ("note" "SL"))))
      (progn
        ;; superman plate
        (add-material
         '(("detail" "P2")
           ("qty" 1)
           ("desc" "PLATE 1/4")
           ("width" "6")
           ("length" "6")))
        (mapcar 'add-hardware
          '((("qty" 2)
             ("desc" "HEX BOLT (A307) 5/8 DIA. x 2")
             ("note" "SL"))
            (("qty" 2)
             ("desc" "HEX NUT 5/8 DIA.")
             ("note" "SL"))
            (("qty" 2)
             ("desc" "FLAT WASHER 5/8 DIA.")
             ("note" "SL"))
            (("qty" 2)
             ("desc" "BEVEL WASHER 5/8 DIA.")
             ("note" "SL"))))))
  ;; supp
  (if (or (ends-with supp "1")
          (ends-with supp "6"))
      (progn
        ;; expansion anchors
        (add-hardware
         '(("qty" 2)
           ("desc" "HILTI KB-TZ2 WEDGE ANCHOR 5/8 DIA. x 4 3/4")
           ("note" "SL")))
        (if (ends-with supp "6")
            (add-material
             '(("detail" "P2")
               ("qty" 1)
               ("desc" "PLATE 1/4")
               ("width" "6")
               ("length" "6")))
            ;; long base plate
            (add-material
             '(("detail" "P1")
               ("qty" 1)
               ("desc" #(strcat "FLAT BAR 1/4 x " (if (begins-with chan "2") "3" "3 1/2")))
               ("length" "9")))))
      ;; square base plate
      (add-material
       '(("detail" "B1a")
         ("qty" 1)
         ("desc" #(strcat "FLAT BAR 1/4 x " (if (begins-with chan "2") "3" "3 1/2")))
         ("length" #(if (begins-with chan "2") "3" "3 1/2")))))
  ;; x bracing
  (if (= (symcat '(#(substr bay 1 1) "_brace")) "1")
      (mapcar 'add-material (land-post--xbrace-material bay))))
(defun-r 'land-post-calc)

(defun-q land-post--xbrace-material (bay / hgt wid npanels len ang ret)
  "Return the material for X-bracing at BAY"
  (declare (vars (bay str (member bay '("f" "b" "l" "r"))))
           with-data)
  (foreach key '("hgt" "wid" "npanels")
           (set (read key) (symcat '(#bay "_brace_" #key))))
  (setq wid (- wid 0.25)
        hgt (- hgt 12.0)
        ang (rtos (rtd (atan (/ hgt wid))) 2 0)
        len (frac (sqrt (+ (square wid) (square hgt)))))
  (add-to-list 'ret (#! '(("detail" "X1")
                          ("qty" #(* npanels 2))
                          ("desc" "HSS 2 1/2 x 1 1/2 x 1/4")
                          ("length" #len)
                          ("miter_angle" #ang)
                          ("note" "SL"))))
  (if (> npanels 1)
      (add-to-list 'ret (#! '(("qty" #(1- npanels))
                              ("desc" "HSS 3 x 3 x 1/4")
                              ("length" #(frac wid))
                              ("note" "SL")))))
  ret)
(defun-r 'land-post--xbrace-material)
