(set-file-docstring "Functions for ship loose landing fabs

New ship loose piece types should be added to LAND-SL--FAB-TYPES")

(setq land-sl--fab-types '("hanger" "ledger" "post"))
;; auto includes for all fab types
(psc-include (mapcar '(lambda(ft) (strcat "fab/land-" ft ".lsp")) land-sl--fab-types))

(defun-q land-sl-make-fabs (lands fab-type / finish-map)
  "Makes ship loose landing fabs of type FAB-TYPE for LANDS"
  (declare (vars (lands list (vl-every 'data-list-p lands))
                 (fab-type str (member fab-type land-sl--fab-types))))
  (foreach land (mapcar '(lambda(ld) (land-sl--calc fab-type ld)) lands)
           (add-to-alist 'finish-map (car land) (cdr land) T))
  (mapcar '(lambda(lands / finish)
            (setq finish (pop! 'lands))
            (while lands
             ((lambda(/ material hardware land)
                (while (and lands (< (length material) 85))
                  (setq land (pop! 'lands)
                        material (append material (car land))
                        hardware (append hardware (cadr land))))
                (land-sl--make-fab fab-type finish material hardware)))))
          finish-map))
(defun-r 'land-sl-make-fabs)

(defun-q land-sl--make-fab (fab-type finish material hardware / dwgno path login modelspace)
  "Makes a ship loose landing fab of FAB-TYPE"
  (declare (vars (fab-type str (member fab-type land-sl--fab-types))
                 (finish str)
                 (material list (vl-every 'data-list-p material))
                 (hardware list (vl-every 'data-list-p hardware))))
  (setq dwgno (land-sl--calc-dwg-no jobno stairno))
  (setq path (vla-get-Path acadDoc)
        login (getvar 'loginname)
        acadDoc (vla-Add (vla-get-Documents acadObj)
                         (strcat psc-src-dir "fab/land-" fab-type ".dwt"))
        modelspace (vla-get-ModelSpace acadDoc))
  (vla-SaveAs acadDoc (strcat path "/" dwgno ".dwg"))
  (write-title-block job-info stairno login (strcat "SHIP LOOSE "(uppercase fab-type) "S"))
  (set-finish finish)
  (table-add-data (get-table-by-title "Misc Material")
                  ((lambda(/ mark i)
                     (setq mark (next-mark-prefix)
                           i 0)
                     (mapcar '(lambda(m) (cons (list "mark" (strcat mark (to-string (inc! 'i)))) m))
                             material))))
  (table-add-data (get-table-by-title "Misc Hardware")
                  (combine-like-data hardware '("desc" "note") nil))
  (vla-Regen acadDoc acAllViewports)
  (vla-SaveAs acadDoc (strcat path "/" dwgno ".dwg"))
  (if (not *dev-mode*) (vla-Close acadDoc))
  (reset-doc))
(defun-r 'land-sl--make-fab)

(defun-q land-sl--calc-dwg-no (jobno stairno / files-with-matching-sigil last-char)
  "Returns the next available ship loose drawing number"
  (declare (vars (jobno str)
                 (stairno str)))
  (setq files-with-matching-sigil (vl-remove-if-not
                                   '(lambda (x)
                                      (wcmatch x (strcat jobno
                                                         "-L"
                                                         stairno
                                                         "`.0?`.dwg")))
                                   (vl-directory-files (vla-get-path acadDoc))))
  (if files-with-matching-sigil
      (setq last-char (substr (last files-with-matching-sigil)
                              (+ (strlen jobno) (strlen stairno) 5)
                              1)))
  (strcat jobno "-L" stairno ".0" (chr (1+ (car (vl-string->list (?? last-char (chr 96))))))))
(defun-r 'land-sl--calc-dwg-no)

(defun-q land-sl--calc (fab-type land-data / material hardware add-material add-hardware)
  "Calculates ship loose pieces for a landing

Returns a list of lists whose car is the finish and cadr is a 2-element list. The elements
are datalists of material and hardware, respectively."
  (declare (vars (fab-type str (member fab-type land-sl--fab-types))
                 (land-data list (data-list-p land-data))))
  (defun add-material (mat) (add-to-list 'material (append base (#! mat))))
  (defun add-hardware (hdw)
    (setq hdw (#! hdw))
    (add-to-list 'hardware
      (if (= finish "Galvanized")
          (subst-key "desc" (hardware-galv-desc (xd-value "desc" hdw)) hdw)
          hdw)))
  (with-data land-data
    '((setq base (#! '(("piece_type" "P")
                       ("stair_number" #number)
                       ("level" #level)
                       ("sequence" #sequence))))
      (foreach corner '("fl" "fr" "bl" "br")
       (if (apply (read (strcat "land-" fab-type "-p")) (list corner))
           (apply (read (strcat "land-" fab-type "-calc")) (list corner))))
      (if material
          (list
           finish
           (list
            (combine-like-data material '("detail" "desc" "width" "length" "note") nil)
            (combine-like-data hardware '("desc" "note") nil)))))))
(defun-r 'land-sl--calc)
