(set-file-docstring "Functions for landing fab drawings")

(psc-include '("fab/land-platform.lsp"
               "fab/land-channels.lsp"))

(defun-q make-land-fabs (lands)
  "Create all fab drawings for LANDS

Essentially a composition of %make-generic-fab and %fab-combine-lands"
  (mapcar '(lambda(x) (make-generic-fab x 'land))
          (fab-combine-lands lands)))
(defun-r 'make-land-fabs)

(defun-q fab-combine-lands (lands)
  "Uses %fab-combine-like-data to combine LANDS"
  (fab-combine-like-data
   lands
   (mapcar (compose '(lowercase to-string))
           '(back_bolt_to
             back_chan
             back_chan_loc
             chk_hold
             finish
             flip_back
             front_bolt_to
             front_chan
             plt_dpt
             plt_wid
             quiet_tread
             tread_style
             bl_chan_equiv
             bl_ext
             br_chan_equiv
             br_ext
             fl_chan_equiv
             fl_ext
             fr_chan_equiv
             fr_ext))))
(defun-r 'fab-combine-lands)

;; this function assumes it is called by make-generic-fab and thus has access
;; to all local vars in make-generic-fab
(defun make-land-fab ( /
                      process-material
                      current-mark-no
		      marks
                      top-front-insx
                      top-view
                      left-view
                      right-view
                      front-view
                      back-view
                      front-chan-detail
                      front-chan-detail-ht
                      back-chan-detail
                      back-chan-detail-ht
                      left-chan-detail
                      right-chan-detail
		      flip-back
                      plt-offset
                      plt-end
                      galv-p
                      part-details
                      )
  ;; add mark to a material list item
  ;; add ids to marks list
  ;; add details to part-details
  (defun process-material (data / mark)
    (setq mark (strcat piece-mark (to-string (post-inc! 'current-mark-no))))
    (add-to-list 'data (list "mark" mark))
    (foreach id (xd-value "id" data)
	     (add-to-list 'marks (list id mark))
             (if (assoc "detail" data)
                 (add-to-list 'part-details
                              (#! '(("mark" #mark)
                                    #(assoc "detail" data))))))
    data)
  
  ;; update assemblies table title
  (vla-SetCellState ass-table 0 0 acCellStateNone)
  (vla-SetText ass-table 0 0 (strcat "Assembly: "
                                     (cond ((begins-with tread_style "1")
                                            "CF ")
                                           ((begins-with tread_style "2")
                                            "CP "))
                                     "Platform "
                                     (to-string (fix plt_wid))
                                     " x "
                                     (to-string (fix plt_dpt))))

  (setq current-mark-no 1
        top-front-insx (+ (* dimspace 4) (max fl_ext bl_ext))
	flip-back (= flip_back "1")
        top-view (define-block "top-view" nil)
	front-view (define-block "front-view" nil)
        front-chan-detail (define-block "front-chan" nil)
        back-view (if (or (vl-some '/=
                                   (list front_chan front_chan_len fl_chan_equiv fr_chan_equiv)
                                   (list back_chan back_chan_len bl_chan_equiv br_chan_equiv))
                          (and flip-back (/= back_bolt_to "None")))
                      (define-block "back-view" nil))
        back-chan-detail (define-block "back-chan" nil)
        left-view (define-block "left-view" nil)
        right-view (if (vl-some '/=
                                (list left_chan fl_chan_equiv bl_chan_equiv)
                                (list right_chan fr_chan_equiv br_chan_equiv))
                       (define-block "right-view" nil))
        left-chan-detail (if (not (and (= (land-fab-lr-chan-detail-name "front" "left") "none")
                                       (= (land-fab-lr-chan-detail-name "back" "left") "none")))
                             (define-block "left-chan" nil))
        right-chan-detail (if (not (and (= (land-fab-lr-chan-detail-name "front" "right") "none")
                                        (= (land-fab-lr-chan-detail-name "back" "right") "none")))
                              (define-block "right-chan" nil))
        plt-offset (if (= fl_chan_num "85") (- fl_ext) 0)
        plt-end (+ plt-offset plt_wid_eff)
        galv-p (= finish "Galvanized"))
  (land-fab-channels)
  (land-fab-platform)
  (land-fab-add-top-view-dims)
  (land-fab-add-fb-view-dims "front")
  (land-fab-add-lr-view-dims "left")
  (if right-view
      (land-fab-add-lr-view-dims "right"))
  (if back-view
      (land-fab-add-fb-view-dims "back"))
  (land-fab-insert-views))

;; arrange and insert all views in the drawing
(defun land-fab-insert-views (/ halfwid)
  (setq halfwid (/ dwg-width 2.0))
  ;; top view
  (insert-block modelSpace
                (list top-front-insx (- dwg-height (* dimspace 4) plt_dpt) 0)
                top-view)
  ;; front view
  (insert-block modelSpace
                (list top-front-insx (- dwg-height (* dimspace 10) plt_dpt flr_thick) 0)
                front-view)
  ;; back view
  (if back-view
      (insert-block modelSpace
                    (list top-front-insx
                          (- dwg-height (* dimspace 16) plt_dpt (* 2 flr_thick) front_chan_dpt)
                          0)
                    back-view))
  ;; left view
  (insert-block modelSpace
		(list halfwid (- dwg-height (* 5 dimspace)) 0)
		left-view)
  ;; right view
  (if right-view
      (insert-block modelSpace
		    (list (+ halfwid plt_dpt) (- dwg-height (* 17 dimspace)) 0)
		    right-view))
  ;; front/back channel details
  ((lambda(ins / back-chan-def)
     (insert-block modelSpace ins front-chan-detail)
     (setq ins (displace-pt ins (list 0 (+ front-chan-detail-ht (* 3 dimspace)) 0))
           back-chan-def (land-fab-calc-fb-chan "back"))
     (if (and (/= (xd-value "front-chan" marks) (xd-value "back-chan" marks))
              (or (/= (xd-value "ldetail" back-chan-def) "none")
                  (/= (xd-value "rdetail" back-chan-def) "none")
                  (land-fab-calc-chan-bolt-slots "back")))
         (insert-block modelSpace ins back-chan-detail)))
   (list halfwid 36))
  ;; left channel detail
  (if left-chan-detail
      (insert-block modelSpace (list top-front-insx (* 2 dimspace) 0) left-chan-detail))
  ;; right channel detail
  (if (and right-chan-detail
           (/= (xd-value "left-chan" marks) (xd-value "right-chan" marks)))
      (insert-block modelSpace (list (if left-chan-detail
                                         (+ top-front-insx (* 6 dimspace) left_chan_len)
                                       top-front-insx)
                                     (* 2 dimspace) 0)
                    right-chan-detail))
  ;; part details
  (if part-details
      ((lambda(ins)
         (foreach d (combine-like-data part-details '("detail") "mark")
                  (insert-block modelspace ins (part-detail (xd-value "mark" d) (xd-value "detail" d)))
                  (setq ins (displace-pt ins (list 0 (* 6 dimspace))))))
       (list top-front-insx (* 12 dimspace)))))

(defun land-fab-add-top-view-dims ()
  ;; front/back section arrows
  ((lambda(/ arrow-toward-back)
     (defun arrow-toward-back (ins mark)
       (add-polyline top-view ins '((-1 0 0) (12 0 0)) nil)
       (vla-SetWidth (add-polyline top-view ins '((0 3 0) (0 2 0) (0 -1 0)) nil) 0 0 1)
       (add-mtext top-view (displace-pt ins '(1 -1 0)) mark 1 'topleft))
     (arrow-toward-back (list (- dimspace) (- (* 2 dimspace))) "F")
     (if back-view
         (arrow-toward-back (list (- dimspace) (- back_chan_loc (* 2 dimspace))) "B"))))
  ;; left section arrow
  ((lambda(ins)
     (add-polyline top-view ins '((0 -1 0) (0 12 0)) nil)
     (vla-SetWidth (add-polyline top-view ins '((-3 0 0) (-2 0 0) (1 0 0)) nil) 0 0 1)
     (add-mtext top-view (displace-pt ins '(1 1 0)) "L" 1 'bottomleft))
   (list (* 2 dimspace) (- dimspace) 0))
  ;; right section arrow
  (if right-view
      ((lambda(ins)
         (add-polyline top-view ins '((0 -1 0) (0 12 0)) nil)
         (vla-SetWidth (add-polyline top-view ins '((3 0 0) (2 0 0) (-1 0 0)) nil) 0 0 1)
         (add-mtext top-view (displace-pt ins '(-1 1 0)) "R" 1 'bottomright))
       (list (- plt_wid (* 2 dimspace)) (- dimspace) 0)))
  ((lambda(/ fx bx dx)
     (setq fx (+ plt-end (if (> fr_ext 0) fr_ext 0))
           bx (+ plt-end
                 (if (> br_ext 0) br_ext 0)
                 (if (and right_chan
                          (not (= right_cope "back"))
                          (not (=f plt_dpt back_chan_loc 0.01)))
                     (- right_chan_wid)
                     0))
           dx (+ (max fx bx) (* 3 dimspace)))
     ;; back channel dim
     (vla-AddDimRotated top-view (3dpt fx 0) (3dpt bx back_chan_loc) (3dpt dx 0) (dtr 90))
     ;; overall depth dim
     (if (not (=f plt_dpt back_chan_loc 0.01))
         (vla-AddDimRotated top-view
                            (3dpt fx 0)
                            (3dpt (if (=f plt_dpt (+ back_chan_loc back_chan_wid) 0.01) bx plt-end)
                                  plt_dpt)
                            (3dpt (+ dx dimspace) 0)
                            (dtr 90)))))
  ((lambda(/ tube-p tube-mark)
     (defun tube-mark (fb) (if fb (strcat "\n" (to-string (xd-value "tube" marks))) ""))
     (setq tube-p (land-fab-platform-200-tube-p))
     ;; front channel/tube callout
     (add-mleader top-view
                  (list (- (/ plt_wid 2.0) dimspace) 0)
                  '((0 0 0)
                    (0 -4 0)
                    (-1 -4 0))
                  (strcat (xd-value "front-chan" marks) (tube-mark (car tube-p))))
     ;; back channel/tube callout
     (add-mleader top-view
                  (list (- (/ plt_wid 2.0) dimspace) plt_dpt)
                  (#! '((0 #(- (- plt_dpt back_chan_loc)) 0)
                        (0 5 0)
                        (-1 5 0)))
                  (strcat (xd-value "back-chan" marks) (tube-mark (last tube-p))))))
  ;; left channel callout
  (if left_chan
      (add-mleader top-view
                   (list (- (min fl_ext bl_ext)) (/ left_chan_len 2.0))
                   '((0 0 0)
                     (-1 0 0)
                     (-1 -4 0)
                     (-2 -4 0))
                   (xd-value "left-chan" marks)))
  ;; right channel callout
  (if right_chan
      (add-mleader top-view
                   (list (+ plt_wid (min fr_ext br_ext)) (/ right_chan_len 2.0))
                   '((0 0 0)
                     (1 0 0)
                     (1 -4 0)
                     (2 -4 0))
                   (xd-value "right-chan" marks))))

(defun land-fab-add-fb-view-dims (fb / view fb-short lext rext chan-x slots top-dim-line)
  (setq view (symcat '(#fb "-view"))
        fb-short (substr fb 1 1)
        lext (symcat '(#fb-short "l_ext"))
        rext (symcat '(#fb-short "r_ext"))
        chan-x (apply (read (strcat "land-fab-" fb "-chan-x")) nil)
        slots (land-fab-calc-chan-bolt-slots fb)
        top-dim-line (+ flr_thick (* 2 dimspace)))
  (if (and (= fb "back") flip-back)
      ((lambda(len)
         (setq slots (mapcar '(lambda(s) (cons (- len (car s)) (cdr s)))
                             (reverse slots))))
       (distof (xd-value "length" (land-fab-calc-fb-chan fb)))))
  (mapcar! '(lambda(s)
              (displace-pt s (list chan-x 1)))
           'slots)
  ;; title
  (add-mtext view
             (list (/ plt_wid 2.0) (- 0 (fbkey "chan_dpt") dimspace))
             (if (= fb "front") "{\\LVIEW F}" "{\\LSECTION B}")
             16
             'topcenter)
  ;; overall
  (vla-AddDimRotated view
                     (3dpt plt-offset flr_thick)
                     (3dpt plt-end flr_thick)
                     (3dpt 0 top-dim-line)
                     0)
  ;; extensions
  (if (> lext 0)
      (vla-AddDimRotated view
                         (3dpt plt-offset flr_thick)
                         (3dpt (- lext) 0)
                         (3dpt 0 top-dim-line)
                         0))
  (if (> rext 0)
      (vla-AddDimRotated view
                         (3dpt plt-end flr_thick)
                         (3dpt (+ plt_wid rext) 0)
                         (3dpt 0 top-dim-line)
                         0))
  ((lambda(/ dim-line)
     (setq dim-line (- top-dim-line dimspace))
     ;; hanger sleeves
     (if (member (substr (symcat '(#fb-short "l_chan")) 1 1) '("4" "5"))
         (vla-AddDimRotated view
                            (3dpt (+ chan-x 1.25) 0)
                            (3dpt chan-x 0)
                            (3dpt 0 dim-line)
                            0))
     (if (member (substr (symcat '(#fb-short "r_chan")) 1 1) '("4" "5"))
         (vla-AddDimRotated view
                            (3dpt (+ plt_wid rext -1.25) 0)
                            (3dpt (+ plt_wid rext) 0)
                            (3dpt 0 dim-line)
                            0))
     (if slots
         (progn
           ;; left slot vertical
           (vla-AddDimrotated view
                              (3dpt 0 flr_thick)
                              (3dpt (displace-pt (car slots) '(-1 -1)))
                              (3dpt (- (min 0 chan-x) (* 2 dimspace)) 0)
                              (dtr 90))
           ;; horizontals
           ;; left slot
           (vla-AddDimRotated view
                              (3dpt (car slots))
                              (3dpt plt-offset flr_thick)
                              (3dpt 0 dim-line)
                              0)
           ;; right slot
           (vla-AddDimRotated view
                              (3dpt (last slots))
                              (3dpt plt-end flr_thick)
                              (3dpt 0 dim-line)
                              0)
           ;; mid slots
           (while (> (length slots) 1)
             (vla-AddDimRotated view
                                (3dpt (car slots))
                                (3dpt (cadr slots))
                                (3dpt 0 dim-line)
                                0)
             (pop! 'slots))))))
  ;; main channel callout
  (add-mleader view
               (list (- (/ plt_wid 3.0) dimspace) (- (fbkey "chan_dpt")))
               '((0 0 0)
                 (0 -4 0)
                 (-1 -4 0))
               (xd-value (strcat fb "-chan") marks))
  ;; left channel callout
  (if left_chan
      (add-mleader view
                   (list (1+ (/ left_chan_wid 2.0)) (- left_chan_dpt))
                   '((0 0 0)
                     (0 -4 0)
                     (-1 -4 0))
                   (xd-value "left-chan" marks)))
  ;; right channel callout
  (if right_chan
      (add-mleader view
                   (list (- plt_wid (1+ (/ right_chan_wid 2.0))) (- right_chan_dpt))
                   '((0 0 0)
                     (0 -4 0)
                     (1 -4 0))
                   (xd-value "right-chan" marks))))

(defun land-fab-add-lr-view-dims (lr / x view)
  ;; choose x direction based on view side
  (defun lrx (x) (if (= lr "left") x (- x)))
  (setq view (symcat '(#lr "-view")))
  ;; title
  (add-mtext view
             (list (lrx (/ plt_dpt 2.0))
                   (- (+ (* 3 dimspace) (max front_chan_dpt back_chan_dpt (?f left_chan_dpt)))))
             (strcat"{\\LSECTION " (uppercase (substr lr 1 1)) "}")
             16
             'topcenter)
  ;; front channel callout
  (add-mleader view
               (list (lrx (/ front_chan_wid 2.0)) (- front_chan_dpt))
               (#! '((0 0 0)
                     (0 -4 0)
                     (#(lrx 1) -4 0)))
               (xd-value "front-chan" marks))
  ;; back channel callout
  (add-mleader view
               (list (lrx ((if flip-back - +) back_chan_loc (/ back_chan_wid 2.0)))
                     (- back_chan_dpt))
               (#! '((0 0 0)
                     (0 -4 0)
                     (#(lrx (if flip-back -1 1)) -4 0)))
               (xd-value "back-chan" marks))
  ;; side channel callout
  (if (symcat '(#lr "_chan"))
      (add-mleader view
                   (list (lrx (* 3 dimspace)) (- (symcat '(#lr "_chan_dpt"))))
                   (#! '((0 0 0)
                         (0 -4 0)
                         (#(lrx 1) -4 0)))
                   (xd-value (strcat lr "-chan") marks)))
  ;; back channel dim
  (vla-AddDimRotated view
                     (3dpt 0 (- front_chan_dpt))
                     (3dpt (lrx back_chan_loc) (- back_chan_dpt))
                     (3dpt 0 (- 0 (max front_chan_dpt back_chan_dpt) (* 2 dimspace)))
                     0))
