;; Functions to draw part details in fab drawings

(psc-include '("shapes/shape.lsp"))

;; draw a detail using its specific function and add the frame with
;; the piece mark in it
(defun part-detail (mark
                    detail-def
                    /
                    dimscale
                    unit
                    detail-block
                    obj
                    clayer
                    cdimstyle
                    dimspace
                    )
  (debug-print '("Adding part detail"))
  (debug-print-vars '(mark detail-def))
  (setq dimscale (vla-getvar 'dimscale)
        unit (* -0.25 dimscale)
        dimspace (* (vla-getvar 'dimdli) dimscale)
        clayer (vla-get-ActiveLayer acadDoc)
        cdimstyle (vla-get-ActiveDimStyle acadDoc)
        detail-block (apply (car detail-def) (cdr detail-def)))
  ;; reset active dim style and layer in case detail function changed it
  (vla-put-ActiveDimStyle acadDoc cdimstyle)
  (vla-put-ActiveLayer acadDoc clayer)
  ;; mark
  (add-part-detail-callout detail-block mark '(0 0 0))
  detail-block)

(defun add-part-detail-callout (container mark ins / dimscale unit)
  (setq dimscale (vla-getvar 'dimscale)
        unit (* -0.25 dimscale)
        mark (uniquify (if (stringp mark) (list mark) mark)))
  (vla-put-Layer
   (add-polyline container
                 ins
                 (list '(0 0 0)
                       (list unit 0 1)
                       (list (* 2 unit) 0 1)
                       (list unit 0 0))
                 nil)
   "DIM")
  ((lambda(ins / add-mark)
     (defun add-mark () (add-mtext container ins (car (take! 1 'mark)) 1 'middlecenter))
     (add-mark)
     (repeat (length mark)
             (setq ins (displace-pt ins (list 0 unit)))
             (vla-AddCircle container (3dpt ins) (/ unit -2.0))
             (add-mark)))
   (list (+ (* -0.375 dimscale) (car ins)) (cadr ins))))

;; 1/4" flat bar with (1) hole
;; 2x scale
(defun 1h-flatbar (len width hole-offset hole-depth hole-dia /
                       blockObj
                       obj
                       galv-p
                       hole-ctr
                       hole-rad
                       )
  (foreach var '(len width hole-offset hole-depth hole-dia)
           (set var (* 2 (vl-symbol-value var))))
  (setq blockObj (p-blk nil)
        galv-p (tk '(= finish "Galvanized"))
        hole-rad (/ hole-dia 2)
        hole-depth (+ dimspace (- width hole-depth)))
  
  (set-active-layer "SOLID")
  (add-rectangle blockObj len width (list 0 dimspace 0))
  (vla-AddCircle blockObj (3dpt hole-offset hole-depth 0) hole-rad)
  (add-rectangle blockObj 0.5 width (list (- dimspace) dimspace 0))
  
  (set-active-layer "CENTER")
  ;; hole centerlines
  (add-polyline blockObj
                (list hole-offset (1- hole-depth) 0)
                '((0 0 0)
                  (0 2 0))
                nil)
  (add-polyline blockObj
                (list (1- hole-offset) hole-depth 0)
                '((0 0 0)
                  (2 0 0))
                nil)
  ;; side view centerline
  (add-polyline blockObj
                (list (- 0 dimspace 0.75)
                      hole-depth
                      0)
                '((0 0 0)
                  (2 0 0))
                nil)
  
  (set-active-layer "DIM")
  (set-active-dimstyle "2xFractional")
  ;; overall dim
  (vla-AddDimRotated blockObj
                     (3dpt 0 dimspace 0)
                     (3dpt len dimspace 0)
                     (3dpt 0 0 0)
                     0)
  ;; depth dims
  (vla-AddDimRotated blockObj
                     (3dpt (1+ hole-offset) hole-depth 0)
                     (3dpt len (+ dimspace width) 0)
                     (3dpt (+ len dimspace) 0 0)
                     (dtr 90))
  (vla-AddDimRotated blockObj
                     (3dpt len (+ dimspace width) 0)
                     (3dpt len dimspace 0)
                     (3dpt (+ len (* 2 dimspace)) 0 0)
                     (dtr 90))
  ;; horizontal hole dims
  (vla-AddDimRotated blockObj
                     (3dpt hole-offset (1+ hole-depth) 0)
                     (3dpt 0 (+ dimspace width) 0)
                     (3dpt 0 (+ dimspace width (* 2 dimspace)) 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt hole-offset (1+ hole-depth) 0)
                     (3dpt len (+ dimspace width) 0)
                     (3dpt 0 (+ dimspace width (* 2 dimspace)) 0)
                     0)

  ;; nonstandard hole dim
  (if (/= hole-dia (* 2 std-hole-dia))
      (progn
        (setq obj (vla-AddDimDiametric blockObj
                                       (3dpt (- hole-offset hole-rad) hole-depth)
                                       (3dpt (+ hole-offset hole-rad) hole-depth)
                                       1))
        (vla-Rotate obj (3dpt hole-offset hole-depth) (dtr -20))))
    
  blockObj)

;; generic template for 2h-flatbar and 2h-angle
;; 1x or 2x scale
(defun 2h-rect (scale
                len
                width
                hole1-offset
                hole2-offset
                hole-depth
                hole-dia)
  (setq blockObj (p-blk nil)
        btm-line (* 2 dimspace)
        top-line (+ btm-line width)
        hole-rad (/ hole-dia 2.0)
        hole-depth (+ btm-line (- width hole-depth))
        hole2-offset (- len hole2-offset))

  (set-active-layer "SOLID")
  (add-rectangle blockObj len width (list 0 btm-line 0))
  (vla-AddCircle blockObj
                 (3dpt hole1-offset hole-depth 0)
                 hole-rad)
  (vla-AddCircle blockObj
                 (3dpt hole2-offset
                       hole-depth
                       0)
                 hole-rad)

  (set-active-layer "CENTER")
  ;; hole centerlines
  (add-bolt-centerline (list hole1-offset hole-depth 0) 'both)
  (add-bolt-centerline (list hole2-offset hole-depth 0) 'both)

  (set-active-layer "DIM")
  ;; bottom dims
  (vla-AddDimRotated blockObj
                     (3dpt 0 btm-line 0)
                     (3dpt len btm-line 0)
                     (3dpt 0 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt hole1-offset (1- hole-depth) 0)
                     (3dpt len btm-line 0)
                     (3dpt 0 dimspace 0)
                     0)
  ;; top dims
  (vla-AddDimRotated blockObj
                     (3dpt hole1-offset (+ hole-depth 1) 0)
                     (3dpt 0 top-line 0)
                     (3dpt 0 (+ top-line (* 2 dimspace)) 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt hole1-offset (+ hole-depth 1) 0)
                     (3dpt hole2-offset (+ hole-depth 1) 0)
                     (3dpt 0 (+ top-line (* 2 dimspace)) 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt hole2-offset (+ hole-depth 1) 0)
                     (3dpt len top-line 0)
                     (3dpt 0 (+ top-line (* 2 dimspace)) 0)
                     0)
  ;; side dims
  (vla-AddDimRotated blockObj
                     (3dpt (+ hole2-offset 1) hole-depth 0)
                     (3dpt len top-line 0)
                     (3dpt (+ len (* 3 dimspace)) 0 0)
                     (dtr 90))
  (vla-AddDimRotated blockObj
                     (3dpt len btm-line 0)
                     (3dpt len top-line 0)
                     (3dpt (+ len (* 4 dimspace)) 0 0)
                     (dtr 90))

  ;; nonstandard hole dims
  (if (/= hole-dia (* scale std-hole-dia))
      (progn
        (vla-setvar 'cmleaderstyle "Standard")
        (add-mleader blockObj
                     (list (+ hole1-offset hole-rad) hole-depth 0)
                     (list '(0 0 0)
                           (list dimspace dimspace 0))
                     (strcat (rtos hole-dia 5 5) "%%C TYP."))
        (vla-setvar 'cmleaderstyle "Callout"))))

;; 1/4" flat bar with (2) holes
(defun 2h-flatbar (scale
                   len
                   width
                   hole1-offset
                   hole2-offset
                   hole-depth
                   hole-dia
                   /
                   blockObj
                   btm-line
                   top-line
                   hole-rad
                   )
  (foreach var
   '(len width hole1-offset hole2-offset hole-depth hole-dia)
    (set var (* scale (vl-symbol-value var)))
  ) ;_ foreach
  (set-active-dimstyle (strcat (to-string scale) "xFractional"))
  (2h-rect scale len width hole1-offset hole2-offset hole-depth hole-dia)

  ;; side view
  (set-active-layer "SOLID")
  (add-rectangle blockObj 0.25 width (list (- dimspace) btm-line 0))

  ;; side view centerline
  (set-active-layer "CENTER")
  (add-bolt-centerline (list (- 0 dimspace -0.125) (- top-line hole-depth) 0) 'h)

  blockObj)

;; arbitrary sized angle with (2) holes
(defun 2h-angle (size
                 leg
                 len
                 hole1-offset
                 hole2-offset
                 hole-depth
                 hole-dia
                 scale
                 /
                 def
                 width
                 thick
                 obj
                 blockObj
                 btm-line
                 top-line
                 hole-rad
                 )
  (setq def (shape-def size T)
        width (cadr (assoc (if (= leg 'short) "leg2" "leg1") def))
        thick (cadr (assoc "thick" def)))

  (foreach var '(len hole1-offset hole2-offset hole-depth hole-dia width thick)
           (set var (* scale (vl-symbol-value var))))

  (set-active-dimstyle (strcat (to-string scale) "xFractional"))

  (2h-rect scale len width hole1-offset hole2-offset hole-depth hole-dia)

  ;; hidden line
  (set-active-layer "HIDDEN")
  (move-to-bottom
   (add-polyline blockObj
                 (list 0 (- top-line thick) 0)
                 (list '(0 0 0)
                       (list len 0 0))
                 nil)
   blockObj)

  ;; side view
  (set-active-layer "SOLID")
  (setq obj (add-shape blockObj (list (- dimspace) top-line 0) size))
  (vla-Rotate obj (3dpt (- dimspace) top-line) (dtr -90))
  (vla-ScaleEntity obj (3dpt (- dimspace) top-line) scale)
  (if (= leg 'short)
      (progn
        (vla-Mirror obj
                    (3dpt (- dimspace) top-line)
                    (3dpt (1+ (- dimspace))
                          (1+ top-line)))
        (vla-Delete obj)))

  ;; side view centerline
  (set-active-layer "CENTER")
  (add-bolt-centerline (list (- 0 dimspace (/ thick 2.0)) (- top-line hole-depth) 0) 'h)

  blockObj)

;; generic template for 4h-flatbar and 4h-angle
(defun 4h-rect (len
                width
                hole1-offset
                hole2-offset
                hbolt2
                hole-depth
                hole-dia)
  (setq blockObj (p-blk nil)
        btm-line (* 4 dimspace)
        top-line (+ btm-line width)
        hole-rad (/ hole-dia 2.0)
        hole-depth (+ btm-line (- width hole-depth))
        hole2-offset (- len hole2-offset))

  (set-active-layer "SOLID")
  (add-rectangle blockObj len width (list 0 btm-line 0))
  (vla-AddCircle blockObj
                 (3dpt hole1-offset hole-depth)
                 hole-rad)
  (vla-AddCircle blockObj
                 (3dpt (+ hole1-offset hbolt2) hole-depth)
                 hole-rad)
  (vla-AddCircle blockObj
                 (3dpt hole2-offset hole-depth)
                 hole-rad)
  (vla-AddCircle blockObj
                 (3dpt (- hole2-offset hbolt2) hole-depth)
                 hole-rad)

  (set-active-layer "CENTER")
  ;; hole centerlines
  (add-bolt-centerline (list hole1-offset hole-depth 0) 'both)
  (add-bolt-centerline (list (+ hole1-offset hbolt2) hole-depth 0) 'both)
  (add-bolt-centerline (list hole2-offset hole-depth 0) 'both)
  (add-bolt-centerline (list (- hole2-offset hbolt2) hole-depth 0) 'both)

  (set-active-layer "DIM")
  ;; bottom dims
  (vla-AddDimRotated blockObj
                     (3dpt len btm-line 0)
                     (3dpt 0 btm-line 0)
                     (3dpt 0 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt len btm-line 0)
                     (3dpt hole1-offset (1- hole-depth) 0)
                     (3dpt 0 dimspace 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt len btm-line 0)
                     (3dpt (+ hole1-offset hbolt2) (1- hole-depth) 0)
                     (3dpt 0 (* 2 dimspace) 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt len btm-line 0)
                     (3dpt (- hole2-offset hbolt2) (1- hole-depth) 0)
                     (3dpt 0 (* 3 dimspace) 0)
                     0)
  ;; top dims
  (vla-AddDimRotated blockObj
                     (3dpt hole1-offset (+ hole-depth 1) 0)
                     (3dpt 0 top-line 0)
                     (3dpt 0 (+ top-line dimspace) 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt (+ hole1-offset hbolt2) (+ hole-depth 1) 0)
                     (3dpt hole1-offset top-line 0)
                     (3dpt 0 (+ top-line (* 2 dimspace)))
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt (+ hole1-offset hbolt2) (+ hole-depth 1) 0)
                     (3dpt (- hole2-offset hbolt2) (+ hole-depth 1) 0)
                     (3dpt 0 (+ top-line (* 2 dimspace)) 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt (- hole2-offset hbolt2) (+ hole-depth 1) 0)
                     (3dpt hole2-offset top-line 0)
                     (3dpt 0 (+ top-line (* 2 dimspace)))
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt hole2-offset (+ hole-depth 1) 0)
                     (3dpt len top-line 0)
                     (3dpt 0 (+ top-line dimspace) 0)
                     0)
  ;; side dims
  (vla-AddDimRotated blockObj
                     (3dpt (+ hole2-offset 1) hole-depth 0)
                     (3dpt len top-line 0)
                     (3dpt (+ len (* 3 dimspace)) 0 0)
                     (dtr 90))
  (vla-AddDimRotated blockObj
                     (3dpt len btm-line 0)
                     (3dpt len top-line 0)
                     (3dpt (+ len (* 4 dimspace)) 0 0)
                     (dtr 90))

  ;; nonstandard hole dims
  (if (/= hole-dia std-hole-dia)
      (progn
        (vla-setvar 'cmleaderstyle "Standard")
        (add-mleader blockObj
                     (list (+ hole1-offset hole-rad) hole-depth 0)
                     (list '(0 0 0)
                           (list dimspace dimspace 0))
                     (strcat (rtos hole-dia 5 5) "%%C TYP."))
        (vla-setvar 'cmleaderstyle "Callout"))))

;; 1/4" flat bar with 4 holes
(defun 4h-flatbar (len
                   width
                   hole1-offset
                   hole2-offset
                   hbolt2
                   hole-depth
                   hole-dia
                   /
                   blockObj
                   btm-line
                   top-line
                   hole-rad
                   )
  (4h-rect len width hole1-offset hole2-offset hbolt2 hole-depth hole-dia)

  ;; side view
  (set-active-layer "SOLID")
  (add-rectangle blockObj 0.25 width (list (- dimspace) btm-line 0))

  ;; side view centerline
  (set-active-layer "CENTER")
  (add-bolt-centerline (list (- 0 dimspace -0.125) (- top-line hole-depth) 0) 'h)

  blockObj)

;; arbitrary sized angle with 4 holes
(defun 4h-angle (size
                 leg
                 len
                 hole1-offset
                 hole2-offset
                 hbolt2
                 hole-depth
                 hole-dia
                 /
                 def
                 width
                 thick
                 obj
                 blockObj
                 btm-line
                 top-line
                 hole-rad
                 )
  (setq def (shape-def size T)
        width (cadr (assoc (if (= leg 'short) "leg2" "leg1") def))
        thick (cadr (assoc "thick" def)))

  (4h-rect len width hole1-offset hole2-offset hbolt2 hole-depth hole-dia)

  ;; hidden line
  (set-active-layer "HIDDEN")
  (move-to-bottom
   (add-polyline blockObj
                 (list 0 (- top-line thick) 0)
                 (list '(0 0 0)
                       (list len 0 0))
                 nil)
   blockObj)

  ;; side view
  (set-active-layer "SOLID")
  (setq obj (add-shape blockObj (list (- dimspace) top-line 0) size))
  (vla-Rotate obj (3dpt (- dimspace) top-line) (dtr -90))
  (if (= leg 'short)
      (progn
        (vla-Mirror obj
                    (3dpt (- dimspace) top-line)
                    (3dpt (1+ (- dimspace))
                          (1+ top-line)))
        (vla-Delete obj)))

  ;; side view centerline
  (set-active-layer "CENTER")
  (add-bolt-centerline (list (- 0 dimspace (/ thick 2.0)) (- top-line hole-depth) 0) 'h)

  blockObj)

;; stiffener plate for x21/x41 connections
(defun bot-triangle-gusset ( / blockObj bot-line top-corner)
  (setq blockObj (p-blk nil)
        bot-line (* 2 dimspace)
        top-corner (+ bot-line 4))
  (vla-put-Layer
   (add-polyline blockObj
                 (list 0 bot-line 0)
                 '((0 0 0)
                   (14.375 0 0)
                   (7.1875 4 0))
                 T)
   "SOLID")
  (set-active-dimstyle "2xFRACTIONAL")
  (vla-AddDimRotated blockObj
                     (3dpt 0 bot-line)
                     (3dpt 14.375 dimspace)
                     (3dpt 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt 7.1875 top-corner)
                     (3dpt 0 bot-line)
                     (3dpt 0 dimspace)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt 7.1875 top-corner)
                     (3dpt 14.375 bot-line)
                     (3dpt 7.1875 dimspace)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt 14.375 bot-line)
                     (3dpt 7.1875 top-corner)
                     (3dpt (+ 14.375 (* 2 dimspace)) bot-line)
                     (dtr 90))
  blockObj)

;; stiffener plate for x22/x42 connections
;; pitch is in degrees
(defun top-triangle-gusset (pitch / blockObj top-corner)
  (setq blockObj (p-blk nil)
        top-corner (* 12 (tan (dtr pitch))))
  (vla-put-Layer
   (add-polyline blockObj
                 (list 0 dimspace 0)
                 (list '(0 0 0)
                       '(12 0 0)
                       (list 0 top-corner 0))
                 T)
   "SOLID")
  (set-active-dimstyle "2xFRACTIONAL")
  (vla-AddDimRotated blockObj
                     (3dpt 0 dimspace)
                     (3dpt 12 dimspace)
                     (3dpt 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt 12 dimspace)
                     (3dpt 0 (+ dimspace top-corner))
                     (3dpt (+ dimspace 12) dimspace)
                     (dtr 90))
  blockObj)
    
(defun can (/ blockObj pl top-line)
  (set-active-layer "SOLID")
  (setq blockObj (p-blk nil)
        pl (add-polyline blockObj
                         (list 0 dimspace 0)
                         (fillet-all-pts '((0 5 0)
                                           (0 0 0)
                                           (12 0 0)
                                           (12 5 0))
                                         0.3)
                         nil)
        top-line (+ 5 dimspace))
  (offset&close pl -0.1494 nil nil)
  (set-active-layer "DIM")
  (set-active-dimstyle "2xFRACTIONAL")
  (vla-AddDimRotated blockObj
                     (3dpt 0 (+ 0.3 dimspace))
                     (3dpt 12 (+ 0.3 dimspace))
                     (3dpt 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt 11.7 dimspace)
                     (3dpt 12 top-line)
                     (3dpt (+ 12 dimspace) 0)
                     (dtr 90))
  blockObj)

(defun edge-form (/ blockObj)
  (setq blockObj (p-blk nil)
        pl (add-polyline blockObj
                         (list 0 dimspace 0)
                         '((0 0 0)
                           (7 0 0)
                           (7 7 0)
                           (5.5 7 0))
                         nil)
        top-line (+ 7 dimspace))
  (vla-put-Layer pl "SOLID")
  (offset&close pl -0.1345 nil nil)
  (set-active-dimstyle "2xFRACTIONAL")
  (vla-AddDimRotated blockObj
                     (3dpt 0 dimspace)
                     (3dpt 7 dimspace)
                     (3dpt 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt 7 dimspace)
                     (3dpt 7 top-line)
                     (3dpt (+ 7 dimspace) 0)
                     (dtr 90))
  (vla-AddDimRotated blockObj
                     (3dpt 7 top-line)
                     (3dpt 5.5 top-line)
                     (3dpt 0 (+ top-line dimspace))
                     0)
  blockObj)

(defun x27-angle-detail (len off-y / blockObj bot-line top-line height)
  (setq blockObj (p-blk nil)
        bot-line (* dimspace 3)
        top-line (+ bot-line 8))

  ;; left gusset
  ((lambda(ins)
     (vla-put-Layer (add-rectangle blockObj 0.375 7.5 ins) "HIDDEN")
     (vla-put-Layer (add-rectangle blockObj 0.375 -2.5 ins) "SOLID")
     (vla-AddDimRotated blockObj
                        (3dpt (car ins) (+ (cadr ins) 7.5))
                        (3dpt 0 top-line)
                        (3dpt 0 (+ top-line (* 2 dimspace)))
                        0))
   (list 2.75 bot-line 0))
  ;; right gusset
  ((lambda(ins)
     (vla-put-Layer (add-rectangle blockObj -0.375 7.5 ins) "HIDDEN")
     (vla-put-Layer (add-rectangle blockObj -0.375 -2.5 ins) "SOLID")
     (vla-AddDimRotated blockObj
                        (3dpt (car ins) (+ (cadr ins) 7.5))
                        (3dpt len top-line)
                        (3dpt len (+ top-line (* 2 dimspace)))
                        0))
   (list (- len 2.75) bot-line 0))

  ;; left clip angle
  ((lambda(ins)
     (vla-put-Layer (add-rectangle blockObj 3 -0.75 ins) "HIDDEN")
     (vla-put-Layer (add-rectangle blockObj 3 2.25 ins) "SOLID")
     (vla-AddDimRotated blockObj
                        (3dpt (car ins) (+ 2.25 (cadr ins)))
                        (3dpt 0 top-line)
                        (3dpt 0 (+ top-line (* 3 dimspace)))
                        0))
   (list (+ 4 l-string_wid) top-line 0))
  ;; right clip angle
  ((lambda(ins)
     (vla-put-Layer (add-rectangle blockObj 3 -0.75 ins) "HIDDEN")
     (vla-put-Layer (add-rectangle blockObj 3 2.25 ins) "SOLID")
     (vla-AddDimRotated blockObj
                        (3dpt (+ 3 (car ins)) (+ 2.25 (cadr ins)))
                        (3dpt len top-line)
                        (3dpt len (+ top-line (* 3 dimspace)))
                        0))
    (list (- len (+ 4 3 r-string_wid)) top-line 0))

  ;; angle outline
  (vla-put-Layer (add-polyline blockObj
                               (list 0 (- top-line 0.5) 0)
                               (list '(0 0 0) (list len 0 0))
                               nil)
                 "HIDDEN")
  (vla-put-Layer (add-rectangle blockObj len 8 (list 0 bot-line 0)) "SOLID")
  (vla-AddDimRotated blockObj
                     (3dpt 0 bot-line)
                     (3dpt len bot-line)
                     (3dpt 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt len bot-line)
                     (3dpt len top-line)
                     (3dpt (+ len (* 4 dimspace)) 0)
                     (dtr 90))

  ;; bolt holes
  ((lambda(bolt-line rad / lins rins)
     (setq lins (list bot_hbolt bolt-line 0)
           rins (list (- len bot_hbolt) bolt-line 0))
     (vla-put-Layer (vla-AddCircle blockObj (3dpt lins) rad) "SOLID")
     (add-bolt-centerline lins 'both)
     (vla-put-Layer (vla-AddCircle blockObj (3dpt rins) rad) "SOLID")
     (add-bolt-centerline rins 'both)
     (vla-AddDimRotated blockObj
                        (3dpt (+ 1 (car rins)) (cadr rins))
                        (3dpt len top-line)
                        (3dpt (+ len (* 3 dimspace)) 0)
                        (dtr 90))
     (setq lins (displace-pt lins '(0 -1 0))
           rins (displace-pt rins '(0 -1 0)))
     (vla-AddDimRotated blockObj
                        (3dpt rins)
                        (3dpt len bot-line)
                        (3dpt 0 (- bot-line dimspace))
                        0)
     (vla-AddDimRotated blockObj
                        (3dpt lins)
                        (3dpt rins)
                        (3dpt 0 (- bot-line dimspace))
                        0)
     (vla-AddDimRotated blockObj
                        (3dpt lins)
                        (3dpt 0 bot-line)
                        (3dpt 0 (- bot-line (* 2 dimspace)))
                        0)
     (vla-AddDimRotated blockObj
                        (3dpt lins)
                        (3dpt len bot-line)
                        (3dpt 0 (- bot-line (* 2 dimspace)))
                        0))
   (- top-line bot_vbolt off-y)
   (/ 11 32.0))

  ;; insert side view
  ((lambda(ins)
     (mirror&delete (insert-block blockObj
                                  ins
                                  ((lambda(blockObj)
                                     (set-active-layer "SOLID")
                                     (detail-side-x27-common blockObj 0)
                                     (set-active-layer "DIM")
                                     blockObj)
                                   (p-blk nil)))
                    ins
                    'y)
     (vla-AddDimRotated blockObj
                        (3dpt (displace-pt ins '(-6 0 0)))
                        (3dpt (displace-pt ins '(-7.25 2.125 0)))
                        (3dpt 0 (+ top-line dimspace))
                        0)
     (add-mleader blockObj
                  (displace-pt ins (list -7.5 0.75 0))
                  '((0 0 0)
                    (-2 0 0)
                    (-2 2 0)
                    (-3 2 0))
                  (xd-value "x27-3x3" marks))
     (add-mleader blockObj
                  (displace-pt ins '(0 -10.5 0))
                  '((0 0 0)
                    (0 -2 0)
                    (-2 -2 0))
                  (xd-value "x27-gusset" marks)))
   (list (- dimspace) top-line 0))

  blockObj)

(defun x27-gusset-detail (/ blockObj pts second-dim-line right-dim-line)
  (setq blockObj (p-blk nil)
        pts (displace-pts '((0.0 0.0 0.0)
                            (5.0 0.0 0.0)
                            (5.0 1.0 0.0)
                            (19.0 1.0 0.0)
                            (20.0 2.0 0.0)
                            (20.0 12.0 0.0))
                          (list 0 (* 2 dimspace) 0))
        second-dim-line (3dpt 0 dimspace 0)
        right-dim-line (3dpt (+ 20 (* 3 dimspace)) 0))
  (set-active-layer "SOLID")
  (add-polyline blockObj '(0 0 0) pts T)
  (add-rectangle blockObj -0.75 11 (list (- (* 2 dimspace)) (* 2 dimspace) 0))
  (set-active-layer "DIM")
  (set-active-dimstyle "2xFRACTIONAL")
  (vla-AddDimRotated blockObj
                     (3dpt (nth 0 pts))
                     (3dpt (nth 4 pts))
                     (3dpt 0 0)
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt (nth 1 pts))
                     (3dpt (nth 0 pts))
                     second-dim-line
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt (nth 1 pts))
                     (3dpt (nth 3 pts))
                     second-dim-line
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt (nth 3 pts))
                     (3dpt (nth 4 pts))
                     second-dim-line
                     0)
  (vla-AddDimRotated blockObj
                     (3dpt (nth 3 pts))
                     (3dpt (nth 1 pts))
                     right-dim-line
                     (dtr 90))
  (vla-AddDimRotated blockObj
                     (3dpt (nth 3 pts))
                     (3dpt (nth 4 pts))
                     right-dim-line
                     (dtr 90))
  (vla-AddDimRotated blockObj
                     (3dpt (nth 0 pts))
                     (3dpt (nth 5 pts))
                     (3dpt (- dimspace) 0)
                     (dtr 90))
  blockObj)

;; add a 2" polyline on the CENTER layer (fab) or NO PLOT layer (sub) at ins
(defun add-bolt-centerline (ins h-or-v / pts)
  (if (= h-or-v 'both)
      (list
       (add-bolt-centerline ins 'h)
       (add-bolt-centerline ins 'v))
      (progn
        (setq pts (cond ((= h-or-v 'h) '((-1 0 0)
                                         (1 0 0)))
                        ((= h-or-v 'v) '((0 -1 0)
                                         (0 1 0)))
                        (T (error "add-bolt-centerline: bad value for h-or-v"))))
        (vla-put-Layer-r (add-polyline blockObj ins pts nil) (if fab-p "CENTER" "NO PLOT")))))

  ;; rectangle with miter cut two ends
  (defun 2-end-miter-45 (scl lng wid / blk bot top shp)
    (foreach var '(lng wid) (set var (* scl (vl-symbol-value var))))
    (set-active-dimstyle (strcat (to-string scl) "xFractional"))
    (setq
      blk (p-blk nil) bot (* 4 dimspace) top (+ bot wid)
      shp (#! '((#wid 0) (0 #wid) (#lng #wid) (#(- lng wid) 0)))
    ) ;_ setq
    (set-active-layer "SOLID")
    (add-polyline blk (list 0 bot) shp T)
    (set-active-layer "DIM")
    (setq shp (displace-pts shp (list 0 bot)))
    (vla-AddDimRotated blk
      (3dpt (caddr shp)) (3dpt (cadr shp))
      (3dpt 0 0) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (caddr shp)) (3dpt (car shp))
      (3dpt 0 dimspace) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (caddr shp)) (3dpt (cadddr shp))
      (3dpt 0 (* 2 dimspace)) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimAngular blk
      (3dpt (car shp)) (3dpt (cadddr shp))
      (3dpt (cadr shp)) (3dpt (* -2 dimspace) (+ bot 0.00001))
    ) ;_ vla-AddDimAngular
    (vla-AddDimAngular blk
      (3dpt (cadddr shp)) (3dpt (car shp))
      (3dpt (caddr shp)) (3dpt (+ lng (* 2 dimspace)) (+ bot 0.00001))
    ) ;_ vla-AddDimAngular
    blk
  ) ;_ defun 2-end-miter-45

  ;; rectangle with miter cut one end
  (defun 1-end-miter-45 (scl lng wid / blk bot top shp)
    (foreach var '(lng wid) (set var (* scl (vl-symbol-value var))))
    (set-active-dimstyle (strcat (to-string scl) "xFractional"))
    (setq
      blk (p-blk nil) bot (* 3 dimspace) top (+ bot wid)
      shp (#! '((#wid 0) (0 #wid) (#lng #wid) (#lng 0)))
    ) ;_ setq
    (set-active-layer "SOLID")
    (add-polyline blk (list 0 bot) shp T)
    (set-active-layer "DIM")
    (setq shp (displace-pts shp (list 0 bot)))
    (vla-AddDimRotated blk
      (3dpt (cadddr shp)) (3dpt (cadr shp))
      (3dpt 0 0) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (cadddr shp)) (3dpt (car shp))
      (3dpt 0 dimspace) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimAngular blk
      (3dpt (car shp)) (3dpt (cadddr shp))
      (3dpt (cadr shp)) (3dpt (* -2 dimspace) (+ bot 0.00001))
    ) ;_ vla-AddDimAngular
    blk
  ) ;_ defun 1-end-miter-45

  (defun clip-fbar (scl lng lct wid wct / blk bot top shp)
    (foreach var '(lng lct wid wct)
      (set var (* scl (vl-symbol-value var)))
    ) ;_ foreach
    (set-active-dimstyle (strcat (to-string scl) "xFractional"))
    (setq
      lct (- lng lct) wct (- wid wct)
      blk (p-blk nil) bot (* 2 dimspace) top (+ bot lng)
      shp (#! '((0 0) (#lng 0) (#lng #wct) (#lct #wid) (0 #wid)))
    ) ;_ setq
    (set-active-layer "SOLID")
    (add-polyline blk (list 0 bot) shp T)
    (set-active-layer "DIM")
    (setq shp (displace-pts shp (list 0 bot)))
    (vla-AddDimRotated blk
      (3dpt (car shp)) (3dpt (cadr shp))
      (3dpt 0 0) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (cadddr shp)) (3dpt (caddr shp))
      (3dpt (displace-pt (last shp) (list 0 (* 2 dimspace)))) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (cadddr shp)) (3dpt (last shp))
      (3dpt (displace-pt (last shp) (list 0 (* 2 dimspace)))) 0
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (car shp)) (3dpt (last shp))
      (3dpt (displace-pt (car shp) (list (* -2 dimspace) 0))) (/ pi 2.0)
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (caddr shp)) (3dpt (cadr shp))
      (3dpt (displace-pt (cadr shp) (list (* 2 dimspace) 0))) (/ pi 2.0)
    ) ;_ vla-AddDimRotated
    (vla-AddDimRotated blk
      (3dpt (caddr shp)) (3dpt (cadddr shp))
      (3dpt (displace-pt (cadr shp) (list (* 2 dimspace) 0))) (/ pi 2.0)
    ) ;_ vla-AddDimRotated
    blk
  ) ;_ defun clip-fbar

  (defun bent-pipe (bend-list / tck-mrk bnd-mrk lng-txt blk)
    (defun tck-mrk (x)
      (add-polyline blk (list x 0) '((0 2) (0 -2)) nil)
    ) ;_ defun tck-mrk
    (defun bnd-mrk (x a)
      (tck-mrk (distof x))
      (add-mtext blk (list (distof x) 3.5) x 12.0 'middlecenter)
      (add-mtext blk (list (distof x) -3.5) a 12.0 'middlecenter)
    ) ;_ defun bnd-mrk
    (defun lng-txt (x)
      (add-polyline blk '(0 0) (list '(0 0) (list (distof x) 0)) nil)
      (mapcar 'tck-mrk (list (distof x) (- (distof x) 0.75)))
      (add-mtext blk (list (+ 1.5 (distof x)) 0) x 12.0 'middleleft)
    ) ;_ defun lng-txt
    (setq blk (p-blk nil))
    (set-active-layer "DIM")
    (while (> (length bend-list) 1)
      (apply 'bnd-mrk (take! 2 'bend-list))
    ) ;_ while
    (lng-txt (car bend-list))
    blk
  ) ;_ defun bent-pipe
