(set-file-docstring "Functions related to pans in fab drawings")

(defun-q process-pan-def (pan / filename defs current)
  "Returns PAN with P# attached

Adds PAN to the P# database if it isn't present"
  (declare (vars (pan list))
           with-data)
  (setq filename (strcat (xd-value "dir" job-info) "/pans.csv")
        pan-def (append (list (xd-value "mat" pan)
                              (xd-value "sty" pan)
                              (xd-value "thk" pan)
                              (xd-value "len" pan))
                        (xd-value "def" pan))
        pan-def (process-generic-def pan-def filename))
  (append! 'pan
           (#! '(("mark" #(strcat "P" (to-string (car pan-def))))
                 ("num" #(car pan-def))))))
(defun-r 'process-pan-def)

(defun-q draw-pan-detail (ins pan / blockObj pts obj style cdimstyle)
  "Draws a detail for PAN at INS"
  (setq blockObj (p-blk "pan")
        style (xd-value "sty" pan)
        pts (rezero-pts (xd-value "pts-x2" pan) 0)
        obj (add-polyline blockObj (list (/ dimspace 2.0) (* 2 dimspace) 0) pts nil))
  (vla-put-Layer obj "SOLID")

  ;; a dim-def is a list of dimensions to apply, each consisting of
  ;; two indices into `pts` and an alignment
  (setq dim-defs (cond ((= style "hv")
                        '((0 2 aligned)
                          (0 1 bottom)
                          (1 2 right)))
                       ((= style "vh")
                        '((0 2 right)
                          (2 1 top)))
                       ((= style "hvh")
                        '((0 2 aligned)
                          (0 1 bottom)
                          (1 3 right)
                          (3 2 top)))
                       ((= style "vhv")
                        '((1 3 aligned)
                          (0 1 left)
                          (1 2 bottom)
                          (2 3 right)))
                       ((= style "vhvh")
                        '((1 3 aligned)
                          (0 1 left)
                          (1 2 bottom)
                          (2 4 right)
                          (4 3 top)))))
  (defun add-dim (dim-def / pt1 pt2 alignment disp args)
    (defun disp(x) (3dpt (displace-pt pt2 x)))
    (setq pt1 (nth (car dim-def) pts)
          pt2 (nth (cadr dim-def) pts)
          alignment (caddr dim-def)
          args (append (list blockObj (3dpt pt1) (3dpt pt2))
                       (cond ((= alignment 'aligned)
                              (list (disp (list 0 dimspace 0))
                                    (angle pt1 pt2)))
                             ((= alignment 'right)
                              (list (disp (list dimspace 0 0))
                                    (dtr 90)))
                             ((= alignment 'left)
                              (list (disp (list (- dimspace) 0 0))
                                    (dtr 90)))
                             ((= alignment 'top)
                              (list (disp (list 0 dimspace 0))
                                    0))
                             ((= alignment 'bottom)
                              (list (disp (list 0 (- dimspace) 0))
                                    0)))))
    (vla-Move (apply 'vla-AddDimRotated args)
              (3dpt 0 0)
              (3dpt (/ dimspace 2.0) (* 2 dimspace))))
  (setq cdimstyle (vla-getvar 'dimstyle))
  (set-active-dimstyle "2xDECIMAL")
  (mapcar 'add-dim dim-defs)
  (set-active-dimstyle cdimstyle)

  ;; callout
  (draw-pan-detail--callout blockObj (strcat "P" (xd-value "num" pan)))
  (insert-block modelspace ins blockObj))
(defun-r 'draw-pan-detail)

(defun-q draw-pan-detail--callout (container mark / unit)
  "Adds the callout for a pan detail with MARK to CONTAINER"
  (setq unit (* (vla-getvar 'dimscale) -0.25))
  (add-polyline container
                '(0 0 0)
                (list '(0 0 1)
                      (list unit 0 1)
                      '(0 0 0)
                      '(45 0 0))
                nil)
  (add-mtext container
             (list (* -0.125 (vla-getvar 'dimscale)) 0)
             mark
             1
             'middlecenter))
(defun-r 'draw-pan-detail--callout)

(defun-q call-out-pans ( / pans processed-pans indices fnosex fnosey)
  "Adds pan MLEADERS to main view"
  (setq pans (mapcar 'process-pan-def (calc-stair-pans)))
  (foreach pan (mapcar '(lambda(p)
                         (list (car (xd-value "ind" p))
                               (list (xd-value "mark" p)
                                     (xd-value "off" p))))
                       pans)
           (if (not (assoc (car pan) indices))
               (add-to-list 'indices pan)
             (add-to-alist 'indices (car pan) (cdr pan) T)))
  (setq fnosex (+ bot_cdist bot_ext)
        fnosey rise_hgt)
  (mapcar 'call-out-pans--process-idx indices))
(defun-r 'call-out-pans)

(defun-q call-out-pans--process-idx (idx / leaders pan nosex nosey)
  "Adds pan callouts at IDX"
  (setq pans (cdr idx)
        idx (car idx)
        nosex (+ fnosex (* idx tread_depth))
        nosey (+ fnosey (* idx rise_hgt)))
  (cond
    ;; first nosing
    ((= idx 0) (call-out-pans--process-first pans))
    ;; last nosing
    ((= idx (1- rise_qty)) (call-out-pans--process-last pans))
    ;; mid nosings
    (T (call-out-pans--process-mid pans)))
  (mapcar '(lambda(l / mark pts)
            (setq mark (car l)
                  pts (cdr l))
            (add-mleader blockObj '(0 0) pts mark))
          leaders))
(defun-r 'call-out-pans--process-idx)

(defun-q call-out-pans--add-leader (mark offset pts)
  "Adds an element of the form '(MARK PTS) to LEADERS"
  (add-to-list 'leaders (cons mark (displace-pts pts (displace-pt offset (list nosex nosey))))))
(defun-r 'call-out-pans--add-leader)

(defun-q call-out-pans--process-first (pans / pts)
  "Adds callouts for first nosing pans"
  (setq pts '((0 0 0)
              (-12 9 0)
              (-13 9 0)))
  (if (begins-with tread_style "2")
      (progn
        ;; haunch: consume first pan
        (if (= bot_conn_det "5")
            ((lambda(/ pan pts)
               (setq pan (pop! 'pans)
                     pts '((-6 0)
                           (-6 6)
                           (-7 6)))
               (apply 'call-out-pans--add-leader (rcons pan pts)))))
        (if (= stn_style "None")
            (apply 'call-out-pans--add-leader (rcons (car pans) pts))
            ;; checker with STN
            ;; combine multiple pans into one leader
            ((lambda(/ mark offset)
               (setq mark (strjoin (mapcar 'car pans) "\n")
                     offset ((lambda(offsets)
                               (sort! 'offsets (sort-by 'car '<))
                               (list (caadr offsets)
                                     (cadar offsets)))
                             (mapcar 'cadr pans)))
               (call-out-pans--add-leader mark offset pts)))))
      ;; not checker
      (apply 'call-out-pans--add-leader (rcons (car pans) pts))))
(defun-r 'call-out-pans--process-first)

(defun-q call-out-pans--process-mid (pans / pan)
  "Adds the callout for the mid pans"
  (setq pan (car pans))
  (call-out-pans--add-leader (car pan)
                             (cadr pan)
                             '((0 0 0)
                               (-1 0 0)
                               (-1 6 0)
                               (-2 6 0))))
(defun-r 'call-out-pans--process-mid)
  
(defun-q call-out-pans--process-last (pans)
  "Adds callouts for the last nosing pans"
  (mapcar '(lambda (pan pts)
            (call-out-pans--add-leader (car pan) (cadr pan) pts))
          (reverse pans)
          '(((1 0 0)
             (1 6 0)
             (2 6 0))
            ((0 0 0)
             (-1 0 0)
             (-1 5 0)
             (-4 5 0)))))
(defun-r 'call-out-pans--process-last)
