(set-file-docstring "Functions for stair rail fabs")

(psc-include '("rail/util.lsp"))

(defun-q calc-stair-rail-fab (stair-data
                              /
                              calc
                              ret
                              )
  "Returns a list of 0-2 stair rail definitions for a stair"
  (declare (vars (stair-data list (data-list-p stair-data))))
  (defun calc (side
               / ret opcl cospitch npanels pcl series hpl t15 otherside)
    (expect-with-data "calc-stair-rail-fab/calc")
    (if (= (sidekey-rail "type") "Stair Rail")
        (progn
          (setq opcl (calc-stair-rail-opcl side)
                cospitch (cos (dtr pitch))
                npanels (ceil (/ opcl 48.0))
                pcl (/ opcl (float npanels))
                series (atoi rail_style)
                otherside (if (= side "i") "o" "i")
                ret (#! '(("series" #series)
                          ("finish" #finish)
                          ("template" #(strcat "stair-rail-" (to-string series)))
                          ("mark" "SR")
                          ("stair_number" #number)
                          ("level" #level)
                          ("sequence" #(rail-sequence side))
                          ("ascend" #(rail-ascend side))
                          ("pitch" #(rtos pitch 2 2))
                          ("tread_depth" #(rtos tread_depth 2 0))
                          ("rise_height" #(rtos rise_hgt 2 2))
                          ("outside_post_centerline" #(frac opcl))
                          ("#_of_posts" #(to-string (1+ npanels)))
                          ("post_centerline" #(frac pcl))
                          ("top_cap_length" #(rtos (- (/ (- opcl 1.5) cospitch) 0.0625) 2 2))
                          ("handgrab_length" #(cond ((and (= (sidekey-rail "top_dloop") "1")
                                                      (= (sidekey-rail "bot_dloop") "1"))
                                                     "DR")
                                               ((= (sidekey-rail "top_dloop") "1") "TR")
                                               ((= (sidekey-rail "bot_dloop") "1") "BR")
                                               (t (rtos (+ 0.25 (/ (+ 0.25 opcl) cospitch)) 2 2)))))))
          (if (= (sidekey-rail "topmnt") "1")
              (progn
                (setq hpl (/ (- 38.375 (sidekey "nosing_mid")) cospitch)
                      t15 (* 1.5 (tan (dtr pitch))))
                (subst-key! "template" (strcat (xd-value "template" ret) "-top") 'ret)
                (append! 'ret (#! '(("high_post_length" #(rtos hpl 2 2))
                                    ("mid_post_length" #(rtos (/ (- 36.875 (sidekey "nosing_mid"))
                                                               cospitch)
                                                         2 2))
                                    ("low_post_length" #(rtos (+ hpl t15) 2 2))))))
              (append! 'ret (#! '(("post_length" "50")
                                  ("gusset_length" #(if (sidekey "string_flg")
                                                     (rtos (- (sidekey "string_wid")
                                                            (sidekey "string_web"))
                                                      2 2)
                                                     ""))))))
          (cond ((= series 100)
                 (append ret (#! '(("midline_length" #(rtos (/ (+ 0.25 opcl) cospitch) 2 2))))))
                ((= series 200)
                 (append ret
                         (#! '(("midline_length" #(rtos (/ (+ 0.25 opcl) cospitch) 2 2))
                               ("int_hss_length" #(rtos (- (/ (- pcl 1.5) cospitch) 0.0625) 2 2))
                               ("#_of_int_hss" #npanels)))))
                ((= series 300)
                 (append ret
                         (#! '(("#_of_int_hss" #(* 6 npanels))
                               ("int_hss_length" #(rtos (- (/ (- pcl 1.5) cospitch) 0.0625) 2 2))))))
                ((= series 400)
                 (setq pspc (rail-calc-picket-spacing (- pcl 1.5)))
                 (append ret
                         (#! '(("int_hss_length" #(rtos (- (/ (- pcl 1.5) cospitch) 0.0625) 2 2))
                               ("#_of_int_hss" #npanels)
                               ("picket_length" #(rtos (- (/ 29.625 cospitch) 0.0625) 2 2))
                               ("picket_spacing" #(rtos pspc 2 2))
                               ("#_of_pickets" #(rtos (* (/ (- pcl 1.5 pspc)
                                                          (+ pspc 0.5))
                                                       npanels)
                                                 2 0))))))))))

  (with-data stair-data '((setq ret nil)
                          (if (setq tmp (calc "i"))
                              (add-to-list 'ret tmp))
                          (if (setq tmp (calc "o"))
                              (add-to-list 'ret tmp))
                          ret)))
(defun-r 'calc-stair-rail-fab)


(defun-q combine-stair-rail-fabs (rail-datas
                                / key keys rail-map returns)
  "Combines RAIL-DATAS based on template and finish"
  (declare (vars (rail-datas list (vl-every 'data-list-p rail-datas))))
  (foreach rail rail-datas
           (setq key (list (xd-value "template" rail)
                           (xd-value "finish" rail)))
           (if (not (member key keys))
               (add-to-list 'keys key))
           (add-to-alist 'rail-map key (list rail) T))
  rail-map)
(defun-r 'combine-stair-rail-fabs)

(defun-q make-stair-rail-fabs (stairs / rail-datas template finish rails)
  "Bundles rails and calls %make-stair-rail-fab"
  (setq rail-datas (combine-stair-rail-fabs (map-append 'calc-stair-rail-fab stairs)))
  (foreach x rail-datas
           (setq template (caar x)
                 finish (cadar x)
                 rails (cdr x))
           (while rails
                  (make-stair-rail-fab template finish (take! 58 'rails)))))
(defun-r 'make-stair-rail-fabs)

(defun-q make-stair-rail-fab (template finish rail-datas)
  "Opens TEMPLATE and creates a standard stair rail fab drawing"
  (debug-print '("Making stair rail fab: " #template))
  (make-rail-fab-common template '(stair-rail-write-table finish rail-datas)))
(defun-r 'make-stair-rail-fab)

(defun-q stair-rail-write-table (finish rail-datas / row col table cols)
  "Writes the table in a stair rail fab drawing"
  (set-finish finish)
  (table-add-data (get-table-by-title "*Stair Rails") rail-datas))
(defun-r 'stair-rail-write-table)

(defun-q calc-stair-rail-dloop (side
                              / ret top-dloop-p bot-dloop-p lvl-p top-AB top-A top-B top-horiz
                              top-lines top-line-args bot-AB bot-A bot-B bot-horiz bot-detail
                              bot-lines bot-line-args overlong-p top-return)
  "Returns a 1-2 item list of returns for this rail"
  (declare (vars (side str))
           with-data)

  ;; return nil if the length is too long to bend
  ;; i.e. if the cut length is greater than 240 + first bend line
  (defun overlong-p (lines)
    (> (apply '- (mapcar 'distof (list (last lines) (car lines)))) 240))

  (setq top-dloop-p (= (sidekey-rail "top_dloop") "1")
        bot-dloop-p (= (sidekey-rail "bot_dloop") "1")
        lvl-p (= rail_lvl-off "1"))

  ;; top
  (if (and (wcmatch (sidekey-rail "type") "*Stair*")
           top-dloop-p)
      (progn
        (setq top-AB (- (stair-rail-AB side)
                        (if bot-dloop-p 18 0))
              top-horiz (- 17.375 (sidekey-rail "top_ext"))
              top-return (+ (sidekey-rail "top_lvl") 3.125)
              top-line-args '(6 90
                              #top-return 90
                              12 90
                              #(sidekey-rail "top_lvl") #pitch
                              #top-B)
              top-B top-AB
              top-lines (pipe-bend-lines-padded (#! top-line-args)))
        
        (if (overlong-p top-lines)
            (setq top-B 18
                  top-A (frac (- top-AB top-B)))
            (setq top-A ""))
        (setq top-lines (pipe-bend-lines-padded (#! top-line-args)))
        
        (add-to-list 'ret
                     (append (#! '(("mark" "SR")
                                   ("stair_number" #number)
                                   ("level" #level)
                                   ("sequence" #(rail-sequence side))
                                   ("ascend" #(rail-ascend side))
                                   ("detail" "4")
                                   ("straight_pipe_length" #top-A)
                                   ("length_@_pitch" #(frac top-B))
                                   ("distance_to_break_centerline"
                                    #(frac (- top-horiz (sidekey-rail "top_lvl"))))
                                   ("level-off" #(frac (sidekey-rail "top_lvl")))
                                   ("return" #(frac top-return))))
                             (mapcar 'list
                                     '("line_1" "angle_1" "line_2" "angle_2"
                                       "line_3" "angle_3" "line_4" "angle_4"
                                       "cut_length")
                                     top-lines)))))

  ;; bot
  (if (and (wcmatch (sidekey-rail "type") "*Stair*")
           bot-dloop-p)
      (progn
        (setq bot-horiz (max (- (+ tread_depth
                                   3.125
                                   (if (= rail_lvl-off "0") 0 (sidekey-rail "bot_lvl")))
                                (sidekey-rail "bot_ext")
                                (min bot_ext tread_depth))
                             6)
              bot-detail (if lvl-p
                             "3"
                             (if (>= bot-horiz 10)
                                 "1" "2"))
              bot-line-args (cond ((= bot-detail "1")
                                   '(6 90
                                     #bot-horiz 90
                                     14 #(- 90 pitch)
                                     #bot-B))
                                  ((= bot-detail "2")
                                   '(6 90
                                     12 #(- 90 pitch)
                                     #bot-B))
                                  ((= bot-detail "3")
                                   '(#bot-B #pitch
                                     #(sidekey-rail "bot_lvl") 90
                                     14 90
                                     #bot-horiz 90
                                     18)))
              bot-AB (if top-dloop-p 18 (stair-rail-AB side))
              bot-B (if (and (= bot-detail "3")
                             (not top-dloop-p)
                             (> bot-AB 50))
                        18 bot-AB)
              bot-lines (pipe-bend-lines-padded (#! bot-line-args)))

        (if (overlong-p bot-lines)
            (setq bot-B (/ bot-AB 2.0)
                  bot-lines (pipe-bend-lines-padded (#! bot-line-args))))
        (if (= bot-B bot-AB)
            (setq bot-A "")
            (setq bot-A (frac (- bot-AB bot-B))))
            
        (add-to-list 'ret
                     (append (#! '(("mark" "SR")
                                   ("stair_number" #number)
                                   ("level" #level)
                                   ("sequence" #(rail-sequence side))
                                   ("ascend" #(rail-ascend side))
                                   ("detail" #bot-detail)
                                   ("straight_pipe_length" #bot-A)
                                   ("length_@_pitch" #(frac bot-B))
                                   ("distance_to_break_centerline"
                                    #(frac (- bot-horiz 0.75 (if (= bot-detail "3") 15 0))))
                                   ("level-off" #(if lvl-p (frac (sidekey-rail "bot_lvl")) ""))
                                   ("return" #(frac bot-horiz))))
                             (mapcar 'list
                                     '("line_1" "angle_1" "line_2" "angle_2"
                                       "line_3" "angle_3" "line_4" "angle_4"
                                       "cut_length")
                                     bot-lines)))))
  (mapcar '(lambda(x) (cons finish x)) ret))
(defun-r 'calc-stair-rail-dloop)

(defun-q calc-stair-rail-dloops (stairs / dloops)
  "Returns a list of D-loops"
  (foreach stair stairs
    (with-data stair
      '((append! 'dloops (calc-stair-rail-dloop "i"))
        (append! 'dloops (calc-stair-rail-dloop "o"))))))
(defun-r 'calc-stair-rail-dloops)

(defun-q combine-stair-rail-dloops (d-loops / finish rail-map)
  "Returns D-LOOPS combined based on finish"
  (foreach d d-loops
	   (setq finish (pop! 'd))
	   (add-to-alist 'rail-map finish (list d) T))
  rail-map)
(defun-r 'combine-stair-rail-dloops)

(defun-q make-stair-rail-dloop-fabs (stairs / d-loops i finish loops)
  "Calls %combine-stair-rail-dloops and makes D-loop drawings"
  (setq d-loops (combine-stair-rail-dloops (calc-stair-rail-dloops stairs))
        i 0)
  (foreach map d-loops
    (setq finish (car map)
	  loops (cdr map))
    (while loops
	   (debug-print '("Making D-loop fab " #(inc! 'i)))
	   (make-rail-fab-common "stair-rail-dloop"
				 '(stair-rail-dloop-write-table finish (take! 47 'loops))))))
(defun-r 'make-stair-rail-dloop-fabs)

(defun-q stair-rail-dloop-write-table (finish d-loop-datas
                                              / row)
  "Writes the D-loop table"
  (set-finish finish)
  (table-add-data (get-table-by-title "Stair Rail Bent Handrails") d-loop-datas))
(defun-r 'stair-rail-dloop-write-table)
