(set-file-docstring "Functions dealing with stringers in fab drawings

Here be dragons. This file is a mess.")

(defun-q stair-fab-process-stringers (/ ins stringer-material caps l-string-bot-pts
                                        l-string-mid-pts l-string-top-pts r-string-bot-pts
                                        r-string-mid-pts r-string-top-pts)
  "Adds both stringer details to drawing and their material to the table"
  (declare (vars (dimspace nil (numberp dimspace))
                 (dwg-height nil (numberp dwg-height))
                 (piece-mark str)
                 (stringers_eq str)
                 (mat-table vla-object)))
  ;; add details and calc material
  (setq ins (list (* 5 dimspace)
                  (- dwg-height (* 5 dimspace))
                  0))
  (fab-stringer--add-detail ins "l" piece-mark)
  (if (= stringers_eq "0")
      (progn
        (setq ins (displace-pt ins (list 0 (- (* dimspace 8)) 0)))
        (fab-stringer--add-detail ins "r" piece-mark)))
  (setq next-part-detail (displace-pt ins (list 0 (- (+ (* 3 dimspace) 70)) 0)))
  ;; add cap material to stringer material and put them in the table
  (fab-stringer--add-cap-material caps)
  (mapcar!
   '(lambda(s / f qty)
     ;; figure out quantities
     (setq f (macro-expand '(lambda(x)
                             (if (= (cadr x) #(xd-value "mark" s)) 1 0)))
      qty (apply '+ (mapcar f marks)))
     (if (= stringers_eq "1")
         (setq qty (* 2 qty)))
     (add-to-list 's (list "qty" (to-string qty))))
   'stringer-material)
  (table-add-data mat-table stringer-material))
(defun-r 'stair-fab-process-stringers)

(defun-q fab-stringer--add-cap-material (caps / cap-marks tmp cap-slot)
  "Adds material from CAPS to STRINGER-MATERIAL"
  ;; combine caps
  (foreach cap caps
           (setq cap-slot (pop! 'cap))
           (if (setq tmp (assoc cap cap-marks))
               (subst! (cons cap (cons cap-slot (cdr tmp)))
                       tmp
                       'cap-marks)
               (add-to-list 'cap-marks (list cap cap-slot))))
  (if stringers-eq-p
      (mapcar! '(lambda(cap / def marks)
                  (setq def (car cap)
                        marks (cdr cap))
                  (cons def
                        (foreach mark (cdr cap)
                                 (add-to-list 'marks (vl-string-subst "r" "l" mark)))))
               'cap-marks))
  ;; add caps to material list
  (foreach cap cap-marks
           (setq cap-mark (strcat piece-mark (to-string current-mark-no)))
           (foreach cap-slot (cdr cap)
                    (add-to-list (read (strcat cap-slot "-cap-marks"))
                                 cap-mark))
           (add-to-list 'stringer-material
                        (list (list "mark" cap-mark)
                              (list "qty" (length (cdr cap)))
                              (list "desc" "PLATE 1/4")
                              (list "width" (frac (caar cap)))
                              (list "length" (frac (cadar cap)))))
           (inc! 'current-mark-no)))
(defun-r 'fab-stringer--add-cap-material)

(defun-q fab-stringer--add-detail (blkins side mark / blockObj ins def bseg bseg-pline
                                     mseg mseg-pline mseg-pgram tseg tseg-pline material
                                     plate-p width cap-width flange depth starter-p clayer
                                     cdimstyle main-dim-depth inc-current-mark obj pt len)
  "Adds a single stringer detail for SIDE at BLKINS

Adds material to STRINGER-MATERIAL and cap material to CAPS"
  (declare (vars (blkins list)
                 (side str)
                 (mark str)))
  (setq blockObj (define-block (strcat side "-stringer") nil)
        ins '(0 0 0)
        def (stair-stringer-calc side)
        bseg (cdr (assoc 'string-bot-seg def))
        tseg (assoc 'string-top-seg def)
        material (sidekey "string_mat")
        plate-p (begins-with material "P")
        width (sidekey "string_wid")
        cap-width (if (not plate-p)
                      ((lambda(/ fb-wids rnd-inc up dn)
                         (setq fb-wids (append (range 1.5 3 0.25) (range 3 5 0.5)))
                         (setq rnd-inc (if (< width 3) 0.25 0.5)
                               up (rnd-up width rnd-inc)
                               dn (rnd-dn width rnd-inc))
                         (if (or (<= (- up width) 0.0625)
                                 (> (- width dn) (+ (sidekey "string_web") 0.03)))
                             up
                             dn))))
        flange (if (not plate-p) (sidekey "string_flg"))
        depth (sidekey "string_dpt")
        starter-p (= starter_p "1")
        clayer (vla-getvar 'clayer)
        cdimstyle (vla-get-Name (vla-get-ActiveDimStyle acadDoc))
        main-dim-depth (- (* (if bseg 2 3) dimspace)))
  ;; find the main dim point
  (defun main-dim-pt () (displace-pt ins (list 0 main-dim-depth 0))) 
  ;; increment the current mark number
  (defun inc-current-mark ()
    (setq current-mark (strcat mark (to-string (inc! 'current-mark-no)))))
  
  (set-active-dimstyle "1xDECIMAL")
  (set-active-layer "SOLID")

  ;; end view
  (setq pt (displace-pt ins (list (- (* dimspace 2)) depth 0)))
  (if plate-p
      (add-rectangle blockObj (sidekey "string_wid") (- depth) pt)
      (progn
        (setq obj (add-shape blockObj pt material))
        (if (= side "l") (mirror&delete obj pt 'y))))
  (setq current-mark (strcat mark (to-string current-mark-no)))

  (if bseg
      (progn
        (fab-stringer--add-bottom-segment)
        (setq ins (mapcar '+ (nth 3 bseg) ins)
              main-dim-depth (- main-dim-depth dimspace))))

  (fab-stringer--add-mid-segment)
  (setq ins (nth 2 mseg-pgram)
        main-dim-depth (- main-dim-depth dimspace))

  (set-active-layer "SOLID")
  (if tseg
      (fab-stringer--add-top-segment))

  (set-active-layer clayer)
  (set-active-dimstyle cdimstyle)
  (insert-block modelSpace blkins blockObj))
(defun-r 'fab-stringer--add-detail)

(defun-q fab-stringer--add-bottom-segment ()
  "Adds the bottom stringer segment to BLOCKOBJ"
  (setq bseg (rezero-pts (add-bulges bseg) 1)
        bseg-pline (add-polyline blockObj ins bseg T))
  (if (not plate-p)
      (progn
        (fab-stringer--add-flange-line bseg-pline flange)
        (fab-stringer--add-flange-line bseg-pline (- depth flange))))
  (set (read (strcat side "-string-bot-pts"))
       (cdr (assoc 'string-bot-seg def)))
  ;; main dim
  (set-active-layer "DIM")
  (setq len (vla-get-Measurement
             (vla-AddDimRotated blockObj
                                (3dpt 0 0)
                                (3dpt (circular-nth -1 bseg))
                                (3dpt (main-dim-pt))
                                0)))
  (vla-AddDimRotated blockObj
                     (3dpt (circular-nth -1 bseg))
                     (3dpt (circular-nth -2 bseg))
                     (3dpt (displace-pt (circular-nth -1 bseg) (list 0 (- dimspace) 0)))
                     0)
  ;; material list
  (fab-stringer--add-segment-material (strcat side "-string-bot") len)
  ;; callout
  (add-part-detail-callout blockObj
                           (xd-value (strcat side "-string-bot") marks)
                           (main-dim-pt))
  (if (not plate-p) (add-to-list 'caps (list (strcat "b" side) cap-width depth)))
  (set-active-layer "SOLID")
  ;; for grouping like segments together
  (if (= side "l")
      (inc-current-mark)
      (if (or (/= l-string_mat r-string_mat)
              (not (equal l-string-bot-pts r-string-bot-pts)))
          (inc-current-mark))))
(defun-r 'fab-stringer--add-bottom-segment)

(defun-q fab-stringer--add-mid-segment ()
  "Adds the mid stringer segment to BLOCKOBJ"
  (setq mseg (rotate-pts (rezero-pts (cdr (assoc 'string-mid-seg def))
                                     (if (and (not starter-p)
                                              (not bseg)
                                              (not tseg))
                                         3 0))
                         (dtr (- pitch)))
        mseg-pgram (fab-stringer--parallelogramize mseg))
  (set (read (strcat side "-string-mid-pts"))
       (cdr (assoc 'string-mid-seg def)))
  (if starter-p
      (setq zero-pt (list (- (caar mseg) (caar mseg-pgram)) 0 0)
            mseg (displace-pts mseg zero-pt)
            mseg-pgram (displace-pts mseg-pgram zero-pt)))
  (vla-put-Color (add-polyline blockObj ins mseg-pgram T) 251)
  (setq mseg-pline (add-polyline blockObj ins mseg T))
  ;; flange lines
  (if (not plate-p)
      (progn
        (fab-stringer--add-flange-line mseg-pline flange)
        (fab-stringer--add-flange-line mseg-pline (- depth flange))))
  ;; mark and dims
  (set-active-layer "DIM")
  (setq mseg (displace-pts mseg ins)
        mseg-pgram (displace-pts mseg-pgram ins))
  ;; overall
  (setq len (vla-get-Measurement
             (vla-AddDimRotated blockObj
                                (3dpt (caddr mseg-pgram))
                                (3dpt (car mseg-pgram))
                                (3dpt 0 main-dim-depth)
                                0)))
  ;; material list
  (fab-stringer--add-segment-material (strcat side "-string-mid") len)
  ;; left miter
  (vla-AddDimRotated blockObj
                     (3dpt (car mseg-pgram))
                     (3dpt (cadr mseg-pgram))
                     (3dpt 0 (if starter-p (+ depth dimspace) (- dimspace)))
                     0)
  ;; right miter
  (vla-AddDimRotated blockObj
                     (3dpt (caddr mseg-pgram))
                     (3dpt (cadddr mseg-pgram))
                     (3dpt 0 (- dimspace))
                     0)
  ;; LTS cut dim
  (vla-AddDimRotated blockObj
                     (3dpt (car mseg-pgram))
                     (3dpt (circular-nth -1 mseg-pgram))
                     (3dpt 0 (- (* 2 dimspace)))
                     0)
  (if starter-p
      (progn
        (vla-AddDimRotated blockObj
                           (3dpt (car mseg-pgram))
                           (3dpt (car mseg))
                           (3dpt 0 (- dimspace))
                           0)
        (vla-AddDimRotated blockObj
                           (3dpt (car mseg-pgram))
                           (3dpt (cadr mseg))
                           (3dpt (- dimspace) 0)
                           (dtr 90))))
  (if (not tseg)
      (progn
        (vla-AddDimRotated blockObj
                           (3dpt (caddr mseg-pgram))
                           (3dpt ((if starter-p cadddr caddr) mseg))
                           (3dpt 0 (+ depth dimspace))
                           0)
        (vla-AddDimRotated blockObj
                           (3dpt (caddr mseg-pgram))
                           (3dpt (nth (if starter-p 4 3) mseg))
                           (3dpt (+ (caaddr mseg-pgram) dimspace) 0)
                           (dtr 90))))
  (foreach cap '(string-mid-cap-1
                 string-mid-cap-2
                 string-mid-cap-3
                 string-mid-cap-4)
           (if (setq cap (assoc cap def))
               (progn
                 (setq cap-slot (strcat (if (< (caadr cap) run) "b" "t") side)
                       cap (rezero-pts (cdr cap) 0))
                 (add-to-list 'caps (list cap-slot
                                          cap-width
                                          (abs (if (=f 0 (cadadr cap) 0.01)
                                                   (caadr cap)
                                                   (cadadr cap))))))))
  (add-part-detail-callout blockObj (xd-value (strcat side "-string-mid") marks) (main-dim-pt))
  (if (or (/= l-string_mat r-string_mat)
          (not (equal l-string-mid-pts r-string-mid-pts)))
      (inc-current-mark))

  (if (and stringers-eq-p (not plate-p))
      (add-mtext blockObj
                 (list (car ins) (- main-dim-depth 3))
                 "(1) AS SHOWN\n(1) OPP. HAND"
                 36
                 'topleft)))
(defun-r 'fab-stringer--add-mid-segment)

(defun-q fab-stringer--add-top-segment ()
  "Adds the top stringer segment to BLOCKOBJ"
  (setq tseg (rezero-pts (add-bulges (cdr tseg)) 0)
        tseg-pline (add-polyline blockObj ins tseg T)
        tseg (displace-pts tseg ins)
        ins (displace-pt ins (list 0 (- depth) 0)))
  (set (read (strcat side "-string-top-pts"))
       (cdr (assoc 'string-top-seg def)))
  ;; flange lines
  (if (not plate-p)
      (progn
        (fab-stringer--add-flange-line tseg-pline flange)
        (fab-stringer--add-flange-line tseg-pline (- depth flange))))
  ;; main dim
  (set-active-layer "DIM")
  (setq len (vla-get-Measurement
             (vla-AddDimRotated blockObj
                                (3dpt (car tseg))
                                (3dpt (caddr tseg))
                                (3dpt 0 main-dim-depth)
                                0)))
  ;; material list
  (fab-stringer--add-segment-material (strcat side "-string-top") len)
  ;; mark and dims
  (add-part-detail-callout blockObj
                           (xd-value (strcat side "-string-top") marks)
                           (main-dim-pt))
  (vla-AddDimRotated blockObj
                     (3dpt (car tseg))
                     (3dpt (cadddr tseg))
                     (3dpt 0 (- dimspace))
                     0)

  (if (not plate-p) (add-to-list 'caps (list (strcat "t" side) cap-width depth)))
  (if (= side "l")
      (inc-current-mark)
      (if (or (/= l-string_mat r-string_mat)
              (not (equal l-string-top-pts r-string-top-pts)))
          (inc-current-mark))))
(defun-r 'fab-stringer--add-top-segment)

(defun-q fab-stringer--parallelogramize (mseg-pts / i hpi prev this next plines ret)
  "Returns a points list representing the outer parallelogram of the middle segment"
  (declare (vars (mseg-pts list)))
  (setq i 0
        hpi (/ pi 2.0))
  (while (< i (length mseg-pts))
         (setq prev (circular-nth (1- i) mseg-pts)
               this (circular-nth i mseg-pts)
               next (circular-nth (1+ i) mseg-pts))
         (if (< (abs (- (int-ang prev this next) hpi)) 0.001)
             (progn
               (setq ret (reverse (cdr (reverse ret))))
               (add-to-list 'ret (inters (circular-nth (- i 2) mseg-pts)
                                         prev
                                         this
                                         next
                                         nil)))
             (add-to-list 'ret this))
         (inc! 'i))
  ret)
(defun-r 'fab-stringer--parallelogramize)

(defun-q fab-stringer--add-flange-line (seg-pline height / xline x-pts)
  "Adds a single flange line to a stringer segment in BLOCKOBJ"
  (declare (vars (seg-pline vla-object)
                 (height nil (numberp height))
                 (blockobj vla-object)))
  (setq xline (vla-AddXLine blockObj (3dpt 0 height) (3dpt 1 height))
        x-pts (vlax-safearray->list
               (vlax-variant-value
                (vla-IntersectWith seg-pline xline acExtendNone)))
        x-pts (list (list (car x-pts)
                          (cadr x-pts)
                          (caddr x-pts))
                    (cdddr x-pts)))
  (move-to-bottom
   (vla-put-Layer-r (add-polyline blockObj '(0 0 0) x-pts nil)
                    (if (= side "l") "HIDDEN" "SOLID"))
   blockObj)
  (vla-Delete xline))
(defun-r 'fab-stringer--add-flange-line)

(defun-q fab-stringer--add-segment-material (mark-id len / mat stringer-desc segs-match-p
                                                     seg-mark elt)
  "Adds material for a stringer segment to STRINGER-MATERIAL"
  (declare (vars (mark-id str)
                 (len nil (numberp len))
                 (material str)
                 (current-mark str)))
  (defun stringer-desc (material)
    (setq material (vl-string-subst " x " "x" material))
    (if (or (begins-with material "C")
            (begins-with material "MC"))
        (strcat "CHANNEL " material)
        (vl-string-subst "PLATE " "PL" material)))
  (setq mat (stringer-desc material)
        segs-match-p (and (begins-with mark-id "r")
                          (= l-string_mat r-string_mat)
                          (equal (symcat '(#mark-id "-pts"))
                                 (symcat '("l" #(substr mark-id 2) "-pts"))))
        seg-mark (if segs-match-p
                     (xd-value (strcat "l" (substr mark-id 2)) marks)
                     current-mark))
  (add-to-list 'marks (list mark-id seg-mark))
  (if (not segs-match-p)
      (add-to-list 'stringer-material
                   (list (list "mark" seg-mark)
                         (list "desc" mat)
                         (list "length" (rtos len 2 2))))))
(defun-r 'fab-stringer--add-segment-material)
