(set-file-docstring "Functions for stair fab drawings")

(psc-include
 '(
   "fab/part-details.lsp"
   "fab/stair-pan.lsp"
   "fab/stair-stringer.lsp"
   "shapes/line.lsp"
   "stair/details.lsp"
   "stair/section.lsp"
   "stair/stringer.lsp"
   "stair/tread.lsp"
   "stair/util.lsp"
   ))

(defun-q make-stair-fabs (stairs)
  "Creates all fab drawings for STAIRS

Essentially a composition of %make-generic-fab and %stair-fab--combine"
  (mapcar '(lambda(x) (make-generic-fab x 'stair))
          (stair-fab--combine stairs)))
(defun-r 'make-stair-fabs)

(defun-q stair-fab--combine (stairs)
  "Uses %fab-combine-like-data to combine STAIRS for fab"
  (fab-combine-like-data
   stairs
   (mapcar (compose '(lowercase to-string))
           '(tread_style
             rise_hgt
             rise_qty
             bot_blkout
             bot_cdist
             bot_conn
             bot_ext
             bot_hbolt
             bot_hbolt2
             bot_hdw
             bot_member
             bot_vbolt
             finish
             kickback
             l-nosing_bot
             l-nosing_mid
             l-nosing_top
             l-string_mat
             l-string_xtra_bex
             l-string_xtra_tex
             pan_mat
             quiet_tread
             r-nosing_bot
             r-nosing_mid
             r-nosing_top
             r-string_mat
             r-string_xtra_bex
             r-string_xtra_tex
             stn_mid_sku
             stn_tb_sku
             stn_style
             stn_tb_only
             top_blkout
             top_cdist
             top_conn
             top_ext
             top_hbolt
             top_hbolt2
             top_hdw
             top_member
             top_slab
             top_vbolt
             tread_depth
             tread_mat
             tread_thick
             width))))
(defun-r 'stair-fab--combine)

(defun-q make-stair-fab (/
                         current-mark
                         current-mark-no
                         marks
                         bl-cap-marks
                         br-cap-marks
                         tl-cap-marks
                         tr-cap-marks
                         stringers-eq-p
                         next-part-detail
                         main-view
                         )
  "Stair-specific function called by %make-generic-fab"
  (declare with-data)
  (setq stringers-eq-p (= stringers_eq "1")
        dir 1)
  (subst-key! "dir" 1 'data)

  ;; upsize pans for galvanized
  (if (and (= finish "Galvanized")
           (= pan_mat "14ga"))
      (setq pan_mat "12ga"))

  ;; stair shop considers both these pans exactly 1/8" for layout
  (if (member pan_mat '("14ga" "12ga"))
      (setq pan_thick 0.125))

  (setq current-mark-no 1)
  (stair-fab-process-stringers)
  (stair-fab--process-connections)
  (if (not (begins-with tread_style "6"))
      (stair-fab--process-pans))
  (if (stair-stn-p)
      (stair-fab--process-stn))

  ;; draw main views
  (setq main-view (stair-fab--main-view "l"))
  (insert-block modelSpace (3dpt 250 (if stringers-eq-p 100 175)) main-view)
  (if (not stringers-eq-p)
      (insert-block modelSpace (3dpt 250 50) (stair-fab--main-view "r")))

  ;; QT takeoff based on main view block
  (if (= quiet_tread "1")
      (stair-fab--process-qt main-view))
  
  (stair-fab--draw-top-view (list 250 (if stringers-eq-p 250 300)))
  (stair-fab--draw-square-detail)

  ;; update assemblies table title
  (vla-SetCellState ass-table 0 0 acCellStateNone)
  (vla-SetText ass-table 0 0
    (strcat "Assembly: "
            (cond ((begins-with tread_style "1") "CF ")
                  ((begins-with tread_style "2") "CP ")
                  ((begins-with tread_style "4") "PC ")
                  ((begins-with tread_style "6") "GT "))
            "Stair "
            (to-string tread_qty)
            "T")))
(defun-r 'make-stair-fab)

(defun-q stair-fab--process-connections (/ material hardware tmp)
  "Adds material, hardware, and part details for connections"
  (declare with-data)
  (mapcar '(lambda(conn / mh)
             (setq mh (attempt-apply (read (strcat "material-"  conn)) nil))
             (mapcar '(lambda(x) (add-to-list 'material x)) (car mh))
             (mapcar '(lambda(x) (add-to-list 'hardware x)) (cadr mh)))
          (list bot_conn_full top_conn_full))
  (if material
      (progn
        ;; combine, sort, flesh out, and add to table
        (setq material (combine-like-data material '("desc" "length" "width" "note" "detail") "id"))
        (sort! 'material
               '(lambda(a b)
                  (setq a (xd-value "desc" a)
                        b (xd-value "desc" b))
                  (cond ((begins-with a "BR DECKING")
                         nil)
                        ((begins-with b "BR DECKING")
                         T)
                        (T (< a b)))))
        (dec! 'current-mark-no)
        (mapcar! '(lambda(x / mark)
                    (setq mark (strcat piece-mark (to-string (inc! 'current-mark-no))))
                    (foreach id (xd-value "id" x)
                             (add-to-list 'marks (list id mark)))
                    (add-to-list 'x (list "mark" mark)))
                 'material)
        (table-add-data mat-table (cons nil material))
        ;; draw part details
        (foreach m material
                 (if (setq tmp (xd-value "detail" m))
                     (add-to-list 'part-details
                                  (vla-get-Name (part-detail (cadr (assoc "mark" m)) tmp)))))
        (foreach d part-details
                 (insert-block modelspace next-part-detail d)
                 (setq next-part-detail (displace-pt next-part-detail '(0 -70 0))))))
  (if hardware
      (table-add-data hdw-table (combine-like-data hardware '("desc" "note") nil))))
(defun-r 'stair-fab--process-connections)

(defun-q stair-fab--process-pans ( / pan-slots pans)
  "Adds pans to material list and draws their details"
  ;; draw pans details
  (setq pan-slots (list (list (- (* 10 dimspace)) dimspace 0)
                        (list (- (* 21 dimspace)) dimspace 0)
                        (list (- (* 10 dimspace)) (* 8 dimspace) 0)
                        (list (- (* 21 dimspace)) (* 8 dimspace) 0)
                        (list (- (* 10 dimspace)) (* 15 dimspace) 0)
                        (list (- (* 21 dimspace)) (* 15 dimspace) 0))
        pan-slots (mapcar '(lambda(x) (displace-pt x (list dwg-width 54 0))) pan-slots)
        pans (mapcar 'process-pan-def
                      (vl-sort (calc-stair-pans)
                               (sort-by-key "mat" sort-strings-asc)))
        pans (vl-sort pans (sort-by-key "num"
					(lambda (a b) (< (atoi a)
                                                         (atoi b))))))
  (table-add-data mat-table (cons nil pans))
  (foreach pan pans
           (if (not (member (xd-value "sty" pan) '("v" "h")))
               (draw-pan-detail (pop! 'pan-slots) pan))))
(defun-r 'stair-fab--process-pans)

(defun-q stair-fab--draw-square-detail ( / blockObj text-size halftread pitch-ins)
  "Draws the square setting detail"
  (declare with-data)
  (setq	blockObj (define-block "square-detail" nil)
	text-size (vla-getvar 'textsize)
	halftread (/ tread_depth 2.0)
  	pitch-ins (list (- halftread) (* halftread (tan (dtr pitch))) 0))

  (add-polyline blockObj
		'(0 0 0)
		(#! '((#(- tread_depth) 0 0)
		      (0 0 0)
		      (0 #rise_hgt 0)))
		nil)
  (add-mtext blockObj
	     (list (- halftread) (- (/ text-size 2.0)) 0)
	     (frac tread_depth)
	     1.0
	     'topcenter)
  (add-mtext blockObj
	     (list (/ text-size 2.0) (/ rise_hgt 2.0) 0)
	     (rtos rise_hgt 2 2)
	     12
	     'middleleft)
  (vla-Rotate-r
   (add-mtext blockObj
	      pitch-ins
	      (strcat (rtos pitch 2 2) "%%d")
	      1.0
	      'bottomcenter)
   (3dpt pitch-ins)
   (dtr pitch))
  (insert-block modelspace '(401 365 0) blockObj))
(defun-r 'stair-fab--draw-square-detail)


(defun-q stair-fab--main-view (side / blockObj nose-pts rise-hgt fnose mnose-i lnose
                              xtra-bex xtra-tex string-bex top-ff top-dim-line
                              left-dim-line nosing-line nosing-top cr-bend-pt
                              top-stringer-line)
  "Returns a stair fab main view block for SIDE"
  (declare (vars (side str (member side '("l" "r"))))
           with-data)
  (setq blockObj (define-block (strcat side "mainview") nil)
        nose-pts (calc-nose-pts)
        fnose (car nose-pts)
        mnose-i (if (> rise_qty 1) (1- (/ (length nose-pts) 2)))
        lnose (last nose-pts)
        top-ff (cadr lnose)
        nosing-line (pt-slope (/ rise_hgt tread_depth) (car fnose) (cadr fnose))
        nosing-top (sidekey "nosing_top")
        top-stringer-line (line-offset nosing-line (sidekey "nosing_mid"))
        cr-bend-pt (inters (list 0 (line-val top-stringer-line 0))
                           (list 1 (line-val top-stringer-line 1))
                           (list 0 (+ nosing-top (cadr lnose)))
                           (list 1 (+ nosing-top (cadr lnose)))
                           nil)
        xtra-bex (sidekey "string_xtra_bex")
        xtra-tex (sidekey "string_xtra_tex")
        string-bex (sidekey "string_bex")
        rise-hgt (rtos rise_hgt 2 2)
        top-dim-line (+ (* 2 dimspace) top-ff)
        left-dim-line (- 0 xtra-bex (* (if (>= rise_qty 3) 3 4)
                                       dimspace)))
  (if (or (/= xtra-tex 0) (/= xtra-bex 0))
      (setq top-dim-line (+ top-dim-line dimspace)))
  (setq top-dim-line2 (- top-dim-line dimspace))
  (if (< rise_qty 5)
      (setq top-dim-line (+ top-dim-line dimspace)))
  
  (stair-fab--main-view-add-stair (if (= side "l") "SOLID" "HIDDEN"))
  (stair-fab--main-view-add-stringer-callouts)
  (stair-fab--main-view-add-h-dims)
  (stair-fab--main-view-add-v-dims)
  (stair-fab--main-view-add-p-dims)
  (call-out-pans)
  ;; view title
  (add-mtext blockObj
             (list (+ bot_ext bot_cdist (/ run 2.0))
                   (- (* 3 dimspace)))
             (strcat "{\\L" (if (= side "l") "SECTION A" "VIEW B") "}")
             24
             'bottomcenter)
  blockObj)
(defun-r 'stair-fab--main-view)

(defun-q stair-fab--main-view-add-stair (parts-layer)
  "Adds the stair, tread/riser, and connection geometry to the main view"
  (declare (vars (parts-layer str))
           with-data)
  (add-stair-stringer '(seg flg cap) blockObj side "SOLID" (if (= side "l") "HIDDEN" "SOLID"))
  (if (= quiet_tread "1")
      (add-stair-qtread blockObj "Stair Quiet Tread"))
  (add-stair-treads blockObj parts-layer)
  (add-stair-conc blockObj)
  (if (stair-stn-p)
      (add-stair-stn blockObj "STN"))
  (foreach tb '("t" "b")
           (draw_stair_block_connection blockObj tb T parts-layer)))
(defun-r 'stair-fab--main-view-add-stair)

(defun-q stair-fab--main-view-add-h-dims (/ add)
  "Adds horizontal dimensions to the main view"
  (defun add (pt1 pt2 y)
    (vla-AddDimRotated blockObj (3dpt pt1) (3dpt pt2) (3dpt 0 y) 0))
  ;; overall run
  (if (> rise_qty 1)
      (apply '(lambda (fnose lnose tread_qty top-dim-line)
                (add fnose lnose top-dim-line)
                (add-mtext blockObj
                           (list (+ (car fnose) (/ (- (car lnose) (car fnose)) 2.0))
                                 (1- top-dim-line))
                           (strcat "{\\T0.7;"
                                   (to-string tread_qty)
                                   "T@"
                                   (to-string (frac tread_depth))
                                   "}")
                           1
                           'topcenter))
             (if (begins-with tread_style "4")
                 ((lambda (/ penultim8nose dim-line)
                    (setq penultim8nose (displace-pt lnose (list (- 0 tread_depth -0.25)
                                                                 (- 0 rise_hgt tread_thick)))
                          dim-line (+ top-dim-line dimspace))
                    (add penultim8nose lnose dim-line)
                   (list (displace-pt fnose (list 0.25 (- tread_thick)))
                         penultim8nose
                         (1- tread_qty)
                         dim-line)))
                 (list fnose lnose tread_qty top-dim-line))))
  (stair-fab--main-view-add-h-dims-top)
  (apply 'stair-fab--main-view-add-h-dims-bot
         (if (begins-with tread_style "4")
             (list (displace-pt fnose (list 0.25 (- tread_thick)))
                   (+ top-dim-line dimspace))
             (list fnose top-dim-line))))
(defun-r 'stair-fab--main-view-add-h-dims)

(defun-q stair-fab--main-view-add-h-dims-top (/ string-edge string-top)
  "Adds horizontal dims for top nosing/stringer edge in main view"
  (setq string-edge (+ (car lnose) top_cdist top_ext)
        string-top (+ top-ff nosing-top))
  (if (= xtra-tex 0)
      ;; single top extension
      (add lnose (list string-edge string-top) top-dim-line)
      (progn
        ;; overall stringer extension
        (add lnose (list (+ string-edge xtra-tex) string-top) top-dim-line)
        ;; platform extension
        (if (> rise_qty 4)
            (add (list string-edge top-ff) lnose top-dim-line2)
            (add lnose (list string-edge top-ff) (+ top-dim-line2 dimspace)))
        ;; stringer extra
        (add (list string-edge top-ff)
             (list (+ string-edge xtra-tex) string-top)
             top-dim-line2))))
(defun-r 'stair-fab--main-view-add-h-dims-top)

(defun-q stair-fab--main-view-add-h-dims-bot (fnose top-dim-line / string-edge string-top)
  "Adds dims for bottom nosing/stringer edge in main view"
  (setq string-edge (- (car fnose) bot_cdist bot_ext (if (begins-with tread_style "4") 0.25 0)))
  (defun string-top (x)
    (if (<= string-bex tread_depth)
        (line-val top-stringer-line x)
        (sidekey "nosing_bot")))
  (if (= xtra-bex 0)
      ;; single bot extension
      (add fnose (list string-edge (string-top 0)) top-dim-line)
      (progn
        ;; overall stringer extension
        (add fnose
             (list (- string-edge xtra-bex)
                   (string-top (- string-bex)))
             top-dim-line)
        ;; platform extension
        (if (= starter_p "0")
            (progn
              (if (> rise_qty 4)
                  (add (list string-edge 0) fnose top-dim-line2)
                  (add fnose (list string-edge 0) (+ top-dim-line2 dimspace)))
              ;; stringer extra
              (add (list string-edge 0)
                   (list (- string-edge xtra-bex)
                         (string-top (- string-bex)))
                   top-dim-line2))))))
(defun-r 'stair-fab--main-view-add-h-dims-bot)

(defun-q stair-fab--main-view-add-v-dims ()
  "Adds vertical dimensions to main view"
  (if (begins-with tread_style "4")
      (stair-fab--main-view-add-v-dims-400)
      (if (and (begins-with tread_style "1")
               (/= stn_style "None"))
          (if (= stn_tb_only "0")
              (stair-fab--main-view-add-v-dims-100-stn)
              (stair-fab--main-view-add-v-dims-100-stn-tb))
          (stair-fab--main-view-add-v-dims-std))))
(defun-r 'stair-fab--main-view-add-v-dims)

(defun-q stair-fab--main-view-add-rise-tag (ins rise_qty / tag-str)
  "Adds rise tag at INS in main view"
  (setq tag-str (strcat (to-string rise_qty) "R@" rise-hgt))
  (if (< rise_qty 3)
      (add-mtext blockObj ins tag-str 1 'middleleft)
      (vla-Rotate (add-mtext blockObj ins tag-str 1 'topcenter)
                  (3dpt ins)
                  (dtr 90))))
(defun-r 'stair-fab--main-view-add-rise-tag)

(defun-q stair-fab--main-view-add-v-dims-std ()
  "Adds standard vertical stair fab dimensions"
  ;; overall rise dim
  (detail-side-add-dim-left (3dpt (if (= nosing-top 0)
                                      cr-bend-pt
                                      lnose))
                            0
                            nil)
  ;; rise tag
  (stair-fab--main-view-add-rise-tag (list (1+ left-dim-line)
                                           (/ top-ff 2.0))
                                     rise_qty)
  ;; top curb reveal dim
  (if (> nosing-top 0)
      (vla-AddDimRotated blockObj
                         (3dpt lnose)
                         (3dpt cr-bend-pt)
                         (3dpt left-dim-line (cadr cr-bend-pt))
                         (dtr 90))))
(defun-r 'stair-fab--main-view-add-v-dims-std)

(defun-q stair-fab--main-view-add-v-dims-100-stn (/ add fn ln dim-line)
  "Adds vertical dimensions for pan-fill w/ STN at all nosings"
  (defun add (pt1 pt2)
    (vla-AddDimRotated blockObj (3dpt pt1) (3dpt pt2) (3dpt dim-line 0) (dtr 90)))
  (setq fn (displace-pt fnose (list 0 (- stn_thick)))
        ln (displace-pt lnose (list 0 (- stn_thick)))
        dim-line (- left-dim-line dimspace))
  (add fn '(0 0))
  (add fn ln)
  (stair-fab--main-view-add-rise-tag (list (1+ dim-line)
                                           (+ (/ (- top-ff rise_hgt stn_thick) 2.0) rise_hgt))
                                     (1- rise_qty))
  (add ln cr-bend-pt))
(defun-r 'stair-fab--main-view-add-v-dims-100-stn)

(defun-q stair-fab--main-view-add-v-dims-100-stn-tb (/ add n1 n2 n3 n4 far-dim-line base-dim-line)
  "Adds vertical dimensions for pan-fill w/ STN at top and bottom only"
  (defun add (pt1 pt2 line)
    (vla-AddDimRotated blockObj (3dpt pt1) (3dpt pt2) (3dpt line 0) (dtr 90)))
  (setq n2 (displace-pt fnose (list tread_depth rise_hgt))
        n1 (displace-pt fnose (list 0 (- stn_thick)))
        n3 (displace-pt lnose (list (- tread_depth) (- rise_hgt)))
        n4 (displace-pt lnose (list 0 (- stn_thick)))
        base-dim-line left-dim-line
        far-dim-line (- base-dim-line (* 2 dimspace)))
  (add n1 '(0 0) far-dim-line)
  (add n1 n2 base-dim-line)
  (add n2 n3 far-dim-line)
  (stair-fab--main-view-add-rise-tag (list (1+ far-dim-line)
                                           (+ (/ (* rise_hgt (- rise_qty 3)) 2.0)
                                              (* 2 rise_hgt)))
                                     (- rise_qty 3))
  (add n4 n3 base-dim-line)
  (add n4 cr-bend-pt far-dim-line))
(defun-r 'stair-fab--main-view-add-v-dims-100-stn-tb)

(defun-q stair-fab--main-view-add-v-dims-400 (/ add n1 n2 n3 base-dim-line far-dim-line)
  "Adds vertical dimensions for precast"
  (defun add (pt1 pt2 line)
    (vla-AddDimRotated blockObj (3dpt pt1) (3dpt pt2) (3dpt line 0) (dtr 90)))
  (setq n1 (displace-pt fnose (list 0.25 (- tread_thick)))
        n2 (displace-pt lnose (list (- 0 tread_depth -0.25)
                                    (- 0 rise_hgt tread_thick)))
        n3 (if (= stn_style "None")
               lnose
               (displace-pt lnose (list 0 (- stn_thick))))
        base-dim-line left-dim-line
        far-dim-line (- base-dim-line dimspace))
  (add n1 '(0 0) far-dim-line)
  (add n1 n2 far-dim-line)
  (stair-fab--main-view-add-rise-tag (list (1+ far-dim-line)
                                           (+ (/ (- flight_hgt (* 2 rise_hgt))
                                                 2.0)
                                              (- rise_hgt tread_thick)))
                                     (- rise_qty 2))
  (add n2 n3 far-dim-line)
  (add n3 cr-bend-pt base-dim-line))
(defun-r 'stair-fab--main-view-add-v-dims-400)

(defun-q stair-fab--main-view-add-p-dims (/ rpitch)
  "Adds pitch dims to main view"
  (setq rpitch (dtr pitch))
  (apply
   '(lambda (fnose lnose mnose mnose-next)
      ;; pitch dims
      (if (> rise_qty 1)
          (progn
            (set-active-dimstyle "1xDECIMAL")
            (vla-AddDimRotated blockObj
                               (3dpt mnose)
                               (3dpt mnose-next)
                               (3dpt (car mnose)
                                     (line-val (line-offset nosing-line 6)
                                               (car mnose)))
                               rpitch)
            (if (> rise_qty 2)
                (vla-AddDimRotated blockObj
                                   (3dpt fnose)
                                   (3dpt lnose)
                                   (3dpt (car fnose)
                                         (line-val (line-offset nosing-line 12)
                                                   (car fnose)))
                                   rpitch))))
      (set-active-dimstyle "1xFRACTIONAL")
      ;; mid nosing depth dim
      (vla-AddDimRotated blockObj
                         (3dpt mnose-next)
                         (3dpt (if (> rise_qty 2)
                                   (list (+ (car mnose-next) 5)
                                         (line-val top-stringer-line (+ (car mnose-next) 5)))
                                   cr-bend-pt))
                         (3dpt (+ (car mnose-next) 5)
                               (line-val top-stringer-line (+ (car mnose-next) 5)))
                         (- (- (dtr 90) rpitch))))
   (displace-pts
    (list (if (and (begins-with tread_style "1")
                   (= stn_tb_only "1"))
              (nth 1 nose-pts)
              fnose)
          (if (or (begins-with tread_style "4")
                  (and (begins-with tread_style "1")
                       (= stn_tb_only "1")))
              (circular-nth -2 nose-pts)
              lnose)
          (nth (?n mnose-i) nose-pts)
          (nth (1+ (?? mnose-i -1)) nose-pts))
    (cond ((begins-with tread_style "4") (list 0.25 (- tread_thick)))
          ((and (begins-with tread_style "1")
                (/= stn_style "None")
                (= stn_tb_only "0"))
           (list 0 (- stn_thick)))
          (T '(0 0))))))
(defun-r 'stair-fab--main-view-add-p-dims)

(defun-q stair-fab--main-view-add-stringer-callouts ()
  "Adds stringer callouts to main view

This only adds the main segment and any haunches resulting from stringer extension
(i.e. not a haunch connection type). Standard haunch callouts are brought in by
%haunch-stringer-callout"
  ;; main stringer callout (details locate haunch segment callouts)
  (add-mleader blockObj
               (list (car (nth (?n mnose-i) nose-pts))
                     (line-val (line-offset nosing-line (- (sidekey "nosing_mid")
                                                           (sidekey "string_dpt")))
                               (car (nth (?n mnose-i) nose-pts))))
               '((0 0)
                 (5 -5)
                 (7 -5))
               (xd-value (strcat side "-string-mid") marks))

  ;; stringer extension callouts
  (if (and (assoc (strcat side "-string-bot") marks)
	   (not (wcmatch bot_conn "*haunch*")))
      (add-mleader blockObj
		   (list (+ (- (sidekey "string_xtra_bex")) 4)
			 (- (- (sidekey "string_dpt") (sidekey "nosing_bot")))
			 0)
		   '((0 0 0)
		     (0 -6 0)
		     (-1 -6 0))
		   (xd-value (strcat side "-string-bot") marks)))
  (if (and (assoc (strcat side "-string-top") marks)
	   (not (wcmatch top_conn "*haunch*")))
      (add-mleader blockObj
		   (list (+ (car lnose) top_cdist top_ext (sidekey "string_xtra_tex") -4)
			 (- flight_hgt (- (sidekey "string_dpt") (sidekey "nosing_top")))
			 0)
		   '((0 0 0)
		     (0 -6 0)
		     (1 -6 0))
		   (xd-value (strcat side "-string-top") marks))))
(defun-r 'stair-fab--main-view-add-stringer-callouts)

(defun-q stair-fab--process-stn (/ stn-mat skus add-len stns qtys)
  "Adds STN to material list"
  (defun stn-mat (def qty / desc)
    (setq desc (get-stn-desc-by-sku (cadr def)))
    (list (list "mark" (strcat "STN" (car def)))
          (list "qty" (to-string qty))
          (list "desc" desc)
          (list "length" (frac (distof (caddr def))))
          (list "note" (if (begins-with tread_style "1") "SL" ""))))

  (setq skus (list stn_tb_sku)
        add-len (macro-expand '(lambda(x) (list x #(- btwn_stringers 0.125)))))
  (if (and (= stn_tb_only "0")
           (/= stn_tb_sku stn_mid_sku))
      (add-to-list 'skus stn_mid_sku))
  (mapcar! add-len 'skus)
  (setq stns (mapcar 'process-generic-def
                     skus
                     (lst* (strcat (xd-value "dir" job-info) "/stn.csv") (length skus)))
        qtys (list (if (or (= stn_tb_only "1")
                           (cdr stns))
                       2
                       rise_qty)
                   (- rise_qty 2)))
  (table-add-data mat-table (mapcar 'stn-mat stns qtys)))
(defun-r 'stair-fab--process-stn)

(defun-q stair-fab--process-qt (stair-block / counts filename takeoff)
  "Takes of QT from STAIR-BLOCK, processes it, and adds it to the material list"
  (declare (vars (stair-block vla-object)))
  (setq filename (strcat (xd-value "dir" job-info) "/qt.csv")
        takeoff (stair-block-qt-takeoff stair-block)
        counts (count-items takeoff)
        takeoff (uniquify takeoff)
        takeoff (mapcar 'process-generic-def
                        takeoff
                        (lst* filename (length takeoff)))
        takeoff (vl-sort takeoff '(lambda(a b) (< (car a) (car b))))
        takeoff (mapcar '(lambda(q)
                             (list (list "mark" (strcat "QT" (car q)))
                                   (list "qty" (xd-value (mapcar 'distof (cdr q)) counts))
                                   (list "desc" "QUIET TREAD 1/8")
                                   (list "width" (frac (distof (cadr q))))
                                   (list "length" (frac (distof (caddr q))))))
                          takeoff))
  (table-add-data mat-table takeoff))
(defun-r 'stair-fab--process-qt)

(defun-q stair-fab--draw-top-view (ins / clayer blockObj)
  "Draws the top view at INS"
  (declare (vars (ins list (point-p ins)))
           with-data)
  (setq blockObj (define-block "topview" nil)
        clayer (vla-getvar 'clayer))
  (set-active-layer "SOLID")

  (stair-fab--draw-top-view-stringers)
  (stair-fab--draw-top-view-treads)
  (stair-fab--draw-top-view-callouts)
  (stair-fab--draw-top-view-dims)
  (stair-fab--draw-top-view-connections)
  (set-active-layer clayer)
  (insert-block modelSpace ins blockObj))
(defun-r 'stair-fab--draw-top-view)

(defun-q stair-fab--draw-top-view-stringers (/ stringer-segs left-dim-line right-dim-line)
  "Adds the stringers to the top view"
  ;; return the callout text for all stringer segments
  (defun stringer-segs (lr)
    (if stringers-eq-p (setq lr "l"))
    (strjoin (strip-nil (mapcar '(lambda(k) (xd-value (strcat lr "-string-" k) marks))
                                '("bot" "mid" "top")))
             "\n"))

  (setq left-dim-line (- 0 (max l-string_xtra_bex r-string_xtra_bex) (* 5 dimspace))
        right-dim-line (+ (max l-string_xtra_tex r-string_xtra_tex) (* 2 dimspace)))

  ;;;;;;;;;
  ;; right
  ;;;;;;;;;
  (add-rectangle blockObj
                 (+ r-string_bex bot_cdist run top_cdist r-string_tex)
                 r-string_wid
                 (list (- r-string_xtra_bex) 0))
  ;; main stringer callout
  (vla-put-Layer (add-mleader blockObj
                              (list (+ bot_cdist bot_ext (/ run 2.0)) 0)
                              '((0 0) (0 -6) (1 -6))
                              (stringer-segs "r"))
                 "DIM")
  (if (not (begins-with r-string_mat "P"))
      (progn
        ;; bottom caps callout
        (vla-put-Layer (add-mleader blockObj
                                    (list 0 (/ r-string_wid 2.0))
                                    (list (list (- r-string_xtra_bex) 0)
                                          (list left-dim-line 0)
                                          (list (- left-dim-line 2) 0)
                                          (list (- left-dim-line 4) 0)
                                          (list (- left-dim-line 4) -6)
                                          (list (- left-dim-line 5) -6))
                                    (strjoin br-cap-marks "\n"))
                       "DIM")
        ;; top caps callout
        (vla-put-Layer (add-mleader blockObj
                                    (list full_run (/ r-string_wid 2.0))
                                    (list (list r-string_xtra_tex 0)
                                          (list right-dim-line 0)
                                          (list (+ right-dim-line 2) 0)
                                          (list (+ right-dim-line 4) 0)
                                          (list (+ right-dim-line 4) -6)
                                          (list (+ right-dim-line 5) -6))
                                    (strjoin tr-cap-marks "\n"))
                       "DIM")))
  ;;;;;;;;
  ;; left
  ;;;;;;;;
  (add-rectangle blockObj
                 (+ l-string_bex bot_cdist run top_cdist l-string_tex)
                 l-string_wid
                 (list (- l-string_xtra_bex) (- width l-string_wid) 0))
  ;; main callout
  ((lambda(/ text y)
     (setq text (stringer-segs "l")
           y (* 6 (length (strsplit text "\n"))))
     (vla-put-Layer (add-mleader blockObj
                                 (list (+ bot_cdist bot_ext (/ run 2.0)) width)
                                 (#! '((0 0) (0 #y) (1 #y)))
                                 text)
                    "DIM")))
  (if (not (begins-with l-string_mat "P"))
      (progn
        ;; bottom caps callout
        (vla-put-Layer (add-mleader blockObj
                                    (list 0 (- width (/ l-string_wid 2.0)))
                                    (list (list (- l-string_xtra_bex) 0)
                                          (list left-dim-line 0)
                                          (list (- left-dim-line 2) 0)
                                          (list (- left-dim-line 4) 0)
                                          (list (- left-dim-line 4) 6)
                                          (list (- left-dim-line 5) 6))
                                    (strjoin bl-cap-marks "\n"))
                       "DIM")
        ;; top caps callout
        (vla-put-Layer (add-mleader blockObj
                                    (list full_run (- width (/ l-string_wid 2.0)))
                                    (list (list l-string_xtra_tex 0)
                                          (list right-dim-line 0)
                                          (list (+ right-dim-line 2) 0)
                                          (list (+ right-dim-line 4) 0)
                                          (list (+ right-dim-line 4) 6)
                                          (list (+ right-dim-line 5) 6))
                                    (strjoin tl-cap-marks "\n"))
                       "DIM"))))
(defun-r 'stair-fab--draw-top-view-stringers)

(defun-q stair-fab--draw-top-view-treads ()
  "Adds the treads and pan extensions to the top view"
  ;; risers
  (setq x (+ bot_ext bot_cdist)
        y (- width l-string_wid r-string_wid))
  (repeat rise_qty
          (add-polyline blockObj
                        (list x r-string_wid 0)
                        (list '(0 0 0)
                              (list 0 y 0))
                        nil)
          (setq x (+ x tread_depth)))
  ;; pan extensions
  (if bot_pan_ext
      (add-polyline blockObj
                    (list 0 r-string_wid 0)
                    (list '(0 0 0)
                          (list (- bot_pan_ext) 0 0)
                          (list (- bot_pan_ext) btwn_stringers 0)
                          (list 0 btwn_stringers 0))
                    nil))
  (if top_pan_ext
      (add-polyline blockObj
                    (list full_run r-string_wid 0)
                    (list '(0 0 0)
                          (list top_pan_ext 0 0)
                          (list top_pan_ext btwn_stringers 0)
                          (list 0 btwn_stringers 0))
                    nil)))
(defun-r 'stair-fab--draw-top-view-treads)

(defun-q stair-fab--draw-top-view-callouts (/ add-view-callout)
  "Adds view callouts to the top view"
  (defun add-view-callout (ins text / )
    (vla-SetWidth (add-polyline blockObj
                                ins
                                '((0 9 0)
                                  (0 5 0)
                                  (0 -2 0))
                                nil)
                  0
                  0
                  2)
    (add-polyline blockObj
                  ins
                  '((-2 0 0)
                    (18 0 0))
                  nil)
    (add-mtext blockObj
               (list (+ 2 (car ins))
                     (+ 5 (cadr ins))
                     0)
               text
               1
               'bottomleft))

  (set-active-layer "DIM")
  (add-view-callout (list (- (* 2 dimspace)) (/ width 2)) "A")
  (if (= stringers_eq "0")
      (add-view-callout (list (- (* 2 dimspace)) (- (* 2 dimspace))) "B")))
(defun-r 'stair-fab--draw-top-view-callouts)

(defun-q stair-fab--draw-top-view-dims (/ xtra-bex)
  "Adds dimensions to the top view"
  (setq xtra-bex (max l-string_xtra_bex r-string_xtra_bex))
  ;; right stringer
  (vla-AddDimRotated blockObj
                     (3dpt (min (- r-string_xtra_bex)
                                (- (?f bot_pan_ext)))
                           r-string_wid)
                     (3dpt (- r-string_xtra_bex) 0)
                     (3dpt (- 0 xtra-bex (* 4 dimspace)) 0)
                     (dtr 90))
  ;; between stringers
  (vla-AddDimRotated blockObj
                     (3dpt (min (- r-string_xtra_bex)
                                (- (?f bot_pan_ext)))
                           r-string_wid)
                     (3dpt (min (- l-string_xtra_bex)
                                (- (?f bot_pan_ext)))
                           (+ r-string_wid btwn_stringers))
                     (3dpt (- 0 xtra-bex (* 4 dimspace)) 0)
                     (dtr 90))
  ;; left stringer
  (vla-AddDimRotated blockObj
                     (3dpt (min (- l-string_xtra_bex)
                                (- (?f bot_pan_ext)))
                           (+ r-string_wid btwn_stringers))
                     (3dpt (- l-string_xtra_bex)
                           width)
                     (3dpt (- 0 xtra-bex (* 4 dimspace)) 0)
                     (dtr 90))
  ;; overall
  (vla-AddDimRotated blockObj
                     (3dpt (- r-string_xtra_bex) 0)
                     (3dpt (- l-string_xtra_bex)
                           width)
                     (3dpt (- 0 xtra-bex (* 5 dimspace)) 0)
                     (dtr 90))

  ;; pan extension dims
  (if (and bot_pan_ext
           (> bot_pan_ext 0))
      (vla-AddDimRotated blockObj
                         (3dpt 0 width)
                         (3dpt (- bot_pan_ext) (- width l-string_wid))
                         (3dpt 0 (+ width (* 2 dimspace)))
                         0))
  (if (and top_pan_ext
           (> top_pan_ext 0))
      (vla-AddDimRotated blockObj
                         (3dpt full_run width)
                         (3dpt (+ full_run top_pan_ext) (- width l-string_wid))
                         (3dpt full_run (+ width (* 2 dimspace)))
                         0)))
(defun-r 'stair-fab--draw-top-view-dims)

(defun-q stair-fab--draw-top-view-connections (/ add-connection)
  "Adds connections to the top view"
  (defun add-connection (container tb / blockObj conn ins clayer)
    (if (begins-with tb "b")
        (setq conn bot_conn_full
              ins (3dpt 0 r-string_wid))
        (setq conn top_conn_full
              ins (3dpt full_run
                        r-string_wid)))
    (setq blockObj (p-blk (strcat tb "-conn"))
          clayer (vla-getvar 'clayer))
    (if (attempt-apply (read (strcat "detail-top-" conn)) nil)
        (move-to-bottom (insert-block container
                                      ins
                                      blockObj)
                        container))
    (set-active-layer clayer))
  (foreach tb '("t" "b")
           (add-connection blockObj tb)))
(defun-r 'stair-fab--draw-top-view-connections)
