(set-file-docstring "Functions for wall rail fabs")

(psc-include '("rail/util.lsp"))

(defun-q calc-wall-rail-fab (stair-data / calc ip op ret)
  "Return a list of wall rail definitions"
  (defun calc (side other-p
               / ab lvl-p cut-len splice-p header brackets
               bend-line-cut-length odd-brackets-p top-ab bot-ab subst-seq bracket-cols)

    ;; return the header with suffix concatenated to the end of the sequence
    (defun subst-seq (header suffix / ret)
      (subst-key "sequence" (strcat (xd-value "sequence" header) suffix) header))

    (if (and (= (sidekey-rail "top_dloop") "0")
             (= (sidekey-rail "bot_dloop") "0"))
        (progn
          (setq ab (fix (rnd-up (/ (+ run tread_depth) (cos (dtr pitch))) 1))
                lvl-p (= rail_lvl-off "1")
                cut-len (+ ab (if lvl-p 54 37))
                splice-p (>= ab 143)
                header (#! '(("mark" "WR")
                             ("stair_number" #number)
                             ("level" #level)
                             ("sequence" #(rail-sequence side))
                             ("ascend" #(rail-ascend side))
                             ("pitch" #(rtos pitch 2 2))
                             ("length_at_pitch" #ab)))
                brackets (wall-rail-brackets ab)
                bracket-cols '("bracket_1" "bracket_2" "bracket_3" "bracket_4"
                               "bracket_5" "bracket_6" "bracket_7"))
          (if splice-p
              (progn
                ;; return cut length as float
                (defun cut-len (lines)
                  (distof (last (pipe-bend-lines lines))))

                (setq odd-brackets-p (not (zerop (rem (length brackets) 2)))
                      top-ab (+ (/ ab 2.0) (if odd-brackets-p 10 0))
                      bot-ab (- (/ ab 2.0) (if odd-brackets-p 10 0)))
                (list
                 (cons (list "wall-rail-top" finish)
                       (append (subst-seq header "-1")
                               (list
                                (list "cut_length"
                                      (ceil (cut-len (#! '(7 90 15.4375 #pitch #top-ab))))))
                               (mapcar 'list bracket-cols
                                 (take (+ (/ (length brackets) 2)
                                          (if odd-brackets-p 1 0))
                                       brackets))))
                 (cons (list (strcat "wall-rail-bot-" (if lvl-p "lvl" "std")) finish)
                       (append (subst-seq header "-2")
                               (list
                                (list "cut_length"
                                      (ceil (cut-len (#! (if lvl-p
                                                             '(7 90 15.4375 #pitch #bot-ab)
                                                             '(7 90 #bot-ab)))))))
                               (mapcar 'list bracket-cols
                                 (take (/ (length brackets) 2) brackets))))))
              (list
               (cons (list (strcat "wall-rail-" (if lvl-p "lvl" "std")) finish)
                     (append header
                             (list (list "cut_length" cut-len))
                             (mapcar 'list bracket-cols brackets))))))))
  (with-data stair-data
    '((setq stairno number
            ip (= i-rail_type "Wall Rail")
            op (= o-rail_type "Wall Rail"))
      (if ip (add-to-list 'ret (calc "i" op)))
      (if op (add-to-list 'ret (calc "o" ip)))
      (apply 'append ret))))
(defun-r 'calc-wall-rail-fab)

(defun-q combine-wall-rail-fabs (rail-datas
                                 / key rail-map stairno)
  "Combine rail-datas based on template and finish"
  (foreach rail rail-datas
           (setq key (pop! 'rail))
           (add-to-alist 'rail-map key (list rail) T))
  rail-map)
(defun-r 'combine-wall-rail-fabs)

(defun-q make-wall-rail-fabs (stairs / rail-datas template finish rails)
  "Bundle rails and call make-wall-rail-fab"
  (setq rail-datas (combine-wall-rail-fabs (map-append 'calc-wall-rail-fab stairs)))
  (foreach rails rail-datas
           (setq template (caar rails)
                 finish (cadar rails)
                 rails (cdr rails))
           (while rails
                  (make-wall-rail-std-fab template finish (take! 60 'rails)))))
(defun-r 'make-wall-rail-fabs)

(defun-q make-wall-rail-std-fab (template finish rails)
  "Open template and call %wall-rail-write-table"
  (debug-print '("Making wall rail fab: " #template))
  (make-rail-fab-common template '(wall-rail-write-table finish rails)))
(defun-r 'make-wall-rail-std-fab)

(defun-q wall-rail-write-table (finish rail-datas / rail-table)
  "Write the table in a wall rail fab drawing"
  (set-finish finish)
  (table-add-data (get-table-by-title "Wall Rails") rail-datas)
  (table-add-data (get-table-by-title "Misc Hardware")
                  (list (list (list "qty" (wall-rail-hardware-qty rail-datas))
                              '("note" "SL"))))

  ;; update "work with" note for splicers
  (if (member template '("wall-rail-top" "wall-rail-bot-std" "wall-rail-bot-lvl"))
      ((lambda(/ dot-pos prefix num other-dwg)
           (setq dot-pos (vl-string-position (ascii ".") dwgno)
                 prefix (substr dwgno 1 (+ 1 dot-pos))
                 num (substr dwgno (+ 2 dot-pos))
                 other-dwg (strcat prefix
                                   (to-string ((if (= template "wall-rail-top") + -)
                                               (atoi num) 1))))
         (vla-replace-string modelSpace "$SPLICER_DWG" other-dwg)))))
(defun-r 'wall-rail-write-table)

(defun-q wall-rail-hardware-qty (rail-datas / count)
  "Returns the number of brackets times 3"
  (setq count 0)
  (foreach rail-data rail-datas
    (foreach i (range 1 8 1)
      (if (xd-value (strcat "bracket_" (to-string i)) rail-data)
          (inc! 'count))))
  (* 3 count))
(defun-r 'wall-rail-hardware-qty)
