(add-to-list 'sub-dialogs '("chan_bolts" "front_chan_bolts" "back_chan_bolts"))

;; Key expansion:

;; This dialog is used for front and back connections. To combine
;; their use, the dialog specifies keys with "front" or "back"
;; replaced with "~". These functions expand or reduce (when reduce-p
;; is T) the keys accordingly.

;; expand or reduce a list of key name strings
;; e.g. '("~_bolt_to" "~_double_bolt")
(defun chan-bolts-exp-keys (fb keylst reduce-p / exp-key)
  (defun exp-key (fb key)
    (apply 'vl-string-subst
           (append (if reduce-p (list "~" fb) (list fb "~"))
                   (list key))))
  (mapcar '(lambda(x) (exp-key fb x))
          keylst))

;; utility function to call chan-bolts-exp-keys for both front and
;; back, and combine the result
(defun chan-bolts-exp-keys-fb (keylst)
  (append (chan-bolts-exp-keys "front" keylst nil)
          (chan-bolts-exp-keys "back" keylst nil)))

;; expand or reduce all keys in a dialog data structure
(defun chan-bolts-exp-data (fb data reduce-p / exp-elt)
  (defun exp-elt (elt)
    (cons
     (apply 'vl-string-subst
            (append
             (if reduce-p (list "~" fb) (list fb "~"))
             (list (car elt))))
     (cdr elt)))
  (mapcar 'exp-elt data))

(setq chan_bolts-dist-keys (chan-bolts-exp-keys-fb
                            '("~_l-stair_offset"
                              "~_l-stair_wid"
                              "~_l-stair_vbolt"
                              "~_l-stair_hbolt"
                              "~_l-stair_hbolt2"
                              "~_r-stair_offset"
                              "~_r-stair_wid"
                              "~_r-stair_vbolt"
                              "~_r-stair_hbolt"
                              "~_r-stair_hbolt2"))
      chan_bolts-toggle-keys (chan-bolts-exp-keys-fb
                              '("~_l-stair_double"
                                "~_r-stair_double"))
      chan_bolts-popup-keys (chan-bolts-exp-keys-fb
                             '("~_bolt_to"))
      chan_bolts-keys (append chan_bolts-dist-keys
                              chan_bolts-toggle-keys
                              chan_bolts-popup-keys))

(defun chan_bolts-sub-init (fb / local-scope tread-style valsym)
  (setq valsym (read (strcat fb "-val"))
        fb (if (begins-with fb "front") "front" "back")
        tread-style (dialog-tile-val "tread_style"))

  (defun local-scope (/
                      popup-keys
                      dist-keys
                      int-keys
                      plain-string-keys
                      toggle-keys
                      tile-actions
                      default-actions
                      dialog-ties
                      sub-dialogs
                      dialog-after-load
                      dialog-before-start
                      dialog-before-done
                      tmp
                      single_flight-action
                      double_bolt-action
                      turn-off
                      turn-on
                      bolt-to-opts
                      )
    ;; hook
    (defun dialog-after-load ()
      ;; turn on only keys which pertain to this invocation of the dialog
      (foreach key all-keys
               (mode_tile key 0))
      ;; (if (member (dialog-tile-val "~_bolt_to") '("Landing Channel" "None"))
      ;;          (mode_tile "~_double_bolt" 1)))
      (~_bolt_to-action (dialog-tile-val "~_bolt_to")))

    ;; determine if a key is a toggle key
    (defun toggle-key-p (key)
      (member (car (chan-bolts-exp-keys-fb (list key)))
                             chan_bolts-toggle-keys))

    ;; remove a dist/toggle key
    (defun turn-off (key / tk-p)
      (setq tk-p (toggle-key-p key))
      (set_tile key (if tk-p "0" ""))
      (mode_tile key 1)
      (remove! 'all-keys '(= x key))
      (remove! 'string-keys '(= x key))
      (if tk-p
          (remove! 'toggle-keys '(= x key))
        (progn
          (remove! 'dist-keys '(= x key))
          (remove! 'edit-box-keys '(= x key)))))

    ;; add a dist/toggle key
    (defun turn-on (key / tk-p oppkey defv add-to-list-maybe)
      (setq tk-p (toggle-key-p key))
      (mode_tile key 0)

      ;; set key to default value
      (setq oppkey (calc-oppkey key)
            ;; toggle key defaults are specal
            defv (if tk-p
                     (if (= (setq tmp (get_tile key)) nil) "0" tmp)
                   ;; try to find value in source
                   (if (setq tmp (assoc key source))
                       (to-string (cadr tmp))
                     ;; try to match opposite side
                     (if (not (= (setq tmp (get_tile oppkey)) ""))
                         tmp
                       ;; fall back to local defun
                       (default-value key)))))
      (set_tile key defv)

      (defun add-to-list-maybe (lst-sym val)
        (if (not (member val (vl-symbol-value lst-sym)))
            (add-to-list lst-sym val)))

      (add-to-list-maybe 'all-keys key)
      (add-to-list-maybe 'string-keys key)
      (add-to-list-maybe (if tk-p 'toggle-keys 'dist-keys) key)
      (if tk-p
          (add-to-list-maybe 'toggle-keys key)
        (progn
          (add-to-list-maybe 'dist-keys key)
          (add-to-list-maybe 'edit-box-keys key))))

    ;; default action for bolt_to popup-list
    (defun ~_bolt_to-action (value / calc-oppkey default-value sideify stair-keys onoff)
      ;; functions for turn-on
      (defun calc-oppkey (key)
        (strcat (if (begins-with key "~_l") "~_r" "~_l")
                             (substr key 4)))
      (defun default-value (key)
        (cadr (assoc (substr key 4)
                     (list
                      '("-stair_offset" "0.0")
                      '("-stair_wid" "44.0")
                      (list "-stair_vbolt" (if (begins-with tread-style "2") "5.125" "5.0"))
                      '("-stair_hbolt" "4.0")
                      '("-stair_hbolt2" "4.0")))))

      (defun sideify (side keys)
        (mapcar '(lambda(x) (strcat "~_" side x)) keys))

      ;; turn keys on/off per side
      (defun set-side (side onoff / keys)
        (setq keys '("-stair_offset"
                     "-stair_wid"
                     "-stair_vbolt"
                     "-stair_hbolt"
                     "-stair_double")
              keys (if (or (= onoff turn-off)
                           (= (get_tile (strcat "~_" side "-stair_double")) "1"))
                       (cons "-stair_hbolt2" keys)
                     keys)
              keys (sideify side keys))

        (mapcar onoff keys))

      (setq onoff (cond ((= value "2 Stairs") '(turn-on turn-on))
                        ((= value "Left Stair") '(turn-on turn-off))
                        ((= value "Right Stair") '(turn-off turn-on))
                        ((member value '("Landing Channel" "None")) '(turn-off turn-off))))
      (set-side "l" (car onoff))
      (set-side "r" (cadr onoff))
      );;action

    ;; default action for double bolt toggles
    (defun double-bolt-action (key value / calc-oppkey default-value)

      (defun calc-oppkey (key)
        (strcat (if (begins-with key "~_l") "~_r" "~_l") "-stair_hbolt2"))

      (defun default-value (key) "4.0")

      (setq lr (substr key 3 1))
      ((if (= value "0") turn-off turn-on) (strcat "~_" lr "-stair_hbolt2"))
      );;action

    (setq toggle-keys '("~_l-stair_double"
                        "~_r-stair_double")
          popup-keys '(("~_bolt_to" . ("2 Stairs"
                                       "Left Stair"
                                       "Right Stair"
                                       "Landing Channel"
                                       "None")))
          dist-keys (chan-bolts-exp-keys fb
                                         (mapcar 'car
                                                 (remove (vl-symbol-value valsym)
                                                         '(not (member (car x) chan_bolts-dist-keys))))
                                         T)
          default-actions '(("~_l-stair_double" . double-bolt-action)
                            ("~_r-stair_double" . double-bolt-action)))
    (if (setq tmp (dialog-init
                   "land/chan_bolts.dcl"
                   "chan_bolts"
                   (chan-bolts-exp-data fb (vl-symbol-value valsym) T)))
        (set valsym (chan-bolts-exp-data fb tmp nil))))
  (local-scope))
