(add-to-list 'sub-dialogs '("conn_opts"
                            "fl_conn_opts"
                            "fr_conn_opts"
                            "bl_conn_opts"
                            "br_conn_opts"))

;; Key expansion:
;; This dialog is used for connections at all four corners. To combine
;; their use, the dialog specifies keys with "fblr" corner replaced
;; with "~". These functions expand or reduce (when reduce-p is T) the
;; keys accordingly.

;; expand or reduce a list of key name strings
(defun conn-opts-exp-keys (fblr keylst reduce-p / exp-key)
  (defun exp-key (fblr key)
    (apply 'vl-string-subst
           (append
            (if reduce-p (list "~" fblr) (list fblr "~"))
            (list key))))
  (mapcar '(lambda (x) (exp-key fblr x))
          keylst))

;; utility function to call conn-opts-exp-keys for all corners and
;; combine the result
(defun conn-opts-exp-keys-fblr (keylst)
  (append (conn-opts-exp-keys "fl" keylst nil)
          (conn-opts-exp-keys "fr" keylst nil)
          (conn-opts-exp-keys "bl" keylst nil)
          (conn-opts-exp-keys "br" keylst nil)))

;; expand or reduce all keys in a dialog data structure
(defun conn-opts-exp-data (fblr data reduce-p / exp-elt)
  (defun exp-elt (elt)
    (cons
     (apply 'vl-string-subst
            (append
             (if reduce-p (list "~" fblr) (list fblr "~"))
             (list (car elt))))
     (cdr elt)))
  (mapcar 'exp-elt data))

;; return the conn definition for corner
(defun get-conn-def (fblr exp-p / value series ret)
  (setq value (dialog-tile-val (strcat fblr "_chan"))
        series (substr value 1 1)
        ret
        (if (/= (substr value 1 2) "X")
            (cond
              ;; posts
              ((member series '("2" "3"))
               (list (list "~_xtra_holes" 0.0)))
              ;; hangers
              ((member series '("4" "5"))
               ((lambda(/ detail plate-opts kick-opts ret)
                  (setq detail (dialog-tile-val (strcat fblr "_supp"))
                        plate-opts '(("~_plt_hgt" 8.0)
                                     ("~_plt_wid" 8.0)
                                     ("~_plt_thick" 0.375))
                        kick-opts '(("~_kick_style" . ("06 WF Beam"
                                                       "07 Decking, anchored"
                                                       "08 Decking, welded"))
                                    ("~_kick_deck" . ("Parallel" "Perpendicular"))
                                    ("~_kick_len" 36.0)))
                  (if (wcmatch detail "*offset w/*")
                      (append! 'ret plate-opts))
                  (if (wcmatch detail "*kicker*")
                      (append! 'ret kick-opts))
                  ret)))
              ;; ledgers
              ((member series '("6" "7"))
               '(("~_gap" 1.0)
                 ("~_angle" . ("L4x4x3/8"
                               "L5x5x3/8"
                               "L6x4x3/8"
                               "L7x4x3/8")))))))
  (if exp-p
      (conn-opts-exp-data fblr ret nil)
    ret))

;; enable or disable support popup list based on connection detail
;; and reset to default value
(defun conn-supp (fblr chan-detail / def detail suppkey enabled prev-lst prev-val)
  (setq suppkey (strcat fblr "_supp")
        def (assoc chan-detail chan-details)
        enabled (if (and (begins-with fblr "b")
                         (= (get_tile (strcat "separate_" fblr)) "0"))
                    nil
                  (caddr def))
        prev-lst (dialog-popup-list-get suppkey)
        prev-val (get_tile suppkey))
  (dialog-popup-list-update suppkey)
  (mode_tile suppkey (if enabled 0 1))
  (if (vl-every '(lambda (a b) (= (substr a 2)
                                  (substr b 2)))
                prev-lst
                (dialog-popup-list-get suppkey))
      (set_tile suppkey prev-val))
  )

;; enable or disable ext edit box based on connection detail
;; and reset to default value
(defun conn-ext (fblr chan-detail / def extkey 85-p enabled)
  (setq extkey (strcat fblr "_ext")
        def (assoc chan-detail chan-details)
        85-p (begins-with chan-detail "85")
        enabled (if (and (begins-with fblr "b")
                         (= (get_tile (strcat "separate_" fblr)) "0"))
                    nil
                  (not 85-p)))
  (mode_tile extkey (if enabled 0 1))
  (set_tile
   extkey
   (if 85-p
       (to-string (- (cadr (assoc "wid" (shape-def (dialog-tile-val (strcat fblr "_supp")) t)))))
     (if (distof (get_tile extkey))
         (get_tile extkey)
       (to-string (cadr def))))))

;; enable or disable len edit box based on connection detail
;; and reset to default value
(defun conn-len (fblr chan-detail / def lenkey 85-p enabled)
  (setq lenkey (strcat fblr "_len")
        def (assoc chan-detail chan-details)
        85-p (begins-with chan-detail "85")
        enabled (if (and (begins-with fblr "b")
                         (= (get_tile (strcat "separate_" fblr)) "0"))
                    nil
                  (and (not 85-p)
                       (caddr def))))
  (mode_tile lenkey (if enabled 0 1))
  (set_tile lenkey (if (and (not 85-p) (caddr def))
                       (if (distof (get_tile lenkey))
                           (get_tile lenkey)
                         "60.0")
                     "")))

;; action for all four "Channel Detail" popups
(defun conn-action (key value / fblr lr back)
  (setq fblr (substr key 1 2))
  (conn-supp fblr value)
  (if (not multi)
      (progn
        (conn-ext fblr value)
        (conn-len fblr value)))
  (setq detail (dialog-tile-val (strcat fblr "_supp")))
  (dialog-sub-update-from-def (strcat fblr "_conn_opts") (get-conn-def fblr T))
  (if (and (begins-with fblr "f")
           (= (get_tile (strcat "separate_b" (setq lr (substr fblr 2 1)))) "0"))
      (progn
        (setq back (strcat "b" lr))
        (conn-supp back value)
        (conn-ext back value)
        (conn-len back value)
        (setq back (strcat back "_supp"))
        (supp-action back (dialog-tile-val back))))
  (if (not multi) (update-x-braces)))

;; action for all four "Support Detail" popups
(defun supp-action (key value / fblr lr)
  (setq fblr (substr key 1 2))
  (dialog-sub-update-from-def (strcat fblr "_conn_opts") (get-conn-def fblr T))
  (if (and (begins-with fblr "f")
           (= (get_tile (strcat "separate_b" (setq lr (substr fblr 2 1)))) "0"))
      (progn
        (setq back (strcat "b" lr "_supp"))
        (supp-action back (dialog-tile-val back))))
  (hos-update-ext key value))

;; for haunched outside stringer configurations only, update the
;; extension to the stringer's width
(defun hos-update-ext (key value / fblr)
  (setq fblr (substr key 1 2)
        value (dialog-tile-val key))
  (if (begins-with (dialog-tile-val (strcat fblr "_chan"))
                   "85")
      (set_tile (strcat fblr "_ext")
                (to-string (- (cadr (assoc "wid" (shape-def value t)))))))
  (if (and (begins-with fblr "f")
           (= (get_tile (strcat "separate_b" (setq lr (substr fblr 2 1)))) "0"))
      (hos-update-ext (strcat "b" (substr fblr 2 1) "_supp")
                      (dialog-tile-val (strcat "b" (substr fblr 2 1) "_supp")))))

;; return the list of support connection details for a given position
(defun supp-conn-list (fblr / chanval ret)
  (setq chanval (dialog-tile-val (strcat fblr "_chan"))
        ret (caddr (assoc chanval chan-details))))

(setq conn_opts-dist-keys (conn-opts-exp-keys-fblr
                           '("~_xtra_holes"
                             "~_plt_wid"
                             "~_plt_hgt"
                             "~_plt_thick"
                             "~_kick_len"
                             "~_gap"))
      conn_opts-popup-keys (conn-opts-exp-keys-fblr
                            '("~_kick_style"
                              "~_angle"
                              "~_kick_deck"))
      conn_opts-keys (append conn_opts-dist-keys
                             conn_opts-popup-keys))

;; main sub dialog init function
(defun conn_opts-sub-init (fblr / lr local-scope valsym def)
  (setq fblr (substr fblr 1 2)
        lr (substr fblr 2 1)
        valsym (read (strcat fblr "_conn_opts-val"))
        def (get-conn-def fblr nil))
  ;; define symbols for dialog-init
  (defun local-scope ( / tmp
                         tread-def
                         popup-keys
                         dist-keys
                         int-keys
                         plain-string-keys
                         toggle-keys
                         tile-actions
                         default-actions
                         dialog-ties
                         sub-dialogs
                         dialog-after-load
                         dialog-before-start
                         dialog-before-done
                         )
    (defun dialog-after-load ()
      ;; turn on only keys which pertain to this invocation of the dialog
      (foreach key all-keys
               (mode_tile key 0))
      (if (assoc "~_kick_style" source)
          (~_kick_style-action (dialog-tile-val "~_kick_style"))))

    (defun ~_kick_style-action (value)
      (if (begins-with value "06")
          (progn
            (dialog-popup-list-set "~_kick_deck" nil)
            (mode_tile "~_kick_deck" 1))
          (progn
            (dialog-popup-list-set "~_kick_deck" '("Parallel" "Perpendicular"))
            (mode_tile "~_kick_deck" 0)
            (if (assoc "~_kick_deck" source)
                (dialog-popup-set "~_kick_deck" (cadr (assoc "~_kick_deck" source)))))))

    (split-list def
                '(member (car (conn-opts-exp-keys fblr (list (car x)) nil))
                         conn_opts-dist-keys)
                'dist-keys
                'popup-keys)
    (mapcar! 'car 'dist-keys)
    (if (setq tmp (dialog-init
                   "land/conn_opts.dcl"
                   "conn_opts"
                   (conn-opts-exp-data fblr (vl-symbol-value valsym) T)))
        (progn
          (set valsym (conn-opts-exp-data fblr tmp nil))
          ;; set back conn opts if tied
          (if (and (begins-with fblr "f")
                   (= (get_tile (strcat "separate_b" lr)) "0"))
              (set (read (strcat "b" lr "_conn_opts-val"))
                   (conn-opts-exp-data (strcat "b" lr) tmp nil))))))
  (local-scope))
