(setq land-derived-keys '("version"
                          "flr_thick"
                          "dist_btwn"
                          "left_chan"
                          "left_cope"
                          "right_chan"
                          "right_cope"
                          "front_chan_dpt"
                          "front_chan_wid"
                          "front_chan_flg"
                          "front_chan_web"
                          "front_chan_len"
                          "back_chan_dpt"
                          "back_chan_wid"
                          "back_chan_flg"
                          "back_chan_web"
                          "back_chan_len"
                          "left_chan_dpt"
                          "left_chan_wid"
                          "left_chan_flg"
                          "left_chan_web"
                          "left_chan_len"
                          "right_chan_dpt"
                          "right_chan_wid"
                          "right_chan_flg"
                          "right_chan_web"
                          "right_chan_len"
                          "assembly"
                          "center_offset"
                          "fl_supp_len"
                          "fr_supp_len"
                          "bl_supp_len"
                          "br_supp_len"
                          "dir"
                          "fl_chan_num"
                          "fl_supp_num"
                          "fr_chan_num"
                          "fr_supp_num"
                          "bl_chan_num"
                          "bl_supp_num"
                          "br_chan_num"
                          "br_supp_num"
                          "fl_chan_equiv"
                          "fr_chan_equiv"
                          "bl_chan_equiv"
                          "br_chan_equiv"
                          "plt_wid_eff"
                          ))

(defun land-add-derived-keys (data / inner this-func)
  (defun-q inner (/
                  add
                  def
                  ts
                  smaller-chan
                  fl-side-chan
                  bl-side-chan
                  fr-side-chan
                  br-side-chan
                  tmp
                  chan-reduction
                  calc-supp-len
                  ret
                  )
    (defun add (key val)
      (debug-print '("Adding " #key ": " #val))
      (set (read key) val)
      (add-to-list 'ret (list key val)))

    ;; calculate *_supp_len
    (defun calc-supp-len (fblr / det len chan)
      (setq det (substr (xd-value (strcat fblr "_chan") data) 1 2)
            len (xd-value (strcat fblr "_len") data)
            chan (substr fblr 1 1)
            chan (if (= chan "f") "front" "back")
            chan (xd-value (strcat chan "_chan") data)
            chan (xd-value "dpt" (shape-def chan T)))
      (cond ((member det '("25" "35"))
             (- len flr_thick chan 0.5))
            ((member (substr det 1 1) '("2" "3"))
             (- len flr_thick 0.25))
            (T
             (+ len chan 9))))

    ;; version
    (add "version" land-version)
    ;; direction
    (add "dir" (if (= "Right" side) 1 -1))
    ;; detail numbers
    (foreach sym '(fl_chan fl_supp fr_chan fr_supp bl_chan bl_supp br_chan br_supp)
             (if (vl-symbol-value sym)
                 (add (strcat (lowercase (vl-symbol-name sym)) "_num")
                      (substr (vl-symbol-value sym) 1 2))))

    ;; floor thickness
    (setq ts (substr tread_style 1 1))
    (add "flr_thick"
         (cond ((= ts "1") 3.5)
               ((= ts "2") (if (= finish "Galvanized") 2.1075 2.625))
               ((= ts "4") 4.125)
               ((= ts "6") 1.5)))
    ;; checker plate properties (width handled as plt_wid_eff)
    (if (begins-with tread_style "2")
        ((lambda(/ f-off b-off)
           (setq f-off (if (= front_bolt_to "None") 0 chk_hold)
                 b-off (if (= back_bolt_to "None")
                           0
                           ((if (= back_bolt_to "Landing Channel") - +) chk_hold)))
           (map-apply 'add
                      (list (list "chk_offset" f-off)
                            (list "chk_thk" (if (= finish "Galvanized") 0.1875 0.125))
                            (list "chk_dpt" (- plt_dpt f-off b-off)))))))
    ;; distance between stairs
    (if (= front_bolt_to "2 Stairs")
        (add "dist_btwn"
             (- plt_wid
                (+ front_l-stair_offset
                   front_l-stair_wid
                   front_r-stair_wid
                   front_r-stair_offset))))
    ;; determine which corners require a side channel
    ;; and set length of main support member if applicable
    (foreach fblr '("fl" "fr" "bl" "br")
             (setq tmp (xd-value (strcat fblr "_chan") data))
             (set (read (strcat fblr "-side-chan"))
                  (or (and (begins-with tmp "6")
                           (not (member (substr tmp 2 1) '("6" "7" "8"))))
                      (and (begins-with tmp "7")
                           (not (member (substr tmp 2 1) '("7" "8" "9"))))
                      (begins-with tmp "25")
                      (begins-with tmp "35")
                      (begins-with tmp "XS")))
             ;; support member length
             (if (assoc (strcat fblr "_len") data)
                 (add (strcat fblr "_supp_len")
                      (calc-supp-len fblr))))
    ;; side channels
    (setq smaller-chan (nth (min (vl-position front_chan channels)
                                 (vl-position back_chan channels))
                           channels))
    (if (or fl-side-chan bl-side-chan)
        (progn
          (add "left_chan" smaller-chan)
          (cond ((>f fl_ext bl_ext 0.01)
                 (add "left_cope" "front"))
                ((and (= flip_back "1")
                      (>f bl_ext fl_ext 0.01))
                 (add "left_cope" "back")))))
    (if (or fr-side-chan br-side-chan)
        (progn
          (add "right_chan" smaller-chan)
          (cond ((>f fr_ext br_ext 0.01)
                 (add "right_cope" "front"))
                ((and (= flip_back "1")
                      (>f br_ext fr_ext 0.01))
                 (add "right_cope" "back")))))
    ;; channel shape properties
    (foreach fblr '("front" "back" "left" "right")
             (if (setq tmp (symcat '(#fblr "_chan")))
                 (progn
                   (setq def (shape-def tmp t))
                   (add (strcat fblr "_chan_dpt")
                        (xd-value "dpt" def))
                   (add (strcat fblr "_chan_wid")
                        (xd-value "wid" def))
                   (add (strcat fblr "_chan_flg")
                        (xd-value "flg" def))
                   (add (strcat fblr "_chan_web")
                        (xd-value "web" def)))))
    ;; reduce F/B channel lengths for side channels
    (defun chan-reduction (fb / lext rext olext orext)
      (setq lext (xd-value (strcat fb "l_ext") data)
            rext (xd-value (strcat fb "r_ext") data)
            fb (if (= fb "f") "b" "f")
            olext (xd-value (strcat fb "l_ext") data)
            orext (xd-value (strcat fb "r_ext") data))
      (- 0
         (if (and left_chan
                  (<= lext olext))
             left_chan_wid
             0)
         (if (and right_chan
                  (<= rext orext))
             right_chan_wid
             0)))

    ;;channel lengths
    (add "front_chan_len"
         (+ plt_wid
            fl_ext
            fr_ext
            (chan-reduction "f")))
    (add "back_chan_len"
         (+ plt_wid
            bl_ext
            br_ext
            (chan-reduction "b")))
    (if left_chan
        (add "left_chan_len"
             (+ back_chan_loc
                back_chan_wid
                (cond ((< fl_ext bl_ext) (- front_chan_wid))
                      ((< bl_ext fl_ext) (- back_chan_wid))
                      (T 0))
                (if (= flip_back "1") (- back_chan_wid) 0))))
    (if right_chan
        (add "right_chan_len"
             (+ back_chan_loc
                back_chan_wid
                (cond ((< fr_ext br_ext) (- front_chan_wid))
                      ((< br_ext fr_ext) (- back_chan_wid))
                      (T 0))
                (if (= flip_back "1") (- back_chan_wid) 0))))
    (add "assembly"
         (strcat "P" number "-" level (to-string sequence)))

    (add "center_offset"
         (+ (cond ((begins-with front_bolt_to "Right")
                   (- (/ plt_wid 2.0) front_r-stair_wid))
                  ((begins-with front_bolt_to "Left")
                   (- (- (/ plt_wid 2.0) front_l-stair_wid)))
                  ((/= front_l-stair_wid front_r-stair_wid)
                   (/ (- front_l-stair_wid front_r-stair_wid) 2.0))
                  (T 0.0))
            (if front_l-stair_offset (/ front_l-stair_offset 2.0) 0)
            (if front_r-stair_offset (/ (- front_r-stair_offset) 2.0) 0)))

    (foreach corner '("fl" "fr" "bl" "br")
       ((lambda(/ chan)
          (setq chan (atoi (cornerkey "chan_num")))
          (add (strcat corner "_chan_equiv")
               (cond ((member chan '(20 27)) "front-post-2")
                     ((member chan '(30 37)) "front-post-3")
                     ((member chan '(25 35)) "bot-post")
                     ((member chan '(60 61 62 70 71 72 76)) "ledger-sc")
                     ((member chan '(66 67 68 77 78 79)) "ledger-no-sc")
                     (T (cornerkey "chan")))))))

    (add "plt_wid_eff" (+ plt_wid
                          (if (= fl_chan_num "85") fl_ext 0)
                          (if (= fr_chan_num "85") fr_ext 0)))
    (land-add-xbrace-derived-keys)
    (append data (vl-sort ret sort-data-by-key)))

  (setq this-func (defun-q-list-ref 'inner))
  (defun-q-list-set 'inner
      (cons (append (car this-func)
                    (mapcar 'read land-derived-keys))
            (cdr this-func)))
  (with-data data '((inner))))

(defun land-add-xbrace-derived-keys (/ calc)
  (defun add-bay (bay / fb-p wid hgt npanels)
    (setq bay-short (substr bay 1 1)
          fb-p (member bay-short '("f" "b"))
          wid ((lambda(/ fb-symcat cunder)
                 ;; bay symcat
                 (defun fb-symcat (lr-key) (symcat '(#bay-short #lr-key)))
                 ;; non-nil if corner is post-under
                 (defun cunder (corner / parts)
                   (= (symcat (rcons (if fb-p
                                         (list bay-short corner)
                                         (list corner bay-short))
                                     "_chan_num"))
                      "35"))
                 (if fb-p
                     (+ plt_wid
                        (- (fb-symcat "l_ext") (if (cunder "l") 3.25 3))
                        (- (fb-symcat "r_ext") (if (cunder "r") 3.25 3)))
                     (- back_chan_loc
                        (if (cunder "f") 3.25 0)
                        (if (= flip_back "1")
                            (if (cunder "b") 3.25 0)
                            (if (cunder "b") (- 3.25 back_chan_wid) 3))))))
          hgt ((lambda(/ corners)
                 (setq corners (cond ((= bay "front") '("fl" "fr"))
                                     ((= bay "back") '("bl" "br"))
                                     ((= bay "left") '("fl" "bl"))
                                     ((= bay "right") '("fr" "br"))))
                 (- (apply 'min (mapcar '(lambda(c) (symcat '(#c "_len"))) corners))
                    flr_thick
                    (if fb-p
                        (symcat '(#bay "_chan_dpt"))
                        (max front_chan_dpt back_chan_dpt))
                    (- (symcat '(#bay-short "_brace_hold")) 6))))
          npanels ((lambda(/ n)
                     (setq n 1)
                     (while (> (/ hgt n) (* 2 wid))
                       (inc! 'n))
                     n))
          hgt (/ hgt npanels))
    (foreach key '(wid hgt npanels)
             (add (strcat bay-short "_brace_" (lowercase (to-string key))) (eval key))))
  (foreach bay '("front" "back" "left" "right")
           (if (= (symcat '(#(substr bay 1 1) "_brace")) "1") (add-bay bay))))
