(psc-include '("land/details.lsp"))

;; define all functions and variables necessary for dialog-init
;; source should be a stair-data variable from an existing landing or nil,
;; which indicates it is to read from defaults-file
(defun land-dialog-init (source
                         stair-enames
                         /
                         ;; variables defined here for dialog-init
                         defaults-file
                         popup-keys
                         dist-keys
                         int-keys
                         toggle-keys
                         plain-string-keys
                         default-actions
                         dialog-ties
                         sub-dialogs
                         chan_bolts-sub-init
                         chan_bolts-val
                         chan_bolts-dist-keys
                         chan_bolts-toggle-keys
                         chan_bolts-popup-keys
                         chan_bolts-keys
                         chan-bolts-exp-keys
                         chan-bolts-exp-keys-fb
                         chan-bolts-exp-data
                         tread_opts-sub-init
                         tread_opts-val
                         tread_opts-dist-keys
                         tread_opts-toggle-keys
                         tread_opts-keys
                         conn_opts-sub-init
                         conn_opts-val
                         conn_opts-dist-keys
                         conn_opts-popup-keys
                         conn_opts-keys
                         ;; hook
                         dialog-after-load
                         dialog-before-start
                         dialog-before-done
                         ;; variables included here for scope,
                         ;; but not defined by land-dialog-init directly
                         extra-keys
                         tile-actions
                         ;; tile actions
                         conn-action
                         supp-action
                         geom-action
                         flip_back-action
                         ;; util function
                         conn-supp
                         conn-ext
                         conn-len
                         hos-update-ext
                         update-x-braces
                         chan-spacing
                         post-detail
                         hanger-detail
                         supp-conn-list
                         update-stair
                         update-source-from-stairs
                         calc-geom-key
                         update-geom
                         enable-back-bolts
                         ;; local variables
                         tread-styles
                         data
                         )
  
  (land-dialog-load-common-defs)

  ;; hook called just before done_dialog
  ;; increments level
  (defun dialog-before-done ( / toggle-z inc-seq inc-lvl)

    (defun inc-lvl ()
      (set_tile "sequence" "A")
      (set_tile "level" (next-lvl)))

    (defun next-lvl ( / lvl prefix tmp)
      (setq lvl (uppercase (cadr (assoc "level" data))))

      (if (wcmatch lvl "@#*")
          (setq prefix (substr lvl 1 1)
                lvl (substr lvl 2))
        (setq prefix ""))

      ;; only attempt to increment numeric levels
      (if (setq tmp (safe-atoi lvl))
          ;; below grade prefixes
          (if (member prefix '("P" "B"))
              ;; switch to above grade
              (if (= tmp 1)
                  "1"
                (strcat prefix (itoa (1- tmp))))
            (strcat prefix (itoa (1+ tmp))))
        lvl))

    (dialog-load-defaults nil)
    (set_tile "comment" "")
    (inc-lvl)
    (dialog-save-defaults defaults-file))

  ;; automatically update the var called "source" with data from the stairs
  ;; if init is non-nil, we are creating a landing from two stairs
  ;; fb is "front" or "back" and affects the t-or-b and l-or-r functions
  (defun update-source-from-stairs (enames
                                    init
                                    fb
                                    /
                                    flip-once
                                    x-or-x
                                    t-or-b
                                    l-or-r
                                    get-ndist
                                    update-geom-key
                                    update-geom
                                    add&subst
                                    update-width
                                    s1-data
                                    s2-data
                                    s1-ndist
                                    s2-ndist
                                    s1-ascend
                                    s2-ascend
                                    s1-z
                                    s2-z
                                    side
                                    old-ndist
                                    new-ndist
                                    ndist-key
                                    )
    (defun flip-once (val-sym opts / val)
      (setq val (vl-symbol-value val-sym))
      (set val-sym (if (= val (car opts)) (cadr opts) (car opts))))

    ;; assume the following conditions and flip as necessary
    ;; channel: front
    ;; ascend: left
    ;; z_pos: near
    ;; land side: right
    (defun x-or-x (opts / ret)
      (setq ret (car opts))
      (if (= fb "back")                 (flip-once 'ret opts))
      (if (= stair-ascend "Right")      (flip-once 'ret opts))
      (if (= stair-z "Far")             (flip-once 'ret opts))
      (if (= land-side "Left")          (flip-once 'ret opts))
      ret)

    ;; is connection at top or bottom of stair?
    (defun t-or-b (fb stair-ascend stair-z land-side /)
      (x-or-x '("top" "bot")))

    ;; is stair on left or right of land?
    (defun l-or-r (fb stair-ascend stair-z land-side / bolt-to)
      (setq bolt-to (xd-value (strcat fb "_bolt_to") source))
      (cond ((begins-with bolt-to "Left") "l")
            ((begins-with bolt-to "Right") "r")
            (T
             (if (= fb "front")
                 (if (= stair-ascend "Left")
                     (if (= stair-z "Near")
                         (if (= land-side "Left")
                             "l" "r")
                         (if (= land-side "Left")
                             "r" "l"))
                     (if (= stair-z "Near")
                         (if (= land-side "Left")
                             "l" "r")
                         (if (= land-side "Left")
                             "r" "l")))
                 (if (= stair-ascend "Left")
                     (if (= stair-z "Near")
                         (if (= land-side "Left")
                             "r" "l")
                         (if (= land-side "Left")
                             "l" "r"))
                     (if (= stair-z "Near")
                         (if (= land-side "Left")
                             "r" "l")
                         (if (= land-side "Left")
                             "l" "r")))))))

    ;; get a single stair's nosing distance
    (defun get-ndist (data / tb)
      (setq tb (tk '(t-or-b fb ascend z_pos side)))
      (tk (list +
                (read (strcat tb "_cdist"))
                (read (strcat tb "_ext")))))

    ;; update a single geometry key
    (defun update-geom-key (key)
      (subst-key! key (calc-geom-key key source) 'source))

    ;; overwrite landing width and depth with minimums based on stairs
    (defun update-geom (s1-data
                        s2-data
                        /
                        s1-width
                        s2-width
                        data
                        dbtwn
                        total
                        )
      (setq dbtwn (cadr (assoc "dist_btwn" s1-data))
            s1-width (cadr (assoc "width" s1-data))
            s2-width (cadr (assoc "width" s2-data))
            total (+ s1-width dbtwn s2-width))
      (if (> total (cadr (assoc "plt_wid" source)))
          (progn
            (subst-key! "plt_wid" total 'source)
            (update-geom-key "tot_wid")))
      (if (< (cadr (assoc "tot_dpt" source)) (min s1-width s2-width))
          (progn
            (subst-key! "tot_dpt" (min s1-width s2-width) 'source)
            (update-geom-key "plt_dpt")
            (update-geom-key "back_chan_loc"))))

    ;; add a key to source, if not present, and set it to val
    (defun add&subst (key val)
      (if (not (assoc key source))
          (add-to-list 'source (list key nil)))
      (subst-key! key val 'source))

    ;; update the bolt info for a stair
    (defun update-width (data / tb lr keys tmp strkey)
      (setq tb (tk '(t-or-b fb ascend z_pos side))
            fblr (tk '(strcat fb "_" (l-or-r fb ascend z_pos side))))
      (tk '(subst-key! (strcat fblr "-stair_wid") width 'source))
      
      ;; perform the actual update
      (foreach key '("_vbolt" "_hbolt" "_hbolt2")
               (setq strkey (strcat fblr "-stair" key))
               (if (setq tmp (assoc (strcat tb key) data))
                   (progn
                     (add&subst strkey (cadr tmp))
                     ;; set double bolt toggle
                     (if (= key "_hbolt2")
                         (add&subst (strcat fblr "-stair_double") "1")))
                 (if (assoc strkey source)
                     (progn
                       (remove! 'source '(= (car x) strkey))
                       (remove! 'source '(ends-with (car x) "-stair_double")))))))

    ;; update checker plate options, if necessary
    (defun update-chkr (data / )
      (if (tk '(begins-with tread_style "2"))
          (progn
            (add&subst "chk_hold" (tk '(if (= stn_style "STNS") 0.25 0.75)))
            (if (tk '(= quiet_tread "1"))
                (add&subst "quiet_tread" "1")))))

    (setq side (cadr (assoc "side" source))
          ndist-key (if (= fb "front") "stair_ndist" "back_gap"))
    (cond
      ;; 2 stairs
      ((= (length enames) 2)
       (setq s1-data (read-xdata (car enames) "Stair")
             s1-ascend (cadr (assoc "ascend" s1-data))
             s1-z (cadr (assoc "z_pos" s1-data))
             s2-data (read-xdata (cadr enames) "")
             s2-ascend (cadr (assoc "ascend" s2-data))
             s2-z (cadr (assoc "z_pos" s2-data)))
       ;; if we have a new landing, overwrite certain properties 
       (if init
           (progn
             (update-geom s1-data s2-data)
             (subst-key! "side"
                         (if (= (stair-dir s1-ascend s1-z) 1)
                             "Right" "Left")
                         'source)
             (setq side (cadr (assoc "side" source)))
             (subst-key! "tread_style" (cadr (assoc "tread_style" s1-data)) 'source)
             (subst-key! "number" (cadr (assoc "number" s1-data)) 'source)
             (subst-key! "level" (cadr (assoc "level" s1-data)) 'source)
             (subst-key! "sequence" (cadr (assoc "sequence" s1-data)) 'source))
           )
       (setq s1-ndist (get-ndist s1-data)
             s2-ndist (get-ndist s2-data)
             old-ndist (cadr (assoc ndist-key source))
             new-ndist (min s1-ndist s2-ndist)
             new-ndist (if (/= old-ndist new-ndist) new-ndist))
       (if new-ndist
           (progn
             (subst-key! ndist-key new-ndist 'source)
             (update-geom-key "plt_dpt")
             (update-geom-key "back_chan_loc")))
       (update-width s1-data)
       (update-width s2-data)
       (update-chkr s1-data)
       (subst-key! "finish" (xd-value "finish" s1-data) 'source))
      ;; 1 stair
      ((= (length enames) 1)
       (setq s1-data (read-xdata (car enames) "Stair")
             s1-ndist (get-ndist s1-data))
       (if (< s1-ndist (cadr (assoc ndist-key source)))
           (progn
             (subst-key! ndist-key s1-ndist 'source)
             (update-geom-key "plt_dpt")
             (update-geom-key "back_chan_loc")))
       (update-width s1-data)
       (update-chkr s1-data)
       (subst-key! "finish" (xd-value "finish" s1-data) 'source))))

  ;; set which bays are available for X-bracing
  (defun update-x-braces ( / corners conn key available)
    (foreach bay '("f" "b" "l" "r")
             (setq corners (if (member bay '("f" "b"))
                               (list (strcat bay "l")
                                     (strcat bay "r"))
                             (list (strcat "f" bay)
                                   (strcat "b" bay)))
                   available T)
             (foreach corner corners
                      (setq conn (substr (dialog-tile-val (strcat corner "_chan")) 1 2))
                      (if (not (member conn '("30" "35")))
                          (setq available nil)))
             (setq key (strcat bay "_brace"))
             (if available
                 (progn
                   (mode_tile key 0)
                   (if (= (get_tile key) "1")
                       (mode_tile (strcat key "_hold") 0)
                       ((lambda (key)
                          (set_tile key "")
                          (mode_tile key 1))
                        (strcat key "_hold"))))
                 (progn
                 (set_tile key "0")
                 (mode_tile key 1)
                 (setq key (strcat key "_hold"))
                 (set_tile key "")
                 (mode_tile key 1)))))

  ;; update the properties of an attached stair
  (defun update-stairs (fb / )
    (defun dialog-dumpout-func (fb / )
      (setq enames (get-stairland 'stair T))
      ;; automatically updates source var
      (update-source-from-stairs enames nil fb))
    (setq source (dialog-get-data T)
          dialog-dumpout-args (list fb))
    (done_dialog 2))

  ;;;;;;;;;;;;;;;;;;;;
  ;; Initialize dialog
  ;;;;;;;;;;;;;;;;;;;;
  (setq defaults-file (dialog-update-defaults-file "land")
        data (dialog-init "land/land.dcl" "land" source))

  ;; post-process and return data
  (if data
      (progn
        (setq data (land-add-derived-keys data))
        (vl-remove-if '(lambda (x) (member (cadr x) '("" nil))) data))))

;; get a delta for land-multi-edit using the special dialog
(defun land-multi-dialog-init ( /
                                ;; variables defined here for dialog-init
                                defaults-file
                                popup-keys
                                dist-keys
                                int-keys
                                toggle-keys
                                plain-string-keys
                                default-actions
                                dialog-ties
                                sub-dialogs
                                chan_bolts-sub-init
                                chan_bolts-val
                                chan_bolts-dist-keys
                                chan_bolts-toggle-keys
                                chan_bolts-popup-keys
                                chan-bolts-exp-keys
                                chan-bolts-exp-keys-fb
                                chan-bolts-exp-data
                                tread_opts-sub-init
                                tread_opts-val
                                tread_opts-dist-keys
                                tread_opts-toggle-keys
                                conn_opts-sub-init
                                conn_opts-val
                                conn_opts-dist-keys
                                conn_opts-popup-keys
                                ;; hook
                                dialog-after-load
                                dialog-before-start
                                dialog-before-done
                                dialog-before-save
                                ;; variables included here for scope,
                                ;; but not defined by land-dialog-init directly
                                extra-keys
                                tile-actions
                                ;; tile actions
                                conn-action
                                supp-action
                                geom-action
                                flip_back-action
                                ;; util function
                                conn-supp
                                conn-ext
                                conn-len
                                hos-update-ext
                                update-x-braces
                                chan-spacing
                                post-detail
                                hanger-detail
                                supp-conn-list
                                calc-geom-key
                                update-geom
                                enable-back-bolts
                                ;; local variables
                                tread-styles
                                data
                                act-lst
                                multi
                                )
  (land-dialog-load-common-defs)
  (setq multi T)

  (defun dialog-before-start( / action setup-mod-conn)
    (setq action dialog-add-action)
    
    ;; general enable toggle setup
    (foreach key (append all-keys
                         '("tread_opts"
                           "front_chan_bolts"
                           "back_chan_bolts"
                           "fl_conn_opts"
                           "fr_conn_opts"
                           "bl_conn_opts"
                           "br_conn_opts"))
             (mode_tile key 1)
             (if (not (member key '(
                                    )))
                 (action (strcat "enable_" key)
                         '(dialog-toggle-enable $value #key))))
    (action "enable_tread_style" '(dialog-toggle-enable $value "tread_opts"))
    (action "enable_level" '(dialog-toggle-enable $value "plts_level"))

    ;; enablers for buttons set them to nil on disable
    (action "enable_tread_style" '(if (= $value "0") (setq tread_opts-val nil)))
    (action "enable_back_chan_bolts" '(if (= $value "0") (setq back_chan_bolts-val nil)))
    (action "enable_front_chan_bolts" '(if (= $value "0") (setq front_chan_bolts-val nil)))

    ;; depths and widths
    (setq act-lst '(progn (dialog-toggle-enable $value #k)
                          (set_tile #k (if (= $value "1") "0.0" ""))))
    (foreach k '("tot_dpt" "stair_ndist" "plt_dpt" "back_gap") (action "mod_dpt" act-lst))
    (foreach k '("tot_wid" "plt_wid" "left_gap" "right_gap") (action "mod_wid" act-lst))
    
    ;; assign actions to toggles that enable connections
    (defun setup-mod-conn (fblr /
                           chankey
                           modkey
                           extkey
                           suppkey
                           lenkey
                           optskey
                           optsvalsym
                           )
      (setq chankey (strcat fblr "_chan")
            modkey (strcat "enable_" chankey)
            suppkey (strcat fblr "_supp")
            optskey (strcat fblr "_conn_opts")
            optsvalsym (strcat fblr "_conn_opts-val"))
      (action modkey '(dialog-toggle-enable $value #suppkey))
      (action modkey '(dialog-toggle-enable $value #optskey))
      (action modkey '(if (= $value "1")
                       (conn-action #chankey (dialog-tile-val #chankey))
                       (set (read #optsvalsym) nil))))
    
    (mapcar 'setup-mod-conn '("fl" "fr" "bl" "br")))

  (defun dialog-before-save ( / test)
    (setq test '(= (get_tile (strcat "enable_" x)) "0"))
    (mapcar 'remove!
            '(popup-keys-strings
              string-keys)
            (list test test))
    (remove! 'popup-keys '(not (member (car x) popup-keys-strings))))

  ;; remove _supp keys without corresponding _chan keys
  (defun remove-supports (delta / suppkey)
    (foreach fblr '("fl" "fr" "bl" "br")
             (setq suppkey (strcat fblr "_supp"))
             (if (and (assoc suppkey delta)
                      (not (assoc (strcat fblr "_chan") delta)))
                 (remove! 'delta (macro-expand '(= (car x) #suppkey)))))
    delta)

  (add-to-list 'int-keys "plts_level")
  (remove-supports (dialog-init "land/land-multi.dcl" "land_multi" nil)))

;; function to make common defuns and setqs -- designed to be called
;; in a scope that makes them local
(defun land-dialog-load-common-defs ()
  ;; hook called just before dialog-start
  (defun dialog-before-start ( / key value 85-p f v)
    (foreach fblr '("fl" "fr" "bl" "br")
             (setq key (strcat fblr "_chan")
                   value (dialog-tile-val key)
                   85-p (begins-with key "85"))
             (foreach k '("supp" "ext" "len")
                      (setq f (vl-symbol-value (read (strcat "conn-" k)))
                            k (strcat fblr "_" k)
                            v (get_tile k))
                      (f fblr value)
                      (set_tile k v))
             (if (or 85-p
                     (= (dialog-popup-list-get (strcat fblr "_supp")) nil))
                 (set_tile (strcat fblr "_len") "")))
    ;; special actions for complex dialog ties
    (dialog-add-action "separate_bl" '(conn-action "bl_chan" (dialog-tile-val "bl_chan")))
    (dialog-add-action "separate_bl"
                       '(if (= $value "0")
                            (progn
                              (mode_tile "bl_conn_opts" 1)
                              (set_tile "bl_supp" (get_tile "fl_supp"))
                              (setq bl_conn_opts-val
                                    (conn-opts-exp-data "bl"
                                                        (conn-opts-exp-data "fl"
                                                                            fl_conn_opts-val
                                                                            T)
                                                        nil)))
                          (mode_tile "bl_conn_opts" 0)))
    (if (= (get_tile "separate_bl") "0") (mode_tile "bl_conn_opts" 1))
    (dialog-add-action "separate_br" '(conn-action "br_chan" (dialog-tile-val "br_chan")))
    (dialog-add-action "separate_br"
                       '(if (= $value "0")
                            (progn
                              (mode_tile "br_conn_opts" 1)
                              (set_tile "br_supp" (get_tile "fr_supp"))
                              (setq br_conn_opts-val
                                    (conn-opts-exp-data "br"
                                                        (conn-opts-exp-data "fr"
                                                                            fr_conn_opts-val
                                                                            T)
                                                        nil)))
                          (mode_tile "br_conn_opts" 0)))
    (if (= (get_tile "separate_br") "0") (mode_tile "br_conn_opts" 1))
    (update-x-braces)
    (enable-back-bolts (dialog-tile-val "flip_back"))
    ;; user selected 2 stairs
    (if stair-enames
        (progn
          (setq source (dialog-get-data T))
          (update-source-from-stairs stair-enames T "front")
          (setq stair-enames nil)
          (dialog-load-defaults source)))
    (tread_style-action (dialog-tile-val "tread_style"))
    (enable-back-stair-mode)
    (dialog-add-action "back_chan_bolts" "(enable-back-stair-mode)"))

  ;; calculate the correct value for a geometry key
  (defun calc-geom-key (key data / )
    (tk '(cond ((= key "tot_wid")
                (+ plt_wid left_gap right_gap))
               ((= key "plt_wid")
                (- tot_wid left_gap right_gap))
               ((= key "tot_dpt")
                (+ stair_ndist plt_dpt back_gap))
               ((= key "plt_dpt")
                (- tot_dpt stair_ndist back_gap))
               ((= key "back_chan_loc")
                (if (= flip_back "1")
                    plt_dpt
                  (chan-spacing plt_dpt))))))

  ;; action for widths and depths
  (defun geom-action (key value / wkeys dkeys data update-elt)
    (defun update-elt (k / v)
      (setq v (calc-geom-key k data))
      (set_tile k (to-string v))
      (subst! (list k v) (assoc k data) 'data))

    (if (= value nil) (set_tile key "0.0"))
    (setq data (dialog-get-data T))
    (update-elt (cond ((= key "plt_wid") "tot_wid")
                      ((= key "plt_dpt") "tot_dpt")
                      ((member key '("tot_wid" "left_gap" "right_gap")) "plt_wid")
                      ((member key '("tot_dpt" "stair_ndist" "back_gap")) "plt_dpt")))
    (if (member key '("tot_dpt" "stair_ndist" "plt_dpt" "back_gap"))
        (update-elt "back_chan_loc")))

  ;; calculate channel spacing
  (defun chan-spacing (plt_dpt)
    (- (- plt_dpt 3)
       (rem (- plt_dpt 3) 3)))

  ;; set channel spacing when flipping back channel
  (defun flip_back-action (value / pd)
    (if (setq pd (distof (get_tile "plt_dpt")))
        (set_tile "back_chan_loc" (if (= value "0")
                                      (to-string (chan-spacing pd))
                                    (to-string pd))))
    (if (not multi)
        (progn
          (enable-back-bolts value)
          (enable-back-stair-mode))))

  ;; set status of rear channel bolts based on flip_back
  (defun enable-back-bolts (flip_back)
    (if (= flip_back "1")
        (mode_tile "back_chan_bolts" 0)
      (progn
        (mode_tile "back_chan_bolts" 1)
        (setq back_chan_bolts-val '(("back_bolt_to" "None"))))))

  ;; react to presence or absence of a back stair
  (defun enable-back-stair-mode ( / tmp)
    (if (and (setq tmp (assoc "back_bolt_to" back_chan_bolts-val))
             (member (cadr tmp) '("2 Stairs" "Left Stair" "Right Stair")))
        (progn
          (mode_tile "update_back_stairs" 0)
          (set_tile "tot_dpt_label" "Nosing to Nosing")
          (set_tile "back_gap_label" "Back Stair Nosing Distance")
          )
      (progn
        (mode_tile "update_back_stairs" 1)
        (set_tile "tot_dpt_label" "Nosing to Wall")
        (set_tile "back_gap_label" "Back Gap")
        )))

  ;; enable *_brace_hold for x-brace toggles
  (defun enable-brace-hold (key value / hkey)
    (setq hkey (strcat key "_hold"))
    (if (= value "0")
        (progn
          (set_tile hkey "")
          (mode_tile hkey 1))
        (progn
          (mode_tile hkey 0)
          (set_tile hkey "6.0"))))

  ;; sub-dialogs and related code
  (psc-load "land/chan_bolts.lsp")
  (psc-load "land/tread_opts.lsp")
  (psc-load "land/conn_opts.lsp")

  (setq popup-keys '(("tread_style" . tread-styles)
                     ("finish" . finishes)
                     ("front_chan" . channels)
                     ("back_chan" . channels)
                     ("side" . ("Left" "Right"))
                     ("fl_chan" . chan-details-strings)
                     ("fl_supp" . (lambda () (supp-conn-list "fl")))
                     ("fr_chan" . chan-details-strings)
                     ("fr_supp" . (lambda () (supp-conn-list "fr")))
                     ("bl_chan" . chan-details-strings)
                     ("bl_supp" . (lambda () (supp-conn-list "bl")))
                     ("br_chan" . chan-details-strings)
                     ("br_supp" . (lambda () (supp-conn-list "br"))))
        
        dist-keys '("tot_wid"
                    "tot_dpt"
                    "left_gap"
                    "right_gap"
                    "plt_wid"
                    "fl_ext"
                    "fl_len"
                    "fr_ext"
                    "fr_len"
                    "bl_ext"
                    "bl_len"
                    "br_ext"
                    "br_len"
                    "plt_dpt"
                    "back_gap"
                    "back_chan_loc"
                    "stair_ndist"
                    "head_clr"
                    "f_brace_hold"
                    "b_brace_hold"
                    "l_brace_hold"
                    "r_brace_hold"
                    )

        plain-string-keys '("number"
                            "level"
                            "sequence"
                            "comment")

        toggle-keys '("flip_back"
                      "f_brace"
                      "b_brace"
                      "l_brace"
                      "r_brace"
                      )

        load-defaults-keys '("number"
                             "head_clr"
                             "tread_style"
                             "chk_hold"
                             "quiet_tread"
                             "finish")
        
        dialog-ties '(((("front_chan" . "back_chan"))
                       "separate_channels")
                      ((("fl_chan" . "bl_chan")
                        ("fl_supp" . "bl_supp")
                        ("fl_ext" . "bl_ext")
                        ("fl_len" . "bl_len"))
                       "separate_bl")
                      ((("fr_chan" . "br_chan")
                        ("fr_supp" . "br_supp")
                        ("fr_ext" . "br_ext")
                        ("fr_len" . "br_len"))
                       "separate_br"))

        default-actions '(("level" . force-uppercase)
                          ("sequence" . force-uppercase)
                          ("fl_chan" . conn-action)
                          ("fr_chan" . conn-action)
                          ("bl_chan" . conn-action)
                          ("br_chan" . conn-action)
                          ("fl_supp" . supp-action)
                          ("fr_supp" . supp-action)
                          ("bl_supp" . supp-action)
                          ("br_supp" . supp-action)
                          ("tot_wid" . geom-action)
                          ("plt_wid" . geom-action)
                          ("left_gap" . geom-action)
                          ("right_gap" . geom-action)
                          ("tot_dpt" . geom-action)
                          ("stair_ndist" . geom-action)
                          ("plt_dpt" . geom-action)
                          ("back_gap" . geom-action)
                          ("f_brace" . enable-brace-hold)
                          ("b_brace" . enable-brace-hold)
                          ("l_brace" . enable-brace-hold)
                          ("r_brace" . enable-brace-hold))
        ) ; setq
  ) ; common defs

(defun land-single-edit (source / app-id land-data stair-enames ins rot)
  (if (= (type source) 'LIST)
      (if (= (length source) 2)
          (setq
           stair-enames source
           app-id (cadr (assoc "app_id" (read-xdata (car source) "")))
           ins
           (mapcar '+
                   (cdr (assoc 10 (entget (cadr stair-enames))))
                   (if (= app-id "Stair")
                       '(0 0 0)
                       ( (lambda (s1-data s2-data / wid btw dir)
                           (list
                            0
                            (*
                             (cadr (assoc "dir" s2-data))
                             (/
                              (+
                               (cadr (assoc "dist_btwn" s1-data))
                               (cadr (assoc "width" s2-data))
                               ) ;_ +
                              2.0
                              ) ;_ /
                             ) ;_ *
                            0
                            ) ;_ list
                           ) ;_ lambda
                         (read-xdata (car source) "")
                         (read-xdata (cadr source) "")
                         ) ;_ lambda func
                       ) ;_ if
                   ) ;_ mapcar
           source nil
           ) ;_ setq
          (error-out)
          ) ;_ if
      (if source
          (setq app-id
                (cadr (assoc "app_id" (read-xdata source "")))
                ) ;_ setq
          ) ;_ if
      ) ;_ if
  (if
   (setq land-data
         (land-dialog-init
          (cond
            ( (null source) nil)
            ( (= T) (draw-xdata-migrate 'source app-id))
            ) ;_ cond
          stair-enames
          ) ;_ land-dialog-init
         ) ;_ setq
   ( (if (member app-id '("3D_Land" "3D_Stair"))
         draw-3d_land
         draw-land
         ) ;_ if
     land-data
     (cond
       ( (null source)
         (if ins ins (getpoint "\nLanding Insertion Point: "))
         ) ;_ condif
       ( (= T)
         (setq rot (vla-get-rotation (ename>vlobj source)))
         (get-ins-and-delete source))
       ) ;_ cond
     ) ;_ draw-land -or- draw-3d_land
   ) ;_ if
  ) ;_ defun land-single-edit

;; edit multiple landings
(defun land-multi-edit (source
                        /
                        delta
                        process-sd-key
                        bay-available-p
                        update-land
                        level
                        sequence
                        plts_level
                        )
  
  ;; process a single sub-dialog key
  ;; designed to be mapcar'd over a list of sd keys
  (defun process-sd-key (key / tmp)
    (remove! 'data '(= (car x) key))
    (if (setq tmp (assoc key delta))
        (progn
          (remove! 'delta (= x tmp))
          (add-to-list 'data tmp))))

  ;; determine if a certain bay is available for x-bracing
  (defun bay-available-p (bay / corners available-p data-conn delta-conn)
    (setq corners (if (member bay '("f" "b"))
                      (list (strcat bay "l")
                            (strcat bay "r"))
                      (list (strcat "f" bay)
                            (strcat "b" bay)))
          available-p T)
    (foreach corner corners
             (setq data-conn (substr (cadr (assoc (strcat corner "_chan") data)) 1 2)
                   delta-conn (assoc (strcat corner "_chan") delta))
             (if (and (not (member data-conn '("30" "35")))
                      (not (and delta-conn
                                (member (substr (cadr delta-conn) 1 2) '("30" "35")))))
                 (setq available-p nil)))
    available-p)

  ;; update the data for and redraw one landing block
  (defun update-land (ename
                      /
                      data
                      app-id
                      keysym
                      tmp
                      rot
                      )
    ;; take the opportunity to do migrations
    (setq
     app-id (cadr (assoc "app_id" (read-xdata ename "")))
     data (draw-xdata-migrate 'ename app-id)
     rot (vla-get-rotation (ename>vlobj ename))
     ) ;_ setq
    ;; switch sides
    (if switch-sides-p
        (subst-key! "side" (if (= (cadr (assoc "side" data)) "Left") "Right" "Left") 'data))
    ;; renumber
    (if level
        (progn
          (subst-key! "level" (to-string level) 'data)
          (subst-key! "sequence" (substr "ABCDEFGH" sequence 1) 'data)
          (inc! 'sequence)
          (if (> sequence plts_level)
              (setq sequence 1
                    level (1+ level)))))
    ;; handle connection supports
    (defun handle-connection (fblr / chankey chanval suppkey lenkey supp-p len-p)
      (setq chankey (strcat fblr "_chan")
            chanval (xd-value chankey (if (assoc chankey delta) delta data))
            suppkey (strcat fblr "_supp")
            lenkey (strcat fblr "_len")
            supp-p (caddr (assoc chanval chan-details))
            len-p (and supp-p (not (begins-with chanval "85"))))
      ;; remove supp and len from data if not required
      (if (not supp-p)
          (remove! 'data '(= (car x) #suppkey)))
      (if (not len-p)
          (remove! 'data '(= (car x) #lenkey)))
      ;; add supp and len to data if required
      (if (and supp-p (not (assoc suppkey data)))
          (add-to-list 'data (list suppkey nil)))
      (if (and len-p (not (assoc lenkey data)))
          (add-to-list 'data (list lenkey nil))))
    (mapcar 'handle-connection '("fl" "fr" "bl" "br"))
    ;; handle sub-dialogs
    (if (vl-some '(lambda(x) (= (car x) "tread_style")) delta)
        (mapcar 'process-sd-key tread_opts-keys))
    (if (vl-some '(lambda(x) (= (car x) "front_chan_bolts")) delta)
        (mapcar 'process-sd-key (remove chan_bolts-keys '(not (begins-with x "f")))))
    (if (vl-some '(lambda(x) (= (car x) "back_chan_bolts")) delta)
        (mapcar 'process-sd-key (remove chan_bolts-keys '(not (begins-with x "b")))))
    (foreach fblr '("fl" "fr" "bl" "br")
             (if (vl-some '(lambda(x) (= (car x) (strcat fblr "_chan"))) delta)
                 (mapcar 'process-sd-key (remove conn_opts-keys '(not (begins-with x fblr))))))
    ;; handle x-bracing changes
    (foreach fblr '("f" "b" "l" "r")
             (if (and (vl-some '(lambda(x) (= (car x) (strcat fblr "_brace"))) delta)
                      (not (bay-available-p fblr)))
                 (remove! 'delta '(= (car x) (strcat fblr "_brace")))))
    ;; rewrite XDATA
    (foreach elt delta (subst-key! (car elt) (cadr elt) 'data))
    ;; remove derived keys
    (remove! 'data '(member (car x) land-derived-keys))
    ;; recalculate derived keys, delete and redraw
    ((if (= app-id "3D_Land") draw-3d_land draw-land)
     (land-add-derived-keys data)
     (get-ins-and-delete ename)
     ) ;_ draw-land -or- draw-3d_land

    ) ;_ defun update-land

  ;; open the dialog and do the deed
  (if (setq delta (land-multi-dialog-init))
      (progn
        (if (setq level (assoc "level" delta))
            (setq level (atoi (cadr level))
                  plts_level (cadr (assoc "plts_level" delta))
                  sequence 1
                  delta (remove delta '(= (car x) "level"))))
        (if (setq switch-sides-p (assoc "side" delta))
            (remove! 'delta '(= (car x) "side")))
        (mapcar 'update-land source))))
