(set-file-docstring "Functions for drawing landing end views")

(psc-include '(
               "shapes/shape.lsp"
               "shapes/decking.lsp"
               ))
(defun-q land-endview-draw (ins lr
                                / blockObj obj clayer mirror-p som fchanx
                                fchany bchanx bchany chanx chan-func
                                plt_dwid blockOffset)
  "Draws a landing end view at INS

LR should be one of the symbols 'left or 'right to indicate which perspective to
draw (relative to the view of the section)."
  (declare (vars (ins list (point-p ins))
                 (lr sym (member lr '(left right))))
           with-data)
  (setq mirror-p (or (and (= side "Left")
                          (= lr 'left))
                     (and (= side "Right")
                          (= lr 'right)))
        blockOffset ((if mirror-p - +) center_offset)
        blocks (vla-get-Blocks acadDoc)
        blockObj (vla-Add blocks (3dpt blockOffset 0 0) (p-blk-name assembly))
        clayer (getvar 'clayer))

  ;; "(setq obj ...), mirror"
  (defun som (new-obj)
    (setq obj new-obj)
    (if mirror-p (vla-mirror-y obj)))

  ;; platform
  (setvar 'clayer "SOLID")
  (setq pltx (- (/ plt_wid 2.0))
        ;; draw width
        plt_dwid plt_wid)
  (if (begins-with fl_chan "85")
      (setq plt_dwid (+ plt_wid fl_ext)
            pltx (- pltx fl_ext)))
  (if (begins-with fr_chan "85")
      (setq plt_dwid (+ plt_dwid fr_ext)))
  (som (add-rectangle blockObj plt_dwid flr_thick (list pltx
                                                        (- flr_thick)
                                                        0)))

  (if (begins-with (tk '(eval tread_style)) "100")
      (som (add-decking blockObj
                        (list pltx (- flr_thick))
                        (list (+ pltx plt_dwid) (- flr_thick)))))
  (setvar 'clayer "OBJ")

  ;;;;;;;;;;;;;;;;
  ;; front channel
  ;;;;;;;;;;;;;;;;

  ;; insert point
  (setq fchanx (- 0
                  (/ plt_wid 2.0)
                  fl_ext)
        fchany (- 0 flr_thick front_chan_dpt))
  ;; take side channels into account
  (tk '(if (and left_chan
            (<= fl_ext bl_ext))
        (setq fchanx (+ fchanx left_chan_wid))))
  ;; main rect
  (som (add-rectangle blockObj
                      front_chan_len
                      front_chan_dpt
                      (list fchanx fchany 0)))
  ;; top flange line
  (som (add-polyline blockObj
                     (list fchanx (- 0 flr_thick front_chan_flg) 0)
                     (list '(0 0 0)
                           (list front_chan_len 0 0))
                     nil))
  (vla-put-Linetype obj "hidden2")
  ;; bottom flange line
  (som (add-polyline blockObj
                     (list fchanx (+ fchany front_chan_flg) 0)
                     (list '(0 0 0)
                           (list front_chan_len 0 0))
                     nil))
  (vla-put-Linetype obj "hidden2")

  ;;;;;;;;;;;;;;;
  ;; back channel
  ;;;;;;;;;;;;;;;

  ;; insert point
  (setq bchanx (- 0
                  (/ plt_wid 2.0)
                  bl_ext)
        bchany (- 0 flr_thick back_chan_dpt))
  ;; take side channels into account
  (tk '(if (and left_chan
            (<= bl_ext fl_ext))
        (setq bchanx (+ bchanx left_chan_wid))))
  ;; main rect
  (som (add-rectangle blockObj
                      back_chan_len
                      back_chan_dpt
                      (list bchanx bchany 0)))
  ;; top flange line
  (som (add-polyline blockObj
                     (list bchanx (- 0 flr_thick back_chan_flg) 0)
                     (list '(0 0 0)
                           (list back_chan_len 0 0))
                     nil))
  (vla-put-Linetype obj "hidden2")
  ;; bottom flange line
  (som (add-polyline blockObj
                     (list bchanx (+ bchany back_chan_flg) 0)
                     (list '(0 0 0)
                           (list back_chan_len 0 0))
                     nil))
  (vla-put-Linetype obj "hidden2")

  ;; left channel
  (if (setq tmp (cadr (assoc "left_chan" data)))
      (progn
        (setq chan-func (if (begins-with tmp "C") 'c-channel-pts 'mc-channel-pts)
              chanx (max fchanx bchanx)
              obj (add-shape blockObj
                             (list chanx (- flr_thick) 0)
                             tmp)
              tmp (vla-Mirror obj
                              (vlax-3d-point chanx 0 0)
                              (vlax-3d-point chanx 1 0)))
        (vla-Delete obj)
        (if mirror-p (vla-mirror-y tmp))))

  ;; right channel
  (if (setq tmp (cadr (assoc "right_chan" data)))
      (progn
        (som (add-shape blockObj
                        (list (- (/ plt_wid 2.0)
                                 (cadr (assoc "right_chan_wid" data))
                                 (- (min fr_ext br_ext)))
                              (- flr_thick)
                              0)
                        tmp))))

  (mapcar 'land-endview--draw-support '("fl" "fr" "bl" "br"))

  ;; head clearance line
  (if (> head_clr 0.0)
      (progn
        (setq tmp (add-polyline blockObj
                                (list (- (/ plt_wid 2.0)) head_clr 0)
                                (list '(0 0 0)
                                      (list plt_wid 0 0))
                                nil))
        (vla-put-Color tmp 252)
        (vla-put-Linetype tmp "HIDDEN2")))

  (setvar 'clayer clayer)
  (setq blockObj (insert-block modelSpace ins blockObj))
  (vla-put-Layer blockObj "LAND")
  (assign-xdata blockObj "EndView" (format-xdata (list (list "lr" (to-string lr))
                                                       (assoc "assembly" data)))))
(defun-r 'land-endview-draw)

(defun-q land-endview--draw-support (corner / con ext lng dir ins def)
  "Draw the landing support for CORNER in BLOCKOBJ"
  (declare (vars (corner str)
                 (blockObj vla-object))
           with-data)
  (setq dir (if (ends-with corner "l") -1 1)
        con (read (cornerkey "chan_num"))
        ext (cornerkey "ext")
        lng (cornerkey "len")
        chan-dpt (if (begins-with corner "b") back_chan_dpt front_chan_dpt))
  (cond
    ;; std post
    ((member con '(20 30))
     (setq ins (list (* dir (+ (/ plt_wid 2.0) ext))
                     (- flr_thick))
           def (land-endview--std-post (if (= con 20) 2.5 3.0)
                                  (- lng flr_thick))))
    ;; post under
    ((member con '(25 35))
     (setq ins (list (* dir (+ (/ plt_wid 2.0) ext))
                     (- (+ flr_thick chan-dpt)))
           def (land-endview--blw-post (if (= con 25) 2.5 3.0)
                                  (- lng (+ flr_thick chan-dpt)))))
    ;; hanger
    ((member con '(40 50))
     (setq ins (list (* dir (+ (/ plt_wid 2.0) ext -1.25))
                     (- flr_thick))
           def (land-endview--hanger (if (= con 40) 0.75 1.0) chan-dpt (+ lng flr_thick))))
    ;; std ledger
    ((member con '(60 61 62 66 67 68 70 71 72 76 77 78 79))
     (setq ins (list (* dir (+ (/ plt_wid 2.0) ext (cornerkey "gap")))
                     (- (+ flr_thick chan-dpt)))
           def (if (= con 76)
		   (land-endview--ledger-inv)
		   (land-endview--ledger (member con '(60 66 70 77))
					(not (member con '(66 67 68 77 78 79)))
					(member con '(66 67 68)))))))
  (if def
      ((lambda (/ obj)
         (setq obj (insert-block blockObj ins def))
         (if (= dir 1) (setq obj (mirror&delete obj ins 'y)))
         (if mirror-p (mirror&delete obj '(0 0) 'y))))))
(defun-r 'land-endview--draw-support)

(defun-q land-endview--std-post (wid lng / blockObj custom-p)
  "Returns a block containing a standard post"
  (declare (vars (wid nil (numberp wid))
                 (lng nil (numberp lng))))
  (setq blockObj (p-blk "land-post")
        custom-p (ends-with (cornerkey "supp_num") "X"))
  ;; post
  (add-rectangle blockObj wid (- (if custom-p lng (- lng 0.25))) '(0 0))
  ;; base plate
  (if (not custom-p)
      (add-rectangle blockObj (+ wid 0.5) 0.25 (list -0.25 (- lng))))
  blockObj)
(defun-r 'land-endview--std-post)

(defun-q land-endview--blw-post (wid lng / blockObj sup custom-p plt-wid)
  "Returns a block containing a post under the channel"
  (declare (vars (wid nil (numberp wid))
                 (lng nil (numberp lng))))
  (setq blockObj (p-blk "land-post-blw")
        sup (cornerkey "supp_num")
        custom-p (ends-with sup "X")
	plt-wid (cond ((member sup '("26" "36")) 6.0) ((> (safe-atoi sup) 30) 3.5) ((= T) 3.0)))
  ;; top plate
  (add-rectangle blockObj 6.0 -0.25 '(0 0))
  ;; post
  (add-rectangle blockObj wid (- (- lng (if custom-p 0.25 0.5))) '(0.25 -0.25))
  ;; base plate
  (if (not custom-p)
      (add-rectangle blockObj plt-wid 0.25 (list 0 (- lng))))
  blockObj)
(defun-r 'land-endview--blw-post)

(defun-q land-endview--hanger (dia chd lng / add-plate blockObj)
  "Returns a block containing a hanger"
  (declare (vars (dia nil (numberp dia))
                 (chd nil (numberp chd))
                 (lng nil (numberp lng))))
  (setq blockObj (p-blk "land-hanger"))
  (add-rectangle blockObj (+ dia 0.5) (- chd) (list (/ (+ dia 0.5) -2.0) 0))
  (add-rectangle blockObj dia (+ lng chd 9.0) (list (/ dia -2.0) (- (+ chd 9.0))))

  (defun add-plate (/ num ins plt-hgt plt-hgt-clip plt-wid plt-wid-clip)
    (setq num (read (substr (cornerkey "supp_num") 2 1)))
    (if (/= num 'x)
        (if (member num '(1 2 4 5 7))
            ;; square plate
            (add-rectangle blockObj -0.375 -8 (list (/ dia -2.0) lng))
            ;; clipped plate
            ((lambda(/ ins hgt hgt-clip wid wid-clip)
               (setq ins (list 1.25 lng)
                     hgt (- (cornerkey "plt_hgt"))
                     hgt-clip (+ hgt 0.75)
                     wid (- (cornerkey "plt_wid"))
                     wid-clip (+ wid 0.75))
               (add-polyline blockObj
                             ins
                             (#! '((0 0 0)
                                   (0 #hgt 0)
                                   (#wid-clip #hgt 0)
                                   (#wid #hgt-clip 0)
                                   (#wid -0.75 0)
                                   (#wid-clip 0 0)))
                             T)
               (if (= num 9)
                   ;; built-up clipped plate
                   (progn
                     (setq wid (+ wid 3.75)
                           ins (displace-pt ins '(-2.5 0)))
                     (add-rectangle blockObj wid -3 ins)
                     (add-polyline blockObj
                                   (displace-pt ins '(0 -0.25))
                                   (#! '((0 0 0) (#wid 0 0)))
                                   nil)
                     (setq ins (displace-pt ins (list 0 hgt)))
                     (add-rectangle blockObj wid 3 ins)
                     (add-polyline blockObj
                                   (displace-pt ins '(0 0.25))
                                   (#! '((0 0 0) (#wid 0 0)))
                                   nil))))))))
  (add-plate)
  blockObj)
(defun-r 'land-endview--hanger)

(defun-q land-endview--ledger (emb ang rod / blockObj shp)
  "Returns a block containing a standard ledger"
  (setq blockObj (p-blk "land-ledger")
        shp (cornerkey "angle"))
  ;; ledger angle
  (add-shape blockObj (list (if emb 0 0.375) 0) shp)
  ;; gusset plate
  (add-polyline blockObj
                (list (if emb 0 0.375) 0)
                (apply '(lambda (l1 l2 th kv)
                          (list (list (- l1 0.125) (- th) 0)
				(list kv (- th) 0)
				(list th (- kv) 0)
				(list th (- (- l2 0.125)) 0)))
		       (shape-def shp nil))
		T)
  ;; anch. plate (embed removed)
  (if (not emb)
      (add-polyline blockObj
		    (list (if emb 0.0 0.375)
			  (- (/ (cadr (shape-def shp nil)) 2.0)))
		    (apply '(lambda (x y)
			      (list (list x y 0)
				    (list x (- y) 0)
				    (list 0 (- y) 0)
				    (list 0 y 0)))
			   (if emb '(-0.25 5.0) '(-0.375 6.0)))
		    T))
  ;; clip angle
  (if ang
      ((lambda (ins)
         (vla-rotate (mirror&delete (add-shape blockObj ins "L4X3X1/4")
				    ins
				    'Y)
		     (3dpt ins)
		     (dtr -90)))
        (apply '(lambda (l1 l2 th kv)
		  (list (+ l1 1.5 (if emb 0.0 0.375)) (- (+ 0.25 th))))
               (shape-def shp nil))))
  ;; rod
  (if rod
      ((lambda (/ obj)
          (setq obj (add-polyline blockObj
				  '(0 0)
				  (list (list (+ (cornerkey "gap") 2) 0 0)
                                        (list (+ (cornerkey "gap") 2) 4 0))
				  nil))
          (offset&close obj 0.3125 T '(0 1))
          (vla-delete obj)
	  ;; slot
          (setq obj (add-polyline blockObj
				  (list (+ (cornerkey "gap") 2) 4)
				  (list (list -1.125 0 0) (list 1.125 0 0))
				  nil))
          (offset&close obj 0.34375 T '(1 1))
          (vla-delete obj))))
  blockObj)
(defun-r 'land-endview--ledger)

(defun-q land-endview--ledger-inv (/ blockObj shp ins)
  "Returns a block containing an inverted ledger"
  (setq blockObj (p-blk "land-ledger")
        shp (cornerkey "angle")
	ins '(0 -0.375))
  (vla-Rotate (add-shape blockObj ins shp) (3dpt ins) (dtr 90))
  blockObj)
(defun-r 'land-endview--ledger-inv)
