  (set-file-docstring "Functions related to landing support creation")

  (psc-include (list "shapes/shape.lsp"))

  (defun-q draw-land-support (view / conn supp ang-gus)
    "Add landing support objects to a block container"
    (declare
      with-data
      (vars (blockobj vla-object) (view sym (member view '(tv sv 3d))))
    ) ;_ declare
    (if (null ang-gus)
      (defun add-ang-gus (con ins shp)
        (add-polyline
          con
          ins
          (apply
           '(lambda (l1 l2 th kv)
              (list
                (list (- l1 0.125) (- th) 0)
                (list kv (- th) 0)
                (list th (- kv) 0)
                (list th (- (- l2 0.125)) 0)
              ) ;_ list
            ) ;_ lambda
            (shape-def shp nil)
          ) ;_ apply
          T
        ) ;_ add-polyline
      ) ;_ defun defun ang-gus
    ) ;_ if
    (foreach corner '("fl" "fr" "bl" "br")
      ( (lambda (def) (apply (car def) (cdr def)))
        (land-support-def corner view)
      ) ;_ lambda-func
    ) ;_ foreach
    (cond
      ( (= view 'tv)
        (top-view-x-brace)
      ) ;_ condif
      ( (= view '3d)
        (draw-xbrace blockobj T "Land Supports" (x-brace-defs))
      ) ;_ condif
      ( (= T) (princ))
    ) ;_ cond
  ) ;_ defun-q draw-land-support
  (defun-r 'draw-land-support)

  (defun-q land-support-def (corner view)
    "Returns a support definition based on corner and view"
    (setq conn (atoi (cornerkey "chan_num")))
    (if ((lambda (s) (and s (not (ends-with s "X")))) (cornerkey "supp_num"))
      (setq supp (atoi (cornerkey "supp_num")))
    ) ;_ if
    (cond
      ( (member conn '(20 27 30 37))
        (list
         'p-out-con ;;; post outside connection
          blockObj
          (land-support-ins corner)
          (if (member conn '(20 27)) 2.5 3.0)
          (if (cornerkey "len") (- (cornerkey "len") flr_thick (if supp 0.25 0.0)))
          supp
          view
        ) ;_ list
      ) ;_ condif
      ( (member conn '(25 35))
        (list
         'p-blw-con ;;; post below connection
          blockObj
          (land-support-ins corner)
          (if (= conn 25) 2.5 3.0)
          ( (lambda (lst) (* 90 (vl-position corner lst)))
            (if (= dir 1)
             '("fl" "fr" "br" "bl") '("br" "bl" "fl" "fr")
            ) ;_ if
          )
          (-
            (cornerkey "len")
            flr_thick
            (max front_chan_dpt back_chan_dpt)
            (if supp 0.5 0.25)
          ) ;_ -
          supp
          view
        ) ;_ list
      ) ;_ condif
      ( (member conn '(40 50))
        (list
         'h-rod-con ;;; hanger rod connection
          blockObj
          (land-support-ins corner)
          (if (= conn 40) 0.375 0.5)
          (+ (cornerkey "len") flr_thick)
          (if (begins-with corner "f") front_chan_dpt back_chan_dpt)
          supp
          view
        ) ;_ list
      ) ;_ condif
      ( (member conn '(60 61 62 66 67 68 70 71 72 77 78 79))
        (list
         'l-ang-con ;;; ledger angle connection
          blockObj
          (land-support-ins corner)
          (* dir (if (ends-with corner "l") 90 -90))
          (if (member conn '(60 66 70 77)) T)
          (if (member conn '(60 61 62 70 71 72)) T)
          (if (member conn '(66 67 68)) T)
          view
        ) ;_ list
      ) ;_ condif
      ( (and (begins-with tread_style "1") (= conn 85))
        (list 'princ '()) ; L2.5x2.5x0.25 btwn. f/b chans. req'd
      ) ;_ condif
      ( (= T)
        (list 'princ '())
      ) ;_ condif
    ) ;_ cond
  ) ;_ defun-q land-support-def
  (defun-r 'land-support-def)

  (defun-q land-support-ins (corner / con)
    "Returns the inertion point of a connection based on corner"
    (setq con (atoi (cornerkey "chan_num")))
    (list
      (* dir
        (+
          (if (begins-with corner "b") back_chan_loc 0)
          (cond
            ( (member con '(20 27))
              (if (and (begins-with corner "b") (= "1" flip_back))
                1.25 -1.25
              ) ;_ if
            ) ;_ condif
            ( (member con '(30 37))
              (if (and (begins-with corner "b") (= "1" flip_back))
                1.5 -1.5
              ) ;_ if
            ) ;_ condif
            ( (member con '(40))
              (if (and (begins-with corner "b") (= "1" flip_back))
                0.625 -0.625
              ) ;_ if
            ) ;_ condif
            ( (member con '(50))
              (if (and (begins-with corner "b") (= "1" flip_back))
                0.75 -0.75
              ) ;_ if
            ) ;_ member
            ( (member con '(25))
              (cond
                ( (begins-with corner "f") 1.5)
                ( (= "1" flip_back) -1.5)
                ( (= T) (- back_chan_wid 1.5))
              ) ;_ cond
            ) ;_ condif
            ( (member con '(35))
              (cond
                ( (begins-with corner "f") 1.75)
                ( (= "1" flip_back) -1.75)
                ( (= T) (- back_chan_wid 1.75))
              ) ;_ cond
            ) ;_ condif
            ( (member con '(60 61 62 70 71 72))
              (cond
                ( (begins-with corner "b")
                  (if (= "1" flip_back) -5 -3)
                ) ;_ condif
                ( (= T) 5)
              ) ;_ cond
            ) ;_ condif
            ( (member con '(66 67 68 77 78 79))
              (cond
                ( (and (begins-with corner "b") (= "1" flip_back))
                  (* 0.5 (- back_chan_wid))
                )
                ( (begins-with corner "b") (* 0.5 back_chan_wid))
                ( (= T) (* 0.5 front_chan_wid))
              ) ;_ cond
            ) ;_ condif
            ( (= T) 0.0)
          ) ;_ cond
        ) ;_ +
      ) ;_ *
      (* ((if (ends-with corner "l") + -) dir)
        (+
          (/ plt_wid 2.0)
          (cornerkey "ext")
          (cond
            ( (member con '(20 27)) -1.25)
            ( (member con '(25)) -1.5)
            ( (member con '(30 37)) -1.5)
            ( (member con '(35)) -1.75)
            ( (member con '(40 50)) -1.25)
            ( (member con '(60 66)) (cornerkey "gap"))
            ( (member con '(61 62 67 68)) (- (cornerkey "gap") 0.375))
            ( (member con '(70 77)) (cornerkey "gap"))
            ( (member con '(71 72 78 79)) (- (cornerkey "gap") 0.375))
            ( (= T) 0.0)
          ) ;_ cond
        ) ;_ +
      ) ;_ *
      (cond
        ( (member con '(20 27 30 37 40 50)) (- flr_thick))
        ( (= T)
          (-
            0
            flr_thick
            (if (begins-with corner "f") front_chan_dpt back_chan_dpt)
          ) ;_ -
        ) ;_ condif
        ( (not 3d-p) 0.0) ; this is handled via remove-last in called functions
      ) ;_ cond
    ) ;_ list
  ) ;_ defun-q land-support-ins
  (defun-r 'land-support-ins)

  (defun-q p-out-con (con ins wid lng bcn cfg)
    "Adds post outside connection to container
con - container
ins - insertion point
wid - HSS post width
lng - HSS post length
bcn - base connection
cfg - desired support configuration currently supports 'tv 'sv or '3d"
    (declare
      (vars
        (con vla-object)
        (ins list (point-p ins))
        (wid nil (numberp wid))
        (lng nil (numberp lng))
        (bcn nil (member bcn '(nil 21 22 23 24 26 31 32 33 34 36)))
        (cfg sym (member cfg '(tv sv 3d)))
      ) ;_ vars
    ) ;_ declare
    (cond
      ( (= cfg 'tv)
        (mapcar
         '(lambda (obj) (vla-put-layer-r obj "HIDDEN"))
          (mapcar
           '(lambda (lst) (add-polyline con (remove-last ins) lst T))
            (apply 'hss-pts-ctr (list wid wid 0.25))
          ) ;_ mapcar
        ) ;_ mapcar
      ) ;_ condif
      ( (= cfg 'sv)
        (setq ins (trans ins 1 '(0 1 0)))
        (mapcar
         '(lambda (obj) (vla-put-layer obj (if lay lay "NO PLOT")))
          (list
            (add-polyline
              con
              (mapcar '* '(-1 1) ins)
              (list
               '(-1.5 0.0)
                (list -1.5 (- lng))
                (list 1.5 (- lng))
               '(1.5 0.0)
              ) ;_ list
              T
            ) ;_ add-polyline
            (add-polyline ;;; BASE-PL
              con
              (mapcar '* '(-1 1) (mapcar '- ins (list 0 lng)))
              (cond
                ( (= bcn 31)
                 '((-4.5 0.0) (4.5 0.0) (4.5 -0.25) (-4.5 -0.25))
                ) ;_ condif
                ( (= T)
                 '((-1.75 0.0) (1.75 0.0) (1.75 -0.25) (-1.75 -0.25))
                ) ;_ condif
              ) ;_ cond
              T
            ) ;_ add-polyline
          ) ;_ list
        ) ;_ mapcar
      ) ;_ condif
      ( (and (= cfg '3d) (> lng 0))
        (mapcar
         '(lambda (obj) (if obj (vla-put-layer-r obj "LAND SUPPORTS")))
          (list
            (apply
             '(lambda (s1 s2) (vla-boolean-r s1 acSubtraction s2))
              (mapcar
               '(lambda (obj) (plineobj->solidobj obj (- lng) T))
                (mapcar
                 '(lambda (lst) (add-polyline con ins lst T))
                  (apply 'hss-pts-ctr (list wid wid 0.25))
                ) ;_ mapcar
              ) ;_ mapcar
            ) ;_ apply
            (if bcn
              (plineobj->solidobj
                (add-polyline
                  con
                  (mapcar '- ins (list 0 0 lng))
                  (post-base bcn)
                  T
                ) ;_ add-polyline
                -0.25
                T
              ) ;_ plineobj->solidobj
            ) ;_ if
          ) ;_ list
        ) ;_ mapcar
      ) ;_ condif
      ( (= T) (princ))
    ) ;_ cond
  ) ;_ defun-q p-out-con
  (defun-r 'p-out-con)

  (defun-q p-blw-con (con ins wid rot lng bcn cfg / cap-dir)
    "Adds post below connection to container
con - container
ins - insertion point
wid - HSS post width
rot - HSS post rotation
lng - HSS post length
bcn - base connection
cfg - desired support configuration currently supports 'tv 'sv or '3d"
    (declare
      (vars
        (con vla-object)
        (ins list (point-p ins))
        (wid nil (numberp wid))
        (rot nil (numberp rot))
        (lng nil (numberp lng))
        (bcn nil (member bcn '(nil 21 22 23 24 26 31 32 33 34 36)))
        (cfg sym (member cfg '(tv sv 3d)))
      ) ;_ vars
    ) ;_ declare
    (mapcar
     '(lambda (obj)
        (if (and obj (not (= cfg 'sv))) (vla-rotate obj (3dpt ins) (dtr rot)))
      ) ;_ lambda
      (cond
        ( (= cfg 'tv)
          (mapcar
           '(lambda (lst)
              (vla-put-layer-r
                (add-polyline con (remove-last ins) lst T)
                "HIDDEN"
              ) ;_ vla-put-layer-r
            ) ;_ lambda
            (cons
              (post-base (if (= wid 2.5) 26 36))
              (apply 'hss-pts-ctr (list wid wid 0.25))
            ) ;_ cons
          ) ;_ mapcar
        ) ;_ condif
        ( (= cfg 'sv)
          (setq
            ins (trans ins 1 '(0 1 0))
            cap-dir (if (begins-with corner "f") dir (- dir))
          ) ;_ setq
          (mapcar
           '(lambda (obj) (vla-put-layer obj (if lay lay "NO PLOT")))
            (list
              (add-polyline ;;; HSS
                con
                (mapcar '* '(-1 1) ins)
                (list
                 '(-1.5 -0.25)
                  (list -1.5 (- (+ lng 0.25)))
                  (list 1.5 (- (+ lng 0.25)))
                 '(1.5 -0.25)
                ) ;_ list
                T
              ) ;_ add-polyline
              (add-polyline ;;; CAP-PL
                con
                (mapcar '* '(-1 1) ins)
                (mapcar
                 '(lambda (pt) (mapcar '* pt (list cap-dir 1)))
                 '((-1.75 0.0) (4.25 0.0) (4.25 -0.25) (-1.75 -0.25))
                ) ;_ mapcar
                T
              ) ;_ add-polyline
              (add-polyline ;;; BASE-PL
                con
                (mapcar '* '(-1 1) (mapcar '- ins (list 0 (+ lng 0.25))))
                (cond
                  ( (= bcn 36)
                    (mapcar
                     '(lambda (pt) (mapcar '* pt (list cap-dir 1)))
                     '((-1.75 0.0) (4.25 0.0) (4.25 -0.25) (-1.75 -0.25))
                    ) ;_ mapcar
                  ) ;_ condif
                  ( (= T)
                   '((-1.75 0.0) (1.75 0.0) (1.75 -0.25) (-1.75 -0.25))
                  ) ;_ condif
                ) ;_ cond
                T
              ) ;_ add-polyline
            ) ;_ list
          ) ;_ mapcar
        ) ;_ condif
        ( (and (= cfg '3d) (> lng 0))
          (mapcar
           '(lambda (obj) (if obj (vla-put-layer-r obj "LAND SUPPORTS")))
            (cons
              (apply
               '(lambda (s1 s2) (vla-boolean-r s1 acSubtraction s2))
                (mapcar
                 '(lambda (obj) (plineobj->solidobj obj (- lng) T))
                  (mapcar
                   '(lambda (lst)
                      (add-polyline
                        con
                        (mapcar '- ins '(0 0 0.25))
                        lst
                        T
                      ) ;_ add-polyline
                    ) ;_ lambda
                    (apply 'hss-pts-ctr (list wid wid 0.25))
                  ) ;_ mapcar
                ) ;_ mapcar
              ) ;_ apply
              (mapcar
               '(lambda (ins lst)
                  (if lst
                    (plineobj->solidobj
                      (add-polyline con ins lst T)
                      -0.25
                      T
                    ) ;_ plineobj->solidobj
                  ) ;_ if
                ) ;_ lambda
                (list ins (mapcar '- ins (list 0 0 (+ lng 0.25))))
                (list
                  (cond
                    ( (= conn 25) (post-base 26))
                    ( (= conn 35) (post-base 36))
                    ( (= T) nil)
                  ) ;_ cond
                  (post-base bcn)
                ) ;_ list
              ) ;_ mapcar
            ) ;_ list
          ) ;_ mapcar
        ) ;_ condif
        ( (= T) (princ))
      ) ;_ cond
    ) ;_ mapcar
  ) ;_ defun-q p-blw-con
  (defun-r 'p-blw-con)

  (defun-q h-rod-con (con ins rad lng cht tcn cfg / plt)
    "Adds hanger rod connection to container
con - container
ins - insertion point
rad - hanger radius
lng - hanger rod length
cht - channel height
tcn - top connection
cfg - desired support configuration currently supports 'tv 'sv or '3d"
    (declare
      (vars
        (con vla-object)
        (ins list (point-p ins))
        (rad nil (numberp rad))
        (lng nil (numberp lng))
        (cht nil (numberp cht))
        (tcn nil (member tcn '(nil 41 42 43 44 45 47 48 49 51 52 53 54 55 57 58 59)))
        (cfg sym (member cfg '(tv sv 3d)))
      ) ;_ vars
    ) ;_ declare
    (defun plt (8x8 / v-l/r p-off hgt hgt-c wid wid-c ret nor ext obj)
      (defun v-l/r () (* dir (if (ends-with corner "l") 1 -1)))
      (defun p-off ()
        (if 8x8
          (list 0 (* rad (v-l/r)) lng)
          (list (* rad dir) (* -1.25 (v-l/r)) lng)
        ) ;_ if
      ) ;_ defun p-off 
      (if 8x8
        (setq
          ret (list '(-4 0) '(-4 -8) '(4 -8) '(4 0))
          nor (3dpt 0 1 0)
          ext (* 0.375 (v-l/r))
        ) ;_ setq
        (setq
          hgt (- (cornerkey "plt_hgt")) hgt-c (+ hgt 0.75)
          wid (- (cornerkey "plt_wid")) wid-c (+ wid 0.75)
          ret (mapcar
               '(lambda (p) (mapcar '* p (list (v-l/r) 1)))
                (#!
                 '( (0 0) (0 #hgt) (#wid-c #hgt)
                    (#wid #hgt-c) (#wid -0.75) (#wid-c 0)
                  ) ;_ quote
                ) ;_ macro-expand
              ) ;_ mapcar
          nor (3dpt -1 0 0)
          ext (* (- dir) (cornerkey "plt_thick"))
        ) ;_ setq
      ) ;_ if
      (setq obj (add-polyline con '(0 0) ret T))
      (vla-put-normal obj nor)
      (setq obj (plineobj->solidobj obj ext T))
      (vla-move obj (3dpt 0 0 0) (3dpt (mapcar '+ ins (p-off))))
      obj
    ) ;_ defun plt
    (cond
      ( (= cfg 'tv)
        (mapcar
         '(lambda (obj) (vla-put-layer-r obj "HIDDEN"))
          (cons
            (vla-addcircle con (3dpt (remove-last ins)) rad)
            (mapcar
             '(lambda (lst) (add-polyline con (remove-last ins) lst T))
              (apply 'hss-pts-ctr
                (if (= rad 0.375) '(1.25 1.25 0.1875) '(1.5 1.5 0.12))
              ) ;_ apply
            ) ;_ mapcar
          ) ;_ cons
        ) ;_ mapcar
      ) ;_ condif
      ( (and (= cfg '3d) (> lng 0))
        (mapcar
         '(lambda (obj) (if obj (vla-put-layer-r obj "LAND SUPPORTS")))
          (list
            (apply
             '(lambda (s1 s2) (vla-boolean-r s1 acSubtraction s2))
              (mapcar
               '(lambda (obj) (plineobj->solidobj obj (- cht) T))
                (mapcar
                 '(lambda (lst) (add-polyline con ins lst T))
                  (apply 'hss-pts-ctr
                    (if (= rad 0.375)
                     '(1.25 1.25 0.1875)
                     '(1.5 1.5 0.12)
                    ) ;_ if
                  ) ;_ apply
                ) ;_ mapcar
              ) ;_ mapcar
            ) ;_ apply
            (vla-addcylinder
              con
              (3dpt (mapcar '+ ins (list 0 0 (/ (- lng cht 9.0) 2.0))))
              rad
              (+ lng cht 9)
            ) ;_ vla-addcylinder
            (cond
              ( (member tcn '(41 42 44 45 47 51 52 54 55 57)) (plt T))
              ( (member tcn '(43 48 49 53 58 59)) (plt nil))
              ( (= T) nil)
            ) ;_ cond
          ) ;_ list
        ) ;_ mapcar
      ) ;_ condif
      ( (= T) (princ))
    ) ;_ cond
  ) ;_ defun-q h-rod-con
  (defun-r 'h-rod-con)

  (defun-q l-ang-con (con ins rot emb ang rod cfg)
    "Adds ledger angle connection to container
con - container
ins - insertion point
rot - ledger ang rotation
emb - T @ Embed nil @ anchor
ang - catch angle applied to ledger T/nil
rod - bent rod applied to ledger T/nil
cfg - desired support configuration
      currently 'tv or '3d, others may be added in future"
    (mapcar
     '(lambda (obj)
        (vla-Rotate obj (3dpt ins) (dtr rot))
        (vla-put-layer-r
          obj
          (if (= view '3d) "LAND SUPPORTS" "HIDDEN")
        ) ;_ vla-put-layer-r
      ) ;_ lambda
      (cond
        ( (= cfg 'tv)
          (setq ins (remove-last ins))
          (vl-remove-if 'null
            ( (lambda (def gap plt adj)
                (list
                  (if (not emb)
                    (add-polyline ; anch. plate (embed removed)
                      con
                      ins
                      (mapcar '(lambda (j) (mapcar '* plt j)) adj)
                      T
                    ) ;_ add-polyline
                  ) ;_ if
                  (add-polyline ; supp. angle
                    con
                    ins
                    (mapcar
                     '(lambda (j)
                        (mapcar '* (list (- (car def)) 4.0) j)
                      ) ;_ lambda
                      adj
                    ) ;_ mapcar
                    T
                  ) ;_ add-polyline
                  (add-polyline ; supp. angle thk.
                    con
                    (mapcar '- ins (list (caddr def) 0.0))
                   '((0.0 -4.0) (0.0 4.0))
                    nil
                  ) ;_ add-polyline
                  (if ang
                    (add-polyline ; clip angle
                      con
                      (mapcar '- ins (list (+ (car def) 1.5) 0.0))
                      (mapcar
                       '(lambda (j) (mapcar '* '(3.0 1.0) j))
                        adj
                      ) ;_ mapcar
                      T
                    ) ;_ add-polyline
                  ) ;_ if
                  (if ang
                    (add-polyline ; clip angle thk.
                      con
                      (mapcar '- ins (list (+ (car def) 1.25) 0.0))
                     '((0.0 -1.0) (0.0 1.0))
                      nil
                    ) ;_ add-polyline
                  ) ;_ if
                  (if rod
                    ( (lambda (ob1 / ob2)
                        (setq ob2 (offset&close ob1 0.3125 T 1.0))
                        (progn (vla-delete ob1) ob2)
                      ) ;_ lambda
                      (add-polyline
                        con
                        (mapcar '-
                          ins
                          (list (+ gap (if emb 0.0 -0.375) 2.0) 0.0)
                        ) ;_ mapcar
                       '((0.0 -2.375 0) (0.0 2.375 0))
                        nil
                      ) ;_ add-polyline
                    ) ;_ lambda func
                  ) ;_ if
                ) ;_ list
              ) ;_ lambda
              (shape-def (cornerkey "angle") nil)
              (cornerkey "gap")
              (if emb '(0.25 5.0) '(0.375 6.0))
             '((1 1) (1 -1) (0 -1) (0 1))
            ) ;_ lambda func
          ) ;_ vl-remove-if
        ) ;_ condif
        ( (= cfg '3d)
          (mapcar
           '(lambda (obj)
              obj
              (vla-rotate3d-r
                obj
                (3dpt ins)
                (3dpt (mapcar '+ ins '(1 0 0)))
                (dtr 90)
              ) ;_ vla-rotate3d
              (vla-rotate3d-r
                obj
                (3dpt ins)
                (3dpt (mapcar '+ ins '(0 0 1)))
                (dtr 180)
              ) ;_ vla-rotate3d
            ) ;_ lambda
            (vl-remove-if 'null
              ( (lambda (shp gap)
                  (list
                    (plineobj->solidobj ; supp. angle
                      (add-shape con (mapcar '- ins '(0 0 4)) shp)
                      8
                      T
                    ) ;_ plineobj->solidobj
                    (plineobj->solidobj ; gusset #1
                      (add-ang-gus con (mapcar '+ ins '(0 0 3.5)) shp)
                      0.25
                      T
                    ) ;_ plineobj->solidobj
                    (plineobj->solidobj ; gusset #2
                      (add-ang-gus con (mapcar '+ ins '(0 0 -3.75)) shp)
                      0.25
                      T
                    ) ;_ plineobj->solidobj
                    (if ang
                      ( (lambda (ins)
                          (plineobj->solidobj
                            (vla-rotate-r
                              (mirror&delete
                                (add-shape con ins "L4X3X1/4")
                                ins
                               'Y
                              ) ;_ vla-rotate3d
                              (3dpt ins)
                              (dtr -90)
                            ) ;_ vla-rotate-r
                            2
                            T
                          ) ;_ plineobj->solidobj
                        ) ;_ lambda
                        (mapcar '+ ins
                          (apply
                           '(lambda (h v d k)
                              (list (+ 1.5 h) (- (+ 0.25 d)) -1)
                            ) ;_ lambda
                            (shape-def shp nil)
                          ) ;_ apply
                        ) ;_ mapcar
                      ) ;_ lambda func
                    ) ;_ if
                    (if rod
                      ( (lambda (ins)
                          (vla-rotate3d-r
                            (plineobj->solidobj
                              (vlax-put-property-r
                                (vla-addcircle
                                  con
                                  (3dpt (mapcar '+ ins '(2.375 0 0)))
                                  0.3125
                                ) ;_ vla-addcircle
                                'normal
                                (3dpt '(0 1 0))
                              ) ;_ vlax-put-property-r
                              (add-polyline
                                con
                                ins
                                (list
                                 '(2.375 0 0)
                                  (list 2.375 3.1875 (calc-bulge (dtr 90)))
                                 '(1.5625 4 0)
                                  (list -1.5625 4 (calc-bulge (dtr 90)))
                                 '(-2.375 3.1875 0)
                                 '(-2.375 0 0)
                                ) ;_ list
                                nil
                              ) ;_ add-polyline
                              T
                            ) ;_ plineobj->solidobj
                            (3dpt ins)
                            (3dpt (mapcar '+ ins '(0 1 0)))
                            (dtr 90)
                          ) ;_ vla-rotate3d-r
                        ) ;_ lambda
                        (mapcar '+ ins
                          (list (+ (- gap (if emb 0.0 0.375)) 2.0) 0 0)
                        ) ;_ mapcar
                      ) ;_ lambda func
                    ) ;_ if
                    (if (not emb)
                      (plineobj->solidobj
                        (add-rectangle
                          con
                          -0.375
                          12.0
                          (mapcar '+ ins
                            (apply
                             '(lambda (h v d k)
                                (list 0 (- (+ 6.0 (/ v 2.0))) -6)
                              ) ;_ lambda
                              (shape-def shp nil)
                            ) ;_ apply
                          ) ;_ mapcar
                        ) ;_ add-rectangle
                        12.0
                        T
                      ) ;_ plineobj->solidobj
                    ) ;_ if
                  ) ;_ list
                ) ;_ lambda
                (cornerkey "angle")
                (cornerkey "gap")
              ) ;_ lambda func
            ) ;_ vl-remove-if
          ) ;_ mapcar
        ) ;_ condif
        ( (= T) (list))
      ) ;_ cond
    ) ;_ mapcar
  ) ;_ defun-q l-ang-con
  (defun-r 'l-ang-con)

  (defun-q post-base (bcn)
    "Adds baseplate to post connections"
    (cond
      ( (= bcn 21)
       '(
          (-4.5 1.5 0)
          (-4.5 -1.5 0)
          (4.5 -1.5 0)
          (4.5 1.5 0)
        )
      ) ;_ condif
      ( (member bcn '(22 23 24))
       '(
          (1.5 -1.5 0)
          (-1.5 -1.5 0)
          (-1.5 1.5 0)
          (1.5 1.5 0)
        )
      ) ;_ condif
      ( (= bcn 26)
       '(
          (-1.5 1.5 0)
          (-1.5 -4.5 0)
          (0.75 -4.5 0)
          (4.5 -0.75 0)
          (4.5 1.5 0)
        )
      ) ;_ condif
      ( (= bcn 31)
       '(
          (-4.5 1.75 0)
          (-4.5 -1.75 0)
          (4.5 -1.75 0)
          (4.5 1.75 0)
        )
      ) ;_ condif
      ( (member bcn '(32 33 34))
       '(
          (1.75 -1.75 0)
          (-1.75 -1.75 0)
          (-1.75 1.75 0)
          (1.75 1.75 0)
        )
      ) ;_ condif
      ( (= bcn 36)
       '(
          (-1.75 1.75 0)
          (-1.75 -4.25 0)
          (0.5 -4.25 0)
          (4.25 -0.5 0)
          (4.25 1.75 0)
        )
      ) ;_ condif
      ( (= T) '())
    ) ;_ cond
  ) ;_ defun-q post-base
  (defun-r 'post-base)

  (defun-q top-view-x-brace ( / def)
    "Draw x-brace representation on top view"
    (setq def
      (apply 'append
        (mapcar
         '(lambda (x b o)
            (if (= "0" x) '()
              (list (append (mapcar 'land-support-ins b) o))
            ) ;_ if
          ) ;_ lambda
          (list f_brace b_brace l_brace r_brace)
         '(("fl" "fr") ("bl" "br") ("fl" "bl") ("fr" "br"))
         '(((1.5 3.0 0)) ((1.5 3.0 0)) ((3.0 1.5 0)) ((3.0 1.5 0)))
        ) ;_ mapcar
      ) ;_ append
    ) ;_ setq
    (while def
      (mapcar
       '(lambda (lst)
          (vla-put-layer
            (add-polyline blockObj '(0 0) lst nil)
            "CENTER"
          ) ;_ vla-put-layer
        ) ;_ lambda
        (apply
         '(lambda (wp1 wp2 off)
            (mapcar
             '(lambda (off)
                (mapcar
                 '(lambda (p o) (mapcar '+ p o))
                  (mapcar
                   '(lambda (v) (subst 0.0 (last v) v))
                    (list wp1 wp2)
                  ) ;_ mapcar
                  off
                ) ;_ mapcar
              ) ;_ lambda
              (mapcar
               '(lambda (v) (mapcar '(lambda (v) (mapcar '* v off)) v))
                (mapcar
                 '(lambda (v1 v2)
                    (list
                      (mapcar '(lambda (v) (* v dir)) v1)
                      (mapcar '(lambda (v) (* v dir)) v2)
                    ) ;_ list
                  ) ;_ lambda
                  (if (apply '< (remove-last off))
                   '((-1 -1 0) (+1 -1 0))
                   '((+1 +1 0) (+1 -1 0))
                  ) ;_ if
                  (if (apply '< (remove-last off))
                   '((+1 +1 0) (-1 +1 0))
                   '((-1 -1 0) (-1 +1 0))
                  ) ;_ if
                ) ;_ mapcar
              ) ;_ mapcar
            ) ;_ mapcar
          ) ;_ lambda
          (car def)
        ) ;_ apply
      ) ;_ mapcar
      (setq def (cdr def))
    ) ;_ while
  ) ;_ defun-q top-view-x-brace
  (defun-r 'top-view-x-brace)

  (defun-q x-brace-defs ( / c-dpt z-ins)
    "Returns definition for x-bracing based on XDATA"
    (declare with-data)
    (setq c-dpt '("front" "back" "left" "right"))
    (mapcar! '(lambda (s) (symcat '(#s "_chan_dpt"))) 'c-dpt)
    (setq c-dpt (apply 'max (vl-remove-if 'null c-dpt)))
    (setq z-ins (- (+ flr_thick c-dpt)))
    (vl-remove-if 'null
      (mapcar
       '(lambda (pos bay)
          (if (= "1" (symcat '(#pos "_brace")))
            (append
              (apply
               '(lambda (p1 p2)
                  (list
                    (rcons (mid-2-pts p1 p2) (- z-ins 6))
                    (if (= (car p1) (car p2)) '(1 0 0) '(0 1 0))
                  ) ;_ list
                ) ;_ lambda
                (mapcar 'remove-last (mapcar 'land-support-ins bay))
              ) ;_ apply
              (mapcar
               '(lambda (var) (symcat '(#pos "_brace" #var)))
               '("_npanels" "_wid" "_hgt")
              ) ;_ mapcar
            ) ;_ append
          ) ;_ if
        ) ;_ lambda
       '("f" "b" "l" "r")
       '(("fl" "fr") ("bl" "br") ("fl" "bl") ("fr" "br"))
      ) ;_ mapcar
    ) ;_ vl-remove-if
  ) ;_ defun-q x-brace-defs
  (defun-r 'x-brace-defs)

  (defun-q x-brace-out (w h o / w h o)
    "Calculates outline for x-bracing based on XDATA"
    (psc-include '("shapes/line.lsp"))
    (mapcar '(lambda (a) (set a (float (eval a)))) '(w h o))
    (mapcar
     '(lambda (l)
        (mapcar '(lambda (v) (mapcar '- v (list (/ w 2.0) 0.0))) l)
      ) ;_ lambda
      (mapcar
       '(lambda (x)
          (mapcar 'list
            x
            (mapcar
             '(lambda (x o)
                (line-val
                  (line-offset
                    (pt-slope (/ (- h) w) 0 0)
                    o
                  ) ;_ line-offset
                  x
                ) ;_ line-val
              ) ;_ lambda
              (list 0 w w 0)
              (list o o (- o) (- o))
            ) ;_ mapcar
          ) ;_ mapcar
        ) ;_ lambda
        (list (list 0 w w 0) (list w 0 0 w))
      ) ;_ mapcar
    ) ;_ mapcar
  ) ;_ defun-q x-brace-out
  (defun-r 'x-brace-out)

  (defun-q draw-xbrace (con 3dp lay def)
    "Adds objects for x-bracing to container"
    (map-apply 'add-x-bay def)
  ) ;_ defun-q draw-xbrace
  (defun-r 'draw-xbrace)

  (defun-q add-x-bay (ins nor qty wid hgt)
    "Adds a single x-brace bay"
    (while (> qty 0)
      (map-apply
       'add-x-pan
        (mapcar 'list (x-brace-out wid (- hgt 12.0) 1.25) '(1.5 -1.5))
      ) ;_ map-apply
      (if (> qty 1) (xbrace-hor))
      (setq qty (1- qty))
      (setq ins (mapcar '- ins (if 3dp (list 0 0 hgt) (list 0 hgt))))
    ) ;_ while
  ) ;_ defun-q add-x-bay
  (defun-r 'add-x-bay)

  (defun-q add-x-pan (pts ext / obj)
    "Adds the pitched members to an x-bracing bay"
    (setq obj (add-polyline con '(0 0 0) pts T))
    (vla-put-normal obj (3dpt nor))
    (if 3dp (setq obj (plineobj->solidobj obj ext T)))
    (vla-put-layer obj lay)
    (vla-move obj (3dpt 0 0 0) (3dpt ins))
  ) ;_ defun-q add-x-pan
  (defun-r 'add-x-pan)

  (defun-q xbrace-hor ( / h-nor h-ins h-rec)
    "Adds the horizontal member to an x-bracing bay if required"
    (if 3dp
      (setq
        h-nor (if (> (car nor) 0) '(0 1 0) '(1 0 0))
        h-ins (mapcar '* h-nor (lst* (* 0.5 wid) 2))
        h-ins (mapcar '- ins (rcons h-ins (- hgt 6.0)))
        h-rec '((-1.5 1.5) (-1.5 -1.5) (1.5 -1.5) (1.5 1.5))
      ) ;_ setq
      (setq
        h-nor '(0 0 1)
        h-ins (mapcar '- ins (list (* 0.5 wid) (- hgt 6.0)))
        h-rec (#! '((0.0 1.5) (#wid 1.5) (#wid -1.5) (0.0 -1.5)))
      ) ;_ setq
    ) ;_ if
    (setq obj (add-polyline con '(0 0 0) h-rec T))
    (vla-put-normal obj (3dpt h-nor))
    (vla-move obj (3dpt 0 0 0) (3dpt h-ins))
    (if 3dp (setq obj (plineobj->solidobj obj wid T)))
    (vla-put-layer obj (if lay lay "NO PLOT"))
  ) ;_ defun-q xbrace-hor
  (defun-r 'xbrace-hor)
