;; Functions to update the landing XDATA specification over time

(setq land-version 16)

;; derived key migrations:
;; 2-6, 9-13, 16

;; rename the ledger connections with side channels
(defun land-migration-1 (data / chankey det code)
  (foreach corner '("fl" "fr" "bl" "br")
           (setq chankey (strcat corner "_chan")
                 det (xd-value chankey data)
                 code (substr det 1 2))
           (if (member code '("60" "61" "62" "70" "71" "72"))
               (subst-key! chankey
                           (strcat (substr det 1 9)
                                   " w/SC"
                                   (substr det 10))
                           'data)))
  data)

(defun land-migration-7 (data / chankey det)
  (foreach corner '("fl" "fr" "bl" "br")
           (setq chankey (strcat corner "_chan")
                 det (xd-value chankey data))
           (if (member (substr det 1 1) '("6" "7"))
               (progn
                 (add-to-list 'data (list (strcat corner "_angle") "L4x4x3/8"))
                 (add-to-list 'data (list (strcat corner "_gap") 1.0)))))
  data)

;; add plate thickness
(defun land-migration-8 (data / chankey det)
  (foreach corner '("fl" "fr" "bl" "br")
           (setq chankey (strcat corner "_chan")
                 det (xd-value chankey data))
           (if (member (substr det 1 1) '("4" "5"))
               (add-to-list 'data (list (strcat corner "_plt_thick") 0.375))))
  data)
           
;; remove " chars from landing details
(defun land-migration-14 (data / chankey det)
  (foreach corner '("fl" "fr" "bl" "br")
           (setq chankey (strcat corner "_chan")
                 det (xd-value chankey data))
           (subst-key! chankey (vl-string-subst "" "\"" det) 'data))
  data)

;; add *_brace_hold keys
(defun land-migration-15 (data / bkey)
  (foreach side '("f" "b" "l" "r")
           (setq bkey (strcat side "_brace"))
           (if (= (xd-value bkey data) "1")
               (add-to-list 'data (list (strcat bkey "_hold") 6.0))))
  data)
