(set-file-docstring "")

  ;;; functions to draw guard rails
  (psc-include
   '( "shapes/line.lsp"
      "rail/derived.lsp"
      "rail/util.lsp"
      "rail/topview.lsp"
    ) ;_ quoted-list
  ) ;_ psc-include

  ;;; draw a guard rail section
  (defun draw-guard-rail (dat ins / blockObj clayer obj)
    (setup-env nil)
    (setq
      blockObj (p-blk "guard")
      clayer (vla-getvar 'clayer)
    ) ;_ setq
    (vla-setvar 'clayer "0")
    (add-calcd-objs blockObj nil (calc-guard-rail dat))
    (vla-setvar 'clayer "RAILS")
    (setq obj (insert-block modelSpace ins blockObj))
    (vla-setvar 'clayer clayer)
    obj
  ) ;_ defun draw-guard-rail

  ;;; draw a guard rail top view
  (defun draw-guard (dat ins / g-hand g-text ret blk lay)
    (defun g-hand ( / plst cntr pipe)
      (setq
        plst (filter (calc-guard-rail dat) '(assoc "handrail" (car x)))
        plst (map-apply '(lambda (x y z) (list x (- z))) (cdar plst))
        cntr (if fab-p (add-polyline blk '(0 0) (uniquify plst) nil))
        pipe (add-polyline blk '(0 0) (fillet-all-pts plst 2.25) nil)
      ) ;_ setq
      (if fab-p
        (add-to-list 'ret (vla-put-layer-r cntr "CENTER"))
        (vla-put-Color pipe 140)
      ) ;_ if
      (add-to-list 'ret
        (vla-put-Layer-r
          (offset&close pipe 0.83 T nil)
          (if fab-p "RAILS" "NO PLOT")
        ) ;_ vla-put-Layer-r
      ) ;_ add-to-list
      (if fab-p (vla-delete pipe) (vla-put-ConstantWidth pipe 1.66))
    ) ;_ defun g-hand
    (defun g-text ( / style x-off y-off r-lst t-rot mirror)
      (setq
        style (read guard_sty)
        x-off (/ guard_lng 2.0)
        t-rot (if (<= (/ pi 1.99999) rot (* pi 1.5)) pi 0)
      ) ;_ setq
      (setq y-off
        (cond
          ( (= "1" guard_hand) -8.66)
          ( (member style '(100 200)) -7.16)
          ( (= T) -5.5)
        ) ;_ cond
      ) ;_ setq
      (setq blockobj blk)
      (set-active-textstyle "Calibri")
      (set-active-layer "No Plot")
      (add-to-list 'r-lst
        (add-rail-tag
          (list ((if (= 0 t-rot) + -) (/ guard_lng 2.0) 15) y-off)
          'MiddleCenter
          (strcat "(" (itoa guard_qty) ")")
        ) ;_ add-rail-tag
      ) ;_ add-to-list
      (set-active-layer "Rails")
      (add-to-list 'r-lst
        (add-rail-tag
          (list x-off y-off)
          'MiddleCenter
          (strcat "GR" number "-" level (to-string sequence))
        ) ;_ add-rail-tag
      ) ;_ add-to-list
      (mapcar
       '(lambda (obj) (vla-put-rotation obj t-rot))
        (mapcar (compose '(ename>vlobj cdar)) r-lst)
      ) ;_ mapcar
    ) ;_ defun g-text
    (setq lay (getvar "clayer"))
    (with-data dat
     '( (setq blk (p-blk (if fab-p "top-view" assembly)))
        (setq ret (g-post 'dcl guard_lng))
        (if (not fab-p) (g-text))
        (if (= "1" guard_hand) (g-hand))
      ) ;_ quoted list
    ) ;_ with-data
    (if fab-p (mapcar '(lambda (ob) (vla-rotate ob (3dpt 0 0) pi)) ret))
    (setq blk (insert-block modelSpace ins blk))
    (if (not fab-p) (assign-xdata blk "Guard" (format-xdata dat)))
    (set-active-layer lay)
    blk
  ) ;_ defun draw-guard

  (defun-q single-guard ( / hr-rem-lst)
    "Processes to draw single guard"
    (setq hr-rem-lst
     '( "hrl_con" "hrl_off" "hrl_ang" "hrl_gap"
        "hrr_con" "hrr_off" "hrr_ang" "hrr_gap"
      ) ;_ quoted-list
    ) ;_ setq
    (setup-env (= app "3D_Guard"))
    (if ent (setq rot (vla-get-rotation (ename>vlobj ent))))
    (remove! 'data '(member (car x) guard-derived-keys))
    (if (= "0" guard_hand) (remove! 'data '(member (car x) hr-rem-lst)))
    (setq obj
      ( (if (= app "3D_Guard") draw-3d_guard draw-guard)
        (guard-add-derived-keys data)
        (if ent
          (get-ins-and-delete ent)
          (getpoint "\nInsertion Point: ")
        ) ;_ if
      ) ;_ draw-guard -or- draw-3d_guard
    ) ;_ setq
    (if rot (vla-put-rotation obj rot))
    (vlobj>ename obj)
  ) ;_ defun-q single-guard
  (defun-r 'single-guard)

  (defun-q multi-guard (delta)
    "Passes data from multiple guards to single-guard process"
    (if (remove! 'delta '(null (cadr x)))
      (mapcar
       '(lambda (ent / )
          (setq
            dat (read-xdata ent "")
            app (xd-value "app_id" dat)
          ) ;_ setq
          (map-apply
           '(lambda (k v)
              (if (assoc k dat)
                (subst-key! k v 'dat)
                (append! 'dat (list (list k v)))
              ) ;_ if
            ) ;_ lambda
            delta
          ) ;_ map-apply
          (with-data dat '((single-guard)))
        ) ;_ lambda
        sel
      ) ;_ mapcar
    ) ;_ if
  ) ;_ defun-q multi-guard
  (defun-r 'multi-guard)

  ;;; draw both guard rails at stair side
  (defun draw-stair-guard-rails (ins side / dat-lst off obj)
    (expect-with-data "draw-stair-guard-rails")
    (setq dat-lst (stair-xdata->guard-xdata))
    (filter! 'dat-lst '(= side (substr (xd-value "location" x) 1 1)))
    (foreach dat dat-lst
      (setq
        off (calc-stair-guard-ins '2d dat)
        obj (draw-guard-rail dat (mapcar '+ ins off))
        obj (if (= dir -1)
              (mirror&delete obj (list (car ins) 0) 'y)
              obj
            ) ;_ if
        obj (move-to-bottom obj modelSpace)
      ) ;_ setq
    ) ;_ foreach
  ) ;_ defun draw-stair-guard-rails

  ;;; return all the calcd objs for a guard rail
  (defun calc-guard-rail (dat / all-con add-lay post-vars out-post-cl n-mid-posts post-cl)
    (defun all-con (sym / lst)
      (setq
        lst (list lpost_con mpost_con rpost_con)
        lst (cons sym (mapcar 'read lst))
      ) ;_ setq
      (apply '= lst)
    ) ;_ defun all-con
    (defun post-vars (mat-thk)
      (setq
        out-post-cl (- guard_lng mat-thk)
        n-mid-posts (rail-calc-n-mid-posts out-post-cl)
        post-cl (/ out-post-cl (1+ n-mid-posts))
      ) ;_ setq
    ) ;_ defun post-vars
    (defun add-lay (lst / lay def pts)
      (setq
        lay (cond (fab-p "Solid") (3d-p "Rail Frame") (T "Rails"))
        def (car lst) pts (cdr lst) lay (list "layer" lay)
        def (if (null (assoc "layer" def)) (add-to-list 'def lay) def)
      ) ;_ setq
      (cons def pts)
    ) ;_ defun add-lay
    (mapcar 'add-lay
      (with-data dat
       '( (cond
            ( (all-con 'X11)
              (append
                (list (post-rail-post guard_lng guard_hgt))
                (post-rail-tabs guard_lng guard_hgt)
              ) ;_ append
            ) ;_ condif
            ( (all-con 'X12)
              (post-vars 1.5)
              (append
                (guard-rail-box-frame guard_lng guard_hgt)
                (attempt-apply
                  (read
                    (strcat "box-rail-" (to-string (read guard_sty)))
                  ) ;_ read
                  (list guard_lng guard_hgt)
                ) ;_ attempt-apply
              ) ;_ append
            ) ;_ condif
            ( (zerop (read guard_sty))
              (post-vars 0.0)
              (append
                (cane-rail-frame out-post-cl guard_hgt)
                (cane-rail-base-pl)
              ) ;_ append
            ) ;_ condif
            ( (= T)
              (post-vars 1.5)
              (append
                (guard-rail-std-frame guard_lng guard_hgt)
                (guard-rail-base-pl)
                (guard-rail-kick-pl)
                (guard-rail-knife-pl)
                (attempt-apply
                  (read
                    (strcat "guard-rail-" (to-string (read guard_sty)))
                  ) ;_ read
                  (list guard_hgt guard_lng)
                ) ;_ attempt-apply
              ) ;_ append
            ) ;_ condif
          ) ;_ cond
        ) ;_ quoted-list
      ) ;_ with-data
    ) ;_ add-lay
  ) ;_ defun calc-guard-rail

  ;;; return a list of calcd-polys and calcd-pipes
  (defun guard-rail-std-frame (len hgt / lst box bpl ret i)
    (setq lst '(("type" "poly") ("extrude" -1.5)))
    ;; l-post
    (setq
      box (if (= (read (?s lpost_con)) 52) 1.5 0.0)
      bpl (if (member (read (?s lpost_con)) '(41 42 48)) 0.25 0.0)
    ) ;_ setq
    (add-to-list 'ret
      (list
        (list 0 (+ (- (?? lpost_ext post-ext)) bpl) 0)
        (list 0 hgt 0)
        (list 1.5 (- hgt 1.5) 0)
        (list 1.5 (+ (- (?? lpost_ext post-ext)) box bpl) 0)
      ) ;_ list
    ) ;_ add-to-list
    ;; r-post
    (setq
      box (if (= (read (?s rpost_con)) 52) 1.5 0.0)
      bpl (if (member (read (?s rpost_con)) '(41 42 48)) 0.25 0.0)
    ) ;_ setq
    (add-to-list 'ret
      (list
        (list len (+ (- (?? rpost_ext post-ext)) bpl) 0)
        (list len hgt 0)
        (list (- len 1.5) (- hgt 1.5) 0)
        (list (- len 1.5) (+ (- (?? rpost_ext post-ext)) box bpl) 0)
      ) ;_ list
    ) ;_ add-to-list
    ;; m-posts
    (setq
      i 0
      box (mapcar '?s (list lpost_con rpost_con))
      box (if (member 52 (mapcar 'read box)) 1.5 0.0)
      bpl (if (member (read (?s mpost_con)) '(41 42 48)) 0.25 0.0)
    ) ;_ setq
    (repeat n-mid-posts
      (inc! 'i)
      (add-to-list 'ret
        (rect-pts
          1.5
          (- hgt (- (?? mpost_ext post-ext)) 1.5 box bpl)
          (list (* i post-cl) (+ (- (?? mpost_ext post-ext)) box bpl) 0)
        ) ;_ rect-pts
      ) ;_ add-to-list
    ) ;_ repeat
    ;; top cap
    (add-to-list 'ret
      (list
        (list 0 hgt 0) (list len hgt 0)
        (list (- len 1.5) (- hgt 1.5) 0) (list 1.5 (- hgt 1.5) 0)
      ) ;_ list
    ) ;_ add-to-list
    ;; bottom cap
    (if (> box 0.0)
      (add-to-list 'ret
        (mapcar 'list
          (list
            (if (= 52 (read (?s lpost_con))) 0.0 1.5)
            1.5 (- len 1.5)
            (if (= 52 (read (?s rpost_con))) len (- len 1.5))
          ) ;_ list
          (mapcar
           '(lambda (y) (+ y (- (min lpost_ext rpost_ext))))
           '(0.0 1.5 1.5 0.0)
          ) ;_ mapcar
         '(0 0 0 0)
        ) ;_ mapcar
      ) ;_ mapcar
    ) ;_ if
    (mapcar! '(lambda(x) (cons lst x)) 'ret)
  ) ;_ defun guard-rail-std-frame

  ;;; return a list of calcd-polys and calcd-pipes for a box frame guard
  (defun guard-rail-box-frame (len hgt / lst ret i)
    (setq lst '(("type" "poly") ("extrude" -1.5)))
    ;;; left post
    (set 'ret
      (list
        (list
         '(0.0 0.0 0)
          (list 0.0 hgt 0)
          (list 1.5 (- hgt 1.5) 0)
         '(1.5 1.5 0)
        ) ;_ list
      ) ;_ list
    ) ;_ set
    ;;; right post
    (append! 'ret (list (mirror-pts-v (car ret) (/ len 2.0))))
    ;;; t&b caps
    (apply
     '(lambda (p1 p2 / p2)
        (setq p2 (reverse p2))
        (append! 'ret
          (list
            (list (car p1) (cadddr p1) (car p2) (cadddr p2))
            (list (cadr p1) (caddr p1) (cadr p2) (caddr p2))
          ) ;_ list
        ) ;_ append!
      ) ;_ lambda
      ret
    ) ;_ apply
    ;;; mid posts
    (setq i 0)
    (repeat n-mid-posts
      (inc! 'i)
      (add-to-list 'ret
        (rect-pts 1.5 (- hgt 3.0) (list (* i post-cl) 1.5 0))
      ) ;_ add-to-list
    ) ;_ repeat
    (mapcar! '(lambda(x) (cons lst x)) 'ret)
  ) ;_ defun guard-rail-box-frame

  ;;; return a calcd-poly list for a single post guard
  (defun post-rail-post (len hgt)
    (list
     '(("type" "poly") ("extrude" -1.5))
      (list (- (/ len 2.0) 0.75) 0.0 0)
      (list (- (/ len 2.0) 0.75) hgt 0)
      (list (+ (/ len 2.0) 0.75) hgt 0)
      (list (+ (/ len 2.0) 0.75) 0.0 0)
    ) ;_ list
  ) ;_ defun post-rail-post

  (defun cane-rail-frame (len hgt / bpl lst mid ret)
    (setq
      bpl  (list lpost_con mpost_con rpost_con)
      bpl  (mapcar (compose '(?s guard-con-read ?s)) bpl)
      bpl  (mapcar '(lambda (i) (if (= 30 i) 0.1875 0.0)) bpl)
      lst '(("type" "pipe") ("radius" 0.83) ("cl" "1"))
    ) ;_ setq
    (repeat n-mid-posts (add-to-list 'mid (+ (?n (last mid)) post-cl)))
    (add-to-list 'ret
      (append
       '(("bulge" (0.0 0.0)))
        (#!
         '( (0.0 #(car bpl) 0.0) (0.0 #hgt 0.0)
            (#len #hgt 0.0) (#len #(caddr bpl) 0.0)
          ) ;_ quoted-list
        ) ;_ macro-expand
        (if (>= hgt 24.0)
          (#! '( ((0.0 4.0) (1 0)) ((#len 4.0) (-1 0))))
        ) ;_ if
        (mapcar '(lambda (x) (#! '((#x #hgt) (0 -1)))) mid)
      ) ;_ append
    ) ;_ add-to-list
    (mapcar
     '(lambda (x)
        (add-to-list 'ret
          (append
           '(("bulge" (0.0 0.542)))
            (#! '( (#x #(car bpl) 0.0) (#x #(- hgt 0.83) 0.0)))
            (if (>= hgt 24.0)
              (#! '( ((#x 4.0) (-1 0)) ((#x 4.0) (1 0))))
            ) ;_ if
          ) ;_ append
        ) ;_ add-to-list
      ) ;_ lambda
      mid
    ) ;_ mapcar
    (if (>= hgt 24.0)
      (mapcar
       '(lambda (x / off)
          (setq x (+ x 0.83) off (+ x post-cl -1.66))
          (add-to-list 'ret
            (append
             '(("bulge" (0.542 0.542)))
              (#! '( (#x 4.0 0.0) (#off 4.0 0.0)))
            ) ;_ append
          ) ;_ add-to-list
        ) ;_ lambda
        (cons 0 mid)
      ) ;_ mapcar
    ) ;_ if
    (mapcar! '(lambda(x) (cons (cons (car x) lst) (cdr x))) 'ret)
  ) ;_ defun cane-rail-frame

;;;;;;;;;;;;;;;
;; fill panels
;;;;;;;;;;;;;;;

  (defun guard-rail-100 (hgt len / ret)
    (add-to-list 'ret
      (cons
        (list
         '("type" "pipe")
          (list "ins" (list 0.3125 (- hgt 20.83) 0.83))
          (list "radius" 0.83)
        ) ;_ list
        (list '(0 0 0) (list (- len 0.625) 0 0))
      ) ;_ cons
    ) ;_ add-to-list
    (if guard_hand (guard-rail-add-hr 100))
    ret
  ) ;_ defun guard-rail-100

  (defun guard-rail-200 (hgt len / i pts-lst offset y ret)
    ;; interior tube steel
    (setq i 0)
    (repeat (1+ n-mid-posts)
      (progn (add-to-list 'pts-lst (+ 1.5 (* i post-cl))) (inc! 'i))
    ) ;_ repeat
    (mapcar!
     '(lambda (x) (rect-pts (- post-cl 1.5) 1.5 (list x (- hgt 6) 0)))
     'pts-lst
    ) ;_ mapcar!
    (foreach p pts-lst
      (add-to-list 'ret (cons '(("type" "poly") ("extrude" -1.5)) p))
    ) ;_ foreach
    ;; midlines
    (setq pts-lst nil y (- hgt 14.33))
    (if location (setq guard_hand nil))
    (while (>= y 4.705) (add-to-list 'pts-lst y) (setq y (- y 5.535)))
    (if (= guard_hand "0") (add-to-list 'pts-lst (- hgt 9.08)))
    (mapcar!
     '(lambda (y) (#! '((0.3125 #y 0.83) (#(- len 0.3125) #y 0.83))))
     'pts-lst
    ) ;_ mapcar!
    (foreach p pts-lst
      (add-to-list 'ret (cons '(("type" "pipe") ("radius" 0.83)) p))
    ) ;_ foreach
    (if guard_hand (guard-rail-add-hr 200))
    ret
  ) ;_ defun guard-rail-200

  (defun guard-rail-300 (hgt len / i offset y ret)
    ;; midlines
    (setq i 0)
    (repeat (1+ n-mid-posts)
      (setq offset (+ 1.5 3.75) y (- hgt 1.5))
      (while (>= y 5.375)
        (setq y (- y offset))
        (add-to-list 'ret
          (cons
           '(("type" "poly") ("ins" (0 0 -0.375)) ("extrude" -0.75))
            (rect-pts
              (- post-cl 1.5) 1.5 (list (+ 1.5 (* i post-cl)) y 0)
            ) ;_ rect-pts
          ) ;_ cons
        ) ;_ add-to-list
      ) ;_ while
      (inc! 'i)
    ) ;_ repeat
    (if guard_hand (guard-rail-add-hr 300))
    ret
  ) ;_ defun guard-rail-300

  (defun guard-rail-400 (hgt len / i offset x ret)
    ;; interior tube steel
    (setq i 0)
    (repeat (1+ n-mid-posts)
      (add-to-list 'ret
        (cons
         '(("type" "poly") ("extrude" -1.5))
          (rect-pts
            (- post-cl 1.5) 1.5 (list (+ 1.5 (* i post-cl)) 5.5 0)
          ) ;_ rect-pts
        ) ;_ cons
      ) ;_ add-to-list
      (inc! 'i)
    ) ;_ repeat
    ;; pickets
    (setq
      i 0
      spc (rail-calc-picket-spacing (- post-cl 1.5))
      offset (+ 0.5 spc)
    ) ;_ setq
    (repeat (1+ n-mid-posts)
      (setq x (+ 1.5 spc))
      (while (< x (- post-cl 1.5))
        (add-to-list 'ret
          (cons
           '(("type" "poly") ("ins" (0 0 -0.5)) ("extrude" -0.5))
            (rect-pts
              0.5 (- hgt 8.5) (list (+ x (* i post-cl)) 7.0 0)
            ) ;_ rect-pts
          ) ;_ cons
        ) ;_ add-to-list
        (setq x (+ x offset))
      ) ;_ while
      (inc! 'i)
    ) ;_ repeat
    (if guard_hand (guard-rail-add-hr 400))
    ret
  ) ;_ defun guard-rail-400

  (defun box-rail-100 (len hgt / ret)
    (add-to-list 'ret
      (cons
        (list
         '("type" "pipe")
          (list "ins" (list 0.3125 (- hgt 20.83) 0.83))
          (list "radius" 0.83)
        ) ;_ list
        (list '(0 0 0) (list (- len 0.625) 0 0))
      ) ;_ cons
    ) ;_ add-to-list
    ret
  ) ;_ defun box-rail-100

  (defun box-rail-200 (len hgt / x-off y-off ret)
    (setq x-off 0.75)
    (repeat (1+ n-mid-posts)
      (foreach y-off '(6.0 10.5) ;;; inline tube steel
        (add-to-list 'ret
          (cons
           '(("type" "poly") ("extrude" -1.5))
            (rect-pts
              (- post-cl 1.5)
              1.5
              (list (+ x-off 0.75) (- hgt y-off) 0)
            ) ;_ rect-pts
          ) ;_ cons
        ) ;_ add-to-list
      ) ;_ foreach
      (setq x-off (+ x-off post-cl))
    ) ;_ repeat
    (setq y-off (- hgt 14.33))
    (repeat 5 ;; midlines
      (add-to-list 'ret
        (list
         '(("type" "pipe") ("radius" 0.83))
          (list 0.3125 y-off 0.83)
          (list (- len 0.3125) y-off 0.83)
        ) ;_ list
      ) ;_ add-to-list
      (setq y-off (- y-off (+ 3.875 1.66)))
    ) ;_ repeat
    ret
  ) ;_ defun box-rail-200

  (defun box-rail-300 (len hgt / x-off y-off ret)
    (setq x-off 0.75)
    (repeat (1+ n-mid-posts)
      (setq y-off (- hgt 6.75))
      (repeat 7 ;; midlines
        (add-to-list 'ret
          (cons
           '(("type" "poly") ("ins" (0 0 -0.375)) ("extrude" -0.75))
            (rect-pts
              (- post-cl 1.5)
              1.5
              (list (+ x-off 0.75) y-off 0)
            ) ;_ rect-pts
          ) ;_ cons
        ) ;_ add-to-list
        (setq y-off (- y-off (+ 3.75 1.5)))
      ) ;_ repeat
      (setq x-off (+ x-off post-cl))
    ) ;_ repeat
    ret
  ) ;_ defun box-rail-300

  (defun box-rail-400 (len hgt / i cen gap ins ret)
    (setq
      i 1
      cen 0.75
      gap (rail-calc-picket-spacing (- post-cl 1.5))
    ) ;_ setq
    (repeat (1+ n-mid-posts)
      (setq ins (+ cen 0.75 gap))
      (while (<= ins (- (* post-cl i) gap))
        (add-to-list 'ret
          (cons
            (#!
             '(("type" "poly") ("ins" (#ins 1.5 -0.5)) ("extrude" -0.5))
            ) ;_ macro-expand
            (rect-pts 0.5 (- hgt 3.0) '(0 0))
          ) ;_ cons
        ) ;_ add-to-list
        (setq ins (+ ins gap 0.5))
      ) ;_ while
      (setq cen (+ cen post-cl) i (1+ i))
    ) ;_ repeat
    ret
  ) ;_ defun box-rail-400

  (defun post-rail-tabs (len hgt / tab-wid ret-lst)
    (setq tab-wid (/ (- len 1.5) 2.0))
    (foreach x (list 0.0 (+ 1.5 tab-wid))
      (foreach z (mapcar '(lambda (d) (- hgt d)) '(5.0 35.0))
        (add-to-list 'ret-lst (list x z))
      ) ;_ foreach
    ) ;_ foreach
    (mapcar!
     '(lambda (ins)
        (cons
         '(("type" "poly") ("ins" (0 0 -0.625)) ("extrude" -0.25))
          (rect-pts tab-wid 2.0 ins)
        ) ;_ cons
      ) ;_ lambda
     'ret-lst
    ) ;_ mapcar
  ) ;_ defun post-rail-tabs

  (defun guard-rail-add-hr (sty / calc-bxy add-fill hgt off lay con lst)
    (defun calc-bxy (ang-sym / ang gap brk x-ang y-ang d-lng)
      (setq
        ang   (?n (eval ang-sym))
        gap   (?n (if (= ang-sym 'hrl_ang) hrl_gap hrr_gap))
        brk   ((if (= ang-sym 'hrl_ang) - +) (+ 1.25 off))
        x-ang (+ (- gap 1.25) (abs off) 0.625)
        y-ang (abs (* x-ang (tan (dtr ang))))
        d-lng (if (= con 'X13) 18.0 6.0)
      ) ;_ setq
      (if (and (= sty 200) (= con 'X13) (<= (+ 1.25 off) -3.5))
        (add-fill)
      ) ;_ if
      (cons brk
        (if fab-p
          (list
            (+ brk ((if (= ang-sym 'hrl_ang) - +) (* d-lng (cos (dtr ang)))))
            ((if (= con 'X14) - +) (* d-lng (sin (dtr ang))))
          ) ;_ list
          (list
            ((if (= ang-sym 'hrl_ang) - +) (+ 1.25 off x-ang))
            ((if (= con 'X14) - +) y-ang)
          ) ;_ list
        ) ;_ if
      ) ;_ cons
    ) ;_ defun calc-bxy
    (defun add-fill ( / def ins shp)
      (setq
        def '(("type" "poly") ("extrude" -0.25))
        ins  (filter ret '(member "pipe" (mapcar 'cadr (car x))))
        ins  (list (cadadr (car ins)) 0.0)
        ins  (cons (if (= ang-sym 'hrl_ang) 1.5 (- len 1.5)) ins)
        shp '((0 0) (9.5 0) (9.5 0.5) (0.5 6) (0 6))
        shp  (if (= ang-sym 'hrr_ang) (mirror-pts-v shp 0) shp)
        shp  (displace-pts shp ins)
      ) ;_ setq
      (add-to-list 'ret (cons def shp))
    ) ;_ defun add-fill
    (setq hgt (- guard_hgt (if (< sty 300) 9.08 7.955)))
    (setq off (?n hrl_off) con (read (?s hrl_con)))
    (setq lay (cond (fab-p "Rails") ((or tv-p 3d-p) "Rail Handgrab") (T "Solid")))
    (setq lst
      (cond
        ( (member con '(X13 X14))
          (apply
           '(lambda (b x y)
              (list (list x (+ hgt y) 2.33) (list b hgt 2.33))
            ) ;_ lambda
            (with-data dat '((calc-bxy 'hrl_ang)))
          ) ;_ apply
        ) ;_ condif
        ( (= con 'X15)
          (list (list (- off) hgt 0.0) (list (- off) hgt 2.33))
        ) ;_ condif
        ( (or (member con '(X16 X17)) (= T))
          (list (list (- off) hgt 2.33))
        ) ;_ condif
      ) ;_ cond
    ) ;_ setq
    (setq off (?n hrr_off) con (read (?s hrr_con)))
    (append! 'lst
      (cond
        ( (member con '(X13 X14))
          (apply
           '(lambda (b x y)
              (setq b (+ b len) x (+ x len))
              (list (list b hgt 2.33) (list x (+ hgt y) 2.33))
            ) ;_ lambda
            (with-data dat '((calc-bxy 'hrr_ang)))
          ) ;_ apply
        ) ;_ condif
        ( (= con 'X15)
          (list (list (+ len off) hgt 2.33) (list (+ len off) hgt 0.0))
        ) ;_ condif
        ( (or (member con '(X16 X17)) (= T))
          (list (list (+ len off) hgt 2.33))
        ) ;_ condif
      ) ;_ cond
    ) ;_ append!
    (setq lst
      (cons
        (append
          (cond
            ( (= 'X15 (read (?s hrl_con)) (read (?s hrr_con)))
             '(("bulge" (1.0 1.0)))
            ) ;_ condif
            ( (= 'X15 (read (?s hrl_con))) '(("bulge" (1.0 0.0))))
            ( (= 'X15 (read (?s hrr_con))) '(("bulge" (0.0 1.0))))
            ( (= T) nil)
          ) ;_ cond
         '(("type" "pipe") ("radius" 0.83) ("handrail" "1"))
          (list (list "layer" lay))
        ) ;_ append
        lst
      ) ;_ cons
    ) ;_ setq
    (if (= guard_hand "1") (add-to-list 'ret lst))
  ) ;_ defun guard-rail-add-hr

  (defun guard-rail-base-pl ( / ret)
    (mapcar
     '(lambda (i / x z d l m r)
        (mapcar
         '(lambda (var val) (set var (car (member val '(41 42 48)))))
         '(l m r)
          (mapcar 'guard-con-read (list lpost_con mpost_con rpost_con))
        ) ;_ mapcar
        (cond
          ( (and l (= i 0))
            (setq x 0.0 z (- lpost_ext) d (if (= l 48) 20.0 6.0))
          ) ;_ condif
          ( (and r (= i (1+ n-mid-posts)))
            (setq x (- guard_lng 3.0) z (- rpost_ext) d (if (= r 48) 20.0 6.0))
          ) ;_ condif
          ( m
            (setq x (- (* i post-cl) 0.75) z (- mpost_ext) d (if (= m 48) 20.0 6.0))
          ) ;_ condif
          ( (= T) (setq x nil))
        ) ;_ cond
        (if x
          (add-to-list 'ret
            (cons
              (#!
               '( ("type" "poly")
                  ("extrude" #(- d))
                  ("ins" (0 #z #(/ (- d 1.5) 2.0)))
                ) ;_ quoted-list
              ) ;_ macro-expand
              (rect-pts 3.0 0.25 (list x 0 0))
            ) ;_ cons
          ) ;_ add-to-list
        ) ;_ if
      ) ;_ lambda
      (cons-ind-lst (+ 2 n-mid-posts) nil)
    ) ;_ mapcar
    ret
  ) ;_ defun guard-rail-base-pl

  (defun cane-rail-base-pl ( / ret bpl)
    (setq
      ret (cons-ind-lst (+ 2 n-mid-posts) nil)
      bpl (list lpost_con mpost_con rpost_con)
      bpl (mapcar (compose '(?s guard-con-read)) bpl)
      bpl (mapcar '(lambda (i) (if (= 30 i) 0.1875 0.0)) bpl)
    ) ;_ setq
    (mapcar!
     '(lambda (i / )
        (cons
         '(("type" "pipe") ("radius" 2.0625))
          (macro-expand
            (cond
              ( (= i 0)
               '((0.0 0.0) (0.0 #(car bpl)))
              ) ;_ condif
              ( (= i (1+ n-mid-posts))
               '((#out-post-cl 0.0) (#out-post-cl #(caddr bpl)))
              ) ;_ condif
              ( (= T)
               '((#(* i post-cl) 0.0) (#(* i post-cl) #(cadr bpl)))
              ) ;_ condif
            ) ;_ cond
          ) ;_ macro-expand
        ) ;_ cons
      ) ;_ lambda
     'ret
    ) ;_ mapcar
    (remove! 'ret '(apply 'equal (cdr x)))
  ) ;_ defun cane-rail-base-pl

  (defun guard-rail-kick-pl ( / con anc em1 ins ret)
    (if (or kick (= guard_kick "1"))
      (progn
        (setq
          con (list lpost_con mpost_con rpost_con)
          con (mapcar '(lambda (s) (guard-con-read (?s s))) con)
          anc (mapcar '(lambda (l) (member l con)) '(42 41))
          anc (apply 'append anc)
          em1 (not (null (member 46 con)))
          ins (if (or anc em1) (if anc 0.25 -1.5))
          ins (if ins (list (list "ins" (list 0 ins 0))))
        ) ;_ setq
        (add-to-list 'ret
          (cons
            (append '(("type" "poly") ("extrude" 0.25)) ins)
            (rect-pts (?? guard_lng len) (if em1 4 2) '(0 0 0))
          ) ;_ cons
        ) ;_ add-to-list
      ) ;_ progn
    ) ;_ if
    ret
  ) ;_ defun guard-rail-kick-pl

  (defun guard-rail-knife-pl ( / lst ret)
    (setq lst '(("type" "poly") ("extrude" #e)))
    (if fab-p (append! 'lst '(("layer" "Hidden"))))
    (mapcar
     '(lambda (i / x y e)
        (setq x (+ 0.625 (* i post-cl)))
        (cond
          ( (= i 0)
            (setq y (?? lpost_ext post-ext) e (?? lpost_tab knife))
          ) ;_ condif
          ( (= i (1+ n-mid-posts))
            (setq y (?? rpost_ext post-ext) e (?? rpost_tab knife))
          ) ;_ condif
          ( (= T)
            (setq y (?? mpost_ext post-ext) e (?? mpost_tab knife))
          ) ;_ condif
        ) ;_ cond
        (if (> e 0)
          (add-to-list 'ret
            (cons (#! lst) (rect-pts 0.25 6 (list x (- y) 0)))
          ) ;_ add-to-list
        ) ;_ if
      ) ;_ lambda
      (cons-ind-lst (+ 2 n-mid-posts) nil)
    ) ;_ mapcar
    ret
  ) ;_ defun guard-rail-knife-pl

  (defun stair-xdata->guard-xdata ( / lst calc-guard-hgt)
    (psc-include '("stair/util.lsp"))
    (expect-with-data "stair-xdata->guard-xdata")
    (foreach loc '("ibot" "obot" "itop" "otop" "btwn")
      (add-to-list 'lst (if (stair-guard--num loc) loc))
    ) ;_ foreach
    (remove! 'lst '(null x))
    (mapcar
     '(lambda (loc / side num t/b ext lng con hgt ext tab asc)
        (setq
          side (substr loc 1 1)
          t/b  (substr loc 2 4)
          num  (strcat "-" (to-string (stair-guard--num loc)))
          ext  (?n (sidekey-rail (strcat t/b "_ext")))
          lng  (cond
                 ( (= loc "btwn")
                   (- dist_btwn (if (= "0" i-rail_topmnt) 3.0 0.0))
                 ) ;_ condif
                 ( (= t/b "bot")
                   (- bot_ext 9.5 3.625 ext)
                 ) ;_ condif
                 ( (= t/b "top")
                   (- top_ext ext 3.625 (- top_cdist) -0.25)
                 ) ;_ condif
                 ( (= T) (error "lng nil"))
               ) ;_ cond
          con  (cond
                 ( (= loc "btwn")
                   (if (<= lng 5.875)
                     "X11 BTWN. NARROW" "X12 BTWN. WIDE"
                   ) ;_ if
                 ) ;_ condif
                 ( (begins-with (sidekey "string_mat") "PL")
                   "06X PL. STR. SIDE"
                 ) ;_ condif
                 ( (not (= (sidekey-rail "topmnt") "1"))
                   "07X CH. STR. SIDE"
                 ) ;_ condif
                 ( (= (sidekey-rail "topmnt") "1")
                   "08X CH. STR. TOP"
                 ) ;_ condif
                 ( (= T) (error "con nil"))
               ) ;_ cond
          hgt  (cond
                 ( (= 'X11 (read con)) 43.0)
                 ( (= 'X12 (read con)) 42.625)
                 ( (= t/b "top") (calc-guard-top))
                 ( (= T) 42.75)
               ) ;_ cond
          ext  (cond
                 ( (= (sidekey-rail "topmnt") "1")
                   (- (sidekey (strcat "nosing_" t/b)))
                 ) ;_ condif
                 ( (member (read con) '(X11 X12)) 0.0)
                 ( (= T) (- 50.0 hgt))
               ) ;_ cond
          tab  (if
                 (and
                   (not (begins-with (?s (sidekey "string_mat")) "PL"))
                   (= (sidekey-rail "topmnt") "0")
                 ) ;_ and
                 (- (sidekey "string_wid") (sidekey "string_web"))
                 0.0
               ) ;_ if
          asc  (if (= "L" (substr ascend 1 1)) 1 -1)
        ) ;_ setq
        (guard-add-derived-keys
          (#!
           '(
              ("app_id" "Guard")
              ("finish" #finish)
              ("guard_hand" "0")
              ("guard_hgt" #hgt)
              ("guard_kick" "0")
              ("guard_lng" #lng)
              ("level" #level)
              ("guard_qty" 1)
              ("sequence" #(strcat #sequence #num))
              ("number" #number)
              ("guard_sty" #rail_style)
              ("hrl_con" "X17 None")
              ("hrl_off" -0.3125)
              ("hrr_con" "X17 None")
              ("hrr_off" -0.3125)
              ("lpost_con" #con)
              ("lpost_ext" #ext)
              ("lpost_tab" #tab)
              ("mpost_con" #con)
              ("mpost_ext" #ext)
              ("mpost_tab" #tab)
              ("rpost_con" #con)
              ("rpost_ext" #ext)
              ("rpost_tab" #tab)
              ("location" #loc)
              ("asc-int" #asc)
            ) ;_ quoted-list
          ) ;_ macro-expand
        ) ;_ guard-add-derived-keys
      ) ;_ lambda
      lst
    ) ;_ mapcar
  ) ;_ defun stair-xdata->guard-xdata

  (defun calc-stair-guard-ins (2d/3d dat / side)
    (apply
     '(lambda (x y z) (if (= 2d/3d '3d) (list x y z) (list x z)))
      (with-data dat
       '( (setq side (substr location 1 1))
          (cond
            ( (member location '("ibot" "obot"))
                (list
                  3.625
                  (- (z-offset-stair-rail side))
                  0
                ) ;_ list
            ) ;_ condif
            ( (member location '("itop" "otop"))
                (list
                  (+ bot_ext bot_cdist run -0.25)
                  (- (z-offset-stair-rail side))
                  flight_hgt
                ) ;_ list
            ) ;_ condif
            ( (= location "btwn")
                (setq side "i")
                (list
                  (- full_run 3.625)
                  (* dir asc-int (+ (* width 0.5) 1.5))
                  (+ flight_hgt (- (calc-guard-top) guard_hgt))
                ) ;_ list
            ) ;_ condif
            ( (= T) '(0 0 0))
          ) ;_cond
        ) ;_ quoted-list
      ) ;_ with-data
    ) ;_ apply
  ) ;_ calc-stair-guard-ins

  (defun calc-guard-top ( / y-tmp y-ins)
    (setq
      y-tmp (calc-stair-rail side "rail frame" "rail handgrab")
      y-ins (?f (cadr (xd-value "ins" (caar y-tmp))))
      y-tmp (mapcar 'cadr (apply 'append (mapcar 'cdr y-tmp)))
      y-tmp (+ y-ins (car (vl-sort y-tmp '>)) (- flight_hgt))
    ) ;_ setq
  ) ;_ defun calc-guard-top
