(set-file-docstring "Common functions for handrail returns")

(psc-include '("shapes/line.lsp"))

;; take two stair enames and draw the return between them
(defun draw-hg-return (s1 s2
                       / tk2 blockobj clayer ins lnosex lnosey data
                       nxoff data s2-ext s2-r-ext s2-line s1-ext
                       s1-r-ext s1-line)

  (defun tk2 (explst) (translate-keys s2 explst))
  (setq ins (cdr (assoc 10 (entget s2)))
        s2 (cons (list "insx" (car ins))
                 (read-xdata s2 "Stair"))
        ins (cdr (assoc 10 (entget s1)))
        s1 (append (list (list "insx" (car ins))
                         (list "insy" (cadr ins)))
                   (read-xdata s1 "Stair"))
        data s1
        blockObj (p-blk "return")
        clayer (vla-getvar 'clayer)
        lnosex (tk '((if (= dir 1) + -) insx (+ bot_ext bot_cdist run)))
        lnosey (tk '(+ insy flight_hgt)))
  (set-active-layer "SOLID")
  (if (setq rtype (rail-return-type s1 s2))
      (progn
        (debug-print '("Drawing return (" #rtype ")"))
        (attempt-apply (read (strcat "rail-return-" rtype))
                       (list s1 s2))))
  (set-active-layer "RAILS")
  (insert-block modelSpace (list lnosex lnosey) blockObj)
  (set-active-layer clayer))

(defun-q rail-return-type (s1 s2 / data tk2)
  "Returns the type of handrail return between stairs S1 and S2, or nil"
  (defun tk2 (explst) (translate-keys s2 explst))
  (with-data s2
   '( (setq nxoff ((if (= dir 1) + -) insx bot_ext bot_cdist))
    ) ;_ quoted-list
  ) ;_ with-data
  (with-data s1
   '( (setq nxoff
        (if (= break "top")
          0.0
          (* dir (- nxoff ((if (= dir 1) + -) insx bot_ext bot_cdist run)))
        ) ;_ if
      ) ;_ setq
    ) ;_ quoted-list
  ) ;_ with-data
  (setq data s1)
  (debug-print-vars '(nxoff))
  (cond ((or (= (xd-value "i-rail_type" s1) "None")
             (= (xd-value "i-rail_type" s2) "None")
             (tk '(= i-rail_top_dloop "1"))
             (tk2 '(= i-rail_bot_dloop "1"))
             (tk '(begins-with i-rail_type "Chop")))
         nil)
        ((< (abs nxoff) 0.01)
         (cond ((tk '(> top_ext tread_depth)) "HH")
               ((or (tk '(= top_ext tread_depth))
                    (tk2 '(= bot_ext tread_depth)))
                "EH")
               (T "AA")))
        ((< nxoff 0) (if (or (wcmatch (xd-value "top_conn" s1) "*exten*")
                             (and (wcmatch (xd-value "bot_conn" s2) "*haunch*")
                                  (<f (- nxoff) (xd-value "bot_ext" s2) 0.01)))
                         "HH"
                         (if (< (abs (- nxoff (tk '(- tread_depth)))) 0.01)
                             "EA"
                             "HA")))
        ((> nxoff 0)
         (setq data s2)
         (cond ((tk '(> bot_ext tread_depth)) "HH")
               ((tk '(= bot_ext tread_depth)) "EH")
               ((and (< (abs (- nxoff (xd-value "top_ext" s1))) 0.01)
                     (< (xd-value "top_ext" s1) (tk '(/ tread_depth 2.0))))
                "AA")
               (T "AH")))))
(defun-r 'rail-return-type)

;; set common variables
(defun rail-return-common()
  (setq data s2
        s2-ext (tk 'bot_ext)
        s2-r-ext (tk 'i-rail_bot_ext)
        s2-line (tk '(line-offset
                      (pt-slope (tan (dtr (- pitch)))
                                nxoff
                                rise_hgt)
                      hg_offset))
        data s1
        s1-ext (tk 'top_ext)
        s1-r-ext (tk 'i-rail_top_ext)
        s1-line (tk '(line-offset (pt-slope (tan (dtr pitch)) 0 0) hg_offset))))

;; common proc to add a return based on centerline
(defun rail-return-add (cl bulge / frad tmp *no-print-errors*)
  ;; attempt to fillet, but reduce radius if too big
  (if (> (length cl) 2)
      (progn
        (setq frad 2.83
              *no-print-errors* T)
        (while (vl-catch-all-error-p (setq tmp (vl-catch-all-apply 'fillet-all-pts (list cl frad))))
          (setq frad (- frad 0.01)))
        (setq cl tmp
              *no-print-errors* nil)))
  (setq cl (add-polyline blockObj '(0 0 0) cl nil))
  (if (tk '(= dir -1)) (vla-mirror-y cl))
  (offset&close cl 0.83 T bulge)
  (vla-Delete cl))

;; calc points for AA
(defun rail-return-AA-pts (s1 s2 / topx botx midx)
  (setq topx (+ -3 (if (< s1-ext (/ (xd-value "tread_depth" s1) 2.0)) s1-ext 0) s2-r-ext)
        botx (+ -3 s1-ext s1-r-ext)
        midx (+ s1-ext (max s1-r-ext s2-r-ext))
        pts (list (list botx (line-val s2-line topx))
                  (list midx (line-val s2-line midx))
                  (list midx (line-val s1-line midx))
                  (list topx (line-val s1-line botx))))
  (debug-print '("Top X: " #topx
                 "\nMid X: " #midx
                 "\nBottom X: " #botx))
  pts)

;; add 2D AA return to blockObj
(defun rail-return-AA (s1 s2 / pts)
  (rail-return-common)
  (setq pts (rail-return-AA-pts s1 s2))
  (rail-return-add pts 0))

(defun rail-return-EH-lines (s1 s2 / botx midx topx brk-hgt lines)
  (setq botx (+ -3 s1-r-ext)
        midx (+ (if (tk2 '(= starter_p "1")) s1-ext s2-ext) nxoff)
        topx (+ -3 nxoff s2-ext s2-r-ext)
        brk-hgt (max (min (line-val s1-line 0)
                          (line-val s2-line midx))
                     33.17))
  (debug-print '("Top X: " #topx
                 "\nMid X: " #midx
                 "\nBottom X: " #botx
                 "\nBreak height: " #brk-hgt))
  (add-to-list 'lines (list (list topx (line-val s2-line topx))
                            (list midx brk-hgt)))
  (add-to-list 'lines (list (list midx brk-hgt)
                            (line-int s1-line (list 0 brk-hgt) nil)
                            (list botx (line-val s1-line botx))))
  lines)

;; add 2D EH return to blockObj
(defun rail-return-EH (s1 s2 / lines)
  (rail-return-common)
  (setq lines (rail-return-EH-lines s1 s2))
  (rail-return-add (car lines) '(0 1))
  (rail-return-add (cadr lines) '(1 0)))

(defun rail-return-HH-lines (s1 s2 / topx midx botx brk-hgt lines)
  (setq botx (+ -3 s1-r-ext)
        midx (max s1-ext (+ s2-ext nxoff))
        topx (+ nxoff s2-r-ext (tk2 '(- (if (= starter_p "1") 0 tread_depth) 3)))
        brk-hgt (max (min (line-val s1-line 0)
                          (line-val s2-line (tk2 '(+ nxoff tread_depth))))
                     33.17))
  (debug-print '("Top X: " #topx
                 "\nMid X: " #midx
                 "\nBottom X: " #botx
                 "\nBreak height: " #brk-hgt))
  (add-to-list 'lines (list (list topx (line-val s2-line topx))
                            (line-int s2-line (list 0 brk-hgt) nil)
                            (list midx brk-hgt)))
  (add-to-list 'lines (list (list midx brk-hgt)
                            (line-int s1-line (list 0 brk-hgt) nil)
                            (list botx (line-val s1-line botx)))))

;; add 2D HH return to blockObj
(defun rail-return-HH (s1 s2 / lines)
  (rail-return-common)
  (setq lines (rail-return-HH-lines s1 s2))
  (rail-return-add (car lines) '(0 1))
  (rail-return-add (cadr lines) '(1 0)))

(defun rail-return-EA-lines (s1 s2 / topx midx botx lines int)
  (setq botx (+ -3 s1-r-ext)
        midx 0
        topx (+ -3 s2-r-ext (tk2 '(- (- tread_depth bot_ext))))
        int (line-int s1-line s2-line nil))
  (debug-print '("Top X: " #topx
                 "\nMid X: " #midx
                 "\nBottom X: " #botx))
  (add-to-list 'lines (list (list topx (line-val s2-line topx))
                          int))
  (add-to-list 'lines (list int
                          (list botx (line-val s1-line botx)))))

;; add 2D EA return to blockObj
(defun rail-return-EA (s1 s2 / lines)
  (rail-return-common)
  (setq lines (rail-return-EA-lines s1 s2))
  (rail-return-add (car lines) '(0 1))
  (rail-return-add (cadr lines) '(1 0)))

(defun rail-return-HA-lines (s1 s2 / botx midx topx brk-hgt lines)
  (setq botx (+ -3 s1-r-ext)
        midx 0
        brk-hgt (line-val s1-line midx)
        topx (+ -3 nxoff s2-r-ext (tk2 '(min tread_depth bot_ext))))
  (add-to-list 'lines (list (list topx (line-val s2-line topx))
                            (line-int s2-line (list 0 brk-hgt) nil)
                            (list midx brk-hgt)))
  (add-to-list 'lines (list (list midx brk-hgt)
                            (list botx (line-val s1-line botx))))
  (debug-print '("Top X: " #topx
                 "\nMid X: " #midx
                 "\nBottom X: " #botx
                 "\nBreak height: " #brk-hgt))
  lines)

;; add 2D HA return to blockObj
(defun rail-return-HA (s1 s2 / lines)
  (rail-return-common)
  (setq lines (rail-return-HA-lines s1 s2))
  (rail-return-add (car lines) '(0 1))
  (rail-return-add (cadr lines) '(1 0)))

(defun rail-return-AH-pts (s1 s2 / topx midx botx brk-hgt pts)
  (setq brk-hgt (line-val s1-line 0)
        botx (+ -3 s1-r-ext)
        midx nxoff
        topx (+ midx -3 s2-r-ext)
        pts (list (list topx (line-val s2-line topx))
                  (list midx (line-val s2-line midx))
                  (list midx brk-hgt)
                  (list 0 brk-hgt)
                  (list botx (line-val s1-line botx))))
  (debug-print '("Top X: " #topx
                 "\nMid X: " #midx
                 "\nBottom X: " #botx
                 "\nBreak height: " #brk-hgt))
  pts)

;; add 2D AA return to blockObj
(defun rail-return-AH (s1 s2 / pts)
  (rail-return-common)
  (setq pts (rail-return-AH-pts s1 s2))
  (rail-return-add pts nil))
