;; functions to draw stair rails

(psc-include '("shapes/line.lsp"
               "rail/util.lsp"
               ))

(defun calc-stair-rail (side rail-layer hg-layer
                        /
                        style
                        x-offset
                        y-offset
                        z-offset
                        z-dir
                        nosing-line
                        nosing-mid
                        top-mount
                        chop-top
                        args
                        rail-len
                        blockObj
                        rail-bot-ext
                        rail-top-ext
                        rail-cutoff
                        top-cap-break
                        slope
                        bot-line
                        top-line
                        mid-posts
                        post-cl
                        corner-offset
                        )
  (expect-with-data "calc-stair-rail")
  (setq rail-bot-ext (sidekey-rail "bot_ext")
        rail-top-ext (sidekey-rail "top_ext")
        rail-cutoff (sidekey-rail "cutoff")
        x-offset (- 3.625 rail-bot-ext)
        x-offset (if (> bot_ext tread_depth)
                          (+ x-offset (- bot_ext tread_depth))
                        x-offset)
        nosing-line (pt-slope (/ rise_hgt tread_depth)
                                   (+ bot_ext bot_cdist)
                                   rise_hgt)
        nosing-mid (sidekey "nosing_mid")
        y-offset (+ (if (= "1" (sidekey-rail "topmnt"))
                        (line-val (line-offset nosing-line nosing-mid)
                                  x-offset)
                      (- (line-val (line-offset nosing-line
                                                38.375)
                                   (+ x-offset 1.5))
                         50))
                    rail_hgt_adj)
        rail-len (calc-stair-rail-opcl side)
        top-mount (if (= "1" (sidekey-rail "topmnt")) nosing-mid)
        chop-top (if rail-cutoff (- flight_hgt rail-cutoff y-offset))
        args (list pitch
                   (?f rail-len)
                   chop-top)
        style (substr rail_style 1 3)
        slope (tan (dtr pitch))
        bot-line (pt-slope slope 0 0)
        top-line (if top-mount
                     (line-offset bot-line (- 38.375 top-mount))
                     (pt-slope slope 1.5 50))
        z-offset (z-offset-stair-rail side)
        z-dir (unit-dir z-offset))

  (append (mapcar '(lambda(x) (add-layer x rail-layer))
                  (append
                   (if chop-top
                       (stair-rail-frame-chopped chop-top top-mount)
                       (stair-rail-frame-std rail-len top-mount))
                   (attempt-apply (read (strcat "stair-rail-" style))
                                  args)))
          (list (add-layer (stair-rail-handgrab side) hg-layer))))

(defun add-layer (def layer)
  (cons (cons (list "layer" layer)
              (car def))
        (cdr def)))

(defun draw-stair-rail (ins side
                        /
                        blockObj
                        )
  (expect-with-data "draw-stair-rail")
  (setq blockObj (p-blk "stair-rail"))
  (add-calcd-objs blockObj nil (calc-stair-rail side "0" "SOLID"))
  (setq blockObj (insert-block modelSpace ins blockObj))
  (vla-put-Layer blockObj "RAILS")
  (move-to-bottom blockObj modelSpace)
  blockObj)

;;;;;;;;;;;;;;;;;;;;;;;;
;; Series calc functions
;;;;;;;;;;;;;;;;;;;;;;;;

;; for all series calc functions,
;; chop-top, if included, is the horizontal post height above rail origin

;; 2-line pipe rail
(defun stair-rail-100 (pitch
                       len
                       chop-top
                       /
                       midline-offset
                       )
  (list
   (if chop-top
       (progn
         (if (>= chop-top (line-val top-line 1.5))
             (setq midline-offset -20.83)
             (setq top-line (pt-slope slope 0 chop-top)
                   midline-offset (/ chop-top -2.0)
                   midline-offset (if top-mount midline-offset (+ midline-offset 3))))
         (stair-rail-pipe-midline midline-offset
                                  (- (car (line-int (line-offset top-line midline-offset)
                                                    (list 0 (- chop-top 0.625))
                                                    nil))
                                     0.625)))
       (stair-rail-pipe-midline -20.83 (+ 0.25 len)))))

;; 6-line pipe rail
(defun stair-rail-200 (pitch len chop-top
                       /
                       x
                       midline-offset
                       chop-limit
                       ret
                       )
  (setq x 1.5
        midline-offset (- 0 31 0.83))
  (if chop-top
      (progn
        (progn
          (setq chop-limit (if corner-offset
                               (min (- corner-offset 2) -14)
                               -14))
          ;; standard int tubes
          (repeat (1- mid-posts)
                  (add-to-list 'ret (stair-rail-int-tube x -4.5 (- post-cl 1.5)))
                  (setq x (+ x post-cl)))
          ;; chopped int tube
          (if (or (not corner-offset)
                  (not (< corner-offset -2)))
              (add-to-list 'ret (stair-rail-int-tube-chopped x -4.5 (- chop-top 1.5))))
          ;; midlines
          (while (< midline-offset chop-limit)
            (add-to-list 'ret
                         (stair-rail-pipe-midline
                          midline-offset
                          (- (car (line-int (line-offset top-line midline-offset)
                                            (list 0 (- chop-top 0.625))
                                            nil))
                             0.625)))
            (setq midline-offset (+ midline-offset 1.66 3.875)))))
      (progn
        (repeat (1+ mid-posts)
                (add-to-list 'ret (stair-rail-int-tube x -4.5 (- post-cl 1.5)))
                (setq x (+ x post-cl)))
        (repeat 4
                (add-to-list 'ret (stair-rail-pipe-midline midline-offset (+ len 0.25)))
                (setq midline-offset (+ midline-offset 1.66 3.875)))))
  ret)

;; 7-line rectangular tube rail
(defun stair-rail-300 (pitch
                       len
                       chop-top
                       /
                       x
                       std-tube
                       std-tubes
                       chop-tube
                       chop-tubes
                       chop-limit
                       i
                       ret
                       )
  ;; return a single calcd interior tube steel in a standard panel
  (defun std-tube (x offset)
    (stair-rail-int-tube-300 nil x offset (- post-cl 1.5)))

  ;; add single interior tube steel at chopped panel
  (defun chop-tube (x offset / height y ret)
    (if (and chop-top
             (setq height (- chop-top 1.5))
             (< (car (line-int
                      ;; bottom line of horizontal post
                      (list 0 height)
                      ;; top line of tube
                      (line-offset top-line offset)
                      nil))
                (if post-cl (+ x post-cl -1.5) 123456789)))
        (setq ret (stair-rail-int-tube-300 T x offset height)
              return (- return 5.25))
        (setq ret (std-tube x offset)))
    (debug-print '("adding tube\noffset: " #offset))
    ret)

  ;; add all tubes at chopped panel
  (defun add-tubes (x limit / offset return)
    (setq offset -31.5
          return limit)
    (debug-print '("adding tubes\nlimit: " #limit))
    (while (<= offset limit)
      (add-to-list 'ret (chop-tube x offset))
      (setq offset (+ offset 5.25)))
    return)

  (setq x 1.5)
  (if chop-top
      (progn
        (setq chop-limit (if corner-offset
                             (min (- corner-offset 2) -5)
                             -5))
        ;; standard panel, if occurs
        (repeat (1+ mid-posts)
                (setq chop-limit (add-tubes x chop-limit))
                (setq x (+ x (if post-cl post-cl 0)))))
    (progn
      (repeat (1+ mid-posts)
              (add-tubes x -5)
              (setq x (+ x post-cl)))))
  ret)

;; vertical picket rail
(defun stair-rail-400 (pitch
                       len
                       chop-top
                       /
                       x
                       n-pick
                       pick-spc
                       add-pickets
                       tmp
                       ret
                       )
  ;; note: x incremented by post-cl here
  (defun add-pickets (post-cl / px tmp)
    (setq pick-spc (rail-calc-picket-spacing (- post-cl 1.5))
          px (+ x pick-spc)
          x (+ x post-cl))
    (while (<f px (- x 1.5) 0.01)
      (add-to-list 'ret (stair-rail-picket px (if (and chop-top
                                                       (> px top-cap-break))
                                                  (- chop-top 1.5))))
      (setq px (+ px pick-spc 0.5))))
  
  (setq x 1.5)
  (if chop-top
      (progn
        (repeat mid-posts
                (add-to-list 'ret (stair-rail-int-tube x -31.125 (- post-cl 1.5)))
                (add-pickets post-cl))
        (add-to-list 'ret (stair-rail-int-tube-chopped x -31.125 (- chop-top 1.5)))
        (add-pickets (- (car (line-int (list 0 (- chop-top 1.5))
                                       (line-offset top-line -31.125)
                                       nil))
                        x)))
    (progn
      (repeat (1+ mid-posts)
              (add-to-list 'ret (stair-rail-int-tube x -31.125 (- post-cl 1.5)))
              (add-pickets post-cl))))
  ret)
  
;;;;;;;;;;;;;;;;;;;
;; Helper functions
;;;;;;;;;;;;;;;;;;;

(defun stair-rail-poly (pts ins / header z)
  (setq ins (if ins
                (displace-pt ins
                             (list x-offset y-offset z-offset))
                (list x-offset y-offset z-offset)))
  (if (= -1 dir)
      (setq ins (cons (- (car ins))
                      (cdr ins))
            pts (mirror-pts-v pts 0)))
  (setq header (macro-expand
                '(("type" "poly")
                  ("extrude" #(* z-dir 1.5))
                  ("ins" #ins))))
  (cons header pts))

;; add a standard 1.5" square tube stair rail frame to blockObj
(defun stair-rail-frame-std (len top-mount /
                             mid-pts
                             x
                             ret
                             )
  (setq mid-posts (rail-calc-n-mid-posts len)
        post-cl (/ len (float (1+ mid-posts))))

  ;; outer posts
  (if (not top-mount)
      (progn
        (add-to-list 'ret (stair-rail-poly (rect-pts 1.5 50 '(0 0 0)) nil))
        (add-to-list 'ret (stair-rail-poly (rect-pts 1.5 -50 (list len (line-val top-line len) 0))
                                           nil)))
    (progn
      (add-to-list 'ret
                   (stair-rail-poly
                    (list '(0 0 0)
                          (list 1.5 (line-val bot-line 1.5) 0)
                          (list 1.5 (line-val top-line 1.5) 0)
                          (list 0 (line-val top-line 1.5) 0))
                    nil))
      (add-to-list 'ret
                   (stair-rail-poly
                    (list '(0 0 0)
                          (list 1.5 (line-val bot-line 1.5) 0)
                          (list 1.5 (line-val top-line 0) 0)
                          (list 0 (line-val top-line 0) 0))
                    (list len (line-val bot-line len) 0)))))

  ;; mid posts
  (if (> mid-posts 0)
      (progn
        (setq mid-pts (stair-rail-mid-post-pts)
              x 0)
        (repeat mid-posts
                (setq x (+ x post-cl))
                (add-to-list 'ret
                             (stair-rail-poly
                              mid-pts
                              (list x (line-val bot-line x) 0))))))

  ;; top cap
  (add-to-list 'ret (stair-rail-int-tube 1.5 0 (- len 1.5)))
  ret)

;; return a calcd chop-top 1.5" square tube stair rail frame
(defun stair-rail-frame-chopped (height top-mount
                                 /
                                 bot-post-height
                                 top-cap-p
                                 top-cap-bot-line
                                 h-post-bot-line
                                 h-post-bot-line
                                 h-post-origin
                                 x
                                 ret
                                 )
  (setq bot-post-height (line-val top-line 1.5)
        top-cap-p (>f height bot-post-height 0.01)
        bot-post-height (if top-cap-p bot-post-height height)
        h-post-top-line (list 0 height)
        h-post-bot-line (list 0 (- height 1.5))
        h-post-origin (line-int h-post-bot-line bot-line T)
        x (- (car h-post-origin) 12)
        mid-posts (rail-calc-n-mid-posts x))
  (if (not top-cap-p)
      (setq corner-offset (- (line-offset-dist top-line
                                               (pt-slope slope 0 chop-top)))))
  
  ;; bottom post
  (add-to-list 'ret
               (stair-rail-poly
                (list '(0 0 0)
                      (list 0 bot-post-height 0)
                      (list 1.5 (if top-cap-p bot-post-height (- height 1.5)) 0)
                      (list 1.5 (if top-mount (line-val bot-line 1.5) 0) 0))
                nil))

  ;; top cap & horizontal post
  (if top-cap-p
      (progn
        (setq top-cap-bot-line (line-offset top-line -1.5))
        ;; top cap
        (add-to-list 'ret
                     (stair-rail-poly
                      (list (list 1.5 (line-val top-line 1.5) 0)
                            (line-int top-line h-post-top-line T)
                            (line-int top-cap-bot-line h-post-bot-line T)
                            (list 1.5 (line-val top-cap-bot-line 1.5) 0))
                      nil))
        ;; horizontal post
        (add-to-list 'ret
                     (stair-rail-poly
                      (list h-post-origin
                            (if top-mount
                                (line-int bot-line h-post-top-line T)
                                (list (car h-post-origin)
                                      (+ (cadr h-post-origin) 1.5)
                                      0))
                            (line-int top-line h-post-top-line T)
                            (line-int top-cap-bot-line h-post-bot-line T))
                      nil)))
      ;; horizontal post only
      (add-to-list 'ret
                   (stair-rail-poly
                    (list h-post-origin
                          (if top-mount
                              (line-int bot-line h-post-top-line T)
                              (list (car h-post-origin) height 0))
                          (list 0 height 0)
                          (list 1.5 (- height 1.5) 0))
                    nil)))
  ;; mid posts
  (if (> mid-posts 0)
      (progn
        (setq top-cap-break (if top-cap-p (car (line-int top-cap-bot-line h-post-bot-line nil)))
              h-post-bot-line (- height 1.5)
              post-cl (/ x (float (1+ mid-posts)))
              x 0)
        (repeat mid-posts
                (setq x (+ x post-cl))
                (add-to-list 'ret
                             (stair-rail-poly
                              (list (list x (line-val bot-line x) 0)
                                    (list (+ 1.5 x)
                                          (if top-mount
                                              (line-val bot-line (+ 1.5 x))
                                              (line-val bot-line x))

                                          0)
                                    (list (+ x 1.5)
                                          (if (> x top-cap-break)
                                              h-post-bot-line
                                              (line-val top-cap-bot-line (+ x 1.5)))
                                          0)
                                    (list x
                                          (if (> x top-cap-break)
                                              h-post-bot-line
                                              (line-val top-cap-bot-line x))
                                          0))
                              nil)))))
  ret)

;; return the points list for a mid post
(defun stair-rail-mid-post-pts (/
                                post-top-line
                                )
  (setq post-top-line (line-offset top-line -1.5))
  (list '(0 0)
        (list 1.5
              (if top-mount (line-val bot-line 1.5) 0))
        (list 1.5 (line-val post-top-line 1.5))
        (list 0 (line-val post-top-line 0))))

;; return calcd interior tube steel
;; offset is from top line
;; len is horizontal distance
(defun stair-rail-int-tube (x
                            offset
                            len
                            /
                            ts-top-line
                            ts-bot-line
                            )
  (setq len (+ len x)
        ts-top-line (line-offset top-line offset)
        ts-bot-line (line-offset ts-top-line -1.5))
  (stair-rail-poly
   (list (list x (line-val ts-top-line x) 0)
         (list len (line-val ts-top-line len) 0)
         (list len (line-val ts-bot-line len) 0)
         (list x (line-val ts-bot-line x) 0))
   nil))

;; return calcd interior tube steel for chop-top rail
;; offset is from top line
;; height is relative to rail origin
(defun stair-rail-int-tube-chopped (x
                                    offset
                                    height
                                    /
                                    height-line
                                    ts-top-line
                                    ts-bot-line
                                    )
  (setq height-line (list 0 height 0)
        ts-top-line (line-offset top-line offset)
        ts-bot-line (line-offset ts-top-line -1.5))
  (stair-rail-poly
   (list (list x (line-val ts-top-line x) 0)
         (line-int ts-top-line height-line T)
         (line-int ts-bot-line height-line T)
         (list x (line-val ts-bot-line x) 0))
   nil))

;; wrapper for stair-rail-int-tube that takes care of Z
(defun stair-rail-int-tube-300 (chop-p x offset len/height / poly data pts)
  (setq poly ((if chop-p stair-rail-int-tube-chopped stair-rail-int-tube)
              x offset len/height)
        data (car poly)
        pts (cdr poly))
  (subst-key! "extrude" (* z-dir 0.75) 'data)
  (subst-key! "ins"
              (displace-pt (xd-value "ins" data)
                           (list 0 0 (* z-dir 0.375)))
              'data)
  (cons data pts))

;; return a calcd 1.66OD pipe midline
;; cl-offset is relative to the top-line
;; len is horizontal distance
(defun stair-rail-pipe-midline (cl-offset
                                len
                                /
                                pts
                                center-line
                                ins
                                )
  (setq center-line (line-offset top-line cl-offset)
        len (+ 0.625 len)
        pts (list (list 0.625 (line-val center-line 0.625) 0)
                  (list len (line-val center-line len) 0))
        ins (list ((if (= dir -1) - +) x-offset)
                  y-offset
                  (- z-offset (* z-dir 0.83))))
  (cons (macro-expand
         '(("type" "pipe")
           ("radius" 0.83)
           ("ins" #ins)))
        (if (= dir -1)
            (mirror-pts-v pts 0)
            pts)))

;; return a calcd single vertical picket at x
;; if chop is nil, use standard height
;; if chop is non-nil, use value as height and picket is flat at the
;; top (chopped condition)
(defun stair-rail-picket (x
                          chop
                          /
                          pick-top-line
                          pick-bot-line
                          poly
                          data
                          )
  (setq pick-top-line (if chop
                          (list 0 chop)
                          (line-offset top-line -1.5))
        pick-bot-line (line-offset top-line -31.125)
        poly (stair-rail-poly
              (list (list x (line-val pick-bot-line x) 0)
                    (list (+ x 0.5) (line-val pick-bot-line (+ x 0.5)) 0)
                    (list (+ x 0.5) (line-val pick-top-line (+ x 0.5)) 0)
                    (list x (line-val pick-top-line x) 0))
              (list 0 0 (* z-dir 0.5)))
        data (car poly)
        poly (cdr poly))
  (cons (subst-key "extrude" (* z-dir 0.5) data)
        poly))

(defun stair-rail-handgrab (side
                            /
                            ab-line
                            fnosex
                            lnosex
                            bpostx
                            tpostx
                            bot-pts
                            top-pts
                            breakx
                            breaky
                            pts
                            z-offset
                            z-dir
                            )
  (expect-with-data "calc-stair-rail-handgrab")
  (setq ab-line (line-offset nosing-line hg_offset)
        fnosex (+ bot_ext bot_cdist)
        lnosex (+ fnosex run)
        bot-dloop-p (if (= (sidekey-rail "bot_dloop") "1") 1)
        bpostx (+ x-offset (if bot-dloop-p 0.75 0.625))
        tpostx (if rail-len (+ bpostx rail-len))
        tpostx (if tpostx (if bot-dloop-p tpostx (+ tpostx 0.125)))
        bot-pts (if bot-dloop-p
                    ;; bottom D-loop
                    (progn
                      (setq breakx (- fnosex tread_depth)
                            breaky (line-val ab-line breakx))
                      (if (= rail_lvl-off "1")
                          ;; level-off
                          (list
                           (list bpostx (- breaky 14))
                           (list (- breakx 15) (- breaky 14))
                           (list (- breakx 15) breaky)
                           (list breakx breaky))
                          ;; no level-off
                          (list
                           (list bpostx (- breaky 14))
                           (list breakx (- breaky 14))
                           (list breakx breaky))))
                    ;; no bottom D-loop
                    (list (list bpostx (line-val ab-line bpostx))))
        z-offset (z-offset-stair-rail-handgrab side)
        z-dir (unit-dir z-offset)
        bot-pts (mapcar '(lambda(x) (append x '(0))) bot-pts)
        bot-pts (if bot-dloop-p
                    (cons (append (first-two (car bot-pts))
                                  (list (* z-dir 2.33)))
                          bot-pts)
                    bot-pts)
        top-dloop-p (if (= (sidekey-rail "top_dloop") "1") 1)
        tpostx (if tpostx (if top-dloop-p tpostx (+ tpostx 0.125)))
        top-pts (if top-dloop-p
                    ;; top D-loop
                    (setq breaky (line-val ab-line lnosex)
                          pts (list (list lnosex breaky)
                                    (list (+ lnosex 15) breaky)
                                    (list (+ lnosex 15) (- breaky 12))
                                    (list (if tpostx tpostx (- lnosex 3)) (- breaky 12))))
                    ;; no D-loop
                    (if tpostx
                        ;; standard placement
                        (list (list tpostx (line-val ab-line tpostx)))
                        ;; chop-top level-off
                        (list (list lnosex (line-val ab-line lnosex))
                              (list (+ lnosex 15) (line-val ab-line lnosex)))))
        top-pts (mapcar '(lambda(x) (append x '(0))) top-pts)
        top-pts (if (or top-dloop-p
                        chop-top)
                    (append top-pts
                            (list (append (first-two (last top-pts))
                                          (list (* z-dir 2.33)))))
                    top-pts)
        pts (append bot-pts top-pts))
  (cons (macro-expand
         '(("type" "pipe")
           ("radius" 0.83)
           ("bulge" #(list (?? bot-dloop-p 0) (if (or top-dloop-p chop-top) 1 0)))
           ("ins" #(list 0 0 z-offset))))
        (if (= dir -1)
            (mirror-pts-v pts 0)
            pts)))
