(psc-include '("stair/util.lsp"))

;; add pipe handgrab polyline to blockObj
(defun add-hg (ins pts / obj)
  (setq obj (add-polyline blockObj ins
                          pts
                          nil))
  (mirror obj)
  (vla-put-Color obj 140)
  (vla-put-Layer (offset&close obj 0.83 T nil) "NO PLOT")
  (vla-put-ConstantWidth obj 1.66)
  obj)

;; add rounded end for D-loops
(defun add-donut (ins / obj)
  (setq obj (vla-AddCircle blockObj (3dpt ins) 0.83))
  (vla-put-Color obj 140)
  (mirror obj)
  (vla-put-Layer obj "NO PLOT")
  (setq obj (add-polyline blockObj
                          ins
                          '((-0.415 0 1)
                            (0.415 0 1))
                          t))
  (mirror obj)
  (vla-put-Color obj 140)
  (vla-put-ConstantWidth obj 0.83)
  obj)

;; add rail tag to blockObj
(defun add-rail-tag (ins alignment str / obj tmp)
  (setq obj (add-mtext blockObj
                       ins
                       (strcat "{\\W0.5;" str "}")
                       1
                       alignment))
  (if mirror (mirror obj))
  (vla-put-Height obj 5.5)
  (vla-put-BackgroundFill obj :vlax-true)
  (vla-put-LineSpacingDistance obj 8.0)
  (vla-put-StyleName obj "Calibri")
  ;; no way to access in vlisp
  (setq tmp (entget (handent (vla-get-Handle obj)))
        ;; scale factor of background mask
        tmp (entmod (subst (cons 45 1.2) (assoc 45 tmp) tmp))
        ;; text frame
        tmp (entmod (subst (cons 90 19) (assoc 90 tmp) tmp))))

;; insert handgrab return block
(defun draw-hg-rtrn (s1
                     s2
                     /
                     assembly
                     blockObj
                     ins
                     startx
                     endx
                     halfwid
                     mirror
                     tk2
                     data
                     s1-ndist
                     s2-ndist
                     flip-tag-p
                     ret-type
                     )
  (defun tk2 (explst) (translate-keys s2 explst))
  (defun mirror (obj / data)
    (setq data s1)
    (if (= (tk 'ascend) "Right")
        (vla-mirror-x obj))
    (if (= (tk 'dir) -1)
        (progn
          (vla-mirror-y obj)
          (vla-mirror-x obj))))
  (if (setq ret-type (rail-return-type s1 s2))
      (progn
        (setq assembly (with-data s1 '((stair-hr-assembly)))
              blockObj (p-blk assembly)
              data s1
              s1-ndist (tk '(+ top_cdist top_ext))
              halfwid (+ (/ (tk 'dist_btwn) 2.0) (if (= (tk 'i-rail_topmnt) "1") 3.83 2.33))
              startx (- (tk '(+ top_ext 3)))
              data s2
              s2-ndist (tk '(+ bot_cdist bot_ext))
              ins (cadr (assoc "ins" s2))
              endx (- 0
                      (tk '(+ bot_ext 3))
                      (if (wcmatch (tk 'bot_conn) "*haunch*") (tk '(- tread_depth)) 0))
              data s1
              midx 0)
        (add-hg (list (if (member ret-type '("EH" "HH"))
                          -1.25
                          (- (min s1-ndist s2-ndist)))
                      0 0)
                (fillet-all-pts
                 (list (list startx (- halfwid) 0)
                       (list midx (- halfwid) 0)
                       (list midx halfwid 0)
                       (list endx halfwid 0))
                 2.83))
        (add-rail-tag (list 3
                            (+ (- halfwid 2.33) 3)
                            0)
                      (if flip-tag-p 'topleft 'bottomleft)
                      assembly)
        (vla-put-layer-r (insert-block modelspace ins blockObj) "RAILS"))))

;; draw all individual top rails for one flight
;; returns happen elsewhere
(defun draw-top-rails (stair
                       /
                       ins
                       mirror
                       start
                       draw-stair-rail
                       draw-guard-rail
                       draw-guard-rail-btwn
                       draw-wall-rail
                       draw-rails
                       obj
                      )
  (defun mirror (obj)
    (progn
      (if (member quadrant '(2 3))
          (vla-mirror-y obj))
      (if (member quadrant '(3 4))
          (vla-mirror-x obj))
      (if (= break "top")
          (vla-mirror-y obj))
      (if (= side "o")
          (vla-mirror-x obj))
      (if *mirror-y* (vla-mirror-y obj))
      (if *mirror-x* (vla-mirror-x obj))))

  ;; return the midpoint between the first nosing and either the last
  ;; nosing or the break line
  (defun pitch-rail-tag-x ()
    (+ (if (= break "no") (/ run 2.0) 15)
       (if (= start "bot")
           (+ bot_cdist bot_ext)
           (+ top_cdist top_ext))))

  ;; insert the stair rail block
  (defun draw-stair-rail (ins side / rext x wid blockObj obj tmp draw-stair-rail-inner name)
    ;; rail outline
    (defun draw-stair-rail-inner ()
      (g-post 'par wid)
      (mirror (add-rectangle blockObj wid -1.5 (list x 0 0)))
      (add-rail-tag (list (pitch-rail-tag-x) 5 0) 'bottomcenter name)
      ;; HG geometry
      (setq x (+ x
                 0.625
                 (if (= start "bot")
                     (if bot-dloop
                         (- (+ tread_depth (if (= rail_lvl-off "1") 18 3)))
                         0)
                     (if top-dloop -18 0)))
            wid (- wid
                   (if (= start "bot")
                       (if bot-dloop
                           (- (+ tread_depth (if (= rail_lvl-off "1") 17.375 2.375)))
                           0.625)
                       0)
                   (if (not (= break "bot"))
                       (if top-dloop
                           -17.375
                           0.625)
                       0)))
      (add-hg (list x 2.33 0)
              (list '(0 0 0)
                    (list wid 0 0)))
      (if (and bot-dloop
               (not (= start "top")))
          (add-donut (list x 2.33 0)))
      (if (and top-dloop
               (not (= break "bot")))
          (add-donut (list (if (= start "top") x (+ x wid)) 2.33 0))))

    (setq name (stair-rail-assembly side)
          blockObj (p-blk name)
          rext (vl-symbol-value (read (strcat side "-rail_" start "_ext")))
          x (if (= start "top")
                (if (<= top_ext (/ tread_depth 2.0))
                    (- 3.625 rext)
                    (+ top_cdist top_ext 2.375 (- rext)))
                (if (wcmatch bot_conn "*haunch*")
                    (- bot_ext rext 7.375)
                    (- 3.625 rext)))
          wid (if (= break "no")
                  (- (+ bot_cdist
                        bot_ext
                        run
                        (if (wcmatch (sidekey-rail "type") "Chop*")
                            (- (/ (sidekey-rail "cutoff")
                                  (tan (dtr pitch))))
                            (+ -2.375
                               (sidekey-rail "top_ext")))
                        (if (and (wcmatch top_conn "*ext*")
                                 (<= top_ext (/ tread_depth 2.0)))
                            top_ext
                            0))
                     x)
                  (- (+ (if (= break "top")
                            (+ top_ext top_cdist)
                            (+ bot_ext bot_cdist))
                        30)
                     x)))
    (if (> wid 0)
        (progn
          (draw-stair-rail-inner)
          (vla-put-Layer (insert-block modelSpace ins blockObj) "RAILS"))))

  ;; insert a guard rail block
  (defun draw-guard-rail (ins side tb / name blockObj wid *mirror-y*)
    (if (and (member break (list "no" tb))
             (stair-guard--num (strcat side tb)))
        (progn
          (setq name (stair-guard-assembly (strcat side tb))
                blockObj (p-blk name)
                ext (vl-symbol-value (read (strcat tb "_ext")))
                wid (- (+ ext 1.5) 3.625 (if (= tb "bot") tread_depth 0))
                *mirror-y* (and (= break "no")
                                (= tb "top")))
          (g-post 'par wid)
          (mirror (add-rectangle blockObj wid -1.5 (list 3.625 0 0)))
          (add-rail-tag (list (+ 3.625 (/ wid 2.0)) 5 0) 'bottomcenter name)
          (vla-put-Layer (insert-block modelSpace ins blockObj) "RAILS"))))

  ;; insert a guard rail block between stairs
  (defun draw-guard-rail-btwn (ins / name blockObj *mirror-y* wid obj)
    (if (and (= side "i")
             (/= break "bot")
             (stair-guard--num "btwn"))
        (progn
          (setq name (stair-guard-assembly "btwn")
                blockObj (p-blk name)
                *mirror-y* (= break "top")
                wid (- dist_btwn (if (= i-rail_topmnt "1") 0.125 3.125)))
          (if (> wid 7.125)
              ;; normal
              (progn
                (g-post 'prp wid)
                (mirror (add-rectangle blockObj -1.5 (- wid) '(-3.625 -1.5625 0)))
                ) ;_ progn
              ;; post w/ tabs
              ( (lambda ( / lng lst)
                  (setq lng (/ (- wid 1.5) 2.0))
                  (setq lst
                        (list
                         (list -0.25 (- lng) '(-4.25 -1.5625 0))
                         (list -1.5 -1.5 (list -3.625 (- 0 1.5625 lng) 0))
                         (list -0.25 (- lng) (list -4.25 (- 0 3.0625 lng) 0))
                         ) ;_ list
                        ) ;_setq
                  (map-apply
                   '(lambda (l w p) (mirror (add-rectangle blockobj l w p)))
                   lst
                   ) ;_ map-apply
                  ) ) ;_ lambda
              ) ;_ if
          (add-rail-tag '(3 3 0) 'bottomleft name)

          (setq blockObj (insert-block modelspace ins blockObj))
          (vla-put-Layer blockObj "RAILS"))))

  ;; insert a wall rail block
  (defun draw-wall-rail (ins
                         side
                         /
                         blockObj
                         startx
                         endx
                         ndist
                         pts
                         )
    (setq blockObj (p-blk (stair-rail-assembly side))
          ndist (+ (symcat '(#start "_cdist"))
                   (symcat '(#start "_ext")))
          startx (- ndist
                    (if (= start "bot")
                        (+ tread_depth (if (= rail_lvl-off "1") 15 0))
                        15))
          endx (if (= break "no")
                   (+ ndist run 15)
                   (+ ndist 30))
          ;; A-B
          pts (list (list startx 2.33 0)
                    (list endx 2.33 0)))
    ;; bends and D-loops
    (if bot-dloop
        (if (= start "bot")
            (add-donut (list startx 1.33 0)))
        (if (= start "bot")
            (setq pts (cons (list startx 0 0) pts))))
    (if top-dloop
        (cond ((= start "top")
               (add-donut (list startx 1.33 0)))
              ((= break "no")
               (add-donut (list endx 1.33 0))))
        (cond ((= start "top")
               (setq pts (cons (list startx 0 0) pts)))
              ((= break "no")
               (setq pts (append pts (list (list endx 0 0)))))))
    ;; fillet
    (if (> (length pts) 2)
        (setq pts (fillet-all-pts pts 2.3)))
    (add-hg '(0 -1 0) pts)
    (add-rail-tag (list (pitch-rail-tag-x) 5 0)
                  'bottomcenter
                  (strcat "WR"
                          number
                          "-"
                          level
                          (?s sequence)
                          (if (= (symcat '(#(if (= side "i") "o" "i") "-rail_type"))
                                       "Wall Rail")
                              (if (or (and (= ascend "Left")
                                           (= side "i"))
                                      (and (= ascend "Right")
                                           (= side "o")))
                                  "-L"
                                  "-R")
                              "")))
    (vla-put-Layer (insert-block modelSpace ins blockObj) "RAILS"))

  ;; draw all rails for one side of a stair
  (defun draw-rails (side ins / rtype top-dloop bot-dloop)
    (setq rtype (sidekey-rail "type"))
    (if (/= rtype "None")
        (progn
          (if (= side "o")
              (setq offset (+ offset width)))
          (setq ins (list (car ins)
                          ((if (= z_pos "Far") + -) (cadr ins) offset)
                          0)
                top-dloop (= (sidekey-rail "top_dloop") "1")
                bot-dloop (= (sidekey-rail "bot_dloop") "1"))
          (set-active-layer "RAILS")
          (if (wcmatch rtype "*Stair*")
              (progn
                (if (= (sidekey-rail "topmnt") "1")
                    (setq ins (list (car ins)
                                    ((if (or (and (= z_pos "Far")
                                                  (= side "i"))
                                             (and (= z_pos "Near")
                                                  (= side "o")))
                                         + -)
                                     (cadr ins) 1.5)
                                    (last ins))))
                ;; stair rail
                (draw-stair-rail ins side)
                ;; bottom guard
                (draw-guard-rail ins side "bot")
                ;; reset ins to top
                (setq ins (if (= break "no")
                              (cons (+ (* dir (+ bot_cdist bot_ext run top_ext top_cdist))
                                       (car ins))
                                    (cdr ins))
                              ins))
                ;; top guard
                (draw-guard-rail ins side "top")
                ;; guard between
                (draw-guard-rail-btwn ins))
              ;; wall rail
              (draw-wall-rail ins side)))))
  (with-data stair
    '((setq start (if (= break "top") "top" "bot"))
      (draw-rails "i" ins)
      (draw-rails "o" ins))))
