;; functions to draw wall rails

(psc-include '("shapes/line.lsp"
               ))

;; return a wall rail data structure
(defun calc-wall-rail (side layer
                       /
                       ab-line
                       fnosex
                       lnosex
                       bot-dloop-p
                       bot-pts
                       top-dloop-p
                       top-pts
                       breakx
                       breaky
                       z-offset
                       z-dir
                       )
  (expect-with-data "calc-wall-rail")
  (if (= (sidekey-rail "type") "Wall Rail")
      (progn
        (setq ab-line (pt-slope (/ rise_hgt tread_depth)
                                (+ bot_ext bot_cdist)
                                rise_hgt)
              ab-line (list (car ab-line)
                            (+ (cadr ab-line)
                               (* hg_offset
                                  (/ 1 (cos (dtr pitch))))))
              fnosex (+ bot_ext bot_cdist)
              lnosex (+ fnosex run)
              bot-dloop-p (= (sidekey-rail "bot_dloop") "1")
              breakx (- fnosex tread_depth)
              breaky (line-val ab-line breakx)
              bot-pts (list (list breakx breaky))
              bot-pts (if (= rail_lvl-off "1")
                          ;; level-off
                          (append (list (list (- breakx 15) breaky))
                                  bot-pts)
                          ;; no level-off
                          bot-pts)
              bot-pts (if bot-dloop-p
                          ;; bottom D-loop
                          (append
                           (list (list (+ breakx tread_depth) (- breaky 14))
                                 (list (if (= rail_lvl-off "1")
                                           ;; level-off
                                           (- breakx 15)
                                           ;; no level-off
                                           breakx)
                                       (- breaky 14)))
                           bot-pts)
                          ;; no bottom D-loop
                          bot-pts)
              ;; z offset of main A-B centerline
              z-offset (z-offset-wall-rail side)
              z-dir (unit-dir z-offset)
              bot-pts (mapcar '(lambda(x) (append x '(0))) bot-pts)
              ;; add tail
              bot-pts (cons (append (first-two (car bot-pts))
                                    (list (* z-dir 2.33)))
                            bot-pts)
              breaky (line-val ab-line lnosex)
              top-dloop-p (= (sidekey-rail "top_dloop") "1")
              top-pts (list (list lnosex breaky)
                            (list (+ lnosex 15) breaky))
              top-pts (if top-dloop-p
                          ;; top D-loop
                          (append top-pts
                                  (list (list (+ lnosex 15) (- breaky 12))
                                        (list lnosex (- breaky 12))))
                          ;; no top D-loop
                          top-pts)
              top-pts (mapcar '(lambda(x) (append x '(0))) top-pts)
              ;; add tail
              top-pts (append top-pts
                              (list (reverse (cons (* z-dir 2.33)
                                                   (cdr (reverse (last top-pts))))))))
        (cons (macro-expand
               '(("type" "pipe")
                 ("radius" 0.83)
                 ("bulge" 1)
                 ("layer" #layer)
                 ("ins" #(list 0 0 z-offset))))
              (mapcar '(lambda(x) (cons (* dir (car x)) (cdr x)))
                      (append bot-pts top-pts))))))

;; draw a wall rail block at a stair block
(defun draw-wall-rail-2d (ins
                          side
                          /
                          pts
                          blockObj
                          obj
                          )
  (expect-with-data "draw-wall-rail-2d")
  (setq blockObj (p-blk "wall-rail"))
  (add-calcd-pipe blockObj nil (calc-wall-rail side "SOLID"))
  (setq blockObj (insert-block modelSpace ins blockObj))
  (vla-put-Layer blockObj "RAILS")
  (move-to-bottom blockObj modelSpace))
