  (set-file-docstring "Functions for calculating transformation matrices")

  (defun-q calc-tmatrix
    (x-rot y-rot z-rot x-off y-off z-off x-inv y-inv z-inv / x-rot y-rot z-rot t-mat)
    "Calculates a transformtion matrix from passed values

x-rot y-rot z-rot numeric values representing X Y or Z rotations
x-off y-off z-off numeric values representing X Y or Z offsets
x-inv y-inv z-inv non-nil values will mirror accross X Y or Z axis"
    (setq t-mat (imat 4))
    (mapcar
     '(lambda (v) (set v (dtr (eval v))))
     '(x-rot y-rot z-rot)
    ) ;_ mapcar
    (if (not (zerop x-rot))
      (setq t-mat
        (mxm
          (list
            (list 1.0 0.0 0.0 0.0)
            (list 0.0 (+ (cos x-rot)) (- (sin x-rot)) 0.0)
            (list 0.0 (+ (sin x-rot)) (+ (cos x-rot)) 0.0)
            (list 0.0 0.0 0.0 1.0)
          ) ;_ list
          t-mat
        ) ;_ mxm
      ) ;_ setq
    ) ;_ if
    (if (not (zerop y-rot))
      (setq t-mat
        (mxm
          (list
            (list (+ (cos y-rot)) 0.0 (+ (sin y-rot)) 0.0)
            (list 0.0 1.0 0.0 0.0)
            (list (- (sin y-rot)) 0.0 (+ (cos y-rot)) 0.0)
            (list 0.0 0.0 0.0 1.0)
          ) ;_ list
          t-mat
        ) ;_ mxm
      ) ;_ setq
    ) ;_ if
    (if (not (zerop z-rot))
      (setq t-mat
        (mxm
          (list
            (list (+ (cos z-rot)) (- (sin z-rot)) 0.0 0.0)
            (list (+ (sin z-rot)) (+ (cos z-rot)) 0.0 0.0)
            (list 0.0 0.0 1.0 0.0)
            (list 0.0 0.0 0.0 1.0)
          ) ;_ list
          t-mat
        ) ;_ mxm
      ) ;_ setq
    ) ;_ if
    (if x-inv
      (setq t-mat
        (mxm
          (list
            (list -1.0 0.0 0.0 0.0)
            (list 0.0 1.0 0.0 0.0)
            (list 0.0 0.0 1.0 0.0)
            (list 0.0 0.0 0.0 1.0)
          ) ;_ list
          t-mat
        ) ;_ mxm
      ) ;_ setq
    ) ;_ if
    (if y-inv
      (setq t-mat
        (mxm
          (list
            (list 1.0 0.0 0.0 0.0)
            (list 0.0 -1.0 0.0 0.0)
            (list 0.0 0.0 1.0 0.0)
            (list 0.0 0.0 0.0 1.0)
          ) ;_ list
          t-mat
        ) ;_ mxm
      ) ;_ setq
    ) ;_ if
    (if z-inv
      (setq t-mat
        (mxm
          (list
            (list 1.0 0.0 0.0 0.0)
            (list 0.0 1.0 0.0 0.0)
            (list 0.0 0.0 -1.0 0.0)
            (list 0.0 0.0 0.0 1.0)
          ) ;_ list
          t-mat
        ) ;_ mxm
      ) ;_ setq
    ) ;_ if
    (if (not (apply '= (list 0 x-off y-off z-off)))
      (setq t-mat       
        (mxm
          (list
            (list 1.0 0.0 0.0 x-off)
            (list 0.0 1.0 0.0 y-off)
            (list 0.0 0.0 1.0 z-off)
            (list 0.0 0.0 0.0 1.0)
          ) ;_ list
          t-mat
        ) ;_ mxm
      ) ;_ setq
    ) ;_ if
    (vlax-tmatrix t-mat)
  ) ;_ defun-q calc-tmatrix
  (defun-r 'calc-tmatrix)

  ;;; Identity Matrix  -  Lee Mac
  ;;; Args: n - matrix dimension
  (defun-q imat (n / i j l m)
    "Creates an identity matrix of size n"
    (repeat (setq i n)
      (repeat (setq j n)
        (setq l (cons (if (= i j) 1.0 0.0) l) j (1- j))
      ) ;_ repeat
      (setq m (cons l m) l nil i (1- i))
    ) ;_ repeat
    m
  ) ;_ defun-q imat
  (defun-r 'imat)

  (defun-q mxm (m n)
    "Multiplies two matrices"
    (declare (vars (m list (tmatrix-p m)) (n list (tmatrix-p n))))
    ((lambda (a) (mapcar '(lambda (r) (mxv a r)) m)) (trp n))
  ) ;_ defun-q mxm
  (defun-r 'mxm)

  ;;; Matrix x Vector  -  Vladimir Nesterovsky
  ;;; Args: m - nxn matrix, v - vector in R^n
  (defun-q mxv (m v)
    "Multiplies a matrix by a vector"
    (declare
      (vars
        (m list (tmatrix-p m))
        (v list (vl-every 'numberp v) (= (length v) (length (car m))))
      ) ;_ vars
    ) ;_ declare
    (mapcar '(lambda (r) (apply '+ (mapcar '* r v))) m)
  ) ;_ defun-q mxv
  (defun-r 'mxv)

  (defun-q trp (m)
    "Transposes a matrix"
    (declare (vars (m list (tmatrix-p m))))
    (apply 'mapcar (cons 'list m))
  ) ;_ defun-q trp
  (defun-r 'trp)

  (defun-q tmatrix-p (m)
    "Returns T if m is a valid tmatrix"
    (declare (vars (m list (vl-every 'listp m))))
    (vl-every
     '(lambda (l) (and (= (length l) (length m)) (vl-every 'numberp l)))
      m
    ) ;_ vl-every
  ) ;_ defun-q tmatrix-p
  (defun-r 'tmatrix-p)
