  (set-file-docstring "Line utility functions")

  (defun-q pt-slope (m x y)
    "returns a line in slope-intercept format given the slope (m) and a point (x,y)

CAR of returned list is slope CADR is y-intercept at X=0"
    (declare (vars (m nil (numberp m)) (x nil (numberp x)) (y nil (numberp y))))
    (list m (- y (* m x)))
  ) ;_ defun-q pt-slope
  (defun-r 'pt-slope)

  (defun-q line-val (line x)
    "return y-value of line at point x

line-yval-@x is an alias for this function"
    (+ (* (car line) x) (cadr line))
  ) ;_ defun-q line-val
  (defun-r 'line-val)

  (defun-q line-yval-@x (line x)
    "return y-value of line at point x

alias for line-val"
    (line-val line x)
  ) ;_ defun-q line-val
  (defun-r 'line-yval-@x)

  (defun-q line-xval-@y (line y)
    "return x-value of line at point y"
    (car (line-int line (list 0 y) nil))
  ) ;_ defun-q line-xval-@y
  (defun-r 'line-xval-@y)

  (defun-q line-point-@x (line x)
    "return point on line at dist x"
    (list x (line-yval-@x line x))
  ) ;_ defun-q line-point-@x
  (defun-r 'line-point-@x)

  (defun-q line-point-@y (line y)
    "return point on line at dist y"
    (list (line-xval-@y line y) y)
  ) ;_ defun-q line-point-@y
  (defun-r 'line-point-@y)

  (defun-q line-off-pt (line pt)
    "returns a line in s-i form from offseting an s-i form line through a point"
    (apply 'pt-slope (list (car line) (car pt) (cadr pt)))
  ) ;_ defun line-off-pt
  (defun-r 'line-off-pt)

  (defun-q points->si-line (pt1 pt2)
    "returns a line in s-i form from input of a pair of 2D points"
    (if (zerop (car (mapcar '- pt2 pt1)))
      (error "X-Val of pt1 and pt2 cannot be equal")
    ) ;_ if
    (apply
     '(lambda (x y)
        (apply
         'pt-slope
          (append (list (/ y x)) (mapcar '+ '(0 0) pt1))
        ) ;_ apply
      ) ;_ lambda
      (mapcar '- pt2 pt1 '(0 0))
    ) ;_ apply
  ) ;_ defun-q points->si-line
  (defun-r 'points->si-line)

  (defun-q line-offset (line offset)
    "returns a line in s-i form from offseting an s-i form line by some amount"
    (list
      (car line)
      (+ (cadr line) (/ offset (sin (- (dtr 90) (atan (car line))))))
    ) ;_ list
  ) ;_ defun-q line-offset
  (defun-r 'line-offset)

  (defun-q line-int (line1 line2 3d-p / pts)
    "return intersect of two lines, non-nil 3d-p includes Z value of 0"
    (setq pts
      (inters
        (list 0 (line-val line1 0))
        (list 1 (line-val line1 1))
        (list 0 (line-val line2 0))
        (list 1 (line-val line2 1))
        nil
      ) ;_ inters
    ) ;_ setq
    (if 3d-p (append pts '(0)) pts)
  ) ;_ defun-q line-int
  (defun-r 'line-int)

  (defun-q line-offset-dist (line1 line2)
    "return offset distance of two like-sloped s-i form lines"
    (if (/= (car line1) (car line2))
      (error "line-offset-dist: slopes must be equal")
    ) ;_ if
    (* (cos (atan (car line1))) (- (cadr line1) (cadr line2)))
  ) ;_ defun-q line-offset-dist
  (defun-r 'line-offset-dist)
