(add-to-list 'sub-dialogs '("conn_opts" "top_conn_opts" "bot_conn_opts"))

;; enable or disable the extension tiles based on connection detail
(defun conn-extension (key value / tb detail ext extkey initial-conn current-conn)
  (setq tb (substr key 1 3)
        detail (get-conn-def value)
        ext (cadr detail)
        extkey (strcat tb "_ext")
        initial-conn (symcat '("initial-" #tb "-conn"))
        current-conn (dialog-tile-val (strcat tb "_conn")))
  (if ext
      (progn
        (mode_tile extkey 0)
        (if (cond (multi T)
                  ((apply '/= (mapcar '(lambda(conn) (substr conn 2 1))
                                      (list initial-conn current-conn)))
                   T))
            (set_tile extkey (to-string ext))))
    (progn
      (set_tile extkey "0.0")
      (mode_tile extkey 1))))

;; default action for top_conn and bot_conn popup lists
(defun conn-action (key value / tb detail)
  (setq tb (substr key 1 3)
        detail (get-conn-def value))
  (conn-extension key value)
  (set (read (strcat "initial-" tb "-conn")) value)
  (dialog-sub-update-from-def (strcat key "_opts") (cddr detail))
  ;; handle bolt depth difference for 200 series
  (if (member (substr value 1 1) '("2" "5"))
      (subst-key! (strcat tb "_vbolt")
                  (if (begins-with (dialog-tile-val "tread_style") "2")
                      5.125
                    5.0)
                  (read (strcat tb "_conn_opts-val")))))

;; return the connection definition for detail
;; which is the full value of top/bot_conn including detail series
;; e.g. "21 Landing, standard"
(defun get-conn-def (detail / tb)
  (setq tb (if (member detail bot-details-strings) "bot" "top"))
  (nth (vl-position detail (symcat '(#tb "-details-strings")))
       (symcat '(#tb "-details"))))

;; starter connection details
;; see bot-details data structure below for usage
(defun starter-detail (name / ret)
  (setq ret
        (list name
              nil))
  ;; even numbers get anchors
  (if (= (rem (atoi (substr name 2 1))
              2)
         0)
      (setq ret (append ret
                        '(("bot_hdw" . exp-anchors)))))
  ret)

;; connection details with extensions (i.e. non-starter details)
;; see *-details data structures below for usage
;; GLOBAL: tb
(defun ext-detail (elt / name ext bw)
  (defun bw (str)
    (begins-with name str))
  (setq name (car elt)
        ext (cadr elt))
  (append
   (list name ext)
   (conn-opts-exp-data
    tb
    (cond ((bw "2")
           '(("~_hdw" . tc-bolts)
             ("~_vbolt" 5.0)
             ("~_hbolt" 4.0)))
          ((bw "3")
           '(("~_blkout" 1.5)))
          ((bw "4")
           '(("~_member" . 4x-angles)
             ("~_hdw" . tc-bolts)
             ("~_vbolt" 4.0)
             ("~_hbolt" 4.0)
             ("~_slab" 5.5)))
          ((bw "5")
           '(("~_hdw" . tc-bolts)
             ("~_vbolt" 5.0)
             ("~_hbolt" 2.0)
             ("~_hbolt2" 4.0)))
          ((bw "6")
           '(("~_member" . 4x-angles)
             ("~_hdw" . tc-bolts)
             ("~_vbolt" 4.0)
             ("~_slab" 5.5)
             ("~_hbolt" 2.0)
             ("~_hbolt2" 4.0))))
    nil)))

;; Key expansion:
;; This dialog is used for top and bottom connections. To combine
;; their use, the dialog specifies keys with "top" or "bot" replaced
;; with "~". These functions expand or reduce (when reduce-p is T)
;; the keys accordingly.

;; expand or reduce a list of key name strings
;; e.g. '("~_slab" "~_blkout")
(defun conn-opts-exp-keys (tb keylst reduce-p / exp-key)
  (defun exp-key (tb key)
    (apply 'vl-string-subst
           (append
            (if reduce-p (list "~" tb) (list tb "~"))
            (list key))))
  (mapcar '(lambda (x) (exp-key tb x))
          keylst))

;; utility function to call conn-opts-exp-keys for both top and
;; bottom, and combine the result
(defun conn-opts-exp-keys-tb (keylst)
  (append (conn-opts-exp-keys "top" keylst nil)
          (conn-opts-exp-keys "bot" keylst nil)))

;; expand or reduce all keys in a dialog data structure
;; e.g. '(("bot_slab" 5.5) ("bot_blkout" 1.5))
(defun conn-opts-exp-data (tb data reduce-p / exp-elt)
  (defun exp-elt (elt)
    (cons
     (apply 'vl-string-subst
            (append
             (if reduce-p (list "~" tb) (list tb "~"))
             (list (car elt))))
     (cdr elt)))
  (mapcar 'exp-elt data))

(setq
 ;; tell dialog-init which keys are relevant to conn_opts
 conn_opts-dist-keys (conn-opts-exp-keys-tb
                      '("~_vbolt"
                        "~_hbolt"
                        "~_hbolt2"
                        "~_slab"
                        "~_blkout"))
 conn_opts-popup-keys (conn-opts-exp-keys-tb
                       '("~_hdw"
                         "~_member"))
 conn_opts-keys (append conn_opts-popup-keys
                        conn_opts-dist-keys)
 tb "bot"
 ;; data structure describing allowable bottom details
 ;; car is a string with the code (omitting the series, which
 ;; is added on later) and description of the detail.
 ;; cadr is the default extension length
 ;; cddr is a list defining the defaults of the options dialog
 bot-details (append
              (mapcar 'starter-detail
                      '("10 Starter, seismic anchor"
                        "11 Starter, seismic embed"
                        "12 Starter, anchor"
                        "13 Starter, embed"
                        "14 Starter, seismic slot anchor"
                        "15 Starter, seismic slot embed"))
              '(("16 Starter, separate angles" nil
                 ("bot_hdw" . exp-anchors)
                 ("bot_hbolt" 2.0)
                 ("bot_hbolt2" 4.0)))
              (mapcar 'ext-detail
                      '(("21 Landing, standard" 0.0)
                        ("23 Landing, extension" 11.0)
                        ("25 Landing, haunch" 22.0)))
              '(("27 Landing, seismic" nil
                 ("bot_hdw" . tc-bolts)
                 ("bot_vbolt" 5.0)
                 ("bot_hbolt" 4.0)))
              (mapcar 'ext-detail
                      '(("31 EM-1, standard" 0.0)
                        ("33 EM-1, extension" 11.0)
                        ("35 EM-1, haunch" 22.0)))
              '(("37 EM-2, seismic" nil))
              (mapcar 'ext-detail
                      '(("41 Steel, standard" 0.0)
                        ("43 Steel, extension" 11.0)
                        ("45 Steel, haunch" 22.0)
                        ("51 Landing, standard (2H)" 0.0)
                        ("53 Landing, extension (2H)" 11.0)
                        ("55 Landing, haunch (2H)" 22.0)
                        ("61 Steel, standard (2H)" 0.0)
                        ("63 Steel, extension (2H)" 11.0)
                        ("65 Steel, haunch (2H)" 22.0)))
              '(("X1 Custom, standard bottom" 0.0)
                ("X5 Custom, haunch bottom" 22.0)
                ("X7 Custom, starter" nil)))
 bot-details-strings (mapcar 'car bot-details)

 tb "top"
 ;; a data structure like bot-details
 top-details (mapcar 'ext-detail
                     '(("20 Landing, standard" 0.0)
                       ("22 Landing, extension" 11.0)
                       ("24 Landing, haunch" 22.0)
                       ("30 EM-1, standard" 0.0)
                       ("32 EM-1, extension" 11.0)
                       ("34 EM-1, haunch" 22.0)
                       ("40 Steel, standard" 0.0)
                       ("42 Steel, extension" 11.0)
                       ("44 Steel, haunch" 22.0)
                       ("50 Landing, standard (2H)" 0.0)
                       ("52 Landing, extension (2H)" 11.0)
                       ("54 Landing, haunch (2H)" 22.0)
                       ("60 Steel, standard (2H)" 0.0)
                       ("62 Steel, extension (2H)" 11.0)
                       ("64 Steel, haunch (2H)" 22.0)
                       ("X0 Custom, standard top" 0.0)
                       ("X4 Custom, haunch top" 22.0)))
 top-details-strings (mapcar 'car top-details)
 tb nil

 4x-angles '("L3x2x1/4_LLV"
             "L3x3x1/4"
             "L4x4x1/4"
             "L4x4x3/8"
             "L5x3x1/4_LLH"
             "L5x3x1/4_LLV"
             "L7x4x3/8_LLH"
             "L7x4x3/8_LLV"))

;; Init function for connection options sub-dialog.
;; Assigned automatically as the action for both top and bottom connection
;; options buttons. Various utility functions are used to handle the split.
(defun conn_opts-sub-init (tb / tb1 detail valsym local-scope)
  (setq valsym (read (strcat tb "-val"))
        tb (substr tb 1 3)
        tb1 (substr tb 1 1)
        detail (get-conn-def (dialog-popup-val (strcat tb "_conn"))))

  (defun local-scope ( / conn-def dkey-p popup-keys dist-keys int-keys plain-string-keys
                         toggle-keys default-actions dialog-ties sub-dialogs tmp
                         dialog-after-load dialog-before-done)

    (defun dialog-after-load ()
      (foreach key all-keys
               (mode_tile key 0)))

    (setq conn-def (cddr detail)
          dkey-p (lambda (x) (member (car x) conn_opts-dist-keys))
          popup-keys (conn-opts-exp-data
                      tb
                      (vl-remove-if 'dkey-p conn-def)
                      t)
          dist-keys (conn-opts-exp-keys
                     tb
                     (mapcar 'car (vl-remove-if-not 'dkey-p conn-def))
                     t))
    (if (setq tmp (dialog-init
                   "stair/conn_opts.dcl"
                   "conn_opts"
                   (conn-opts-exp-data tb (vl-symbol-value valsym) T)))
        (set valsym (conn-opts-exp-data tb tmp nil))))
  (local-scope))
