;; keys not present in the dialog, but calcuated and added to the block
(setq stair-derived-keys '("version"
                           "assembly"
                           "flight_hgt"
                           "tread_qty" 
                           "run"
                           "full_run"
                           "pitch"
                           "l-string_tex"
                           "l-string_bex"
                           "l-string_wid"
                           "l-string_dpt"
                           "l-string_flg"
                           "l-string_web"
                           "r-string_tex"
                           "r-string_bex"
                           "r-string_wid"
                           "r-string_dpt"
                           "r-string_flg"
                           "r-string_web"
                           "btwn_stringers"
                           "stringers_eq"
                           "pan_thick"
                           "top_conn_ser"
                           "top_conn_det"
                           "top_conn_num"
                           "top_conn_full"
                           "bot_conn_ser"
                           "bot_conn_det"
                           "bot_conn_num"
                           "bot_conn_full"
                           "starter_p"
                           "dir"
                           "hg_offset"
                           "rail_hgt_adj"
                           "i-string_tex"
                           "i-string_bex"
                           "i-string_wid"
                           "i-string_dpt"
                           "i-string_flg"
                           "i-string_web"
                           "i-string_mat"
                           "i-string_xtra_tex"
                           "i-string_xtra_bex"
                           "i-nosing_top"
                           "i-nosing_mid"
                           "i-nosing_bot"
                           "o-string_tex"
                           "o-string_bex"
                           "o-string_wid"
                           "o-string_dpt"
                           "o-string_flg"
                           "o-string_web"
                           "o-string_mat"
                           "o-string_xtra_tex"
                           "o-string_xtra_bex"
                           "o-nosing_top"
                           "o-nosing_mid"
                           "o-nosing_bot"
                           "guard_rails"
                           "stn_thick"
                           "stn_len"
                           "top_pan_ext"
                           "bot_pan_ext"
                           "i-rail_top_lvl"
                           "i-rail_bot_lvl"
                           "o-rail_top_lvl"
                           "o-rail_bot_lvl"
                           ))

;; add keys calculated from dialog keys
(defun stair-add-derived-keys (data / inner this-func)
  (defun-q inner ( / add stringers-equal io guard-rails this-func ret)
    ;; make all derived keys local to this function
    (defun add (key val)
      (set (read key) val)
      (add-to-list 'ret (list key val)))

    (add "version" stair-version)
    (add "assembly" (strcat "S" number "-" level (to-string sequence)))
    (add "flight_hgt" (* rise_qty rise_hgt))
    (add "tread_qty" (- rise_qty 1))
    (add "run" (* tread_qty tread_depth))
    (add "full_run" (+ bot_ext bot_cdist run top_cdist top_ext))
    (add "top_conn_ser" (substr top_conn 1 1))
    (add "top_conn_det" (substr top_conn 2 1))
    (add "top_conn_num" (substr top_conn 1 2))
    (add "top_conn_full" (strcat (substr tread_style 1 1)
                                 top_conn))
    (add "bot_conn_ser" (substr bot_conn 1 1))
    (add "bot_conn_det" (substr bot_conn 2 1))
    (add "bot_conn_num" (substr bot_conn 1 2))
    (add "bot_conn_full" (strcat (substr tread_style 1 1)
                                 bot_conn))
    (add "starter_p" (if (or (= "1" bot_conn_ser)
                             (= "7" bot_conn_det))
                         "1"
                         "0"))
    (add "dir" (stair-dir ascend z_pos))

    ;; stringer data
    (foreach side '("l" "r")
             ;; extensions
             (add (strcat side "-string_bex")
                  (+ bot_ext (symcat '(#side "-string_xtra_bex"))))
             (add (strcat side "-string_tex")
                  (+ top_ext (symcat '(#side "-string_xtra_tex"))))
             
             ;; material properties
             (setq mat (cadr (assoc (strcat side "-string_mat") data))
                   string-data (remove (shape-def mat t) '(= (car x) "name"))
                   string-data (mapcar '(lambda(x) (list (strcat side "-string_" (car x))
                                                    (cadr x)))
                                       string-data)
                   ret (append ret string-data))
             ;; add i/o versions of stringer data
             (setq io (if (begins-with (lowercase ascend) side) "i" "o"))
             (foreach item (filter (append data ret) '(begins-with (car x) (strcat side "-")))
                      (add (strcat io (substr (car item) 2))
                           (cadr item))))

    (add "hg_offset" (+ (if (or (begins-with rail_style "1")
                                (begins-with rail_style "2"))
                            28.545
                            29.545)
                        (- i-nosing_mid 2)))
    (add "pitch" (rtd (atan (/ rise_hgt tread_depth))))
    (add "rail_hgt_adj" (/ (- i-nosing_mid 2)
                           (sin (dtr (- 90 pitch)))))
    (add "btwn_stringers" (- width (translate-keys ret '(+ l-string_wid r-string_wid))))

    (defun stringers-equal-p ( / left right tmp)
      (foreach elt data
               (setq tmp (substr (car elt) 1 2))
               (cond ((= tmp "l-") (add-to-list 'left elt))
                     ((= tmp "r-") (add-to-list 'right elt))))
      (vl-every '(lambda(x) (= x T))
                (mapcar '(lambda(a b) (eq (cadr a) (cadr b))) left right)))

    (add "stringers_eq" (if (stringers-equal-p) "1" "0"))

    (if pan_mat
        (add "pan_thick"
             (cond ((= pan_mat "14ga") 0.0747)
                   ((= pan_mat "12ga") 0.1046)
                   ((= pan_mat "3/16") 0.1875)
                   ((= pan_mat "1/4") 0.25))))

    ;; encode guard rail presence as bits
    ;; bit 0 inside bottom
    ;; bit 1 outside bottom
    ;; bit 2 inside top
    ;; bit 3 outside top
    ;; bit 4 between
    (setq guard-rails 0)
    (if (wcmatch bot_conn "*haunch*")
        (progn
          (if (and (= i-rail_no_bot_guard "0")
                   (wcmatch i-rail_type "*Stair*"))
              (setq guard-rails (logior guard-rails 1)))
          (if (and (= o-rail_no_bot_guard "0")
                   (wcmatch o-rail_type "*Stair*"))
              (setq guard-rails (logior guard-rails 2)))))
    (if (or (wcmatch top_conn "*haunch*")
            (wcmatch top_conn "*extension*"))
        (progn
          (if (and (= i-rail_no_top_guard "0")
                   (begins-with i-rail_type "S"))
              (setq guard-rails (logior guard-rails 4)))
          (if (and (= o-rail_no_top_guard "0")
                   (begins-with o-rail_type "S"))
              (setq guard-rails (logior guard-rails 8)))))
    (if (and (= rail_no_guard_btwn "0")
             (> dist_btwn (if (= i-rail_topmnt "1") 3 5)))
        (setq guard-rails (logior guard-rails 16)))
    (add "guard_rails" guard-rails)

    (if stn_style
        (cond ((= stn_style "STNP")
               (add "stn_thick" 0.1875)
               (add "stn_len" 2.0))
              ((= stn_style "STNS")
               (add "stn_thick" 0.125)
               (add "stn_len" 1.5))
              ((= stn_style "STNRS")
               (add "stn_thick" 0.0817)
               (add "stn_len" 2.0))
              ((= stn_style "STNPT")
               (add "stn_thick" 0.125)
               (add "stn_len" 1.5))
              ((= stn_style "STNC")
               (add "stn_thick" 0.125)
               (add "stn_len" 2.0))
              ((= stn_style "STNC3")
               (add "stn_thick" 0.125)
               (add "stn_len" 3.0))))

    (if pan_thick
        (progn
          (add-top-pan-ext)
          (add-bot-pan-ext)))

    (foreach key '("i-rail_top_lvl"
                   "i-rail_bot_lvl"
                   "o-rail_top_lvl"
                   "o-rail_bot_lvl")
             (add key 15.0))


    (append! 'data (vl-sort ret sort-data-by-key)))

  (setq this-func (defun-q-list-ref 'inner))
  (defun-q-list-set 'inner
      (cons (append (car this-func)
                    (mapcar 'read stair-derived-keys))
            (cdr this-func)))
  (with-data data '((inner))))

;; *_pan_ext is used in fab for correct dimensioning and to draw the
;; end of the last pan at the right location
(defun add-top-pan-ext (/ ts chkr-test)
  (setq ts (substr tread_style 1 1))
  (add "top_pan_ext"
       (cond ((member ts '("1" "4"))
              (if (= top_conn_det "0") -0.5 0.0))
             ((= ts "2")
              (defun chkr-test (default)
                (if (member top_conn_ser '("2" "5")) default 0.0))
              (if (= stn_style "STNS")
                  (chkr-test 0.25)
                  (chkr-test 0.75))))))

(defun add-bot-pan-ext (/ ts chkr-test)
  (setq ts (substr tread_style 1 1))
  (if (or (/= starter_p "1")
          (= (atoi bot_conn_full) 227))
      (add "bot_pan_ext"
           (cond ((member ts '("1" "4")) 0.0)
                 ((= ts "2")
                  (defun chkr-test (default)
                    (if (member bot_conn_ser '("2" "5")) default 0.0))
                  (if (= stn_style "STNS")
                  (chkr-test 0.25)
                  (chkr-test 0.75)))))))
