  (set-file-docstring "Function for modeling 3d connections")

  (psc-include '("shapes/shape.lsp" "shapes/decking.lsp"))

  (defun-q add-3d-conns ( / conn con-blk z-ins)
    "handling function for modeling 3d stair connections"
    (if (= show_conn "1")
      (foreach conn-tb '("top" "bot")
        (setq
          conn (symcat '(#conn-tb "_conn_full"))
          z-ins (if (= 227 (read conn)) (- pan_thick) 0)
        ) ;_ setq
        (if (eval (read (strcat "3d-conns--model-conn-" conn)))
          (progn
            (setq con-blk (p-blk (strcat conn-tb "-conn")))
            (mapcar
             '(lambda (def)
                (apply
                 '(lambda (lst ins ext nor lay / obj)
                    (setq obj (add-polyline con-blk '(0 0 0) lst T))
                    (vlax-put-property-r obj 'normal (3dpt nor))
                    (vla-move obj (3dpt 0 0 0) (3dpt ins))
                    (setq obj (plineobj->solidobj obj ext T))
                    (vla-put-layer-r obj lay)
                  ) ;_ lambda
                  def
                ) ;_ apply
              ) ;_ lambda
              ((eval (read (strcat "3d-conns--model-conn-" conn))))
            ) ;_ mapcar	
            (if con-blk (insert-block blockobj (list 0 0 z-ins) con-blk))
          ) ;_ progn
        ) ;_ if
      ) ;_ foreach
    ) ;_ if
  ) ;_ defun-q add-3d-conns
  (defun-r 'add-3d-conns)

  ;| ==================================================================== |;
  ;| ========================= HELPER FUNCTIONS ========================= |;
  ;| ==================================================================== |;

  (defun-q 3d-conns--bot-con-ins ()
    "calculates bottom connection insert"
    (mapcar '*
      (list dir dir 1)
      (list
        (if (= "1" starter_p) (+ bot_cdist (get-bot-kick-dist)) 0)
        (- (/ width 2.0) l-string_wid)
        0
      ) ;_ list
    ) ;_ mapcar
  ) ;_ defun-q 3d-conns--bot-con-ins
  (defun-r '3d-conns--bot-con-ins)

  (defun-q 3d-conns--top-con-ins ()
    "calculates top connection insert"
    (mapcar '*
      (list dir dir 1)
      (list
        (+ bot_ext bot_cdist run top_cdist top_ext)
        (- (/ width 2.0) l-string_wid)
        (* rise_qty rise_hgt)
      ) ;_ list
    ) ;_ mapcar
  ) ;_ defun-q 3d-conns--top-con-ins
  (defun-r '3d-conns--top-con-ins)

  (defun-q 3d-conns--ang-def-llv (shp)
    "orients an angle def to LLV"
    (apply
     '(lambda (l1 l2 th kv)
        (append (vl-sort (list l2 l1) '<) (list th kv))
      ) ;_lambda
      (shape-def shp nil)
    ) ;_ apply
  ) ;_ defun-q 3d-conns--ang-def-llv
  (defun-r '3d-conns--ang-def-llv)

  (defun-q 3d-conns--ang-def-llh (shp)
    "orients an angle def to LLH"
    (apply
     '(lambda (l1 l2 th kv)
        (append (vl-sort (list l2 l1) '>) (list th kv))
      ) ;_lambda
      (shape-def shp nil)
    ) ;_ apply
  ) ;_ defun-q 3d-conns--ang-def-llh
  (defun-r '3d-conns--ang-def-llh)

  (defun-q 3d-conns--mod-pts-lst (lst adj)
    "modifies XYZ vertices simulating XYZ mirroring in one operation"
    (mapcar '(lambda (p) (mapcar '* p adj)) lst)
  ) ;_ defun-q 3d-conns--mod-pts-lst
  (defun-r '3d-conns--mod-pts-lst)

  (defun-q 3d-conns--conc-haunch-top ( / ang-off ang-len ang-size)
    "inserts angles and decking for top conc haunch"
    (setq
      ang-size "L3x2x1/4"
      ang-off (+ (get-end-braced-len "top") 0.125)
      ang-len (- (+ top_ext top_cdist) ang-off 2.625)
    ) ;_ setq
    (append
      (list
        (list
          (3d-conns--mod-pts-lst
            (apply 'angle-pts (shape-def "L2x2x1/4" nil))
           '(-1 1 -1)
          ) ;_ 3d-conns--mod-pts-lst
          (mapcar '-
            (3d-conns--top-con-ins)
            (list (* dir (+ ang-off ang-len 0.125)) 0 3.5)
          ) ;_ mapcar
          btwn_stringers
          (list 0 (- dir) 0)
          "Stair Supports"
        ) ;_ list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '- (3d-conns--top-con-ins) (list (* dir ang-off) 0 3.5))
          ang-len
          (list (- dir) 0 0)
          "Stair Supports"
        ) ;_ list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '-
            (mapcar '* (3d-conns--top-con-ins) '(1 -1 1))
            (list (* dir ang-off) 0 3.5)
          ) ;_ mapcar
          (- ang-len)
          (list dir 0 0)
          "Stair Supports"
        ) ;_ list
      ) ;_ list
      (list
        (list
          (decking-vertices (- (+ ang-len 0.75)))
          (mapcar '-
            (3d-conns--top-con-ins)
            (list (* dir (- ang-off 0.375)) 0 3.5)
          ) ;_ mapcar
          btwn_stringers
          (list 0 (- dir) 0)
          "Stair Pans"
        ) ;_ list
      ) ;_ list
    ) ;_ append
  ) ;_ defun-q 3d-conns--conc-haunch-top
  (defun-r '3d-conns--conc-haunch-top)

  (defun-q 3d-conns--conc-haunch-bot ( / ang-off ang-len ang-size)
    "inserts angles and decking for bottom conc haunch"
    (setq
      ang-size "L3x2x1/4"
      ang-off (+ (get-end-braced-len "bot") 0.125)
      ang-len (- (+ bot_ext bot_cdist) ang-off 5.375)
    ) ;_ setq
    (append
      (list
        (list
          (apply 'angle-pts (shape-def "L3x2x1/4" nil))
          (mapcar '+
            (3d-conns--bot-con-ins)
            (list
              (* dir (+ ang-off ang-len 2.125))
              0
              (- (+ 1.5 pan_thick))
            ) ;_ list
          ) ;_ mapcar
          btwn_stringers
          (list 0 (- dir) 0)
          "Stair Supports"
        ) ;_ list
        (list
          (3d-conns--mod-pts-lst
            (apply 'angle-pts (shape-def "L2x2x1/4" nil))
           '(-1 -1 1)
          ) ;_ 3d-conns--mod-pts-lst
          (mapcar '+
            (3d-conns--bot-con-ins)
            (list (* dir (+ ang-off ang-len 2.125)) 0 -3.75)
          ) ;_ mapcar
          btwn_stringers
          (list 0 (- dir) 0)
          "Stair Supports"
        ) ;_ list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '+ (3d-conns--bot-con-ins) (list (* dir ang-off) 0 -3.5))
          (- ang-len)
          (list (- dir) 0 0)
          "Stair Supports"
        ) ;_ list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '+
            (mapcar '* (3d-conns--bot-con-ins) '(1 -1 1))
            (list (* dir ang-off) 0 -3.5)
          ) ;_ mapcar
          ang-len
          (list dir 0 0)
          "Stair Supports"
        ) ;_ list
      ) ;_ list
      (list
        (list
          (decking-vertices (+ ang-len 0.75))
          (mapcar '+
            (3d-conns--bot-con-ins)
            (list (* dir (- ang-off 0.375)) 0 -3.5)
          ) ;_ mapcar
          btwn_stringers
          (list 0 (- dir) 0)
          "Stair Pans"
        ) ;_ list
      ) ;_ list
    ) ;_ append
  ) ;_ defun-q 3d-conns--conc-haunch-bot
  (defun-r '3d-conns--conc-haunch-bot)

  (defun-q 3d-conns--chkr-haunch-top ( / can-cen ang-len ang-size)
    "inserts angles and cans for top checker haunch"
    (append
      (3d-conns--chkr-haunch-com (3d-conns--top-con-ins) T)
      (list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '-
            (3d-conns--top-con-ins)
            (list (* dir (- (car can-cen) 3.0)) 0 (+ 2.5 pan_thick))
          ) ;_ mapcar
          ang-len
          (list (- dir) 0 0)
          "Stair Supports"
        ) ;_ list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '-
            (3d-conns--top-con-ins)
            (list
              (* dir (- (car can-cen) 3.0))
              (* dir btwn_stringers)
              (+ 2.5 pan_thick)
            ) ;_ list
          ) ;_ mapcar
          (- ang-len)
          (list dir 0 0)
          "Stair Supports"
        ) ;_ list
      ) ;_ list
    ) ;_ append
  ) ;_ defun-q 3d-conns--chkr-haunch-top
  (defun-r '3d-conns--chkr-haunch-top)

  (defun-q 3d-conns--chkr-haunch-bot ( / can-cen ang-len ang-size)
    "inserts angles and cans for bottom checker haunch"
    (append
      (3d-conns--chkr-haunch-com (3d-conns--bot-con-ins) nil)
      (list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '+
            (3d-conns--bot-con-ins)
            (list (* dir (1- bot_ext)) 0 (- pan_thick))
          ) ;_ mapcar
          btwn_stringers
          (list 0 (- dir) 0)
          "Stair Supports"
        ) ;_ list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '+
            (3d-conns--bot-con-ins)
            (list (* dir (- (car can-cen) 3.0)) 0 (- (+ 2.5 pan_thick)))
          ) ;_ mapcar
          (- ang-len)
          (list (- dir) 0 0)
          "Stair Supports"
        ) ;_ list
        (list
          (apply 'angle-pts (shape-def "L2x2x1/4" nil))
          (mapcar '+
            (3d-conns--bot-con-ins)
            (list
              (* dir (- (car can-cen) 3.0))
              (* (- dir) btwn_stringers)
              (- (+ 2.5 pan_thick))
            ) ;_ list
          ) ;_ mapcar
          ang-len
          (list dir 0 0)
          "Stair Supports"
        ) ;_ list
      ) ;_ list
    ) ;_ append
  ) ;_ defun-q 3d-conns--chkr-haunch-bot
  (defun-r '3d-conns--chkr-haunch-bot)

  (defun-q 3d-conns--chkr-haunch-com (ins top / t/b-ext brace-y brace-n)
    "common checker haunch handling for cans and QT"
    (setq
      ang-size "L5x3x1/4"
      t/b-ext (if top (+ top_ext top_cdist) (- bot_ext 1))
      brace-y (get-end-braced-len (if top "top" "bot"))
      brace-n (- t/b-ext brace-y (if top (top-chkr-nosing-braced-len) 0))
      can-cen (stair-calc-can-spacing brace-y brace-n)
      ang-len (+ (- (last can-cen) (car can-cen)) 6.0)
    ) ;_ setq
    (append
      (mapcar ;;; Pl Cans
       '(lambda (x)
          (list
           '((-3.0 2.5) (-3.0 0.0) (3.0 0.0) (3.0 2.5)
             (2.9253 2.5) (2.9253 0.0747) (-2.9253 0.0747) (-2.9253 2.5))
            (mapcar '- ins (list (* dir x) 0 (+ 2.5 pan_thick)))
            btwn_stringers
            (list 0 (- dir) 0)
            "Stair Pans"
          ) ;_ list
        ) ;_ lambda
        (mapcar (if top '+ '-) can-cen)
      ) ;_ mapcar
      (if (= quiet_tread "1")
        (mapcar ;;; Q-Tread-Lo
         '(lambda (x)
            (list
             '((-2.0 0.0) (-2.0 -0.125) (2.0 -0.125) (2.0 0.0))
              (mapcar '-
                ins
                (list (* dir x) (* dir 3.0) (+ 2.5 pan_thick))
              ) ;_ mapcar
              (- btwn_stringers 6.0)
              (list 0 (- dir) 0)
              "Stair Quiet Tread"
            ) ;_ list
          ) ;_ lambda
          (mapcar (if top '+ '-) can-cen)
        ) ;_ mapcar
      ) ;_ if
    ) ;_ append
  ) ;_ defun-q 3d-conns--chkr-haunch-com
  (defun-r '3d-conns--chkr-haunch-com)

  ;| ==================================================================== |;
  ;| ================= STARTER CONNECTIONS (X1X SERIES) ================= |;
  ;| ==================================================================== |;

  (defun-q 3d-conns--model-conn-X10 ()
    ""
    (list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (3d-conns--ang-def-llh "L4X3X1/4"))
         '(1 -1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (3d-conns--bot-con-ins)
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X10
  (defun-r '3d-conns--model-conn-X10)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X10))
    (list
      '3d-conns--model-conn-110 '3d-conns--model-conn-111 '3d-conns--model-conn-114 '3d-conns--model-conn-115
      '3d-conns--model-conn-210 '3d-conns--model-conn-211 '3d-conns--model-conn-214 '3d-conns--model-conn-215
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X12 ()
    ""
    (list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (shape-def "L3X3X1/4" nil))
         '(1 -1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (3d-conns--bot-con-ins)
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X12
  (defun-r '3d-conns--model-conn-X12)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X12))
    (list
      '3d-conns--model-conn-112 '3d-conns--model-conn-113
      '3d-conns--model-conn-212 '3d-conns--model-conn-213
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X16 ()
    ""
    (list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (3d-conns--ang-def-llv "L4X3X1/4"))
         '(1 -1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (3d-conns--bot-con-ins)
        (- (+ (* 2 bot_hbolt) bot_hbolt2))
        (list (- dir) 0 0)
        "Stair Supports"
      ) ;_ list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (3d-conns--ang-def-llv "L4X3X1/4"))
         '(1 -1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '* (3d-conns--bot-con-ins) '(1 -1 1))
        (+ (* 2 bot_hbolt) bot_hbolt2)
        (list dir 0 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X16
  (defun-r '3d-conns--model-conn-X16)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X16))
    (list
      '3d-conns--model-conn-116
      '3d-conns--model-conn-216
    ) ;_ list
  ) ;_ mapcar

  ;| ==================================================================== |;
  ;| ================= LANDING CONNECTIONS (X2X SERIES) ================= |;
  ;| ==================================================================== |;

  (defun-q 3d-conns--model-conn-X20 ()
    ""
    (list
      (list
        (apply 'pl-pts (shape-def "PL1/4x4" nil))
        (mapcar '- (3d-conns--top-con-ins) (list (* dir 0.25) 0.0 (- top_vbolt 2.0)))
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X20
  (defun-r '3d-conns--model-conn-X20)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X20))
    (list
      '3d-conns--model-conn-120 '3d-conns--model-conn-150
      '3d-conns--model-conn-220 '3d-conns--model-conn-250
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X21 ()
    ""
    (list
      (list
        (apply 'angle-pts (3d-conns--ang-def-llv (x4x-inner-angle-size "bot")))
        (mapcar '-
          (3d-conns--bot-con-ins)
          (list 0 0 (+ pan_thick (?n tread_thick)))
        ) ;_ mapcar
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X21
  (defun-r '3d-conns--model-conn-X21)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X21))
    (list
      '3d-conns--model-conn-121 '3d-conns--model-conn-123 '3d-conns--model-conn-151 '3d-conns--model-conn-153
      '3d-conns--model-conn-221 '3d-conns--model-conn-223 '3d-conns--model-conn-251 '3d-conns--model-conn-253
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X22 ()
    ""
    (list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (3d-conns--ang-def-llv (x4x-inner-angle-size "top")))
         '(-1 1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '-
          (3d-conns--top-con-ins)
          (list 0 0 pan_thick)
          (list 0 0 (?n tread_thick))
        ) ;_ mapcar
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X22
  (defun-r '3d-conns--model-conn-X22)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X22))
    (list
      '3d-conns--model-conn-122 '3d-conns--model-conn-152
      '3d-conns--model-conn-222 '3d-conns--model-conn-252
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X24 ()
    ""
    (if (begins-with tread_style "2")
      (append
        (3d-conns--model-conn-X22)
        (3d-conns--chkr-haunch-top)
      ) ;_ append
      (append
        (list
          (list
            (3d-conns--mod-pts-lst
              (apply 'angle-pts (3d-conns--ang-def-llv "L3x2x1/4"))
             '(-1 1 -1)
            ) ;_ 3d-conns--mod-pts-lst
            (mapcar '- (3d-conns--top-con-ins) '(0 0 3.5))
            btwn_stringers
            (list 0 (- dir) 0)
            "Stair Supports"
          ) ;_ list
        ) ;_ list
        (3d-conns--conc-haunch-top)
      ) ;_ append
    ) ;_ if
  ) ;_ defun-q 3d-conns--model-conn-X24
  (defun-r '3d-conns--model-conn-X24)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X24))
    (list
      '3d-conns--model-conn-124 '3d-conns--model-conn-154
      '3d-conns--model-conn-224 '3d-conns--model-conn-254
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X25 ()
    ""
    (if (begins-with tread_style "2")
      (append
        (3d-conns--model-conn-X21)
        (3d-conns--chkr-haunch-bot)
      ) ;_ append
      (append
        (list
          (list
            (apply 'angle-pts (3d-conns--ang-def-llv "L3x2x1/4"))
            (mapcar '- (3d-conns--bot-con-ins) '(0 0 3.5))
            btwn_stringers
            (list 0 (- dir) 0)
            "Stair Supports"
          ) ;_ list
        ) ;_ list
        (3d-conns--conc-haunch-bot)
      ) ;_ append
    ) ;_ if
  ) ;_ defun-q 3d-conns--model-conn-X25
  (defun-r '3d-conns--model-conn-X25)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X25))
    (list
      '3d-conns--model-conn-125 '3d-conns--model-conn-155
      '3d-conns--model-conn-225 '3d-conns--model-conn-255
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X27 ( / X27-gus-lst)
    ""
    (setq X27-gus-lst
      (list
        '(-5.5 -0.5 0.0) '(-1.0 -0.5 0.0) '(-0.5 -1.0 0.0)
        '(-0.5 -7.75 0.414214) '(-0.25 -8.0 0.0) '(0.0 -8.0 0.0)
        '(0.0 -10.5 0.0)
      ) ;_ list
    ) ;_ setq
    (list
      (list
        (apply 'angle-pts (3d-conns--ang-def-llv "L8x6x1/2"))
        (list 0 (* dir (/ width 2.0)) 0)
        width
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (3d-conns--ang-def-llv "L3X2X1/4"))
         '(1 -1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (3d-conns--bot-con-ins)
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (shape-def "L3X3X1/4" nil))
         '(1 -1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '*
          (list dir dir 1)
          (list 7.5 (- (/ width 2.0) (+ l-string_wid 4)) -0.75)
        ) ;_ mapcar
        -4.0
        (list 0 dir 0)
        "Stair Supports"
      ) ;_ list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (shape-def "L3X3X1/4" nil))
         '(1 -1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '*
          (list dir (- dir) 1)
          (list 7.5 (- (/ width 2.0) (+ r-string_wid 4)) -0.75)
        ) ;_ mapcar
        4.0
        (list 0 dir 0)
        "Stair Supports"
      ) ;_ list
      (list
        X27-gus-lst
        (mapcar '*
          (list dir dir 1)
          (list 0 (- (/ width 2.0) (+ l-string_wid 2.75)) 0)
        ) ;_ mapcar
        0.25
        (list 0 dir 0)
        "Stair Supports"
      ) ;_ list
      (list
        X27-gus-lst
        (mapcar '*
          (list dir (- dir) 1)
          (list 0 (- (/ width 2.0) (+ r-string_wid 2.75)) 0)
        ) ;_ mapcar
        -0.25
        (list 0 dir 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X27
  (defun-r '3d-conns--model-conn-X27)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X27))
    (list
      '3d-conns--model-conn-127
      '3d-conns--model-conn-227
    ) ;_ list
  ) ;_ mapcar

  ;| ==================================================================== |;
  ;| ================== SLAB CONNECTIONS  (X3X SERIES) ================== |;
  ;| ==================================================================== |;

  (defun-q 3d-conns--model-conn-X30 ()
    ""
    (list
      (list
        (apply 'angle-pts (3d-conns--ang-def-llv "L4X3X1/4"))
        (mapcar '- (3d-conns--top-con-ins) (list (* dir 0.5) 0.0 1.25))
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X30
  (defun-r '3d-conns--model-conn-X30)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X30))
    (list '3d-conns--model-conn-130 '3d-conns--model-conn-230)
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X31 ()
    ""
    (list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (3d-conns--ang-def-llh "L4X3X1/4"))
         '(-1 -1 1)
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '+
          (3d-conns--bot-con-ins)
          (list (* dir (if (begins-with tread_style "1") 2.5 2.25)) 0 -1.5)
        ) ;_ mapcar
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X31
  (defun-r '3d-conns--model-conn-X31)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X31))
    (list '3d-conns--model-conn-131 '3d-conns--model-conn-231)
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X32 ()
    ""
    (list
      (list
        (apply 'angle-pts (3d-conns--ang-def-llv "L3X2X1/4"))
        (mapcar '- (3d-conns--top-con-ins) (list (* dir 0.5) 0.0 1.25))
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (shape-def "L2X2X1/4" nil))
         '(-1 1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '-
          (3d-conns--top-con-ins)
          (list
            (* dir 0.5)
            0.0
            (cond
              ( (begins-with tread_style "2") pan_thick)
              ( (= top_conn_det "2") (+ pan_thick tread_thick))
              ( (= T) 3.5)
            ) ;_ cond
          ) ;_ list
        ) ;_ mapcar
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X32
  (defun-r '3d-conns--model-conn-X32)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X32))
    (list '3d-conns--model-conn-132 '3d-conns--model-conn-232)
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X33 ()
    ""
    (list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts (3d-conns--ang-def-llv "L3X2X1/4"))
         '(-1 1 -1)
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '-
          (3d-conns--bot-con-ins)
          (list (* dir -0.5) 0 1.25)
        ) ;_ mapcar
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
      (list
        (apply 'angle-pts (shape-def "L2X2X1/4" nil))
        (mapcar '-
          (3d-conns--bot-con-ins)
          (list
            (* dir -0.5)
            0.0
            (cond
              ( (begins-with tread_style "2") pan_thick)
              ( (= top_conn_det "2") (+ pan_thick tread_thick))
              ( (= T) 3.5)
            ) ;_ cond
          ) ;_ list
        ) ;_ mapcar
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--model-conn-X33
  (defun-r '3d-conns--model-conn-X33)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X33))
    (list '3d-conns--model-conn-133 '3d-conns--model-conn-233)
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X34 ()
    ""
    (append
      (3d-conns--model-conn-X32)
      (if (begins-with tread_style "1")
        (3d-conns--conc-haunch-top)
        (3d-conns--chkr-haunch-top)
      ) ;_ if
    ) ;_ append
  ) ;_ defun-q 3d-conns--model-conn-X34
  (defun-r '3d-conns--model-conn-X34)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X34))
    (list '3d-conns--model-conn-134 '3d-conns--model-conn-234)
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X35 ()
    ""
    (append
      (3d-conns--model-conn-X33)
      (if (begins-with tread_style "1")
        (3d-conns--conc-haunch-bot)
        (3d-conns--chkr-haunch-bot)
      ) ;_ if
    ) ;_ append
  ) ;_ defun-q 3d-conns--model-conn-X35
  (defun-r '3d-conns--model-conn-X35)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X35))
    (list '3d-conns--model-conn-135 '3d-conns--model-conn-235)
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X37 ()
    ""
    (append
      (3d-conns--model-conn-X12)
      (list
        (list
          (3d-conns--mod-pts-lst
            (apply 'angle-pts (shape-def "L2X2X1/4" nil))
            (list dir dir 1)
          ) ;_ 3d-conns--mod-pts-lst
          (mapcar '+ (3d-conns--bot-con-ins) (list (* dir 6) 0 0))
          4
          (list 0 0 1)
          "Stair Supports"
        ) ;_ list
        (list
          (3d-conns--mod-pts-lst
            (apply 'angle-pts (shape-def "L2X2X1/4" nil))
            (list dir (- dir) -1)
          ) ;_ 3d-conns--mod-pts-lst
          (mapcar '+ (3d-conns--bot-con-ins) (list (* dir 6) (* (- dir) btwn_stringers) 0))
          4
          (list 0 0 1)
          "Stair Supports"
        ) ;_ list
      ) ;_ list
    ) ;_ append
  ) ;_ defun-q 3d-conns--model-conn-X37
  (defun-r '3d-conns--model-conn-X37)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X37))
    (list '3d-conns--model-conn-137 '3d-conns--model-conn-237)
  ) ;_ mapcar

  ;| ==================================================================== |;
  ;| ================== BEAM CONNECTIONS  (X4X SERIES) ================== |;
  ;| ==================================================================== |;

  (defun-q 3d-conns--X4X-out-ang (t/b / ang llo slb)
    ""
    ( (lambda (def) (setq ang (car def) llo (cadr def)))
      (strsplit (symcat '(#t/b "_member")) "_")
    ) ;_ lambda func
    (setq slb (symcat '(#t/b "_slab")))
    (list
      (list
        (3d-conns--mod-pts-lst
          (apply 'angle-pts
            (cond
              ( (= llo "LLV") (3d-conns--ang-def-llv ang))
              ( (= llo "LLH") (3d-conns--ang-def-llh ang))
              ( (= T) (3d-conns--ang-def-llv ang))
            ) ;_ cond
          ) ;_ apply
          (if (= t/b "top") (list 1 -1 -1) (list -1 -1 1))
        ) ;_ 3d-conns--mod-pts-lst
        (mapcar '-
          ((eval (read (strcat "3d-conns--" t/b "-con-ins"))))
          (list 0.0 0.0 slb)
        ) ;_ mapcar
        btwn_stringers
        (list 0 (- dir) 0)
        "Stair Supports"
      ) ;_ list
    ) ;_ list
  ) ;_ defun-q 3d-conns--X4X-out-ang
  (defun-r '3d-conns--X4X-out-ang)

  (defun-q 3d-conns--model-conn-X40 ()
    ""
    (append (3d-conns--model-conn-X20) (3d-conns--X4X-out-ang "top"))
  ) ;_ defun-q 3d-conns--model-conn-X40
  (defun-r '3d-conns--model-conn-X40)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X40))
    (list
      '3d-conns--model-conn-140 '3d-conns--model-conn-160
      '3d-conns--model-conn-240 '3d-conns--model-conn-260
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X41 ()
    ""
    (append (3d-conns--model-conn-X21) (3d-conns--X4X-out-ang "bot"))
  ) ;_ defun-q 3d-conns--model-conn-X41
  (defun-r '3d-conns--model-conn-X41)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X41))
    (list
      '3d-conns--model-conn-141 '3d-conns--model-conn-161 '3d-conns--model-conn-143 '3d-conns--model-conn-163
      '3d-conns--model-conn-241 '3d-conns--model-conn-261 '3d-conns--model-conn-243 '3d-conns--model-conn-263
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X42 ()
    ""
    (append (3d-conns--model-conn-X22) (3d-conns--X4X-out-ang "top"))
  ) ;_ defun-q 3d-conns--model-conn-X42
  (defun-r '3d-conns--model-conn-X42)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X42))
    (list
      '3d-conns--model-conn-142 '3d-conns--model-conn-162
      '3d-conns--model-conn-242 '3d-conns--model-conn-262
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X44 ()
    ""
    (append (3d-conns--model-conn-X24) (3d-conns--X4X-out-ang "top"))
  ) ;_ defun-q 3d-conns--model-conn-X44
  (defun-r '3d-conns--model-conn-X44)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X44))
    (list
      '3d-conns--model-conn-144 '3d-conns--model-conn-164
      '3d-conns--model-conn-244 '3d-conns--model-conn-264
    ) ;_ list
  ) ;_ mapcar

  (defun-q 3d-conns--model-conn-X45 ()
    ""
    (append (3d-conns--model-conn-X25) (3d-conns--X4X-out-ang "bot"))
  ) ;_ defun-q 3d-conns--model-conn-X45
  (defun-r '3d-conns--model-conn-X45)

  (mapcar
   '(lambda (f) (set f 3d-conns--model-conn-X45))
    (list
      '3d-conns--model-conn-145 '3d-conns--model-conn-165
      '3d-conns--model-conn-245 '3d-conns--model-conn-265
    ) ;_ list
  ) ;_ mapcar
