;; x4x series

;; handle the orientation automatically
(defun add-x4x-outer-angle (tb / ang)
  (setq ang (add-shape blockObj (list 0 oang-y 0) oang-size))
  (if (member oang-orient '("LLH" nil))
      (if (= tb "top")
          (mirror&delete ang (list 0 oang-y) 'x)
          (vla-Rotate ang (3dpt 0 oang-y) pi))
      (progn
        (vla-Rotate ang (3dpt 0 oang-y) (dtr 90))
        (if (= tb "bot")
            (vla-mirror-y ang)))))

;; part detail for outer angle
;; depends on x4x-material-vars being set
(defun x4x-outer-angle-detail (tb)
  (list "detail"
        (if (= bolt-qty 2)
            (list '2h-angle
                  oang-size
                  (if (= oang-orient "LLH") 'short 'long)
                  len
                  h1
                  h2
                  (- (tbkey "slab") (tbkey "vbolt"))
                  hole-dia
                  1)
            (list '4h-angle
                  oang-size
                  (if (= oang-orient "LLH") 'short 'long)
                  len
                  h1
                  h2
                  (tbkey "hbolt2")
                  (- (tbkey "slab") (tbkey "vbolt"))
                  hole-dia))))

;; determine inner angle size
;; minimum 1" edge distance
(defun x4x-inner-angle-size (tb / bolt-y ang-y)
  (setq bolt-y (- (?n (symcat '(#tb "_vbolt")))))
  (if (begins-with tread_style "2")
      (progn
        (setq ang-y (- 0 (?f pan_thick) (?f tread_thick)))
        (if (> (- (- bolt-y) (- ang-y)) 4)
            "L7x4x3/8"
            "L5x3x1/4"))
      (if (member (tbkey "conn_det") '("4" "5"))
          (if (< bolt-y -5.5)
              "L4x3x1/4"
              "L3x2x1/4")
          (if (< bolt-y -4.5)
              "L5x3x1/4"
              "L4x3x1/4"))))

;; part detail for inner angle
;; depends on x4x-material-vars being set
(defun x4x-inner-angle-detail (tb / this-bolt-y len h1 h2)
  (if (wcmatch (tbkey "conn") "*haunch*")
      (setq len (- btwn_stringers 0.0625)
            h1 (- (tbkey "hbolt") l-string_wid 0.03125)
            h2 (- (tbkey "hbolt") r-string_wid 0.03125))
      (setq len (- btwn_stringers 0.25)
            h1 (- (tbkey "hbolt") l-string_wid 0.125)
            h2 (- (tbkey "hbolt") r-string_wid 0.125)))
  (list "detail"
        (if (= bolt-qty 2)
            (list '2h-angle
                  iang-size
                  'long
                  len
                  h1
                  h2
                  bolt-y
                  hole-dia
                  1)
            (list '4h-angle
                  iang-size
                  'long
                  len
                  h1
                  h2
                  (tbkey "hbolt2")
                  bolt-y
                  hole-dia))))

;; set common vars
(defun x4x-side-vars (tb)
  (setq bolt-y (- (tbkey "vbolt"))
        oang-y (- (tbkey "slab"))
        oang-size (strsplit (tbkey "member") "_")
        oang-orient (cadr oang-size)
        oang-size (car oang-size)
        oang-hlen (distof ((if (= oang-orient "LLV") cadr car)
                           (strsplit (substr oang-size 2) "x")))
        iang-size (x4x-inner-angle-size tb)
        iang-hlen (distof (cadr (strsplit iang-size "x")))
        iang-y (- 0
                  (?f tread_thick)
                  (?f pan_thick)
                  (if (= tb "top")
                      (?f stn_thick)
                      0))))

(defun x4x-top-vars (tb / iang-size oang-size)
  (setq oang-size (strsplit (tbkey "member") "_")
        oang-orient (cadr oang-size)
        oang-size (strsplit (substr (car oang-size) 2) "x")
        oang-hlen (distof ((if (= oang-orient "LLV") cadr car) oang-size))
        oang-thick (distof (last oang-size))
        iang-size (strsplit (substr (x4x-inner-angle-size tb) 2) "x")
        iang-hlen (distof (cadr iang-size))
        iang-thick (distof (last iang-size))))

(defun x4x-material-vars (tb)
  (setq oang-size (strsplit (tbkey "member") "_")
        oang-orient (cadr oang-size)
        oang-size (car oang-size)
        iang-size (x4x-inner-angle-size tb)
        iang-hlen (distof (cadr (strsplit iang-size "x")))
        iang-y (- 0 (?f tread_thick) (?f pan_thick))
        len (- btwn_stringers 0.25)
        hole-dia (x2x-hole-dia (tbkey "hdw"))
        h1 (- (tbkey "hbolt") l-string_wid 0.125)
        h2 (- (tbkey "hbolt") r-string_wid 0.125)
        bolt-y (if (and (begins-with tread_style "1")
                             (member (tbkey "conn_det") '("4" "5")))
                   (- (tbkey "vbolt") 3.5)
                 (- (tbkey "vbolt") (?f tread_thick) (?f pan_thick)))))

;; common top view
(defun detail-top-x4x (tb bolt-qty
                       / oang-orient oang-thick oang-hlen iang-hlen iang-thick
                       i o)
  (x4x-top-vars tb)
  ;; "inside" and "outside" directionals
  (if (= tb "top")
      (setq i - o +)
      (setq i + o -))
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj (i iang-hlen) btwn_stringers '(0 0 0))
  (add-line-btwn (i iang-thick))

  (set-active-layer "SOLID")
  (add-rectangle blockObj (o oang-hlen) btwn_stringers '(0 0 0))
  (add-line-btwn (o oang-thick))

  (set-active-layer "DIM")
  (add-mleader blockObj
               (list (i (/ iang-hlen 2.0)) 0 0)
               (#! '((0 0 0)
                     (#(o 1) -12 0)
                     (#(o 2) -12 0)))
               (strcat (xd-value (strcat "x4" (substr tb 1 1) "-iang") marks) " (LLV)"))
  (add-mleader blockObj
               (list (o (/ oang-hlen 2.0)) btwn_stringers 0)
               (#! '((0 0 0)
                     (#(o 1) 12 0)
                     (#(o 2) 12 0)))
               (strcat (xd-value (strcat "x4" (substr tb 1 1) "-oang") marks)
                       (if oang-orient
                           (strcat " (" oang-orient ")")
                           "")))

  ((if (= tb "top") bolts-top-right bolts-top-left) bolt-qty))

;;;;;;;;
;; x40
;;;;;;;;

(defun detail-side-x40 (fab-p
                        / bolt-y oang-y oang-size oang-orient oang-hlen iang-size iang-y iang-hlen)
  (x4x-side-vars "top")
  (add-x4x-outer-angle "top")
  (add-rectangle blockObj -0.25 -4 (list 0 (+ bolt-y 2)))
  (add-bolt-centerline (list 0 bolt-y) 'h)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt oang-hlen oang-y) 1 T)
        (detail-side-add-dim-right (3dpt 1 bolt-y) 0 T)
        (add-mleader blockObj
                     (list 0 oang-y 0)
                     '((0 0 0)
                       (2 -6 0)
                       (3 -6 0))
                     (strcat (xd-value "x40-fb" marks) "\n"
                             (xd-value "x40-oang" marks)
                             (if oang-orient
                                 (strcat " (" oang-orient ")")
                                 "")))))

  T)

(setq detail-side-140 detail-side-x40
      detail-side-160 detail-side-x40
      detail-side-240 detail-side-x40
      detail-side-260 detail-side-x40
      detail-side-440 detail-side-x40
      detail-side-460 detail-side-x40)

(defun detail-top-x40 (bolt-qty
                       / oang-orient oang-thick oang-hlen iang-hlen iang-thick)
  (x4x-top-vars "top")
  
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj -0.25 btwn_stringers (list 0 0 0))
  
  (set-active-layer "SOLID")
  (add-rectangle blockObj oang-hlen btwn_stringers (list 0 0 0))
  (add-line-btwn oang-thick)

  (set-active-layer "DIM")
  (add-mleader blockObj
               (list (/ oang-hlen 2.0) 0 0)
               '((0 0 0)
                 (0 -6 0)
                 (-1 -6 0))
               (strcat (xd-value "x40-oang" marks)
                       (if oang-orient
                           (strcat " (" oang-orient ")")
                           "")))
  (add-mleader blockObj
               (list -0.125 btwn_stringers 0)
               '((0 0 0)
                 (0 6 0)
                 (-1 6 0))
               (xd-value "x40-fb" marks))

  (bolts-top-right bolt-qty))

(defun detail-top-140 () (detail-top-x40 2))
(defun detail-top-160 () (detail-top-x40 4))
(defun detail-top-240 () (detail-top-x40 2))
(defun detail-top-260 () (detail-top-x40 4))
(defun detail-top-440 () (detail-top-x40 2))
(defun detail-top-460 () (detail-top-x40 4))

(defun material-x40 (bolt-qty
                     / oang-size oang-orient iang-size iang-y iang-hlen len hole-dia h1 h2 bolt-y
                     fb-detail l-detail)
  (x4x-material-vars "top")
  (setq fb-detail (list "detail"
                        (if (= bolt-qty 2)
                            (list '2h-flatbar 1 len 4 h1 h2 2 hole-dia)
                            (list '4h-flatbar len 4 h1 h2 top_hbolt2 2 hole-dia)))
        l-detail (x4x-outer-angle-detail "top"))
  (#! '(
        ;; material
        (
         (("id" "x40-fb")
          ("desc" "FLAT BAR 1/4 x 4")
          ("qty" 1)
          ("length" #(frac len))
          #fb-detail)
         (("id" "x40-oang")
          ("desc" #(angle-desc oang-size))
          ("qty" 1)
          ("length" #(frac len))
          #l-detail)
         )
        ;;hardware
        (
         (("qty" #bolt-qty)
          ("desc" #(tc-bolt-desc top_hdw)))
         ))))

(defun material-140 () (material-x40 2))
(defun material-160 () (material-x40 4))
(defun material-240 () (material-x40 2))
(defun material-260 () (material-x40 4))
(defun material-440 () (material-x40 2))
(defun material-460 () (material-x40 4))

;;;;;;;;
;; x41
;;;;;;;;

(defun detail-side-x41 (fab-p
                        / bolt-y oang-y oang-size oang-orient oang-hlen iang-y iang-size iang-hlen
                        iang gusset-p)
  (x4x-side-vars "bot")
  (setq iang (add-shape blockObj (list 0 iang-y 0) iang-size)
        gusset-p (bot-triangle-gusset-p iang-y iang-hlen (?? side "i") T))
  (vla-rotate iang (3dpt 0 iang-y) (dtr -90))
  (vla-mirror-y iang)
  (add-x4x-outer-angle "bot")
  (add-bolt-centerline (list 0 bolt-y) 'h)
  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt -1 bolt-y) -1 T)
        (detail-side-add-dim-left (3dpt (- oang-hlen) oang-y) 0 nil)
        (add-mleader blockObj
                     (list 0 bolt-y 0)
                     '((0 0 0)
                       (3 -8 0)
                       (4 -8 ))
                     (strcat (xd-value "x4b-iang" marks) " (LLV)\n"
                             (xd-value "x4b-oang" marks)
                             (if oang-orient
                                 (strcat " (" oang-orient ")")
                                 "")
                             (if gusset-p (strcat "\n" (xd-value "bot-gusset" marks)) "")))))
  T)

(setq detail-side-141 detail-side-x41
      detail-side-241 detail-side-x41
      detail-side-441 detail-side-x41
      detail-side-161 detail-side-x41
      detail-side-261 detail-side-x41
      detail-side-261 detail-side-x41)

(defun detail-top-141 () (detail-top-x4x "bot" 2))
(defun detail-top-161 () (detail-top-x4x "bot" 4))
(defun detail-top-241 () (detail-top-x4x "bot" 2))
(defun detail-top-261 () (detail-top-x4x "bot" 4))
(defun detail-top-441 () (detail-top-x4x "bot" 2))
(defun detail-top-461 () (detail-top-x4x "bot" 4))

(defun material-x41 (bolt-qty
                     / oang-size oang-orient iang-size iang-y iang-hlen len hole-dia h1 h2 bolt-y
                     material-list)
  (x4x-material-vars "bot")
  (setq material-list (#! '(
                            (("id" "x4b-oang")
                             ("desc" #(angle-desc oang-size))
                             ("qty" 1)
                             ("length" #(frac len))
                             #(x4x-outer-angle-detail "bot"))
                            (("id" "x4b-iang")
                             ("desc" #(angle-desc iang-size))
                             ("qty" 1)
                             ("length" #(frac len))
                             #(x4x-inner-angle-detail "bot"))
                            ))
        gusset-qty (+ (if (bot-triangle-gusset-p iang-y iang-hlen "i" nil) 1 0)
                      (if (bot-triangle-gusset-p iang-y iang-hlen "o" nil) 1 0)))
  (if (> gusset-qty 0)
      (add-to-list 'material-list
                   (#! '(("id" "bot-gusset")
                         ("desc" "FLAT BAR 1/4 x 2")
                         ("qty" #gusset-qty)
                         ("length" "7 3/16")
                         ("detail" (bot-triangle-gusset))))))
  (#! '(#material-list
        ;; hardware
        (
         (("qty" #bolt-qty)
          ("desc"  #(tc-bolt-desc bot_hdw)))
         ))))

(defun material-141 () (material-x41 2))
(defun material-161 () (material-x41 4))
(defun material-241 () (material-x41 2))
(defun material-261 () (material-x41 4))
(defun material-441 () (material-x41 2))
(defun material-461 () (material-x41 4))

;;;;;;;;
;; x42
;;;;;;;;

;; determine if a gusset is needed
(defun x42-gusset-p (side / string-line iang-vbury iang-thick min-vbury iang)
  (setq string-line (bot-string-line-from-top side)
        ;; how much the vertical leg is buried by the stringer
        iang-vbury (- (- (line-val string-line 0)) (- iang-y))
        iang (car (strsplit top_member "_"))
        iang-thick (distof (last (strsplit iang "x")))
        min-vbury (- 4 (- iang-hlen iang-thick)))
  (< (- iang-vbury iang-thick) min-vbury))

(defun detail-side-x42 (fab-p
                        / bolt-y oang-y oang-size oang-orient oang-hlen iang-y iang-size iang-hlen
                        )
  (x4x-side-vars "top")
  (setq ang (add-shape blockObj (list 0 iang-y 0) iang-size))
  (vla-Rotate ang (3dpt 0 iang-y) (dtr -90))
  (add-x4x-outer-angle "top")
  (add-bolt-centerline (list 0 bolt-y) 'h)
  (if (x42-gusset-p (?? side "i"))
      (add-top-triangle-gusset (?? side "i")))

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1 bolt-y) 0 T)
        (detail-side-add-dim-right (3dpt oang-hlen oang-y) 1 T)
        (add-mleader blockObj
                     (list 0 (1- bolt-y) 0)
                     '((0 0 0)
                       (2 -6 0)
                       (3 -6 0))
                     (strcat (xd-value "x4t-iang" marks) " (LLV)\n"
                             (xd-value "x4t-oang" marks)
                             (if oang-orient
                                 (strcat " (" oang-orient ")")
                                 "")
                             (if (x42-gusset-p "i") (strcat "\n" (xd-value "top-gusset" marks)) "")))))

  T)

(setq detail-side-142 detail-side-x42
      detail-side-162 detail-side-142
      detail-side-242 detail-side-x42
      detail-side-262 detail-side-242
      detail-side-442 detail-side-x42
      detail-side-462 detail-side-142)

(defun detail-top-142 () (detail-top-x4x "top" 2))
(defun detail-top-162 () (detail-top-x4x "top" 4))
(defun detail-top-242 () (detail-top-x4x "top" 2))
(defun detail-top-262 () (detail-top-x4x "top" 4))
(defun detail-top-442 () (detail-top-x4x "top" 2))
(defun detail-top-462 () (detail-top-x4x "top" 4))

(defun material-x42 (bolt-qty
                     / oang-size oang-orient iang-size iang-y iang-hlen len hole-dia h1 h2 bolt-y
                     material-list gusset-qty)
  (x4x-material-vars "top")
  (setq material-list (#! '(
                            (("id" "x4t-iang")
                             ("qty" 1)
                             ("desc" #(angle-desc iang-size))
                             ("length" #(frac len))
                             #(x4x-inner-angle-detail "top"))
                            (("id" "x4t-oang")
                             ("qty" 1)
                             ("desc" #(angle-desc oang-size))
                             ("length" #(frac len))
                             #(x4x-outer-angle-detail "top"))
                            ))
        gusset-qty (+ (if (x42-gusset-p "i") 1 0)
                      (if (x42-gusset-p "o") 1 0)))
  (if (> gusset-qty 0)
      (add-to-list 'material-list
                   (#! '(("id" "top-gusset")
                         ("desc" "FLAT BAR 1/4 x 6")
                         ("qty" #gusset-qty)
                         ("length" #(frac (* 6 (tan (dtr pitch)))))
                         ("detail" (top-triangle-gusset #pitch))))))
  (#! '(#material-list
        ;; hardware
        (
         (("qty" #bolt-qty)
          ("desc"  #(tc-bolt-desc top_hdw))
          )))))

(defun material-142 () (material-x42 2))
(defun material-162 () (material-x42 4))
(defun material-242 () (material-x42 2))
(defun material-262 () (material-x42 4))
(defun material-442 () (material-x42 2))
(defun material-462 () (material-x42 4))

;;;;;;;;
;; x43
;;;;;;;;

(defun detail-side-243 (fab-p)
  (detail-side-x41 fab-p)
  (if (= quiet_tread "1") (add-bot-ext-qt))
  T)
(setq detail-side-143 detail-side-x41
      detail-side-163 detail-side-x41
      detail-side-263 detail-side-243
      detail-side-443 detail-side-x41
      detail-side-463 detail-side-x41

      material-143 material-141
      material-163 material-161
      material-243 material-241
      material-263 material-261
      material-443 material-141
      material-463 material-161)

;;;;;;;;
;; 144
;;;;;;;;

(defun detail-side-x44 (fab-p
                        / bolt-y oang-y oang-size oang-orient oang-hlen iang-y iang-size iang-hlen
                        iang)
  (x4x-side-vars "top")
  (setq iang-y (if (begins-with tread_style "2") iang-y -3.5)
        iang (add-shape blockObj (list 0 iang-y 0) iang-size))
  (vla-Rotate iang (3dpt 0 iang-y) (dtr -90))
  (add-x4x-outer-angle "top")
  (add-bolt-centerline (list 0 bolt-y) 'h)
  
  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1 bolt-y) 0 T)
        (detail-side-add-dim-right (3dpt oang-hlen oang-y) 1 T)
        (add-mleader blockObj
                     (list 0 (1- bolt-y) 0)
                     '((0 0 0)
                       (2 -6 0)
                       (3 -6 0))
                     (strcat (xd-value "x4t-iang" marks) " (LLV)\n"
                             (xd-value "x4t-oang" marks)
                             (if oang-orient
                                 (strcat " (" oang-orient ")")
                                 "")))))
  T)

(defun detail-side-144 (fab-p /)
  (top-conc-haunch-side fab-p)
  (detail-side-x44 fab-p)
  T)

(setq detail-side-164 detail-side-144
      detail-side-444 detail-side-144
      detail-side-464 detail-side-144)

(defun detail-top-144 ()
  (top-conc-haunch-top)
  (detail-top-x4x "top" 2))

(setq detail-top-444 detail-top-144)

(defun detail-top-164 ()
  (top-conc-haunch-top)
  (detail-top-x4x "top" 4))

(setq detail-top-464 detail-top-164)

(defun material-x44 (bolt-qty
                      / oang-size oang-orient iang-size iang-y iang-hlen len hole-dia h1 h2 bolt-y)
  (x4x-material-vars "top")
  (list (append (if (begins-with tread_style "2")
                    (top-chkr-haunch-material)
                  (top-conc-haunch-material))
                (#! '(
                      (("id" "x4t-iang")
                       ("qty" 1)
                       ("desc" #(angle-desc iang-size))
                       ("length" #(frac (- btwn_stringers 0.0625)))
                       #(x4x-inner-angle-detail "top"))
                      (("id" "x4t-oang")
                       ("qty" 1)
                       ("desc" #(angle-desc oang-size))
                       ("length" #(frac len))
                       #(x4x-outer-angle-detail "top"))
                      )))
        (#! '(
              (("qty" #bolt-qty)
               ("desc"  #(tc-bolt-desc top_hdw)))
              ))))

(defun material-144 () (material-x44 2))
(defun material-164 () (material-x44 4))
(defun material-444 () (material-x44 2))
(defun material-464 () (material-x44 4))

;;;;;;;;
;; 244
;;;;;;;;

(defun detail-side-244 (fab-p / ang-size)
  (top-chkr-haunch-side fab-p)
  (detail-side-x44 fab-p)
  T)

(setq detail-side-264 detail-side-244)

(defun detail-top-244 ()
  (top-chkr-haunch-top)
  (detail-top-x4x "top" 2))

(defun detail-top-264 ()
  (top-chkr-haunch-top)
  (detail-top-x4x "top" 4))

(defun material-244 () (material-x44 2))
(defun material-264 () (material-x44 4))

;;;;;;;;
;; 145
;;;;;;;;

(defun detail-side-x45 (fab-p
                        / bolt-y oang-y oang-size oang-orient oang-hlen iang-y iang-size iang-hlen
                        iang)
  (x4x-side-vars "bot")
  (setq iang-y (if (begins-with tread_style "2") iang-y -3.5)
        iang (add-shape blockObj (list 0 iang-y 0) iang-size))
  (vla-Rotate iang (3dpt 0 iang-y) (dtr -90))
  (vla-mirror-y iang)
  (add-x4x-outer-angle "bot")
  (add-bolt-centerline (list 0 bolt-y) 'h)
  
  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt -1 bolt-y) -1 T)
        (detail-side-add-dim-left (3dpt (- oang-hlen) oang-y) 0 nil)
        (add-mleader blockObj
                     (list 0 (1- bolt-y) 0)
                     '((0 0 0)
                       (0 -12 0)
                       (-1 -12 0))
                     (strcat (xd-value "x4b-iang" marks) " (LLV)\n"
                             (xd-value "x4b-oang" marks)
                             (if oang-orient
                                 (strcat " (" oang-orient ")")
                               "")))))
  T)

(defun detail-side-145 (fab-p)
  (bot-conc-haunch-side fab-p)
  (detail-side-x45 fab-p)
  T)

(setq detail-side-165 detail-side-145
      detail-side-445 detail-side-145
      detail-side-465 detail-side-145)

(defun detail-top-145 ()
  (bot-conc-haunch-top)
  (detail-top-x4x "bot" 2))

(setq detail-top-445 detail-top-145)

(defun detail-top-165 ()
  (bot-conc-haunch-top)
  (detail-top-x4x "bot" 4))

(setq detail-top-465 detail-top-165)

(defun material-x45 (bolt-qty
                      / oang-size oang-orient iang-size iang-y iang-hlen len hole-dia h1 h2 bolt-y)
  (x4x-material-vars "bot")
  (list (append (if (begins-with tread_style "2")
                    (bot-chkr-haunch-material)
                  (bot-conc-haunch-material))
                (#! '(
                      (("id" "x4b-iang")
                       ("qty" 1)
                       ("desc" #(angle-desc iang-size))
                       ("length" #(frac (- btwn_stringers 0.0625)))
                       #(x4x-inner-angle-detail "bot"))
                      (("id" "x4b-oang")
                       ("qty" 1)
                       ("desc" #(angle-desc oang-size))
                       ("length" #(frac len))
                       #(x4x-outer-angle-detail "bot"))
                      )))
        (#! '(
              (("qty" #bolt-qty)
               ("desc"  #(tc-bolt-desc bot_hdw)))
              ))))

(defun material-145 () (material-x45 2))
(defun material-165 () (material-x45 4))
(defun material-445 () (material-x45 2))
(defun material-465 () (material-x45 4))

;;;;;;;;
;; 245
;;;;;;;;

(defun detail-side-245 (fab-p / ang-size)
  (bot-chkr-haunch-side fab-p)
  (detail-side-x45 fab-p)
  T)

(setq detail-side-265 detail-side-245)

(defun detail-top-245 ()
  (bot-chkr-haunch-top)
  (detail-top-x4x "bot" 2))

(defun detail-top-265 ()
  (bot-chkr-haunch-top)
  (detail-top-x4x "bot" 4))

(defun material-245 () (material-x45 2))
(defun material-265 () (material-x45 4))
