;; x2x and x5x series 

(defun detail-side-x20 (fab-p / y xtra-tex nosing-top)
  (setq y (- top_vbolt))
  (add-rectangle blockObj 0.25 4 (list -0.25 (- y 2)))
  (add-bolt-centerline (list 0 y) 'h)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (setq xtra-tex (xd-value (strcat side "-string_xtra_tex") data)
              nosing-top (xd-value (strcat side "-nosing_top") data))
        (detail-side-add-dim-right (3dpt 1 y) 0 nil)
        (add-mleader blockObj
                     (list 0 (- y 1) 0)
                     (list '(0 0 0)
                           '(2 0 0)
                           (list 2 (* -2 dimspace) 0)
                           (list 4 (* -2 dimspace) 0))
                     (cadr (assoc "x20-1" marks)))))
  T)

(setq detail-side-120 detail-side-x20
      detail-side-150 detail-side-x20
      detail-side-220 detail-side-x20
      detail-side-250 detail-side-x20
      detail-side-420 detail-side-x20
      detail-side-450 detail-side-x20)

(defun detail-top-x20 (bolt-qty / top-bolt bot-bolt right-dim-line)
  ;; flat bar
  (set-active-layer (if (< top_pan_ext 0) "SOLID" "HIDDEN"))
  (add-rectangle blockObj -0.25 btwn_stringers '(0 0 0))

  ;; bolts
  (bolts-top-right bolt-qty)

  ;; callout
  (add-mleader blockObj
               (list 0 (/ btwn_stringers 2.0) 0)
               '((0 0 0)
                 (2 0 0)
                 (2 8 0)
                 (3 8 0))
               (cadr (assoc "x20-1" marks))))

(defun detail-top-120 () (detail-top-x20 2))
(defun detail-top-150 () (detail-top-x20 4))
(defun detail-top-220 () (detail-top-x20 2))
(defun detail-top-250 () (detail-top-x20 4))
(defun detail-top-420 () (detail-top-x20 2))
(defun detail-top-450 () (detail-top-x20 4))

(defun material-x20 (bolt-qty / len h1 h2 hole-dia)
  (setq len (- btwn_stringers 0.25)
        h1 (- top_hbolt r-string_wid 0.125)
        h2 (- top_hbolt l-string_wid 0.125)
        hole-dia (if (begins-with top_hdw "5/8") 0.59375 0.71875))
  (#!
   '(
     ;; material
     (
      (("id" "x20-1")
       ("desc" "FLAT BAR 1/4 x 4")
       ("qty" 1)
       ("length" #(frac len))
       #(if (= bolt-qty 2)
         (list "detail" (list '2h-flatbar 1 len 4 h1 h2 2 hole-dia))
         (list "detail" (list '4h-flatbar len 4 h1 h2 top_hbolt2 2 hole-dia))))
      )
     ;; hardware
     (
      (("qty" #bolt-qty)
       ("desc" #(tc-bolt-desc top_hdw)))
      ))))

(defun material-120 () (material-x20 2))
(defun material-150 () (material-x20 4))
(defun material-220 () (material-x20 2))
(defun material-250 () (material-x20 4))
(defun material-420 () (material-x20 2))
(defun material-450 () (material-x20 4))

;;;;;;
;; x21
;;;;;;

(defun detail-side-x21 (fab-p ang-size / ang ang-y bolt-y gusset-p ang-vlen ang-hlen)
  (setq ang-y (- 0 (?f tread_thick) pan_thick)
        ang-vlen (strsplit (substr ang-size 2) "x")
        ang-hlen (distof (cadr ang-vlen))
        ang-vlen (distof (car ang-vlen))
        bolt-y (- bot_vbolt)
        ang (add-shape blockObj (list 0 ang-y 0) ang-size))
  (vla-rotate ang (3dpt 0 ang-y) (dtr -90))
  (vla-mirror-y ang)
  (add-bolt-centerline (list 0 bolt-y) 'h)

  (if fab-p
      (progn
        ;; gusset
        (setq gusset-p (bot-triangle-gusset-p ang-y ang-hlen side T))
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt -1 bolt-y) 0 nil)
        ;; angle callout
        (add-mleader blockObj
                     (list 0 (- ang-y ang-vlen) 0)
                     (list '(0 0 0)
                           '(0 -6 0)
                           '(2 -6 0))
                     (strcat (xd-value "x21-1" marks) " (LLV)"))
        ;; gusset callout
        (if gusset-p
            (add-mleader blockObj
                         (list 1 (- ang-y 2) 0)
                         (list '(0 0 0)
                               '(8 0 0))
                         (cadr (assoc "x21-2" marks))))
        ) ;; progn
      (bot-triangle-gusset-p ang-y ang-hlen "i" T))
  T)

(defun detail-side-121 (fab-p) (detail-side-x21 fab-p "L5x3x1/4"))
(defun detail-side-221 (fab-p) (detail-side-x21 fab-p "L7x4x3/8"))
(setq detail-side-151 detail-side-121
      detail-side-251 detail-side-221
      detail-side-421 detail-side-121
      detail-side-451 detail-side-121)

(defun detail-top-x21 (ang-wid ang-thick bolt-qty)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj ang-wid btwn_stringers '(0 0 0))
  (add-line-btwn ang-thick)

  (bolts-top-left bolt-qty)

  (add-mleader blockObj
               (list 0 (- (/ btwn_stringers 2.0) 1) 0)
               '((0 0 0)
                 (-2 0 0)
                 (-2 -5 0)
                 (-3 -5 0))
               (strcat (xd-value "x21-1" marks) "\n(LLV)")))

(defun detail-top-121 () (detail-top-x21 3.0 0.25 2))
(defun detail-top-151 () (detail-top-x21 3.0 0.25 4))
(defun detail-top-221 () (detail-top-x21 4.0 0.375 2))
(defun detail-top-251 () (detail-top-x21 4.0 0.375 4))
(defun detail-top-421 () (detail-top-x21 3.0 0.25 2))
(defun detail-top-451 () (detail-top-x21 3.0 0.25 4))

(defun x2x-hole-dia (hdw) (+ (distof (substr hdw 1 3)) (if galv-p 0.125 0.0625)))

(defun material-x21 (bolt-qty ang-size
                     / len ang-y hole-dia gusset-qty material-list ang-hlen h1 h2 bolt-y detail)
  (setq len (- btwn_stringers 0.25)
        ang-y (- 0 (?f tread_thick) (?f pan_thick))
        ang-hlen (distof (cadr (strsplit ang-size "x")))
        hole-dia (x2x-hole-dia bot_hdw)
        gusset-qty (+ (if (bot-triangle-gusset-p ang-y ang-hlen "i" nil) 1 0)
                      (if (bot-triangle-gusset-p ang-y ang-hlen "o" nil) 1 0))
        h1 (- bot_hbolt l-string_wid 0.125)
        h2 (- bot_hbolt r-string_wid 0.125)
        bolt-y (- bot_vbolt (?f tread_thick) (?f pan_thick))
        detail (if (= bolt-qty 2)
                   (list "detail"
                         (list '2h-angle ang-size 'long len h1 h2 bolt-y hole-dia 1))
                   (list "detail"
                         (list '4h-angle ang-size 'long len h1 h2 bot_hbolt2 bolt-y hole-dia)))
        material-list (#! '((("id" "x21-1")
                             ("qty" 1)
                             ("desc" #(strcat "ANGLE "
                                              (strjoin (strsplit (substr ang-size 2) "x") " x ")))
                             ("length" #(frac len))
                             #detail))))
  (if (> gusset-qty 0)
      (append! 'material-list
               (#! '((("id" "x21-2")
                      ("desc" "FLAT BAR 1/4 x 2")
                      ("qty" #gusset-qty)
                      ("length" "7 3/16")
                      ("detail" (bot-triangle-gusset)))))))
  (#! '(#material-list
        ;; hardware
        (
         (("qty" #bolt-qty)
          ("desc" #(tc-bolt-desc bot_hdw))
          ("note" "SL"))
         ))))

(defun material-121 () (material-x21 2 "L5x3x1/4"))
(defun material-151 () (material-x21 4 "L5x3x1/4"))
(defun material-221 () (material-x21 2 "L7x4x3/8"))
(defun material-251 () (material-x21 4 "L7x4x3/8"))
(defun material-421 () (material-x21 2 "L5x3x1/4"))
(defun material-451 () (material-x21 4 "L5x3x1/4"))

;;;;;;;;
;; x22
;;;;;;;;

(defun x22-vbury (side)
  (- (+ (line-val (bot-string-line-from-top side) 0)
        (?f tread_thick)
        (?f pan_thick)
        (?f stn_thick))))

(defun detail-side-x22 (fab-p ang-size min-vbury / ang ang-y string-line bolt-y gusset-p)
  (setq ang-y (- 0 (?f tread_thick) (?f pan_thick) (?f stn_thick))
        ang (add-shape blockObj (list 0 ang-y 0) ang-size)
        string-line (bot-string-line-from-top (?? side "i"))
        ;; how much the vertical leg is buried by the stringer
        ang-vbury (x22-vbury (?? side "i"))
        gusset-p (< ang-vbury min-vbury)
        bolt-y (- top_vbolt))
  (vla-rotate ang (3dpt 0 ang-y) (dtr -90))
  (add-bolt-centerline (list 0 bolt-y) 'h)

  (if gusset-p
      (add-top-triangle-gusset (?? side "i")))
  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1 bolt-y) 0 nil)
        (add-mleader blockObj
                     (list 0 (- bolt-y 1))
                     '((0 0 0)
                       (0 -4 0)
                       (1 -4 0))
                     (strcat (xd-value "x22-1" marks) "\n(LLV)"))
        (if gusset-p
            (add-mleader blockObj
                         (list -2 -4)
                         '((0 0 0)
                           (0 -10 0)
                           (1 -10 0))
                         (xd-value "x22-2" marks)))
        ))
  T)

(defun detail-side-122 (fab-p) (detail-side-x22 fab-p "L5x3x1/4" 1.25))
(defun detail-side-222 (fab-p) (detail-side-x22 fab-p "L7x4x3/8" 0.75))
(setq detail-side-152 detail-side-122
      detail-side-422 detail-side-122
      detail-side-452 detail-side-122)

(defun detail-top-x22 (ang-wid ang-thick bolt-qty)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj (- ang-wid) btwn_stringers '(0 0 0))
  (add-line-btwn (- ang-thick))

  (bolts-top-right bolt-qty)
        
  (add-mleader blockObj
               (list 0 (- (/ btwn_stringers 2.0) 1) 0)
               '((0 0 0)
                 (2 0 0)
                 (2 -5 0)
                 (3 -5 0))
               (strcat (xd-value "x22-1" marks) "\n(LLV)")))

(defun detail-top-122 () (detail-top-x22 3 0.25 2))
(defun detail-top-152 () (detail-top-x22 3 0.25 4))
(defun detail-top-222 () (detail-top-x22 4 0.375 2))
(defun detail-top-252 () (detail-top-x22 4 0.375 4))
(defun detail-top-422 () (detail-top-x22 3 0.25 2))
(defun detail-top-452 () (detail-top-x22 3 0.25 4))

(defun material-x22 (bolt-qty ang-size min-vbury
                     / len hole-dia gusset-qty material-list h1 h2 bolt-y detail)
  (setq len (- btwn_stringers 0.25)
        hole-dia (x2x-hole-dia top_hdw)
        gusset-qty (+ (if (< (x22-vbury "i") min-vbury) 1 0)
                      (if (< (x22-vbury "o") min-vbury) 1 0))
        h1 (- top_hbolt l-string_wid 0.125)
        h2 (- top_hbolt r-string_wid 0.125)
        bolt-y (- top_vbolt (?f tread_thick) (?f pan_thick))
        detail (if (= bolt-qty 2)
                   (list "detail"
                         (list '2h-angle ang-size 'long len h1 h2 bolt-y hole-dia 1))
                 (list "detail"
                       (list '4h-angle ang-size 'long len h1 h2 top_hbolt2 bolt-y hole-dia)))
        material-list (#! '((("id" "x22-1")
                             ("desc" #(strcat "ANGLE "
                                              (strjoin (strsplit (substr ang-size 2) "x") " x ")))
                             ("qty" 1)
                             ("length" #(frac len))
                             #detail))))
  (if (> gusset-qty 0)
      (append! 'material-list
               (#! '((("id" "x22-2")
                      ("desc" "FLAT BAR 1/4 x 6")
                      ("detail" (top-triangle-gusset #pitch))
                      ("qty" #gusset-qty)
                      ("length" #(frac (* 6 (tan (dtr pitch))))))))))
  (#! '(#material-list
        ;; hardware
        (
         (("qty" #bolt-qty)
          ("desc" #(tc-bolt-desc top_hdw))
          ("note" "SL"))
         ))))

(defun material-122 () (material-x22 2 "L5x3x1/4" 1.25))
(defun material-152 () (material-x22 4 "L5x3x1/4" 1.25))
(defun material-222 () (material-x22 2 "L7x4x3/8" 0.75))
(defun material-252 () (material-x22 4 "L7x4x3/8" 0.75))
(defun material-422 () (material-x22 2 "L5x3x1/4" 1.25))
(defun material-452 () (material-x22 4 "L5x3x1/4" 1.25))

;;;;;;;;
;; x23
;;;;;;;;

(defun detail-side-x23 (fab-p ang-size / ang ang-y bolt-y gusset-p)
  (detail-side-x21 fab-p ang-size)
  (if (= quiet_tread "1") (add-bot-ext-qt))

  T)

(defun detail-side-123 (fab-p) (detail-side-x23 fab-p "L5x3x1/4"))
(defun detail-side-223 (fab-p) (detail-side-x23 fab-p "L7x4x3/8"))
(setq detail-side-153 detail-side-123
      detail-side-253 detail-side-223
      detail-side-423 detail-side-123
      detail-side-453 detail-side-123)

(defun detail-top-123 () (detail-top-x21 3 0.25 2))
(defun detail-top-153 () (detail-top-x21 3 0.25 4))
(defun detail-top-223 () (detail-top-x21 4 0.375 2))
(defun detail-top-253 () (detail-top-x21 4 0.375 4))
(defun detail-top-423 () (detail-top-x21 3 0.25 2))
(defun detail-top-453 () (detail-top-x21 3 0.25 4))

(defun material-123 () (material-x21 2 "L5x3x1/4"))
(defun material-153 () (material-x21 4 "L5x3x1/4"))
(defun material-223 () (material-x21 2 "L7x4x3/8"))
(defun material-253 () (material-x21 4 "L7x4x3/8"))
(defun material-423 () (material-x21 2 "L5x3x1/4"))
(defun material-453 () (material-x21 4 "L5x3x1/4"))

;;;;;;;;
;; 124
;;;;;;;;

(defun detail-side-124 (fab-p / ang bolt-y)
  (setq bolt-y (- top_vbolt)
        ang (add-shape blockObj '(0 -3.5 0) "L3x2x1/4"))
  (vla-rotate ang (3dpt 0 -3.5) (dtr -90))
  (add-bolt-centerline (list 0 bolt-y) 'h)
  (top-conc-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1 bolt-y 0) 0 nil)
        (add-mleader blockObj
                     (list 0 (- bolt-y 1) 0)
                     '((0 0 0)
                       (2 0 0)
                       (1 -6 0)
                       (2 -6 0))
                     (strcat (xd-value "124-1" marks) " (LLV)"))))
  T)

(setq detail-side-154 detail-side-124
      detail-side-424 detail-side-124
      detail-side-454 detail-side-124)

(defun detail-top--124 (bolt-qty / )
  (top-conc-haunch-top)

  (set-active-layer "HIDDEN")
  (add-rectangle blockObj -2 btwn_stringers '(0 0 0))
  (add-line-btwn -0.25)

  (bolts-top-right bolt-qty)

  (vla-put-Layer
   (add-mleader blockObj
                (list 0 (/ btwn_stringers 2.0) 0)
                '((0 0 0)
                  (2 0 0)
                  (1 -6 0)
                  (2 -6 0))
                (strcat (xd-value "124-1" marks) "\n(LLV)"))
   "DIM")

  (vla-put-Layer-r
   (add-line-btwn 0)
   "SOLID"))

(defun detail-top-124 () (detail-top--124 2))
(defun detail-top-154 () (detail-top--124 4))
(defun detail-top-424 () (detail-top--124 2))
(defun detail-top-454 () (detail-top--124 4))

(defun material--124 (bolt-qty / len h1 h2 hole-dia bolt-y detail)
  (setq len (- btwn_stringers 0.0625)
        h1 (- top_hbolt r-string_wid 0.03125)
        h2 (- top_hbolt l-string_wid 0.03125)
        hole-dia (x2x-hole-dia top_hdw)
        bolt-y (- top_vbolt 3.5)
        detail (if (= bolt-qty 2)
                   (list "detail" (list '2h-angle "L3x2x1/4" 'long len h1 h2 bolt-y hole-dia 1))
                 (list "detail" (list '4h-angle "L3x2x1/4" 'long len h1 h2 top_hbolt2 bolt-y hole-dia))))
  (list (append (#! '((("id" "124-1")
                       ("desc" "ANGLE 3 x 2 x 1/4")
                       ("qty" 1)
                       ("length" #(frac len))
                       #detail)))
                (top-conc-haunch-material))
        (#! '(
              (("qty" #bolt-qty)
               ("desc" #(tc-bolt-desc top_hdw))
               ("note" "SL"))
              ))))

(defun material-124 () (material--124 2))
(defun material-154 () (material--124 4))
(defun material-424 () (material--124 2))
(defun material-454 () (material--124 4))

;;;;;;;;
;; 224
;;;;;;;;

(defun detail-side-224 (fab-p / ang bolt-y)
  (setq bolt-y (- top_vbolt)
        ang (add-shape blockObj (list 0 (- pan_thick) 0) "L7x4x3/8"))
  (vla-rotate ang (3dpt 0 (- pan_thick)) (dtr -90))
  (add-bolt-centerline (list 0 bolt-y) 'h)
  (top-chkr-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1 bolt-y) 0 nil)
        (add-mleader blockObj
                     (list 0 (1- bolt-y) 0)
                     '((0 0 0)
                       (1 0 0)
                       (0 -6 0)
                       (1 -6 0))
                     (strcat (xd-value "224-1" marks) "\n(LLV)"))))
  T)

(defun detail-top--224 (bolt-qty
                        /)
  (top-chkr-haunch-top)
  
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj -4 btwn_stringers '(0 0 0))
  (add-line-btwn -0.25)
  (bolts-top-right bolt-qty)
  T)

(defun detail-top-224 () (detail-top--224 2))
(defun detail-top-254 () (detail-top--224 4))

(defun material--224 (bolt-qty
                      / len h1 h2 hole-dia bolt-y detail)
  (setq len (- btwn_stringers 0.0625)
        h1 (- top_hbolt r-string_wid 0.03125)
        h2 (- top_hbolt l-string_wid 0.03125)
        hole-dia (x2x-hole-dia top_hdw)
        bolt-y (- top_vbolt pan_thick)
        detail
        (if (= bolt-qty 2)
            (list "detail" (list '2h-angle "L7x4x3/8" 'long len h1 h2 bolt-y hole-dia 1))
          (list "detail" (list '4h-angle "L7x4x3/8" 'long len h1 h2 top_hbolt2 bolt-y hole-dia))))
  (list (append (top-chkr-haunch-material)
                (#! '(
                      (("id" "224-1")
                       ("qty" 1)
                       ("desc" "ANGLE 7 x 4 x 3/8")
                       ("length" #(frac len))
                       #detail)
                      )))
        (#! '(
              (("qty" #bolt-qty)
               ("desc" #(tc-bolt-desc top_hdw))
               ("note" "SL"))
              ))))

(defun material-224 () (material--224 2))
(defun material-254 () (material--224 4))

(setq detail-side-254 detail-side-224)

;;;;;;;;
;; 125
;;;;;;;;

(defun detail-side-125 (fab-p / ang bolt-y)
  (setq bolt-y (- bot_vbolt)
        ang (add-shape blockObj '(0 -3.5 0) "L3x2x1/4"))
  (vla-rotate ang (3dpt 0 -3.5) (dtr -90))
  (vla-mirror-y ang)
  (add-bolt-centerline (list 0 bolt-y) 'h)
  (bot-conc-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (add-mleader blockObj
                     (list 0 (1- bolt-y) 0)
                     '((0 0 0)
                       (-1 0 0)
                       (-1 -4 0)
                       (-2 -4 0))
                     (strcat (xd-value "125-1" marks) "\n(LLV)"))
        (detail-side-add-dim-left (3dpt -1 bolt-y) 0 nil)))
  T)

(setq detail-side-155 detail-side-125
      detail-side-425 detail-side-125
      detail-side-455 detail-side-125)

(defun detail-top--125 (bolt-qty)
  (bot-conc-haunch-top)
  (bolts-top-left bolt-qty)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj 2 btwn_stringers '(0 0 0))
  (add-line-btwn 0.25)
  (set-active-layer "DIM")
  (add-mleader blockObj
               (list 0 (- (/ btwn_stringers 2.0) 2) 0)
               '((0 0 0)
                 (-1 0 0)
                 (-1 -4 0)
                 (-2 -4 0))
               (strcat (xd-value "125-1" marks) "\n(LLV)")))

(defun detail-top-125 () (detail-top--125 2))
(defun detail-top-155 () (detail-top--125 4))
(defun detail-top-425 () (detail-top--125 2))
(defun detail-top-455 () (detail-top--125 4))

(defun material--125 (bolt-qty / len h1 h2 hole-dia bolt-y)
  (setq len (- btwn_stringers 0.0625)
        h1 (- bot_hbolt r-string_wid 0.03125)
        h2 (- bot_hbolt l-string_wid 0.03125)
        hole-dia (x2x-hole-dia bot_hdw)
        bolt-y (- bot_vbolt 3.5)
        detail (if (= bolt-qty 2)
                   (list "detail" (list '2h-angle "L3x2x1/4" 'long len h1 h2 bolt-y hole-dia 1))
                   (list "detail"
                         (list '4h-angle "L3x2x1/4" 'long len h1 h2 bot_hbolt2 bolt-y hole-dia))))
  (list (append (#! '((("id" "125-1")
                       ("desc" "ANGLE 3 x 2 x 1/4")
                       ("qty" 1)
                       ("length" #(frac len))
                       #detail)))
                (bot-conc-haunch-material))
        (#! '(
              (("qty" #bolt-qty)
               ("desc" #(tc-bolt-desc bot_hdw))
               ("note" "SL"))
              ))))

(defun material-125 () (material--125 2))
(defun material-155 () (material--125 4))
(defun material-425 () (material--125 2))
(defun material-455 () (material--125 4))

;;;;;;;;
;; 225
;;;;;;;;

(defun detail-side-225 (fab-p / ang bolt-y)
  (setq bolt-y (- bot_vbolt)
        ang (add-shape blockObj (list 0 (- pan_thick) 0) "L7x4x3/8"))
  (vla-rotate ang (3dpt 0 (- pan_thick)) (dtr -90))
  (vla-mirror-y ang)
  (add-bolt-centerline (list 0 bolt-y) 'h)
  (bot-chkr-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt 0 bolt-y) 0 nil)
        (add-mleader blockObj
                     (list 0 (1- bolt-y) 0)
                     '((0 0 0)
                       (-1 0 0)
                       (0 -6 0)
                       (-1 -6 0))
                     (strcat (xd-value "225-1" marks) "\n(LLV)"))))
  T)

(defun detail-top--225 (bolt-qty)
  (bot-chkr-haunch-top)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj 4 btwn_stringers '(0 0 0))
  (add-line-btwn 0.25)
  (bolts-top-left bolt-qty)
  T)

(defun detail-top-225 () (detail-top--225 2))
(defun detail-top-255 () (detail-top--225 4))

(defun material--225 (bolt-qty / len h1 h2 hole-dia bolt-y detail)
  (setq len (- btwn_stringers 0.0625)
        h1 (- bot_hbolt r-string_wid 0.03125)
        h2 (- bot_hbolt l-string_wid 0.03125)
        hole-dia (x2x-hole-dia bot_hdw)
        bolt-y (- bot_vbolt pan_thick)
        detail
        (if (= bolt-qty 2)
            (list "detail" (list '2h-angle "L7x4x3/8" 'long len h1 h2 bolt-y hole-dia 1))
          (list "detail" (list '4h-angle "L7x4x3/8" 'long len h1 h2 bot_hbolt2 bolt-y hole-dia))))
  (list (append (bot-chkr-haunch-material)
                (#! '(
                      (("id" "225-1")
                       ("desc" "ANGLE 7 x 4 x 3/8")
                       ("qty" 1)
                       ("length" #(frac len))
                       #detail)
                      )))
        (#! '(
              (("qty" #bolt-qty)
               ("desc" #(tc-bolt-desc bot_hdw))
               ("note" "SL"))
              ))))

(defun material-225 () (material--225 2))
(defun material-255 () (material--225 4))

(setq detail-side-255 detail-side-225)

;;;;;;;;
;; x27
;;;;;;;;

(defun detail-side-x27-common (container off-y / bolt-y ang-x obj)
  (setq bolt-y (- bot_vbolt))
  (setq obj (add-shape container '(0 0 0) "L8x6x1/2"))
  (vla-rotate obj (3dpt 0 0) (dtr (- 90)))
  (vla-move obj (3dpt 0 0) (3dpt 0 off-y))
  (vla-mirror-y obj)
  (setq ang-x 7.5)
  (setq obj (add-shape container (list ang-x -0.75 0) "L3x3x1/4"))
  (vla-rotate obj (3dpt ang-x -0.75) pi)
  (vla-move obj (3dpt 0 0) (3dpt 0 off-y))
  (add-polyline container
                 (list 0 off-y 0)
                '((0.0 -10.5 0.0)
                  (0.0 -8.0 0.0)
                  (0.5 -8.0 0.0)
                  (0.5 -1.0 0.0)
                  (1.0 -0.5 0.0)
                  (4.5 -0.5 0.0)
                  (4.5 -0.75 0.0)
                  (5.875 -0.75 0.0))
                T)
  (add-bolt-centerline (list 0 bolt-y) 'h))

(defun detail-side-x27 (fab-p off-y / ang-x obj)
  (setq ang-x (+ bot_cdist (get-bot-kick-dist)))

  (setq obj (add-shape blockObj (list ang-x 0 0) "L3x2x1/4"))
  (vla-rotate obj (3dpt ang-x 0) (dtr 90))
  (vla-move obj (3dpt 0 0) (3dpt 0 off-y))
  (if fab-p
      (progn
        (set-active-layer "DIM")
        (add-mleader blockObj
                     (list (+ 1 bot_cdist (get-bot-kick-dist)) 0 0)
                     '((0 0 0)
                       (0 -6 0)
                       (1 -6 0))
                     (strcat (xd-value "x27-3x2" marks) " (LLV)")))

      (detail-side-x27-common blockObj off-y))

  T)

(defun detail-side-127 (fab-p) (detail-side-x27 fab-p 0))
(defun detail-side-227 (fab-p) (detail-side-x27 fab-p (- pan_thick)))
(defun detail-side-427 (fab-p) (detail-side-x27 fab-p 0))

(defun material-x27 (off-y
                     / len)
  (setq len (- btwn_stringers 0.25))
  (#! '((
         (("id" "x27-8x6")
          ("desc" "ANGLE 8 x 6 x 1/2")
          ("qty" 1)
          ("length" #(frac width))
          ("note" "SL")
          ("detail" (x27-angle-detail #width #off-y)))
         (("id" "x27-3x3")
          ("desc" "ANGLE 3 x 3 x 1/4")
          ("length" 4)
          ("qty" 2))
         (("id" "x27-gusset")
          ("desc" "PLATE 3/8")
          ("qty" 2)
          ("width" "10")
          ("length" "6")
          ("note" "LC")
          ("detail" (x27-gusset-detail)))
         (("id" "x27-3x2")
          ("desc" "ANGLE 3 x 2 x 1/4")
          ("qty" 1)
          ("length" #(frac len)))
         )
        (
         (("qty" 2)
          ("desc" #(tc-bolt-desc bot_hdw))
          ("note" "SL"))
         ))))

(defun material-127 () (material-x27 0))
(defun material-227 () (material-x27 (- pan_thick)))
(defun material-427 () (material-x27 0))
