;; x3x series

;; back, mid, and front are x offsets
;; id is the part id for the callout
(defun x3x-angle-top (back mid front mark / dir)
  (setq dir (unit-dir front))
  (set-active-layer "HIDDEN")
  (add-line-btwn back)
  (add-line-btwn mid)

  (set-active-layer "SOLID")
  (add-polyline blockObj
                (list 0 0 0)
                (list '(0 0 0)
                      (list front 0 0)
                      (list front btwn_stringers 0)
                      (list 0 btwn_stringers 0))
                nil)

  (set-active-layer "DIM")
  (add-mleader blockObj
               (list front (- (/ btwn_stringers 2.0) 2) 0)
               (#! '((0 0 0)
                     (#(* dir 2) 0 0)
                     (#(* dir 1) -6 0)
                     (#(* dir 2) -6 0)))
               mark))

;;;;;;;;
;; x30
;;;;;;;;

(defun detail-side-x30 (fab-p / ang ang-ins)
  (setq ang-ins (list -0.5 (+ (- top_blkout) 0.25))
        ang (add-shape blockObj ang-ins "L4x3x1/4"))
  (vla-Rotate ang (apply '3dpt ang-ins) (dtr -90))
  (mirror&delete ang ang-ins 'y)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 2.375 (- top_blkout)) 0 nil)
        (add-mleader blockObj
                     (list -0.25 (1- (- top_blkout)) 0)
                     '((0 0 0)
                       (2 0 0)
                       (1 -6 0)
                       (2 -6 0))
                     (strcat (xd-value "x30-1" marks) "\n(LLV)"))))
  T)

(setq detail-side-130 detail-side-x30
      detail-side-230 detail-side-x30
      detail-side-430 detail-side-x30)

(defun detail-top-x30 () (x3x-angle-top -0.5 -0.25 2.5 (strcat (xd-value "x30-1" marks) "\n(LLV)")))
(setq detail-top-130 detail-top-x30
      detail-top-230 detail-top-x30
      detail-top-430 detail-top-x30)

(defun material-x30 ()
  (#! '(
        ;; material
        (
         (("id" "x30-1")
          ("qty" 1)
          ("desc" "ANGLE 4 x 3 x 1/4")
          ("length" #(frac (- btwn_stringers 0.25))))
         )
        ;; hardware
        nil)))

(setq material-130 material-x30
      material-230 material-x30
      material-430 material-x30)

;;;;;;;;
;; x31
;;;;;;;;

(defun detail-side-x31 (fab-p / ang ang-ins)
  (setq ang-ins (list 2.5 (- bot_blkout))
        ang (add-shape blockObj ang-ins "L4x3x1/4"))
  (vla-Rotate ang (apply '3dpt ang-ins) pi)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt -1.5 (- bot_blkout)) 0 nil)
        (add-mleader blockObj
                     ang-ins
                     '((0 0 0)
                       (0 -6 0)
                       (1 -6 0))
                     (strcat (xd-value "x31-1" marks) "\n(LLH)"))))
  T)

(setq detail-side-131 detail-side-x31
      detail-side-231 detail-side-x31
      detail-side-431 detail-side-x31)

(defun detail-top-x31 () (x3x-angle-top 2.5 2.25 -1.5 (strcat (xd-value "x31-1" marks) "\n(LLH)")))

(setq detail-top-131 detail-top-x31
      detail-top-231 detail-top-x31
      detail-top-431 detail-top-x31)

(defun material-x31 ()
  (#! '(
        ;; material
        (
         (("id" "x31-1")
          ("qty" 1)
          ("desc" "ANGLE 4 x 3 x 1/4")
          ("length" #(frac (- btwn_stringers 0.25))))
         )
        ;; hardware
        nil)))

(setq material-131 material-x31
      material-231 material-x31
      material-431 material-x31)

;;;;;;;;
;; x32
;;;;;;;;

;; return how much the 3x2's vert leg is buried
(defun x32-vbury (side / )
  (- (+ (line-val (bot-string-line-from-top side) -0.5)
        (?f tread_thick)
        (?f pan_thick))))

(defun detail-side-x32 (fab-p / min-ang-vbury ang ang-ins gusset-p)
  (setq min-ang-vbury (if (begins-with tread_style "1") 1.5 2)
        ang-ins (list -0.5 (+ (- top_blkout) 0.25))
        ang (add-shape blockObj ang-ins "L3x2x1/4")
        gusset-p (< (x32-vbury "i") min-ang-vbury))
  (vla-Rotate ang (apply '3dpt ang-ins) (dtr -90))
  (mirror&delete ang ang-ins 'y)

  (setq ang (add-shape blockObj
                       (list -0.5 (- 0
                                     (?f tread_thick)
                                     (?f pan_thick)
                                     (if (begins-with tread_style "1") (?f stn_thick) 0)))
                       "L2x2x1/4"))
  (mirror&delete ang '(-0.5 0) 'y)

  (if (< (x32-vbury "i") min-ang-vbury)
      (add-top-triangle-gusset "i"))

  (if (= quiet_tread "1") (add-top-ext-qt))

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1.375 (- top_blkout)) 0 nil)
        (add-mleader blockObj
                     (list -0.5 (1- (- top_blkout)) 0)
                     '((0 0 0)
                       (1 0 0)
                       (0 -12 0)
                       (1 -12 0))
                     (strcat (xd-value "x32-2" marks) "\n"
                             (xd-value "x32-1" marks) " (LLV)"
                             (if gusset-p (strcat  "\n" (xd-value "x32-3" marks)) "")))))
  T)

(setq detail-side-132 detail-side-x32
      detail-side-232 detail-side-x32
      detail-side-432 detail-side-x32)

(defun detail-top-x32 ()
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj -2 btwn_stringers (list -0.5 0 0))
  (add-line-btwn -0.75)

  (set-active-layer "DIM")
  (add-mleader blockObj
               (list -2.5 (1+ (/ btwn_stringers 2.0)) 0)
               '((0 0 0)
                 (6 0 0)
                 (5 6 0)
                 (6 6 0))
               (xd-value "x32-2" marks))
  (x3x-angle-top -0.5 -0.25 1.5 (strcat (xd-value "x32-1" marks) " (LLV)")))
                             
(setq detail-top-132 detail-top-x32
      detail-top-232 detail-top-x32
      detail-top-432 detail-top-x32)

(defun material-x32 (/ min-vbury gusset-qty material)
  (setq min-vbury (if (begins-with tread_style "1") 1.5 2)
        gusset-qty (+ (if (< (x32-vbury "i") min-vbury) 1 0)
                      (if (< (x32-vbury "o") min-vbury) 1 0))
        material (#! '(
                       (("id" "x32-1")
                        ("qty" 1)
                        ("desc" "ANGLE 3 x 2 x 1/4")
                        ("length" #(frac (- btwn_stringers 0.25))))
                       (("id" "x32-2")
                        ("qty" 1)
                        ("desc" "ANGLE 2 x 2 x 1/4")
                        ("length" #(frac (- btwn_stringers 0.25))))
                       )))
  (if (> gusset-qty 0)
      (append! 'material
               (#! '(
                     (("id" "x32-3")
                      ("desc" "FLAT BAR 1/4 x 6")
                      ("detail" (top-triangle-gusset #pitch))
                      ("qty" #gusset-qty)
                      ("length" #(frac (* 6 (tan (dtr pitch))))))
                     ))))
  (debug-print '("material: " #material))
  (list material nil))

(setq material-132 material-x32
      material-232 material-x32
      material-432 material-x32)

;;;;;;;;
;; x33
;;;;;;;;

(defun detail-side-x33 (fab-p)
  (setq ang-ins (list 0.5 (+ (- bot_blkout) 0.25))
        ang (add-shape blockObj ang-ins "L3x2x1/4"))
  (vla-Rotate ang (apply '3dpt ang-ins) (dtr -90))

  (add-shape blockObj
             (list 0.5 (- 0
                          (if tread_thick tread_thick 0)
                          (if pan_thick pan_thick 0)))
             "L2x2x1/4")

  (if (= quiet_tread "1") (add-bot-ext-qt))

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt -1.375 (- bot_blkout)) 0 nil)
        (add-mleader blockObj
                     ang-ins
                     '((0 0 0)
                       (0 -12 0)
                       (-1 -12 0))
                     (strcat (xd-value "x33-2" marks) "\n"
                             (xd-value "x33-1" marks) " (LLV)"))))

  T)

(setq detail-side-133 detail-side-x33
      detail-side-233 detail-side-x33
      detail-side-433 detail-side-x33)

(defun detail-top-x33 ()
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj 2 btwn_stringers (list 0.5 0 0))
  (add-line-btwn 0.75)
  (set-active-layer "DIM")
  (add-mleader blockObj
               (list 0.5 (+ (/ btwn_stringers 2.0) 10) 0)
               '((0 0 0)
                 (-4 0 0)
                 (-3 6 0)
                 (-4 6 0))
               (xd-value "x33-2" marks))
  (x3x-angle-top 0.5 0.25 -1.5 (strcat (xd-value "x33-1" marks) " (LLV)")))

(setq detail-top-133 detail-top-x33
      detail-top-233 detail-top-x33
      detail-top-433 detail-top-x33)

(defun material-x33 ()
  (#! '(
        ;; material
        (
         (("id" "x33-1")
          ("qty" 1)
          ("desc" "ANGLE 3 x 2 x 1/4")
          ("length" #(frac (- btwn_stringers 0.25))))
         (("id" "x33-2")
          ("qty" 1)
          ("desc" "ANGLE 2 x 2 x 1/4")
          ("length" #(frac (- btwn_stringers 0.25))))
         )
        ;; hardware
        nil)))

(setq material-133 material-x33
      material-233 material-x33
      material-433 material-x33)

;;;;;;;;
;; 134
;;;;;;;;

(defun detail-side-134 (fab-p / ang ang-ins)
  (setq ang-ins (list -0.5 (+ (- top_blkout) 0.25))
        ang (add-shape blockObj ang-ins (if (< top_blkout 1.5) "L4x3x1/4" "L3x2x1/4")))
  (vla-Rotate ang (apply '3dpt ang-ins) (dtr -90))
  (mirror&delete ang ang-ins 'y)

  (setq ang (add-shape blockObj
                       (list -0.5 -3.5)
                       "L2x2x1/4"))
  (mirror&delete ang '(-0.5 0) 'y)

  (top-conc-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1.375 (- top_blkout)) 0 nil)
        (add-mleader blockObj
                     (list -0.5 -3.5 0)
                     '((0 0 0)
                       (2 0 0)
                       (1 -12 0)
                       (2 -12 0))
                     (strcat (xd-value "134-1" marks) "\n"
                             (xd-value "134-2" marks) " (LLV)"))))

  T)

(setq detail-side-434 detail-side-134)

(defun detail-top-134 ()
  (top-conc-haunch-top)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj -2 btwn_stringers (list -0.5 0 0))
  (add-line-btwn -0.75)
  (set-active-layer "DIM")
  (add-mleader blockObj
               (list -0.5 (+ 2 (/ btwn_stringers 2.0)) 0)
               '((0 0 0)
                 (5 0 0)
                 (4 6 0)
                 (5 6 0))
               (xd-value "134-1" marks))
  (x3x-angle-top -0.5 -0.25 1.5 (strcat (xd-value "134-2" marks) "\n(LLV)")))

(setq detail-top-434 detail-top-134)

(defun material-134 ()
  (list (append (top-conc-haunch-material)
                (#! '((("id" "134-1")
                       ("desc" "ANGLE 2 x 2 x 1/4")
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.25))))
                      (("id" "134-2")
                       ("desc" #(if (< top_blkout 1.5) "ANGLE 4 x 3 x 1/4" "ANGLE 3 x 2 x 1/4"))
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.0625)))))))
        nil))

(setq material-434 material-134)

;;;;;;;;
;; 234
;;;;;;;;

(defun detail-side-234 (fab-p / ang ang-ins)
  (setq ang-ins (list -0.5 (+ (- top_blkout) 0.25))
        ang (add-shape blockObj ang-ins "L3x2x1/4"))
  (vla-Rotate ang (apply '3dpt ang-ins) (dtr -90))
  (mirror&delete ang ang-ins 'y)

  (setq ang (add-shape blockObj
                       (list -0.5 (- pan_thick))
                       "L2x2x1/4"))
  (mirror&delete ang '(-0.5 0) 'y)

  (top-chkr-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-right (3dpt 1.375 (- top_blkout)) 0 nil)
        (add-mleader blockObj
                     ang-ins
                     '((0 0 0)
                       (6 -18 0)
                       (5 -18 0))
                     (strcat (xd-value "234-1" marks) "\n"
                             (xd-value "234-2" marks) " (LLV)"))))
  T)

(defun detail-top-234 ()
  (top-chkr-haunch-top)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj -2 btwn_stringers (list -0.5 0 0))
  (add-line-btwn -0.75)
  (set-active-layer "DIM")
  (add-mleader blockObj
               (list -0.5 (+ 2 (/ btwn_stringers 2.0)) 0)
               '((0 0 0)
                 (5 0 0)
                 (4 6 0)
                 (5 6 0))
               (xd-value "234-1" marks))
  (x3x-angle-top -0.5 -0.25 1.5 (strcat (xd-value "234-2" marks) "\n(LLV)")))

(defun material-234 ()
  (list (append (top-chkr-haunch-material)
                (#! '((("id" "234-1")
                       ("desc" "ANGLE 2 x 2 x 1/4")
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.25))))
                      (("id" "234-2")
                       ("desc" #(if (< top_blkout 1.5) "ANGLE 4 x 3 x 1/4" "ANGLE 3 x 2 x 1/4"))
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.0625)))))))
        nil))

;;;;;;;;
;; 135
;;;;;;;;

(defun detail-side-135 (fab-p / ang ang-ins)
  (setq ang-ins (list 0.5 (+ (- bot_blkout) 0.25))
        ang (add-shape blockObj ang-ins (if (< bot_blkout 1.5) "L4x3x1/4" "L3x2x1/4")))
  (vla-Rotate ang (apply '3dpt ang-ins) (dtr -90))

  (add-shape blockObj (list 0.5 -3.5) "L2x2x1/4")

  (bot-conc-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt -1.375 (- bot_blkout)) 0 nil)
        (add-mleader blockObj
                     ang-ins
                     '((0 0 0)
                       (0 -12 0)
                       (-1 -12 0))
                     (strcat (xd-value "135-1" marks) "\n"
                             (xd-value "135-2" marks) " (LLV)"))))
  T)

(setq detail-side-435 detail-side-135)

(defun detail-top-135 ()
  (bot-conc-haunch-top)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj 2 btwn_stringers (list 0.5 0 0))
  (add-line-btwn 0.75)

  (set-active-layer "DIM")
  (add-mleader blockObj
               (list 0.5 (+ (/ btwn_stringers 2.0) 10) 0)
               '((0 0 0)
                 (-4 0 0)
                 (-3 6 0)
                 (-4 6 0))
               (xd-value "135-1" marks))
  (x3x-angle-top 0.5 0.25 -1.5 (strcat (xd-value "135-2" marks) " (LLV)")))

(setq detail-top-435 detail-top-135)

(defun material-135 ()
  (list (append (bot-conc-haunch-material)
                (#! '((("id" "135-1")
                       ("desc" "ANGLE 2 x 2 x 1/4")
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.25))))
                      (("id" "135-2")
                       ("desc" #(if (< bot_blkout 1.5) "ANGLE 4 x 3 x 1/4" "ANGLE 3 x 2 x 1/4"))
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.0625)))))))
        nil))

(setq material-435 material-135)

;;;;;;;;
;; 235
;;;;;;;;

(defun detail-side-235 (fab-p / ang ang-ins)
  (setq ang-ins (list 0.5 (+ (- bot_blkout) 0.25))
        ang (add-shape blockObj ang-ins "L3x2x1/4"))
  (vla-Rotate ang (apply '3dpt ang-ins) (dtr -90))

  (add-shape blockObj (list 0.5 (- 0 pan_thick)) "L2x2x1/4")

  (bot-chkr-haunch-side fab-p)

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (detail-side-add-dim-left (3dpt -1.375 (- bot_blkout)) 0 nil)
        (add-mleader blockObj
                     ang-ins
                     '((0 0 0)
                       (0 -12 0)
                       (-1 -12 0))
                     (strcat (xd-value "235-1" marks) "\n"
                             (xd-value "235-2" marks) " (LLV)"))))
  T)


(defun detail-top-235 ()
  (bot-chkr-haunch-top)
  (set-active-layer "HIDDEN")
  (add-rectangle blockObj 2 btwn_stringers (list 0.5 0 0))
  (add-line-btwn 0.75)

  (set-active-layer "DIM")
  (add-mleader blockObj
               (list 0.5 (+ (/ btwn_stringers 2.0) 10) 0)
               '((0 0 0)
                 (-4 0 0)
                 (-3 6 0)
                 (-4 6 0))
               (xd-value "235-1" marks))
  (x3x-angle-top 0.5 0.25 -1.5 (strcat (xd-value "235-2" marks) " (LLV)")))

(defun material-235 ()
  (list (append (bot-chkr-haunch-material)
                (#! '((("id" "235-1")
                       ("desc" "ANGLE 2 x 2 x 1/4")
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.25))))
                      (("id" "235-2")
                       ("desc" #(if (< bot_blkout 1.5) "ANGLE 4 x 3 x 1/4" "ANGLE 3 x 2 x 1/4"))
                       ("qty" 1)
                       ("length" #(frac (- btwn_stringers 0.0625)))))))
        nil))

;;;;;;;;
;; x37
;;;;;;;;

(defun detail-side-x37 (fab-p / ang-x)
  (setq ang-x (+ bot_cdist (get-bot-kick-dist)))
  (vla-mirror-x
   (add-shape blockObj (list ang-x 0 0) "L3x3x1/4"))

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (add-mleader blockObj
                     (list ang-x 0 0)
                     '((0 0 0)
                       (0 -6 0)
                       (1 -6 0))
                     (xd-value "starter-angle" marks)))
      (progn
        (add-rectangle blockObj 2 4 (list (+ ang-x 6) 0 0))
        (add-polyline blockObj (list (+ ang-x 6.25) 0 0) '((0 0 0) (0 4 0)) nil)))

  T)

(setq detail-side-137 detail-side-x37
      detail-side-237 detail-side-x37
      detail-side-437 detail-side-x37)

(defun detail-top-x37 (/ kickback)
  (setq kickback (cond ((begins-with tread_style "1") (+ 0.75 pan_thick))
                       ((begins-with tread_style "2") (+ 0.5 (* 2 pan_thick)))
                       ((begins-with tread_style "3") (+ 0.625 pan_thick))
                       ((begins-with tread_style "4") 1)))
  (top-starter-angle-btwn kickback nil))

(setq detail-top-137 detail-top-x37
      detail-top-237 detail-top-x37
      detail-top-437 detail-top-x37)

(defun material-x37 ()
  (#! '(
        ;; material
        ((("id" "starter-angle")
          ("desc" "ANGLE 3 x 3 x 1/4")
          ("qty" 1)
          ("length" #(frac (- btwn_stringers 0.25))))
         (("id" "x37-2")
          ("desc" "ANGLE 2 x 2 x 1/4")
          ("qty" 2)
          ("length" 4)
          ("note" "SL"))
         )
        ;; hardware
        nil)))

(setq material-137 material-x37
      material-237 material-x37
      material-437 material-x37)
