;; Starter detail definitions

;; side detail for x10-x13
(defun side-starter-angle-btwn (fab-p kick angle-size hole-p / x angle-l1 angle-l2)
  (setq x (+ bot_cdist kick (?n (if pan_thick (if (begins-with tread_style "2")
                                              (* 2 pan_thick)
                                              pan_thick))))
        angle-l1 (shape-def angle-size T)
        angle-l2 (cadr (assoc "leg2" angle-l1))
        angle-l1 (cadr (assoc "leg1" angle-l1)))

  (vla-mirror-x (add-shape blockObj (list x 0) angle-size))
  (if hole-p (add-bolt-centerline (list (+ x (/ angle-l1 2.0)) 0) 'v))

  (if fab-p
      (progn
        (set-active-layer "DIM")
        (setq x (+ x 0.75))
        (add-mleader blockObj
                     (list x 0 0)
                     (list '(0 0 0)
                           (list 0 (- dimspace) 0)
                           (list 2 (- dimspace) 0))
                     (strcat (cadr (assoc "starter-angle" marks))
                             (if (/= angle-l1 angle-l2)
                                 " (LLH)"
                             "")))
        ))
  T)

;; top detail for x10-x13
(defun top-starter-angle-btwn (kick hole-dia / x bs)
  (setq x (+ bot_cdist kick (if pan_thick pan_thick 0))
        bs btwn_stringers
        l-xtra (- l-string_xtra_bex)
        r-xtra (- r-string_xtra_bex))

  (set-active-layer "HIDDEN")
  ;; angle
  (add-rectangle blockObj 4 bs (list x 0 0))
  (add-polyline blockObj
                (list (+ 0.25 x) 0 0)
                (list '(0 0 0)
                      (list 0 bs 0))
                nil)
  ;; callouts
  (set-active-layer "DIM")
  (add-mleader blockObj
               (list x 0 0)
               (list '(0 0 0)
                     (list 0 (- dimspace) 0)
                     (list 2 (- dimspace) 0))
               (xd-value "starter-angle" marks))
  (if hole-dia
      (progn
        ;; holes
        (set-active-layer "HIDDEN")
        (vla-AddCircle blockObj (3dpt (+ x 2) 4) (/ hole-dia 2.0))
        (vla-AddCircle blockObj (3dpt (+ x 2) (- bs 4)) (/ hole-dia 2.0))
        ;; centerlines
        (set-active-layer "CENTER")
        (add-polyline blockObj
                      (list (+ x 2) 4 0)
                      '((-4 0 0)
                        (4 0 0))
                      nil)
        (add-polyline blockObj
                      (list (+ x 2) (- bs 4) 0)
                      '((-4 0 0)
                        (4 0 0))
                      nil)
        (set-active-layer "DIM")
        ;; dimensions
        (setq x (- x 2)
              left-dim-line (- (min l-xtra r-xtra) (* 3 dimspace)))
        (vla-AddDimRotated blockObj
                           (3dpt x 4)
                           (3dpt r-xtra 0)
                           (3dpt left-dim-line 0)
                           (dtr 90))
        (vla-AddDimRotated blockObj
                           (3dpt x 4)
                           (3dpt x (- bs 4))
                           (3dpt left-dim-line 0)
                           (dtr 90))
        (vla-AddDimRotated blockObj
                           (3dpt x (- bs 4))
                           (3dpt l-xtra bs)
                           (3dpt left-dim-line 0)
                           (dtr 90))))
  T)

(defun starter-hole-dia()
  (+ (if galv-p 0.125 0.0625)
     (if (begins-with bot_hdw "5/8") 0.625 0.75)))

(defun material-x10 ( / len)
  (setq len (- btwn_stringers 0.25))
  (#! '(
        ;; material
        (
         (("id" "starter-angle")
          ("desc" "ANGLE 4 x 3 x 1/4")
          ("qty" 1)
          ("length" #(frac len))
          ("detail" (2h-angle "L4x3x1/4" long #len 3.875 3.875 2 3 1)))
         (("id" "starter-washer-plate")
          ("desc" "FLAT BAR 1/4 x 1 1/2")
          ("qty" 2)
          ("length" "5")
          ("note" "SL")
          ("detail" (1h-flatbar 5 1.5 2.5 0.75 #(starter-hole-dia))))
         )
        ;; hardware
        (
         (("desc" #(strcat "HILTI KB-TZ2 WEDGE ANCHOR "
                           (if galv-p "SS304 " "")
                           bot_hdw))
          ("qty" 2)
          ("note" "SL"))
        ))))

(defun detail-side-110 (fab-p) (side-starter-angle-btwn fab-p 0.75 "L4x3x1/4" T))
(defun detail-top-110 () (top-starter-angle-btwn 0.75 3))
(defun material-110 () (material-x10))

(defun detail-side-210 (fab-p) (side-starter-angle-btwn fab-p 0.5 "L4x3x1/4" T))
(defun detail-top-210 () (top-starter-angle-btwn 0.5 3))
(defun material-210 () (material-x10))

(defun detail-side-410 (fab-p) (side-starter-angle-btwn fab-p 0.625 "L4x3x1/4" T))
(defun detail-top-410 () (top-starter-angle-btwn 0.625 3))
(defun material-410 () (material-x10))

(defun detail-side-610 (fab-p) (side-starter-angle-btwn fab-p 1 "L4x3x1/4" T))
(defun detail-top-610 () (top-starter-angle-btwn 1 3))
(defun material-610 () (material-x10))

(defun material-x11 ( / len hole-dia)
  (setq hole-dia (+ (if galv-p 0.125 0.0625) 0.625)
        len (- btwn_stringers 0.25))
  (#! '(
        ;; material
        (
         (("id" "starter-angle")
          ("desc" "ANGLE 4 x 3 x 1/4")
          ("qty" 1)
          ("length" #(frac len))
          ("detail" (2h-angle "L4x3x1/4" long #len 3.875 3.875 2 3 1)))
         (("id" "starter-washer-plate")
          ("desc" "FLAT BAR 1/4 x 1 1/2")
          ("qty" 2)
          ("length" "5")
          ("note" "SL")
          ("detail" (1h-flatbar 5 1.5 2.5 0.75 #hole-dia)))
         )
        ;; hardware
        (
         (("desc" "HEX NUT 5/8 DIA.")
          ("qty" 2)
          ("note" "SL"))
         (("desc" "ALL THREAD 5/8 DIA. x 2")
          ("qty" 2)
          ("note" "SL"))
         ))))

(defun detail-side-111 (fab-p) (side-starter-angle-btwn fab-p 0.75 "L4x3x1/4" T))
(defun detail-top-111 () (top-starter-angle-btwn 0.75 3))
(defun material-111 () (material-x11))

(defun detail-side-211 (fab-p) (side-starter-angle-btwn fab-p 0.5 "L4x3x1/4" T))
(defun detail-top-211 () (top-starter-angle-btwn 0.5 3))
(defun material-211 () (material-x11))

(defun detail-side-411 (fab-p) (side-starter-angle-btwn fab-p 0.625 "L4x3x1/4" T))
(defun detail-top-411 () (top-starter-angle-btwn 0.625 3))
(defun material-411 () (material-x11))

(defun detail-side-611 (fab-p) (side-starter-angle-btwn fab-p 1 "L4x3x1/4" T))
(defun detail-top-611 () (top-starter-angle-btwn 1 3))
(defun material-611 () (material-x11))

(defun material-x12 ( / len)
  (setq len (- btwn_stringers 0.25))
  (#! '(
        ;; material
        (
         (("id" "starter-angle")
          ("desc" "ANGLE 3 x 3 x 1/4")
          ("qty" 1)
          ("length" #(frac len))
          ("detail" (2h-angle "L3x3x1/4" long #len 3.875 3.875 1.5 #(starter-hole-dia) 1)))
         )
        ;; hardware
        (
         (("desc" #(strcat "HILTI KB-TZ2 WEDGE ANCHOR "
                           (if galv-p "SS304 " "")
                           bot_hdw))
          ("qty" 2)
          ("note" "SL"))
         ))))

(defun detail-side-112 (fab-p) (side-starter-angle-btwn fab-p 0.75 "L3x3x1/4" T))
(defun detail-top-112 () (top-starter-angle-btwn 0.75 (starter-hole-dia)))
(defun material-112 () (material-x12))

(defun detail-side-212 (fab-p) (side-starter-angle-btwn fab-p 0.5 "L3x3x1/4" T))
(defun detail-top-212 () (top-starter-angle-btwn 0.5 (starter-hole-dia)))
(defun material-212 () (material-x12))

(defun detail-side-412 (fab-p) (side-starter-angle-btwn fab-p 0.625 "L3x3x1/4" T))
(defun detail-top-412 () (top-starter-angle-btwn 0.625 (starter-hole-dia)))
(defun material-412 () (material-x12))

(defun detail-side-612 (fab-p) (side-starter-angle-btwn fab-p 1 "L3x3x1/4" T))
(defun detail-top-612 () (top-starter-angle-btwn 1 (starter-hole-dia)))
(defun material-612 () (material-x12))

(defun material-x13 ( / len)
  (setq len (- btwn_stringers 0.25))
  (#! '(
        ;; material
        (
         (("id" "starter-angle")
          ("desc" "ANGLE 3 x 3 x 1/4")
          ("qty" 1)
          ("length" #(frac len)))
         ))))

(defun detail-side-113 (fab-p) (side-starter-angle-btwn fab-p 0.75 "L3x3x1/4" nil))
(defun detail-top-113 () (top-starter-angle-btwn 0.75 nil))
(defun material-113 () (material-x13))

(defun detail-side-213 (fab-p) (side-starter-angle-btwn fab-p 0.5 "L3x3x1/4" nil))
(defun detail-top-213 () (top-starter-angle-btwn 0.5 nil))
(defun material-213 () (material-x13))

(defun detail-side-413 (fab-p) (side-starter-angle-btwn fab-p 0.625 "L3x3x1/4" nil))
(defun detail-top-413 () (top-starter-angle-btwn 0.625 nil))
(defun material-413 () (material-x13))

(defun detail-side-613 (fab-p) (side-starter-angle-btwn fab-p 1 "L3x3x1/4" nil))
(defun detail-top-613 () (top-starter-angle-btwn 1 nil))
(defun material-613 () (material-x13))

(defun detail-side-x16 (fab-p kick / x bolt-spc bolt-edge len)
  (setq x (+ bot_cdist kick (if pan_thick (if (begins-with tread_style "2")
                                              (* 2 pan_thick)
                                              pan_thick)))
        bolt-edge bot_hbolt
        bolt-spc bot_hbolt2
        len (+ (* 2 bolt-edge) bolt-spc))

  (add-rectangle blockObj len 4 (list x 0))
  (add-polyline blockObj
                (list x 0.25)
                (list '(0 0 0)
                      (list len 0 0))
                nil)
  (foreach offset (list (+ x bolt-edge)
                        (+ x bolt-edge bolt-spc))
           (add-bolt-centerline (list offset 0) 'v))
  (if fab-p
      (progn
        (vla-Rotate (add-shape blockObj (list x 0 0) "L2x2x1/4") (3dpt x 0) (dtr 90))
        (set-active-layer "DIM")
        (add-mleader blockObj
                     (list (+ x len) 2 0)
                     (list '(0 0 0)
                           '(2 0 0)
                           (list 6 (- dimspace) 0)
                           (list 6 (- dimspace) 0))
                     (strcat (cadr (assoc "x16-angle" marks))
                             " (LLV)"))
        (add-mleader blockObj
                     (list x 0 0)
                     '((0 0 0)
                       (0 -4 0))
                     (xd-value "x16-stiffener" marks))))
  T)

(defun detail-top-x16 (kick / x bolt-edge bolt-spc len hole-rad mirror)
  (setq x (+ bot_cdist kick (if pan_thick pan_thick 0))
        bolt-edge bot_hbolt
        bolt-spc bot_hbolt2
        len (+ (* 2 bolt-edge) bolt-spc)
        hole-rad (/ (starter-hole-dia) 2.0))

  (defun mirror (obj)
    (vla-Mirror obj
                (3dpt 0 (/ btwn_stringers 2.0))
                (3dpt 1 (/ btwn_stringers 2.0))))

  (set-active-layer "HIDDEN")
  (mirror (add-rectangle blockObj len 3 (list x 0 0)))
  (mirror (add-polyline blockObj
                        (list x 0.25 0)
                        (list '(0 0 0)
                              (list len 0 0))
                        nil))
  (mirror (vla-AddCircle blockObj (3dpt (+ x bolt-edge) 1.5) hole-rad))
  (mirror (vla-AddCircle blockObj (3dpt (+ x bolt-edge bolt-spc) 1.5) hole-rad))
  (add-rectangle blockObj 2 (- btwn_stringers 6) (list x 3 0))
  (add-polyline blockObj (list (+ x 0.25) 3 0) (list '(0 0 0) (list 0 (- btwn_stringers 6) 0)) nil)
  (set-active-layer "DIM")
  (mirror (add-mleader blockObj
                       (list (+ x len) 1.5 0)
                       (list '(0 0 0)
                             '(2 0 0)
                             (list 2 (- dimspace) 0)
                             (list 2 (- dimspace) 0))
                       (strcat (cadr (assoc "x16-angle" marks))
                               " (LLV)")))
  (add-mleader blockObj
               (list x dimspace 0)
               '((0 0 0)
                 (-6 0 0)
                 (-6 6 0)
                 (-8 6 0))
               (xd-value "x16-stiffener" marks)))

(defun material-x16 ( / len)
  (setq len (+ bot_hbolt2 (* 2 bot_hbolt)))
  (#! '(
        ;; material
        (
         (("id" "x16-angle")
          ("desc" "ANGLE 4 x 3 x 1/4")
          ("qty" 2)
          ("length" #(frac len))
          ("detail" (2h-angle "L4x3x1/4" short #len #bot_hbolt #bot_hbolt 1.5 #(starter-hole-dia) 2)))
         (("id" "x16-stiffener")
          ("desc" "ANGLE 2 x 2 x 1/4")
          ("qty" 1)
          ("length" #(frac (- btwn_stringers 6.125))))
         )
        (
         ;; hardware
         (("desc" #(strcat "HILTI KB-TZ2 WEDGE ANCHOR "
                           (if galv-p "SS304 " "")
                           bot_hdw))
          ("qty" 4)
          ("note" "SL"))
         ))))

(defun detail-side-116 (fab-p) (detail-side-x16 fab-p 0.75))
(defun detail-top-116 () (detail-top-x16 0.75))
(defun material-116 () (material-x16))

(defun detail-side-216 (fab-p) (detail-side-x16 fab-p 0.5))
(defun detail-top-216 () (detail-top-x16 0.5))
(defun material-216 () (material-x16))

(defun detail-side-416 (fab-p) (detail-side-x16 fab-p 0.625))
(defun detail-top-416 () (detail-top-x16 0.625))
(defun material-416 () (material-x16))

(defun detail-side-616 (fab-p) (detail-side-x16 fab-p 1))
(defun detail-top-616 () (detail-top-x16 1))
(defun material-616 () (material-x16))
