;; define all functions and variables necessary for dialog-init
;; source should be a stair-data variable from an existing stair or nil,
;; which indicates it is to read from defaults-file
(defun stair-dialog-init (source /
                          ;; NOTE: vars listed here do not appear in this
                          ;; order in the function definition. Here they
                          ;; are grouped by type, for clarity. In the
                          ;; definition they are grouped by dialog
                          ;; functionality.

                          ;; variables defined here for dialog-init
                          defaults-file
                          popup-keys
                          dist-keys
                          int-keys
                          toggle-keys
                          plain-string-keys
                          default-actions
                          dialog-ties
                          sub-dialogs
                          tread_opts-popup-keys
                          tread_opts-dist-keys
                          tread_opts-plain-string-keys
                          tread_opts-toggle-keys
                          conn_opts-popup-keys
                          conn_opts-dist-keys
                          rail_opts-popup-keys
                          rail_opts-toggle-keys
                          rail_opts-dist-keys
                          ;; variables included here for scope,
                          ;; but not defined by stair-dialog-init directly
                          extra-keys
                          tile-actions
                          tread_opts-val
                          conn_opts-val
                          rail_opts-val
                          ;; tile actions
                          height_f2f-action
                          risers_level-action
                          tread_style-action
                          conn-action
                          ;; init functions for sub-dialogs
                          tread_opts-sub-init
                          conn_opts-sub-init
                          rail_opts-sub-init
                          ;; hooks
                          dialog-after-load
                          dialog-before-done
                          ;; stair definition variables
                          top-details
                          top-details-strings
                          bot-details
                          bot-details-strings
                          4x-angles
                          ;; local util functions
                          conn-opts-exp-keys
                          conn-opts-exp-keys-tb
                          conn-opts-exp-data
                          get-conn-def
                          starter-detail
                          ext-detail
                          ;; misc
                          tb
                          data
                          ;; track connections going in for extension defaults
                          initial-top-conn
                          initial-bot-conn)

  (stair-dialog-load-common-defs)

  ;; hook called at the end of dialog-load-defaults
  (defun dialog-after-load ()
    (mapcar '(lambda(tb / conn-key ext-key ext)
               (setq conn-key (strcat tb "_conn")
                     ext-key (strcat tb "_ext")
                     ext (get_tile ext-key))
               (set (read (strcat "initial-" tb "-conn"))
                    (dialog-tile-val conn-key))
               (conn-extension conn-key (dialog-tile-val conn-key))
               (set_tile ext-key ext))
            '("top" "bot")))

  ;; hook called just before done_dialog
  ;; increments level and sequence
  (defun dialog-before-done ( / toggle-z inc-seq inc-lvl)

    (defun toggle-z ()
      (dialog-popup-set "z_pos"
                        (if (= (cadr (assoc "z_pos" data)) "Near")
                            "Far"
                          "Near")))

    (defun inc-seq ()
      (set_tile "sequence" "B")
      (set_tile "level" (cadr (assoc "level" data)))
      (toggle-z))

    (defun inc-lvl ()
      (set_tile "sequence" "A")
      (toggle-z)
      (set_tile "level" (next-lvl)))

    (defun next-lvl ( / lvl prefix tmp)
      (setq lvl (uppercase (cadr (assoc "level" data))))

      (if (wcmatch lvl "@#*")
          (setq prefix (substr lvl 1 1)
                lvl (substr lvl 2))
        (setq prefix ""))

      ;; only attempt to increment numeric levels
      (if (setq tmp (safe-atoi lvl))
          ;; below grade prefixes
          (if (member prefix '("P" "B"))
              ;; switch to above grade
              (if (= tmp 1)
                  "1"
                (strcat prefix (itoa (1- tmp))))
            (strcat prefix (itoa (1+ tmp))))
        lvl))
    (setq extra-keys nil)
    (dialog-load-defaults nil)
    (set_tile "comment" "")
    (if (= (cadr (assoc "sequence" data)) "A")
        (inc-seq)
      (inc-lvl))
    (foreach key '("height_f2f" "risers_level" "rise_qty" "rise_hgt")
             (set_tile key (to-string (cadr (assoc key data)))))
    (dialog-save-defaults defaults-file))

  ;; keys which are tied-via-toggle
  (setq dialog-ties '(((("l-string_mat" . "r-string_mat")
                        ("l-nosing_top" . "r-nosing_top")
                        ("l-nosing_mid" . "r-nosing_mid")
                        ("l-nosing_bot" . "r-nosing_bot")
                        ("l-string_xtra_tex" . "r-string_xtra_tex")
                        ("l-string_xtra_bex" . "r-string_xtra_bex"))
                       "separate_stringers")))

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; this line actually initializes the dialog and gets the data
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  (setq defaults-file (dialog-update-defaults-file "stair")
        data (dialog-init "stair/stair.dcl" "stair" source))

  ;; post-process and return data gathered from dialog
  (if (= data nil)
      nil
    (stair-add-derived-keys data)))

;; get a delta for stair-multi-edit using the special dialog
(defun stair-multi-dialog-init ( / dcl-id
                                   ;; variables defined here for dialog-init
                                   defaults-file
                                   popup-keys
                                   popup-keys-strings
                                   dist-keys
                                   int-keys
                                   toggle-keys
                                   plain-string-keys
                                   edit-box-keys
                                   string-keys
                                   all-keys
                                   default-actions
                                   dialog-ties
                                   sub-dialogs
                                   tread_opts-popup-keys
                                   tread_opts-dist-keys
                                   tread_opts-toggle-keys
                                   conn_opts-popup-keys
                                   conn_opts-dist-keys
                                   rail_opts-popup-keys
                                   rail_opts-toggle-keys
                                   ;; variables included here for scope,
                                   ;; but not defined here directly
                                   extra-keys
                                   tile-actions
                                   tread_opts-val
                                   conn_opts-val
                                   ;; tile actions
                                   height_f2f-action
                                   risers_level-action
                                   tread_style-action
                                   conn-action
                                   ;; init functions for sub-dialogs
                                   tread_opts-sub-init
                                   conn_opts-sub-init
                                   rail_opts-sub-init
                                   ;; hooks
                                   dialog-after-load
                                   dialog-before-save
                                   ;; local variables
                                   cur-stn-styles
                                   cur-tread-mats
                                   ;; stair definition variables
                                   top-details
                                   top-details-strings
                                   bot-details
                                   bot-details-strings
                                   4x-angles
                                   ;; local util functions
                                   conn-opts-exp-keys
                                   conn-opts-exp-keys-tb
                                   conn-opts-exp-data
                                   get-conn-def
                                   starter-detail
                                   ext-detail
                                   ;; misc
                                   tb
                                   tmp
                                   enkey
                                   data
                                   multi
                                   )
  (stair-dialog-load-common-defs)
  (setq multi T)

  (defun dialog-before-save ( / test)
    (setq test '(= (get_tile (strcat "enable_" x)) "0"))
    (mapcar 'remove!
            '(popup-keys-strings
              string-keys)
            (list test test))
    (remove! 'popup-keys '(not (member (car x) popup-keys-strings))))

  (defun dialog-after-load ()
    ;; basic dialog-toggle-enable on all enable_ tiles
    (foreach key (vl-remove-if '(lambda(x) (member x '("switch_z_pos"
                                                       "mod_risers")))
                               all-keys)
             (mode_tile key 1)
             (if (not (member key '("f2f_height"
                                    "risers_level"
                                    "rise_qty"
                                    "rise_hgt")))
                 (dialog-add-action (strcat "enable_" key)
                                    (strcat "(dialog-toggle-enable $value \"" key "\")"))))
    
    (dialog-add-action "enable_tread_style" '(dialog-toggle-enable $value "tread_opts"))
    (dialog-add-action "enable_tread_style" '(tread_style-action (dialog-tile-val "tread_style")))
    (dialog-add-action "enable_tread_style" '(if (= $value "0") (setq tread_opts-val nil)))
    (dialog-add-action "enable_rail_style" '(dialog-toggle-enable $value "rail_opts"))
    (foreach tb '("top" "bot")
       (setq enkey (strcat "enable_" tb "_conn"))
       (foreach key '("_conn_opts" "_ext")
          (setq tmp (strcat tb key))
          (dialog-add-action enkey '(dialog-toggle-enable $value #tmp)))
       (setq tmp (strcat tb "_conn"))
       (dialog-add-action enkey '(if (= $value "1") (conn-action #tmp (dialog-tile-val #tmp))))
       (setq tmp (strcat tb "_ext"))
       (dialog-add-action enkey '(if (= $value "0")
                                     (progn
                                       (setq conn_opts-val nil)
                                       (set_tile #tmp "")))))
    (mode_tile "flights_level" 1)
    (dialog-add-action "enable_level" '(dialog-toggle-enable $value "flights_level"))
    (foreach key '("height_f2f"
                   "risers_level"
                   "rise_qty")
       (dialog-add-action "mod_risers" '(dialog-toggle-enable $value #key)))
    );;dialog-after-load

  ;; substitute keys
  (setq popup-keys (vl-remove-if '(lambda (x) (= (car x) "z_pos")) popup-keys)
        popup-keys-strings (mapcar 'car popup-keys)
        plain-string-keys (vl-remove-if '(lambda (x) (= x "sequence")) plain-string-keys)
        toggle-keys (append toggle-keys
                            '("switch_z_pos"
                              "mod_geom"))
        int-keys (append int-keys '("flights_level")))
  (dialog-init "stair/stair-multi.dcl" "stair_multi" nil))

;; function to make common defuns and setqs -- designed to be called
;; in a scope that makes them local
(defun stair-dialog-load-common-defs ()
  (defun height_f2f-action (value / risers i)
    (set_tile "height_f2f"
              (if (> value 0)
                  (to-string value)
                "ERR"))
    ;; get minimum riser count
    (setq risers (if (> value 0)
                     (progn
                       (setq i 1)
                       (while (> (/ (float value) i) 7)
                         (setq i (1+ i)))
                       i)
                   nil))
    (set_tile "risers_level"
              (if (defined risers) (to-string risers) "ERR"))
    (set_tile "rise_hgt"
              (if (defined risers) (to-string (/ (float value) risers)) "ERR")))

  (defun risers_level-action (value / tmp)
    (set_tile "rise_hgt"
              (if (and (> value 0)
                       (defined (setq tmp (distof (get_tile "height_f2f")))))
                  (to-string (/ tmp value))
                "ERR")))

  (psc-load "stair/tread_opts.lsp")
  (psc-load "stair/conn_opts.lsp")
  (psc-load "stair/rail_opts.lsp")

  (setq
   ;; keys for popup lists
   popup-keys '(("tread_style" . tread-styles-strings)
                ("top_conn" .  top-details-strings)
                ("bot_conn" .  bot-details-strings)
                ("l-string_mat" . stringers)
                ("r-string_mat" . stringers)
                ("finish" . finishes)
                ("ascend" "Left" "Right")
                ("z_pos" "Near" "Far")
                ("rail_style" . rail-styles))

   ;; keys for edit boxes to be read with distof
   dist-keys '("tread_depth"
               "width"
               "dist_btwn"
               "rise_hgt"
               "top_ext"
               "bot_ext"
               "top_cdist"
               "bot_cdist"
               "l-nosing_top"
               "l-nosing_mid"
               "l-nosing_bot"
               "r-nosing_top"
               "r-nosing_mid"
               "r-nosing_bot"
               "height_f2f"
               "r-string_xtra_tex"
               "r-string_xtra_bex"
               "l-string_xtra_tex"
               "l-string_xtra_bex"
               "head_clr")

   ;; keys for edit boxes to be read with safe-atoi
   int-keys '("rise_qty"
              "risers_level")

   ;; keys for edit boxes and toggles to be read raw
   plain-string-keys '("number"
                       "level"
                       "sequence"
                       "comment")

   ;; check boxes
   toggle-keys '("show_conn"
                 "show_arc")

   ;; default actions for tiles
   ;; (only need to be specified here if not named key_name-action)
   default-actions '(("top_conn" . conn-action)
                     ("bot_conn" . conn-action)
                     ("level" . force-uppercase)
                     ("sequence" . force-uppercase))

   ;; tiles to be loaded by Load Defaults button
   load-defaults-keys '("number"
                        "tread_depth"
                        "width"
                        "dist_btwn"
                        "top_cdist"
                        "bot_cdist"
                        "head_clr"
                        "ascend"
                        "tread_style"
                        "pan_mat"
                        "tread_mat"
                        "stn_style"
                        "floor_thick"
                        "stn_tb_only"
                        "kickback"
                        "quiet_tread"
                        "stn_tb_nclr"
                        "stn_tb_iclr"
                        "stn_tb_ins1"
                        "stn_tb_ins2"
                        "stn_tb_sku"
                        "stn_mid_nclr"
                        "stn_mid_iclr"
                        "stn_mid_ins1"
                        "stn_mid_ins2"
                        "stn_mid_sku"
                        "rail_style"
                        "finish"
                        "l-string_mat"
                        "l-string_xtra_tex"
                        "l-string_xtra_bex"
                        "l-nosing_top"
                        "l-nosing_mid"
                        "l-nosing_bot"
                        "r-string_mat"
                        "r-string_xtra_tex"
                        "r-string_xtra_bex"
                        "r-nosing_top"
                        "r-nosing_mid"
                        "r-nosing_bot"
                        )
    )
  )

(defun stair-multi-edit (source
                         delta
                         /
                         switch-z-pos
                         process-sd-key
                         process-rail-opts
                         update-stair
                         flights
                         level
                         sequence
                         flights_level
                         )
  (defun switch-z-pos ( / elt val)
    (if (= (cadr (assoc "switch_z_pos" delta)) "1")
        (progn
          (setq elt (assoc "z_pos" stair-data)
                val (cadr elt))
          (subst! (list "z_pos"
                        (if (= val "Near") "Far" "Near"))
                  elt
                  'stair-data))))
  ;; substitute sub-dialog options
  ;; for tread_opts and conn_opts
  (defun process-sd-key (key)
    (remove! 'stair-data '(= (car x) key))
    (if (setq tmp (assoc key delta))
        (progn
          (remove! 'delta (= x tmp))
          (add-to-list 'stair-data tmp))))

  ;; ensure correct keys for multi-edit rail_opts
  (defun process-rail_opts (delta / rtype def sds all)
    (foreach side '("i" "o")
      (if (setq rtype (assoc (strcat side "-rail_type") delta))
          (progn
            (setq rtype (cadr rtype)
                  sds (mapcar 'car stair-data)
                  all (uniquify (map-append 'cdr rail-types))
                  all (mapcar '(lambda(x) (strcat side "-rail_" x)) all)
                  def (cdr (assoc rtype rail-types)))
            ;; remove all non-members from stair-data
            (foreach key sds
                     (if (and (member key all)
                              (not (member (substr key 8) def)))
                         (remove! 'stair-data '(= (car x) key))))
            ;; add ones that are not present (for subst to work)
            ;; set them to default values (in case they are not present in delta)
            (foreach key def
                     (setq key (strcat side "-rail_" key))
                     (if (not (member key sds))
                         (add-to-list
                          'stair-data
                          (list key (if (member key rail_opts-dist-keys) 0.0 "0")))))))))

  ;; update the data for and redraw one stair block
  (defun update-stair (ename
                       /
                       app-id
                       stair-data
                       ins
                       tmp
                       tb
                       rot
                       )
    (setq app-id (cadr (assoc "app_id" (read-xdata ename "")))
          stair-data (vl-sort (draw-xdata-migrate 'ename app-id) sort-data-by-key)
          rot (vla-get-rotation (ename>vlobj ename))
          ins (get-ins-and-delete ename))
    (switch-z-pos)
    ;; renumber flights
    (if level
        (progn
          (subst-key! "level" (to-string level) 'stair-data)
          (subst-key! "sequence" (substr "ABCDEFGH" sequence 1) 'stair-data)
          (inc! 'sequence)
          (if (> sequence flights_level)
              (setq sequence 1
                    level (1+ level)))))
    ;; handle sub-dialog keys
    (if (vl-some '(lambda(x) (= (car x) "tread_style")) delta)
        (mapcar 'process-sd-key tread_opts-keys))
    (if (vl-some '(lambda(x) (= (car x) "top_conn")) delta)
        (mapcar 'process-sd-key (remove conn_opts-keys '(not (begins-with x "t")))))
    (if (vl-some '(lambda(x) (= (car x) "bot_conn")) delta)
        (mapcar 'process-sd-key (remove conn_opts-keys '(not (begins-with x "b")))))
    (if (vl-some '(lambda(x) (= (car x) "rail_style")) delta)
        (process-rail_opts delta))
    ;; add comment key if not present
    (if (and (member "comment" (mapcar 'car delta))
             (not (member "comment" (mapcar 'car stair-data))))
        (setq stair-data (vl-sort (cons '("comment" "") stair-data)
                                  sort-data-by-key)))
    ;; rewrite XDATA
    (foreach elt delta
             (subst! elt (assoc (car elt) stair-data) 'stair-data))
    ;; recalculate derived keys
    (remove! 'stair-data '(member (car x) stair-derived-keys))
      ((if (= app-id "3D_Stair") draw-3d_stair draw-stair)
        (stair-add-derived-keys stair-data)
        ins
      ) ;_ draw-stair -or- draw-3d_stair
    ) ;; update-stair
  
  (if (setq level (assoc "level" delta))
      (setq level (atoi (cadr level))
            flights_level (cadr (assoc "flights_level" delta))
            sequence 1
            delta (vl-remove-if '(lambda(x) (= (car x) "level")) delta)
            source (vl-sort source 'sort-stairland)))
  
  (mapcar 'update-stair source))
