(set-file-docstring "Function to draw a stair end view")

(defun-q stair-endview-draw (ins lr offset /
                                 mirror-p
                                 blockObj
                                 y
                                 obj
                                 istrwid
                                 ostrwid
                                 inswid)
  "Draws an end view of a stair at INS

LR should be one of the symbols 'left or 'right to indicate which perspective to
draw (relative to the view of the section). OFFSET is the distance from the center of the
shaft to the insert point of the stair."
  (declare (vars (ins list (point-p ins))
                 (lr sym (member lr '(left right)))
                 (offset nil (numberp offset)))
           with-data)
  (setq blockObj (p-blk assembly)
        clayer (getvar 'clayer))

  (setvar 'clayer "SOLID")

  (setq mirror-p
        (or (and (= lr 'left)
                 (= z_pos "Far"))
            (and (= lr 'right)
                 (= z_pos "Near"))))
  (mapcar 'set
          (if (= ascend "Left")
              '(istrwid ostrwid)
            '(ostrwid istrwid))
          (list l-string_wid r-string_wid))

  ;; "(setq obj ...), mirror"
  (defun som (new-obj)
    (setq obj new-obj)
    (if mirror-p (vla-mirror-y obj)))
    
  ;; inside stringer
  (som (add-rectangle blockObj istrwid flight_hgt (list offset 0 0)))

  ;; outside stringer
  (som (add-rectangle blockObj ostrwid flight_hgt (list (- (+ width offset) ostrwid) 0 0)))

  ;; risers
  (setq y 0)
  (setq inswid (- width istrwid ostrwid))
  (repeat (1+ rise_qty)
          (som (add-polyline blockObj
                             (list (+ offset istrwid) y 0)
                             (list '(0 0 0)
                                   (list inswid 0 0))
                             nil))
          (setq y (+ y rise_hgt)))

  ;; insert block
  (setvar 'clayer clayer)
  (setq blockObj (insert-block modelSpace ins blockObj))
  (vla-put-Layer blockObj (tk '(strcat z_pos "-stair")))
  (assign-xdata blockObj "EndView" (format-xdata (list (list "offset" offset)
                                                       (list "lr" (to-string lr))
                                                       (assoc "assembly" data)))))
(defun-r 'stair-endview-draw)
