;; Functions to update the stair XDATA specification over time

(setq stair-version 23)

;; derived-key only migrations: 5-9, 11, 13-16, 19-20, 22-23

;; add bolt spacing and remove extraneous keys
(defun stair-migration-1 (data / conn-series)
  ;; add ~_vbolt ~_hbolt and ~_hbolt2
  (foreach tb '("top" "bot")
           (setq conn-series (substr (cadr (assoc (strcat tb "_conn") data)) 2 1))
           (if (member conn-series '("2" "5"))
               (progn
                 (add-to-list 'data (list (strcat tb "_vbolt") 5.0))
                 (if (= conn-series "2")
                     (add-to-list 'data (list (strcat tb "_hbolt") 4.0))
                   (progn
                     (add-to-list 'data (list (strcat tb "_hbolt") 2.0))
                     (add-to-list 'data (list (strcat tb "_hbolt2") 4.0)))))))
  ;; l/r-string_name was removed as a derived key
  (remove! 'data '(ends-with (car x) "string_name"))
  data)

;; add rail_opts keys
(defun stair-migration-2 (data / )
  (append data
          '(("rail_topmnt" "0")
            ("i-rail_type" "Stair Rail")
            ("i-rail_top_ext" 0.0)
            ("i-rail_bot_ext" 0.0)
            ("i-rail_top_dloop" "0")
            ("i-rail_bot_dloop" "0")
            ("o-rail_type" "Wall Rail")
            ("o-rail_top_dloop" "0")
            ("o-rail_bot_dloop" "0"))))

;; remove tread series from connections
(defun stair-migration-3 (data / )
  (subst-key! "top_conn" (tk '(substr top_conn 2)) 'data)
  (subst-key! "bot_conn" (tk '(substr bot_conn 2)) 'data)
  data)

;; change head_clr from toggle to dist
(defun stair-migration-4 (data / )
  (if (assoc "head_clr" data)
      (subst-key! "head_clr" 80.0 'data)
    (add-to-list 'data '("head_clr" 80.0))))

;; include LLH/V on 4x-angles
(defun stair-migration-10 (data / memb)
  (if (and (setq memb (assoc "top_member" data))
           (/= (setq memb (cadr memb)) "L3x3x1/4"))
      (subst-key! "top_member" (strcat memb "_LLH") 'data))
  (if (and (setq memb (assoc "bot_member" data))
           (/= (setq memb (cadr memb)) "L3x3x1/4"))
      (subst-key! "top_member" (strcat memb "_LLH") 'data))
  data)

;; add show_arc
(defun stair-migration-12 (data)
  (append data
          '(("show_arc" "0"))))

;; change 3/16" and 1/4" to 3/16 and 1/4
(defun stair-migration-17 (data)
  (if (member (xd-value "pan_mat" data) '("3/16\"" "1/4\""))
      (subst-key "pan_mat" (vl-string-right-trim "\"" (xd-value "pan_mat" data)) data)
      data))
      
;; split rail_topmnt
(defun stair-migration-21 (data / topmnt)
  (setq topmnt (xd-value "rail_topmnt" data))
  (if (wcmatch (xd-value "i-rail_type" data) "*Stair*")
      (add-to-list 'data (list "i-rail_topmnt" topmnt)))
  (if (wcmatch (xd-value "o-rail_type" data) "*Stair*")
      (add-to-list 'data (list "o-rail_topmnt" topmnt)))
  (remove data '(= (car x) "rail_topmnt")))
