;;; Functions for drawing the main section view of a stair

  (psc-include
   '(
     "stair/stringer.lsp"
     "stair/tread.lsp"
     "shapes/line.lsp"
     "shapes/decking.lsp"
     "stair/util.lsp"
     "fab/part-details.lsp"
    )
  ) ;_ psc-include

  ;;; insert the stair section block at ins
  (defun draw-stair (data ins / blockObj layer)
    (with-data data '((setq blockObj (p-blk assembly))))
    (setq
      layer (tk '(strcat z_pos "-Stair"))
    ) ;_ setq
    (with-data data
     '(
        ;;; non-plotting debug geometry
        (if nil (draw-stair-debug))

        ;;; top and bottom connections
        (foreach t/b '("t" "b")
          (draw_stair_block_connection
            blockObj
            t/b
            nil
            (if (= show_conn "1") layer "NO PLOT")
          ) ;_ draw_stair_block_connection
        ) ;_ foreach

        ;;; outside stringer
        (if (= "0" stringers_eq)
          (add-stair-stringer
           '(out cap)
            blockObj
            (if (= "Left" ascend) "R" "L")
            "NO PLOT"
            "NO PLOT"
          ) ;_ add-stair-stringer
        ) ;_ if

        ;;; inside stringer
        (add-stair-stringer
         '(out cap flg)
          blockObj
          (substr ascend 1 1)
          layer
          "HIDDEN"
        ) ;_ add-stair-stringer

        ;;; quiet tread
        (if (= "1" quiet_tread)
          (add-stair-qtread blockObj "Stair Quiet Tread")
        ) ;_ if

        ;;; conc. fill/precast
        (if (member (substr tread_style 1 1) '("1" "4"))
          (add-stair-conc blockObj)
        ) ;_ if

        ;;; treads
        (add-stair-treads blockObj layer)

        ;;; safe-t-nose
        (if (not (member stn_style (list nil "None")))
          (add-stair-stn blockObj "STN")
        ) ;_ if

        ;;; head clear
        (if (> head_clr 0) (draw_head_clear blockObj '(0 0)))

        ;;; stair label
        (draw_stair_label blockObj)
      )
    ) ;_ with-data

    (setq blockObj (insert-block modelSpace ins blockObj))
    (vla-put-Layer blockObj layer)
    (assign-xdata blockObj "Stair" (format-xdata data))
    (progn
      (setvar 'cmdecho 0)
      (command "DRAWORDER" (entlast) "" "b")
      (setvar 'cmdecho 1)
    ) ;_ progn
    (vlax-vla-object->ename blockObj)
  ) ;_ defun draw-stair

  (defun draw_head_clear (add-to ins / plobj)
    (setq plobj
      (add-polyline
        add-to
        (list (+ bot_ext bot_cdist) (+ rise_hgt head_clr) 0)
        (list
          (mapcar '- (list width rise_hgt 0.0))
          (mapcar '- (list tread_depth rise_hgt 0.0))
          (list run (- flight_hgt rise_hgt) 0.0)
          (list (+ run width) (- flight_hgt rise_hgt) 0.0)
        ) ;_ list
        nil
      ) ;_ add-polyline
    ) ;_ setq
    (if (= dir -1) (vla-mirror-y plobj))
    (vla-put-Layer plobj (strcat z_pos "-stair"))
    (vla-put-Color plobj 252)
    (vla-put-Linetype plobj "HIDDEN2")
  ) ;_ defun draw_head_clear

  (defun draw_stair_label (add-to / ins)
    (setq ins
      (line-int
        (line-offset
          (stair-slope)
          (if (= ascend "Left")
            (+ (- l-string_dpt) l-nosing_mid -4)
            (+ (- r-string_dpt) r-nosing_mid -4)
          ) ;_ if
        ) ;_ line-offset
        (apply 'pt-slope
          (cons
            (* (- dir) (/ tread_depth rise_hgt))
            (mapcar '+
              ((if (< rise_qty 3) car cadr) (reverse (calc-nose-pts)))
              (mapcar '/ (list (* dir tread_depth) rise_hgt) '(-4 -4))
            ) ;_ mapcar
          ) ;_ cons
        ) ;_ apply
        nil
      ) ;_ line-int
    ) ;_ setq
    (
      (lambda (obj)
        (vla-put-Alignment obj acAlignmentMiddleCenter)
        (vla-put-Layer obj "TEXT")
        (vla-put-StyleName obj "CALIBRI")
        (vla-put-Rotation obj (* dir (dtr pitch)))
        (vla-put-Scalefactor obj 0.8)
        (vla-Move obj (3dpt 0 0) (3dpt ins))
      ) ;_ lambda
      (vla-AddText add-to (strcase assembly) (3dpt 0 0) 4.5)
    )
  ) ;_ defun draw_stair_label

  ;;; insert a block with the geometry and optionally dimensions for a connection detail
  (defun draw_stair_block_connection (container tb dims-p layer / conn ins clayer blockObj obj)
    (debug-print '("Drawing " #(uppercase tb) " stair connection"))
    (if (begins-with tb "b")
      (setq
        conn bot_conn_full
        ins (3dpt 0 0)
      ) ;_ setq
      (setq
        conn top_conn_full
        ins (3dpt (+ bot_ext bot_cdist run top_cdist top_ext) (* rise_qty rise_hgt))
      ) ;_ setq
    ) ;_ if
    (setq
      blockObj (p-blk (strcat tb "-conn"))
      clayer (vla-getvar 'clayer)
    ) ;_ setq
    (set-active-layer layer)
    (if (attempt-apply (read (strcat "detail-side-" conn)) (list dims-p))
      (progn
        (setq obj (insert-block container ins blockObj))
        (if (= dir -1) (vla-mirror-y obj))
      ) ;_ progn
    ) ;_ if
    (set-active-layer clayer)
  ) ;_ defun draw_stair_block_connection

  ;; add non-plotting debug geometry
  (defun draw-stair-debug ()
    (vla-put-Layer ; BLOCK INS
      (vla-AddCircle blockObj (vlax-3d-point '(0 0 0)) 1.0)
      "NO PLOT"
    ) ;_ vla-put-Layer
    (mapcar ; NOSING
     '(lambda (p)
        (vla-put-Layer
          (vla-AddCircle blockObj (vlax-3d-point p) 0.375)
          "NO PLOT"
        ) ;_ vla-put-Layer
      ) ;_ lambda
      (calc-nose-pts)
    ) ;_ mapcar
    (add-stair-stringer ; STRINGER SEGMENTS
     '(seg)
      blockObj
      (substr ascend 1 1)
      "NO PLOT"
      "NO PLOT"
    ) ;_ add-stair-stringer
  ) ;_ defun draw-stair-debug
