(set-file-docstring
 "Stair stringer calculation")

(psc-include '("shapes/line.lsp"))

(defun-q stair-stringer-calc (side / keys nosing_bot nosing_mid nosing_top string_dpt
      string_bex string_xtra_bex string_tex string_xtra_tex string_flg string-max-x-val
                              str_slp flg_slp string-def cap-num)
  "Returns the stringer properties for one side of a stair

SIDE should be a string beginning with l, r, i, or o."
  (declare (vars (side str))
           with-data)
  (setq side (substr side 1 1)
        cap-num 0
        keys '(nosing_bot
               nosing_mid
               nosing_top
               string_dpt
               string_bex
               string_xtra_bex
               string_tex
               string_xtra_tex
               string_flg))
  (mapcar '(lambda(k) (set k (sidekey (to-string k)))) keys)
  (setq string-max-x-val (+ string_bex
                            bot_cdist
                            run
                            string_tex
                            top_cdist
                            (- string_xtra_bex)
                            (if string_flg -0.25 0.0))
        str_slp (pt-slope (/ rise_hgt tread_depth)
                          (+ string_bex bot_cdist (- string_xtra_bex))
                          rise_hgt))
  (if string_flg
      (setq flg_slp (apply '(lambda (s n d f)
                             (mapcar '(lambda (off) (line-offset s (- n off)))
                                     (list (- d f) f)))
                           (list str_slp nosing_mid string_dpt string_flg))))
  (strip-nil
   (cond
     ;; F
     ((and (= starter_p "1")
           (<= (+ flight_hgt nosing_top) string_dpt))
      (stair-stringer--shape-f))
     ;; HS
     ((and (= "1" starter_p)
           (or (= "4" top_conn_det)
               (> string_xtra_tex tread_depth)))
      (stair-stringer--shape-hs))
     ;; HH
     ((and (or (= "4" top_conn_det)
               (> string_xtra_tex tread_depth))
           (or (= "5" bot_conn_det)
               (> string_xtra_bex tread_depth)))
      (stair-stringer--shape-hh))
     ;; HA
     ((or (= "4" top_conn_det)
          (> string_xtra_tex tread_depth))
      (stair-stringer--shape-ha))
     ;; AS
     ((= starter_p "1")
      (stair-stringer--shape-as))
     ;; AH
     ((or (= "5" bot_conn_det)
          (> string_xtra_bex tread_depth))
      (stair-stringer--shape-ah))
     ;; AA
     (T (stair-stringer--shape-aa)))))
(defun-r 'stair-stringer-calc)

;;;;;;;;;;;;;;;;;;;;;;;;;;
;; shape calc functions ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun-q stair-stringer--shape-f (/ f-bot)
  "Returns F stringer properties"
  (declare with-data)
  (setq f-bot (stair-stringer--pts-f-bot))
  (append (list (cons 'string-outline f-bot))
          (list (cons 'string-mid-seg f-bot))
          (if string_flg (stair-stringer--caps 'f-bot))))
(defun-r 'stair-stringer--shape-f)

(defun-q stair-stringer--shape-hh (/ h-bot h-top)
  "Returns HH stringer properties"
  (declare with-data)
  (setq h-bot (stair-stringer--pts-h-bot)
        h-top (stair-stringer--pts-h-top))
  (append (list (cons 'string-outline (append h-bot h-top)))
          (list (cons 'string-bot-seg h-bot))
          (if string_flg (stair-stringer--caps 'h-bot))
          (list (cons 'string-mid-seg
                      (list (car h-bot) (last h-bot) (car h-top) (last h-top))))
          (if string_flg
              (list (cons 'string-mid-flg-1
                          (mapcar '(lambda (y) (line-int (car flg_slp) (list 0.0 y) nil))
                                  (list (+ (cadr (car h-bot)) string_flg)
                                        (+ (cadr (last h-top)) string_flg))))
                    (cons 'string-mid-flg-2
                          (mapcar '(lambda (y) (line-int (cadr flg_slp) (list 0.0 y) nil))
                                  (list (- (cadr (last h-bot)) string_flg)
                                        (- (cadr (car h-top)) string_flg))))))
          (list (cons 'string-top-seg h-top))
          (if string_flg (stair-stringer--caps 'h-top))))
(defun-r 'stair-stringer--shape-hh)

(defun-q stair-stringer--shape-hs (/ s-bot h-top)
  "Returns HS stringer properties"
  (declare with-data)
  (setq s-bot (stair-stringer--pts-s-bot)
        h-top (stair-stringer--pts-h-top))
  (append (list (cons 'string-outline (append s-bot h-top)))
          (list (cons 'string-mid-seg (append s-bot (list (car h-top) (last h-top)))))
          (if string_flg
              (list (cons 'string-mid-flg-1
                          (mapcar '(lambda (y) (line-int (car flg_slp) (list 0.0 y) nil))
                                  (list (if (= 227 (read bot_conn_full))
                                            (- 0.25 pan_thick)
                                            0.25)
                                        (+ (cadr (last h-top)) string_flg))))
                    (cons 'string-mid-flg-2
                          (list (list 0.25 (line-val (cadr flg_slp) 0.25))
                                (line-int (cadr flg_slp)
                                          (list 0.0 (- (cadr (car h-top)) string_flg))
                                          nil)))))
          (if string_flg (stair-stringer--caps 's-bot))
          (list (cons 'string-top-seg h-top))
          (if string_flg (stair-stringer--caps 'h-top))))
(defun-r 'stair-stringer--shape-hs)

(defun-q stair-stringer--shape-ha (/ a-bot h-top)
  "Returns HA stringer properties"
  (declare with-data)
  (setq a-bot (stair-stringer--pts-a-bot)
        h-top (stair-stringer--pts-h-top))
  (append (list (cons 'string-outline (append a-bot h-top)))
          (list (cons 'string-mid-seg (append a-bot (list (car h-top) (last h-top)))))
          (if string_flg
              (list (cons 'string-mid-flg-1
                          (list (list 0.25 (line-val (car flg_slp) 0.25))
                                (line-int (car flg_slp)
                                          (list 0.0 (+ (cadr (last h-top)) string_flg))
                                          nil)))
                    (cons 'string-mid-flg-2
                          (list (list 0.25 (line-val (cadr flg_slp) 0.25))
                                (line-int (cadr flg_slp)
                                          (list 0.0 (- (cadr (car h-top)) string_flg))
                                          nil)))))
          (if string_flg (stair-stringer--caps 'a-bot))
          (list (cons 'string-top-seg h-top))
          (if string_flg (stair-stringer--caps 'h-top))))
(defun-r 'stair-stringer--shape-ha)

(defun-q stair-stringer--shape-as (/ s-bot a-top)
  "Returns AS stringer properties"
  (declare with-data)
  (setq s-bot (stair-stringer--pts-s-bot)
        a-top (stair-stringer--pts-a-top))
  (append (list (cons 'string-outline (append s-bot a-top)))
          (list (cons 'string-mid-seg (append s-bot a-top)))
          (if string_flg
              (list (cons 'string-mid-flg-1
                          (list (line-int (car flg_slp)
                                          (list 0.0
                                                (if (= 227 (read bot_conn_full))
                                                    (- 0.25 pan_thick)
                                                    0.25))
                                          nil)
                                (list string-max-x-val
                                      (line-val (car flg_slp) string-max-x-val))))
                    (cons 'string-mid-flg-2
                          (list (list 0.25 (line-val (cadr flg_slp) 0.25))
                                (line-int (cadr flg_slp)
                                          (list 0.0 (cadr (car a-top)))
                                          nil)))))
          (if string_flg (stair-stringer--caps 's-bot))
          (if string_flg (stair-stringer--caps 'a-top))))
(defun-r 'stair-stringer--shape-as)

(defun-q stair-stringer--shape-ah (/ h-bot a-top)
  "Returns AH stringer properties"
  (declare with-data)
  (setq h-bot (stair-stringer--pts-h-bot)
        a-top (stair-stringer--pts-a-top))
  (append (list (cons 'string-outline (append h-bot a-top)))
          (list (cons 'string-bot-seg h-bot))
          (if string_flg (stair-stringer--caps 'h-bot))
          (list (cons 'string-mid-seg (append (list (car h-bot) (last h-bot)) a-top)))
          (if string_flg
              (list (cons 'string-mid-flg-1
                          (list (line-int (car flg_slp)
                                          (list 0.0 (+ (cadr (car h-bot)) string_flg))
                                          nil)
                                (list string-max-x-val
                                      (line-val (car flg_slp) string-max-x-val))))
                    (cons 'string-mid-flg-2
                          (mapcar '(lambda (y) (line-int (cadr flg_slp) (list 0.0 y) nil))
                                  (list (- (cadr (last h-bot)) string_flg)
                                        (cadr (car a-top)))))))
          (if string_flg (stair-stringer--caps 'a-top))))
(defun-r 'stair-stringer--shape-ah)

(defun-q stair-stringer--shape-aa (/ a-bot a-top)
  "Returns AA stringer properties"
  (declare with-data)
  (setq a-bot (stair-stringer--pts-a-bot)
        a-top (stair-stringer--pts-a-top))
  (append (list (cons 'string-outline (append a-bot a-top)))
          (list (cons 'string-mid-seg (append a-top a-bot)))
          (if string_flg
              (list (cons 'string-mid-flg-1
                          (mapcar '(lambda (x) (list x (line-val (car flg_slp) x)))
                                  (list 0.25 string-max-x-val)))
                    (cons 'string-mid-flg-2
                          (list (list 0.25 (line-val (cadr flg_slp) 0.25))
                                (line-int (cadr flg_slp)
                                          (list 0.0 (cadr (car a-top)))
                                          nil)))))
          (if string_flg (stair-stringer--caps 'a-top))
          (if string_flg (stair-stringer--caps 'a-bot))))
(defun-r 'stair-stringer--shape-aa)

;;;;;;;;;;;;;;;;;;;
;; pts functions ;;
;;;;;;;;;;;;;;;;;;;

(defun-q stair-stringer--pts-f-bot ()
  "Returns points for F shaped stringer"
  (declare with-data)
  (list
   (list (+ (- string_xtra_bex) (if string_flg 0.25 0.0)) 0.0)
   (list (+ (- string_xtra_bex) (if string_flg 0.25 0.0))
         (line-val (line-offset str_slp
                                (- nosing_mid (if string_flg 0.25 0.0)))
                   (+ (- string_xtra_bex) (if string_flg 0.25 0.0))))
   (line-int (line-offset str_slp
                          (- nosing_mid (if string_flg 0.25 0.0)))
             (list 0.0
                   (if (< (+ flight_hgt nosing_top) string_dpt)
                       (- (+ flight_hgt nosing_top)
                          (if string_flg 0.25 0.0))
                       string_dpt))
             nil)
   (list string-max-x-val
         (if (< (+ flight_hgt nosing_top) string_dpt)
             (- (+ flight_hgt nosing_top) (if string_flg 0.25 0.0))
             string_dpt))
   (list string-max-x-val 0.0)))
(defun-r 'stair-stringer--pts-f-bot)

(defun-q stair-stringer--pts-s-bot ()
  "Returns points for starter stringer bottom"
  (declare with-data)
  (list
   (line-int (line-offset str_slp (- nosing_mid string_dpt))
             (list 0.0
                   (+ (if string_flg 0.25 0.0)
                      (if (= 227 (read bot_conn_full)) (- pan_thick) 0)))
             nil)
   (mapcar '+
           (list (- string_xtra_bex) 0.0)
           (if string_flg '(0.25 0.25) '(0.0 0.0))
           (if (= 227 (read bot_conn_full)) (list 0 (- pan_thick)) '(0 0)))
   (list (+ (- string_xtra_bex) (if string_flg 0.25 0.0))
         (line-val (line-offset str_slp nosing_mid)
                   (+ (- string_xtra_bex) (if string_flg 0.25 0.0))))))
(defun-r 'stair-stringer--pts-s-bot)

(defun-q stair-stringer--pts-h-bot ()
  "Returns points for haunched stringer bottom"
  (declare with-data)
  (list
   (line-int (line-offset str_slp (- nosing_mid string_dpt))
             (list 0.0 (- nosing_bot string_dpt))
             nil)
   (list (+ (- string_xtra_bex) (if string_flg 0.25 0.0))
         (- nosing_bot string_dpt))
   (list (+ (- string_xtra_bex) (if string_flg 0.25 0.0))
         nosing_bot)
   (line-int (line-offset str_slp nosing_mid)
             (list 0.0 nosing_bot)
             nil)))
(defun-r 'stair-stringer--pts-h-bot)

(defun-q stair-stringer--pts-a-bot ()
  "Returns points for standard stringer bottom"
  (declare with-data)
  (list
   (list (+ (- string_xtra_bex) (if string_flg 0.25 0.0))
         (line-val (line-offset str_slp (- nosing_mid string_dpt))
                   (+ (- string_xtra_bex) (if string_flg 0.25 0.0))))
   (list (+ (- string_xtra_bex) (if string_flg 0.25 0.0))
         (line-val (line-offset str_slp nosing_mid)
                   (+ (- string_xtra_bex) (if string_flg 0.25 0.0))))))
(defun-r 'stair-stringer--pts-a-bot)

(defun-q stair-stringer--pts-h-top ()
  "Returns points for haunched stringer top"
  (declare with-data)
  (list
   (line-int (line-offset str_slp nosing_mid)
             (list 0.0 (+ flight_hgt nosing_top))
             nil)
   (list string-max-x-val
         (+ flight_hgt nosing_top))
   (list string-max-x-val
         (+ flight_hgt (- nosing_top string_dpt)))
   (line-int (line-offset str_slp (- nosing_mid string_dpt))
             (list 0.0 (+ flight_hgt (- nosing_top string_dpt)))
             nil)))
(defun-r 'stair-stringer--pts-h-top)

(defun-q stair-stringer--pts-a-top ()
  "Returns points for standard stringer top"
  (declare with-data)
  (list
   (line-int (line-offset str_slp nosing_mid)
             (list 0.0
                   (+ flight_hgt nosing_top (if string_flg -0.25 0.0)))
             nil)
   (list string-max-x-val
         (+ flight_hgt nosing_top (if string_flg -0.25 0.0)))
   (list string-max-x-val
         (line-val (line-offset str_slp (- nosing_mid string_dpt))
                   string-max-x-val))))
(defun-r 'stair-stringer--pts-a-top)

;;;;;;;;;;;;;;;;;;;
;; cap functions ;;
;;;;;;;;;;;;;;;;;;;

(defun-q stair-stringer--caps (style / pts ret)
  "Returns the cap properties for a stringer style

STYLE should correspond to a stringer shape like h-top or a-bot."
  (declare with-data)
  (setq style (to-string style)
        pts (attempt-apply (read (strcat "stair-stringer--pts-" style)) nil))
  (if (not pts) (error "stringer style not found"))
  (apply (read (strcat "stair-stringer--caps-" style)) nil))
(defun-r 'stair-stringer--caps)

(defun-q stair-stringer--caps-f-bot ()
  "Returns cap properties for F shaped stringer"
  (declare with-data)
  (list
   (cons 'string-mid-flg-1
         (mapcar '(lambda (v) (mapcar '+ v (list 0.0 string_flg)))
                 (list (car pts) (last pts))))
   (if (= (+ flight_hgt nosing_top) string_dpt)
       (cons 'string-mid-flg-2
             (list
              (mapcar '- (cadddr pts) (list 0.0 string_flg))
              (line-int (line-offset str_slp (- nosing_mid 0.25))
                        (list
                         0.0
                         (- (+ flight_hgt nosing_top) string_flg))
                        nil))))
   (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
         (append
          (list (car pts) (cadr pts))
          (mapcar '(lambda (v) (mapcar '- v '(0.25 0.0)))
                  (reverse (list (car pts) (cadr pts))))))
   (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
         (apply '(lambda (pts ang)
                  (append pts
                   (mapcar '(lambda (p) (polar p (+ (/ pi 2.0) ang) 0.25))
                    (reverse pts))))
                (apply '(lambda (ang)
                         (list
                          (list (cadr pts)
                           (if (< (+ flight_hgt nosing_top) string_dpt)
                               (caddr pts)
                               (polar (caddr pts) ang -0.125)))
                          ang))
                       (list (angle '(0 0) (list tread_depth rise_hgt))))))
   (if (< (+ flight_hgt nosing_top) string_dpt)
       (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
             (append
              (list (caddr pts) (mapcar '+ (cadddr pts) '(0.25 0.0)))
              (mapcar '(lambda (v) (mapcar '+ v '(0.0 0.25)))
                      (reverse
                       (list (caddr pts)
                             (mapcar '+ (cadddr pts) '(0.25 0.0))))))))
   (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
         (append
          (list (cadddr pts) (last pts))
          (mapcar '(lambda (v) (mapcar '+ v '(0.25 0.0)))
                  (reverse (list (cadddr pts) (last pts))))))))
(defun-r 'stair-stringer--caps-f-bot)

(defun-q stair-stringer--caps-s-bot ()
  "Returns cap properties for starter stringer bottom"
  (declare with-data)
  (list (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
              (apply '(lambda (pts)
                       (append pts
                        (mapcar '(lambda (v) (mapcar '- v '(0.0 0.25)))
                         (reverse pts))))
                     (list (list (mapcar '- (car pts) '(0.125 0.0)) (cadr pts)))))
        (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
              (apply '(lambda (pts)
                       (append pts
                        (mapcar '(lambda (v) (mapcar '- v '(0.25 0.0)))
                         (reverse pts))))
                     (list
                      (list (mapcar '- (cadr pts) '(0.0 0.25))
                            (mapcar '- (caddr pts) '(0.0 0.125))))))))
(defun-r 'stair-stringer--caps-s-bot)

(defun-q stair-stringer--caps-h-bot ()
  "Returns cap properties for haunched stringer bottom"
  (declare with-data)
  (list
   (cons 'string-bot-flg-1
         (apply '(lambda (d-mid d-bot)
                  (list (line-int (line-offset str_slp d-mid)
                         (list 0.0 d-bot)
                         nil)
                   (mapcar '+ (cadr pts) (list 0.0 string_flg))))
                (mapcar '(lambda (n) (- n (- string_dpt string_flg)))
                        (list nosing_mid nosing_bot))))
   (cons 'string-bot-flg-2
         (apply '(lambda (d-mid d-bot)
                  (list (line-int (line-offset str_slp d-mid)
                         (list 0.0 d-bot)
                         nil)
                   (mapcar '- (caddr pts) (list 0.0 string_flg))))
                (mapcar '(lambda (n) (- n string_flg))
                        (list nosing_mid nosing_bot))))
   (cons 'string-bot-cap-1
         (append
          (list (cadr pts) (caddr pts))
          (mapcar '(lambda (v) (mapcar '- v '(0.25 0.0)))
                  (reverse (list (cadr pts) (caddr pts))))))))
(defun-r 'stair-stringer--caps-h-bot)

(defun-q stair-stringer--caps-a-bot ()
  "Returns cap properties for standard stringer bottom"
  (declare with-data)
  (list
   (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
         (apply '(lambda (pts)
                  (append pts
                   (mapcar '(lambda (v) (mapcar '- v '(0.25 0.0)))
                    (reverse pts))))
                (list (list (car pts) (mapcar '- (cadr pts) '(0.0 0.125))))))))
(defun-r 'stair-stringer--caps-a-bot)

(defun-q stair-stringer--caps-h-top ()
  "Returns cap properties for haunched stringer top"
  (declare with-data)
  (list
   (cons 'string-top-flg-1
         (apply '(lambda (d-mid d-top)
                  (list (mapcar '+ (caddr pts) (list 0.0 string_flg))
                   (line-int (line-offset str_slp d-mid)
                    (list 0.0 d-top)
                    nil)))
                (mapcar '(lambda (n) (- n (- string_dpt string_flg)))
                        (list nosing_mid (+ flight_hgt nosing_top)))))
   (cons 'string-top-flg-2
         (apply '(lambda (d-mid d-top)
                  (list (mapcar '- (cadr pts) (list 0.0 string_flg))
                   (line-int (line-offset str_slp d-mid)
                    (list 0.0 d-top)
                    nil)))
                (mapcar '(lambda (n) (- n string_flg))
                        (list nosing_mid (+ flight_hgt nosing_top)))))
   (cons 'string-top-cap-1
         (append (list (cadr pts) (caddr pts))
                 (mapcar '(lambda (v) (mapcar '+ v '(0.25 0.0)))
                         (reverse (list (cadr pts) (caddr pts))))))))
(defun-r 'stair-stringer--caps-h-top)

(defun-q stair-stringer--caps-a-top ()
  "Returns cap properties for standard stringer top"
  (declare with-data)
  (list
   (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
         (apply '(lambda (pts)
                  (append pts
                   (mapcar '(lambda (v) (mapcar '+ v '(0.0 0.25)))
                    (reverse pts))))
                (list
                 (list (mapcar '+ (car pts) '(0.125 0.0))
                       (mapcar '+ (cadr pts) '(0.25 0.0))))))
   (cons (read (strcat "string-mid-cap-" (itoa (setq cap-num (1+ cap-num)))))
         (apply '(lambda (pts)
                  (append pts
                   (mapcar '(lambda (v) (mapcar '+ v '(0.25 0.0)))
                    (reverse pts))))
                (list (list (cadr pts) (mapcar '+ (caddr pts) '(0.0 0.125))))))))
(defun-r 'stair-stringer--caps-a-top)
