;; set up for functions which draw full or short views
(defun draw-stair-top-view (data ins break offset /
                            blockObj
                            tv-stair-bolt
                            obj
                            quadrant
                            clayer
                            x
                            y
                            tmp
                            ext
                            istrwid
                            istrbex
                            istrxtex
                            istrtex
                            istrxtex
                            ostrwid
                            ostrbex
                            ostrxbex
                            ostrtex
                            ostrxtex
                            som
                            add-tag
                            add-str
                            add-arc
                            )
  (setq blockObj (p-blk assembly)
        clayer (getvar 'clayer))

  ;; put the obj in the correct quadrant 
  (defun mirror (obj)
    (if (member quadrant '(2 3))
        (vla-mirror-y obj))
    (if (member quadrant '(3 4))
        (vla-mirror-x obj))
    (if (= break "Top")
        (vla-mirror-y obj)))
  ;; "(setq obj ...), (mirror ...)"
  (defun som (new-obj)
    (setq obj new-obj)
    (mirror obj))

  ;; add the ID tag
  (defun add-tag ( / tmp)
    ;; mtext
    (setvar 'clayer "TEXT")
    (setq x (+ (if (= break "Top") (max ix ox) ext) bot_cdist)
          y (+ offset (/ width 2.0)))
    (setq obj (add-mtext blockObj
                         (list (+ x tread_depth) y 0)
                         (strcat "{\\W0.5;"
                                 assembly
                                 "\\P"
                                 (if (= break "Top")
                                     "DN"
                                   "UP")
                                 "}")
                         (* 2 tread_depth)
                         'MiddleCenter))
    (vla-put-Height obj 5.5)
    (vla-put-StyleName obj "Calibri")
    (mirror obj)
    (vla-put-BackgroundFill obj :vlax-true)
    (vla-put-LineSpacingDistance obj 8.0)
    ;; set scale factor of background mask, text frame and defined height
    ;; no way to access in vlisp
    (setq tmp (entget (handent (vla-get-Handle obj)))
          tmp (entmod (subst (cons 45 1.0) (assoc 45 tmp) tmp))
          tmp (entmod (subst (cons 46 16.0) (assoc 46 tmp) tmp))
          tmp (entmod (subst (cons 90 19) (assoc 90 tmp) tmp)))
    ;; arrow
    (setvar 'clayer "SOLID")
    (som (add-polyline blockObj
                       (list x y 0)
                       (list '(0 0 0)
                             (list (* tread_depth 2) 0 0)
                             (list (- (* tread_depth 3) (/ tread_depth 2.0)) 0 0))
                       nil))
    (vla-SetWidth obj 1 3 0)
    ;; circle
    (setvar 'clayer "DIM")
    (som (add-polyline blockObj
                       (list x y 0)
                       (list (list -1 0 (calc-bulge (dtr 180)))
                             (list 1 0 (calc-bulge (dtr 180))))
                       T))
    (vla-put-ConstantWidth obj 2.0))

  ;; add the stringer tags
  (defun add-str ()
    (mapcar
     '(lambda (txt off / txtObj)
        (setq txtObj (add-text blockObj
                               (list (+ x tread_depth) (+ offset off))
                               txt
                               4.0
                               0.5
                               'MiddleCenter))
        (vla-put-Layer txtObj "TEXT")
        (vla-put-StyleName txtObj "Calibri")
        (mirror txtObj)
      ) ;_ lambda
      (list l-string_mat r-string_mat)
      (list
          (if (= ascend "Left") (+ l-string_wid 3) (- width l-string_wid 3))
          (if (= ascend "Right") (+ r-string_wid 3) (- width r-string_wid 3))
        );_ list
    ) ;_ mapcar
  ) ;_ defun add-str

  ;; add clearance arc at top or bottom
  (defun add-arc (tb / topp x pts obj)
    (setq topp (= tb 't)
          x (+ (if topp
                   top_cdist
                   bot_cdist)
               (if (and topp (= break "No")) (+ bot_ext run top_ext) 0))
          pts (fillet-all-pts
               (list (list (- width) 0)
                     (list (- width) width)
                     (list 0 width 0))
               (- width 0.001)))
    ;; add distance between at top
    (if (and topp
             (> dist_btwn 0))
        (setq pts (cons (list (- width) (- dist_btwn) 0)
                        pts)))
    ;; complicated mirror
    (setq obj (add-polyline blockObj
                            (list x offset 0)
                            pts
                            nil))
    (if (and topp (= break "No")) (setq obj (mirror&delete obj (list x offset) 'y)))
    (som obj)
    (vla-put-Color obj 252)
    (vla-put-Layer obj "HIDDEN"))

  ;;; draw bolt centerlines
  (defun tv-stair-bolt ( / bot-cen top-cen max-len)
    (setq max-len (+ bot_ext bot_cdist run top_cdist top_ext))
    (if (and bot_hbolt bot_vbolt (member break '("No" "Bot")))
      (progn
        (setq bot-cen (list bot_hbolt (- width bot_hbolt)))
        (if bot_hbolt2
          (append! 'bot-cen
            (mapcar '+ bot-cen (list bot_hbolt2 (- bot_hbolt2)))
          ) ;_ append!
        ) ;_ if
        (mapcar!
         '(lambda (y) (list (list 0 (+ y offset) 0) dir))
         'bot-cen
        ) ;_ mapcar!
      ) ;_ progn
    ) ;_ if
    (if (and top_hbolt (member break '("No" "Top")))
      (progn
        (setq top-cen (list top_hbolt (- width top_hbolt)))
        (if top_hbolt2
          (append! 'top-cen
            (mapcar '+ top-cen (list top_hbolt2 (- top_hbolt2)))
          ) ;_ append!
        ) ;_ if
        (mapcar!
         '(lambda (y)
            (list
              (list
                (if (= break "No") (* dir max-len) 0)
                (+ y offset)
                0
              ) ;_ list
              (if (= break "No") (- dir) dir)
            ) ;_ list
          ) ;_ lambda
         'top-cen
        ) ;_ mapcar
      ) ;_ progn
    ) ;_ if
    (mapcar
     '(lambda (v)
        (som
          (vla-put-layer-r
            (add-polyline
              blockobj
              (mapcar '* (list dir 1 0) (car v))
              (list '(0 0 0) (list (* dir (cadr v)) 0 0))
              nil
            ) ;_ add-polyline
            "center"
          ) ;_ vlax-put-property-r
        ) ;_ som
      ) ;_ lambda
      (append bot-cen top-cen)
    ) ;_ mapcar
  ) ;_ defun tv-stair-bolt

  ;; ascertain which quadrant to move to
  (setq quadrant (if (= ascend "Right")
                     (if (= z_pos "Far")
                         1
                       3)
                   (if (= z_pos "Far")
                       2
                     4)))
  (add-to-list 'data (list "quadrant" quadrant))
  ;; determine inside/outside stringers
  (mapcar 'set
          (if (= ascend "Left")
              '(istrwid
                istrtex
                istrxtex
                istrbex
                istrxbex
                ostrwid
                ostrtex
                ostrxtex
                ostrbex
                ostrxbex)
            '(ostrwid
              ostrtex
              ostrxtex
              ostrbex
              ostrxbex
              istrwid
              istrtex
              istrxtex
              istrbex
              istrxbex))
          (list
           l-string_wid
           l-string_tex
           l-string_xtra_tex
           l-string_bex
           l-string_xtra_bex
           r-string_wid
           r-string_tex
           r-string_xtra_tex
           r-string_bex
           r-string_xtra_bex))
  (setq ext (max (- istrbex istrxbex)
                 (- ostrbex ostrxbex)))

  (if (= break "No")
      (draw-stair-top-view-full)
    (draw-stair-top-view-short))
  
  (setvar 'clayer clayer)
  (setq blockObj (insert-block modelSpace ins blockObj))
  (add-to-list 'data (list "break" (lowercase break)))
  (add-to-list 'data (list "offset" offset))
  (assign-xdata blockObj "StairTop" (format-xdata data))
  (vla-put-Layer blockObj (strcat z_pos "-stair")))

(defun draw-stair-top-view-full ( / x y)
  (setvar 'clayer "SOLID")
  ;; inside stringer
  (som (add-rectangle blockObj
                      (+ istrbex bot_cdist run top_cdist istrtex)
                      istrwid
                      (list (- 0 istrxbex) offset 0)))
  ;; outside stringer
  (som (add-rectangle blockObj
                      (+ ostrbex bot_cdist run top_cdist ostrtex)
                      ostrwid
                      (list (- 0 ostrxbex) (+ (- width ostrwid) offset) 0)))
  ;; risers
  (setq x (+ bot_ext bot_cdist)
        y (- width istrwid ostrwid))
  (repeat rise_qty
          (som (add-polyline blockObj
                             (list x (+ istrwid offset) 0)
                             (list '(0 0 0)
                                   (list 0 y 0))
                             nil))
          (setq x (+ x tread_depth)))
  ;; clear arcs
  (if (= show_arc "1")
      (progn
        (add-arc 'b)
        (add-arc 't)))
  (add-tag)
  (add-str)
  (tv-stair-bolt)
  )

(defun draw-stair-top-view-short ( / end x y ix ox)
  (mapcar 'set
          '(ix ox)
          (if (= break "Top")
              (list (- istrtex istrxtex)
                    (- ostrtex ostrxtex))
            (list (- istrbex istrxbex)
                  (- ostrbex ostrxbex))))
  (setq end (+ (max ix ox) bot_cdist (* 2 tread_depth) (- tread_depth 3)))
  (setvar 'clayer "SOLID")
  ;; inside stringer
  (som (add-rectangle blockObj
                      (+ (if (= break "Top") istrxtex istrxbex) end)
                      istrwid
                      (list (- 0 (if (= break "Top") istrxtex istrxbex)) offset 0)))
  ;; outside stringer
  (som (add-rectangle blockObj
                      (+ (if (= break "Top") ostrxtex ostrxbex) end)
                      ostrwid
                      (list (- 0 (if (= break "Top") ostrxtex ostrxbex)) (+ (- width ostrwid) offset) 0)))
  ;; risers
  (setq x (if (= break "Top")
              (+ top_ext top_cdist)
            (+ bot_ext bot_cdist))
        y (- width istrwid ostrwid))
  (repeat 3
          (som (add-polyline blockObj
                             (list x (+ offset istrwid) 0)
                             (list '(0 0 0)
                                   (list 0 y 0))
                             nil))
          (setq x (+ x tread_depth)))
  ;; breakline
  (setq y (/ width 2.0))
  (setvar 'clayer "DIM")
  (setq obj (add-polyline blockObj
                          (list end offset 0)
                          (list '(0 0 0)
                                (list 0 (- y 1.5) 0)
                                (list (+ 0 2)
                                      (- y 0.75)
                                      0)
                                (list (- 0 2)
                                      (+ y 0.75)
                                      0)
                                (list 0 (+ y 1.5) 0)
                                (list 0 width 0))
                          nil))
  ;; clearance arc
  (if (= show_arc "1")
      (add-arc (if (= break "Top") 't 'b)))
  (if (or (member quadrant '(2 3))
          (= break "Top"))
      (progn
        (setq tmp
              (vla-Mirror obj
                          (vlax-3d-point end 0 0)
                          (vlax-3d-point end 1 0)))
        (vla-Delete obj)
        (setq obj tmp)))
  (mirror obj)
  (add-tag)
  (add-str)
  (tv-stair-bolt))
