(set-file-docstring "Functions related to stair tread/riser assemblies")

(psc-include
 '(
   "shapes/line.lsp"
   "stair/stringer.lsp"
   ))

(defun-q calc-nose-pts ( / dir-op last-pt pt nose-pts)
  "Returns a list of nosing points relative to stair insert"
  (declare with-data
           (vars (dir nil (numberp dir))
                 (tread_depth nil (numberp tread_depth))
                 (rise_hgt nil (numberp rise_hgt))
                 (bot_cdist nil (numberp bot_cdist))
                 (bot_ext nil (numberp bot_ext))))
  (setq dir-op (if (> dir 0) + -)
        disp (list (dir-op tread_depth) rise_hgt)
        last-pt (list (dir-op (+ bot_cdist bot_ext)) rise_hgt)
        nose-pts (list last-pt))
  (repeat (1- rise_qty)
    (setq pt (displace-pt last-pt disp))
    (add-to-list 'nose-pts pt)
    (setq last-pt pt))
  nose-pts)
(defun-r 'calc-nose-pts)

(defun-q add-stair-qtread (stair-block layer / nos_lst)
  "Adds Quiet Tread to checker plate stair treads in STAIR-BLOCK on LAYER

Note: main treads only. See %chkr-haunch-common and %add-qt for haunches."
  (declare with-data
           (vars (stair-block vla-object)
                 (layer str)))
  (setq nos_lst (remove-last (calc-nose-pts)))
  (mapcar '(lambda (nos-pt / stn_dpt)
            (setq stn_dpt (if (or (not (stair-stn-p))
                                  (and (= stn_tb_only "1") (not (= nos-pt (car nos_lst)))))
                              1.5
                              3.5))
            (vla-put-layer-r
             (apply 'add-rectangle
              (list stair-block
                    (* dir (- tread_depth stn_dpt))
                    -0.125
                    (mapcar '+ nos-pt (list (* dir stn_dpt) (- pan_thick)))))
             layer))
          nos_lst))
(defun-r 'add-stair-qtread)

(defun-q add-stair-conc (stair-block / add conc-lines nose-pts)
  "Adds concrete tread lines to STAIR-BLOCK"
  (declare with-data
           (vars (stair-block vla-object)))
  ;; used here to draw the entities
  (defun add (nose-pt conc-line / closed-p)
    (setq closed-p (pop! 'conc-line))
    (vla-put-color-r (add-polyline stair-block nose-pt conc-line closed-p) 252))

  (setq conc-lines (attempt-apply (read (strcat "calc-conc-lines--" (substr tread_style 1 3)))
                                  nil)
        nose-pts (if (= starter_p "0")
                     (cons '(0.0 0.0) (calc-nose-pts))
                     (calc-nose-pts)))
  (mapcar 'add nose-pts conc-lines))
(defun-r 'add-stair-conc)

(defun-q calc-conc-lines--add-line (x1 x2 / y)
  "Adds a line to CONC-LINES

x1 and x2 are relative to the nosing point"
  (declare (vars (x1 real)
                 (x2 real))
           with-data)
  (if (> x2 x1)
      (add-to-list 'conc-lines
                   (#! '(nil
                         (#(* dir x1) 0)
                         (#(* dir x2) 0))))))
(defun-r 'calc-conc-lines--add-line)

(defun-q calc-conc-lines--add-line-mid (x)
  "Adds a line to CONC-LINES

X is relative to the nosing point."
  (declare (vars (x real))
           with-data)
  (calc-conc-lines--add-line x (+ tread_depth k_dist)))
(defun-r 'calc-conc-lines--add-line-mid)

(defun-q calc-conc-lines--100 (/ x1-off k_dist conc-lines add add-mid)
  "Returns concrete lines for 100 series

Called by %add-stair-conc"
  (declare with-data)
  (setq add calc-conc-lines--add-line
        add-mid calc-conc-lines--add-line-mid
        x1-off (?? stn_len 0.75)
        k_dist (if (= kickback "0") 0.0 0.75))
  (if (= starter_p "0")
      (add 0.0 (+ bot_cdist bot_ext k_dist)))
  (if (> rise_qty 1)
      (add-mid x1-off))
  (if (and (= "1" stn_tb_only)
           (> rise_qty 1))
      (progn
        (repeat (- rise_qty 3)
          (add-mid 0.75))
        (add-mid 0.75))
      (repeat (- rise_qty 2)
        (add-mid x1-off)))
  (add x1-off (+ top_cdist top_ext))
  conc-lines)
(defun-r 'calc-conc-lines--100)

(defun-q calc-conc-lines--400 (/ nrt-pts add-line conc-lines)
  "Returns concrete lines for 400 series

Called by %add-stair-conc"
  (declare with-data)
  (setq nrt-pts (cons T
                  (map-apply
                   '(lambda (x y) (list (* dir x) y))
                   (#! '((0.25 #(- tread_thick))
                         (11.375 #(- tread_thick))
                         (11.625 0.0)
                         (0.0 0.0)))))
        add-line calc-conc-lines--add-line)
  (if (= starter_p "0")
      (add-line 0.0 (+ bot_cdist bot_ext 0.6875)))
  (repeat (1- rise_qty)
          (add-to-list 'conc-lines nrt-pts))
  (add-line (?? stn_len 0.75) (+ top_cdist top_ext))
  conc-lines)
(defun-r 'calc-conc-lines--400)

(defun-q calc-stair-pans (/ tread-style k_dist oset pans ind-lst
			    inc-ind-lst add-ang add-off)
  "Returns a list of stair pans"
  (declare with-data)
  ;; used by calc-stair-pans-- function
  (defun inc-ind-lst (n)
    (setq ind-lst (cons-ind-lst n (if ind-lst (1+ (last ind-lst))))))
  (defun add-ang (pan / ang)
    (setq ang (?? (xd-value "ang" pan) ang-default))
    (subst-key "def"(cons ang (xd-value "def" pan)) pan))
  (defun add-off (pan)
    (if (assoc "off" pan)
        pan
        (cons (list "off" off-default) pan)))

  (setq tread-style (read (substr tread_style 1 1))
        k_dist (cond ((= "0" kickback) 0.0)
                     ((= tread-style 1) 0.75)
		     ((= tread-style 2) 0.5)
		     ((= tread-style 4) 0.6875)
                     (T 0.0))
        oset ((lambda (/ ang)
                (setq ang (angle '(0 0)
                                 (list (- rise_hgt (if (= tread-style 2) pan_thick 0.0))
                                       k_dist)))
                (+ (/ (?f pan_thick) (cos ang))
                   (?f (if (member tread-style '(2 4))
                           (* pan_thick (tan ang)))))))
        pans (attempt-apply (read (strcat "calc-stair-pans--" (substr tread_style 1 3))) nil)
        pans (vl-remove-if '(lambda (l) (< (xd-value "qty" l) 1)) pans))
  (calc-stair-pans--add-defaults pans))
(defun-r 'calc-stair-pans)

(defun-q calc-stair-pans--add-defaults (pans / add-length add-thk add-mat add-pts add-wid)
  "Adds default key values to pans that were omitted during initial calc"
  (defun add-length (pan / len)
    (setq len (- btwn_stringers 0.0625))
    (append (#! '(("len" #len)
                  ("length" #(frac len))))
            pan))
  (defun add-thk (pan)
    (if (assoc "thk" pan)
        pan
        (add-to-list 'pan (list "thk" pan_thick))))
  (defun add-mat (pan / mat-elt mat-str)
    (setq mat-elt (cadr (assoc "mat" pan))
          mat-str (or* (list mat-elt
                             (strcat (if (= tread-style 2)
                                         "CHECKER PLATE "
                                         "PLATE ")
                                     (uppercase pan_mat)))))
    (cons (list "desc" mat-str)
          (if mat-elt
              pan
              (cons (list "mat" mat-str) pan))))
  (defun add-desc (pan)
    (cons (list "desc" (xd-value "mat" pan))
          pan))
  (defun add-pts (pan)
    (cons (list "pts" (apply (read (strcat "calc-stair-pans--" (xd-value "sty" pan)))
                             (xd-value "def" pan)))
          pan))
  (defun add-pts-x2 (pan / def)
    (setq def (xd-value "def" pan)
          def (cons (car def)
                    (mapcar '(lambda(len) (* 2 len))
                            (cdr def))))
    (cons (list "pts-x2"
                (apply (read (strcat "calc-stair-pans--" (xd-value "sty" pan))) def))
          pan))
  (defun add-width (pan / sty def pts i width tread)
    (cons (list "width" (calc-stair-pans--width pan)) pan))
  (mapcar (compose
           (if fab-p
               '(add-width add-pts-x2 add-pts add-mat add-thk add-length)
               '(add-pts add-thk)))
          pans))
(defun-r 'calc-stair-pans--add-defaults)

(defun calc-stair-pans--width (pan / sty def pts mat i len deduct)
  (setq sty (xd-value "sty" pan)
        def (xd-value "def" pan)
        pts (apply (read (strcat "calc-stair-pans--" sty)) def)
        mat (xd-value "mat" pan)
        i 1
        len 0
        deduct (* (1- (strlen sty))
                  (cond ((wcmatch mat "*14GA") 0.18)
                        ((wcmatch mat "*12GA") 0.22)
                        ((wcmatch mat "*3/16") 0.38)
                        ((wcmatch mat "*1/4") 0.5))))
  (while (< i (length pts))
    (setq len (+ len (distance (nth (1- i) pts)
                               (nth i pts))))
    (inc! 'i))
  (rtos (- len deduct) 2 2))

(defun-q calc-stair-pans--h (a h1)
  "Returns vertices for an h style pan"
  (declare (vars (a nil (numberp a))
                 (h1 nil (numberp h1))))
  (list (list h1 0) '(0 0)))
(defun-r 'calc-stair-pans--h)

(defun-q calc-stair-pans--v (a v1)
  "Returns vertices for a v style pan"
  (declare (vars (a nil (numberp a))
                 (v1 nil (numberp v1))))
  (list (list (* (tan a) v1)
              (- v1))
        '(0 0)))
(defun-r 'calc-stair-pans--v)

(defun-q calc-stair-pans--hv (a h1 v1)
  "Returns vertices for an hv style pan"
  (declare (vars (a nil (numberp a))
                 (h1 nil (numberp h1))
                 (v1 nil (numberp v1))))
  (list (list (- h1)
              (- v1))
        (list (* (tan a) v1)
              (- v1))
        '(0 0)))
(defun-r 'calc-stair-pans--hv)

(defun-q calc-stair-pans--vh (a v1 h1)
  "Returns vertices for a vh style pan"
  (declare (vars (a nil (numberp a))
                 (v1 nil (numberp v1))
                 (h1 nil (numberp h1))))
  (list (list (* (tan a) v1)
              (- v1))
        '(0 0)
        (list h1 0)))
(defun-r 'calc-stair-pans--vh)

(defun-q calc-stair-pans--hvh (a h1 v1 h2)
  "Returns vertices for an hvh style pan"
  (declare (vars (a nil (numberp a))
                 (h1 nil (numberp h1))
                 (v1 nil (numberp v1))
                 (h2 nil (numberp h2))))
  (list (list (- h1)
              (- v1))
        (list (* (tan a) v1)
              (- v1))
        '(0 0)
        (list h2 0)))
(defun-r 'calc-stair-pans--hvh)

(defun-q calc-stair-pans--vhv (a v1 h1 v2)
  "Returns vertices for a vhv style pan"
  (declare (vars (a nil (numberp a))
                 (v1 nil (numberp v1))
                 (h1 nil (numberp h1))
                 (v2 nil (numberp v2))))
  (list (list (+ (- h1) (* (tan a) v1))
              (- (+ v1 v2)))
        (list (- h1)
              (- v2))
        (list (* (tan a) v2)
              (- v2) 0)
        '(0 0)))
(defun-r 'calc-stair-pans--vhv)

(defun-q calc-stair-pans--vhvh (a v1 h1 v2 h2)
  "Returns vertices for a vhvh style pan"
  (declare (vars (a nil (numberp a))
                 (v1 nil (numberp v1))
                 (h1 nil (numberp h1))
                 (v2 nil (numberp v2))
                 (h2 nil (numberp h2))))
  (list (list (+ (- h1) (* (tan a) v1))
              (- (+ v1 v2)))
        (list (- h1)
              (- v2))
        (list (* (tan a) v2)
              (- v2))
        '(0 0)
        (list h2 0)))
(defun-r 'calc-stair-pans--vhvh)

(defun-q calc-stair-pans--100 (/ ang-default vhvh-def)
  "Returns a list of 100 series pans

Called by %calc-stair-pans

This function just sets some defaults and calls either
%calc-stair-pans--100-no-stn or %calc-stair-pans--100-stn"
  (declare with-data)
  (defun vhvh-def (height)
    (list 0.75
          (- tread_depth oset (* tread_thick (tan ang-default)))
          height
          0.75))
  (mapcar 'add-ang
          (if (= stn_style "None")
              (progn
                (setq ang-default (angle '(0 0) (list rise_hgt k_dist)))
                (calc-stair-pans--100-no-stn))
              (progn
                  (setq ang-default (angle '(0 0) (list (- rise_hgt stn_thick) k_dist)))
                  (calc-stair-pans--100-stn)))))
(defun-r 'calc-stair-pans--100)

(defun-q calc-stair-pans--100-no-stn (/ off-default)
  "Returns a list of 100 series pans without STN

Called by %calc-stair-pans--100"
  (declare with-data)
  (setq off-default '(0 0))
  (mapcar 'add-off
          (append
           (if (= starter_p "1")
               (#! '((("qty" 1)
                      ("sty" "vh")
                      ("ind" #(inc-ind-lst 1))
                      ("def" (#rise_hgt 0.75)))))
             (#! '((("qty" 1)
                    ("sty" "hvh")
                    ("ind" #(inc-ind-lst 1))
                    ("def" #(list (+ bot_cdist
                                     (if (= "5" bot_conn_det) 0.0 bot_ext)
                                     (if (= bot_conn_num "33")
                                         -0.5
                                       0.0))
                                  (+ rise_hgt
                                     (if (= bot_conn_num "31")
					 (- bot_blkout pan_thick 0.25)
                                       tread_thick))
                                  0.75))))))
           (#! '((("qty" #(1- rise_qty))
                  ("sty" "vhvh")
                  ("ind" #(inc-ind-lst (1- rise_qty)))
                  ("def" #(vhvh-def (+ rise_hgt tread_thick))))))
           (if (= top_conn_det "2")
               (#! '((("qty" 1)
                      ("sty" "vh")
                      ("off" #(list (+ oset (* tread_thick (tan ang-default)))
                                    (- tread_thick)))
                      ("ind" #(list (1- rise_qty)))
                      ("def" #(list 0.75
                                    (- (+ top_cdist top_ext)
                                       (+ oset (* tread_thick (tan ang-default)))
                                     (if (= top_conn_ser "3") 0.5 0.0)))))))))))
(defun-r 'calc-stair-pans--100-no-stn)

(defun-q calc-stair-pans--100-stn (/ off-default pans)
  "Returns a list of 100 series pans with STN

Called by %calc-stair-pans--100"
  (declare with-data)
  (setq off-default (list 0 (- stn_thick)))
  
  (mapcar 'add-off
          (append
           (if (= starter_p "1")
               (#! '((("qty" 1)
                      ("sty" "vh")
                      ("ind" #(inc-ind-lst 1))
                      ("def" (#(- rise_hgt stn_thick) 0.75)))))
             (#! '((("qty" 1)
                    ("sty" "hvh")
                    ("ind" #(inc-ind-lst 1))
                    ("def" (#(+ bot_cdist
                                (if (= bot_conn_det "5") 0.0 bot_ext)
                                (if (= bot_conn_num "33")
                                    -0.5
                                  0.0))
                            #(+ (- rise_hgt stn_thick)
                                (if (= bot_conn_num "31")
                                    (- bot_blkout pan_thick 0.25)
                                  tread_thick))
                            0.75))))))
           (if (= "0" stn_tb_only)
               (#! '((("qty" #(1- rise_qty))
                      ("sty" "vhvh")
                      ("ind" #(inc-ind-lst (1- rise_qty)))
                      ("def" #(vhvh-def (+ rise_hgt tread_thick (- stn_thick)))))))
             (#! '((("qty" #(- rise_qty 2))
                    ("sty" "vhvh")
                    ("off" (0 0))
                    ("ind" #(inc-ind-lst (- rise_qty 2)))
                    ("def" #(vhvh-def (+ rise_hgt tread_thick))))
                   (("qty" #(if (> rise_qty 1) 1 0))
                    ("sty" "vhvh")
                    ("ind" #(list (1- rise_qty)))
                    ("ang" #(angle '(0 0) (list (- rise_hgt stn_thick) k_dist)))
                    ("def" #(vhvh-def (+ rise_hgt tread_thick (- stn_thick))))))))
           (if (= top_conn_det "2")
               (#! '((("qty" 1)
                      ("sty" "vh")
                      ("off" #(list (+ oset (* tread_thick (tan ang-default)))
                                    (- (+ stn_thick tread_thick))))
                      ("ind" #(list (1- rise_qty)))
                      ("def" #(list 0.75
                                    (- (+ top_cdist top_ext)
                                       (+ oset (* tread_thick (tan ang-default)))
                                     (if (= top_conn_ser "3") 0.5 0.0)))))))))))
(defun-r 'calc-stair-pans--100-stn)

(defun-q calc-stair-pans--200 (/ ang-default)
  "Returns a list of 200 series pans

Called by %calc-stair-pans

This function just sets some defaults and calls either
%calc-stair-pans--200-no-stn or %calc-stair-pans--200-stn"
  (declare with-data)
  (setq ang-default (angle '(0 0) (list rise_hgt k_dist)))
  (mapcar 'add-ang
          (if (= stn_style "None")
              (calc-stair-pans--200-no-stn)
              (calc-stair-pans--200-stn))))
(defun-r 'calc-stair-pans--200)

(defun-q calc-stair-pans--200-stn (/ stn-hang x-off off-default)
  "Returns a list of 200 series pans with STN

Called by %calc-stair-pans--200

Attaches the results of %calc-stair-pans--200-sub-nosings to the regular pans it
calculates."
  (declare with-data)
  (setq stn-hang 0.375
        x-off (* (tan ang-default) (* stn-hang (cos ang-default)))
        off-default (list x-off (- stn-hang)))
  (mapcar 'add-off
          (append
           (if (and (= starter_p "1")
                    (not (= 227 (read bot_conn_full))))
               (#! '((("qty" 1)
                      ("sty" "v")
                      ("ind" #(inc-ind-lst 1))
                      ("def" (#(- rise_hgt stn-hang))))))
             (#! '((("qty" #(if (= bot_conn_det "5") 1 0))
                    ("sty" "h")
                    ("off" (-1.25 #(- rise_hgt)))
                    ("ind" (0))
                    ("def" (#(+ (cond ((not (member bot_conn_ser '("2" "5")))
                                       0.0)
                                      ((= stn_style "STNS") -0.25)
                                      ((= stn_style "STNRS") -0.75))
                                (- bot_cdist)
                                (- bot_ext)
                                1.25))))
                   (("qty" 1)
                    ("sty" "hv")
                    ("ind" #(inc-ind-lst 1))
                    ("def" #(list (+ x-off
                                     bot_cdist
                                     (if (= "5" bot_conn_det)
                                         0.0
                                       (+ bot_ext
                                          (cond
                                           ((not (member bot_conn_ser '("2" "5")))
                                            0.0)
                                           ((= stn_style "STNS") 0.25)
                                           ((= stn_style "STNRS") 0.75)))))
                                  (if (= bot_conn_num "31")
                                      (+ rise_hgt
                                         (- bot_blkout (+ 0.25 pan_thick) stn-hang))
                                    (- rise_hgt stn-hang))))))))
           (if (or (< rise_qty 3)
                   (= "0" stn_tb_only))
               (#! '((("qty" #(1- rise_qty))
                      ("sty" "hv")
                      ("ind" #(inc-ind-lst (1- rise_qty)))
                      ("def" #(list (+ x-off
                                       (- tread_depth
                                          (if (= stn_style "STNS") 1.5 2.0)))
                                    (- rise_hgt stn-hang))))))
	     (rcons
	      (mapcar '(lambda(pan)
			 (add-to-list 'pan
				      (list "ang" (angle '(0 0)
							 (list rise_hgt 
                                                               (- k_dist pan_thick))))))
		      (#! '((("qty" 1)
			     ("sty" "hvh")
			     ("off" #(list oset (- pan_thick)))
			     ("ind" #(inc-ind-lst 1))
			     ("def" #(list (+ tread_depth
					      oset
					      (if (= stn_style "STNS") -1.5 -2.0))
					   (- rise_hgt pan_thick)
					   0.75)))
			    (("qty" #(- rise_qty 3))
			     ("sty" "vhvh")
			     ("off" #(list oset (- pan_thick)))
			     ("ind" #(inc-ind-lst (- rise_qty 3)))
			     ("def" #(list 0.75
					   (+ tread_depth oset)
					   (- rise_hgt pan_thick)
					   0.75))))))
	      (#! '(("qty" 1)
		    ("sty" "vhv")
		    ("ind" #(if (> rise_qty 3) (inc-ind-lst 1) '(2)))
		    ("def" #(list 0.75
				  (+ tread_depth x-off)
				  (- rise_hgt stn-hang)))))))
           (if (and (or (member top_conn_det '("2" "4"))
                        (= top_conn_ser "X"))
                    (> (+ top_ext top_cdist)
                       (if (= stn_style "STNS") 1.5 2.0)))
               (#! '((("qty" 1)
		      ("sty" "h")
		      ("off" #(list (if (= stn_style "STNS") 1.5 2.0)
                                    (- pan_thick)))
		      ("ind" #(list (1- rise_qty)))
		      ("def" (#(+ (+ top_cdist top_ext)
                                  ;; offset
                                  (cond ((= stn_style "STNS") -1.5)
				        ((= stn_style "STNRS") -2.0))
                                  ;; overhang
                                  (cond ((member top_conn_ser '("3" "X")) 0.0)
				        ((= stn_style "STNS") 0.25)
				        ((= stn_style "STNRS") 0.75)))))))))
           (calc-stair-pans--200-sub-nosings))))
(defun-r 'calc-stair-pans--200-stn)

(defun-q calc-stair-pans--200-sub-nosings ( / tb-only off ret)
  "Returns a list of sub nosing pans for 200 series with STN

Called by %calc-stair-pans--200-stn"
  (declare with-data)
  (setq off (apply '(lambda (d a) (list (+ (* d (tan a))
                                         (/ d (cos a)))
				   (- d)))
                   (list pan_thick (angle '(0 0) (list rise_hgt k_dist))))
        tb-only (= "1" stn_tb_only)
        ret (#! '(("mat" "PLATE 12GA")
                  ("qty" #(if tb-only (min 1 (1- rise_qty)) (1- rise_qty)))
                  ("sty" "vh")
                  ("off" #off)
                  ("thk" 0.1126)
                  ("ind" #(if tb-only '(0) (cons-ind-lst (1- rise_qty) nil)))
                  ("def" #(if (= stn_style "STNS") '(1.375 2.5) '(1.875 3.0))))))
  (if (member top_conn_num '(20 50))
      (setq ret (rcons (list ret)
                       (#! '(("mat" "PLATE 14GA")
                             ("qty" 1)
                             ("sty" "vh")
                             ("off" #off)
                             ("thk" 0.0817)
                             ("ind" #(list (1- rise_qty)))
                             ("def" #(reverse (xd-value "def" ret)))))))
      (progn
        (subst-key! "qty" (1+ (xd-value "qty" ret)) 'ret)
        (subst-key! "ind" (rcons (xd-value "ind" ret) (1- rise_qty)) 'ret)
        (setq ret (list ret)))))
(defun-r 'calc-stair-pans--200-sub-nosings)

(defun-q calc-stair-pans--200-no-stn (/ off-default)
  "Returns a list of 200 series pans without STN

Called by %calc-stair-pans--200"
  (declare with-data)
  (setq off-default (list oset (- pan_thick)))
  (mapcar 'add-off
          (append
           (if (and (= starter_p "1")
                    (not (= 227 (read bot_conn_full))))
               (#! '((("qty" 1)
                      ("sty" "vh")
                      ("ind" #(inc-ind-lst 1))
                      ("def" (#(- rise_hgt pan_thick) 0.75)))))
             (#! '((("qty" #(if (= bot_conn_det "5") 1 0))
                    ("sty" "h")
                    ("off" #(list -1.25 (- rise_hgt)))
                    ("ind" (0))
                    ("def" (#(+ (if (member bot_conn_ser '("2" "5")) -0.75 0.0)
                                (- bot_cdist)
                                (- bot_ext)
                                1.25))))
                   (("qty" 1)
                    ("sty" "hvh")
                    ("ind" #(inc-ind-lst 1))
                    ("def" #(list (+ oset
                                     bot_cdist
                                     (if (= "5" bot_conn_det)
                                         0.0
                                       (+ bot_ext
                                          (if (member bot_conn_ser '("2" "5")) 0.75 0.0))))
                                  (if (= bot_conn_num "31")
                                      (+ rise_hgt (- bot_blkout (+ 0.25 (* 2.0 pan_thick))))
                                    (- rise_hgt pan_thick))
                                  0.75))))))
           (#! '((("qty" #(1- rise_qty))
                  ("sty" "vhvh")
                  ("ind" #(inc-ind-lst (1- rise_qty)))
                  ("def" #(list 0.75
				(+ tread_depth oset)
				(- rise_hgt pan_thick)
				0.75)))
                 (("qty" 1)
                  ("sty" "vh")
                  ("off" (0 0))
                  ("ind" #(list (1- rise_qty)))
                  ("def" #(list 0.75
				(+ top_cdist
				   top_ext
				   (if (member top_conn_ser '("2" "5")) 0.75 0.0))))
                  ("note" #(if (= top_conn_num "20") "SL" ""))))))))
(defun-r 'calc-stair-pans--200-no-stn)

(defun-q calc-stair-pans--400 (/ off-default ang-default)
  "Returns a list of 400 series stair pans

Called by %calc-stair-pans"
  (declare with-data)
  (setq ang-default (angle '(0 0) (list (- rise_hgt tread_thick) (- k_dist 0.25)))
	off-default (list 0.25 (- tread_thick)))
  (mapcar (compose '(add-off add-ang))
          (append
           ;; bottom
           (#! (cond ((= rise_qty 1) '())
                     ((= starter_p "1")
                      '((("qty" 1)
                         ("sty" "vh")
                         ("ind" #(inc-ind-lst 1))
                         ("def" #(list (- rise_hgt tread_thick)
                                       0.75)))))
                     (T
                      '((("qty" 1)
                         ("sty" "hvh")
                         ("ind" #(inc-ind-lst 1))
                         ("def" #(list (+ bot_cdist
					  0.25
					  (if (= "5" bot_conn_det) 0.0 bot_ext))
				       rise_hgt
				  0.75)))))))
           ;; mid
           (#! '((("qty" #(- rise_qty 2))
                  ("sty" "vhvh")
                  ("ind" #(inc-ind-lst (- rise_qty 2)))
                  ("def" #(list 0.75
                                (- tread_depth oset)
                                (+ rise_hgt pan_thick)
                           0.75)))))
           ;; top
           (#! (cond ((and (= rise_qty 1) (= starter_p "1"))
                      '((("qty" 1)
                         ("sty" "vh")
                         ("off" #(if (stair-stn-p)
                                     '(0.0 -0.125)
                                   '(0.0 0.0)))
                         ("ind" #(inc-ind-lst 1))
                         ("def" #(list (if (stair-stn-p)
                                           (- rise_hgt 0.125)
                                         rise_hgt)
				       (if (= stn_style "STNPT") 1.375 0.75))))))
                     ((= rise_qty 1)
                      '((("qty" 1)
                         ("sty" "hvh")
                         ("off" (0 0))
                         ("ind" (inc-ind-lst 1))
                         ("def" #(list (if (= "5" bot_conn_det)
                                           bot_cdist
                                         (+ bot_cdist bot_ext))
				       (if (stair-stn-p)
                                           (+ rise_hgt tread_thick -0.125)
                                         (+ rise_hgt tread_thick))
				       0.75)))))
                     (T
                      '((("qty" 1)
                         ("sty" "vhvh")
                         ("off" #(cond
                                  ((not (stair-stn-p)) '(0.0 0.0))
                                  ((member stn_style '("STNC3" "STNS")) '(0.0 -0.125))
                                  (T '(0.125 -0.125))))
                         ("ind" #(list (1- rise_qty)))
			 ("ang" #(angle '(0 0) (list (- rise_hgt (?f stn_thick)) k_dist)))
                         ("def" #(list 0.75
				       (- tread_depth 0.25 oset)
				       (+ rise_hgt
					  tread_thick
					  pan_thick
					  (if (stair-stn-p) -0.125 0.0))
				  (if (= stn_style "STNPT") 1.375 0.75)))))))))))
(defun-r 'calc-stair-pans--400)

(defun-q add-stair-treads (stair-block layer / nose-pts plineLst)
  "Adds treads to STAIR-BLOCK on LAYER

This function sets some defaults and calls either %add-stair-treads--grating
or %add-stair-treads--pan"
  (declare (vars (stair-block vla-object)
                 (layer str))
           with-data)
  (setq nose-pts (calc-nose-pts) plineLst '(plineLst))
  (if (begins-with tread_style "600")
      (add-stair-treads--grating)
      (add-stair-treads--pan))
  (cdr plineLst))
(defun-r 'add-stair-treads)

(defun-q add-stair-treads--grating (/ add)
  "Adds grating treads to STAIR-BLOCK

Called by %add-stair-treads"
  (declare (vars (stair-block vla-object)
                 (layer str))
           with-data)
  (defun add (len ins)
    (add-to-list 'plineLst
      (vla-put-layer-r (add-rectangle stair-block (* dir len) -1.25 ins) layer)
    ) ;_ add-to-list
  ) ;_ defun
  (if (= starter_p "0")
      (add (+ bot_ext bot_cdist) '(0 0)))
  (mapcar '(lambda (ins) (add tread_depth ins)) (remove-last nose-pts))
  (add (+ top_ext top_cdist) (last nose-pts)))
(defun-r 'add-stair-treads--grating)

(defun-q add-stair-treads--pan (/ pans add pos)
  "Adds pan treads to STAIR-BLOCK on LAYER

Called by %add-stair-treads"
  (declare (vars (stair-block vla-object)
                 (layer str))
           with-data)
  (defun add (pan / ins obj)
    (if (member pos (xd-value "ind" pan))
        (setq ins (mapcar '* (list dir 1) (displace-pt npt (xd-value "off" pan)))
              obj (add-polyline stair-block ins (dirmirror (xd-value "pts" pan)) nil)
              obj (offset&close obj (* dir (xd-value "thk" pan)) nil nil)
              obj (vla-put-layer-r obj layer)))
    (if obj (add-to-list 'plineLst obj)))
  (setq pos 0
        pans (calc-stair-pans))
  (if (= dir -1) (setq nose-pts (mirror-pts-v nose-pts 0)))
  (foreach npt nose-pts
           (mapcar 'add pans)
           (inc! 'pos)))
(defun-r 'add-stair-treads--pan)

(defun-q add-stair-stn (stair-block layer / nose-pts k_dist ris_ang func)
  "Adds STN to STAIR-BLOCK on LAYER"
  (declare (vars (stair-block vla-object)
                 (layer str))
           with-data)
  (setq nose-pts (calc-nose-pts)
        k_dist (cond ((= "0" kickback) 0.0)
                     ((begins-with tread_style "100") 0.75)
                     ((begins-with tread_style "200") 0.5)
                     (T 0.0))
        ris_ang (angle '(0.0 0.0) (list k_dist (- rise_hgt)))
        func (eval (read (strcat "add-stair-stn--" (substr tread_style 1 3)))))
  (if (= "1" stn_tb_only)
      (setq nose-pts (list (car nose-pts) (last nose-pts))))
  (mapcar '(lambda (ins) (vla-put-layer-r (func ins) layer))
          nose-pts))
(defun-r 'add-stair-stn)

(defun-q add-stair-stn--100 (ins)
  "Adds STN to STAIR-BLOCK at INS for 100 series stair

Called by %add-stair-stn"
  (declare (vars (stair-block vla-object)
                 (ins list (point-p ins)))
           with-data)
  (apply '(lambda (x y) (add-rectangle stair-block x y ins))
         (mapcar '*
                 (list dir 1)
                 (if (= stn_style "STNP")
                     '(2.0 -0.1875)
                     '(3.0 -0.125)))))
(defun-r 'add-stair-stn--100)

(defun-q add-stair-stn--200 (ins)
  "Adds STN to STAIR-BLOCK at INS for 200 series stair

Called by %add-stair-stn"
  (declare (vars (stair-block vla-object)
                 (ins list (point-p ins)))
           with-data)
  (offset&close
   (add-polyline stair-block
                 ins
                 (mapcar '(lambda (x) (mapcar '* x (list dir 1 1)))
                         (list (polar '(0 0 0) ris_ang 0.375)
                               '(0 0 0)
                               (list (if (= stn_style "STNS") 1.5 2.0) 0 0)))
                 nil)
   (* dir pan_thick)
   nil
   nil))
(defun-r 'add-stair-stn--200)

(defun-q add-stair-stn--400 (ins)
  "Adds STN to STAIR-BLOCK at INS for 400 series stair

Called by %add-stair-stn"
  (declare (vars (stair-block vla-object)
                 (ins list (point-p ins)))
           with-data)
  (add-polyline stair-block
                ins
                (mapcar '(lambda (x) (mapcar '* x (list dir 1 1)))
                        (apply '(lambda (x y)
                                 (list '(0.0 0.0 0.0)
                                  (list x 0.0 0.0)
                                  (list x y 0.0)
                                  (if (equal ins (last nose-pts))
                                      (list 0.0 y 0.0)
                                      (line-int
                                       (pt-slope (/ (- tread_thick) 0.25) 0.0 0.0)
                                       (list 0.0 y)
                                       T))))
                               (cond ((= stn_style "STNC3") '(3.0 -0.125))
                                     ((= stn_style "STNS") '(1.5 -0.125))
                                     ((= stn_style "STNRS") '(2.0 -0.125)))))
                T))
(defun-r 'add-stair-stn--400)
