(add-to-list 'sub-dialogs "tread_opts")

(setq
 ;; tell dialog-init which keys are relevant to tread_opts
 tread_opts-popup-keys '("stn_style"
                         "pan_mat"
                         "tread_mat"
                         "stn_tb_nclr"
                         "stn_tb_iclr"
                         "stn_tb_ins1"
                         "stn_tb_ins2"
                         "stn_mid_nclr"
                         "stn_mid_iclr"
                         "stn_mid_ins1"
                         "stn_mid_ins2")
 tread_opts-dist-keys '("tread_thick")
 tread_opts-plain-string-keys '("stn_tb_sku"
                                "stn_mid_sku")
 tread_opts-toggle-keys '("kickback"
                          "quiet_tread"
                          "stn_tb_only")
 tread_opts-keys (append tread_opts-popup-keys
                         tread_opts-dist-keys
                         tread_opts-plain-string-keys
                         tread_opts-toggle-keys)
 ;; data structure describing allowable tread styles
 ;; car is a string with the code and the description.
 ;; cdr is a definition suitable for passing to dialog-sub-update-from-def
 tread-styles '(("100 Pan-Filled"
                 ("stn_style" "None" "STNP" "STNC3")
                 ("pan_mat" . bent-plates)
                 ("tread_thick" 1.5)
                 ("kickback" "1")
                 ("stn_tb_only" "0")
                 ("stn_tb_sku" "")
                 ("stn_mid_sku" ""))
                ("200 Checker Plate"
                 ("stn_style" "None" "STNS" "STNRS")
                 ("pan_mat" . bent-plates)
                 ("quiet_tread" "0")
                 ("kickback" "1")
                 ("stn_tb_only" "0")
                 ("stn_tb_sku" "")
                 ("stn_mid_sku" ""))
                ("400 Precast"
                 ("tread_mat" "NRT Rhino Tred" "Custom")
                 ("pan_mat" . bent-plates)
                 ("stn_style" "None" "STNC3" "STNRS" "STNS")
                 ("tread_thick" 1.5)
                 ("kickback" "1")
                 ("stn_tb_only" "0")
                 ("stn_tb_sku" "")
                 ("stn_mid_sku" ""))
                ("600 Grating"
                 ("tread_mat" . ("19-W-4 x 1 1/4\""
                                 "11-W-4 x 1 1/4\""
                                 "7-W-4 x 1 1/4\""))))
 tread-styles-strings (mapcar 'car tread-styles)

 ;; options associated with each stn style
 stn-styles '(("PM" "ins1")
              ("STNPT" "ins1")
              ("STNS" "nclr" "ins1")
              ("STNRS" "nclr" "ins1")
              ("STNP" "nclr" "iclr" "ins1")
              ("STNC" "nclr" "iclr" "ins1")
              ("STNC3" "nclr" "ins1" "ins2"))
 ;; nosing color
 nclr-opts '("Clear" "Yellow" "Black")
 ;; isolator color
 iclr-opts '("Yellow" "Grey" "Black")
 ;; 1" insert strip
 ins1-opts '("STNF Photo-Glo"
             "Yellow Traction Tape"
             "Black Traction Tape")
 ;; 1.5" insert strip
 ins2-opts '("Yellow Traction Tape"
             "Black Traction Tape"
             "Ribbed Aluminum"))

;; default action for tread_style
(defun tread_style-action (value / tread-def stn-style)
  (setq tread-def (cdr (assoc value tread-styles))
        stn-style (cadr (assoc "stn_style" tread_opts-val)))
  ;; preserve stn opts, which are not automatically present in tread-def
  (if (/= stn-style "None")
      (foreach
       stn (cdr (assoc stn-style stn-styles))
       (if (vl-some '(lambda(x) (ends-with (car x) stn)) tread_opts-val)
           (foreach tbm '("mid_" "tb_")
                    (add-to-list 'tread-def (cons (strcat "stn_" tbm stn)
                                                  (read (strcat stn "-opts"))))))))
  (dialog-sub-update-from-def "tread_opts" tread-def)
  ;; keys need cleaning up if we went from some stn to "None"
  (if (= (cadr (assoc "stn_style" tread_opts-val)) "None")
      (remove! 'tread_opts-val '(and (wcmatch (car x) "*stn*")
                                     (/= (car x) "stn_style"))))
  ;; if we switched to 400 series, ensure kickback is on
  (if (begins-with value "4") (subst-key! "kickback" "1" 'tread_opts-val))
  ;; handle bolt depth difference for 200 series
  (tread_style-set-bolt-depth))

;; set bolt depth to 5.125 for 200 series and 5 for others
(defun tread_style-set-bolt-depth ( / dpt key)
  (setq dpt (if (begins-with value "2") 5.125 5.0))
  (foreach tb '("top" "bot")
           (setq key (strcat tb "_conn"))
           (if (member (substr (dialog-tile-val key) 1 1) '("2" "5"))
               (subst-key! (strcat tb "_vbolt")
                           dpt
                           (read (strcat tb "_conn_opts-val"))))))

;; Initialize tread_opts sub-dialog.
;; This function is assigned automatically as the action for
;; tread_opts button.
;; It takes a dummy arg because the auto-action includes the key
;; which triggered it, which this function does not need.
(defun tread_opts-sub-init (dummy / tread-style local-scope )
  (setq tread-style (dialog-popup-val "tread_style"))

  ;; define symbols for dialog-init
  (defun local-scope ( / tmp
                         tread-def
                         popup-keys
                         dist-keys
                         int-keys
                         plain-string-keys
                         toggle-keys
                         tile-actions
                         default-actions
                         dialog-ties
                         sub-dialogs
                         dialog-after-load
                         dialog-before-done
                         stn-opts
                         stn_style-action
                         upd8-stn-group
                         kill-stn-opt
                         recalc-skus)

    (defun dialog-after-load ()
      ;; turn on only keys which pertain to this invocation of the dialog
      (foreach key all-keys
               (mode_tile key 0))
      (if (not (begins-with tread-style "6"))
          (progn
            ;; enable correct STN options
            (stn_style-action (dialog-tile-val "stn_style"))
            ;; turn off mids if necessary
            (stn_tb_only-action (dialog-tile-val "stn_tb_only"))
            ;; populate STN options lists
            (foreach elt (vl-remove-if-not '(lambda (x) (begins-with (car x) "stn_"))
                                           tread_opts-val)
                     (if (member (car elt) popup-keys-strings)
                         (dialog-popup-set (car elt) (cadr elt))))
            (recalc-skus nil nil)
            (mapcar 'dialog-add-default-action '("stn_tb_nclr"
                                                 "stn_tb_iclr"
                                                 "stn_tb_ins1"
                                                 "stn_tb_ins2"
                                                 "stn_mid_nclr"
                                                 "stn_mid_iclr"
                                                 "stn_mid_ins1"
                                                 "stn_mid_ins2"))))
      ;; force kickback at 400 series
      (if (begins-with tread-style "4")
          (progn
            (set_tile "kickback" "1")
            (mode_tile "kickback" 1))))

    (defun stn_style-action (value / )
      (setq
       ;; assess whether we need STN options on load
       stn-opts (if (and (not (= (substr tread-style 1 1) "6"))
                         (not (= value "None")))
                    (cdr (assoc value stn-styles))))

      ;; kill all options until we call upd8-stn-group to reinstate the needed ones
      (mapcar 'kill-stn-opt '("stn_tb_nclr"
                              "stn_tb_iclr"
                              "stn_tb_ins1"
                              "stn_tb_ins2"
                              "stn_mid_nclr"
                              "stn_mid_iclr"
                              "stn_mid_ins1"
                              "stn_mid_ins2"))

      (if (= value "None")
          (progn
            (set_tile "stn_tb_only" "0")
            (mode_tile "stn_tb_only" 1))
        (mode_tile "stn_tb_only" 0))

      (foreach opt stn-opts
               (upd8-stn-group "tb")
               (if (= (dialog-tile-val "stn_tb_only") "0")
                   (upd8-stn-group "mid")))
      );;stn_style-action

    ;; set which STN options are available for a given base model
    (defun upd8-stn-group (tbm / fullopt)
      (add-to-list 'popup-keys (cons (setq fullopt (strcat "stn_" tbm "_" opt))
                                     ;; S9203 only applies to PM model
                                     (if (= value "PM")
                                         (cons "S9203" (vl-symbol-value (read (strcat opt "-opts"))))
                                       (read (strcat opt "-opts")))))
      (mapcar '(lambda (x) (add-to-list x fullopt))
              '(popup-keys-strings all-keys))
      (dialog-popup-list-update fullopt)
      (mode_tile fullopt 0)
      );;upd8-stn-group

    (defun stn_tb_only-action (value / opts)
      (if (= value "1")
          (progn
            (mapcar 'kill-stn-opt '("stn_mid_nclr"
                                    "stn_mid_iclr"
                                    "stn_mid_ins1"
                                    "stn_mid_ins2"))
            (set_tile "stn_mid_sku" ""))
        (progn
          (foreach opt stn-opts
                   (upd8-stn-group "mid"))
          (recalc-skus nil nil)
          (mode_tile "stn_tb_sku" 1)))
      (mode_tile "stn_mid_sku" 1))

    ;; turn off an option and remove it from key lists
    (defun kill-stn-opt (key)
      (dialog-popup-list-set key '())
      (mode_tile key 1)
      (remove! 'popup-keys (list '= '(car x) key))
      (remove! 'popup-keys-strings (list '= 'x key))
      (remove! 'all-keys (list '= 'x key)))

    ;; reset full STN model numbers
    (defun recalc-skus (key val / calc style tbm)

      ;; actual calculation performed here
      (defun calc (tbm / nclr iclr ins1 ins2)
        (foreach key stn-opts
           (set (read key) (dialog-tile-val (strcat "stn_" tbm "_" key))))
        (if nclr
            (setq nclr
                  (cond ((= nclr "Clear") "")
                        ((= nclr "Yellow") "Y")
                        ((= nclr "Black") "B"))))
        (if iclr
            (setq iclr
                  (cond ((= iclr "Grey") "G")
                        ((= iclr "Yellow") "Y")
                        ((= iclr "Black") "B"))))
        (if ins1
            (setq ins1
                  (cond ((begins-with ins1 "STNF")
                         (if (= style "PM") "STNF" ""))
                        ((begins-with ins1 "S9203") "9203")
                        ((begins-with ins1 "Yellow") "TY")
                        ((begins-with ins1 "Black") "TB"))))
        (if ins2
            (setq ins2
                  (cond ((begins-with ins2 "Yellow") "TY")
                        ((begins-with ins2 "Black") "TB")
                        ((begins-with ins2 "Ribbed") "RB"))))
        (strcat style
                (cond
                 ((= style "PM") ins1)
                 ((= style "STNPT") ins1)
                 ((or (= style "STNS")
                      (begins-with style "STNRS"))
                  (strcat nclr ins1))
                 ((or (= style "STNP")
                      (= style "STNC"))
                  (strcat iclr nclr ins1))
                 ((= style "STNC3")
                  (strcat nclr
                          (if (= ins2 "RB")
                              (strcat ins2 ins1)
                            (strcat ins1 ins2))))))
        );;calc

      (setq style (dialog-tile-val "stn_style")
            tbm '("tb"))
      (if (= (dialog-tile-val "stn_tb_only") "0")
          (add-to-list 'tbm "mid"))
      (foreach x tbm
               (setq key (strcat "stn_" x "_sku"))
               (if (= style "None")
                   (set_tile key "")
                 (set_tile key (calc x)))
               (mode_tile key 1)))

    (setq
     tread-def (cdr (assoc tread-style tread-styles))
     popup-keys (remove tread-def '(not (member (car x) tread_opts-popup-keys)))
     dist-keys (mapcar 'car (remove tread-def '(not (member (car x) tread_opts-dist-keys))))
     plain-string-keys (mapcar 'car (remove tread-def '(not (member (car x) tread_opts-plain-string-keys))))
     toggle-keys (mapcar 'car (remove tread-def '(not (member (car x) tread_opts-toggle-keys))))
     default-actions '(("stn_style" . recalc-skus)
                       ("stn_tb_nclr" . recalc-skus)
                       ("stn_tb_iclr" . recalc-skus)
                       ("stn_tb_ins1" . recalc-skus)
                       ("stn_tb_ins2" . recalc-skus)
                       ("stn_mid_nclr" . recalc-skus)
                       ("stn_mid_iclr" . recalc-skus)
                       ("stn_mid_ins1" . recalc-skus)
                       ("stn_mid_ins2" . recalc-skus)))
    
    (if (setq tmp (dialog-init "stair/tread_opts.dcl" "tread_opts" tread_opts-val))
        (setq tread_opts-val (vl-remove-if '(lambda (x) (= (cdr x) nil)) tmp))))

  (local-scope))

