(set-file-docstring
 "Aliases and wrappers")

(setq ename>vlobj vlax-ename->vla-object
      vlobj>ename vlax-vla-object->ename
      3dpt vlax-3d-point)

(defun-q objectify (enames)
  "Return ENAMES as VLA objects"
  (mapcar 'ename>vlobj enames))
(defun-r 'objectify)

(defun-q vlax-put-property-r (obj pro val)
  "Wrapper for vlax-put-property that returns OBJ"
  (declare (vars (obj vla-object)
                 (pro (str sym))))
  (vlax-put-property obj pro val)
  obj)
(defun-r 'vlax-put-property-r)

(defun-q vla-put-layer-r (obj lay)
  "Wrapper for vla-put-layer that returns OBJ"
  (declare (vars (obj vla-object)
                 (lay str)))
  (vla-put-Layer obj lay)
  obj)
(defun-r 'vla-put-layer-r)

(defun-q vla-put-color-r (obj col)
  "Wrapper for vla-put-color that returns OBJ"
  (declare (vars (obj vla-object)
                 (col int (>= col 0) (<= col 256))))
  (vla-put-color obj col)
  obj)
(defun-r 'vla-put-color-r)

(defun-q vla-rotate-r (obj ins rot)
  "Wrapper for vla-rotate that returns OBJ"
  (declare (vars (obj vla-object)
                 (ins variant)
                 (rot nil (numberp rot))))
  (vla-rotate obj ins rot)
  obj)
(defun-r 'vla-rotate-r)

(defun-q vla-rotate3d-r (obj ax1 ax2 rot)
  "Wrapper for vla-rotate3d that returns OBJ"
  (declare (vars (obj vla-object)
                 (ax1 variant)
                 (ax2 variant)
                 (rot nil (numberp rot))))
  (vla-rotate3d obj ax1 ax2 rot)
  obj)
(defun-r 'vla-rotate3d-r)

(defun-q vla-transformby-r (obj mat)
  "Wrapper for vla-transformby that returns OBJ"
  (declare (vars (obj vla-object)
                 (mat variant)))
  (vla-transformby obj mat)
  obj)
(defun-r 'vla-transformby-r)

(defun-q vla-boolean-r (ob1 bop ob2)
  "Wrapper for vla-boolean that returns OB1"
  (declare (vars (ob1 vla-object)
                 (bop nil (member bop (list acUnion acIntersection acSubtraction)))
                 (ob2 vla-object)))
  (vla-boolean ob1 bop ob2)
  ob1)
(defun-r 'vla-boolean-r)

(defun-q vla-put-normal-r (obj nor)
  "Wrapper for vla-put-normal that returns OBJ"
  (declare (vars (obj vla-object)
                 (nor variant)))
  (vla-put-normal obj nor)
  obj)
(defun-r 'vla-put-normal-r)

(defun-q vla-logic (e)
  "Returns :vlax-true or :vlax-false based on truthiness of E"
  (if e :vlax-true :vlax-false))
(defun-r 'vla-logic)
