(set-file-docstring
 "File-handling functions")

(defun-q get-files-this-dir (full-path /
                                ;; for dialog-init
                                defaults-file
                                popup-keys
                                dist-keys
                                int-keys
                                toggle-keys
                                plain-string-keys
                                default-actions
                                dialog-ties
                                sub-dialogs
                                extra-keys
                                tile-actions
                                dialog-save-action
                                ;; local
                                files
                                path
                                )
  "Prompts to select files from current directory using dialog listbox

The DCL for this dialog is files-this-dir.dcl."
  (psc-include '("dialog/dialog.lsp"))
  (setq files (vl-sort (vl-directory-files (vla-get-Path acadDoc) "*.dwg" 1) '<)
        popup-keys '(("files" . files))
        path (vla-get-path acaddoc))

  (defun dialog-save-action ()
    (setq data (mapcar '(lambda(x) (nth (atoi x) files))
                       (strsplit (get_tile "files") " ")))
    (done_dialog 0))

  (setq files (dialog-init "files-this-dir.dcl" "files" nil))

  (if full-path
      (mapcar '(lambda(x) (strcat path "/" x)) files)
      files))
(defun-r 'get-files-this-dir)

(defun-q read-file-to-string (file / line ret)
  "Returns the contents of FILE as a string"
  (declare (vars (file str)))
  (setq file (findfile file))
  (if file
      (progn
        (setq file (open file "r"))
        (while (setq line (read-line file))
          (add-to-list 'ret line))
        (close file)))
  (strjoin ret "\n"))
(defun-r 'read-file-to-string)

(defun-q open-csv (file)
  "Opens FILE with Excel"
  (declare (vars (file str)))
  (startapp "C:\\Program Files\\Microsoft Office\\root\\Office16\\EXCEL.EXE"
            (strcat "\"" file "\"")))
(defun-r 'open-csv)

(defun-q read-csv (file / line ret)
  "Return the contents of csv FILE as a list of lists of strings"
  (declare (vars (file str)))
  (setq file (findfile file))
  (if file
      (progn
        (setq file (open file "r"))
        (while (setq line (read-line file))
          (add-to-list 'ret (strsplit line ",")))
        (close file)))
  ret)
(defun-r 'read-csv)

(defun-q write-csv (lines filename wrap-str / file line)
  "Write the contents of LINES to FILENAME

LINES should be a list of lists of strings. FILENAME should be a file name as a string. If
WRAP-STR is non-nil, wrap each cell in a formula like =\"CELL-CONTENT\"."
  (declare (vars (lines list)
                 (filename str)))
  (setq file (open filename "w"))
  (if (not file) (error (strcat "Cannot open " filename)))
  (foreach line lines
    (setq line (strjoin (mapcar '(lambda(s)
                                  (setq s (string-subst-all "" "," (to-string s)))
                                  (if wrap-str
                                      (strcat "=\"" s "\"")
                                      s))
                                line)
                        ","))
    (write-line line file))
  (close file))
(defun-r 'write-csv)

(defun-q read-key-val-file (file / line data)
  "Return the contents of key/value FILE as a list of 2-element lists"
  (declare (vars (file str)))
  (if (setq file (open file "r"))
      (progn
        (while (setq line (read-line file))
          (setq line (strsplit line ":"))
          (add-to-list 'data (list (car line) (cadr line))))
        (close file)))
  data)
(defun-r 'read-key-val-file)

(defun-q process-generic-def (def repo-file / defs filename current found-match-p)
  "Retrieves the unique primary key for DEF in REPO-FILE

DEF should be the definition of an object as a list. REPO-FILE should be the name of a csv
file that serves as the database of that object type. Each unique object definition is
assigned a primary key number in the database. This function returns DEF with the primary
key as the first item.

This function handles creating REPO-FILE and assigning primary keys."
  (declare (vars (def list)
                 (repo-file str)))
  (setq def (mapcar 'to-string def)
        filename repo-file
        defs (read-csv filename)
        current 1)
  (if defs
      (foreach d defs
               (if (and (= (length (cdr d))
                           (length def))
                        (vl-every '= (cdr d) def))
                   (setq def (cons (car d) def)
                         found-match-p T))
               (inc! 'current))
      (progn
        (setq repo-file (open filename "w")
              def (cons "1" def)
              found-match-p T)
        (write-line (strjoin def ",") repo-file)
        (close repo-file)))
  (if (not found-match-p)
      (progn
        (setq repo-file (open filename "a")
              def (cons (to-string current) def))
        (write-line (strjoin def ",") repo-file)
        (close repo-file)))
  def)
(defun-r 'process-generic-def)
