(set-file-docstring
 "Block-handling functions")

(defun-q blockdef (ename)
  "Returns the block definition for insert with ENAME"
  (declare (vars (ename ename)))
  (vla-Item (vla-get-Blocks acadDoc)
            (vla-get-EffectiveName (ename>vlobj ename))))
(defun-r 'blockdef)

(defun-q blockref2def (blockref)
  "Returns the block definition for insert BLOCKREF"
  (declare (vars (blockref vla-object)))
  (vla-Item (vla-get-Blocks acadDoc)
            (vla-get-EffectiveName blockref)))
(defun-r 'blockref2def)

(defun-q blkRef-p (obj)
  "Returns T if OBJ is a block reference"
  (declare (vars (obj vla-object)))
  (= "AcDbBlockReference" (vla-get-objectname obj)))
(defun-r 'blkRef-p)

(defun-q define-block (name plines / blocks blockObj)
  "Returns a block definition named NAME consisting of closed polylines

PLINES is a list of closed polylines inserted at the origin."
  (declare (vars (name str)
                 (plines nil (listp plines) (vl-every 'pts-list-p plines))))
  (setq blocks (vla-get-Blocks acadDoc)
        blockObj (vla-Add blocks (vlax-3d-point 0 0 0) name))
  (foreach pline plines
           (vla-put-Layer (add-polyline blockObj '(0 0 0) pline T) "0"))
  blockObj)
(defun-r 'define-block)

(defun-q define-block-maybe (name plines)
  "Calls %define-block only if no block with NAME exists yet"
  (declare (vars (name str)
                 (plines nil (listp plines) (vl-every 'pts-list-p plines))))
  (if (not (tblsearch "block" name))
      (define-block name plines)))
(defun-r 'define-block-maybe)

(defun-q insert-block (container ins block)
  "Wrapper for vla-InsertBlock

BLOCK may be the block name as a string or a block definition obj. INS may be a point list
or vlax point variant."
  (declare (vars (container vla-object)
                 (ins (list variant nil))
                 (block (str vla-object))))
  (if (/= (type block) 'STR)
      (setq block (vla-get-Name block)))
  (if (listp ins)
      (setq ins (3dpt ins)))
  (vla-InsertBlock container ins block 1 1 1 0))
(defun-r 'insert-block)

(defun-q insert-lib-block (container ins name scale)
  "Inserts a block from custom block library

NAME should be the filename of a drawing from the blocks/ folder without the extension."
  (declare (vars (container vla-object)
                 (ins (list safearray nil))
                 (name str)
                 (scale nil (numberp scale))))
  (vla-AttachExternalReference container
                               (findfile (strcat "blocks/" name ".dwg"))
                               name
                               (3dpt ins)
                               scale scale 1 0 :vlax-false))
(defun-r 'insert-lib-block)

(defun-q block-containing (sourceObj)
  "Searches for the block definition that contains SOURCEOBJ

Returns the block name if found, or nil."
  (declare (vars (sourceobj vla-object)))
  (car
   (vl-remove-if-not
    '(lambda (obj)
      (= (vla-get-ownerid sourceObj) (vla-get-objectid obj)))
    (collection->list (vla-get-Blocks acadDoc)))))
(defun-r 'block-containing)

(defun-q move-to-bottom (obj container / edict sortents sarr)
  "Moves OBJ to bottom of draw order in CONTAINER"
  (declare (vars (obj vla-object)
                 (container vla-object)))
  (setq edict (vla-GetExtensionDictionary container)
        sortents (vl-catch-all-apply 'vla-GetObject (list edict "ACAD_SORTENTS")))
  (if (= (type sortents) 'vl-catch-all-apply-error)
      (setq sortents (vla-AddObject edict "ACAD_SORTENTS" "AcDbSortentsTable")))
  (setq sarr (vlax-make-safearray vlax-vbObject '(0 . 0)))
  (vlax-safearray-put-element sarr 0 obj)
  (vla-MoveToBottom sortents sarr)
  obj)
(defun-r 'move-to-bottom)

(defun-q p-blk-name (str)
  "Returns a unique pseudonymous block name"
  (vl-propagate 'p-blk-name-i)
  (setq p-blk-name-i (1+ (?n p-blk-name-i)))
  (if str
      (setq str (strcat str  "."))
      (setq str ""))
  (strcat str
          (rtos (getvar 'cdate) 2 4)
          "."
          (itoa p-blk-name-i)))
(defun-r 'p-blk-name)

(defun-q p-blk (str)
  "Returns a new empty pseudonymous block"
  (define-block (p-blk-name str) nil))
(defun-r 'p-blk)
