(set-file-docstring
 "Functions pertaining to text, mtext, leader, and mleader objects")

(defun-q vla-replace-string (block from to / name string)
  "Replaces FROM with TO in all text, mtext, and mleaders in BLOCK"
  (declare (vars (block vla-object)
                 (from str)
                 (to str)))
  (vlax-for ent block
            (setq name (vla-get-ObjectName ent))
            (cond ((= name "AcDbBlockReference")
                   (setq name (vla-Item (vla-get-Blocks acadDoc)
                                        (vla-get-EffectiveName ent)))
                   (vla-replace-string name from to))
                  ((or (= name "AcDbText")
                       (= name "AcDbMText")
                       (= name "AcDbMLeader"))
                   (setq string (vla-get-TextString ent))
                   (if (wcmatch string (strcat "*" from "*"))
                       (vla-put-TextString ent
                                           (vl-string-subst to from string)))))))
(defun-r 'vla-replace-string)

(defun-q add-mtext (container ins text width attachmentpoint / obj)
  "Wrapper for vla-AddMText

ATTACHMENTPOINT should be a symbol like 'middlecenter or 'topleft."
  (declare (vars (container vla-object)
                 (ins list (point-p ins))
                 (text str)
                 (width nil (numberp width))
                 (attachmentpoint sym)))
  (setq obj (vla-AddMText container (3dpt 0 0 0) width text))
  (vla-put-AttachmentPoint
   obj
   (vl-symbol-value (read (strcat "acAttachmentPoint" (to-string attachmentpoint)))))
  (vla-put-InsertionPoint obj (3dpt ins))
  obj)
(defun-r 'add-mtext)

(defun-q add-text (container ins text height width alignment / obj)
  "Wrapper for vla-AddMText

ALIGNMENT should be a symbol like 'middlecenter or 'topleft."
  (declare (vars (container vla-object)
                 (ins list (point-p ins))
                 (text str)
                 (height nil (numberp height))
                 (width nil (numberp width))
                 (alignment sym)))
  (setq obj (vla-AddText container text (3dpt ins) height))
  (vla-put-Alignment
   obj
   (vl-symbol-value (read (strcat "acAlignment" (to-string alignment)))))
  (if (wcmatch (lowercase (to-string alignment)) "left")
      (vla-put-InsertionPoint obj (3dpt ins))
    (vla-put-TextAlignmentPoint obj (3dpt ins)))
  (vla-put-ScaleFactor obj width)
  obj)
(defun-r 'add-text)

(defun-q add-mleader (container ins points text / obj)
  "Wrapper for vla-AddMLeader

POINTS is a list of leader points which are relative to INS"
  (declare (vars (container vla-object)
                 (ins list (point-p ins))
                 (points list (vl-every 'point-p points))
                 (text str)))
  (mapcar! '(lambda(lst) (list (+ (car lst) (car ins))
                               (+ (cadr lst) (cadr ins))
                               (+ (last lst) (last ins))))
           'points)
  (setq obj (vla-AddMLeader container (vlist->safearray points) 0))
  (vla-put-TextString obj text)
  (vla-put-TextHeight obj (vla-getvar 'textsize))
  (if (< (car (last points))
         (car (circular-nth -2 points)))
      (progn
        (vla-setdoglegdirection obj 0 (3dpt -1 0))
        (vla-setleaderlinevertices obj 0 (vlist->safearray points))
        (vla-put-TextJustify obj acAttachmentPointMiddleRight)))
  obj)
(defun-r 'add-mleader)
