(set-file-docstring
 "Time- and date-related functions")

(defun-q curdate ( / cdate)
  "Returns the current date formatted as DD/MM/YYYY"
  (setq cdate (rtos (getvar 'cdate) 2 6))
  (strcat (substr cdate 5 2)
          "/"
          (substr cdate 7 2)
          "/"
          (substr cdate 3 2)))
(defun-r 'curdate)

(defun-q !time (func alert-p / t1 t2 dur str ret)
  "Prints the number of milliseconds it takes to run FUNC

FUNC should be a lisp form to be eval'd. If ALERT-P is nil, prints to the command
line. Otherwise, displays an alert box with the result.

Example: (!time '(repeat 100 (read-xdata (entlast) \"\")) nil)"
  (declare (vars (func list (function-p (eval (car func))))))
  (setq t1 (getvar 'millisecs)
        ret (eval func)
        t2 (getvar 'millisecs)
        dur (rtos (* 0.001 (- t2 t1)) 2 2)
        str (strcat "\nThe duration of " (vl-symbol-name (car func))
                    " was " dur " seconds"))
  (if alert-p (alert str) (vla-prompt (vla-get-utility acaddoc) str))
  ret)
(defun-r '!time)

(defun-q !avg-time (fun num / lst rt1 rt2 tot ms1 ms2 dur)
  "Prints statistical results after calling %!time NUM times

FUN is the first argument to %!time. This function passes nil for ALERT-P."
  (declare (vars (fun list (function-p (eval (car fun))))
                 (num int)))
  (setq lst '(0)
        rt1 (getvar 'millisecs))
  (repeat num
	  (setq ms1 (getvar 'millisecs))
	  (!time fun nil)
	  (setq ms2 (getvar 'millisecs)
	        dur (* 0.001 (- ms2 ms1)))
	  (append! 'lst (list dur)))
  (setq rt2 (getvar 'millisecs)
        tot (* 0.001 (- rt2 rt1)))
  (if (and (apply 'and (mapcar 'numberp lst)) (setq lst (cdr lst)))
      (progn
        (print (strcat (vl-symbol-name (car fun)) " (" (itoa num) "x)"))
        (print (strcat "min: " (rtos (apply 'min lst) 2 2)))
        (print (strcat "max: " (rtos (apply 'max lst) 2 2)))
        (print (strcat "avg: " (rtos (/ (apply '+ lst) (length lst)) 2 2)))
        (print (strcat "tot: " (rtos tot 2 2)))))
  (/ tot num))
(defun-r '!avg-time)

