  (set-file-docstring "Translation functions for legacy XDATA")

  (defun-q XD-old->XD-new
    (
      lst                     off
      /                        ins
      dat                      calc-geom-key
      chan-spacing             default-actions
      dialog-before-start      dialog-ties
      dist-keys                enable-back-bolts
      enable-back-stair-mode   flip_back-action
      geom-action              height_f2f-action
      int-keys                 load-defaults-keys
      plain-string-keys        popup-keys
      risers_level-action      toggle-keys
      tread-styles
    )
    "Performs appropriate xdata translations dependent on XDATA type"
    (foreach ent lst
      (setq ins (mapcar '+ (cdr (assoc 10 (entget ent))) off))
      (setq dat (read-xdata ent ""))
      (cond
        ( (= (xd-value "xdata_type" dat) "Stair")
          (with-data (XD-old->XD-new_Stair dat)
           '((draw-stair data ins))
          ) ;_ with-data
        ) ;_ condif
        ( (= (xd-value "xdata_type" dat) "Landing")
          (with-data (XD-old->XD-new_Land dat)
           '((draw-land data ins))
          ) ;_ with-data
        ) ;_ condif
        ( (assoc "app_id" dat) (error "XDATA already current"))
        ( (= T) (error "XDATA type not recognized"))
      ) ;_ cond
    ) ;_ foreach
  ) ;_ defun-q XD-old->XD-new
  (defun-r 'XD-old->XD-new)

  (defun-q XD-old->XD-new_Stair (xd-old / xd-def eq-lst)
    "Translates old stair XDATA to new"
    (psc-include
     '("common-data.lsp" "stair/derived.lsp" "stair/dialog.lsp")
    ) ;_ psc-include
    (stair-dialog-load-common-defs)
    (setq xd-def
      (read-key-val-file (strcat psc-data-dir "/stair-defaults.dat"))
    ) ;_ setq
    (setq eq-lst
      (list
       '("tread_depth" "tread_depth")
       '("overall_width" "width")
       '("distance_between_stairs" "dist_btwn")
       '("stair_number" "number")
       '("level_designation" "level")
       '("stringer_designation" "sequence")
       '("number_of_risers" "risers_level")
       '("number_of_risers" "rise_qty")
       '("stair_height" "height_f2f")
       '("nose_clear_bottom" "l-nosing_bot")
       '("nose_clear_pitch" "l-nosing_mid")
       '("nose_clear_top" "l-nosing_top")
       '("nose_clear_bottom" "r-nosing_bot")
       '("nose_clear_pitch" "r-nosing_mid")
       '("nose_clear_top" "r-nosing_top")
      ) ;_ list
    ) ;_ setq
    (foreach key (mapcar 'car xd-def)
      (subst-key! key
        (cond
          ( (= key "ascend")
            (if (= (xd-value "ascend_flag" xd-old) 0) "Left" "Right")
          ) ;_ condif
          ( (= key "z_pos")
            (if (= (xd-value "near_stair_flag" xd-old) 0) "Far" "Near")
          ) ;_ condif
          ( (= key "kickback")
            (itoa (abs (1- (xd-value "kickback_flag" xd-old))))
          ) ;_ condif
          ( (= key "quiet_tread")
            (itoa (xd-value "quiet_tread_flag" xd-old))
          ) ;_ condif
          ( (= key "rail_lvl-off")
            (itoa (xd-value "level_off_flag" xd-old))
          ) ;_ condif
          ( (= key "tread_style")
            (xd-value
              (xd-value "tread_flag" xd-old)
              (mapcar 'list '(2 1 7 6) tread-styles-strings)
            ) ;_ value
          ) ;_ condif
          ( (= key "rail_style")
            (nth (xd-value "rail_type_flag" xd-old) rail-styles)
          ) ;_ condif
          ( (= key "l-string_mat")
            (xd-value
              (xd-value "left_stringer_flag" xd-old)
              (mapcar 'list
               '(0 2 15 1 3 16 8 9 10 11 4 5 6 7 12 13 14 17)
                (append stringers '(nil nil nil nil))
              ) ;_ mapcar
            ) ;_ value
          ) ;_ condif
          ( (= key "r-string_mat")
            (xd-value
              (xd-value "right_stringer_flag" xd-old)
              (mapcar 'list
               '(0 2 15 1 3 16 8 9 10 11 4 5 6 7 12 13 14 17)
                (append stringers '(nil nil nil nil))
              ) ;_ mapcar
            ) ;_ value
          ) ;_ condif
          ( (= key "finish")
            (xd-value
              (xd-value "finish_flag" xd-old)
              (mapcar 'list
               '(1 3 2 4 6 5 13 14 0 nil 7 9 8 10 12 11)
                finishes
              ) ;_ mapcar
            ) ;_ value
          ) ;_ condif
          ( (= key "pan_mat")
            (xd-value
              (xd-value "pan_flag" xd-old)
              (mapcar 'list '(0 1 2 3) bent-plates)
            ) ;_ value
          ) ;_ condif
          ( (= key "rise_hgt")
            (with-data xd-old '((/ stair_height number_of_risers)))
          ) ;_ condif
          ( (= key "top_ext")
            (with-data xd-old
             '(
                (if (= (substr style_picked 1 1) "A")
                  0.0
                  top_haunch_distance
                ) ;_ if
              ) ;_ list
            ) ;_ with-data
          ) ;_ condif
          ( (= key "bot_ext")
            (with-data xd-old
             '(
                (if (member (substr style_picked 2 1) '("S" "A"))
                  0.0
                  bottom_haunch_distance
                ) ;_ if
              ) ;_ list
            ) ;_ with-data
          ) ;_ condif
          ( (= key "bot_conn")
            (nth
              (vl-position
                (atoi (substr (xd-value "bottom_conn" xd-old) 3 2))
                (mapcar 'read bot-details-strings)
              ) ;_ vl-position
              bot-details-strings
            ) ;_ nth
          ) ;_ condif
          ( (= key "top_conn")
            (nth
              (vl-position
                (atoi (substr (xd-value "top_conn" xd-old) 3 2))
                (mapcar 'read top-details-strings)
              ) ;_ vl-position
              top-details-strings
            ) ;_ nth
          ) ;_ condif
          ( (member key (mapcar 'cadr eq-lst))
            (xd-value
              (cadr (assoc key (mapcar 'reverse eq-lst)))
              xd-old
            ) ;_ value
          ) ;_ condif
          ( (= T) (xd-value key xd-def))
        ) ;_ cond
       'xd-def
      ) ;_ subst-key!
    ) ;_ foreach
    (fill-null-keys 'xd-def
      (read-key-val-file (strcat psc-data-dir "/stair-defaults.dat"))
    ) ;_ fill-null-keys
    (foreach key (mapcar 'car xd-def)
      (subst-key! key
        (cond
          ( (member
              key
              (append dist-keys conn_opts-dist-keys tread_opts-dist-keys rail_opts-dist-keys)
            ) ;_ member
            ( (lambda (v) (if (numberp v) (float v) (distof v)))
              (xd-value key xd-def)
            ) ;_ lambda_func
          ) ;_ condif
          ( (member key int-keys)
            ( (lambda (v) (if (numberp v) (fix v) (safe-atoi v)))
              (xd-value key xd-def)
            ) ;_ lambda_func
          ) ;_ condif
          ( (= T) (xd-value key xd-def))
        ) ;_ cond
       'xd-def
      ) ;_ subst-key!
    ) ;_ foreach
    (stair-add-derived-keys xd-def)
  ) ;_ defun-q XD-old->XD-new_Stair
  (defun-r 'XD-old->XD-new_Stair)

  (defun-q XD-old->XD-new_Land (xd-old / xd-def eq-lst conv_chan conv_supp)
    "Translates old landing XDATA to new"
    (psc-include
     '("common-data.lsp" "land/derived.lsp" "land/dialog.lsp")
    ) ;_ psc-include
    (defun conv_chan (key / int det)
      (setq int
        (xd-value
          (cond
            ( (= key "bl_chan") "rear_left_flag")
            ( (= key "br_chan") "rear_right_flag")
            ( (= key "fl_chan") "front_left_flag")
            ( (= key "fr_chan") "front_right_flag")
          ) ;_ cond
          xd-old
        ) ;_ value
      ) ;_ setq
      (conv_supp key int)
      (setq det
        (nth
          (vl-position
            (nth int '(XX XX 30 30 40 77 78 79 XX 66 67 68 20 20 50))
            (mapcar 'read chan-details-strings)
          ) ;_ vl-position
          chan-details-strings
        ) ;_ nth
      ) ;_ setq
    ) ;_ defun conv_chan
    (defun conv_supp (key int / det lst key)
      (setq det (nth int '(X X 31 32 41 X X X X X X X 21 22 51)))
      (setq key (strcat (substr key 1 3) "supp"))
      (setq lst
        (mapcar 'car
            (remove-last
              (vl-remove-if-not
               'listp
                (mapcar 'last chan-details)
              ) ;_ vl-remove-if-not
            ) ;_ remove-last
          ) ;_ mapcar
      ) ;_ setq
      (if (numberp det)
        (subst-key! key
          (nth (vl-position det (mapcar 'read lst)) lst)
         'xd-def
        ) ;_ subst-key!
      ) ;_ if
    ) ;_ defun conv_supp
    (psc-load "land/tread_opts.lsp")
    (land-dialog-load-common-defs)
    (setq xd-def
      (read-key-val-file (strcat psc-data-dir "/land-defaults.dat"))
    ) ;_ setq
    (setq eq-lst
      (list
       '("stair_number" "number")
       '("level" "level")
       '("sequence" "sequence")
       '("overall_landing_width" "tot_wid")
       '("overall_landing_depth" "tot_dpt")
       '("back_channel_location" "back_chan_loc")
       '("back_gap" "back_gap")
       '("rear_left_extension" "bl_ext")
       '("rear_right_extension" "br_ext")
       '("front_left_extension" "fl_ext")
       '("front_right_extension" "fr_ext")
       '("rear_left_length" "bl_len")
       '("rear_right_length" "br_len")
       '("front_left_length" "fl_len")
       '("front_right_length" "fr_len")
       '("left_stair_width" "front_l-stair_wid")
       '("right_stair_width" "front_r-stair_wid")
       '("side_gaps" "left_gap")
       '("side_gaps" "right_gap")
      ) ;_ list
    ) ;_ setq
    (foreach key (mapcar 'car xd-def)
      (subst-key! key
        (cond
        ( (= key "side")
          (if (= (xd-value "side_flag" xd-old) 0) "Right" "Left")
        ) ;_ condif
        ( (= key "flip_back")
          (itoa (xd-value "flip_back_channel_flag" xd-old))
        ) ;_ condif
        ( (= key "tread_style")
          (nth
            (xd-value "landing_type_flag" xd-old)
            (append tread-styles '(nil))
          ) ;_ nth
        ) ;_ condif
        ( (= key "quiet_tread")
          (itoa (xd-value "quiet_tread_flag" xd-old))
        ) ;_ condif
        ( (= key "plt_wid")
          (with-data xd-old
           '((- overall_landing_width (* 2 side_gaps)))
          ) ;_ with-data
        ) ;_ condif
        ( (= key "plt_dpt")
          (with-data xd-old
           '((- overall_landing_depth 1.25 back_gap))
          ) ;_ with-data
        ) ;_ condif
        ( (= key "chk_hold")
          (with-data xd-old
           '(
              (cond
                ( (= 15_safety_nose_flag 1) 0.25)
                ( (= 20_safety_nose_flag 1) 0.75)
                ( (= T) 1.0)
              ) ;_ cond
            ) ;_ list
          ) ;_ with-data
        ) ;_ condif
        ( (member key '("front_chan" "back_chan"))
          (xd-value
            (xd-value "channel_type_flag" xd-old)
            (mapcar 'list '(0 4 1 5 2 6 3 7) channels)
          ) ;_ value
        ) ;_ condif
        ( (member key '("bl_chan" "br_chan" "fl_chan" "fr_chan"))
          (conv_chan key)
        ) ;_ condif
        ( (= key "finish")
          (xd-value
            (xd-value "finish_flag" xd-old)
            (mapcar 'list
             '(1 3 2 4 6 5 13 14 0 nil 7 9 8 10 12 11)
              finishes
            ) ;_ mapcar
          ) ;_ value
        ) ;_ condif
        ( (member key (mapcar 'cadr eq-lst))
          (xd-value
            (cadr (assoc key (mapcar 'reverse eq-lst)))
            xd-old
          ) ;_ value
        ) ;_ condif
        ( (= T) (xd-value key xd-def))
        ) ;_ cond
       'xd-def
      ) ;_ subst-key!
    ) ;_ foreach
    (fill-null-keys 'xd-def
      (read-key-val-file (strcat psc-data-dir "/land-defaults.dat"))
    ) ;_ fill-null-keys
    (foreach key (mapcar 'car xd-def)
      (subst-key!
        key
        (cond
          ( (member key (append dist-keys conn_opts-dist-keys chan_bolts-dist-keys))
            ( (lambda (v) (if (numberp v) (float v) (distof v)))
              (xd-value key xd-def)
            ) ;_ lambda_func
          ) ;_ condif
          ( (member key int-keys)
            ( (lambda (v) (if (numberp v) (fix v) (safe-atoi v)))
              (xd-value key xd-def)
            ) ;_ lambda_func
          ) ;_ condif
          ( (= T) (xd-value key xd-def))
        ) ;_ cond
       'xd-def
      ) ;_ subst-key!
    ) ;_ foreach
    (land-add-derived-keys xd-def)
  ) ;_ defun-q XD-old->XD-new_Land
  (defun-r 'XD-old->XD-new_Land)

  (defun-q fill-null-keys (chk-lst sub-lst)
    "fills unfilled XDATA keys from appropriate defaults list"
    (foreach key
	    (mapcar 'car
		    (vl-remove-if-not
		     '(lambda (k/v) (null (cadr k/v)))
		      (eval chk-lst)
		    ) ;_ vl-remove-if-not
		  ) ;_ mapcar
	    (subst-key! key (xd-value key sub-lst) chk-lst)
	  ) ;_ foreach
  ) ;_ defun fill-null-keys
  (defun-r 'fill-null-keys)
